@@ -3,20 +3,33 @@
 
 from . import controllers
 from . import models
+from . import demo
 from . import wizard
 from . import report
+from . import populate
 
 from odoo import api, SUPERUSER_ID
 
 SYSCOHADA_LIST = ['BJ', 'BF', 'CM', 'CF', 'KM', 'CG', 'CI', 'GA', 'GN', 'GW', 'GQ', 'ML', 'NE', 'CD', 'SN', 'TD', 'TG']
 
-def _auto_install_l10n(cr, registry):
+def _set_fiscal_country(env):
+    """ Sets the fiscal country on existing companies when installing the module.
+    That field is an editable computed field. It doesn't automatically get computed
+    on existing records by the ORM when installing the module, so doing that by hand
+    ensures existing records will get a value for it if needed.
+    """
+    env['res.company'].search([]).compute_account_tax_fiscal_country()
+
+
+def _auto_install_l10n(env):
     #check the country of the main company (only) and eventually load some module needed in that country
-    env = api.Environment(cr, SUPERUSER_ID, {})
     country_code = env.company.country_id.code
     if country_code:
         #auto install localization module(s) if available
-        to_install_l10n = env['ir.module.module'].search_count([('name', 'like', 'l10n_'), ('state', '=', 'to install')])
+        to_install_l10n = env['ir.module.module'].search_count([
+            ('category_id', '=', env.ref('base.module_category_accounting_localizations_account_charts').id),
+            ('state', '=', 'to install'),
+        ])
         module_list = []
         if to_install_l10n:
             # We don't install a CoA if one was passed in the command line
@@ -30,20 +43,13 @@ def _auto_install_l10n(cr, registry):
         elif country_code == 'DE':
             module_list.append('l10n_de_skr03')
             module_list.append('l10n_de_skr04')
-        elif country_code == 'CN':
-            module_list.append('l10n_cn_small_business')
-            module_list.append('l10n_cn_standard')
         else:
             if env['ir.module.module'].search([('name', '=', 'l10n_' + country_code.lower())]):
                 module_list.append('l10n_' + country_code.lower())
             else:
                 module_list.append('l10n_generic_coa')
-        if country_code == 'US':
-            module_list.append('account_plaid')
         if country_code in ['US', 'CA']:
             module_list.append('account_check_printing')
-        if country_code in ['US', 'AU', 'NZ', 'CA', 'CO', 'EC', 'ES', 'FR', 'IN', 'MX', 'GB']:
-            module_list.append('account_yodlee')
         if country_code in SYSCOHADA_LIST + [
             'AT', 'BE', 'CA', 'CO', 'DE', 'EC', 'ES', 'ET', 'FR', 'GR', 'IT', 'LU', 'MX', 'NL', 'NO',
             'PL', 'PT', 'RO', 'SI', 'TR', 'GB', 'VE', 'VN'
@@ -51,13 +57,15 @@ def _auto_install_l10n(cr, registry):
             module_list.append('base_vat')
         if country_code == 'MX':
             module_list.append('l10n_mx_edi')
+        if country_code == 'IT':
+            module_list.append('l10n_it_edi_sdicoop')
+        if country_code == 'SA':
+            module_list.append('l10n_sa_invoice')
 
-        # European countries will be using SEPA
-        europe = env.ref('base.europe', raise_if_not_found=False)
-        if europe:
-            europe_country_codes = [x.code for x in europe.country_ids]
-            if country_code in europe_country_codes:
-                module_list.append('account_sepa')
-                module_list.append('account_bank_statement_import_camt')
         module_ids = env['ir.module.module'].search([('name', 'in', module_list), ('state', '=', 'uninstalled')])
         module_ids.sudo().button_install()
+
+def _account_post_init(cr, registry):
+    env = api.Environment(cr, SUPERUSER_ID, {})
+    _auto_install_l10n(env)
+    _set_fiscal_country(env)
