@@ -1,14 +1,14 @@
 # -*- coding: utf-8 -*-
 
-from datetime import timedelta, datetime
+from datetime import timedelta, datetime, date
 import calendar
 from dateutil.relativedelta import relativedelta
 
 from odoo import fields, models, api, _
 from odoo.exceptions import ValidationError, UserError, RedirectWarning
-from odoo.tools.misc import DEFAULT_SERVER_DATE_FORMAT, format_date
+from odoo.tools.mail import is_html_empty
+from odoo.tools.misc import format_date
 from odoo.tools.float_utils import float_round, float_is_zero
-from odoo.tools import date_utils
 from odoo.tests.common import Form
 
 
@@ -27,6 +27,13 @@ MONTH_SELECTION = [
     ('12', 'December'),
 ]
 
+ONBOARDING_STEP_STATES = [
+    ('not_done', "Not done"),
+    ('just_done', "Just done"),
+    ('done', "Done"),
+]
+DASHBOARD_ONBOARDING_STATES = ONBOARDING_STEP_STATES + [('closed', 'Closed')]
+
 
 class ResCompany(models.Model):
     _inherit = "res.company"
@@ -45,33 +52,37 @@ class ResCompany(models.Model):
     cash_account_code_prefix = fields.Char(string='Prefix of the cash accounts')
     default_cash_difference_income_account_id = fields.Many2one('account.account', string="Cash Difference Income Account")
     default_cash_difference_expense_account_id = fields.Many2one('account.account', string="Cash Difference Expense Account")
+    account_journal_suspense_account_id = fields.Many2one('account.account', string='Journal Suspense Account')
+    account_journal_payment_debit_account_id = fields.Many2one('account.account', string='Journal Outstanding Receipts Account')
+    account_journal_payment_credit_account_id = fields.Many2one('account.account', string='Journal Outstanding Payments Account')
     transfer_account_code_prefix = fields.Char(string='Prefix of the transfer accounts')
     account_sale_tax_id = fields.Many2one('account.tax', string="Default Sale Tax")
     account_purchase_tax_id = fields.Many2one('account.tax', string="Default Purchase Tax")
-    tax_cash_basis_journal_id = fields.Many2one('account.journal', string="Cash Basis Journal")
     tax_calculation_rounding_method = fields.Selection([
         ('round_per_line', 'Round per Line'),
         ('round_globally', 'Round Globally'),
         ], default='round_per_line', string='Tax Calculation Rounding Method')
     currency_exchange_journal_id = fields.Many2one('account.journal', string="Exchange Gain or Loss Journal", domain=[('type', '=', 'general')])
-    income_currency_exchange_account_id = fields.Many2one('account.account', related='currency_exchange_journal_id.default_credit_account_id', readonly=False,
-        string="Gain Exchange Rate Account", domain="[('internal_type', '=', 'other'), ('deprecated', '=', False), ('company_id', '=', id)]")
-    expense_currency_exchange_account_id = fields.Many2one('account.account', related='currency_exchange_journal_id.default_debit_account_id', readonly=False,
-        string="Loss Exchange Rate Account", domain="[('internal_type', '=', 'other'), ('deprecated', '=', False), ('company_id', '=', id)]")
+    income_currency_exchange_account_id = fields.Many2one(
+        comodel_name='account.account',
+        string="Gain Exchange Rate Account",
+        domain=lambda self: "[('internal_type', '=', 'other'), ('deprecated', '=', False), ('company_id', '=', id), \
+                             ('user_type_id', 'in', %s)]" % [self.env.ref('account.data_account_type_revenue').id,
+                                                             self.env.ref('account.data_account_type_other_income').id])
+    expense_currency_exchange_account_id = fields.Many2one(
+        comodel_name='account.account',
+        string="Loss Exchange Rate Account",
+        domain=lambda self: "[('internal_type', '=', 'other'), ('deprecated', '=', False), ('company_id', '=', id), \
+                             ('user_type_id', '=', %s)]" % self.env.ref('account.data_account_type_expenses').id)
     anglo_saxon_accounting = fields.Boolean(string="Use anglo-saxon accounting")
     property_stock_account_input_categ_id = fields.Many2one('account.account', string="Input Account for Stock Valuation")
     property_stock_account_output_categ_id = fields.Many2one('account.account', string="Output Account for Stock Valuation")
     property_stock_valuation_account_id = fields.Many2one('account.account', string="Account Template for Stock Valuation")
     bank_journal_ids = fields.One2many('account.journal', 'company_id', domain=[('type', '=', 'bank')], string='Bank Journals')
-    tax_exigibility = fields.Boolean(string='Use Cash Basis')
-    account_bank_reconciliation_start = fields.Date(string="Bank Reconciliation Threshold", help="""The bank reconciliation widget won't ask to reconcile payments older than this date.
-                                                                                                       This is useful if you install accounting after having used invoicing for some time and
-                                                                                                       don't want to reconcile all the past payments with bank statements.""")
-
     incoterm_id = fields.Many2one('account.incoterms', string='Default incoterm',
         help='International Commercial Terms are a series of predefined commercial terms used in international transactions.')
 
-    qr_code = fields.Boolean(string='Display SEPA QR code')
+    qr_code = fields.Boolean(string='Display QR-code on invoices')
 
     invoice_is_email = fields.Boolean('Email by default', default=True)
     invoice_is_print = fields.Boolean('Print by default', default=True)
@@ -79,21 +90,27 @@ class ResCompany(models.Model):
     #Fields of the setup step for opening move
     account_opening_move_id = fields.Many2one(string='Opening Journal Entry', comodel_name='account.move', help="The journal entry containing the initial balance of all this company's accounts.")
     account_opening_journal_id = fields.Many2one(string='Opening Journal', comodel_name='account.journal', related='account_opening_move_id.journal_id', help="Journal where the opening entry of this company's accounting has been posted.", readonly=False)
-    account_opening_date = fields.Date(string='Opening Date', related='account_opening_move_id.date', help="Date at which the opening entry of this company's accounting has been posted.", readonly=False)
+    account_opening_date = fields.Date(string='Opening Entry', default=lambda self: fields.Date.context_today(self).replace(month=1, day=1), required=True, help="That is the date of the opening entry.")
 
     # Fields marking the completion of a setup step
-    # YTI FIXME : The selection should be factorize as a static list in base, like ONBOARDING_STEP_STATES
-    account_setup_bank_data_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done")], string="State of the onboarding bank data step", default='not_done')
-    account_setup_fy_data_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done")], string="State of the onboarding fiscal year step", default='not_done')
-    account_setup_coa_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done")], string="State of the onboarding charts of account step", default='not_done')
-    account_onboarding_invoice_layout_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done")], string="State of the onboarding invoice layout step", default='not_done')
-    account_onboarding_sample_invoice_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done")], string="State of the onboarding sample invoice step", default='not_done')
-    account_onboarding_sale_tax_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done")], string="State of the onboarding sale tax step", default='not_done')
+    account_setup_bank_data_state = fields.Selection(ONBOARDING_STEP_STATES, string="State of the onboarding bank data step", default='not_done')
+    account_setup_fy_data_state = fields.Selection(ONBOARDING_STEP_STATES, string="State of the onboarding fiscal year step", default='not_done')
+    account_setup_coa_state = fields.Selection(ONBOARDING_STEP_STATES, string="State of the onboarding charts of account step", default='not_done')
+    account_setup_taxes_state = fields.Selection(ONBOARDING_STEP_STATES, string="State of the onboarding Taxes step", default='not_done')
+    account_onboarding_invoice_layout_state = fields.Selection(ONBOARDING_STEP_STATES, string="State of the onboarding invoice layout step", default='not_done')
+    account_onboarding_create_invoice_state = fields.Selection(ONBOARDING_STEP_STATES, string="State of the onboarding create invoice step", default='not_done')
+    account_onboarding_sale_tax_state = fields.Selection(ONBOARDING_STEP_STATES, string="State of the onboarding sale tax step", default='not_done')
 
     # account dashboard onboarding
-    account_invoice_onboarding_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done"), ('closed', "Closed")], string="State of the account invoice onboarding panel", default='not_done')
-    account_dashboard_onboarding_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done"), ('closed', "Closed")], string="State of the account dashboard onboarding panel", default='not_done')
-    invoice_terms = fields.Text(string='Default Terms and Conditions', translate=True)
+    account_invoice_onboarding_state = fields.Selection(DASHBOARD_ONBOARDING_STATES, string="State of the account invoice onboarding panel", default='not_done')
+    account_dashboard_onboarding_state = fields.Selection(DASHBOARD_ONBOARDING_STATES, string="State of the account dashboard onboarding panel", default='not_done')
+    invoice_terms = fields.Html(string='Default Terms and Conditions', translate=True)
+    terms_type = fields.Selection([('plain', 'Add a Note'), ('html', 'Add a link to a Web Page')],
+                                  string='Terms & Conditions format', default='plain')
+    invoice_terms_html = fields.Html(string='Default Terms and Conditions as a Web page', translate=True,
+                                     compute='_compute_invoice_terms_html', store=True, readonly=False)
+
+    account_setup_bill_state = fields.Selection(ONBOARDING_STEP_STATES, string="State of the onboarding bill step", default='not_done')
 
     # Needed in the Point of Sale
     account_default_pos_receivable_account_id = fields.Many2one('account.account', string="Default PoS Receivable Account")
@@ -101,11 +118,42 @@ class ResCompany(models.Model):
     # Accrual Accounting
     expense_accrual_account_id = fields.Many2one('account.account',
         help="Account used to move the period of an expense",
-        domain="[('internal_group', '=', 'liability'), ('internal_type', 'not in', ('receivable', 'payable')), ('reconcile', '=', True), ('company_id', '=', id)]")
+        domain="[('internal_group', '=', 'liability'), ('internal_type', 'not in', ('receivable', 'payable')), ('company_id', '=', id)]")
     revenue_accrual_account_id = fields.Many2one('account.account',
         help="Account used to move the period of a revenue",
-        domain="[('internal_group', '=', 'asset'), ('internal_type', 'not in', ('receivable', 'payable')), ('reconcile', '=', True), ('company_id', '=', id)]")
-    accrual_default_journal_id = fields.Many2one('account.journal', help="Journal used by default for moving the period of an entry", domain="[('type', '=', 'general')]")
+        domain="[('internal_group', '=', 'asset'), ('internal_type', 'not in', ('receivable', 'payable')), ('company_id', '=', id)]")
+    automatic_entry_default_journal_id = fields.Many2one('account.journal', help="Journal used by default for moving the period of an entry", domain="[('type', '=', 'general')]")
+
+    # Technical field to hide country specific fields in company form view
+    country_code = fields.Char(related='country_id.code')
+
+    # Taxes
+    account_fiscal_country_id = fields.Many2one(
+        string="Fiscal Country",
+        comodel_name='res.country',
+        compute='compute_account_tax_fiscal_country',
+        store=True,
+        readonly=False,
+        help="The country to use the tax reports from for this company")
+
+    account_enabled_tax_country_ids = fields.Many2many(
+        string="l10n-used countries",
+        comodel_name='res.country',
+        compute='_compute_account_enabled_tax_country_ids',
+        help="Technical field containing the countries for which this company is using tax-related features"
+             "(hence the ones for which l10n modules need to show tax-related fields).")
+
+    # Cash basis taxes
+    tax_exigibility = fields.Boolean(string='Use Cash Basis')
+    tax_cash_basis_journal_id = fields.Many2one(
+        comodel_name='account.journal',
+        string="Cash Basis Journal")
+    account_cash_basis_base_account_id = fields.Many2one(
+        comodel_name='account.account',
+        domain=[('deprecated', '=', False)],
+        string="Base Tax Received Account",
+        help="Account that will be set on lines created in cash basis journal entry and used to keep track of the "
+             "tax base amount.")
 
     @api.constrains('account_opening_move_id', 'fiscalyear_last_day', 'fiscalyear_last_month')
     def _check_fiscalyear_last_day(self):
@@ -124,6 +172,26 @@ class ResCompany(models.Model):
             if rec.fiscalyear_last_day > max_day:
                 raise ValidationError(_("Invalid fiscal year last day"))
 
+    @api.depends('country_id')
+    def compute_account_tax_fiscal_country(self):
+        for record in self:
+            record.account_fiscal_country_id = record.country_id
+
+    @api.depends('account_fiscal_country_id')
+    def _compute_account_enabled_tax_country_ids(self):
+        for record in self:
+            foreign_vat_fpos = self.env['account.fiscal.position'].search([('company_id', '=', record.id), ('foreign_vat', '!=', False)])
+            record.account_enabled_tax_country_ids = foreign_vat_fpos.country_id + record.account_fiscal_country_id
+
+    @api.depends('terms_type')
+    def _compute_invoice_terms_html(self):
+        term_template = self.env.ref("account.account_default_terms_and_conditions", False)
+        if not term_template:
+            return
+
+        for company in self.filtered(lambda company: is_html_empty(company.invoice_terms_html) and company.terms_type == 'html'):
+            company.invoice_terms_html = term_template._render({'company_name': company.name, 'company_country': company.country_id.name}, engine='ir.qweb')
+
     def get_and_update_account_invoice_onboarding_state(self):
         """ This method is called on the controller rendering method and ensures that the animations
             are displayed only one time. """
@@ -138,7 +206,7 @@ class ResCompany(models.Model):
         return [
             'base_onboarding_company_state',
             'account_onboarding_invoice_layout_state',
-            'account_onboarding_sample_invoice_state',
+            'account_onboarding_create_invoice_state',
         ]
 
     def get_and_update_account_dashboard_onboarding_state(self):
@@ -152,67 +220,12 @@ class ResCompany(models.Model):
     def get_account_dashboard_onboarding_steps_states_names(self):
         """ Necessary to add/edit steps from other modules (account_winbooks_import in this case). """
         return [
-            'base_onboarding_company_state',
             'account_setup_bank_data_state',
             'account_setup_fy_data_state',
             'account_setup_coa_state',
+            'account_setup_taxes_state',
         ]
 
-    def compute_fiscalyear_dates(self, current_date):
-        '''Computes the start and end dates of the fiscal year where the given 'date' belongs to.
-
-        :param current_date: A datetime.date/datetime.datetime object.
-        :return: A dictionary containing:
-            * date_from
-            * date_to
-            * [Optionally] record: The fiscal year record.
-        '''
-        self.ensure_one()
-        date_str = current_date.strftime(DEFAULT_SERVER_DATE_FORMAT)
-
-        # Search a fiscal year record containing the date.
-        # If a record is found, then no need further computation, we get the dates range directly.
-        fiscalyear = self.env['account.fiscal.year'].search([
-            ('company_id', '=', self.id),
-            ('date_from', '<=', date_str),
-            ('date_to', '>=', date_str),
-        ], limit=1)
-        if fiscalyear:
-            return {
-                'date_from': fiscalyear.date_from,
-                'date_to': fiscalyear.date_to,
-                'record': fiscalyear,
-            }
-
-        date_from, date_to = date_utils.get_fiscal_year(
-            current_date, day=self.fiscalyear_last_day, month=int(self.fiscalyear_last_month))
-
-        date_from_str = date_from.strftime(DEFAULT_SERVER_DATE_FORMAT)
-        date_to_str = date_to.strftime(DEFAULT_SERVER_DATE_FORMAT)
-
-        # Search for fiscal year records reducing the delta between the date_from/date_to.
-        # This case could happen if there is a gap between two fiscal year records.
-        # E.g. two fiscal year records: 2017-01-01 -> 2017-02-01 and 2017-03-01 -> 2017-12-31.
-        # => The period 2017-02-02 - 2017-02-30 is not covered by a fiscal year record.
-
-        fiscalyear_from = self.env['account.fiscal.year'].search([
-            ('company_id', '=', self.id),
-            ('date_from', '<=', date_from_str),
-            ('date_to', '>=', date_from_str),
-        ], limit=1)
-        if fiscalyear_from:
-            date_from = fiscalyear_from.date_to + timedelta(days=1)
-
-        fiscalyear_to = self.env['account.fiscal.year'].search([
-            ('company_id', '=', self.id),
-            ('date_from', '<=', date_to_str),
-            ('date_to', '>=', date_to_str),
-        ], limit=1)
-        if fiscalyear_to:
-            date_to = fiscalyear_to.date_from - timedelta(days=1)
-
-        return {'date_from': date_from, 'date_to': date_to}
-
     def get_new_account_code(self, current_code, old_prefix, new_prefix):
         digits = len(current_code)
         return new_prefix + current_code.replace(old_prefix, '', 1).lstrip('0').rjust(digits-len(new_prefix), '0')
@@ -226,12 +239,47 @@ class ResCompany(models.Model):
 
     def _validate_fiscalyear_lock(self, values):
         if values.get('fiscalyear_lock_date'):
-            nb_draft_entries = self.env['account.move'].search([
-                ('company_id', 'in', [c.id for c in self]),
+
+            draft_entries = self.env['account.move'].search([
+                ('company_id', 'in', self.ids),
                 ('state', '=', 'draft'),
                 ('date', '<=', values['fiscalyear_lock_date'])])
-            if nb_draft_entries:
-                raise ValidationError(_('There are still unposted entries in the period you want to lock. You should either post or delete them.'))
+            if draft_entries:
+                error_msg = _('There are still unposted entries in the period you want to lock. You should either post or delete them.')
+                action_error = {
+                    'view_mode': 'tree',
+                    'name': 'Unposted Entries',
+                    'res_model': 'account.move',
+                    'type': 'ir.actions.act_window',
+                    'domain': [('id', 'in', draft_entries.ids)],
+                    'search_view_id': [self.env.ref('account.view_account_move_filter').id, 'search'],
+                    'views': [[self.env.ref('account.view_move_tree').id, 'list'], [self.env.ref('account.view_move_form').id, 'form']],
+                }
+                raise RedirectWarning(error_msg, action_error, _('Show unposted entries'))
+
+            unreconciled_statement_lines = self.env['account.bank.statement.line'].search([
+                ('company_id', 'in', self.ids),
+                ('is_reconciled', '=', False),
+                ('date', '<=', values['fiscalyear_lock_date']),
+                ('move_id.state', 'in', ('draft', 'posted')),
+            ])
+            if unreconciled_statement_lines:
+                error_msg = _("There are still unreconciled bank statement lines in the period you want to lock."
+                            "You should either reconcile or delete them.")
+                action_error = {
+                    'type': 'ir.actions.client',
+                    'tag': 'bank_statement_reconciliation_view',
+                    'context': {'statement_line_ids': unreconciled_statement_lines.ids, 'company_ids': self.ids},
+                }
+                raise RedirectWarning(error_msg, action_error, _('Show Unreconciled Bank Statement Line'))
+
+    def _get_user_fiscal_lock_date(self):
+        """Get the fiscal lock date for this company depending on the user"""
+        self.ensure_one()
+        lock_date = max(self.period_lock_date or date.min, self.fiscalyear_lock_date or date.min)
+        if self.user_has_groups('account.group_account_manager'):
+            lock_date = self.fiscalyear_lock_date or date.min
+        return lock_date
 
     def write(self, values):
         #restrict the closing of FY if there are still unposted entries
@@ -242,6 +290,7 @@ class ResCompany(models.Model):
             if values.get('bank_account_code_prefix'):
                 new_bank_code = values.get('bank_account_code_prefix') or company.bank_account_code_prefix
                 company.reflect_code_prefix_change(company.bank_account_code_prefix, new_bank_code)
+
             if values.get('cash_account_code_prefix'):
                 new_cash_code = values.get('cash_account_code_prefix') or company.cash_account_code_prefix
                 company.reflect_code_prefix_change(company.cash_account_code_prefix, new_cash_code)
@@ -306,7 +355,7 @@ class ResCompany(models.Model):
             'res_model': 'account.account',
             'view_mode': 'tree',
             'limit': 99999999,
-            'search_view_id': self.env.ref('account.view_account_search').id,
+            'search_view_id': [self.env.ref('account.view_account_search').id],
             'views': [[view_id, 'list']],
             'domain': domain,
         }
@@ -324,13 +373,10 @@ class ResCompany(models.Model):
             if not default_journal:
                 raise UserError(_("Please install a chart of accounts or create a miscellaneous journal before proceeding."))
 
-            today = datetime.today().date()
-            opening_date = today.replace(month=int(self.fiscalyear_last_month), day=self.fiscalyear_last_day) + timedelta(days=1)
-            if opening_date > today:
-                opening_date = opening_date + relativedelta(years=-1)
+            opening_date = self.account_opening_date - timedelta(days=1)
 
             self.account_opening_move_id = self.env['account.move'].create({
-                'name': _('Opening Journal Entry'),
+                'ref': _('Opening Journal Entry'),
                 'company_id': self.id,
                 'journal_id': default_journal.id,
                 'date': opening_date,
@@ -425,73 +471,32 @@ class ResCompany(models.Model):
     @api.model
     def action_open_account_onboarding_sale_tax(self):
         """ Onboarding step for the invoice layout. """
-        action = self.env.ref('account.action_open_account_onboarding_sale_tax').read()[0]
+        action = self.env["ir.actions.actions"]._for_xml_id("account.action_open_account_onboarding_sale_tax")
         action['res_id'] = self.env.company.id
         return action
 
     @api.model
-    def _get_sample_invoice(self):
-        """ Get a sample invoice or create one if it does not exist. """
-        # use current user as partner
-        partner = self.env.user.partner_id
-
-        company_id = self.env.company.id
-        # try to find an existing sample invoice
-        sample_invoice = self.env['account.move'].search(
-            [('company_id', '=', company_id),
-             ('partner_id', '=', partner.id)], limit=1)
-
-        if len(sample_invoice) == 0:
-            # If there are no existing accounts or no journal, fail
-            account = self.env.company.get_chart_of_accounts_or_fail()
-
-            journal = self.env['account.journal'].search([('company_id', '=', company_id)], limit=1)
-            if len(journal) == 0:
-                action = self.env.ref('account.action_account_journal_form')
-                msg = _("We cannot find any journal for this company. You should create one."
-                        "\nPlease go to Configuration > Journals.")
-                raise RedirectWarning(msg, action.id, _("Go to the journal configuration"))
-
-            account = journal.default_credit_account_id or account
-            sample_invoice = self.env['account.move'].with_context(default_type='out_invoice', default_journal_id=journal.id).create({
-                'invoice_payment_ref': _('Sample invoice'),
-                'partner_id': partner.id,
-                'invoice_line_ids': [
-                    (0, 0, {
-                        'name': _('Sample invoice line name'),
-                        'account_id': account.id,
-                        'quantity': 2,
-                        'price_unit': 199.99,
-                    }),
-                    (0, 0, {
-                        'name': _('Sample invoice line name 2'),
-                        'account_id': account.id,
-                        'quantity': 1,
-                        'price_unit': 25.0,
-                    }),
-                ],
-            })
-        return sample_invoice
+    def action_open_account_onboarding_create_invoice(self):
+        action = self.env["ir.actions.actions"]._for_xml_id("account.action_open_account_onboarding_create_invoice")
+        return action
 
     @api.model
-    def action_open_account_onboarding_sample_invoice(self):
-        """ Onboarding step for sending a sample invoice. Open a window to compose an email,
-            with the edi_invoice_template message loaded by default. """
-        sample_invoice = self._get_sample_invoice()
-        template = self.env.ref('account.email_template_edi_invoice', False)
-        action = self.env.ref('account.action_open_account_onboarding_sample_invoice').read()[0]
-        action['context'] = {
-            'default_res_id': sample_invoice.id,
-            'default_use_template': bool(template),
-            'default_template_id': template and template.id or False,
-            'default_model': 'account.move',
-            'default_composition_mode': 'comment',
-            'mark_invoice_as_sent': True,
-            'custom_layout': 'mail.mail_notification_borders',
-            'force_email': True,
-            'mail_notify_author': True,
+    def action_open_taxes_onboarding(self):
+        """ Called by the 'Taxes' button of the setup bar."""
+
+        company = self.env.company
+        company.sudo().set_onboarding_step_done('account_setup_taxes_state')
+        view_id_list = self.env.ref('account.view_tax_tree').id
+        view_id_form = self.env.ref('account.view_tax_form').id
+
+        return {
+            'type': 'ir.actions.act_window',
+            'name': _('Taxes'),
+            'res_model': 'account.tax',
+            'target': 'current',
+            'views': [[view_id_list, 'list'], [view_id_form, 'form']],
+            'context': {'search_default_sale': True, 'search_default_purchase': True, 'active_test': False},
         }
-        return action
 
     def action_save_onboarding_invoice_layout(self):
         """ Set the onboarding step as done """
@@ -526,7 +531,7 @@ class ResCompany(models.Model):
         journals = self.env['account.journal'].search([('company_id', '=', self.id)])
         results_by_journal = {
             'results': [],
-            'printing_date': format_date(self.env, fields.Date.to_string(fields.Date.today()))
+            'printing_date': format_date(self.env, fields.Date.to_string(fields.Date.context_today(self)))
         }
 
         for journal in journals:
@@ -562,7 +567,7 @@ class ResCompany(models.Model):
             hash_corrupted = False
             for move in moves:
                 if move.inalterable_hash != move._compute_hash(previous_hash=previous_hash):
-                    rslt.update({'msg_cover': _('Corrupted data on journal entry with id %s.') % move.id})
+                    rslt.update({'msg_cover': _('Corrupted data on journal entry with id %s.', move.id)})
                     results_by_journal['results'].append(rslt)
                     hash_corrupted = True
                     break
@@ -591,15 +596,15 @@ class ResCompany(models.Model):
 
         return results_by_journal
 
-    def get_fiscal_country(self):
-        """ Returns the country to be used for this company's tax report.
-        By default, it'll be the one from the address; but a config parameter
-        may be used for each company to customize that behavior.
+    def compute_fiscalyear_dates(self, current_date):
+        """
+        The role of this method is to provide a fallback when account_accounting is not installed.
+        As the fiscal year is irrelevant when account_accounting is not installed, this method returns the calendar year.
+        :param current_date: A datetime.date/datetime.datetime object.
+        :return: A dictionary containing:
+            * date_from
+            * date_to
         """
-        self.ensure_one()
 
-        fiscal_country_key = 'account_fiscal_country_%s' % self.id
-        forced_country_code = self.env['ir.config_parameter'].sudo().get_param(fiscal_country_key)
-        forced_country = forced_country_code and self.env['res.country'].search([('code', 'ilike', forced_country_code)], limit=1)
-
-        return forced_country or self.country_id
+        return {'date_from': datetime(year=current_date.year, month=1, day=1).date(),
+                'date_to': datetime(year=current_date.year, month=12, day=31).date()}
