@@ -1,9 +1,8 @@
 # -*- coding: utf-8 -*-
 
-from odoo import models, api, _
+from odoo import models, _
 from odoo.exceptions import UserError
-
-from odoo.tools import float_compare
+from odoo.tools import pdf
 
 
 class IrActionsReport(models.Model):
@@ -26,18 +25,18 @@ class IrActionsReport(models.Model):
                 raise UserError(_("No original vendor bills could be found for any of the selected vendor bills."))
         return super(IrActionsReport, self)._post_pdf(save_in_attachment, pdf_content=pdf_content, res_ids=res_ids)
 
-    def postprocess_pdf_report(self, record, buffer):
+    def _postprocess_pdf_report(self, record, buffer):
         # don't save the 'account.report_original_vendor_bill' report as it's just a mean to print existing attachments
         if self.report_name == 'account.report_original_vendor_bill':
             return None
-        res = super(IrActionsReport, self).postprocess_pdf_report(record, buffer)
+        res = super(IrActionsReport, self)._postprocess_pdf_report(record, buffer)
         if self.model == 'account.move' and record.state == 'posted' and record.is_sale_document(include_receipts=True):
             attachment = self.retrieve_attachment(record)
             if attachment:
                 attachment.register_as_main_attachment(force=False)
         return res
 
-    def render_qweb_pdf(self, res_ids=None, data=None):
+    def _render_qweb_pdf(self, res_ids=None, data=None):
         # Overridden so that the print > invoices actions raises an error
         # when trying to print a miscellaneous operation instead of an invoice.
         if self.model == 'account.move' and res_ids:
@@ -47,9 +46,13 @@ class IrActionsReport(models.Model):
                 if any(not move.is_invoice(include_receipts=True) for move in moves):
                     raise UserError(_("Only invoices could be printed."))
 
-        return super().render_qweb_pdf(res_ids=res_ids, data=data)
+        return super()._render_qweb_pdf(res_ids=res_ids, data=data)
 
-    def _get_rendering_context(self, docids, data):
-        data = data and dict(data) or {}
-        data.update({'float_compare': float_compare})
-        return super()._get_rendering_context(docids=docids, data=data)
+    def _retrieve_stream_from_attachment(self, attachment):
+        # Overridden in order to add a banner in the upper right corner of the exported Vendor Bill PDF.
+        stream = super()._retrieve_stream_from_attachment(attachment)
+        vendor_bill_export = self.env.ref('account.action_account_original_vendor_bill')
+        if self == vendor_bill_export and attachment.mimetype == 'application/pdf':
+            record = self.env[attachment.res_model].browse(attachment.res_id)
+            return pdf.add_banner(stream, record.name, logo=True)
+        return stream
