@@ -1,35 +1,38 @@
 # -*- coding: utf-8 -*-
 
+from . import test_account_move_reconcile
+from . import test_account_move_payments_widget
 from . import test_account_move_out_invoice
 from . import test_account_move_out_refund
 from . import test_account_move_in_invoice
 from . import test_account_move_in_refund
 from . import test_account_move_entry
+from . import test_invoice_tax_totals
 from . import test_account_journal
 from . import test_account_account
 from . import test_account_tax
 from . import test_account_analytic
 from . import test_account_payment
+from . import test_account_bank_statement
 from . import test_account_move_partner_count
 from . import test_account_move_rounding
 from . import test_account_invoice_report
+from . import test_account_move_line_tax_details
 from . import test_account_journal_dashboard
-from . import test_bank_statement_reconciliation
 from . import test_fiscal_position
-from . import test_reconciliation_widget
-from . import test_payment
 from . import test_reconciliation
-from . import test_search
+from . import test_sequence_mixin
 from . import test_settings
 from . import test_tax
 from . import test_invoice_taxes
 from . import test_templates_consistency
-from . import test_account_fiscal_year
 from . import test_account_all_l10n
 from . import test_reconciliation_matching_rules
 from . import test_account_onboarding
 from . import test_portal_attachment
-from . import test_reconciliation_heavy_load
+from . import test_tax_report
+from . import test_transfer_wizard
 from . import test_account_incoming_supplier_invoice
-from . import test_reconcile_account
 from . import test_payment_term
+from . import test_account_payment_register
+from . import test_tour
