@@ -1,11 +1,8 @@
 # -*- coding: utf-8 -*-
-from odoo.addons.account.tests.account_test_savepoint import AccountTestInvoicingCommon
-from odoo.tests.common import Form
+from odoo.addons.account.tests.common import AccountTestInvoicingCommon
 from odoo.tests import tagged
 from odoo import fields
 
-from unittest.mock import patch
-
 
 @tagged('post_install', '-at_install')
 class TestAccountInvoiceReport(AccountTestInvoicingCommon):
@@ -16,7 +13,7 @@ class TestAccountInvoiceReport(AccountTestInvoicingCommon):
 
         cls.invoices = cls.env['account.move'].create([
             {
-                'type': 'out_invoice',
+                'move_type': 'out_invoice',
                 'partner_id': cls.partner_a.id,
                 'invoice_date': fields.Date.from_string('2016-01-01'),
                 'currency_id': cls.currency_data['currency'].id,
@@ -34,7 +31,7 @@ class TestAccountInvoiceReport(AccountTestInvoicingCommon):
                 ]
             },
             {
-                'type': 'out_receipt',
+                'move_type': 'out_receipt',
                 'invoice_date': fields.Date.from_string('2016-01-01'),
                 'currency_id': cls.currency_data['currency'].id,
                 'invoice_line_ids': [
@@ -46,7 +43,7 @@ class TestAccountInvoiceReport(AccountTestInvoicingCommon):
                 ]
             },
             {
-                'type': 'out_refund',
+                'move_type': 'out_refund',
                 'partner_id': cls.partner_a.id,
                 'invoice_date': fields.Date.from_string('2017-01-01'),
                 'currency_id': cls.currency_data['currency'].id,
@@ -59,7 +56,7 @@ class TestAccountInvoiceReport(AccountTestInvoicingCommon):
                 ]
             },
             {
-                'type': 'in_invoice',
+                'move_type': 'in_invoice',
                 'partner_id': cls.partner_a.id,
                 'invoice_date': fields.Date.from_string('2016-01-01'),
                 'currency_id': cls.currency_data['currency'].id,
@@ -72,7 +69,7 @@ class TestAccountInvoiceReport(AccountTestInvoicingCommon):
                 ]
             },
             {
-                'type': 'in_receipt',
+                'move_type': 'in_receipt',
                 'partner_id': cls.partner_a.id,
                 'invoice_date': fields.Date.from_string('2016-01-01'),
                 'currency_id': cls.currency_data['currency'].id,
@@ -85,7 +82,7 @@ class TestAccountInvoiceReport(AccountTestInvoicingCommon):
                 ]
             },
             {
-                'type': 'in_refund',
+                'move_type': 'in_refund',
                 'partner_id': cls.partner_a.id,
                 'invoice_date': fields.Date.from_string('2017-01-01'),
                 'currency_id': cls.currency_data['currency'].id,
@@ -100,24 +97,23 @@ class TestAccountInvoiceReport(AccountTestInvoicingCommon):
         ])
 
     def assertInvoiceReportValues(self, expected_values_list):
-        reports = self.env['account.invoice.report'].search([('company_id', '=', self.company_data['company'].id)], order='price_subtotal DESC')
+        reports = self.env['account.invoice.report'].search([('company_id', '=', self.company_data['company'].id)], order='price_subtotal DESC, quantity ASC')
         expected_values_dict = [{
-            'amount_total': vals[0],
-            'price_average': vals[1],
-            'price_subtotal': vals[2],
-            'residual': vals[3],
-            'quantity': vals[4],
+            'price_average': vals[0],
+            'price_subtotal': vals[1],
+            'quantity': vals[2],
         } for vals in expected_values_list]
+
         self.assertRecordValues(reports, expected_values_dict)
 
     def test_invoice_report_multiple_types(self):
         self.assertInvoiceReportValues([
-            # amount_total  price_average   price_subtotal  residual    quantity
-            [2000,          2000,           2000,           2000,       1],
-            [1000,          1000,           1000,           1000,       1],
-            [750,           250,            750,            750,        3],
-            [6,             6,              6,              6,          1],
-            [-20,           -20,            -20,            -20,        -1],
-            [-20,           -20,            -20,            -20,        -1],
-            [-600,          -600,           -600,           -600,       -1],
+            #price_average   price_subtotal  quantity
+            [2000,           2000,           1],
+            [1000,           1000,           1],
+            [250,            750,            3],
+            [6,              6,              1],
+            [20,             -20,           -1],
+            [20,             -20,           -1],
+            [600,            -600,          -1],
         ])
