@@ -1,10 +1,17 @@
 # -*- coding: utf-8 -*-
-from odoo.addons.account.tests.account_test_savepoint import AccountTestInvoicingCommon
+from odoo.addons.account.tests.common import AccountTestInvoicingCommon
 from odoo.tests import tagged, new_test_user
 from odoo.tests.common import Form
-from odoo import fields
-from odoo.exceptions import ValidationError, UserError
+from odoo import fields, api, SUPERUSER_ID
+from odoo.exceptions import ValidationError, UserError, RedirectWarning
+from odoo.tools import mute_logger
 
+from dateutil.relativedelta import relativedelta
+from functools import reduce
+import json
+import psycopg2
+
+from collections import defaultdict
 
 @tagged('post_install', '-at_install')
 class TestAccountMove(AccountTestInvoicingCommon):
@@ -16,7 +23,7 @@ class TestAccountMove(AccountTestInvoicingCommon):
         tax_repartition_line = cls.company_data['default_tax_sale'].refund_repartition_line_ids\
             .filtered(lambda line: line.repartition_type == 'tax')
         cls.test_move = cls.env['account.move'].create({
-            'type': 'entry',
+            'move_type': 'entry',
             'date': fields.Date.from_string('2016-01-01'),
             'line_ids': [
                 (0, None, {
@@ -65,7 +72,7 @@ class TestAccountMove(AccountTestInvoicingCommon):
         self.test_move.line_ids[0].account_id = custom_account
 
     def test_misc_fiscalyear_lock_date_1(self):
-        self.test_move.post()
+        self.test_move.action_post()
 
         # Set the lock date after the journal entry date.
         self.test_move.company_id.fiscalyear_lock_date = fields.Date.from_string('2017-01-01')
@@ -133,22 +140,30 @@ class TestAccountMove(AccountTestInvoicingCommon):
         with self.assertRaises(UserError), self.cr.savepoint():
             self.test_move.button_draft()
 
-        copy_move = self.test_move.copy()
-
         # Try to add a new journal entry prior to the lock date.
-        with self.assertRaises(UserError), self.cr.savepoint():
-            copy_move.post()
+        copy_move = self.test_move.copy({'date': '2017-01-01'})
+        # The date has been changed to the first valid date.
+        self.assertEqual(copy_move.date, copy_move.company_id.fiscalyear_lock_date + relativedelta(days=1))
 
-        # You can change the date as the journal entry is not posted.
-        copy_move.date = fields.Date.from_string('2018-01-01')
-        copy_move.post()
+    def test_misc_fiscalyear_lock_date_2(self):
+        self.test_move.action_post()
 
-        # You can't change the date to one being in a locked period.
-        with self.assertRaises(UserError), self.cr.savepoint():
-            copy_move.date = fields.Date.from_string('2017-01-01')
+        # Create a bank statement to get a balance in the suspense account.
+        statement = self.env['account.bank.statement'].create({
+            'journal_id': self.company_data['default_journal_bank'].id,
+            'date': '2016-01-01',
+            'line_ids': [
+                (0, 0, {'payment_ref': 'test', 'amount': 10.0})
+            ],
+        })
+        statement.button_post()
+
+        # You can't lock the fiscal year if there is some unreconciled statement.
+        with self.assertRaises(RedirectWarning), self.cr.savepoint():
+            self.test_move.company_id.fiscalyear_lock_date = fields.Date.from_string('2017-01-01')
 
     def test_misc_tax_lock_date_1(self):
-        self.test_move.post()
+        self.test_move.action_post()
 
         # Set the tax lock date after the journal entry date.
         self.test_move.company_id.tax_lock_date = fields.Date.from_string('2017-01-01')
@@ -234,10 +249,10 @@ class TestAccountMove(AccountTestInvoicingCommon):
         with self.assertRaises(UserError), self.cr.savepoint():
             self.test_move.button_draft()
 
-        copy_move = self.test_move.copy()
+        copy_move = self.test_move.copy({'date': self.test_move.date})
 
         # /!\ The date is changed automatically to the next available one during the post.
-        copy_move.post()
+        copy_move.action_post()
 
         # You can't change the date to one being in a locked period.
         with self.assertRaises(UserError), self.cr.savepoint():
@@ -246,7 +261,7 @@ class TestAccountMove(AccountTestInvoicingCommon):
     def test_misc_draft_reconciled_entries_1(self):
         draft_moves = self.env['account.move'].create([
             {
-                'type': 'entry',
+                'move_type': 'entry',
                 'line_ids': [
                     (0, None, {
                         'name': 'move 1 receivable line',
@@ -263,7 +278,7 @@ class TestAccountMove(AccountTestInvoicingCommon):
                 ]
             },
             {
-                'type': 'entry',
+                'move_type': 'entry',
                 'line_ids': [
                     (0, None, {
                         'name': 'move 2 receivable line',
@@ -285,6 +300,7 @@ class TestAccountMove(AccountTestInvoicingCommon):
         # lines[1] = 'move 1 counterpart line'
         # lines[2] = 'move 1 receivable line'
         # lines[3] = 'move 2 counterpart line'
+        draft_moves.action_post()
         lines = draft_moves.mapped('line_ids').sorted('balance')
 
         (lines[0] + lines[2]).reconcile()
@@ -327,22 +343,10 @@ class TestAccountMove(AccountTestInvoicingCommon):
         # You can remove journal items if the related journal entry is still balanced.
         self.test_move.line_ids.unlink()
 
-    def test_misc_unique_sequence_number(self):
-        ''' Ensure two journal entries can't share the same name when using the same sequence. '''
-        self.test_move.post()
-
-        # Edit the sequence to force the next move to get the same name.
-        self.test_move.journal_id\
-            .sequence_id.date_range_ids\
-            .filtered(lambda seq: seq.date_from == fields.Date.from_string('2016-01-01')).number_next -= 1
-
-        test_move2 = self.test_move.copy()
-        with self.assertRaises(ValidationError):
-            test_move2.post()
-
     def test_add_followers_on_post(self):
         # Add some existing partners, some from another company
         company = self.env['res.company'].create({'name': 'Oopo'})
+        company.flush()
         existing_partners = self.env['res.partner'].create([{
             'name': 'Jean',
             'company_id': company.id,
@@ -357,7 +361,7 @@ class TestAccountMove(AccountTestInvoicingCommon):
         partner = self.env['res.partner'].create({'name': 'Belouga'})
         move.partner_id = partner
 
-        move.post()
+        move.action_post()
         self.assertEqual(move.message_partner_ids, self.env.user.partner_id | existing_partners | partner)
 
     def test_misc_move_onchange(self):
@@ -485,3 +489,197 @@ class TestAccountMove(AccountTestInvoicingCommon):
         # You can remove journal items if the related journal entry is draft.
         self.test_move.button_draft()
         self.test_move.line_ids.unlink()
+
+    def test_account_move_inactive_currency_raise_error_on_post(self):
+        """ Ensure a move cannot be posted when using an inactive currency """
+        move = self.env['account.move'].create({
+            'move_type': 'in_invoice',
+            'partner_id': self.partner_a.id,
+            'invoice_date': fields.Date.from_string('2019-01-01'),
+            'currency_id': self.currency_data['currency'].id,
+            'invoice_payment_term_id': self.pay_terms_a.id,
+            'invoice_line_ids': [{}]
+        })
+
+        move.currency_id.active = False
+
+        with self.assertRaises(UserError), self.cr.savepoint():
+            move.action_post()
+
+        # Make sure that the invoice can still be posted when the currency is active
+        move.action_activate_currency()
+        move.action_post()
+
+        self.assertEqual(move.state, 'posted')
+
+    def test_invoice_like_entry_reverse_caba(self):
+        tax_waiting_account = self.env['account.account'].create({
+            'name': 'TAX_WAIT',
+            'code': 'TWAIT',
+            'user_type_id': self.env.ref('account.data_account_type_current_liabilities').id,
+            'reconcile': True,
+            'company_id': self.company_data['company'].id,
+        })
+        tax_final_account = self.env['account.account'].create({
+            'name': 'TAX_TO_DEDUCT',
+            'code': 'TDEDUCT',
+            'user_type_id': self.env.ref('account.data_account_type_current_assets').id,
+            'company_id': self.company_data['company'].id,
+        })
+        tax_base_amount_account = self.env['account.account'].create({
+            'name': 'TAX_BASE',
+            'code': 'TBASE',
+            'user_type_id': self.env.ref('account.data_account_type_current_assets').id,
+            'company_id': self.company_data['company'].id,
+        })
+        self.env.company.account_cash_basis_base_account_id = tax_base_amount_account
+        tax_tags = defaultdict(dict)
+        for line_type, repartition_type in [(l, r) for l in ('invoice', 'refund') for r in ('base', 'tax')]:
+            tax_tags[line_type][repartition_type] = self.env['account.account.tag'].create({
+                'name': '%s %s tag' % (line_type, repartition_type),
+                'applicability': 'taxes',
+                'country_id': self.env.ref('base.us').id,
+            })
+        tax = self.env['account.tax'].create({
+            'name': 'cash basis 10%',
+            'type_tax_use': 'sale',
+            'amount': 10,
+            'tax_exigibility': 'on_payment',
+            'cash_basis_transition_account_id': tax_waiting_account.id,
+            'invoice_repartition_line_ids': [
+                (0, 0, {
+                    'factor_percent': 100,
+                    'repartition_type': 'base',
+                    'tag_ids': [(6, 0, tax_tags['invoice']['base'].ids)],
+                }),
+                (0, 0, {
+                    'factor_percent': 100,
+                    'repartition_type': 'tax',
+                    'account_id': tax_final_account.id,
+                    'tag_ids': [(6, 0, tax_tags['invoice']['tax'].ids)],
+                }),
+            ],
+            'refund_repartition_line_ids': [
+                (0, 0, {
+                    'factor_percent': 100,
+                    'repartition_type': 'base',
+                    'tag_ids': [(6, 0, tax_tags['refund']['base'].ids)],
+                }),
+                (0, 0, {
+                    'factor_percent': 100,
+                    'repartition_type': 'tax',
+                    'account_id': tax_final_account.id,
+                    'tag_ids': [(6, 0, tax_tags['refund']['tax'].ids)],
+                }),
+            ],
+        })
+        move = self.env['account.move'].create({
+            'move_type': 'entry',
+            'date': fields.Date.from_string('2016-01-01'),
+            'line_ids': [
+                (0, None, {
+                    'name': 'revenue line',
+                    'account_id': self.company_data['default_account_revenue'].id,
+                    'debit': 0.0,
+                    'credit': 1000.0,
+                    'tax_ids': [(6, 0, tax.ids)],
+                    'tax_tag_ids': [(6, 0, tax_tags['invoice']['base'].ids)],
+                }),
+                (0, None, {
+                    'name': 'tax line 1',
+                    'account_id': tax_waiting_account.id,
+                    'debit': 0.0,
+                    'credit': 100.0,
+                    'tax_tag_ids': [(6, 0, tax_tags['invoice']['tax'].ids)],
+                    'tax_repartition_line_id': tax.invoice_repartition_line_ids.filtered(lambda x: x.repartition_type == 'tax').id,
+                }),
+                (0, None, {
+                    'name': 'counterpart line',
+                    'account_id': self.company_data['default_account_receivable'].id,
+                    'debit': 1100.0,
+                    'credit': 0.0,
+                }),
+            ]
+        })
+        move.action_post()
+        # make payment
+        payment = self.env['account.payment'].create({
+            'payment_type': 'inbound',
+            'payment_method_id': self.env.ref('account.account_payment_method_manual_in').id,
+            'partner_type': 'customer',
+            'partner_id': self.partner_a.id,
+            'amount': 1100,
+            'date': move.date,
+            'journal_id': self.company_data['default_journal_bank'].id,
+        })
+        payment.action_post()
+        (payment.move_id + move).line_ids.filtered(lambda x: x.account_id == self.company_data['default_account_receivable']).reconcile()
+        # check caba move
+        partial_rec = move.mapped('line_ids.matched_credit_ids')
+        caba_move = self.env['account.move'].search([('tax_cash_basis_rec_id', '=', partial_rec.id)])
+        expected_values = [
+            {
+                'tax_line_id': False,
+                'tax_repartition_line_id': False,
+                'tax_ids': [],
+                'tax_tag_ids': [],
+                'account_id': tax_base_amount_account.id,
+                'debit': 1000.0,
+                'credit': 0.0,
+            },
+            {
+                'tax_line_id': False,
+                'tax_repartition_line_id': False,
+                'tax_ids': tax.ids,
+                'tax_tag_ids': tax_tags['invoice']['base'].ids,
+                'account_id': tax_base_amount_account.id,
+                'debit': 0.0,
+                'credit': 1000.0,
+            },
+
+            {
+                'tax_line_id': False,
+                'tax_repartition_line_id': False,
+                'tax_ids': [],
+                'tax_tag_ids': [],
+                'account_id': tax_waiting_account.id,
+                'debit': 100.0,
+                'credit': 0.0,
+            },
+            {
+                'tax_line_id': tax.id,
+                'tax_repartition_line_id': tax.invoice_repartition_line_ids.filtered(lambda x: x.repartition_type == 'tax').id,
+                'tax_ids': [],
+                'tax_tag_ids': tax_tags['invoice']['tax'].ids,
+                'account_id': tax_final_account.id,
+                'debit': 0.0,
+                'credit': 100.0,
+            },
+        ]
+        self.assertRecordValues(caba_move.line_ids, expected_values)
+        # unreconcile
+        debit_aml = move.line_ids.filtered('debit')
+        debit_aml.remove_move_reconcile()
+        # check caba move reverse is same as caba move with only debit/credit inverted
+        reversed_caba_move = self.env['account.move'].search([('reversed_entry_id', '=', caba_move.id)])
+        for value in expected_values:
+            value.update({
+                'debit': value['credit'],
+                'credit': value['debit'],
+            })
+        self.assertRecordValues(reversed_caba_move.line_ids, expected_values)
+
+    def _get_cache_count(self, model_name='account.move', field_name='name'):
+        model = self.env[model_name]
+        field = model._fields[field_name]
+        return len(self.env.cache.get_records(model, field))
+
+    def test_cache_invalidation(self):
+        self.env['account.move'].invalidate_cache()
+        lines = self.test_move.line_ids
+        # prefetch
+        lines.mapped('move_id.name')
+        # check account.move cache
+        self.assertEqual(self._get_cache_count(), 1)
+        self.env['account.move.line'].invalidate_cache(ids=lines.ids)
+        self.assertEqual(self._get_cache_count(), 0)
