@@ -1,92 +1,99 @@
 # -*- coding: utf-8 -*-
-from odoo.addons.account.tests.account_test_users import AccountTestUsers
+from odoo.addons.account.tests.common import AccountTestInvoicingCommon
 from odoo.tests import tagged
 
 
 @tagged('post_install', '-at_install')
-class TestTax(AccountTestUsers):
+class TestTaxCommon(AccountTestInvoicingCommon):
 
-    def setUp(self):
-        super(TestTax, self).setUp()
+    @classmethod
+    def setUpClass(cls, chart_template_ref=None):
+        super().setUpClass(chart_template_ref=chart_template_ref)
 
-        self.fixed_tax = self.tax_model.create({
+        # Setup another company having a rounding of 1.0.
+        cls.currency_data['currency'].rounding = 1.0
+        cls.currency_no_decimal = cls.currency_data['currency']
+        cls.company_data_2 = cls.setup_company_data('company_2', currency_id=cls.currency_no_decimal.id)
+        cls.env.user.company_id = cls.company_data['company']
+
+        cls.fixed_tax = cls.env['account.tax'].create({
             'name': "Fixed tax",
             'amount_type': 'fixed',
             'amount': 10,
             'sequence': 1,
         })
-        self.fixed_tax_bis = self.tax_model.create({
+        cls.fixed_tax_bis = cls.env['account.tax'].create({
             'name': "Fixed tax bis",
             'amount_type': 'fixed',
             'amount': 15,
             'sequence': 2,
         })
-        self.percent_tax = self.tax_model.create({
+        cls.percent_tax = cls.env['account.tax'].create({
             'name': "Percent tax",
             'amount_type': 'percent',
             'amount': 10,
             'sequence': 3,
         })
-        self.percent_tax_bis = self.tax_model.create({
+        cls.percent_tax_bis = cls.env['account.tax'].create({
             'name': "Percent tax bis",
             'amount_type': 'percent',
             'amount': 10,
             'sequence': 4,
         })
-        self.division_tax = self.tax_model.create({
+        cls.division_tax = cls.env['account.tax'].create({
             'name': "Division tax",
             'amount_type': 'division',
             'amount': 10,
             'sequence': 4,
         })
-        self.group_tax = self.tax_model.create({
+        cls.group_tax = cls.env['account.tax'].create({
             'name': "Group tax",
             'amount_type': 'group',
             'amount': 0,
             'sequence': 5,
             'children_tax_ids': [
-                (4, self.fixed_tax.id, 0),
-                (4, self.percent_tax.id, 0)
+                (4, cls.fixed_tax.id, 0),
+                (4, cls.percent_tax.id, 0)
             ]
         })
-        self.group_tax_bis = self.tax_model.create({
+        cls.group_tax_bis = cls.env['account.tax'].create({
             'name': "Group tax bis",
             'amount_type': 'group',
             'amount': 0,
             'sequence': 6,
             'children_tax_ids': [
-                (4, self.fixed_tax.id, 0),
-                (4, self.percent_tax.id, 0)
+                (4, cls.fixed_tax.id, 0),
+                (4, cls.percent_tax.id, 0)
             ]
         })
-        self.group_tax_percent = self.tax_model.create({
+        cls.group_tax_percent = cls.env['account.tax'].create({
             'name': "Group tax percent",
             'amount_type': 'group',
             'amount': 0,
             'sequence': 6,
             'children_tax_ids': [
-                (4, self.percent_tax.id, 0),
-                (4, self.percent_tax_bis.id, 0)
+                (4, cls.percent_tax.id, 0),
+                (4, cls.percent_tax_bis.id, 0)
             ]
         })
-        self.group_of_group_tax = self.tax_model.create({
+        cls.group_of_group_tax = cls.env['account.tax'].create({
             'name': "Group of group tax",
             'amount_type': 'group',
             'amount': 0,
             'sequence': 7,
             'children_tax_ids': [
-                (4, self.group_tax.id, 0),
-                (4, self.group_tax_bis.id, 0)
+                (4, cls.group_tax.id, 0),
+                (4, cls.group_tax_bis.id, 0)
             ]
         })
-        self.tax_with_no_account = self.tax_model.create({
+        cls.tax_with_no_account = cls.env['account.tax'].create({
             'name': "Tax with no account",
             'amount_type': 'fixed',
             'amount': 0,
             'sequence': 8,
         })
-        some_account = self.env['account.account'].search([], limit=1)
-        self.tax_with_account = self.tax_model.create({
+        some_account = cls.env['account.account'].search([], limit=1)
+        cls.tax_with_account = cls.env['account.tax'].create({
             'name': "Tax with account",
             'amount_type': 'fixed',
             'amount': 0,
@@ -117,49 +124,50 @@ class TestTax(AccountTestUsers):
             ],
         })
 
-        self.tax_0_percent = self.tax_model.create({
+        cls.tax_0_percent = cls.env['account.tax'].with_company(cls.company_data['company']).create({
             'name': "test_0_percent",
             'amount_type': 'percent',
             'amount': 0,
         })
 
-        self.tax_5_percent = self.tax_model.create({
+        cls.tax_5_percent = cls.env['account.tax'].with_company(cls.company_data['company']).create({
             'name': "test_5_percent",
             'amount_type': 'percent',
             'amount': 5,
         })
 
-        self.tax_8_percent = self.tax_model.create({
+        cls.tax_8_percent = cls.env['account.tax'].with_company(cls.company_data['company']).create({
             'name': "test_8_percent",
             'amount_type': 'percent',
             'amount': 8,
         })
-
-        self.tax_12_percent = self.tax_model.create({
+        cls.tax_12_percent = cls.env['account.tax'].with_company(cls.company_data['company']).create({
             'name': "test_12_percent",
             'amount_type': 'percent',
             'amount': 12,
         })
 
-        self.tax_19_percent = self.tax_model.create({
+        cls.tax_19_percent = cls.env['account.tax'].with_company(cls.company_data_2['company']).create({
             'name': "test_19_percent",
             'amount_type': 'percent',
             'amount': 19,
         })
 
-        self.tax_21_percent = self.tax_model.create({
+        cls.tax_21_percent = cls.env['account.tax'].with_company(cls.company_data['company']).create({
             'name': "test_21_percent",
             'amount_type': 'percent',
+            'amount': 19,
+        })
+
+        cls.tax_21_percent = cls.env['account.tax'].with_company(cls.company_data['company']).create({
+            'name': "test_rounding_methods_2",
+            'amount_type': 'percent',
             'amount': 21,
         })
 
-        self.bank_journal = self.env['account.journal'].search([('type', '=', 'bank'), ('company_id', '=', self.account_manager.company_id.id)])[0]
-        self.bank_account = self.bank_journal.default_debit_account_id
-        self.expense_account = self.env['account.account'].search([('user_type_id.type', '=', 'payable')], limit=1) #Should be done by onchange later
-
-        # Ensure the rounding method is 'round_per_line' to avoid issue when launching the tests on
-        # an existing DB.
-        self.env.user.company_id.tax_calculation_rounding_method = 'round_per_line'
+        cls.bank_journal = cls.company_data['default_journal_bank']
+        cls.bank_account = cls.bank_journal.default_account_id
+        cls.expense_account = cls.company_data['default_account_expense']
 
     def _check_compute_all_results(self, total_included, total_excluded, taxes, res):
         self.assertAlmostEqual(res['total_included'], total_included)
@@ -168,6 +176,14 @@ class TestTax(AccountTestUsers):
             self.assertAlmostEqual(res['taxes'][i]['base'], taxes[i][0])
             self.assertAlmostEqual(res['taxes'][i]['amount'], taxes[i][1])
 
+
+@tagged('post_install', '-at_install')
+class TestTax(TestTaxCommon):
+
+    @classmethod
+    def setUpClass(cls):
+        super(TestTax, cls).setUpClass()
+
     def test_tax_group_of_group_tax(self):
         self.fixed_tax.include_base_amount = True
         res = self.group_of_group_tax.compute_all(200.0)
@@ -741,12 +757,11 @@ class TestTax(AccountTestUsers):
             res2
         )
 
-    def test_rounding_tax_excluded_round_globally_01(self):
+    def test_rounding_tax_excluded_round_globally(self):
         ''' Test the rounding of a 19% price excluded tax in an invoice having 22689 and 9176 as lines.
         The decimal precision is set to zero.
         The computation must be similar to round((22689 + 9176) * 0.19).
         '''
-        self.tax_19_percent.company_id.currency_id.rounding = 1.0
         self.tax_19_percent.company_id.tax_calculation_rounding_method = 'round_globally'
 
         res1 = self.tax_19_percent.compute_all(22689)
@@ -879,8 +894,8 @@ class TestTax(AccountTestUsers):
 
         res1 = self.tax_5_percent.compute_all(5)
         self._check_compute_all_results(
-            5,          # 'total_included'
-            4.75,       # 'total_excluded'
+            5,      # 'total_included'
+            4.75,      # 'total_excluded'
             [
                 # base , amount
                 # ---------------
@@ -892,8 +907,8 @@ class TestTax(AccountTestUsers):
 
         res2 = self.tax_5_percent.compute_all(10)
         self._check_compute_all_results(
-            10,         # 'total_included'
-            9.5,        # 'total_excluded'
+            10,      # 'total_included'
+            9.5,      # 'total_excluded'
             [
                 # base , amount
                 # ---------------
@@ -905,8 +920,8 @@ class TestTax(AccountTestUsers):
 
         res3 = self.tax_5_percent.compute_all(50)
         self._check_compute_all_results(
-            50,         # 'total_included'
-            47.6,       # 'total_excluded'
+            50,      # 'total_included'
+            47.6,      # 'total_excluded'
             [
                 # base , amount
                 # ---------------
@@ -922,7 +937,6 @@ class TestTax(AccountTestUsers):
         The computation must be similar to round((27000 + 10920) / 1.19).
         '''
         self.tax_19_percent.price_include = True
-        self.tax_19_percent.company_id.currency_id.rounding = 1.0
         self.tax_19_percent.company_id.tax_calculation_rounding_method = 'round_globally'
 
         res1 = self.tax_19_percent.compute_all(27000)
@@ -995,8 +1009,8 @@ class TestTax(AccountTestUsers):
 
         res1 = self.tax_5_percent.compute_all(5)
         self._check_compute_all_results(
-            5,          # 'total_included'
-            4.75,       # 'total_excluded'
+            5,      # 'total_included'
+            4.75,      # 'total_excluded'
             [
                 # base , amount
                 # ---------------
@@ -1008,8 +1022,8 @@ class TestTax(AccountTestUsers):
 
         res2 = self.tax_5_percent.compute_all(10)
         self._check_compute_all_results(
-            10,         # 'total_included'
-            9.5,        # 'total_excluded'
+            10,      # 'total_included'
+            9.5,      # 'total_excluded'
             [
                 # base , amount
                 # ---------------
@@ -1021,8 +1035,8 @@ class TestTax(AccountTestUsers):
 
         res3 = self.tax_5_percent.compute_all(50)
         self._check_compute_all_results(
-            50,         # 'total_included'
-            47.6,       # 'total_excluded'
+            50,      # 'total_included'
+            47.6,      # 'total_excluded'
             [
                 # base , amount
                 # ---------------
@@ -1032,12 +1046,76 @@ class TestTax(AccountTestUsers):
             res3
         )
 
+    def test_is_base_affected(self):
+        taxes = self.env['account.tax'].create([{
+            'name': 'test_is_base_affected%s' % i,
+            'amount_type': 'percent',
+            'amount': amount,
+            'include_base_amount': include_base_amount,
+            'is_base_affected': is_base_affected,
+            'sequence': i,
+        } for i, amount, include_base_amount, is_base_affected in [
+            (0, 6, True, True),
+            (1, 6, True, False),
+            (2, 10, False, True),
+        ]])
+
+        compute_all_results = taxes.compute_all(100.0)
+
+        # Check the balance of the generated move lines
+        self._check_compute_all_results(
+            123.2,      # 'total_included'
+            100.0,      # 'total_excluded'
+            [
+                # base, amount
+                # -------------------------
+                (100.0, 6.0),
+                (100.0, 6.0),
+                (112.0, 11.2),
+                # -------------------------
+            ],
+            compute_all_results,
+        )
+
+        # Check the tax_ids on tax lines
+        expected_tax_ids_list = [taxes[2].ids, taxes[2].ids, []]
+        tax_ids_list = [tax_line['tax_ids'] for tax_line in compute_all_results['taxes']]
+        self.assertEqual(tax_ids_list, expected_tax_ids_list, "Only a tax affected by previous taxes should have tax_ids set on its tax line when used after an 'include_base_amount' tax.")
+
+    def test_mixing_price_included_excluded_with_affect_base(self):
+        tax_10_fix = self.env['account.tax'].create({
+            'name': "tax_10_fix",
+            'amount_type': 'fixed',
+            'amount': 10.0,
+            'include_base_amount': True,
+        })
+        tax_21 = self.env['account.tax'].create({
+            'name': "tax_21",
+            'amount_type': 'percent',
+            'amount': 21.0,
+            'price_include': True,
+            'include_base_amount': True,
+        })
+
+        self._check_compute_all_results(
+            1222.1,     # 'total_included'
+            1000.0,     # 'total_excluded'
+            [
+                # base , amount
+                # ---------------
+                (1000.0, 10.0),
+                (1010.0, 212.1),
+                # ---------------
+            ],
+            (tax_10_fix + tax_21).compute_all(1210),
+        )
+
     def test_price_included_repartition_sum_0(self):
         """ Tests the case where a tax with a non-zero value has a sum
         of tax repartition factors of zero and is included in price. It
         shouldn't behave in the same way as a 0% tax.
         """
-        test_tax = self.tax_model.create({
+        test_tax = self.env['account.tax'].create({
             'name': "Definitely not a 0% tax",
             'amount_type': 'percent',
             'amount': 42,
