@@ -2,7 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 
-from . import account_accrual_accounting
+from . import account_automatic_entry_wizard
 from . import account_unreconcile
 from . import account_validate_account_move
 from . import pos_box
@@ -10,7 +10,11 @@ from . import account_move_reversal
 from . import account_report_common
 from . import account_report_common_journal
 from . import account_report_print_journal
+from . import account_resequence
 from . import setup_wizards
 from . import wizard_tax_adjustments
 from . import account_invoice_send
 from . import base_document_layout
+from . import account_payment_register
+from . import account_tour_upload_bill
+from . import accrued_orders
