@@ -11,16 +11,23 @@ class TaxAdjustments(models.TransientModel):
     def _get_default_journal(self):
         return self.env['account.journal'].search([('type', '=', 'general')], limit=1).id
 
+    def _domain_tax_report(self):
+        fiscal_country_ids = self.env['account.fiscal.position'].search([('company_id', '=', self.env.company.id), ('foreign_vat', '!=', False)]).country_id.ids
+        return [('tag_name', '!=', None), '|', ('report_id.country_id', '=', self.env.company.country_id.id), ('report_id.country_id', 'in', fiscal_country_ids)]
+
     reason = fields.Char(string='Justification', required=True)
     journal_id = fields.Many2one('account.journal', string='Journal', required=True, default=_get_default_journal, domain=[('type', '=', 'general')])
     date = fields.Date(required=True, default=fields.Date.context_today)
-    debit_account_id = fields.Many2one('account.account', string='Debit account', required=True, domain=[('deprecated', '=', False)])
-    credit_account_id = fields.Many2one('account.account', string='Credit account', required=True, domain=[('deprecated', '=', False)])
+    debit_account_id = fields.Many2one('account.account', string='Debit account', required=True,
+                                       domain="[('deprecated', '=', False), ('is_off_balance', '=', False)]")
+    credit_account_id = fields.Many2one('account.account', string='Credit account', required=True,
+                                        domain="[('deprecated', '=', False), ('is_off_balance', '=', False)]")
     amount = fields.Monetary(currency_field='company_currency_id', required=True)
     adjustment_type = fields.Selection([('debit', 'Applied on debit journal item'), ('credit', 'Applied on credit journal item')], string="Adjustment Type", required=True)
-    tax_report_line_id = fields.Many2one(string="Report Line", comodel_name='account.tax.report.line', required=True, help="The report line to make an adjustment for.")
+    tax_report_line_id = fields.Many2one(string="Report Line", comodel_name='account.tax.report.line', required=True, help="The report line to make an adjustment for.",
+                                         domain=_domain_tax_report)
     company_currency_id = fields.Many2one('res.currency', readonly=True, default=lambda x: x.env.company.currency_id)
-    country_id = fields.Many2one(string="Country", comodel_name='res.country', readonly=True, default=lambda x: x.env.company.country_id)
+    report_id = fields.Many2one(string="Report", related='tax_report_line_id.report_id')
 
     def create_move(self):
         move_line_vals = []
@@ -36,7 +43,7 @@ class TaxAdjustments(models.TransientModel):
             'debit': is_debit and abs(self.amount) or 0,
             'credit': not is_debit and abs(self.amount) or 0,
             'account_id': is_debit and self.debit_account_id.id or self.credit_account_id.id,
-            'tag_ids': [(6, False, [adjustment_tag.id])],
+            'tax_tag_ids': [(6, False, [adjustment_tag.id])],
         }))
 
         # Vals for the counterpart line
@@ -55,11 +62,10 @@ class TaxAdjustments(models.TransientModel):
             'line_ids': move_line_vals,
         }
         move = self.env['account.move'].create(vals)
-        move.post()
+        move._post()
 
         # Return an action opening the created move
-        action = self.env.ref(self.env.context.get('action', 'account.action_move_line_form'))
-        result = action.read()[0]
+        result = self.env['ir.actions.act_window']._for_xml_id('account.action_move_line_form')
         result['views'] = [(False, 'form')]
         result['res_id'] = move.id
         return result
