@@ -3,7 +3,7 @@
 
 {
     'name': 'OAuth2 Authentication',
-    'category': 'Tools',
+    'category': 'Hidden/Tools',
     'description': """
 Allow users to login through OAuth2 Provider.
 =============================================
@@ -18,5 +18,10 @@ Allow users to login through OAuth2 Provider.
         'views/auth_oauth_templates.xml',
         'security/ir.model.access.csv',
     ],
+    'assets': {
+        'web.assets_frontend': [
+            'auth_oauth/static/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
