@@ -2,19 +2,18 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import odoo
-from odoo import api, fields, models, tools, SUPERUSER_ID, _
+from odoo import api, fields, models, tools, SUPERUSER_ID, _, Command
 from odoo.exceptions import MissingError, UserError, ValidationError, AccessError
 from odoo.osv import expression
 from odoo.tools.safe_eval import safe_eval, test_python_expr
 from odoo.tools.float_utils import float_compare
+from odoo.http import request
 
 import base64
 from collections import defaultdict
-import datetime
+import functools
 import logging
-import time
 
-import dateutil
 from pytz import timezone
 
 _logger = logging.getLogger(__name__)
@@ -67,56 +66,108 @@ class IrActions(models.Model):
         self.clear_caches()
         return res
 
+    @api.ondelete(at_uninstall=True)
+    def _unlink_check_home_action(self):
+        self.env['res.users'].with_context(active_test=False).search([('action_id', 'in', self.ids)]).sudo().write({'action_id': None})
+
     @api.model
     def _get_eval_context(self, action=None):
         """ evaluation context to pass to safe_eval """
         return {
             'uid': self._uid,
             'user': self.env.user,
-            'time': time,
-            'datetime': datetime,
-            'dateutil': dateutil,
+            'time': tools.safe_eval.time,
+            'datetime': tools.safe_eval.datetime,
+            'dateutil': tools.safe_eval.dateutil,
             'timezone': timezone,
             'float_compare': float_compare,
             'b64encode': base64.b64encode,
             'b64decode': base64.b64decode,
+            'Command': Command,
         }
 
     @api.model
-    @tools.ormcache('frozenset(self.env.user.groups_id.ids)', 'model_name')
     def get_bindings(self, model_name):
+        return self._get_bindings(model_name, bool(request) and request.session.debug)
+
+    @tools.ormcache('frozenset(self.env.user.groups_id.ids)', 'model_name', 'debug')
+    def _get_bindings(self, model_name, debug=False):
         """ Retrieve the list of actions bound to the given model.
 
            :return: a dict mapping binding types to a list of dict describing
                     actions, where the latter is given by calling the method
                     ``read`` on the action record.
         """
-        # DLE P19: Need to flush before doing the SELECT, which act as a search.
-        # Test `test_bindings`
-        self.flush()
         cr = self.env.cr
-        query = """ SELECT a.id, a.type, a.binding_type
-                    FROM ir_actions a, ir_model m
-                    WHERE a.binding_model_id=m.id AND m.model=%s
-                    ORDER BY a.id """
-        cr.execute(query, [model_name])
+        IrModelAccess = self.env['ir.model.access']
 
         # discard unauthorized actions, and read action definitions
         result = defaultdict(list)
         user_groups = self.env.user.groups_id
+        if not debug:
+            user_groups -= self.env.ref('base.group_no_one')
+
+        self.flush()
+        cr.execute("""
+            SELECT a.id, a.type, a.binding_type
+              FROM ir_actions a
+              JOIN ir_model m ON a.binding_model_id = m.id
+             WHERE m.model = %s
+          ORDER BY a.id
+        """, [model_name])
         for action_id, action_model, binding_type in cr.fetchall():
             try:
-                action = self.env[action_model].browse(action_id)
+                action = self.env[action_model].sudo().browse(action_id)
                 action_groups = getattr(action, 'groups_id', ())
+                action_model = getattr(action, 'res_model', False)
                 if action_groups and not action_groups & user_groups:
                     # the user may not perform this action
                     continue
-                result[binding_type].append(action.read()[0])
+                if action_model and not IrModelAccess.check(action_model, mode='read', raise_exception=False):
+                    # the user won't be able to read records
+                    continue
+                fields = ['name', 'binding_view_types']
+                if 'sequence' in action._fields:
+                    fields.append('sequence')
+                result[binding_type].append(action.read(fields)[0])
             except (AccessError, MissingError):
                 continue
 
+        # sort actions by their sequence if sequence available
+        if result.get('action'):
+            result['action'] = sorted(result['action'], key=lambda vals: vals.get('sequence', 0))
         return result
 
+    @api.model
+    def _for_xml_id(self, full_xml_id):
+        """ Returns the action content for the provided xml_id
+
+        :param xml_id: the namespace-less id of the action (the @id
+                       attribute from the XML file)
+        :return: A read() view of the ir.actions.action safe for web use
+        """
+        record = self.env.ref(full_xml_id)
+        assert isinstance(self.env[record._name], type(self))
+        action = record.sudo().read()[0]
+        return {
+            field: value
+            for field, value in action.items()
+            if field in record._get_readable_fields()
+        }
+
+    def _get_readable_fields(self):
+        """ return the list of fields that are safe to read
+
+        Fetched via /web/action/load or _for_xml_id method
+        Only fields used by the web client should included
+        Accessing content useful for the server-side must
+        be done manually with superuser
+        """
+        return {
+            "binding_model_id", "binding_type", "binding_view_types",
+            "display_name", "help", "id", "name", "type", "xml_id",
+        }
+
 
 class IrActionsActWindow(models.Model):
     _name = 'ir.actions.act_window'
@@ -130,9 +181,9 @@ class IrActionsActWindow(models.Model):
     def _check_model(self):
         for action in self:
             if action.res_model not in self.env:
-                raise ValidationError(_('Invalid model name %r in action definition.') % action.res_model)
+                raise ValidationError(_('Invalid model name %r in action definition.', action.res_model))
             if action.binding_model_id and action.binding_model_id.model not in self.env:
-                raise ValidationError(_('Invalid model name %r in action definition.') % action.binding_model_id.model)
+                raise ValidationError(_('Invalid model name %r in action definition.', action.binding_model_id.model))
 
     @api.depends('view_ids.view_mode', 'view_mode', 'view_id.type')
     def _compute_views(self):
@@ -156,6 +207,15 @@ class IrActionsActWindow(models.Model):
                     act.views.append((act.view_id.id, act.view_id.type))
                 act.views.extend([(False, mode) for mode in missing_modes])
 
+    @api.constrains('view_mode')
+    def _check_view_mode(self):
+        for rec in self:
+            modes = rec.view_mode.split(',')
+            if len(modes) != len(set(modes)):
+                raise ValidationError(_('The modes in view_mode must not be duplicated: %s', modes))
+            if ' ' in modes:
+                raise ValidationError(_('No spaces allowed in view_mode: %r', modes))
+
     @api.depends('res_model', 'search_view_id')
     def _compute_search_view(self):
         for act in self:
@@ -205,18 +265,6 @@ class IrActionsActWindow(models.Model):
                     values['help'] = self.with_context(**ctx).env[model].get_empty_list_help(values.get('help', ''))
         return result
 
-    @api.model
-    def for_xml_id(self, module, xml_id):
-        """ Returns the act_window object created for the provided xml_id
-
-        :param module: the module the act_window originates in
-        :param xml_id: the namespace-less id of the action (the @id
-                       attribute from the XML file)
-        :return: A read() view of the ir.actions.act_window
-        """
-        record = self.env.ref("%s.%s" % (module, xml_id))
-        return record.read()[0]
-
     @api.model_create_multi
     def create(self, vals_list):
         self.clear_caches()
@@ -241,6 +289,17 @@ class IrActionsActWindow(models.Model):
         return set(row[0] for row in self._cr.fetchall())
 
 
+    def _get_readable_fields(self):
+        return super()._get_readable_fields() | {
+            "context", "domain", "filter", "groups_id", "limit", "res_id",
+            "res_model", "search_view", "search_view_id", "target", "view_id",
+            "view_mode", "views",
+            # `flags` is not a real field of ir.actions.act_window but is used
+            # to give the parameters to generate the action
+            "flags"
+        }
+
+
 VIEW_TYPES = [
     ('tree', 'Tree'),
     ('form', 'Form'),
@@ -279,6 +338,13 @@ class IrActionsActWindowclose(models.Model):
 
     type = fields.Char(default='ir.actions.act_window_close')
 
+    def _get_readable_fields(self):
+        return super()._get_readable_fields() | {
+            # 'effect' is not a real field of ir.actions.act_window_close but is
+            # used to display the rainbowman
+            "effect"
+        }
+
 
 class IrActionsActUrl(models.Model):
     _name = 'ir.actions.act_url'
@@ -294,6 +360,11 @@ class IrActionsActUrl(models.Model):
     target = fields.Selection([('new', 'New Window'), ('self', 'This Window')],
                               string='Action Target', default='new', required=True)
 
+    def _get_readable_fields(self):
+        return super()._get_readable_fields() | {
+            "target", "url",
+        }
+
 
 class IrActionsServer(models.Model):
     """ Server actions model. Server action work on a base model and offer various
@@ -329,14 +400,10 @@ class IrActionsServer(models.Model):
 #  - time, datetime, dateutil, timezone: useful Python libraries
 #  - float_compare: Odoo function to compare floats based on specific precisions
 #  - log: log(message, level='info'): logging function to record debug information in ir.logging table
-#  - Warning: Warning Exception to use with raise
+#  - UserError: Warning Exception to use with raise
+#  - Command: x2Many commands namespace
 # To return an action, assign: action = {...}\n\n\n\n"""
 
-    @api.model
-    def _select_objects(self):
-        records = self.env['ir.model'].search([])
-        return [(record.model, record.name) for record in records] + [('', '')]
-
     name = fields.Char(string='Action Name', translate=True)
     type = fields.Char(default='ir.actions.server')
     usage = fields.Selection([
@@ -348,7 +415,7 @@ class IrActionsServer(models.Model):
         ('object_create', 'Create a new Record'),
         ('object_write', 'Update the Record'),
         ('multi', 'Execute several actions')], string='Action To Do',
-        default='object_write', required=True,
+        default='object_write', required=True, copy=True,
         help="Type of server action. The following values are available:\n"
              "- 'Execute Python Code': a block of python code that will be executed\n"
              "- 'Create': create a new record with new values\n"
@@ -373,12 +440,12 @@ class IrActionsServer(models.Model):
     child_ids = fields.Many2many('ir.actions.server', 'rel_server_actions', 'server_id', 'action_id',
                                  string='Child Actions', help='Child server actions that will be executed. Note that the last return returned action value will be used as global return value.')
     # Create
-    crud_model_id = fields.Many2one('ir.model', string='Create/Write Target Model',
+    crud_model_id = fields.Many2one('ir.model', string='Target Model',
                                     help="Model for record creation / update. Set this field only to specify a different model than the base model.")
-    crud_model_name = fields.Char(related='crud_model_id.model', string='Target Model', readonly=True)
-    link_field_id = fields.Many2one('ir.model.fields', string='Link using field',
+    crud_model_name = fields.Char(related='crud_model_id.model', string='Target Model Name', readonly=True)
+    link_field_id = fields.Many2one('ir.model.fields', string='Link Field',
                                     help="Provide the field used to link the newly created record "
-                                         "on the record on used by the server action.")
+                                         "on the record used by the server action.")
     fields_lines = fields.One2many('ir.server.object.lines', 'server_id', string='Value Mapping', copy=True)
     groups_id = fields.Many2many('res.groups', 'ir_act_server_group_rel',
                                  'act_id', 'gid', string='Groups')
@@ -395,14 +462,38 @@ class IrActionsServer(models.Model):
         if not self._check_m2m_recursion('child_ids'):
             raise ValidationError(_('Recursion found in child server actions'))
 
+    def _get_readable_fields(self):
+        return super()._get_readable_fields() | {
+            "groups_id", "model_name",
+        }
+
+    def _get_runner(self):
+        multi = True
+        t = type(self)
+        fn = getattr(t, f'_run_action_{self.state}_multi', None)\
+          or getattr(t, f'run_action_{self.state}_multi', None)
+        if not fn:
+            multi = False
+            fn = getattr(t, f'_run_action_{self.state}', None)\
+              or getattr(t, f'run_action_{self.state}', None)
+        if fn and fn.__name__.startswith('run_action_'):
+            fn = functools.partial(fn, self)
+        return fn, multi
+
+    def _register_hook(self):
+        super()._register_hook()
+
+        for cls in type(self).mro():
+            for symbol in vars(cls).keys():
+                if symbol.startswith('run_action_'):
+                    _logger.warning(
+                        "RPC-public action methods are deprecated, found %r (in class %s.%s)",
+                        symbol, cls.__module__, cls.__name__
+                    )
+
     @api.onchange('crud_model_id')
     def _onchange_crud_model_id(self):
         self.link_field_id = False
-        self.crud_model_name = self.crud_model_id.model
-
-    @api.onchange('model_id')
-    def _onchange_model_id(self):
-        self.model_name = self.model_id.model
 
     def create_action(self):
         """ Create a contextual action for each server action. """
@@ -417,55 +508,45 @@ class IrActionsServer(models.Model):
         self.filtered('binding_model_id').write({'binding_model_id': False})
         return True
 
-    @api.model
-    def run_action_code_multi(self, action, eval_context=None):
-        safe_eval(action.sudo().code.strip(), eval_context, mode="exec", nocopy=True)  # nocopy allows to return 'action'
-        if 'action' in eval_context:
-            return eval_context['action']
+    def _run_action_code_multi(self, eval_context):
+        safe_eval(self.code.strip(), eval_context, mode="exec", nocopy=True)  # nocopy allows to return 'action'
+        return eval_context.get('action')
 
-    @api.model
-    def run_action_multi(self, action, eval_context=None):
+    def _run_action_multi(self, eval_context=None):
         res = False
-        for act in action.child_ids.sorted():
-            result = act.run()
-            if result:
-                res = result
+        for act in self.child_ids.sorted():
+            res = act.run() or res
         return res
 
-    @api.model
-    def run_action_object_write(self, action, eval_context=None):
+    def _run_action_object_write(self, eval_context=None):
         """Apply specified write changes to active_id."""
-        res = {}
-        for exp in action.fields_lines:
-            res[exp.col1.name] = exp.eval_value(eval_context=eval_context)[exp.id]
+        vals = self.fields_lines.eval_value(eval_context=eval_context)
+        res = {line.col1.name: vals[line.id] for line in self.fields_lines}
 
         if self._context.get('onchange_self'):
             record_cached = self._context['onchange_self']
             for field, new_value in res.items():
                 record_cached[field] = new_value
         else:
-            self.env[action.model_id.model].browse(self._context.get('active_id')).write(res)
+            self.env[self.model_id.model].browse(self._context.get('active_id')).write(res)
 
-    @api.model
-    def run_action_object_create(self, action, eval_context=None):
+    def _run_action_object_create(self, eval_context=None):
         """Create specified model object with specified values.
 
         If applicable, link active_id.<self.link_field_id> to the new record.
         """
-        res = {}
-        for exp in action.fields_lines:
-            res[exp.col1.name] = exp.eval_value(eval_context=eval_context)[exp.id]
+        vals = self.fields_lines.eval_value(eval_context=eval_context)
+        res = {line.col1.name: vals[line.id] for line in self.fields_lines}
 
-        res = self.env[action.crud_model_id.model].create(res)
+        res = self.env[self.crud_model_id.model].create(res)
 
-        if action.link_field_id:
-            record = self.env[action.model_id.model].browse(self._context.get('active_id'))
-            if action.link_field_id.ttype in ['one2many', 'many2many']:
-                record.write({action.link_field_id.name: [(4, res.id)]})
+        if self.link_field_id:
+            record = self.env[self.model_id.model].browse(self._context.get('active_id'))
+            if self.link_field_id.ttype in ['one2many', 'many2many']:
+                record.write({self.link_field_id.name: [Command.link(res.id)]})
             else:
-                record.write({action.link_field_id.name: res.id})
+                record.write({self.link_field_id.name: res.id})
 
-    @api.model
     def _get_eval_context(self, action=None):
         """ Prepare the context used when evaluating python code, like the
         python formulas or code server actions.
@@ -497,6 +578,7 @@ class IrActionsServer(models.Model):
             'model': model,
             # Exceptions
             'Warning': odoo.exceptions.Warning,
+            'UserError': odoo.exceptions.UserError,
             # record
             'record': record,
             'records': records,
@@ -507,8 +589,12 @@ class IrActionsServer(models.Model):
 
     def run(self):
         """ Runs the server action. For each server action, the
-        run_action_<STATE> method is called. This allows easy overriding
-        of the server actions.
+        :samp:`_run_action_{TYPE}[_multi]` method is called. This allows easy
+        overriding of the server actions.
+
+        The `_multi` suffix means the runner can operate on multiple records,
+        otherwise if there are multiple records the runner will be called once
+        for each
 
         :param dict context: context should contain following keys
 
@@ -524,43 +610,58 @@ class IrActionsServer(models.Model):
                  return action
         """
         res = False
-        for action in self:
+        for action in self.sudo():
             action_groups = action.groups_id
-            if action_groups and not (action_groups & self.env.user.groups_id):
-                raise AccessError(_("You don't have enough access rights to run this action."))
+            if action_groups:
+                if not (action_groups & self.env.user.groups_id):
+                    raise AccessError(_("You don't have enough access rights to run this action."))
+            else:
+                try:
+                    self.env[action.model_name].check_access_rights("write")
+                except AccessError:
+                    _logger.warning("Forbidden server action %r executed while the user %s does not have access to %s.",
+                        action.name, self.env.user.login, action.model_name,
+                    )
+                    raise
 
             eval_context = self._get_eval_context(action)
-            if hasattr(self, 'run_action_%s_multi' % action.state):
-                # call the multi method
-                run_self = self.with_context(eval_context['env'].context)
-                func = getattr(run_self, 'run_action_%s_multi' % action.state)
-                res = func(action, eval_context=eval_context)
+            records = eval_context.get('record') or eval_context['model']
+            records |= eval_context.get('records') or eval_context['model']
+            if records:
+                try:
+                    records.check_access_rule('write')
+                except AccessError:
+                    _logger.warning("Forbidden server action %r executed while the user %s does not have access to %s.",
+                        action.name, self.env.user.login, records,
+                    )
+                    raise
 
-            elif hasattr(self, 'run_action_%s' % action.state):
+            runner, multi = action._get_runner()
+            if runner and multi:
+                # call the multi method
+                run_self = action.with_context(eval_context['env'].context)
+                res = runner(run_self, eval_context=eval_context)
+            elif runner:
                 active_id = self._context.get('active_id')
                 if not active_id and self._context.get('onchange_self'):
                     active_id = self._context['onchange_self']._origin.id
                     if not active_id:  # onchange on new record
-                        func = getattr(self, 'run_action_%s' % action.state)
-                        res = func(action, eval_context=eval_context)
+                        res = runner(action, eval_context=eval_context)
                 active_ids = self._context.get('active_ids', [active_id] if active_id else [])
                 for active_id in active_ids:
                     # run context dedicated to a particular active_id
-                    run_self = self.with_context(active_ids=[active_id], active_id=active_id)
+                    run_self = action.with_context(active_ids=[active_id], active_id=active_id)
                     eval_context["env"].context = run_self._context
-                    # call the single method related to the action: run_action_<STATE>
-                    func = getattr(run_self, 'run_action_%s' % action.state)
-                    res = func(action, eval_context=eval_context)
+                    res = runner(run_self, eval_context=eval_context)
+            else:
+                _logger.warning(
+                    "Found no way to execute server action %r of type %r, ignoring it. "
+                    "Verify that the type is correct or add a method called "
+                    "`_run_action_<type>` or `_run_action_<type>_multi`.",
+                    action.name, action.state
+                )
         return res or False
 
-    @api.model
-    def _run_actions(self, ids):
-        """
-            Run server actions with given ids.
-            Allow crons to run specific server actions
-        """
-        return self.browse(ids).run()
-
 
 class IrServerObjectLines(models.Model):
     _name = 'ir.server.object.lines'
@@ -568,7 +669,7 @@ class IrServerObjectLines(models.Model):
     _sequence = 'ir_actions_id_seq'
 
     server_id = fields.Many2one('ir.actions.server', string='Related Server Action', ondelete='cascade')
-    col1 = fields.Many2one('ir.model.fields', string='Field', required=True)
+    col1 = fields.Many2one('ir.model.fields', string='Field', required=True, ondelete='cascade')
     value = fields.Text(required=True, help="Expression containing a value specification. \n"
                                             "When Formula type is selected, this field may be a Python expression "
                                             " that can use the same values as for the code field on the server action.\n"
@@ -584,8 +685,7 @@ class IrServerObjectLines(models.Model):
 
     @api.model
     def _selection_target_model(self):
-        models = self.env['ir.model'].search([])
-        return [(model.model, model.name) for model in models]
+        return [(model.model, model.name) for model in self.env['ir.model'].sudo().search([])]
 
     @api.depends('col1.relation', 'value', 'evaluation_type')
     def _compute_resource_ref(self):
@@ -595,10 +695,10 @@ class IrServerObjectLines(models.Model):
                 try:
                     value = int(value)
                     if not self.env[line.col1.relation].browse(value).exists():
-                        record = self.env[line.col1.relation]._search([], limit=1)
+                        record = list(self.env[line.col1.relation]._search([], limit=1))
                         value = record[0] if record else 0
                 except ValueError:
-                    record = self.env[line.col1.relation]._search([], limit=1)
+                    record = list(self.env[line.col1.relation]._search([], limit=1))
                     value = record[0] if record else 0
                 line.resource_ref = '%s,%s' % (line.col1.relation, value)
             else:
@@ -616,7 +716,7 @@ class IrServerObjectLines(models.Model):
                 line.value = str(line.resource_ref.id)
 
     def eval_value(self, eval_context=None):
-        result = dict.fromkeys(self.ids, False)
+        result = {}
         for line in self:
             expr = line.value
             if line.evaluation_type == 'equation':
@@ -682,8 +782,7 @@ class IrActionsTodo(models.Model):
     def _name_search(self, name, args=None, operator='ilike', limit=100, name_get_uid=None):
         args = args or []
         if name:
-            action_ids = self._search(expression.AND([[('action_id', operator, name)], args]), limit=limit, access_rights_uid=name_get_uid)
-            return models.lazy_name_get(self.browse(action_ids).with_user(name_get_uid))
+            return self._search(expression.AND([[('action_id', operator, name)], args]), limit=limit, access_rights_uid=name_get_uid)
         return super(IrActionsTodo, self)._name_search(name, args=args, operator=operator, limit=limit, name_get_uid=name_get_uid)
 
     def action_launch(self):
@@ -759,3 +858,9 @@ class IrActionsActClient(models.Model):
         params_store = doc.find(".//field[@name='params_store']")
         params_store.getparent().remove(params_store)
         return doc
+
+
+    def _get_readable_fields(self):
+        return super()._get_readable_fields() | {
+            "context", "params", "res_model", "tag", "target",
+        }
