@@ -1,15 +1,16 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+from markupsafe import Markup
+
 from odoo import api, fields, models, tools, SUPERUSER_ID, _
 from odoo.exceptions import UserError, AccessError
-from odoo.tools.safe_eval import safe_eval
+from odoo.tools.safe_eval import safe_eval, time
 from odoo.tools.misc import find_in_path
-from odoo.tools import config
+from odoo.tools import config, is_html_empty
 from odoo.sql_db import TestCursor
 from odoo.http import request
 from odoo.osv.expression import NEGATIVE_TERM_OPERATORS, FALSE_DOMAIN
 
-import time
 import base64
 import io
 import logging
@@ -103,8 +104,7 @@ class IrActionsReport(models.Model):
         ' rendering method. HTML means the report will be opened directly in your'
         ' browser PDF means the report will be rendered using Wkhtmltopdf and'
         ' downloaded by the user.')
-    report_name = fields.Char(string='Template Name', required=True,
-                              help="For QWeb reports, name of the template used in the rendering. The method 'render_html' of the model 'report.template_name' will be called (if any) to give the html. For RML reports, this is the LocalService name.")
+    report_name = fields.Char(string='Template Name', required=True)
     report_file = fields.Char(string='Report File', required=False, readonly=False, store=True,
                               help="The path to the main report file (depending on Report Type) or empty if the content is in another field")
     groups_id = fields.Many2many('res.groups', 'res_groups_report_rel', 'uid', 'gid', string='Groups')
@@ -114,7 +114,7 @@ class IrActionsReport(models.Model):
     print_report_name = fields.Char('Printed Report Name', translate=True,
                                     help="This is the filename of the report going to download. Keep empty to not change the report filename. You can use a python expression with the 'object' and 'time' variables.")
     attachment_use = fields.Boolean(string='Reload from Attachment',
-                                    help='If you check this, then the second time the user prints with same attachment name, it returns the previous report.')
+                                    help='If enabled, then the second time the user prints with same attachment name, it returns the previous report.')
     attachment = fields.Char(string='Save as Attachment Prefix',
                              help='This is the filename of the attachment used to store the printing result. Keep empty to not save the printed reports. You can use a python expression with the object and time variables.')
 
@@ -144,6 +144,17 @@ class IrActionsReport(models.Model):
         else:
             return FALSE_DOMAIN
 
+    def _get_readable_fields(self):
+        return super()._get_readable_fields() | {
+            "report_name", "report_type", "target",
+            # these two are not real fields of ir.actions.report but are
+            # expected in the route /report/<converter>/<reportname> and must
+            # not be removed by clean_action
+            "context", "data",
+            # and this one is used by the frontend later on.
+            "close_on_report_download",
+        }
+
     def associated_view(self):
         """Used in the ir.actions.report form view in order to search naively after the view(s)
         used in the rendering.
@@ -199,7 +210,7 @@ class IrActionsReport(models.Model):
                 ('res_id', '=', record.id)
         ], limit=1)
 
-    def postprocess_pdf_report(self, record, buffer):
+    def _postprocess_pdf_report(self, record, buffer):
         '''Hook to handle post processing during the pdf report generation.
         The basic behavior consists to create a new attachment containing the pdf
         base64 encoded.
@@ -213,7 +224,7 @@ class IrActionsReport(models.Model):
             return None
         attachment_vals = {
             'name': attachment_name,
-            'datas': base64.encodebytes(buffer.getvalue()),
+            'raw': buffer.getvalue(),
             'res_model': self.model,
             'res_id': record.id,
             'type': 'binary',
@@ -239,7 +250,6 @@ class IrActionsReport(models.Model):
         '''
         return wkhtmltopdf_state
 
-    @api.model
     def get_paperformat(self):
         return self.paperformat_id or self.env.company.paperformat_id
 
@@ -315,6 +325,8 @@ class IrActionsReport(models.Model):
                 command_args.extend(['--orientation', str(paperformat_id.orientation)])
             if paperformat_id.header_line:
                 command_args.extend(['--header-line'])
+            if paperformat_id.disable_shrinking:
+                command_args.extend(['--disable-smart-shrinking'])
 
         if landscape:
             command_args.extend(['--orientation', 'landscape'])
@@ -336,13 +348,13 @@ class IrActionsReport(models.Model):
         :return: bodies, header, footer, specific_paperformat_args
         '''
         IrConfig = self.env['ir.config_parameter'].sudo()
-        base_url = IrConfig.get_param('report.url') or IrConfig.get_param('web.base.url')
 
         # Return empty dictionary if 'web.minimal_layout' not found.
         layout = self.env.ref('web.minimal_layout', False)
         if not layout:
             return {}
         layout = self.env['ir.ui.view'].browse(self.env['ir.ui.view'].get_view_id('web.minimal_layout'))
+        base_url = IrConfig.get_param('report.url') or layout.get_base_url()
 
         root = lxml.html.fromstring(html)
         match_klass = "//div[contains(concat(' ', normalize-space(@class), ' '), ' {} ')]"
@@ -371,7 +383,12 @@ class IrActionsReport(models.Model):
             # set context language to body language
             if node.get('data-oe-lang'):
                 layout_with_lang = layout_with_lang.with_context(lang=node.get('data-oe-lang'))
-            body = layout_with_lang.render(dict(subst=False, body=lxml.html.tostring(node), base_url=base_url))
+            body = layout_with_lang._render({
+                'subst': False,
+                'body': Markup(lxml.html.tostring(node, encoding='unicode')),
+                'base_url': base_url,
+                'report_xml_id' : self.xml_id
+            })
             bodies.append(body)
             if node.get('data-oe-model') == self.model:
                 res_ids.append(int(node.get('data-oe-id', 0)))
@@ -379,7 +396,7 @@ class IrActionsReport(models.Model):
                 res_ids.append(None)
 
         if not bodies:
-            body = bytearray().join([lxml.html.tostring(c) for c in body_parent.getchildren()])
+            body = ''.join(lxml.html.tostring(c, encoding='unicode') for c in body_parent.getchildren())
             bodies.append(body)
 
         # Get paperformat arguments set in the root html tag. They are prioritized over
@@ -389,8 +406,16 @@ class IrActionsReport(models.Model):
             if attribute[0].startswith('data-report-'):
                 specific_paperformat_args[attribute[0]] = attribute[1]
 
-        header = layout.render(dict(subst=True, body=lxml.html.tostring(header_node), base_url=base_url))
-        footer = layout.render(dict(subst=True, body=lxml.html.tostring(footer_node), base_url=base_url))
+        header = layout._render({
+            'subst': True,
+            'body': Markup(lxml.html.tostring(header_node, encoding='unicode')),
+            'base_url': base_url
+        })
+        footer = layout._render({
+            'subst': True,
+            'body': Markup(lxml.html.tostring(footer_node, encoding='unicode')),
+            'base_url': base_url
+        })
 
         return bodies, res_ids, header, footer, specific_paperformat_args
 
@@ -406,13 +431,14 @@ class IrActionsReport(models.Model):
         '''Execute wkhtmltopdf as a subprocess in order to convert html given in input into a pdf
         document.
 
-        :param bodies: The html bodies of the report, one per page.
-        :param header: The html header of the report containing all headers.
-        :param footer: The html footer of the report containing all footers.
+        :param list[str] bodies: The html bodies of the report, one per page.
+        :param str header: The html header of the report containing all headers.
+        :param str footer: The html footer of the report containing all footers.
         :param landscape: Force the pdf to be rendered under a landscape format.
         :param specific_paperformat_args: dict of prioritized paperformat arguments.
         :param set_viewport_size: Enable a viewport sized '1024x1280' or '1280x1024' depending of landscape arg.
-        :return: Content of the pdf as a string
+        :return: Content of the pdf as bytes
+        :rtype: bytes
         '''
         paperformat_id = self.get_paperformat()
 
@@ -428,13 +454,13 @@ class IrActionsReport(models.Model):
         if header:
             head_file_fd, head_file_path = tempfile.mkstemp(suffix='.html', prefix='report.header.tmp.')
             with closing(os.fdopen(head_file_fd, 'wb')) as head_file:
-                head_file.write(header)
+                head_file.write(header.encode())
             temporary_files.append(head_file_path)
             files_command_args.extend(['--header-html', head_file_path])
         if footer:
             foot_file_fd, foot_file_path = tempfile.mkstemp(suffix='.html', prefix='report.footer.tmp.')
             with closing(os.fdopen(foot_file_fd, 'wb')) as foot_file:
-                foot_file.write(footer)
+                foot_file.write(footer.encode())
             temporary_files.append(foot_file_path)
             files_command_args.extend(['--footer-html', foot_file_path])
 
@@ -443,7 +469,7 @@ class IrActionsReport(models.Model):
             prefix = '%s%d.' % ('report.body.tmp.', i)
             body_file_fd, body_file_path = tempfile.mkstemp(suffix='.html', prefix=prefix)
             with closing(os.fdopen(body_file_fd, 'wb')) as body_file:
-                body_file.write(body)
+                body_file.write(body.encode())
             paths.append(body_file_path)
             temporary_files.append(body_file_path)
 
@@ -462,6 +488,7 @@ class IrActionsReport(models.Model):
                         'Wkhtmltopdf failed (error code: %s). Memory limit too low or maximum file number of subprocess reached. Message : %s')
                 else:
                     message = _('Wkhtmltopdf failed (error code: %s). Message: %s')
+                _logger.warning(message, process.returncode, err[-1000:])
                 raise UserError(message % (str(process.returncode), err[-1000:]))
             else:
                 if err:
@@ -489,10 +516,28 @@ class IrActionsReport(models.Model):
         report_obj = self.env['ir.actions.report']
         conditions = [('report_name', '=', report_name)]
         context = self.env['res.users'].context_get()
-        return report_obj.with_context(context).search(conditions, limit=1)
+        return report_obj.with_context(context).sudo().search(conditions, limit=1)
 
     @api.model
-    def barcode(self, barcode_type, value, width=600, height=100, humanreadable=0, quiet=1):
+    def barcode(self, barcode_type, value, **kwargs):
+        defaults = {
+            'width': (600, int),
+            'height': (100, int),
+            'humanreadable': (False, lambda x: bool(int(x))),
+            'quiet': (True, lambda x: bool(int(x))),
+            'mask': (None, lambda x: x),
+            'barBorder': (4, int),
+            # The QR code can have different layouts depending on the Error Correction Level
+            # See: https://en.wikipedia.org/wiki/QR_code#Error_correction
+            # Level 'L' – up to 7% damage   (default)
+            # Level 'M' – up to 15% damage  (i.e. required by l10n_ch QR bill)
+            # Level 'Q' – up to 25% damage
+            # Level 'H' – up to 30% damage
+            'barLevel': ('L', lambda x: x in ('L', 'M', 'Q', 'H') and x or 'L'),
+        }
+        kwargs = {k: validator(kwargs.get(k, v)) for k, (v, validator) in defaults.items()}
+        kwargs['humanReadable'] = kwargs.pop('humanreadable')
+
         if barcode_type == 'UPCA' and len(value) in (11, 12, 13):
             barcode_type = 'EAN13'
             if len(value) in (11, 12):
@@ -500,31 +545,51 @@ class IrActionsReport(models.Model):
         elif barcode_type == 'auto':
             symbology_guess = {8: 'EAN8', 13: 'EAN13'}
             barcode_type = symbology_guess.get(len(value), 'Code128')
-        try:
-            width, height, humanreadable, quiet = int(width), int(height), bool(int(humanreadable)), bool(int(quiet))
+        elif barcode_type == 'QR':
             # for `QR` type, `quiet` is not supported. And is simply ignored.
             # But we can use `barBorder` to get a similar behaviour.
-            bar_border = 4
-            if barcode_type == 'QR' and quiet:
-                bar_border = 0
+            if kwargs['quiet']:
+                kwargs['barBorder'] = 0
+
+        try:
+            barcode = createBarcodeDrawing(barcode_type, value=value, format='png', **kwargs)
+
+            # If a mask is asked and it is available, call its function to
+            # post-process the generated QR-code image
+            if kwargs['mask']:
+                available_masks = self.get_available_barcode_masks()
+                mask_to_apply = available_masks.get(kwargs['mask'])
+                if mask_to_apply:
+                    mask_to_apply(kwargs['width'], kwargs['height'], barcode)
 
-            barcode = createBarcodeDrawing(
-                barcode_type, value=value, format='png', width=width, height=height,
-                humanReadable=humanreadable, quiet=quiet, barBorder=bar_border
-            )
             return barcode.asString('png')
         except (ValueError, AttributeError):
             if barcode_type == 'Code128':
                 raise ValueError("Cannot convert into barcode.")
+            elif barcode_type == 'QR':
+                raise ValueError("Cannot convert into QR code.")
             else:
-                return self.barcode('Code128', value, width=width, height=height,
-                    humanreadable=humanreadable, quiet=quiet)
+                return self.barcode('Code128', value, **kwargs)
 
-    def render_template(self, template, values=None):
+    @api.model
+    def get_available_barcode_masks(self):
+        """ Hook for extension.
+        This function returns the available QR-code masks, in the form of a
+        list of (code, mask_function) elements, where code is a string identifying
+        the mask uniquely, and mask_function is a function returning a reportlab
+        Drawing object with the result of the mask, and taking as parameters:
+            - width of the QR-code, in pixels
+            - height of the QR-code, in pixels
+            - reportlab Drawing object containing the barcode to apply the mask on
+        """
+        return {}
+
+    def _render_template(self, template, values=None):
         """Allow to render a QWeb template python-side. This function returns the 'ir.ui.view'
         render but embellish it with some variables/methods used in reports.
-        :param values: additionnal methods/variables used in the rendering
+        :param values: additional methods/variables used in the rendering
         :returns: html representation of the template
+        :rtype: bytes
         """
         if values is None:
             values = {}
@@ -539,16 +604,16 @@ class IrActionsReport(models.Model):
                 website = request.website
                 context = dict(context, translatable=context.get('lang') != request.env['ir.http']._get_default_lang().code)
 
-        view_obj = self.env['ir.ui.view'].with_context(context)
+        view_obj = self.env['ir.ui.view'].sudo().with_context(context)
         values.update(
             time=time,
             context_timestamp=lambda t: fields.Datetime.context_timestamp(self.with_context(tz=user.tz), t),
             user=user,
-            res_company=user.company_id,
+            res_company=self.env.company,
             website=website,
             web_base_url=self.env['ir.config_parameter'].sudo().get_param('web.base.url', default=''),
         )
-        return view_obj.render_template(template, values)
+        return view_obj._render_template(template, values).encode()
 
     def _post_pdf(self, save_in_attachment, pdf_content=None, res_ids=None):
         '''Merge the existing attachments by adding one by one the content of the attachments
@@ -576,7 +641,7 @@ class IrActionsReport(models.Model):
 
         # Create a list of streams representing all sub-reports part of the final result
         # in order to append the existing attachments and the potentially modified sub-reports
-        # by the postprocess_pdf_report calls.
+        # by the _postprocess_pdf_report calls.
         streams = []
 
         # In wkhtmltopdf has been called, we need to split the pdf in order to call the postprocess method.
@@ -592,7 +657,7 @@ class IrActionsReport(models.Model):
                 if len(res_ids) == 1:
                     # Only one record, so postprocess directly and append the whole pdf.
                     if res_ids[0] in record_map and not res_ids[0] in save_in_attachment:
-                        new_stream = self.postprocess_pdf_report(record_map[res_ids[0]], pdf_content_stream)
+                        new_stream = self._postprocess_pdf_report(record_map[res_ids[0]], pdf_content_stream)
                         # If the buffer has been modified, mark the old buffer to be closed as well.
                         if new_stream and new_stream != pdf_content_stream:
                             close_streams([pdf_content_stream])
@@ -627,7 +692,7 @@ class IrActionsReport(models.Model):
                             stream = io.BytesIO()
                             attachment_writer.write(stream)
                             if res_ids[i] and res_ids[i] not in save_in_attachment:
-                                new_stream = self.postprocess_pdf_report(record_map[res_ids[i]], stream)
+                                new_stream = self._postprocess_pdf_report(record_map[res_ids[i]], stream)
                                 # If the buffer has been modified, mark the old buffer to be closed as well.
                                 if new_stream and new_stream != stream:
                                     close_streams([stream])
@@ -652,7 +717,7 @@ class IrActionsReport(models.Model):
             try:
                 result = self._merge_pdfs(streams)
             except utils.PdfReadError:
-                raise UserError(_("One of the documents, you try to merge is encrypted"))
+                raise UserError(_("One of the documents you are trying to merge is encrypted"))
 
         # We have to close the streams after PdfFileWriter's call to write()
         close_streams(streams)
@@ -694,15 +759,21 @@ class IrActionsReport(models.Model):
         writer.write(result_stream)
         return result_stream.getvalue()
 
-    def render_qweb_pdf(self, res_ids=None, data=None):
+    def _render_qweb_pdf(self, res_ids=None, data=None):
+        """
+        :rtype: bytes
+        """
         if not data:
             data = {}
         data.setdefault('report_type', 'pdf')
 
+        # access the report details with sudo() but evaluation context as sudo(False)
+        self_sudo = self.sudo()
+
         # In case of test environment without enough workers to perform calls to wkhtmltopdf,
         # fallback to render_html.
         if (tools.config['test_enable'] or tools.config['test_file']) and not self.env.context.get('force_report_rendering'):
-            return self.render_qweb_html(res_ids, data=data)
+            return self_sudo._render_qweb_html(res_ids, data=data)
 
         # As the assets are generated during the same transaction as the rendering of the
         # templates calling them, there is a scenario where the assets are unreachable: when
@@ -719,35 +790,28 @@ class IrActionsReport(models.Model):
         # Disable the debug mode in the PDF rendering in order to not split the assets bundle
         # into separated files to load. This is done because of an issue in wkhtmltopdf
         # failing to load the CSS/Javascript resources in time.
-        # Without this, the header/footer of the reports randomly disapear
+        # Without this, the header/footer of the reports randomly disappear
         # because the resources files are not loaded in time.
         # https://github.com/wkhtmltopdf/wkhtmltopdf/issues/2083
         context['debug'] = False
 
-        # The test cursor prevents the use of another environnment while the current
-        # transaction is not finished, leading to a deadlock when the report requests
-        # an asset bundle during the execution of test scenarios. In this case, return
-        # the html version.
-        if isinstance(self.env.cr, TestCursor):
-            return self.with_context(context).render_qweb_html(res_ids, data=data)[0]
-
         save_in_attachment = OrderedDict()
         # Maps the streams in `save_in_attachment` back to the records they came from
         stream_record = dict()
         if res_ids:
             # Dispatch the records by ones having an attachment and ones requesting a call to
             # wkhtmltopdf.
-            Model = self.env[self.model]
+            Model = self.env[self_sudo.model]
             record_ids = Model.browse(res_ids)
             wk_record_ids = Model
-            if self.attachment:
+            if self_sudo.attachment:
                 for record_id in record_ids:
-                    attachment = self.retrieve_attachment(record_id)
+                    attachment = self_sudo.retrieve_attachment(record_id)
                     if attachment:
-                        stream = self._retrieve_stream_from_attachment(attachment)
+                        stream = self_sudo._retrieve_stream_from_attachment(attachment)
                         save_in_attachment[record_id.id] = stream
                         stream_record[stream] = record_id
-                    if not self.attachment_use or not attachment:
+                    if not self_sudo.attachment_use or not attachment:
                         wk_record_ids += record_id
             else:
                 wk_record_ids = record_ids
@@ -759,7 +823,7 @@ class IrActionsReport(models.Model):
         if save_in_attachment and not res_ids:
             _logger.info('The PDF report has been generated from attachments.')
             self._raise_on_unreadable_pdfs(save_in_attachment.values(), stream_record)
-            return self._post_pdf(save_in_attachment), 'pdf'
+            return self_sudo._post_pdf(save_in_attachment), 'pdf'
 
         if self.get_wkhtmltopdf_state() == 'install':
             # wkhtmltopdf is not installed
@@ -768,14 +832,11 @@ class IrActionsReport(models.Model):
             # bypassed
             raise UserError(_("Unable to find Wkhtmltopdf on this system. The PDF can not be created."))
 
-        html = self.with_context(context).render_qweb_html(res_ids, data=data)[0]
+        html = self_sudo.with_context(context)._render_qweb_html(res_ids, data=data)[0]
 
-        # Ensure the current document is utf-8 encoded.
-        html = html.decode('utf-8')
+        bodies, html_ids, header, footer, specific_paperformat_args = self_sudo.with_context(context)._prepare_html(html)
 
-        bodies, html_ids, header, footer, specific_paperformat_args = self.with_context(context)._prepare_html(html)
-
-        if self.attachment and set(res_ids) != set(html_ids):
+        if self_sudo.attachment and set(res_ids) != set(html_ids):
             raise UserError(_("The report's template '%s' is wrong, please contact your administrator. \n\n"
                 "Can not separate file to save as attachment because the report's template does not contains the attributes 'data-oe-model' and 'data-oe-id' on the div with 'article' classname.") %  self.name)
 
@@ -788,35 +849,39 @@ class IrActionsReport(models.Model):
             set_viewport_size=context.get('set_viewport_size'),
         )
         if res_ids:
-            _logger.info('The PDF report has been generated for model: %s, records %s.' % (self.model, str(res_ids)))
             self._raise_on_unreadable_pdfs(save_in_attachment.values(), stream_record)
-            return self._post_pdf(save_in_attachment, pdf_content=pdf_content, res_ids=html_ids), 'pdf'
+            _logger.info('The PDF report has been generated for model: %s, records %s.' % (self_sudo.model, str(res_ids)))
+            return self_sudo._post_pdf(save_in_attachment, pdf_content=pdf_content, res_ids=html_ids), 'pdf'
         return pdf_content, 'pdf'
 
     @api.model
-    def render_qweb_text(self, docids, data=None):
+    def _render_qweb_text(self, docids, data=None):
+        """
+        :rtype: bytes
+        """
         if not data:
             data = {}
         data.setdefault('report_type', 'text')
+        data.setdefault('__keep_empty_lines', True)
         data = self._get_rendering_context(docids, data)
-        return self.render_template(self.report_name, data), 'text'
+        return self._render_template(self.report_name, data), 'text'
 
     @api.model
-    def render_qweb_html(self, docids, data=None):
+    def _render_qweb_html(self, docids, data=None):
         """This method generates and returns html version of a report.
+
+        :rtype: bytes
         """
         if not data:
             data = {}
         data.setdefault('report_type', 'html')
         data = self._get_rendering_context(docids, data)
-        return self.render_template(self.report_name, data), 'html'
+        return self._render_template(self.report_name, data), 'html'
 
-    @api.model
     def _get_rendering_context_model(self):
         report_model_name = 'report.%s' % self.report_name
         return self.env.get(report_model_name)
 
-    @api.model
     def _get_rendering_context(self, docids, data):
         # If the report is using a custom model to render its html, we must use it.
         # Otherwise, fallback on the generic html rendering.
@@ -825,19 +890,23 @@ class IrActionsReport(models.Model):
         data = data and dict(data) or {}
 
         if report_model is not None:
+            # _render_ may be executed in sudo but evaluation context as real user
+            report_model = report_model.sudo(False)
             data.update(report_model._get_report_values(docids, data=data))
         else:
-            docs = self.env[self.model].browse(docids)
+            # _render_ may be executed in sudo but evaluation context as real user
+            docs = self.env[self.model].sudo(False).browse(docids)
             data.update({
                 'doc_ids': docids,
                 'doc_model': self.model,
                 'docs': docs,
             })
+        data['is_html_empty'] = is_html_empty
         return data
 
-    def render(self, res_ids, data=None):
+    def _render(self, res_ids, data=None):
         report_type = self.report_type.lower().replace('-', '_')
-        render_func = getattr(self, 'render_' + report_type, None)
+        render_func = getattr(self, '_render_' + report_type, None)
         if not render_func:
             return None
         return render_func(res_ids, data=data)
@@ -845,8 +914,10 @@ class IrActionsReport(models.Model):
     def report_action(self, docids, data=None, config=True):
         """Return an action of type ir.actions.report.
 
-        :param docids: id/ids/browserecord of the records to print (if not used, pass an empty list)
-        :param report_name: Name of the template to generate an action for
+        :param docids: id/ids/browse record of the records to print (if not used, pass an empty list)
+        :param data:
+        :param bool config:
+        :rtype: bytes
         """
         context = self.env.context
         if docids:
@@ -870,7 +941,7 @@ class IrActionsReport(models.Model):
 
         discard_logo_check = self.env.context.get('discard_logo_check')
         if self.env.is_admin() and not self.env.company.external_report_layout_id and config and not discard_logo_check:
-            action = self.env.ref('base.action_base_document_layout_configurator').read()[0]
+            action = self.env["ir.actions.actions"]._for_xml_id("web.action_base_document_layout_configurator")
             ctx = action.get('context')
             py_ctx = json.loads(ctx) if ctx else {}
             report_action['close_on_report_download'] = True
