@@ -2,18 +2,22 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import base64
 import hashlib
+import io
 import itertools
 import logging
 import mimetypes
 import os
 import re
-from collections import defaultdict
 import uuid
 
+from collections import defaultdict
+from PIL import Image
+
 from odoo import api, fields, models, tools, _
-from odoo.exceptions import AccessError, ValidationError, MissingError
-from odoo.tools import config, human_size, ustr, html_escape
+from odoo.exceptions import AccessError, ValidationError, MissingError, UserError
+from odoo.tools import config, human_size, ustr, html_escape, ImageProcess, str2bool
 from odoo.tools.mimetypes import guess_mimetype
+from odoo.osv import expression
 
 _logger = logging.getLogger(__name__)
 
@@ -52,21 +56,34 @@ class IrAttachment(models.Model):
     def _filestore(self):
         return config.filestore(self._cr.dbname)
 
+    @api.model
+    def _get_storage_domain(self):
+        # domain to retrieve the attachments to migrate
+        return {
+            'db': [('store_fname', '!=', False)],
+            'file': [('db_datas', '!=', False)],
+        }[self._storage()]
+
     @api.model
     def force_storage(self):
         """Force all attachments to be stored in the currently configured storage"""
         if not self.env.is_admin():
             raise AccessError(_('Only administrators can execute this action.'))
 
-        # domain to retrieve the attachments to migrate
-        domain = {
-            'db': [('store_fname', '!=', False)],
-            'file': [('db_datas', '!=', False)],
-        }[self._storage()]
+        # Migrate only binary attachments and bypass the res_field automatic
+        # filter added in _search override
+        self.search(expression.AND([
+            self._get_storage_domain(),
+            ['&', ('type', '=', 'binary'), '|', ('res_field', '=', False), ('res_field', '!=', False)]
+        ]))._migrate()
 
-        for attach in self.search(domain):
-            attach.write({'datas': attach.datas})
-        return True
+    def _migrate(self):
+        record_count = len(self)
+        storage = self._storage().upper()
+        for index, attach in enumerate(self):
+            _logger.debug("Migrate attachment %s/%s to %s", index + 1, record_count, storage)
+            # pass mimetype, to avoid recomputation
+            attach.write({'raw': attach.raw, 'mimetype': attach.mimetype})
 
     @api.model
     def _full_path(self, path):
@@ -90,25 +107,23 @@ class IrAttachment(models.Model):
         dirname = os.path.dirname(full_path)
         if not os.path.isdir(dirname):
             os.makedirs(dirname)
+        # prevent sha-1 collision
+        if os.path.isfile(full_path) and not self._same_content(bin_data, full_path):
+            raise UserError("The attachment is colliding with an existing file.")
         return fname, full_path
 
     @api.model
-    def _file_read(self, fname, bin_size=False):
+    def _file_read(self, fname):
         full_path = self._full_path(fname)
-        r = ''
         try:
-            if bin_size:
-                r = human_size(os.path.getsize(full_path))
-            else:
-                with open(full_path,'rb') as fd:
-                    r = base64.b64encode(fd.read())
+            with open(full_path, 'rb') as f:
+                return f.read()
         except (IOError, OSError):
             _logger.info("_read_file reading %s", full_path, exc_info=True)
-        return r
+        return b''
 
     @api.model
-    def _file_write(self, value, checksum):
-        bin_value = base64.b64decode(value)
+    def _file_write(self, bin_value, checksum):
         fname, full_path = self._get_path(bin_value, checksum)
         if not os.path.exists(full_path):
             try:
@@ -127,6 +142,7 @@ class IrAttachment(models.Model):
 
     def _mark_for_gc(self, fname):
         """ Add ``fname`` in a checklist for the filestore garbage collection. """
+        fname = re.sub('[.]', '', fname).strip('/\\')
         # we use a spooldir: add an empty file in the subdirectory 'checklist'
         full_path = os.path.join(self._full_path('checklist'), fname)
         if not os.path.exists(full_path):
@@ -136,8 +152,8 @@ class IrAttachment(models.Model):
                     os.makedirs(dirname)
             open(full_path, 'ab').close()
 
-    @api.model
-    def _file_gc(self):
+    @api.autovacuum
+    def _gc_file_store(self):
         """ Perform the garbage collection of the filestore. """
         if self._storage() != 'file':
             return
@@ -191,18 +207,37 @@ class IrAttachment(models.Model):
         cr.commit()
         _logger.info("filestore gc %d checked, %d removed", len(checklist), removed)
 
-    @api.depends('store_fname', 'db_datas')
+    @api.depends('store_fname', 'db_datas', 'file_size')
+    @api.depends_context('bin_size')
     def _compute_datas(self):
-        bin_size = self._context.get('bin_size')
+        if self._context.get('bin_size'):
+            for attach in self:
+                attach.datas = human_size(attach.file_size)
+            return
+
+        for attach in self:
+            attach.datas = base64.b64encode(attach.raw or b'')
+
+    @api.depends('store_fname', 'db_datas')
+    def _compute_raw(self):
         for attach in self:
             if attach.store_fname:
-                attach.datas = self._file_read(attach.store_fname, bin_size)
+                attach.raw = attach._file_read(attach.store_fname)
             else:
-                attach.datas = attach.db_datas
+                attach.raw = attach.db_datas
+
+    def _inverse_raw(self):
+        self._set_attachment_data(lambda a: a.raw or b'')
 
     def _inverse_datas(self):
+        self._set_attachment_data(lambda attach: base64.b64decode(attach.datas or b''))
+
+    def _set_attachment_data(self, asbytes):
         for attach in self:
-            vals = self._get_datas_related_values(attach.datas, attach.mimetype)
+            # compute the fields that depend on datas
+            bin_data = asbytes(attach)
+            vals = self._get_datas_related_values(bin_data, attach.mimetype)
+
             # take current location in filestore to possibly garbage-collect it
             fname = attach.store_fname
             # write as superuser, as user probably does not have write access
@@ -211,12 +246,10 @@ class IrAttachment(models.Model):
                 self._file_delete(fname)
 
     def _get_datas_related_values(self, data, mimetype):
-        # compute the fields that depend on datas
-        bin_data = base64.b64decode(data) if data else b''
         values = {
-            'file_size': len(bin_data),
-            'checksum': self._compute_checksum(bin_data),
-            'index_content': self._index(bin_data, mimetype),
+            'file_size': len(data),
+            'checksum': self._compute_checksum(data),
+            'index_content': self._index(data, mimetype),
             'store_fname': False,
             'db_datas': data,
         }
@@ -232,6 +265,20 @@ class IrAttachment(models.Model):
         # an empty file has a checksum too (for caching)
         return hashlib.sha1(bin_data or b'').hexdigest()
 
+    @api.model
+    def _same_content(self, bin_data, filepath):
+        BLOCK_SIZE = 1024
+        with open(filepath, 'rb') as fd:
+            i = 0
+            while True:
+                data = fd.read(BLOCK_SIZE)
+                if data != bin_data[i * BLOCK_SIZE:(i + 1) * BLOCK_SIZE]:
+                    return False
+                if not data:
+                    break
+                i += 1
+        return True
+
     def _compute_mimetype(self, values):
         """ compute the mimetype of the given values
             :param values : dict of values to create or write an ir_attachment
@@ -243,21 +290,67 @@ class IrAttachment(models.Model):
         if not mimetype and values.get('name'):
             mimetype = mimetypes.guess_type(values['name'])[0]
         if not mimetype and values.get('url'):
-            mimetype = mimetypes.guess_type(values['url'])[0]
-        if values.get('datas') and (not mimetype or mimetype == 'application/octet-stream'):
-            mimetype = guess_mimetype(base64.b64decode(values['datas']))
+            mimetype = mimetypes.guess_type(values['url'].split('?')[0])[0]
+        if not mimetype or mimetype == 'application/octet-stream':
+            raw = None
+            if values.get('raw'):
+                raw = values['raw']
+            elif values.get('datas'):
+                raw = base64.b64decode(values['datas'])
+            if raw:
+                mimetype = guess_mimetype(raw)
         return mimetype or 'application/octet-stream'
 
+    def _postprocess_contents(self, values):
+        ICP = self.env['ir.config_parameter'].sudo().get_param
+        supported_subtype = ICP('base.image_autoresize_extensions', 'png,jpeg,gif,bmp,tif').split(',')
+
+        mimetype = values['mimetype'] = self._compute_mimetype(values)
+        _type, _subtype = mimetype.split('/')
+        is_image_resizable = _type == 'image' and _subtype in supported_subtype
+        if is_image_resizable and (values.get('datas') or values.get('raw')):
+            is_raw = values.get('raw')
+
+            # Can be set to 0 to skip the resize
+            max_resolution = ICP('base.image_autoresize_max_px', '1920x1920')
+            if str2bool(max_resolution, True):
+                try:
+                    img = fn_quality = False
+                    if is_raw:
+                        img = ImageProcess(False, verify_resolution=False)
+                        img.image = Image.open(io.BytesIO(values['raw']))
+                        img.original_format = (img.image.format or '').upper()
+                        fn_quality = img.image_quality
+                    else:  # datas
+                        img = ImageProcess(values['datas'], verify_resolution=False)
+                        fn_quality = img.image_base64
+
+                    w, h = img.image.size
+                    nw, nh = map(int, max_resolution.split('x'))
+                    if w > nw or h > nh:
+                        img.resize(nw, nh)
+                        quality = int(ICP('base.image_autoresize_quality', 80))
+                        values[is_raw and 'raw' or 'datas'] = fn_quality(quality=quality)
+                except UserError as e:
+                    # Catch error during test where we provide fake image
+                    # raise UserError(_("This file could not be decoded as an image file. Please try with a different file."))
+                    _logger.info('Post processing ignored : %s', e)
+                    pass
+        return values
+
     def _check_contents(self, values):
         mimetype = values['mimetype'] = self._compute_mimetype(values)
         xml_like = 'ht' in mimetype or ( # hta, html, xhtml, etc.
                 'xml' in mimetype and    # other xml (svg, text/xml, etc)
                 not 'openxmlformats' in mimetype)  # exception for Office formats
         user = self.env.context.get('binary_field_real_user', self.env.user)
-        force_text = (xml_like and (not user._is_system() or
-            self.env.context.get('attachments_mime_plainxml')))
+        force_text = xml_like and (
+            self.env.context.get('attachments_mime_plainxml') or
+            not self.env['ir.ui.view'].with_user(user).check_access_rights('write', False))
         if force_text:
             values['mimetype'] = 'text/plain'
+        if not self.env.context.get('image_no_postprocess'):
+            values = self._postprocess_contents(values)
         return values
 
     @api.model
@@ -303,7 +396,8 @@ class IrAttachment(models.Model):
     access_token = fields.Char('Access Token', groups="base.group_user")
 
     # the field 'datas' is computed and may use the other fields below
-    datas = fields.Binary(string='File Content', compute='_compute_datas', inverse='_inverse_datas')
+    raw = fields.Binary(string="File Content (raw)", compute='_compute_raw', inverse='_inverse_raw')
+    datas = fields.Binary(string='File Content (base64)', compute='_compute_datas', inverse='_inverse_datas')
     db_datas = fields.Binary('Database Data', attachment=False)
     store_fname = fields.Char('Stored Filename')
     file_size = fields.Integer('File Size', readonly=True)
@@ -328,20 +422,19 @@ class IrAttachment(models.Model):
             # XDO note: if read on sudo, read twice, one for constraints, one for _inverse_datas as user
             if attachment.type == 'binary' and attachment.url:
                 has_group = self.env.user.has_group
-                if not any([has_group(g) for g in attachment.get_serving_groups()]):
+                if not any(has_group(g) for g in attachment.get_serving_groups()):
                     raise ValidationError("Sorry, you are not allowed to write on this document")
 
     @api.model
     def check(self, mode, values=None):
-        """Restricts the access to an ir.attachment, according to referred model
-        In the 'document' module, it is overridden to relax this hard rule, since
-        more complex ones apply there.
-        """
+        """ Restricts the access to an ir.attachment, according to referred mode """
         if self.env.is_superuser():
             return True
+        # Always require an internal user (aka, employee) to access to a attachment
+        if not (self.env.is_admin() or self.env.user.has_group('base.group_user')):
+            raise AccessError(_("Sorry, you are not allowed to access this document."))
         # collect the records to check (by model)
         model_ids = defaultdict(set)            # {model_name: set(ids)}
-        require_employee = False
         if self:
             # DLE P173: `test_01_portal_attachment`
             self.env['ir.attachment'].flush(['res_model', 'res_id', 'create_uid', 'public', 'res_field'])
@@ -352,8 +445,6 @@ class IrAttachment(models.Model):
                 if not self.env.is_system() and (res_field or (not res_id and create_uid != self.env.uid)):
                     raise AccessError(_("Sorry, you are not allowed to access this document."))
                 if not (res_model and res_id):
-                    if create_uid != self._uid:
-                        require_employee = True
                     continue
                 model_ids[res_model].add(res_id)
         if values and values.get('res_model') and values.get('res_id'):
@@ -365,25 +456,19 @@ class IrAttachment(models.Model):
             # when checking access rights (resource was deleted but attachment
             # was not)
             if res_model not in self.env:
-                require_employee = True
                 continue
-            elif res_model == 'res.users' and len(res_ids) == 1 and self._uid == list(res_ids)[0]:
+            if res_model == 'res.users' and len(res_ids) == 1 and self.env.uid == list(res_ids)[0]:
                 # by default a user cannot write on itself, despite the list of writeable fields
                 # e.g. in the case of a user inserting an image into his image signature
                 # we need to bypass this check which would needlessly throw us away
                 continue
             records = self.env[res_model].browse(res_ids).exists()
-            if len(records) < len(res_ids):
-                require_employee = True
             # For related models, check if we can write to the model, as unlinking
             # and creating attachments can be seen as an update to the model
             access_mode = 'write' if mode in ('create', 'unlink') else mode
             records.check_access_rights(access_mode)
             records.check_access_rule(access_mode)
 
-        if require_employee:
-            if not (self.env.is_admin() or self.env.user.has_group('base.group_user')):
-                raise AccessError(_("Sorry, you are not allowed to access this document."))
 
     def _read_group_allowed_fields(self):
         return ['type', 'company_id', 'res_id', 'create_date', 'create_uid', 'name', 'mimetype', 'id', 'url', 'res_field', 'res_model']
@@ -487,14 +572,16 @@ class IrAttachment(models.Model):
     def write(self, vals):
         self.check('write', values=vals)
         # remove computed field depending of datas
-        for field in ('file_size', 'checksum'):
+        for field in ('file_size', 'checksum', 'store_fname'):
             vals.pop(field, False)
-        if 'mimetype' in vals or 'datas' in vals:
+        if 'mimetype' in vals or 'datas' in vals or 'raw' in vals:
             vals = self._check_contents(vals)
         return super(IrAttachment, self).write(vals)
 
     def copy(self, default=None):
-        self.check('write')
+        if not (default or {}).keys() & {'datas', 'db_datas', 'raw'}:
+            # ensure the content is kept and recomputes checksum/store_fname
+            default = dict(default or {}, raw=self.raw)
         return super(IrAttachment, self).copy(default)
 
     def unlink(self):
@@ -516,21 +603,37 @@ class IrAttachment(models.Model):
     @api.model_create_multi
     def create(self, vals_list):
         record_tuple_set = set()
+
+        # remove computed field depending of datas
+        vals_list = [{
+            key: value
+            for key, value
+            in vals.items()
+            if key not in ('file_size', 'checksum', 'store_fname')
+        } for vals in vals_list]
+
         for values in vals_list:
-            # remove computed field depending of datas
-            for field in ('file_size', 'checksum'):
-                values.pop(field, False)
             values = self._check_contents(values)
-            if 'datas' in values:
-                values.update(self._get_datas_related_values(values.pop('datas'), values['mimetype']))
+            raw, datas = values.pop('raw', None), values.pop('datas', None)
+            if raw or datas:
+                if isinstance(raw, str):
+                    # b64decode handles str input but raw needs explicit encoding
+                    raw = raw.encode()
+                values.update(self._get_datas_related_values(
+                    raw or base64.b64decode(datas or b''),
+                    values['mimetype']
+                ))
+
             # 'check()' only uses res_model and res_id from values, and make an exists.
             # We can group the values by model, res_id to make only one query when 
             # creating multiple attachments on a single record.
             record_tuple = (values.get('res_model'), values.get('res_id'))
             record_tuple_set.add(record_tuple)
-        for record_tuple in record_tuple_set:
-            (res_model, res_id) = record_tuple
-            self.check('create', values={'res_model':res_model, 'res_id':res_id})
+
+        # don't use possible contextual recordset for check, see commit for details
+        Attachments = self.browse()
+        for res_model, res_id in record_tuple_set:
+            Attachments.check('create', values={'res_model':res_model, 'res_id':res_id})
         return super(IrAttachment, self).create(vals_list)
 
     def _post_add_create(self):
@@ -552,15 +655,10 @@ class IrAttachment(models.Model):
 
     @api.model
     def action_get(self):
-        return self.env['ir.actions.act_window'].for_xml_id('base', 'action_attachment')
+        return self.env['ir.actions.act_window']._for_xml_id('base.action_attachment')
 
     @api.model
     def get_serve_attachment(self, url, extra_domain=None, extra_fields=None, order=None):
         domain = [('type', '=', 'binary'), ('url', '=', url)] + (extra_domain or [])
         fieldNames = ['__last_update', 'datas', 'mimetype'] + (extra_fields or [])
         return self.search_read(domain, fieldNames, order=order, limit=1)
-
-    @api.model
-    def get_attachment_by_key(self, key, extra_domain=None, order=None):
-        domain = [('key', '=', key)] + (extra_domain or [])
-        return self.search(domain, order=order, limit=1)
