@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import base64
-from collections import defaultdict
+from collections import defaultdict, OrderedDict
 from decorator import decorator
 from operator import attrgetter
 import importlib
@@ -11,6 +11,7 @@ import os
 import pkg_resources
 import shutil
 import tempfile
+import threading
 import zipfile
 
 import requests
@@ -148,11 +149,12 @@ STATES = [
     ('to install', 'To be installed'),
 ]
 
+
 class Module(models.Model):
     _name = "ir.module.module"
     _rec_name = "shortdesc"
     _description = "Module"
-    _order = 'sequence,name'
+    _order = 'application desc,sequence,name'
 
     @api.model
     def fields_view_get(self, view_id=None, view_type='form', toolbar=False, submenu=False):
@@ -174,8 +176,13 @@ class Module(models.Model):
     @api.depends('name', 'description')
     def _get_desc(self):
         for module in self:
-            path = modules.get_module_resource(module.name, 'static/description/index.html')
-            if path:
+            if not module.name:
+                module.description_html = False
+                continue
+            module_path = modules.get_module_path(module.name, display_warning=False)  # avoid to log warning for fake community module
+            if module_path:
+                path = modules.check_resource_path(module_path, 'static/description/index.html')
+            if module_path and path:
                 with tools.file_open(path, 'rb') as desc_file:
                     doc = desc_file.read()
                     html = lxml.html.document_fromstring(doc)
@@ -239,8 +246,10 @@ class Module(models.Model):
             if module.icon:
                 path_parts = module.icon.split('/')
                 path = modules.get_module_resource(path_parts[1], *path_parts[2:])
-            else:
+            elif module.id:
                 path = modules.module.get_module_icon(module.name)
+            else:
+                path = ''
             if path:
                 with tools.file_open(path, 'rb') as image_file:
                     module.icon_image = base64.b64encode(image_file.read())
@@ -295,17 +304,23 @@ class Module(models.Model):
     icon = fields.Char('Icon URL')
     icon_image = fields.Binary(string='Icon', compute='_get_icon_image')
     to_buy = fields.Boolean('Odoo Enterprise Module', default=False)
+    has_iap = fields.Boolean(compute='_compute_has_iap')
 
     _sql_constraints = [
         ('name_uniq', 'UNIQUE (name)', 'The name of the module must be unique!'),
     ]
 
-    def unlink(self):
-        if not self:
-            return True
+    def _compute_has_iap(self):
+        for module in self:
+            module.has_iap = bool(module.id) and 'iap' in module.upstream_dependencies(exclude_states=('',)).mapped('name')
+
+    @api.ondelete(at_uninstall=False)
+    def _unlink_except_installed(self):
         for module in self:
             if module.state in ('installed', 'to upgrade', 'to remove', 'to install'):
                 raise UserError(_('You are trying to remove a module that is installed or will be installed.'))
+
+    def unlink(self):
         self.clear_caches()
         return super(Module, self).unlink()
 
@@ -365,6 +380,10 @@ class Module(models.Model):
         demo = False
 
         for module in self:
+            if module.state not in states_to_update:
+                demo = demo or module.demo
+                continue
+
             # determine dependency modules to update/others
             update_mods, ready_mods = self.browse(), self.browse()
             for dep in module.dependencies_id:
@@ -559,29 +578,37 @@ class Module(models.Model):
         }
 
     def _button_immediate_function(self, function):
+        if getattr(threading.currentThread(), 'testing', False):
+            raise RuntimeError(
+                "Module operations inside tests are not transactional and thus forbidden.\n"
+                "If you really need to perform module operations to test a specific behavior, it "
+                "is best to write it as a standalone script, and ask the runbot/metastorm team "
+                "for help."
+            )
         try:
             # This is done because the installation/uninstallation/upgrade can modify a currently
             # running cron job and prevent it from finishing, and since the ir_cron table is locked
             # during execution, the lock won't be released until timeout.
             self._cr.execute("SELECT * FROM ir_cron FOR UPDATE NOWAIT")
         except psycopg2.OperationalError:
-            raise UserError(_("The server is busy right now, module operations are not possible at"
-                              " this time, please try again later."))
+            raise UserError(_("Odoo is currently processing a scheduled action.\n"
+                              "Module operations are not possible at this time, "
+                              "please try again later or contact your system administrator."))
         function(self)
 
         self._cr.commit()
-        api.Environment.reset()
-        modules.registry.Registry.new(self._cr.dbname, update_module=True)
-
+        registry = modules.registry.Registry.new(self._cr.dbname, update_module=True)
         self._cr.commit()
-        env = api.Environment(self._cr, self._uid, self._context)
+        self._cr.reset()
+        assert self.env.registry is registry
+
         # pylint: disable=next-method-called
-        config = env['ir.module.module'].next() or {}
+        config = self.env['ir.module.module'].next() or {}
         if config.get('type') not in ('ir.actions.act_window_close',):
             return config
 
         # reload the client; open the first available root menu
-        menu = env['ir.ui.menu'].search([('parent_id', '=', False)])[:1]
+        menu = self.env['ir.ui.menu'].search([('parent_id', '=', False)])[:1]
         return {
             'type': 'ir.actions.client',
             'tag': 'reload',
@@ -601,7 +628,7 @@ class Module(models.Model):
     def button_uninstall(self):
         if 'base' in self.mapped('name'):
             raise UserError(_("The `base` module cannot be uninstalled"))
-        if not all(state in ('installed', 'to upgrade') for state in self.mapped('state')):
+        if any(state not in ('installed', 'to upgrade') for state in self.mapped('state')):
             raise UserError(_(
                 "One or more of the selected modules have already been uninstalled, if you "
                 "believe this to be an error, you may try again later or contact support."
@@ -642,6 +669,16 @@ class Module(models.Model):
         self.update_list()
 
         todo = list(self)
+        if 'base' in self.mapped('name'):
+            # If an installed module is only present in the dependency graph through
+            # a new, uninstalled dependency, it will not have been selected yet.
+            # An update of 'base' should also update these modules, and as a consequence,
+            # install the new dependency.
+            todo.extend(self.search([
+                ('state', '=', 'installed'),
+                ('name', '!=', 'studio_customization'),
+                ('id', 'not in', self.ids),
+            ]))
         i = 0
         while i < len(todo):
             module = todo[i]
@@ -793,7 +830,7 @@ class Module(models.Model):
                     content = response.content
                 except Exception:
                     _logger.exception('Failed to fetch module %s', module_name)
-                    raise UserError(_('The `%s` module appears to be unavailable at the moment, please try again later.') % module_name)
+                    raise UserError(_('The `%s` module appears to be unavailable at the moment, please try again later.', module_name))
                 else:
                     zipfile.ZipFile(io.BytesIO(content)).extractall(tmp)
                     assert os.path.isdir(os.path.join(tmp, module_name))
@@ -888,7 +925,7 @@ class Module(models.Model):
             cat_id = modules.db.create_categories(self._cr, categs)
             self.write({'category_id': cat_id})
 
-    def _update_translations(self, filter_lang=None):
+    def _update_translations(self, filter_lang=None, overwrite=False):
         if not filter_lang:
             langs = self.env['res.lang'].get_installed()
             filter_lang = [code for code, _ in langs]
@@ -901,7 +938,7 @@ class Module(models.Model):
             for mod in update_mods
         }
         mod_names = topological_sort(mod_dict)
-        self.env['ir.translation']._load_module_terms(mod_names, filter_lang)
+        self.env['ir.translation']._load_module_terms(mod_names, filter_lang, overwrite)
 
     def _check(self):
         for module in self:
@@ -917,6 +954,54 @@ class Module(models.Model):
             for module in self.sudo().search([('state', '=', 'installed')])
         }
 
+    @api.model
+    def search_panel_select_range(self, field_name, **kwargs):
+        if field_name == 'category_id':
+            enable_counters = kwargs.get('enable_counters', False)
+            domain = [('parent_id', '=', False), ('child_ids.module_ids', '!=', False)]
+
+            excluded_xmlids = [
+                'base.module_category_website_theme',
+                'base.module_category_theme',
+            ]
+            if not self.user_has_groups('base.group_no_one'):
+                excluded_xmlids.append('base.module_category_hidden')
+
+            excluded_category_ids = []
+            for excluded_xmlid in excluded_xmlids:
+                categ = self.env.ref(excluded_xmlid, False)
+                if not categ:
+                    continue
+                excluded_category_ids.append(categ.id)
+
+            if excluded_category_ids:
+                domain = expression.AND([
+                    domain,
+                    [('id', 'not in', excluded_category_ids)],
+                ])
+
+            records = self.env['ir.module.category'].search_read(domain, ['display_name'], order="sequence")
+
+            values_range = OrderedDict()
+            for record in records:
+                record_id = record['id']
+                if enable_counters:
+                    model_domain = expression.AND([
+                        kwargs.get('search_domain', []),
+                        kwargs.get('category_domain', []),
+                        kwargs.get('filter_domain', []),
+                        [('category_id', 'child_of', record_id), ('category_id', 'not in', excluded_category_ids)]
+                    ])
+                    record['__count'] = self.env['ir.module.module'].search_count(model_domain)
+                values_range[record_id] = record
+
+            return {
+                'parent_field': 'parent_id',
+                'values': list(values_range.values()),
+            }
+
+        return super(Module, self).search_panel_select_range(field_name, **kwargs)
+
 
 DEP_STATES = STATES + [('unknown', 'Unknown')]
 
