@@ -29,15 +29,15 @@ class IrTranslationImport(object):
     """
     _table = 'tmp_ir_translation_import'
 
-    def __init__(self, model):
+    def __init__(self, cr, overwrite=False):
         """ Store some values, and also create a temporary SQL table to accept
         the data.
 
         :param model: the model to insert the data into (as a recordset)
         """
-        self._cr = model._cr
-        self._model_table = model._table
-        self._overwrite = model._context.get('overwrite', False)
+        self._cr = cr
+        self._model_table = "ir_translation"
+        self._overwrite = overwrite
         self._debug = False
         self._rows = []
 
@@ -92,14 +92,6 @@ class IrTranslationImport(object):
         # referencing non-existent data.
         cr.execute("DELETE FROM %s WHERE res_id IS NULL AND module IS NOT NULL" % self._table)
 
-        # detect the xml_translate fields, where the src must be the same
-        env = api.Environment(cr, SUPERUSER_ID, {})
-        src_relevant_fields = []
-        for model in env:
-            for field_name, field in env[model]._fields.items():
-                if hasattr(field, 'translate') and callable(field.translate):
-                    src_relevant_fields.append("%s,%s" % (model, field_name))
-
         count = 0
         # Step 2: insert new or upsert non-noupdate translations
         if self._overwrite:
@@ -139,9 +131,9 @@ class IrTranslationImport(object):
         cr.execute(""" INSERT INTO %s(name, lang, res_id, src, type, value, module, state, comments)
                        SELECT name, lang, res_id, src, type, value, module, state, comments
                        FROM %s
-                       WHERE %s
+                       WHERE %%s OR noupdate is true
                        ON CONFLICT DO NOTHING;
-                   """ % (self._model_table, self._table, 'noupdate IS TRUE' if self._overwrite else 'TRUE'))
+                   """ % (self._model_table, self._table), [not self._overwrite])
         count += cr.rowcount
 
         if self._debug:
@@ -525,9 +517,13 @@ class IrTranslation(models.Model):
         # check for read/write access on translated field records
         fmode = 'read' if mode == 'read' else 'write'
         for mname, ids in model_ids.items():
-            records = self.env[mname].browse(ids)
+            records = self.env[mname].browse(ids).exists()
             records.check_access_rights(fmode)
             records.check_field_access_rights(fmode, model_fields[mname])
+            if mode == 'create' and set(records._ids) != set(ids):
+                raise ValidationError(_("Creating translation on non existing records"))
+            if not records:
+                continue
             records.check_access_rule(fmode)
 
     @api.constrains('type', 'name', 'value')
@@ -549,7 +545,7 @@ class IrTranslation(models.Model):
                         continue
                     value2 = field.translate({val: src}.get, value1)
                     if value2 != value0:
-                        raise ValidationError(_("Translation is not valid:\n%s") % val)
+                        raise ValidationError(_("Translation is not valid:\n%s", val))
 
     @api.model_create_multi
     def create(self, vals_list):
@@ -790,25 +786,20 @@ class IrTranslation(models.Model):
 
         return action
 
-    @api.model
-    def _get_import_cursor(self):
+    def _get_import_cursor(self, overwrite):
         """ Return a cursor-like object for fast inserting translations """
-        return IrTranslationImport(self)
+        return IrTranslationImport(self._cr, overwrite)
 
-    def _load_module_terms(self, modules, langs):
+    def _load_module_terms(self, modules, langs, overwrite=False):
         """ Load PO files of the given modules for the given languages. """
-        # make sure the given languages are active
-        res_lang = self.env['res.lang'].sudo()
-        for lang in langs:
-            res_lang.load_lang(lang)
         # load i18n files
         for module_name in modules:
             modpath = get_module_path(module_name)
             if not modpath:
                 continue
             for lang in langs:
-                context = dict(self._context)
                 lang_code = tools.get_iso_codes(lang)
+                lang_overwrite = overwrite
                 base_lang_code = None
                 if '_' in lang_code:
                     base_lang_code = lang_code.split('_')[0]
@@ -818,28 +809,28 @@ class IrTranslation(models.Model):
                     base_trans_file = get_module_resource(module_name, 'i18n', base_lang_code + '.po')
                     if base_trans_file:
                         _logger.info('module %s: loading base translation file %s for language %s', module_name, base_lang_code, lang)
-                        tools.trans_load(self._cr, base_trans_file, lang, verbose=False, module_name=module_name, context=context)
-                        context['overwrite'] = True  # make sure the requested translation will override the base terms later
+                        tools.trans_load(self._cr, base_trans_file, lang, verbose=False, overwrite=lang_overwrite)
+                        lang_overwrite = True  # make sure the requested translation will override the base terms later
 
                     # i18n_extra folder is for additional translations handle manually (eg: for l10n_be)
                     base_trans_extra_file = get_module_resource(module_name, 'i18n_extra', base_lang_code + '.po')
                     if base_trans_extra_file:
                         _logger.info('module %s: loading extra base translation file %s for language %s', module_name, base_lang_code, lang)
-                        tools.trans_load(self._cr, base_trans_extra_file, lang, verbose=False, module_name=module_name, context=context)
-                        context['overwrite'] = True  # make sure the requested translation will override the base terms later
+                        tools.trans_load(self._cr, base_trans_extra_file, lang, verbose=False, overwrite=lang_overwrite)
+                        lang_overwrite = True  # make sure the requested translation will override the base terms later
 
                 # Step 2: then load the main translation file, possibly overriding the terms coming from the base language
                 trans_file = get_module_resource(module_name, 'i18n', lang_code + '.po')
                 if trans_file:
                     _logger.info('module %s: loading translation file (%s) for language %s', module_name, lang_code, lang)
-                    tools.trans_load(self._cr, trans_file, lang, verbose=False, module_name=module_name, context=context)
+                    tools.trans_load(self._cr, trans_file, lang, verbose=False, overwrite=lang_overwrite)
                 elif lang_code != 'en_US':
                     _logger.info('module %s: no translation for language %s', module_name, lang_code)
 
                 trans_extra_file = get_module_resource(module_name, 'i18n_extra', lang_code + '.po')
                 if trans_extra_file:
                     _logger.info('module %s: loading extra translation file (%s) for language %s', module_name, lang_code, lang)
-                    tools.trans_load(self._cr, trans_extra_file, lang, verbose=False, module_name=module_name, context=context)
+                    tools.trans_load(self._cr, trans_extra_file, lang, verbose=False, overwrite=lang_overwrite)
         return True
 
     @api.model
@@ -877,7 +868,7 @@ class IrTranslation(models.Model):
             mods = [x['name'] for x in self.env['ir.module.module'].sudo().search_read(
                 [('state', '=', 'installed')], ['name'])]
         if not lang:
-            lang = self._context["lang"]
+            lang = self._context.get("lang")
         langs = self.env['res.lang']._lang_get(lang)
         lang_params = None
         if langs:
