@@ -3,11 +3,11 @@ odoo.define('base.settings', function (require) {
 
 var BasicModel = require('web.BasicModel');
 var core = require('web.core');
-var config = require('web.config');
 var FormView = require('web.FormView');
 var FormController = require('web.FormController');
 var FormRenderer = require('web.FormRenderer');
 var view_registry = require('web.view_registry');
+const Dialog = require('web.Dialog');
 
 var QWeb = core.qweb;
 var _t = core._t;
@@ -24,16 +24,6 @@ var BaseSettingRenderer = FormRenderer.extend({
         this.activeTab = false;
     },
 
-    start: function () {
-        var prom = this._super.apply(this, arguments);
-        if (config.device.isMobile) {
-            core.bus.on("DOM_updated", this, function () {
-                this._moveToTab(this.currentIndex || this._currentAppIndex());
-            });
-        }
-        return prom;
-    },
-
     /**
      * @override
      * overridden to show a message, informing user that there are changes
@@ -113,49 +103,6 @@ var BaseSettingRenderer = FormRenderer.extend({
             this.searchText = "";
         }
     },
-    /**
-     * In mobile view behaviour is like swipe content left / right and apps tab will be shown on the top.
-     * This method will set the required properties (styling and css).
-     *
-     * @private
-     * @param {int} currentTab
-     */
-    _activateSettingMobileTab: function (currentTab) {
-        var self = this;
-        var moveTo = currentTab;
-        var next = moveTo + 1;
-        var previous = moveTo - 1;
-
-        this.$(".settings .app_settings_block").removeClass("previous next current before after");
-        this.$(".settings_tab .tab").removeClass("previous next current before after");
-        _.each(this.modules, function (module, index) {
-            var tab = self.$(".tab[data-key='" + module.key + "']");
-            var view = module.settingView;
-
-            if (index === previous) {
-                tab.addClass("previous");
-                tab.css("margin-left", "0px");
-                view.addClass("previous");
-            } else if (index === next) {
-                tab.addClass("next");
-                tab.css("margin-left", "-" + tab.outerWidth() + "px");
-                view.addClass("next");
-            } else if (index < moveTo) {
-                tab.addClass("before");
-                tab.css("margin-left", "-" + tab.outerWidth() + "px");
-                view.addClass("before");
-            } else if (index === moveTo) {
-                var marginLeft = tab.outerWidth() / 2;
-                tab.css("margin-left", "-" + marginLeft + "px");
-                tab.addClass("current");
-                view.addClass("current");
-            } else if (index > moveTo) {
-                tab.addClass("after");
-                tab.css("margin-left", "0");
-                view.addClass("after");
-            }
-        });
-    },
     /**
      * find current app index in modules
      *
@@ -167,22 +114,6 @@ var BaseSettingRenderer = FormRenderer.extend({
         });
         return index;
     },
-    /**
-     * Enables swipe navigation between settings pages
-     *
-     * @private
-     */
-    _enableSwipe: function () {
-        var self = this;
-        this.$('.settings').swipe({
-            swipeLeft: function () {
-                self._moveToTab(self.currentIndex + 1);
-            },
-            swipeRight: function () {
-                self._moveToTab(self.currentIndex - 1);
-            }
-        });
-    },
     /**
      *
      * @private
@@ -225,12 +156,7 @@ var BaseSettingRenderer = FormRenderer.extend({
             view.removeClass("o_hidden");
             this.activeView = view;
             this.activeTab = tab;
-
-            if (config.device.isMobile) {
-                this._activateSettingMobileTab(this.currentIndex);
-            } else {
-                tab.addClass("selected");
-            }
+            tab.addClass("selected");
         }
     },
 
@@ -250,10 +176,6 @@ var BaseSettingRenderer = FormRenderer.extend({
     _onKeyUpSearch: function (event) {
         this.searchText = this.searchInput.val();
         this.activeTab.removeClass('selected');
-        if (config.device.isMobile) {
-            this.$('.settings_tab').addClass('o_hidden');
-            this.$('.settings').addClass('d-block');
-        }
         this._searchSetting();
     },
     /**
@@ -268,14 +190,10 @@ var BaseSettingRenderer = FormRenderer.extend({
             module.settingView.find('.o_setting_box').removeClass('o_hidden');
             module.settingView.find('h2').removeClass('o_hidden');
             module.settingView.find('.settingSearchHeader').addClass('o_hidden');
-            module.settingView.find('.o_settings_container').addClass('mt16');
+            module.settingView.find('.o_settings_container').addClass('mt16').removeClass('mb-0');
         });
         this.activeTab.removeClass('o_hidden').addClass('selected');
         this.activeView.removeClass('o_hidden');
-        if (config.device.isMobile) {
-            this.$('.settings_tab').removeClass('o_hidden');
-            this.$('.settings').removeClass('d-block');
-        }
     },
 
     _render: function () {
@@ -284,10 +202,6 @@ var BaseSettingRenderer = FormRenderer.extend({
             self._initModules();
             self._renderLeftPanel();
             self._initSearch();
-            
-            if (config.device.isMobile) {
-                self._enableSwipe();
-            }
         });
     },
 
@@ -319,47 +233,60 @@ var BaseSettingRenderer = FormRenderer.extend({
             module.settingView.find('.o_setting_box').addClass('o_hidden');
             module.settingView.find('h2').addClass('o_hidden');
             module.settingView.find('.settingSearchHeader').addClass('o_hidden');
-            module.settingView.find('.o_settings_container').removeClass('mt16');
-            var resultSetting = module.settingView.find(".o_form_label:containsTextLike('" + self.searchText + "')");
-            if (resultSetting.length > 0) {
-                resultSetting.each(function () {
-                    var settingBox = $(this).closest('.o_setting_box');
+            module.settingView.find('.o_settings_container').removeClass('mt16').addClass('mb-0');
+
+            const upperCasedSearchText = self.searchText.toUpperCase();
+            const [matches, others] = _.partition(module.settingView.find(".o_form_label"),
+                (e) => e.textContent.toUpperCase().includes(upperCasedSearchText));
+            if (matches.length) {
+                for (let result of matches) {
+                    const settingBox = $(result).closest('.o_setting_box');
                     if (!settingBox.hasClass('o_invisible_modifier')) {
                         settingBox.removeClass('o_hidden');
-                        $(this).html(self._wordHighlighter($(this).html(), self.searchText));
+                        self._wordHighlighter(result, upperCasedSearchText);
                     } else {
                         self.inVisibleCount++;
                     }
-                });
-                if (self.inVisibleCount !== resultSetting.length) {
+                }
+                if (self.inVisibleCount !== matches.length) {
                     module.settingView.find('.settingSearchHeader').removeClass('o_hidden');
                     module.settingView.removeClass('o_hidden');
                 }
             } else {
                 ++self.count;
             }
+            others.filter(e => e.firstElementChild).forEach(e => self._removeHighlight(e));
         });
         this.count === _.size(this.modules) ? this.$('.notFound').removeClass('o_hidden') : this.$('.notFound').addClass('o_hidden');
         if (this.searchText.length === 0) {
             this._resetSearch();
         }
     },
+
     /**
      * highlight search word
      *
      * @private
-     * @param {string} text
-     * @param {string} word
+     * @param {HTMLElement} node
+     * @param {string} upperCasedSearchText
      */
-    _wordHighlighter: function (text, word) {
-        if (text.indexOf('highlighter') !== -1) {
-            text = text.replace('<span class="highlighter">', "");
-            text = text.replace("</span>", "");
-        }
-        var match = text.search(new RegExp(word, "i"));
-        word = text.substring(match, match + word.length);
-        var highlightedWord = "<span class='highlighter'>" + word + '</span>';
-        return text.replace(word, highlightedWord);
+    _wordHighlighter: function (node, upperCasedSearchText) {
+        const text = node.textContent;
+        const startIndex = text.toUpperCase().indexOf(upperCasedSearchText);
+        const endIndex = startIndex + upperCasedSearchText.length;
+        $(node).empty().append(
+            document.createTextNode(text.substring(0, startIndex)),
+            $('<span class="highlighter">').text(text.substring(startIndex, endIndex)),
+            document.createTextNode(text.substring(endIndex))
+        );
+    },
+
+    /**
+     * @param {HTMLElement} node
+     * @private
+     */
+    _removeHighlight: function(node) {
+        node.textContent = node.textContent;
     },
 });
 
@@ -371,6 +298,10 @@ var BaseSettingController = FormController.extend({
         this._super.apply(this, arguments);
         this.disableAutofocus = true;
         this.renderer.activeSettingTab = this.initialState.context.module;
+        // discardingDef is used to ensure that we don't ask twice the user if
+        // he wants to discard changes, when 'canBeDiscarded' is called several
+        // times "in parallel"
+        this.discardingDef = null;
     },
     /**
      * Settings view should always be in edit mode, so we have to override
@@ -381,6 +312,100 @@ var BaseSettingController = FormController.extend({
     willRestore: function () {
         this.mode = 'edit';
     },
+    /**
+     * @override
+     * @returns {Promise}
+     */
+    canBeRemoved: function () {
+        return this.discardChanges(undefined, {
+            noAbandon: true,
+            readonlyIfRealDiscard: true,
+        });
+    },
+    /**
+     * @override
+     * @param {string} recordId
+     * @returns {Promise}
+     */
+    canBeDiscarded: function (recordId) {
+        if (this.discardingDef) {
+            return this.discardingDef;
+        }
+        if (!this.isDirty(recordId)) {
+            return Promise.resolve(false);
+        }
+        const message = _t('Would you like to save your changes?');
+        this.discardingDef = new Promise((resolve, reject) => {
+            const reset = () => {
+                // enable buttons if user first save which fails because of required field missed
+                // and then cancel confirmation dialog
+                this._enableButtons();
+                this.discardingDef = null;
+            };
+            const cancel = () => {
+                reject();
+                reset();
+            };
+            const dialog = Dialog.confirm(this, message, {
+                title: _t('Unsaved changes'),
+                buttons: [{
+                    text: _t('Save'),
+                    classes: 'btn-primary',
+                    click: async () => {
+                        this._disableButtons();
+                        try {
+                            // _onButtonClicked always saves the record even if
+                            // it's discarded. Here we need to save before
+                            // triggering the changes on the server.
+                            await this.saveRecord(recordId, {
+                                stayInEdit: true,
+                            });
+                            const record = this.model.get(recordId);
+                            this.trigger_up('execute_action', {
+                                action_data: {
+                                    context: record.getContext({
+                                        additionalContext: {},
+                                    }),
+                                    name: "execute",
+                                    type: "object",
+                                },
+                                env: {
+                                    context: record.getContext(),
+                                    currentID: record.data.id,
+                                    model: record.model,
+                                    resIDs: record.res_ids,
+                                },
+                                on_success() {
+                                    resolve(false);
+                                    dialog.close();
+                                },
+                                on_fail() {
+                                    cancel();
+                                    dialog.close();
+                                },
+                            });
+                        } catch (e) {
+                            cancel();
+                            dialog.close();
+                        }
+                    },
+                }, {
+                    text: _t('Discard'),
+                    close: true,
+                    click: () => {
+                        resolve(true);
+                        reset();
+                    },
+                }, {
+                    text: _t('Stay Here'),
+                    close: true,
+                    click: cancel,
+                }],
+            });
+            dialog.on('closed', this.discardingDef, cancel);
+        });
+        return this.discardingDef;
+    },
 
     //--------------------------------------------------------------------------
     // Handlers
@@ -395,27 +420,33 @@ var BaseSettingController = FormController.extend({
             var recordID = ev.data.recordID;
             var _super = this._super;
             var args = arguments;
-            this._discardChanges(recordID).then(function () {
+            this._discardChanges(recordID, { noAbandon: true }).then(function () {
                 _super.apply(self, args);
             });
         } else {
             this._super.apply(this, arguments);
         }
     },
+    /**
+     * @override
+     * @private
+     */
+    _onBeforeUnload: function () {
+        // We should not save when leaving Odoo in the settings
+    },
 
 });
 
-var BaseSettingsModel = BasicModel.extend({
-    /**
-     * @override
-     */
-    save: function (recordID) {
-        var self = this;
-        return this._super.apply(this, arguments).then(function (result) {
-            // we remove here the res_id, because the record should still be
-            // considered new.  We want the web client to always perform a
-            // default_get to fetch the settings anew.
-            delete self.localData[recordID].res_id;
+const BaseSettingsModel = BasicModel.extend({
+    save(recordID, options) {
+        const savePoint = options && options.savePoint;
+        return this._super.apply(this, arguments).then(result => {
+            if (!savePoint && this.localData[recordID].model === 'res.config.settings') {
+                // we remove here the res_id, because the record should still be
+                // considered new.  We want the web client to always perform a
+                // onchange to fetch the settings anew.
+                delete this.localData[recordID].res_id;
+            }
             return result;
         });
     },
@@ -429,18 +460,6 @@ var BaseSettingView = FormView.extend({
         Renderer: BaseSettingRenderer,
         Controller: BaseSettingController,
     }),
-
-    /**
-     * Overrides to lazy-load touchSwipe library in mobile.
-     *
-     * @override
-    */
-    init: function () {
-        if (config.device.isMobile) {
-            this.jsLibs.push('/web/static/lib/jquery.touchSwipe/jquery.touchSwipe.js');
-        }
-        this._super.apply(this, arguments);
-    },
 });
 
 view_registry.add('base_settings', BaseSettingView);
