@@ -1,4 +1,7 @@
 # -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
+
+from . import common
 from . import test_acl
 from . import test_api
 from . import test_base
@@ -8,7 +11,9 @@ from . import test_db_cursor
 from . import test_expression
 from . import test_float
 from . import test_func
+from . import test_http_case
 from . import test_image
+from . import test_avatar_mixin
 from . import test_ir_actions
 from . import test_ir_attachment
 from . import test_ir_cron
@@ -33,7 +38,6 @@ from . import test_search
 from . import test_translate
 #import test_uninstall  # loop
 from . import test_user_has_group
-from . import test_view_validation
 from . import test_views
 from . import test_xmlrpc
 from . import test_res_partner
@@ -42,6 +46,7 @@ from . import test_res_users
 from . import test_reports
 from . import test_test_suite
 from . import test_tests_tags
-from . import test_base_document_layout
 from . import test_form_create
 from . import test_cloc
+from . import test_profiler
+from . import test_pdf
