@@ -1,15 +1,20 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import api, models
+from odoo import api, models, Command
+from odoo.addons.base.tests.common import SavepointCaseWithUserDemo
 from odoo.tools import mute_logger
-from odoo.tests import common
 from odoo.exceptions import AccessError
 
 
-class TestAPI(common.TransactionCase):
+class TestAPI(SavepointCaseWithUserDemo):
     """ test the new API of the ORM """
 
+    @classmethod
+    def setUpClass(cls):
+        super(TestAPI, cls).setUpClass()
+        cls._load_partners_set()
+
     def assertIsRecordset(self, value, model):
         self.assertIsInstance(value, models.BaseModel)
         self.assertEqual(value._name, model)
@@ -25,7 +30,7 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_00_query(self):
         """ Build a recordset, and check its contents. """
-        domain = [('name', 'ilike', 'j')]
+        domain = [('name', 'ilike', 'j'), ('id', 'in', self.partners.ids)]
         partners = self.env['res.partner'].search(domain)
 
         # partners is a collection of browse records
@@ -39,8 +44,8 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_01_query_offset(self):
         """ Build a recordset with offset, and check equivalence. """
-        partners1 = self.env['res.partner'].search([], offset=10)
-        partners2 = self.env['res.partner'].search([])[10:]
+        partners1 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], offset=5)
+        partners2 = self.env['res.partner'].search([('id', 'in', self.partners.ids)])[5:]
         self.assertIsRecordset(partners1, 'res.partner')
         self.assertIsRecordset(partners2, 'res.partner')
         self.assertEqual(list(partners1), list(partners2))
@@ -48,8 +53,8 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_02_query_limit(self):
         """ Build a recordset with offset, and check equivalence. """
-        partners1 = self.env['res.partner'].search([], limit=10)
-        partners2 = self.env['res.partner'].search([])[:10]
+        partners1 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], order='id asc', limit=5)
+        partners2 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], order='id asc')[:5]
         self.assertIsRecordset(partners1, 'res.partner')
         self.assertIsRecordset(partners2, 'res.partner')
         self.assertEqual(list(partners1), list(partners2))
@@ -57,8 +62,8 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_03_query_offset_limit(self):
         """ Build a recordset with offset and limit, and check equivalence. """
-        partners1 = self.env['res.partner'].search([], offset=3, limit=7)
-        partners2 = self.env['res.partner'].search([])[3:10]
+        partners1 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], order='id asc', offset=3, limit=7)
+        partners2 = self.env['res.partner'].search([('id', 'in', self.partners.ids)], order='id asc')[3:10]
         self.assertIsRecordset(partners1, 'res.partner')
         self.assertIsRecordset(partners2, 'res.partner')
         self.assertEqual(list(partners1), list(partners2))
@@ -76,7 +81,7 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_05_immutable(self):
         """ Check that a recordset remains the same, even after updates. """
-        domain = [('name', 'ilike', 'g')]
+        domain = [('name', 'ilike', 'g'), ('id', 'in', self.partners.ids)]
         partners = self.env['res.partner'].search(domain)
         self.assertTrue(partners)
         ids = partners.ids
@@ -97,24 +102,23 @@ class TestAPI(common.TransactionCase):
         self.assertIsRecord(user.partner_id, 'res.partner')
         self.assertIsRecordset(user.groups_id, 'res.groups')
 
-        partners = self.env['res.partner'].search([])
-        for name, field in partners._fields.items():
+        for name, field in self.partners._fields.items():
             if field.type == 'many2one':
-                for p in partners:
+                for p in self.partners:
                     self.assertIsRecord(p[name], field.comodel_name)
             elif field.type == 'reference':
-                for p in partners:
+                for p in self.partners:
                     if p[name]:
                         self.assertIsRecord(p[name], field.comodel_name)
             elif field.type in ('one2many', 'many2many'):
-                for p in partners:
+                for p in self.partners:
                     self.assertIsRecordset(p[name], field.comodel_name)
 
     @mute_logger('odoo.models')
     def test_07_null(self):
         """ Check behavior of null instances. """
         # select a partner without a parent
-        partner = self.env['res.partner'].search([('parent_id', '=', False)])[0]
+        partner = self.env['res.partner'].search([('parent_id', '=', False), ('id', 'in', self.partners.ids)])[0]
 
         # check partner and related null instances
         self.assertTrue(partner)
@@ -136,7 +140,7 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_40_new_new(self):
         """ Call new-style methods in the new API style. """
-        partners = self.env['res.partner'].search([('name', 'ilike', 'g')])
+        partners = self.env['res.partner'].search([('name', 'ilike', 'g'), ('id', 'in', self.partners.ids)])
         self.assertTrue(partners)
 
         # call method write on partners itself, and check its effect
@@ -147,7 +151,7 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_45_new_new(self):
         """ Call new-style methods on records (new API style). """
-        partners = self.env['res.partner'].search([('name', 'ilike', 'g')])
+        partners = self.env['res.partner'].search([('name', 'ilike', 'g'), ('id', 'in', self.partners.ids)])
         self.assertTrue(partners)
 
         # call method write on partner records, and check its effects
@@ -161,7 +165,7 @@ class TestAPI(common.TransactionCase):
     def test_50_environment(self):
         """ Test environment on records. """
         # partners and reachable records are attached to self.env
-        partners = self.env['res.partner'].search([('name', 'ilike', 'j')])
+        partners = self.env['res.partner'].search([('name', 'ilike', 'j'), ('id', 'in', self.partners.ids)])
         self.assertEqual(partners.env, self.env)
         for x in (partners, partners[0], partners[0].company_id):
             self.assertEqual(x.env, self.env)
@@ -172,8 +176,12 @@ class TestAPI(common.TransactionCase):
         partners[0].company_id.name
         partners[0].company_id.write({'name': 'Fools'})
 
-        # create an environment with the demo user
-        demo = self.env['res.users'].search([('login', '=', 'demo')])[0]
+        # create an environment with a demo user
+        demo = self.env['res.users'].create({
+            'name': 'test_environment_demo',
+            'login': 'test_environment_demo',
+            'password': 'test_environment_demo',
+        })
         demo_env = self.env(user=demo)
         self.assertNotEqual(demo_env, self.env)
 
@@ -193,16 +201,18 @@ class TestAPI(common.TransactionCase):
             self.assertEqual(p.env, demo_env)
 
         # demo user can read but not modify company data
-        demo_partners[0].company_id.name
+        demo_partner = self.env['res.partner'].search([('name', '=', 'Landon Roberts')]).with_user(demo)
+        self.assertTrue(demo_partner.company_id, 'This partner is supposed to be linked to a company')
+        demo_partner.company_id.name
         with self.assertRaises(AccessError):
-            demo_partners[0].company_id.write({'name': 'Pricks'})
+            demo_partner.company_id.write({'name': 'Pricks'})
 
         # remove demo user from all groups
-        demo.write({'groups_id': [(5,)]})
+        demo.write({'groups_id': [Command.clear()]})
 
         # demo user can no longer access partner data
         with self.assertRaises(AccessError):
-            demo_partners[0].company_id.name
+            demo_partner.company_id.name
 
     @mute_logger('odoo.models')
     def test_60_cache(self):
@@ -217,7 +227,7 @@ class TestAPI(common.TransactionCase):
         for p in data:
             pids.append(Partners.create({
                 'name': p,
-                'child_ids': [(0, 0, {'name': c}) for c in data[p]],
+                'child_ids': [Command.create({'name': c}) for c in data[p]],
             }).id)
 
         partners = Partners.search([('id', 'in', pids)])
@@ -263,12 +273,12 @@ class TestAPI(common.TransactionCase):
         partner.country_id, partner.child_ids
         data = partner._convert_to_write(partner._cache)
         self.assertEqual(data['country_id'], partner.country_id.id)
-        self.assertEqual(data['child_ids'], [(6, 0, partner.child_ids.ids)])
+        self.assertEqual(data['child_ids'], [Command.set(partner.child_ids.ids)])
 
     @mute_logger('odoo.models')
     def test_60_prefetch(self):
         """ Check the record cache prefetching """
-        partners = self.env['res.partner'].search([], limit=models.PREFETCH_MAX)
+        partners = self.env['res.partner'].search([('id', 'in', self.partners.ids)], limit=models.PREFETCH_MAX)
         self.assertTrue(len(partners) > 1)
 
         # all the records in partners are ready for prefetching
@@ -303,7 +313,7 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_60_prefetch_model(self):
         """ Check the prefetching model. """
-        partners = self.env['res.partner'].search([], limit=models.PREFETCH_MAX)
+        partners = self.env['res.partner'].search([('id', 'in', self.partners.ids)], limit=models.PREFETCH_MAX)
         self.assertTrue(partners)
 
         def same_prefetch(a, b):
@@ -315,11 +325,11 @@ class TestAPI(common.TransactionCase):
         # the recordset operations below use different prefetch sets
         diff_prefetch(partners, partners.browse())
         diff_prefetch(partners, partners[0])
-        diff_prefetch(partners, partners[:10])
+        diff_prefetch(partners, partners[:5])
 
         # the recordset operations below share the prefetch set
         same_prefetch(partners, partners.browse(partners.ids))
-        same_prefetch(partners, partners.with_user(self.env.ref('base.user_demo')))
+        same_prefetch(partners, partners.with_user(self.user_demo))
         same_prefetch(partners, partners.with_context(active_test=False))
         same_prefetch(partners, partners[:10].with_prefetch(partners._prefetch_ids))
 
@@ -337,8 +347,8 @@ class TestAPI(common.TransactionCase):
         vals1 = {
             'name': 'Non-empty relational fields',
             'country_id': self.ref('base.be'),
-            'bank_ids': [(0, 0, {'acc_number': 'FOO42'})],
-            'category_id': [(4, self.ref('base.res_partner_category_0'))],
+            'bank_ids': [Command.create({'acc_number': 'FOO42'})],
+            'category_id': [Command.link(self.partner_category.id)],
         }
         partners = partners.create(vals0) + partners.create(vals1)
         for partner in partners:
@@ -376,7 +386,7 @@ class TestAPI(common.TransactionCase):
     def test_70_one(self):
         """ Check method one(). """
         # check with many records
-        ps = self.env['res.partner'].search([('name', 'ilike', 'a')])
+        ps = self.env['res.partner'].search([('name', 'ilike', 'a'), ('id', 'in', self.partners.ids)])
         self.assertTrue(len(ps) > 1)
         with self.assertRaises(ValueError):
             ps.ensure_one()
@@ -393,15 +403,15 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_80_contains(self):
         """ Test membership on recordset. """
-        p1 = self.env['res.partner'].search([('name', 'ilike', 'a')], limit=1).ensure_one()
-        ps = self.env['res.partner'].search([('name', 'ilike', 'a')])
+        p1 = self.env['res.partner'].search([('name', 'ilike', 'a'), ('id', 'in', self.partners.ids)], limit=1).ensure_one()
+        ps = self.env['res.partner'].search([('name', 'ilike', 'a'), ('id', 'in', self.partners.ids)])
         self.assertTrue(p1 in ps)
 
     @mute_logger('odoo.models')
     def test_80_set_operations(self):
         """ Check set operations on recordsets. """
-        pa = self.env['res.partner'].search([('name', 'ilike', 'a')])
-        pb = self.env['res.partner'].search([('name', 'ilike', 'b')])
+        pa = self.env['res.partner'].search([('name', 'ilike', 'a'), ('id', 'in', self.partners.ids)])
+        pb = self.env['res.partner'].search([('name', 'ilike', 'b'), ('id', 'in', self.partners.ids)])
         self.assertTrue(pa)
         self.assertTrue(pb)
         self.assertTrue(set(pa) & set(pb))
@@ -453,7 +463,7 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_80_filter(self):
         """ Check filter on recordsets. """
-        ps = self.env['res.partner'].search([])
+        ps = self.partners
         customers = ps.browse([p.id for p in ps if p.employee])
 
         # filter on a single field
@@ -469,7 +479,7 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_80_map(self):
         """ Check map on recordsets. """
-        ps = self.env['res.partner'].search([])
+        ps = self.partners
         parents = ps.browse()
         for p in ps:
             parents |= p.parent_id
@@ -499,7 +509,7 @@ class TestAPI(common.TransactionCase):
     @mute_logger('odoo.models')
     def test_80_sorted(self):
         """ Check sorted on recordsets. """
-        ps = self.env['res.partner'].search([])
+        ps = self.env['res.partner'].search([('id', 'in', self.partners.ids)])
 
         # sort by model order
         qs = ps[:len(ps) // 2] + ps[len(ps) // 2:]
@@ -511,11 +521,11 @@ class TestAPI(common.TransactionCase):
         self.assertEqual(ps.sorted('name').ids, by_name_ids)
 
         # sort by inverse name, with a field name
-        by_name_ids.reverse()
+        by_name_ids = [p.id for p in sorted(ps, key=lambda p: p.name, reverse=True)]
         self.assertEqual(ps.sorted('name', reverse=True).ids, by_name_ids)
 
 
-class TestExternalAPI(common.TransactionCase):
+class TestExternalAPI(SavepointCaseWithUserDemo):
 
     def test_call_kw(self):
         """kwargs is not modified by the execution of the call"""
