@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from functools import partial
 
 import odoo
 from odoo.sql_db import TestCursor
@@ -32,21 +33,15 @@ class TestExecute(BaseCase):
 
 
 class TestTestCursor(common.TransactionCase):
-    @classmethod
-    def setUpClass(cls):
-        super(TestTestCursor, cls).setUpClass()
-        r = registry()
-        r.enter_test_mode(r.cursor())
-
-    @classmethod
-    def tearDownClass(cls):
-        r = registry()
-        r.test_cr.close()
-        r.leave_test_mode()
-        super(TestTestCursor, cls).tearDownClass()
-
     def setUp(self):
-        super(TestTestCursor, self).setUp()
+        super().setUp()
+        # make the registry in test mode
+        self.registry.enter_test_mode(self.cr)
+        self.addCleanup(self.registry.leave_test_mode)
+        # now we make a test cursor for self.cr
+        self.cr = self.registry.cursor()
+        self.addCleanup(self.cr.close)
+        self.env = odoo.api.Environment(self.cr, odoo.SUPERUSER_ID, {})
         self.record = self.env['res.partner'].create({'name': 'Foo'})
 
     def write(self, record, value):
@@ -118,33 +113,57 @@ class TestTestCursor(common.TransactionCase):
 
 
 class TestCursorHooks(common.TransactionCase):
-    def test_hooks(self):
-        log = []
+    def setUp(self):
+        super().setUp()
+        self.log = []
 
-        def make_hook(msg):
-            def hook():
-                log.append(msg)
-            return hook
+    def prepare_hooks(self, cr):
+        self.log.clear()
+        cr.precommit.add(partial(self.log.append, 'preC'))
+        cr.postcommit.add(partial(self.log.append, 'postC'))
+        cr.prerollback.add(partial(self.log.append, 'preR'))
+        cr.postrollback.add(partial(self.log.append, 'postR'))
+        self.assertEqual(self.log, [])
 
+    def test_hooks_on_cursor(self):
         cr = self.registry.cursor()
 
         # check hook on commit()
-        cr.after('commit', make_hook('C1'))
-        cr.after('rollback', make_hook('R1'))
-        self.assertEqual(log, [])
+        self.prepare_hooks(cr)
         cr.commit()
-        self.assertEqual(log, ['C1'])
+        self.assertEqual(self.log, ['preC', 'postC'])
 
-        # check hook on rollback()
-        cr.after('commit', make_hook('C2'))
-        cr.after('rollback', make_hook('R2'))
-        self.assertEqual(log, ['C1'])
+        # check hook on flush(), then on rollback()
+        self.prepare_hooks(cr)
+        cr.flush()
+        self.assertEqual(self.log, ['preC'])
         cr.rollback()
-        self.assertEqual(log, ['C1', 'R2'])
+        self.assertEqual(self.log, ['preC', 'preR', 'postR'])
 
         # check hook on close()
-        cr.after('commit', make_hook('C3'))
-        cr.after('rollback', make_hook('R3'))
-        self.assertEqual(log, ['C1', 'R2'])
+        self.prepare_hooks(cr)
         cr.close()
-        self.assertEqual(log, ['C1', 'R2', 'R3'])
+        self.assertEqual(self.log, ['preR', 'postR'])
+
+    def test_hooks_on_testcursor(self):
+        self.registry.enter_test_mode(self.cr)
+        self.addCleanup(self.registry.leave_test_mode)
+
+        cr = self.registry.cursor()
+
+        # check hook on commit(); post-commit hooks are ignored
+        self.prepare_hooks(cr)
+        cr.commit()
+        self.assertEqual(self.log, ['preC'])
+
+        # check hook on flush(), then on rollback()
+        self.prepare_hooks(cr)
+        cr.flush()
+        self.assertEqual(self.log, ['preC'])
+        cr.rollback()
+        self.assertEqual(self.log, ['preC', 'preR', 'postR'])
+
+        # check hook on close()
+        self.prepare_hooks(cr)
+        cr.close()
+        self.assertEqual(self.log, ['preR', 'postR'])
