@@ -8,6 +8,7 @@ import psycopg2
 from odoo.exceptions import AccessError, MissingError
 from odoo.tests.common import TransactionCase
 from odoo.tools import mute_logger
+from odoo import Command
 
 
 class TestORM(TransactionCase):
@@ -16,9 +17,9 @@ class TestORM(TransactionCase):
     @mute_logger('odoo.models')
     def test_access_deleted_records(self):
         """ Verify that accessing deleted records works as expected """
-        p1 = self.env['res.partner'].create({'name': 'W'})
-        p2 = self.env['res.partner'].create({'name': 'Y'})
-        p1.unlink()
+        c1 = self.env['res.partner.category'].create({'name': 'W'})
+        c2 = self.env['res.partner.category'].create({'name': 'Y'})
+        c1.unlink()
 
         # read() is expected to skip deleted records because our API is not
         # transactional for a sequence of search()->read() performed from the
@@ -28,14 +29,14 @@ class TestORM(TransactionCase):
         user = self.env['res.users'].create({
             'name': 'test user',
             'login': 'test2',
-            'groups_id': [(6, 0, [self.ref('base.group_user')])],
+            'groups_id': [Command.set([self.ref('base.group_user')])],
         })
-        ps = (p1 + p2).with_user(user)
-        self.assertEqual([{'id': p2.id, 'name': 'Y'}], ps.read(['name']), "read() should skip deleted records")
-        self.assertEqual([], ps[0].read(['name']), "read() should skip deleted records")
+        cs = (c1 + c2).with_user(user)
+        self.assertEqual([{'id': c2.id, 'name': 'Y'}], cs.read(['name']), "read() should skip deleted records")
+        self.assertEqual([], cs[0].read(['name']), "read() should skip deleted records")
 
         # Deleting an already deleted record should be simply ignored
-        self.assertTrue(p1.unlink(), "Re-deleting should be a no-op")
+        self.assertTrue(c1.unlink(), "Re-deleting should be a no-op")
 
     @mute_logger('odoo.models')
     def test_access_partial_deletion(self):
@@ -63,7 +64,7 @@ class TestORM(TransactionCase):
         user = self.env['res.users'].create({
             'name': 'test user',
             'login': 'test2',
-            'groups_id': [(6, 0, [self.ref('base.group_user')])],
+            'groups_id': [Command.set([self.ref('base.group_user')])],
         })
 
         partner_model = self.env['ir.model'].search([('model','=','res.partner')])
@@ -228,14 +229,14 @@ class TestORM(TransactionCase):
         user = self.env['res.users'].create({
             'name': 'test',
             'login': 'test_m2m_store_trigger',
-            'groups_id': [(6, 0, [])],
+            'groups_id': [Command.set([])],
         })
         self.assertTrue(user.share)
 
-        group_user.write({'users': [(4, user.id)]})
+        group_user.write({'users': [Command.link(user.id)]})
         self.assertFalse(user.share)
 
-        group_user.write({'users': [(3, user.id)]})
+        group_user.write({'users': [Command.unlink(user.id)]})
         self.assertTrue(user.share)
 
     @mute_logger('odoo.models')
@@ -244,24 +245,21 @@ class TestORM(TransactionCase):
         user = self.env['res.users'].create({
             'name': 'Justine Bridou',
             'login': 'saucisson',
-            'groups_id': [(6, 0, [self.ref('base.group_partner_manager')])],
+            'groups_id': [Command.set([self.ref('base.group_partner_manager')])],
         })
         p1 = self.env['res.partner'].with_user(user).create({'name': 'Zorro'})
-        p1_prop = self.env['ir.property'].with_user(user).create({
-            'name': 'Slip en laine',
-            'res_id': 'res.partner,{}'.format(p1.id),
-            'fields_id': self.env['ir.model.fields'].search([
-                ('model', '=', 'res.partner'), ('name', '=', 'ref')], limit=1).id,
-            'value_text': 'Nain poilu',
-            'type': 'char',
-        })
+        self.env['ir.property'].with_user(user)._set_multi("ref", "res.partner", {p1.id: "Nain poilu"})
+        p1_prop = self.env['ir.property'].with_user(user)._get("ref", "res.partner", res_id=p1.id)
+        self.assertEqual(
+            p1_prop, "Nain poilu", 'p1_prop should have been created')
 
         # Unlink with unprivileged user
         p1.unlink()
 
         # ir.property is deleted
+        p1_prop = self.env['ir.property'].with_user(user)._get("ref", "res.partner", res_id=p1.id)
         self.assertEqual(
-            p1_prop.exists(), self.env['ir.property'], 'p1_prop should have been deleted')
+            p1_prop, False, 'p1_prop should have been deleted')
 
     def test_create_multi(self):
         """ create for multiple records """
@@ -287,16 +285,16 @@ class TestORM(TransactionCase):
         vals_list = [{
             'name': 'Foo',
             'state_ids': [
-                (0, 0, {'name': 'North Foo', 'code': 'NF'}),
-                (0, 0, {'name': 'South Foo', 'code': 'SF'}),
-                (0, 0, {'name': 'West Foo', 'code': 'WF'}),
-                (0, 0, {'name': 'East Foo', 'code': 'EF'}),
+                Command.create({'name': 'North Foo', 'code': 'NF'}),
+                Command.create({'name': 'South Foo', 'code': 'SF'}),
+                Command.create({'name': 'West Foo', 'code': 'WF'}),
+                Command.create({'name': 'East Foo', 'code': 'EF'}),
             ],
         }, {
             'name': 'Bar',
             'state_ids': [
-                (0, 0, {'name': 'North Bar', 'code': 'NB'}),
-                (0, 0, {'name': 'South Bar', 'code': 'SB'}),
+                Command.create({'name': 'North Bar', 'code': 'NB'}),
+                Command.create({'name': 'South Bar', 'code': 'SB'}),
             ],
         }]
         foo, bar = self.env['res.country'].create(vals_list)
@@ -410,141 +408,3 @@ class TestInherits(TransactionCase):
         user.write({'image_1920': 'R0lGODlhAQABAIAAAP///////yH5BAEKAAEALAAAAAABAAEAAAICTAEAOw=='})
         write_date_after = user.write_date
         self.assertNotEqual(write_date_before, write_date_after)
-
-
-CREATE = lambda values: (0, False, values)
-UPDATE = lambda id, values: (1, id, values)
-DELETE = lambda id: (2, id, False)
-FORGET = lambda id: (3, id, False)
-LINK_TO = lambda id: (4, id, False)
-DELETE_ALL = lambda: (5, False, False)
-REPLACE_WITH = lambda ids: (6, False, ids)
-
-
-class TestO2MSerialization(TransactionCase):
-    """ test the orm method 'write' on one2many fields """
-
-    def setUp(self):
-        super(TestO2MSerialization, self).setUp()
-        self.partner = self.registry('res.partner')
-
-    def test_no_command(self):
-        " empty list of commands yields an empty list of records "
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', [])
-        self.assertEqual(results, [])
-
-    def test_CREATE_commands(self):
-        " returns the VALUES dict as-is "
-        values = [{'foo': 'bar'}, {'foo': 'baz'}, {'foo': 'baq'}]
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', [CREATE(v) for v in values])
-        self.assertEqual(results, values)
-
-    def test_LINK_TO_command(self):
-        " reads the records from the database, records are returned with their ids. "
-        ids = [
-            self.env['res.partner'].create({'name': 'foo'}).id,
-            self.env['res.partner'].create({'name': 'bar'}).id,
-            self.env['res.partner'].create({'name': 'baz'}).id,
-        ]
-        commands = [LINK_TO(v) for v in ids]
-
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', commands, ['name'])
-        self.assertItemsEqual(results, [
-            {'id': ids[0], 'name': 'foo'},
-            {'id': ids[1], 'name': 'bar'},
-            {'id': ids[2], 'name': 'baz'},
-        ])
-
-    def test_bare_ids_command(self):
-        " same as the equivalent LINK_TO commands "
-        ids = [
-            self.env['res.partner'].create({'name': 'foo'}).id,
-            self.env['res.partner'].create({'name': 'bar'}).id,
-            self.env['res.partner'].create({'name': 'baz'}).id,
-        ]
-
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', ids, ['name'])
-        self.assertItemsEqual(results, [
-            {'id': ids[0], 'name': 'foo'},
-            {'id': ids[1], 'name': 'bar'},
-            {'id': ids[2], 'name': 'baz'},
-        ])
-
-    def test_UPDATE_command(self):
-        " take the in-db records and merge the provided information in "
-        foo = self.env['res.partner'].create({'name': 'foo'})
-        bar = self.env['res.partner'].create({'name': 'bar'})
-        baz = self.env['res.partner'].create({'name': 'baz', 'city': 'tag'})
-        commands = [
-            LINK_TO(foo.id),
-            UPDATE(bar.id, {'name': 'qux', 'city': 'tagtag'}),
-            UPDATE(baz.id, {'name': 'quux'}),
-        ]
-
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', commands, ['name', 'city'])
-        self.assertItemsEqual(results, [
-            {'id': foo.id, 'name': 'foo', 'city': False},
-            {'id': bar.id, 'name': 'qux', 'city': 'tagtag'},
-            {'id': baz.id, 'name': 'quux', 'city': 'tag'},
-        ])
-
-    def test_DELETE_command(self):
-        " deleted records are not returned at all. "
-        ids = [
-            self.env['res.partner'].create({'name': 'foo'}).id,
-            self.env['res.partner'].create({'name': 'bar'}).id,
-            self.env['res.partner'].create({'name': 'baz'}).id,
-        ]
-        commands = [DELETE(v) for v in ids]
-
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', commands, ['name'])
-        self.assertEqual(results, [])
-
-    def test_mixed_commands(self):
-        ids = [
-            self.env['res.partner'].create({'name': name}).id
-            for name in ['NObar', 'baz', 'qux', 'NOquux', 'NOcorge', 'garply']
-        ]
-        commands = [
-            CREATE({'name': 'foo'}),
-            UPDATE(ids[0], {'name': 'bar'}),
-            LINK_TO(ids[1]),
-            DELETE(ids[2]),
-            UPDATE(ids[3], {'name': 'quux',}),
-            UPDATE(ids[4], {'name': 'corge'}),
-            CREATE({'name': 'grault'}),
-            LINK_TO(ids[5]),
-        ]
-
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', commands, ['name'])
-        self.assertItemsEqual(results, [
-            {'name': 'foo'},
-            {'id': ids[0], 'name': 'bar'},
-            {'id': ids[1], 'name': 'baz'},
-            {'id': ids[3], 'name': 'quux'},
-            {'id': ids[4], 'name': 'corge'},
-            {'name': 'grault'},
-            {'id': ids[5], 'name': 'garply'},
-        ])
-
-    def test_LINK_TO_pairs(self):
-        "LINK_TO commands can be written as pairs, instead of triplets"
-        ids = [
-            self.env['res.partner'].create({'name': 'foo'}).id,
-            self.env['res.partner'].create({'name': 'bar'}).id,
-            self.env['res.partner'].create({'name': 'baz'}).id,
-        ]
-        commands = [(4, id) for id in ids]
-
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', commands, ['name'])
-        self.assertItemsEqual(results, [
-            {'id': ids[0], 'name': 'foo'},
-            {'id': ids[1], 'name': 'bar'},
-            {'id': ids[2], 'name': 'baz'},
-        ])
-
-    def test_singleton_commands(self):
-        "DELETE_ALL can appear as a singleton"
-        commands = [DELETE_ALL()]
-        results = self.env['res.partner'].resolve_2many_commands('child_ids', commands, ['name'])
-        self.assertEqual(results, [])
