@@ -3,10 +3,10 @@
 
 from unittest import TestCase
 
+from odoo.tests import MetaCase
 
-class TestTestSuite(TestCase):
-    test_tags = {'standard', 'at_install'}
-    test_module = 'base'
+
+class TestTestSuite(TestCase, metaclass=MetaCase):
 
     def test_test_suite(self):
         """ Check that OdooSuite handles unittest.TestCase correctly. """
