@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+from odoo.exceptions import AccessError, ValidationError
 from odoo.tools import mute_logger
 from odoo.tools.translate import quote, unquote, xml_translate, html_translate
-from odoo.tests.common import TransactionCase, BaseCase
+from odoo.tests.common import TransactionCase, BaseCase, new_test_user
 from psycopg2 import IntegrityError
 
 
@@ -18,7 +19,7 @@ class TranslationToolsTestCase(BaseCase):
             #print "2:", repr(quoted)
             unquoted = unquote("".join(quoted.split('"\n"')))
             #print "3:", repr(unquoted)
-            self.assertEquals(str, unquoted)
+            self.assertEqual(str, unquoted)
 
         test_string("""test \nall kinds\n \n o\r
          \\\\ nope\n\n"
@@ -41,7 +42,7 @@ class TranslationToolsTestCase(BaseCase):
                         <field name="foo"/>
                     </form>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['Form stuff', 'Blah blah blah', 'Put some more text here'])
 
@@ -50,7 +51,7 @@ class TranslationToolsTestCase(BaseCase):
         terms = []
         source = "Blah blah blah"
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms, [source])
 
     def test_translate_xml_unicode(self):
@@ -58,7 +59,7 @@ class TranslationToolsTestCase(BaseCase):
         terms = []
         source = u"Un heureux évènement"
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms, [source])
 
     def test_translate_xml_text_entity(self):
@@ -66,7 +67,7 @@ class TranslationToolsTestCase(BaseCase):
         terms = []
         source = "Blah&amp;nbsp;blah&amp;nbsp;blah"
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms, [source])
 
     def test_translate_xml_inline1(self):
@@ -78,7 +79,7 @@ class TranslationToolsTestCase(BaseCase):
                         <field name="foo"/>
                     </form>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['Form stuff', 'Blah <i>blah</i> blah', 'Put some <b>more text</b> here'])
 
@@ -91,7 +92,7 @@ class TranslationToolsTestCase(BaseCase):
                         <field name="foo"/>
                     </form>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['Form stuff', 'Blah <i>blah</i> blah', 'Put <em>some <b>more text</b></em> here'])
 
@@ -108,7 +109,7 @@ class TranslationToolsTestCase(BaseCase):
                         </div>
                     </form>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['Form stuff', 'Blah blah blah'])
 
@@ -122,7 +123,7 @@ class TranslationToolsTestCase(BaseCase):
                         </div>
                     </form>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['Form stuff', '<span class="fa fa-globe" title="Title stuff"/>'])
 
@@ -136,7 +137,7 @@ class TranslationToolsTestCase(BaseCase):
                         </div>
                     </form>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms, ['Form stuff'])
 
     def test_translate_xml_t(self):
@@ -148,7 +149,7 @@ class TranslationToolsTestCase(BaseCase):
                         stuff after
                     </t>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['stuff before', 'stuff after'])
 
@@ -161,7 +162,7 @@ class TranslationToolsTestCase(BaseCase):
                         stuff after
                     </div>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['stuff before', 'stuff after'])
 
@@ -173,7 +174,7 @@ class TranslationToolsTestCase(BaseCase):
                         <attribute name="option">Do not translate this</attribute>
                     </field>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['Translate this'])
 
@@ -190,7 +191,7 @@ class TranslationToolsTestCase(BaseCase):
                         </ul>
                     </t>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms,
             ['<span class="oe_menu_text">Blah</span>'])
 
@@ -205,10 +206,10 @@ class TranslationToolsTestCase(BaseCase):
                         </t>
                     </Invoice>"""
         result = xml_translate(terms.append, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
         self.assertItemsEqual(terms, ['Oasis'])
         result = xml_translate(lambda term: term, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
 
     def test_translate_xml_invalid_translations(self):
         """ Test xml_translate() with invalid translations. """
@@ -222,39 +223,40 @@ class TranslationToolsTestCase(BaseCase):
         }
         expect = """<form string="Form stuff">
                         <h1>Blah <i>blah</i> blah</h1>
-                        Mettre &lt;b&gt;plus de texte&lt;/i&gt; ici
-                        <field name="foo"/>
+                        Mettre <b>plus de texte ici
+                        </b><field name="foo"/>
                     </form>"""
         result = xml_translate(translations.get, source)
-        self.assertEquals(result, expect)
+        self.assertEqual(result, expect)
 
     def test_translate_html(self):
         """ Test html_translate(). """
         source = """<blockquote>A <h2>B</h2> C</blockquote>"""
         result = html_translate(lambda term: term, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
 
     def test_translate_html_i(self):
         """ Test xml_translate() and html_translate() with <i> elements. """
         source = """<p>A <i class="fa-check"></i> B</p>"""
         result = xml_translate(lambda term: term, source)
-        self.assertEquals(result, """<p>A <i class="fa-check"/> B</p>""")
+        self.assertEqual(result, """<p>A <i class="fa-check"/> B</p>""")
         result = html_translate(lambda term: term, source)
-        self.assertEquals(result, source)
+        self.assertEqual(result, source)
 
 
 class TestTranslation(TransactionCase):
-
-    def setUp(self):
-        super(TestTranslation, self).setUp()
-        self.env['ir.translation']._load_module_terms(['base'], ['fr_FR'])
-        self.customers = self.env['res.partner.category'].create({'name': 'Customers'})
-        self.env['ir.translation'].create({
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.env['res.lang']._activate_lang('fr_FR')
+        cls.env.ref('base.module_base')._update_translations(['fr_FR'])
+        cls.customers = cls.env['res.partner.category'].create({'name': 'Customers'})
+        cls.env['ir.translation'].create({
             'type': 'model',
             'name': 'res.partner.category,name',
-            'module':'base',
+            'module': 'base',
             'lang': 'fr_FR',
-            'res_id': self.customers.id,
+            'res_id': cls.customers.id,
             'value': 'Clients',
             'state': 'translated',
         })
@@ -395,10 +397,10 @@ class TestTranslation(TransactionCase):
         self.assertEqual(translation_fr.src, 'Customers', "Did not set English version as source")
 
 class TestTranslationWrite(TransactionCase):
-
-    def setUp(self):
-        super().setUp()
-        self.category = self.env['res.partner.category'].create({'name': 'Reblochon'})
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.category = cls.env['res.partner.category'].create({'name': 'Reblochon'})
 
     def test_01_en(self):
         langs = self.env['res.lang'].get_installed()
@@ -435,7 +437,7 @@ class TestTranslationWrite(TransactionCase):
         self.assertEqual(source_name[0]['name'], "English Name", "Reference field not updated")
 
     def test_03_fr_single(self):
-        self.env['res.lang'].load_lang('fr_FR')
+        self.env['res.lang']._activate_lang('fr_FR')
         self.env['res.partner'].with_context(active_test=False).search([]).write({'lang': 'fr_FR'})
         self.env.ref('base.lang_en').active = False
 
@@ -453,7 +455,7 @@ class TestTranslationWrite(TransactionCase):
         self.assertEqual(len(translations), 0, "No French translation should be created when writing in French")
 
     def test_04_fr_multi(self):
-        self.env['res.lang'].load_lang('fr_FR')
+        self.env['res.lang']._activate_lang('fr_FR')
 
         langs = self.env['res.lang'].get_installed()
         self.assertEqual([('en_US', 'English (US)'), ('fr_FR', 'French / Français')], langs,
@@ -480,9 +482,9 @@ class TestTranslationWrite(TransactionCase):
         ])
 
     def test_04_fr_multi_no_en(self):
-        self.env['res.lang'].load_lang('fr_FR')
+        self.env['res.lang']._activate_lang('fr_FR')
+        self.env['res.lang']._activate_lang('es_ES')
         self.env['res.partner'].with_context(active_test=False).search([]).write({'lang': 'fr_FR'})
-        self.env['res.lang'].load_lang('es_ES')
         self.env.ref('base.lang_en').active = False
 
         langs = self.env['res.lang'].get_installed()
@@ -509,7 +511,7 @@ class TestTranslationWrite(TransactionCase):
         self._test_05_remove_multi(False)
 
     def _test_05_remove_multi(self, empty_value):
-        self.env['res.lang'].load_lang('fr_FR')
+        self.env['res.lang']._activate_lang('fr_FR')
 
         langs = self.env['res.lang'].get_installed()
         self.assertEqual([('en_US', 'English (US)'), ('fr_FR', 'French / Français')], langs,
@@ -569,6 +571,55 @@ class TestTranslationWrite(TransactionCase):
             "Did not fallback to source when reset"
         )
 
+    def test_orphan(self):
+        """ What happens with orphan translations. """
+        self.env['res.lang']._activate_lang('fr_FR')
+
+        # create a user with access rights on partner categories
+        user = new_test_user(self.env, 'deleter')
+        group = self.env.ref('base.group_partner_manager')
+        user.groups_id = [(4, group.id)]
+
+        # this access rule triggers a MissingError
+        self.env['ir.rule'].create({
+            'model_id': self.env['ir.model']._get_id('res.partner.category'),
+            'groups': [(4, group.id)],
+            'domain_force': "[('name', 'ilike', 'e')]",
+        })
+
+        # create a translation, and delete the record from the database
+        translation = self.env['ir.translation'].create({
+            'type': 'model',
+            'name': 'res.partner.category,name',
+            'lang': 'fr_FR',
+            'res_id': self.category.id,
+            'src': 'Reblochon',
+            'value': 'Parfum Exquis',
+            'state': 'translated',
+        })
+        translation.flush()
+        translation.invalidate_cache()
+        self.cr.execute("DELETE FROM res_partner_category WHERE id=%s", [self.category.id])
+
+        # deleting the translation should be possible, provided the user has
+        # access rights on the translation's model
+        user0 = new_test_user(self.env, 'cannot modify category')
+        with self.assertRaises(AccessError):
+            translation.with_user(user0).unlink()
+
+        translation.with_user(user).unlink()
+
+        # however, creating orphan translations should not be possible
+        with self.assertRaises(ValidationError):
+            translation.with_user(user).create({
+                'type': 'model',
+                'name': 'res.partner.category,name',
+                'lang': 'fr_FR',
+                'res_id': self.category.id,
+                'src': 'Reblochon',
+                'value': 'Parfum Exquis',
+                'state': 'translated',
+            })
 
     def test_field_selection(self):
         """ Test translations of field selections. """
@@ -587,11 +638,39 @@ class TestTranslationWrite(TransactionCase):
         self.assertEqual(fg['state']['selection'],
                          [('manual', 'Custo'), ('base', 'Pas touche!')])
 
+    def test_fields_view_get(self):
+        """ Test translations of field descriptions in fields_view_get(). """
+        self.env['res.lang']._activate_lang('fr_FR')
+
+        # add translation for the string of field ir.model.name
+        ir_model_field = self.env['ir.model.fields']._get('ir.model', 'name')
+        LABEL = "Description du Modèle"
+        self.env['ir.translation'].create({
+            'type': 'model',
+            'name': 'ir.model.fields,field_description',
+            'lang': 'fr_FR',
+            'res_id': ir_model_field.id,
+            'src': 'Name',
+            'value': LABEL,
+        })
+
+        # check that fields_get() returns the expected label
+        model = self.env['ir.model'].with_context(lang='fr_FR')
+        info = model.fields_get(['name'])
+        self.assertEqual(info['name']['string'], LABEL)
+
+        # check that fields_view_get() also returns the expected label
+        info = model.fields_view_get()['fields']
+        self.assertEqual(info['name']['string'], LABEL)
+
 
 class TestXMLTranslation(TransactionCase):
-    def setUp(self):
-        super(TestXMLTranslation, self).setUp()
-        self.env['ir.translation']._load_module_terms(['base'], ['fr_FR', 'nl_NL'])
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.env['res.lang']._activate_lang('fr_FR')
+        cls.env['res.lang']._activate_lang('nl_NL')
+        cls.env.ref('base.module_base')._update_translations(['fr_FR', 'nl_NL'])
 
     def create_view(self, archf, terms, **kwargs):
         view = self.env['ir.ui.view'].create({
