@@ -21,8 +21,16 @@ trigger an automatic reminder email.
         'data/base_automation_data.xml',
         'views/base_automation_view.xml',
     ],
-    'demo': [
-        'data/base_automation_demo.xml',
-    ],
+    'assets': {
+        'web.assets_qweb': [
+            'base_automation/static/src/xml/*.xml',
+        ],
+        'web.assets_backend': [
+            'base_automation/static/src/js/**/*',
+        ],
+        'web.qunit_suite_tests': [
+            'base_automation/static/tests/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
