@@ -22,13 +22,25 @@ Re-implement Odoo's file import system:
   need or want an online import can avoid it being available to users.
 """,
     'depends': ['web'],
-    'category': 'Tools',
+    'version': '2.0',
+    'category': 'Hidden/Tools',
     'installable': True,
     'auto_install': True,
     'data': [
         'security/ir.model.access.csv',
-        'views/base_import_templates.xml',
     ],
-    'qweb': ['static/src/xml/base_import.xml'],
+    'assets': {
+        'web.assets_qweb': [
+            'base_import/static/src/**/*.xml',
+        ],
+        'web.assets_backend': [
+            'base_import/static/lib/javascript-state-machine/state-machine.js',
+            'base_import/static/src/**/*.scss',
+            'base_import/static/src/**/*.js',
+        ],
+        'web.qunit_suite_tests': [
+            'base_import/static/tests/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
