@@ -15,27 +15,33 @@ class ResConfigSettings(models.TransientModel):
         "Default Access Rights",
         config_parameter='base_setup.default_user_rights')
     external_email_server_default = fields.Boolean(
-        "External Email Servers",
+        "Custom Email Servers",
         config_parameter='base_setup.default_external_email_server')
     module_base_import = fields.Boolean("Allow users to import data from CSV/XLS/XLSX/ODS files")
     module_google_calendar = fields.Boolean(
         string='Allow the users to synchronize their calendar  with Google Calendar')
+    module_microsoft_calendar = fields.Boolean(
+        string='Allow the users to synchronize their calendar with Outlook Calendar')
+    module_mail_plugin = fields.Boolean(
+        string='Allow integration with the mail plugins'
+    )
     module_google_drive = fields.Boolean("Attach Google documents to any record")
     module_google_spreadsheet = fields.Boolean("Google Spreadsheet")
     module_auth_oauth = fields.Boolean("Use external authentication providers (OAuth)")
     module_auth_ldap = fields.Boolean("LDAP Authentication")
+    # TODO: remove in master
     module_base_gengo = fields.Boolean("Translate Your Website with Gengo")
-    module_inter_company_rules = fields.Boolean("Manage Inter Company")
+    module_account_inter_company_rules = fields.Boolean("Manage Inter Company")
     module_pad = fields.Boolean("Collaborative Pads")
     module_voip = fields.Boolean("Asterisk (VoIP)")
     module_web_unsplash = fields.Boolean("Unsplash Image Library")
     module_partner_autocomplete = fields.Boolean("Partner Autocomplete")
     module_base_geolocalize = fields.Boolean("GeoLocalize")
-    report_footer = fields.Text(related="company_id.report_footer", string='Custom Report Footer', help="Footer text displayed at the bottom of all reports.", readonly=False)
+    module_google_recaptcha = fields.Boolean("reCAPTCHA")
+    report_footer = fields.Html(related="company_id.report_footer", string='Custom Report Footer', help="Footer text displayed at the bottom of all reports.", readonly=False)
     group_multi_currency = fields.Boolean(string='Multi-Currencies',
             implied_group='base.group_multi_currency',
             help="Allows to work in a multi currency environment")
-    paperformat_id = fields.Many2one(related="company_id.paperformat_id", string='Paper format', readonly=False)
     external_report_layout_id = fields.Many2one(related="company_id.external_report_layout_id", readonly=False)
     show_effect = fields.Boolean(string="Show Effect", config_parameter='base_setup.show_effect')
     company_count = fields.Integer('Number of Companies', compute="_compute_company_count")
@@ -43,6 +49,8 @@ class ResConfigSettings(models.TransientModel):
     language_count = fields.Integer('Number of Languages', compute="_compute_language_count")
     company_name = fields.Char(related="company_id.display_name", string="Company Name")
     company_informations = fields.Text(compute="_compute_company_informations")
+    profiling_enabled_until = fields.Datetime("Profiling enabled until", config_parameter='base.profiling_enabled_until')
+    module_product_images = fields.Boolean("Get product pictures using barcode")
 
     def open_company(self):
         return {
@@ -58,7 +66,7 @@ class ResConfigSettings(models.TransientModel):
         }
 
     def open_default_user(self):
-        action = self.env.ref('base.action_res_users').read()[0]
+        action = self.env["ir.actions.actions"]._for_xml_id("base.action_res_users")
         if self.env.ref('base.default_user', raise_if_not_found=False):
             action['res_id'] = self.env.ref('base.default_user').id
         else:
