@@ -2,64 +2,22 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import datetime
-import logging
 import string
 import re
+import stdnum
+from stdnum.eu.vat import check_vies
+from stdnum.exceptions import InvalidComponent
+from stdnum.util import clean
 
-_logger = logging.getLogger(__name__)
-try:
-    import vatnumber
-except ImportError:
-    _logger.warning("VAT validation partially unavailable because the `vatnumber` Python library cannot be found. "
-                    "Install it to support more countries, for example with `easy_install vatnumber`.")
-    vatnumber = None
+import logging
 
-# Although stdnum is a dependency of vatnumber, the import of the latter is surrounded by a try/except
-# if it is not installed. Therefore, we cannot be sure stdnum is installed in all cases.
-try:
-    import stdnum
-except ImportError:
-    stdnum = None
-
-from odoo import api, models, tools, _
+from odoo import api, models, fields, tools, _
 from odoo.tools.misc import ustr
 from odoo.exceptions import ValidationError
-from stdnum.at.uid import compact as compact_at
-from stdnum.be.vat import compact as compact_be
-from stdnum.bg.vat import compact as compact_bg
-from stdnum.ch.vat import compact as compact_ch, format as format_ch
-from stdnum.cy.vat import compact as compact_cy
-from stdnum.cz.dic import compact as compact_cz
-from stdnum.de.vat import compact as compact_de
-from stdnum.ee.kmkr import compact as compact_ee
-# el not in stdnum
-from stdnum.es.nif import compact as compact_es
-from stdnum.fi.alv import compact as compact_fi
-from stdnum.fr.tva import compact as compact_fr
-from stdnum.gb.vat import compact as compact_gb
-from stdnum.gr.vat import compact as compact_gr
-from stdnum.hu.anum import compact as compact_hu
-from stdnum.hr.oib import compact as compact_hr
-from stdnum.ie.vat import compact as compact_ie
-from stdnum.it.iva import compact as compact_it
-from stdnum.lt.pvm import compact as compact_lt
-from stdnum.lu.tva import compact as compact_lu
-from stdnum.lv.pvn import compact as compact_lv
-from stdnum.mt.vat import compact as compact_mt
-from stdnum.mx.rfc import compact as compact_mx
-from stdnum.nl.btw import compact as compact_nl
-from stdnum.no.mva import compact as compact_no
-# pe is not in stdnum
-from stdnum.pl.nip import compact as compact_pl
-from stdnum.pt.nif import compact as compact_pt
-from stdnum.ro.cf import compact as compact_ro
-from stdnum.se.vat import compact as compact_se
-from stdnum.si.ddv import compact as compact_si
-from stdnum.sk.dph import compact as compact_sk
-from stdnum.ar.cuit import compact as compact_ar
-# tr compact vat is not in stdnum
 
 
+_logger = logging.getLogger(__name__)
+
 _eu_country_vat = {
     'GR': 'EL'
 }
@@ -67,42 +25,53 @@ _eu_country_vat = {
 _eu_country_vat_inverse = {v: k for k, v in _eu_country_vat.items()}
 
 _ref_vat = {
+    'al': 'ALJ91402501L',
+    'ar': 'AR200-5536168-2 or 20055361682',
     'at': 'ATU12345675',
+    'au': '83 914 571 673',
     'be': 'BE0477472701',
     'bg': 'BG1234567892',
     'ch': 'CHE-123.456.788 TVA or CHE-123.456.788 MWST or CHE-123.456.788 IVA',  # Swiss by Yannick Vaucher @ Camptocamp
     'cl': 'CL76086428-5',
     'co': 'CO213123432-1 or CO213.123.432-1',
-    'cy': 'CY12345678F',
+    'cy': 'CY10259033P',
     'cz': 'CZ12345679',
     'de': 'DE123456788',
     'dk': 'DK12345674',
+    'do': 'DO1-01-85004-3 or 101850043',
+    'ec': 'EC1792060346-001',
     'ee': 'EE123456780',
     'el': 'EL12345670',
     'es': 'ESA12345674',
     'fi': 'FI12345671',
-    'fr': 'FR32123456789',
+    'fr': 'FR23334175221',
     'gb': 'GB123456782 or XI123456782',
     'gr': 'GR12345670',
     'hu': 'HU12345676',
     'hr': 'HR01234567896',  # Croatia, contributed by Milan Tribuson
     'ie': 'IE1234567FA',
+    'in': "12AAAAA1234AAZA",
+    'is': 'IS062199',
     'it': 'IT12345670017',
     'lt': 'LT123456715',
     'lu': 'LU12345613',
     'lv': 'LV41234567891',
+    'mc': 'FR53000004605',
     'mt': 'MT12345634',
-    'mx': 'ABC123456T1B',
+    'mx': 'MXGODE561231GR8 or GODE561231GR8',
     'nl': 'NL123456782B90',
     'no': 'NO123456785',
     'pe': '10XXXXXXXXY or 20XXXXXXXXY or 15XXXXXXXXY or 16XXXXXXXXY or 17XXXXXXXXY',
     'pl': 'PL1234567883',
     'pt': 'PT123456789',
     'ro': 'RO1234567897',
+    'rs': 'RS101134702',
+    'ru': 'RU123456789047',
     'se': 'SE123456789701',
     'si': 'SI12345679',
-    'sk': 'SK0012345675',
-    'tr': 'TR1234567890 (VERGINO) veya TR12345678901 (TCKIMLIKNO)',  # Levent Karakas @ Eska Yazilim A.S.
+    'sk': 'SK2022749619',
+    'sm': 'SM24165',
+    'tr': 'TR1234567890 (VERGINO) or TR17291716060 (TCKIMLIKNO)',  # Levent Karakas @ Eska Yazilim A.S.
     'xi': 'XI123456782',
 }
 
@@ -114,6 +83,8 @@ _region_specific_vat_codes = {
 class ResPartner(models.Model):
     _inherit = 'res.partner'
 
+    vat = fields.Char(string="VAT/Tax ID")
+
     def _split_vat(self, vat):
         vat_country, vat_number = vat[:2].lower(), vat[2:].replace(' ', '')
         return vat_country, vat_number
@@ -127,7 +98,7 @@ class ResPartner(models.Model):
         if not ustr(country_code).encode('utf-8').isalpha():
             return False
         check_func_name = 'check_vat_' + country_code
-        check_func = getattr(self, check_func_name, None) or getattr(vatnumber, check_func_name, None)
+        check_func = getattr(self, check_func_name, None) or getattr(stdnum.util.get_cc_module(country_code, 'vat'), 'is_valid', None)
         if not check_func:
             # No VAT validation available, default to check that the country code exists
             if country_code.upper() == 'EU':
@@ -143,20 +114,24 @@ class ResPartner(models.Model):
     def _check_vies(self, vat):
         # Store the VIES result in the cache. In case an exception is raised during the request
         # (e.g. service unavailable), the fallback on simple_vat_check is not kept in cache.
-        return vatnumber.check_vies(vat)
+        return check_vies(vat)
 
     @api.model
     def vies_vat_check(self, country_code, vat_number):
         try:
             # Validate against  VAT Information Exchange System (VIES)
             # see also http://ec.europa.eu/taxation_customs/vies/
-            return self._check_vies(country_code.upper() + vat_number)
+            vies_result = self._check_vies(country_code.upper() + vat_number)
+            return vies_result['valid']
+        except InvalidComponent:
+            return False
         except Exception:
             # see http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl
             # Fault code may contain INVALID_INPUT, SERVICE_UNAVAILABLE, MS_UNAVAILABLE,
             # TIMEOUT or SERVER_BUSY. There is no way we can validate the input
             # with VIES if any of these arise, including the first one (it means invalid
             # country code or empty VAT number), so we fall back to the simple check.
+            _logger.exception("Failed VIES VAT check.")
             return self.simple_vat_check(country_code, vat_number)
 
     @api.model
@@ -178,56 +153,90 @@ class ResPartner(models.Model):
         # This is for API pushes from external platforms where you have no control over VAT numbers.
         if self.env.context.get('no_vat_validation'):
             return
-        if self.env.context.get('company_id'):
-            company = self.env['res.company'].browse(self.env.context['company_id'])
-        else:
-            company = self.env.company
-        eu_countries = self.env.ref('base.europe').country_ids
+
         for partner in self:
-            if not partner.vat:
-                continue
-            is_eu_country = partner.commercial_partner_id.country_id in eu_countries
-            if company.vat_check_vies and is_eu_country and partner.is_company:
-                # force full VIES online check
-                check_func = self.vies_vat_check
-            else:
-                # quick and partial off-line checksum validation
-                check_func = self.simple_vat_check
-            #check with country code as prefix of the TIN
-            failed_check = False
-            vat_country_code, vat_number = self._split_vat(partner.vat)
-            vat_has_legit_country_code = self.env['res.country'].search([('code', '=', vat_country_code.upper())])
-            if not vat_has_legit_country_code:
-                vat_has_legit_country_code = vat_country_code.lower() in _region_specific_vat_codes
-            if vat_has_legit_country_code:
-                failed_check = not check_func(vat_country_code, vat_number)
-
-            #if fails, check with country code from country
-            partner_country_code = partner.commercial_partner_id.country_id.code
-            if (not vat_has_legit_country_code or failed_check) and partner_country_code:
-                failed_check = not check_func(partner_country_code.lower(), partner.vat)
-
-            # We allow any number if it doesn't start with a country code and the partner has no country.
-            # This is necessary to support an ORM limitation: setting vat and country_id together on a company
-            # triggers two distinct write on res.partner, one for each field, both triggering this constraint.
-            # If vat is set before country_id, the constraint must not break.
-
-            if failed_check:
-                country_code = partner_country_code or vat_country_code
-                msg = partner._construct_constraint_msg(country_code.lower() if country_code else None)
+            country = partner.commercial_partner_id.country_id
+            if partner.vat and self._run_vat_test(partner.vat, country, partner.is_company) is False:
+                partner_label = _("partner [%s]", partner.name)
+                msg = partner._build_vat_error_message(country and country.code.lower() or None, partner.vat, partner_label)
                 raise ValidationError(msg)
 
-    def _construct_constraint_msg(self, country_code):
-        self.ensure_one()
-        vat_no = "'CC##' (CC=Country Code, ##=VAT Number)"
-        vat_no = _ref_vat.get(country_code) or vat_no
+    @api.model
+    def _run_vat_test(self, vat_number, default_country, partner_is_company=True):
+        """ Checks a VAT number, either syntactically or using VIES, depending
+        on the active company's configuration.
+        A first check is made by using the first two characters of the VAT as
+        the country code. It it fails, a second one is made using default_country instead.
+
+        :param vat_number: a string with the VAT number to check.
+        :param default_country: a res.country object
+        :param partner_is_company: True if the partner is a company, else False
+
+        :return: The country code (in lower case) of the country the VAT number
+                 was validated for, if it was validated. False if it could not be validated
+                 against the provided or guessed country. None if no country was available
+                 for the check, and no conclusion could be made with certainty.
+        """
+        # Get company
         if self.env.context.get('company_id'):
             company = self.env['res.company'].browse(self.env.context['company_id'])
         else:
             company = self.env.company
+
+        # Get check function: either simple syntactic check or call to VIES service
+        eu_countries = self.env.ref('base.europe').country_ids
+        if company.vat_check_vies and default_country in eu_countries and partner_is_company:
+            check_func = self.vies_vat_check
+        else:
+            check_func = self.simple_vat_check
+
+        check_result = None
+
+        # First check with country code as prefix of the TIN
+        vat_country_code, vat_number_split = self._split_vat(vat_number)
+        vat_has_legit_country_code = self.env['res.country'].search([('code', '=', vat_country_code.upper())])
+        if not vat_has_legit_country_code:
+            vat_has_legit_country_code = vat_country_code.lower() in _region_specific_vat_codes
+        if vat_has_legit_country_code:
+            check_result = check_func(vat_country_code, vat_number_split)
+            if check_result:
+                return vat_country_code
+
+        # If it fails, check with default_country (if it exists)
+        if default_country:
+            check_result = check_func(default_country.code.lower(), vat_number)
+            if check_result:
+                return default_country.code.lower()
+
+        # We allow any number if it doesn't start with a country code and the partner has no country.
+        # This is necessary to support an ORM limitation: setting vat and country_id together on a company
+        # triggers two distinct write on res.partner, one for each field, both triggering this constraint.
+        # If vat is set before country_id, the constraint must not break.
+        return check_result
+
+    @api.model
+    def _build_vat_error_message(self, country_code, wrong_vat, record_label):
+        if self.env.context.get('company_id'):
+            company = self.env['res.company'].browse(self.env.context['company_id'])
+        else:
+            company = self.env.company
+
+        expected_format = _ref_vat.get(country_code, "'CC##' (CC=Country Code, ##=VAT Number)")
+
         if company.vat_check_vies:
-            return '\n' + _('The VAT number [%s] for partner [%s] either failed the VIES VAT validation check or did not respect the expected format %s.') % (self.vat, self.name, vat_no)
-        return '\n' + _('The VAT number [%s] for partner [%s] does not seem to be valid. \nNote: the expected format is %s') % (self.vat, self.name, vat_no)
+            return '\n' + _(
+                "The VAT number [%(wrong_vat)s] for %(record_label)s either failed the VIES VAT validation check or did not respect the expected format %(expected_format)s.",
+                wrong_vat=wrong_vat,
+                record_label=record_label,
+                expected_format=expected_format,
+            )
+
+        return '\n' + _(
+            'The VAT number [%(wrong_vat)s] for %(record_label)s does not seem to be valid. \nNote: the expected format is %(expected_format)s',
+            wrong_vat=wrong_vat,
+            record_label=record_label,
+            expected_format=expected_format,
+        )
 
     __check_vat_ch_re = re.compile(r'E([0-9]{9}|-[0-9]{3}\.[0-9]{3}\.[0-9]{3})(MWST|TVA|IVA)$')
 
@@ -259,6 +268,23 @@ class ResPartner(models.Model):
             return check == int(num[8])
         return False
 
+
+    def is_valid_ruc_ec(self, vat):
+        ci = stdnum.util.get_cc_module("ec", "ci")
+        ruc = stdnum.util.get_cc_module("ec", "ruc")
+        if len(vat) == 10:
+            return ci.is_valid(vat)
+        elif len(vat) == 13:
+            if vat[2] == "6" and ci.is_valid(vat[:10]):
+                return True
+            else:
+                return ruc.is_valid(vat)
+        return False
+
+    def check_vat_ec(self, vat):
+        vat = clean(vat, ' -.').upper().strip()
+        return self.is_valid_ruc_ec(vat)
+
     def _ie_check_char(self, vat):
         vat = vat.zfill(8)
         extra = 0
@@ -416,6 +442,43 @@ class ResPartner(models.Model):
             dig_check = 1
         return int(vat[10]) == dig_check
 
+    def check_vat_ru(self, vat):
+        '''
+        Check Russia VAT number.
+        Method copied from vatnumber 1.2 lib https://code.google.com/archive/p/vatnumber/
+        '''
+        if len(vat) != 10 and len(vat) != 12:
+            return False
+        try:
+            int(vat)
+        except ValueError:
+            return False
+
+        if len(vat) == 10:
+            check_sum = 2 * int(vat[0]) + 4 * int(vat[1]) + 10 * int(vat[2]) + \
+                3 * int(vat[3]) + 5 * int(vat[4]) + 9 * int(vat[5]) + \
+                4 * int(vat[6]) + 6 * int(vat[7]) + 8 * int(vat[8])
+            check = check_sum % 11
+            if check % 10 != int(vat[9]):
+                return False
+        else:
+            check_sum1 = 7 * int(vat[0]) + 2 * int(vat[1]) + 4 * int(vat[2]) + \
+                10 * int(vat[3]) + 3 * int(vat[4]) + 5 * int(vat[5]) + \
+                9 * int(vat[6]) + 4 * int(vat[7]) + 6 * int(vat[8]) + \
+                8 * int(vat[9])
+            check = check_sum1 % 11
+
+            if check != int(vat[10]):
+                return False
+            check_sum2 = 3 * int(vat[0]) + 7 * int(vat[1]) + 2 * int(vat[2]) + \
+                4 * int(vat[3]) + 10 * int(vat[4]) + 3 * int(vat[5]) + \
+                5 * int(vat[6]) + 9 * int(vat[7]) + 4 * int(vat[8]) + \
+                6 * int(vat[9]) + 8 * int(vat[10])
+            check = check_sum2 % 11
+            if check != int(vat[11]):
+                return False
+        return True
+
     # VAT validation in Turkey, contributed by # Levent Karakas @ Eska Yazilim A.S.
     def check_vat_tr(self, vat):
 
@@ -459,27 +522,6 @@ class ResPartner(models.Model):
 
         return False
 
-    def check_vat_al(self, vat):
-        try:
-            import stdnum.al
-            return stdnum.al.vat.is_valid(vat)
-        except ImportError:
-            return True
-
-    def check_vat_cl(self, vat):
-        return stdnum.util.get_cc_module('cl', 'vat').is_valid(vat) if stdnum else True
-
-    def check_vat_co(self, vat):
-        return stdnum.util.get_cc_module('co', 'vat').is_valid(vat) if stdnum else True
-
-    # Argentinian VAT validation, contributed by ADHOC
-    def check_vat_ar(self, vat):
-        try:
-            import stdnum.ar
-            return stdnum.ar.cuit.is_valid(vat)
-        except ImportError:
-            return True
-
     def check_vat_ua(self, vat):
         res = []
         for partner in self:
@@ -505,32 +547,65 @@ class ResPartner(models.Model):
         As of January 1st 2021, companies in Northern Ireland have a
         new VAT number starting with XI
         TODO: remove when stdnum is updated to 1.16 in supported distro"""
-        return stdnum.util.get_cc_module('gb', 'vat').is_valid(vat) if stdnum else True
+        check_func = getattr(stdnum.util.get_cc_module('gb', 'vat'), 'is_valid', None)
+        if not check_func:
+            return len(vat) == 9
+        return check_func(vat)
 
-    def default_compact(self, vat):
-        return vat
+    def check_vat_in(self, vat):
+        #reference from https://www.gstzen.in/a/format-of-a-gst-number-gstin.html
+        if vat and len(vat) == 15:
+            all_gstin_re = [
+                r'[0-9]{2}[a-zA-Z]{5}[0-9]{4}[a-zA-Z]{1}[1-9A-Za-z]{1}[Zz1-9A-Ja-j]{1}[0-9a-zA-Z]{1}', # Normal, Composite, Casual GSTIN
+                r'[0-9]{4}[A-Z]{3}[0-9]{5}[UO]{1}[N][A-Z0-9]{1}', #UN/ON Body GSTIN
+                r'[0-9]{4}[a-zA-Z]{3}[0-9]{5}[N][R][0-9a-zA-Z]{1}', #NRI GSTIN
+                r'[0-9]{2}[a-zA-Z]{4}[a-zA-Z0-9]{1}[0-9]{4}[a-zA-Z]{1}[1-9A-Za-z]{1}[DK]{1}[0-9a-zA-Z]{1}', #TDS GSTIN
+                r'[0-9]{2}[a-zA-Z]{5}[0-9]{4}[a-zA-Z]{1}[1-9A-Za-z]{1}[C]{1}[0-9a-zA-Z]{1}' #TCS GSTIN
+            ]
+            return any(re.compile(rx).match(vat) for rx in all_gstin_re)
+        return False
+
+    def check_vat_au(self, vat):
+        '''
+        The Australian equivalent of a VAT number is an ABN number.
+        TFN (Australia Tax file numbers) are private and not to be
+        entered into systems or publicly displayed, so ABN numbers
+        are the public facing number that legally must be displayed
+        on all invoices
+        '''
+        check_func = getattr(stdnum.util.get_cc_module('au', 'abn'), 'is_valid', None)
+        if not check_func:
+            vat = vat.replace(" ", "")
+            return len(vat) == 11 and vat.isdigit()
+        return check_func(vat)
+
+    def format_vat_ch(self, vat):
+        stdnum_vat_format = getattr(stdnum.util.get_cc_module('ch', 'vat'), 'format', None)
+        return stdnum_vat_format('CH' + vat)[2:] if stdnum_vat_format else vat
 
     def _fix_vat_number(self, vat, country_id):
         code = self.env['res.country'].browse(country_id).code if country_id else False
         vat_country, vat_number = self._split_vat(vat)
         if code and code.lower() != vat_country:
             return vat
-        check_func_name = 'compact_' + vat_country
-        check_func = globals().get(check_func_name) or getattr(self, 'default_compact')
-        vat_number = check_func(vat_number)
-        format_func = globals().get('format_' + vat_country)
-        return format_func(vat_country.upper() + vat_number) if format_func else vat_country.upper() + vat_number
+        stdnum_vat_fix_func = getattr(stdnum.util.get_cc_module(vat_country, 'vat'), 'compact', None)
+        #If any localization module need to define vat fix method for it's country then we give first priority to it.
+        format_func_name = 'format_vat_' + vat_country
+        format_func = getattr(self, format_func_name, None) or stdnum_vat_fix_func
+        if format_func:
+            vat_number = format_func(vat_number)
+        return vat_country.upper() + vat_number
 
-    @api.model
-    def create(self, values):
-        if values.get('vat'):
-            country_id = values.get('country_id')
-            values['vat'] = self._fix_vat_number(values['vat'], country_id)
-        return super(ResPartner, self).create(values)
+    @api.model_create_multi
+    def create(self, vals_list):
+        for values in vals_list:
+            if values.get('vat'):
+                country_id = values.get('country_id')
+                values['vat'] = self._fix_vat_number(values['vat'], country_id)
+        return super(ResPartner, self).create(vals_list)
 
     def write(self, values):
         if values.get('vat') and len(self.mapped('country_id')) == 1:
             country_id = values.get('country_id', self.country_id.id)
             values['vat'] = self._fix_vat_number(values['vat'], country_id)
         return super(ResPartner, self).write(values)
-
