@@ -8,6 +8,7 @@ import threading
 import time
 
 import odoo
+import odoo.service.server as servermod
 from odoo import api, fields, models, SUPERUSER_ID
 from odoo.tools.misc import DEFAULT_SERVER_DATETIME_FORMAT
 from odoo.tools import date_utils
@@ -29,6 +30,14 @@ def hashable(key):
     return key
 
 
+def channel_with_db(dbname, channel):
+    if isinstance(channel, models.Model):
+        return (dbname, channel._name, channel.id)
+    if isinstance(channel, str):
+        return (dbname, channel)
+    return channel
+
+
 class ImBus(models.Model):
 
     _name = 'bus.bus'
@@ -37,39 +46,44 @@ class ImBus(models.Model):
     channel = fields.Char('Channel')
     message = fields.Char('Message')
 
-    @api.model
-    def gc(self):
+    @api.autovacuum
+    def _gc_messages(self):
         timeout_ago = datetime.datetime.utcnow()-datetime.timedelta(seconds=TIMEOUT*2)
         domain = [('create_date', '<', timeout_ago.strftime(DEFAULT_SERVER_DATETIME_FORMAT))]
         return self.sudo().search(domain).unlink()
 
     @api.model
-    def sendmany(self, notifications):
+    def _sendmany(self, notifications):
         channels = set()
-        for channel, message in notifications:
+        values = []
+        for target, notification_type, message in notifications:
+            channel = channel_with_db(self.env.cr.dbname, target)
             channels.add(channel)
-            values = {
-                "channel": json_dump(channel),
-                "message": json_dump(message)
-            }
-            self.sudo().create(values)
+            values.append({
+                'channel': json_dump(channel),
+                'message': json_dump({
+                    'type': notification_type,
+                    'payload': message,
+                })
+            })
+        self.sudo().create(values)
         if channels:
             # We have to wait until the notifications are commited in database.
             # When calling `NOTIFY imbus`, some concurrent threads will be
             # awakened and will fetch the notification in the bus table. If the
             # transaction is not commited yet, there will be nothing to fetch,
             # and the longpolling will return no notification.
+            @self.env.cr.postcommit.add
             def notify():
                 with odoo.sql_db.db_connect('postgres').cursor() as cr:
                     cr.execute("notify imbus, %s", (json_dump(list(channels)),))
-            self._cr.after('commit', notify)
 
     @api.model
-    def sendone(self, channel, message):
-        self.sendmany([[channel, message]])
+    def _sendone(self, channel, notification_type, message):
+        self._sendmany([[channel, notification_type, message]])
 
     @api.model
-    def poll(self, channels, last=0, options=None):
+    def _poll(self, channels, last=0, options=None):
         if options is None:
             options = {}
         # first poll return the notification in the 'buffer'
@@ -78,7 +92,7 @@ class ImBus(models.Model):
             domain = [('create_date', '>', timeout_ago.strftime(DEFAULT_SERVER_DATETIME_FORMAT))]
         else:  # else returns the unread notifications
             domain = [('id', '>', last)]
-        channels = [json_dump(c) for c in channels]
+        channels = [json_dump(channel_with_db(self.env.cr.dbname, c)) for c in channels]
         domain.append(('channel', 'in', channels))
         notifications = self.sudo().search_read(domain)
         # list of notification to return
@@ -86,7 +100,6 @@ class ImBus(models.Model):
         for notif in notifications:
             result.append({
                 'id': notif['id'],
-                'channel': json.loads(notif['channel']),
                 'message': json.loads(notif['message']),
             })
         return result
@@ -100,7 +113,10 @@ class ImDispatch(object):
         self.channels = {}
         self.started = False
 
-    def poll(self, dbname, channels, last, options=None, timeout=TIMEOUT):
+    def poll(self, dbname, channels, last, options=None, timeout=None):
+        channels = [channel_with_db(dbname, channel) for channel in channels]
+        if timeout is None:
+            timeout = TIMEOUT
         if options is None:
             options = {}
         # Dont hang ctrl-c for a poll request, we need to bypass private
@@ -117,7 +133,7 @@ class ImDispatch(object):
         # immediatly returns if past notifications exist
         with registry.cursor() as cr:
             env = api.Environment(cr, SUPERUSER_ID, {})
-            notifications = env['bus.bus'].poll(channels, last, options)
+            notifications = env['bus.bus']._poll(channels, last, options)
 
         # immediatly returns in peek mode
         if options.get('peek'):
@@ -136,7 +152,7 @@ class ImDispatch(object):
                 event.wait(timeout=timeout)
                 with registry.cursor() as cr:
                     env = api.Environment(cr, SUPERUSER_ID, {})
-                    notifications = env['bus.bus'].poll(channels, last, options)
+                    notifications = env['bus.bus']._poll(channels, last, options)
             except Exception:
                 # timeout
                 pass
@@ -170,6 +186,15 @@ class ImDispatch(object):
                     for event in events:
                         event.set()
 
+    def wakeup_workers(self):
+        """
+        Wake up all http workers that are waiting for an event, useful
+        on server shutdown when they can't reveive anymore messages.
+        """
+        for events in self.channels.values():
+            for event in events:
+                event.set()
+
     def run(self):
         while True:
             try:
@@ -197,3 +222,5 @@ dispatch = None
 if not odoo.multi_process or odoo.evented:
     # We only use the event dispatcher in threaded and gevent mode
     dispatch = ImDispatch()
+    if servermod.server:
+        servermod.server.on_stop(dispatch.wakeup_workers)
