@@ -5,12 +5,12 @@ import datetime
 from datetime import datetime, timedelta, time
 
 from odoo import fields
-from odoo.tests.common import TransactionCase
+from odoo.addons.base.tests.common import SavepointCaseWithUserDemo
 import pytz
 import re
 
 
-class TestCalendar(TransactionCase):
+class TestCalendar(SavepointCaseWithUserDemo):
 
     def setUp(self):
         super(TestCalendar, self).setUp()
@@ -27,143 +27,6 @@ class TestCalendar(TransactionCase):
             'name': 'Technical Presentation'
         })
 
-    def test_calender_simple_event(self):
-        m = self.CalendarEvent.create({
-            'name': "Test compute",
-            'start': '2017-07-12 14:30:00',
-            'allday': False,
-            'stop': '2017-07-12 15:00:00',
-        })
-
-        self.assertEqual(
-            (str(m.start_datetime), str(m.stop_datetime)),
-            (u'2017-07-12 14:30:00', u'2017-07-12 15:00:00'),
-            "Sanity check"
-        )
-
-    def test_calender_event(self):
-        # Now I will set recurrence for this event to occur monday and friday of week
-        data = {
-            'fr': 1,
-            'mo': 1,
-            'interval': 1,
-            'rrule_type': 'weekly',
-            'end_type': 'end_date',
-            'final_date': '2011-05-31 00:00:00',
-            'recurrency': True
-        }
-
-        self.event_tech_presentation.write(data)
-
-        # In order to check that recurrent events are views successfully in calendar view, I will open calendar view of events|
-        self.CalendarEvent.fields_view_get(False, 'calendar')
-
-        # In order to check that recurrent events are views successfully in calendar view, I will search for one of the recurrent event and count the number of events
-        rec_events = self.CalendarEvent.with_context({'virtual_id': True}).search([
-            ('start', '>=', '2011-04-30 16:00:00'), ('start', '<=', '2011-05-31 00:00:00')
-        ])
-        self.assertEqual(len(rec_events), 9, 'Wrong number of events found')
-
-        # Now I move a virtual event, to see that a real event is well created and depending from the native recurrence
-        before = self.CalendarEvent.with_context({'virtual_id': False}).search([
-            ('start', '>=', '2011-04-30 16:00:00'), ('start', '<=', '2011-05-31 00:00:00')
-        ])
-
-        # We start by detach the event
-        newevent = rec_events[1].detach_recurring_event()
-        newevent.with_context({'virtual_id': True}).write({'name': 'New Name', 'recurrency': True})
-        after = self.CalendarEvent.with_context({'virtual_id': False}).search([
-            ('start', '>=', '2011-04-30 16:00:00'), ('start', '<=', '2011-05-31 00:00:00')
-        ])
-        self.assertEqual(len(after), len(before) + 1, 'Wrong number of events found, after to have moved a virtual event')
-        new_event = after - before
-        self.assertEqual(new_event[0].recurrent_id, before.id, 'Recurrent_id not correctly passed to the new event')
-
-        # Now I will test All day event
-        allday_event = self.CalendarEvent.create({
-            'allday': 1,
-            'privacy': 'confidential',
-            'start': '2011-04-30 00:00:00',
-            'stop': '2011-04-30 00:00:00',
-            'description': 'All day technical test',
-            'location': 'School',
-            'name': 'All day test event'
-        })
-
-        # In order to check reminder I will first create reminder
-        res_alarm_day_before_event_starts = self.env['calendar.alarm'].create({
-            'name': '1 Day before event starts',
-            'duration': 1,
-            'interval': 'days',
-            'alarm_type': 'notification'
-        })
-
-        # Now I will assign this reminder to all day event|
-        allday_event.write({'alarm_ids': [(6, 0, [res_alarm_day_before_event_starts.id])]})
-
-        # I create a recuring rule for my event
-        calendar_event_sprint_review = self.CalendarEvent.create({
-            'name': 'Begin of month meeting',
-            'start': datetime.combine(fields.Date.today(), time(12, 0)),
-            'stop': datetime.combine(fields.Date.today(), time(18, 0)),
-            'recurrency': True,
-            'rrule': 'FREQ=MONTHLY;INTERVAL=1;COUNT=12;BYDAY=1MO'
-        })
-
-        # I check that the attributes are set correctly
-        self.assertEqual(calendar_event_sprint_review.rrule_type, 'monthly', 'rrule_type should be mothly')
-        self.assertEqual(calendar_event_sprint_review.count, 12, 'rrule_type should be mothly')
-        self.assertEqual(calendar_event_sprint_review.month_by, 'day', 'rrule_type should be mothly')
-        self.assertEqual(calendar_event_sprint_review.byday, '1', 'rrule_type should be mothly')
-        self.assertEqual(calendar_event_sprint_review.week_list, 'MO', 'rrule_type should be mothly')
-
-    def test_validation_error(self):
-        """
-        Ideally this should build the base event in such a way that calling
-        write() triggers detach_recurring_event, but I've no idea how that
-        actually works so just calling it directly for now
-        """
-        m = self.CalendarEvent.create({
-            'name': "wheee",
-            'start': '2017-07-12 14:30:00',
-            'allday': False,
-            'rrule': u'FREQ=WEEKLY;BYDAY=WE;INTERVAL=1;COUNT=100',
-            'duration': 0.5,
-            'stop': '2017-07-12 15:00:00',
-        })
-        self.assertEqual(
-            (str(m.start_datetime), str(m.stop_datetime)),
-            (u'2017-07-12 14:30:00', u'2017-07-12 15:00:00'),
-            "Sanity check"
-        )
-        values = {
-            'allday': False,
-            'name': u'wheee',
-            'attendee_ids': [
-                (0, 0, {'state': u'needsAction', 'partner_id': 8, 'email': u'bob@example.com'}),
-                (0, 0, {'state': u'needsAction', 'partner_id': 10, 'email': u'ed@example.com'}),
-            ],
-            'recurrency': True,
-            'privacy': u'public',
-            'stop': '2017-07-10 16:00:00',
-            'alarm_ids': [(6, 0, [])],
-            'start': '2017-07-10 15:30:00',
-            'location': u"XXX",
-            'duration': 0.5,
-            'partner_ids': [(4, 10), (4, 8)],
-            'description': u"A thing"
-        }
-
-        records = m.detach_recurring_event(values)
-        self.assertEqual(
-            (str(m.start_datetime), str(m.stop_datetime)),
-            ('2017-07-12 14:30:00', u'2017-07-12 15:00:00'),
-        )
-        self.assertEquals(
-            (str(records.start_datetime), str(records.stop_datetime)),
-            (u'2017-07-10 15:30:00', u'2017-07-10 16:00:00'),
-        )
-
     def test_event_order(self):
         """ check the ordering of events when searching """
         def create_event(name, date):
@@ -171,7 +34,6 @@ class TestCalendar(TransactionCase):
                 'name': name,
                 'start': date + ' 12:00:00',
                 'stop': date + ' 14:00:00',
-                'duration': 2.0,
             })
         foo1 = create_event('foo', '2011-04-01')
         foo2 = create_event('foo', '2011-06-01')
@@ -225,7 +87,7 @@ class TestCalendar(TransactionCase):
             'name': 'Test',
         })
         now = datetime.now()
-        test_user = self.env.ref('base.user_demo')
+        test_user = self.user_demo
         test_name, test_description, test_description2 = 'Test-Meeting', 'Test-Description', 'NotTest'
         test_note, test_note2 = '<p>Test-Description</p>', '<p>NotTest</p>'
 
@@ -261,7 +123,7 @@ class TestCalendar(TransactionCase):
         self.assertEqual(test_record.activity_ids.date_deadline, (now + timedelta(days=-2)).date())
 
         # update event with a description that have a special character and a new line
-        test_description3 = 'Test & \n Description'
+        test_description3 = 'Test & <br> Description'
         test_note3 = '<p>Test &amp; <br> Description</p>'
         test_event.write({
             'description': test_description3,
@@ -274,7 +136,7 @@ class TestCalendar(TransactionCase):
         self.assertEqual(self.env['calendar.event'], self.env['calendar.event'].search([('name', '=', test_name)]))
 
         # create using active_model keys
-        test_event = self.env['calendar.event'].with_user(self.env.ref('base.user_demo')).with_context(
+        test_event = self.env['calendar.event'].with_user(self.user_demo).with_context(
             active_model=test_record._name,
             active_id=test_record.id,
         ).create({
@@ -295,13 +157,11 @@ class TestCalendar(TransactionCase):
             'name': 'All Day',
             'start': "2018-10-16 00:00:00",
             'start_date': "2018-10-16",
-            'start_datetime': False,
             'stop': "2018-10-18 00:00:00",
             'stop_date': "2018-10-18",
-            'stop_datetime': False,
             'allday': True,
         })
-
+        event.invalidate_cache()
         self.assertEqual(str(event.start), '2018-10-16 08:00:00')
         self.assertEqual(str(event.stop), '2018-10-18 18:00:00')
 
@@ -311,15 +171,15 @@ class TestCalendar(TransactionCase):
             'start': '2018-10-27 14:30:00',
             'allday': False,
             'rrule': u'FREQ=DAILY;INTERVAL=1;COUNT=4',
-            'duration': 2,
+            'recurrency': True,
             'stop': '2018-10-27 16:30:00',
+            'event_tz': 'Europe/Brussels',
         })
 
-        start_recurring_dates = m.with_context({'tz': 'Europe/Brussels'})._get_recurrent_date_by_event()
+        start_recurring_dates = m.recurrence_id.calendar_event_ids.sorted('start').mapped('start')
         self.assertEqual(len(start_recurring_dates), 4)
 
         for d in start_recurring_dates:
-            self.assertEqual(d.tzinfo, pytz.UTC)
             if d.day < 28:  # DST switch happens between 2018-10-27 and 2018-10-28
                 self.assertEqual(d.hour, 14)
             else:
@@ -335,8 +195,8 @@ class TestCalendar(TransactionCase):
         activity_id = self.env['mail.activity'].create({
             'summary': 'Meeting with partner',
             'activity_type_id': activty_type.id,
-            'res_model_id': self.env['ir.model'].search([('model', '=', 'res.partner')], limit=1).id,
-            'res_id': self.env['res.partner'].search([('name', 'ilike', 'Deco Addict')], limit=1).id,
+            'res_model_id': self.env['ir.model']._get_id('res.partner'),
+            'res_id': self.env['res.partner'].create({'name': 'A Partner'}).id,
         })
 
         calendar_event = self.env['calendar.event'].create({
@@ -368,18 +228,16 @@ class TestCalendar(TransactionCase):
         activity_id = self.env['mail.activity'].create({
             'summary': 'Meeting with partner',
             'activity_type_id': activty_type.id,
-            'res_model_id': self.env['ir.model'].search([('model', '=', 'res.partner')], limit=1).id,
-            'res_id': self.env['res.partner'].search([('name', 'ilike', 'Deco Addict')], limit=1).id,
+            'res_model_id': self.env['ir.model']._get_id('res.partner'),
+            'res_id': self.env['res.partner'].create({'name': 'A Partner'}).id,
         })
 
         calendar_event = self.env['calendar.event'].create({
             'name': 'All Day',
             'start': "2018-10-16 00:00:00",
             'start_date': "2018-10-16",
-            'start_datetime': False,
             'stop': "2018-10-18 00:00:00",
             'stop_date': "2018-10-18",
-            'stop_datetime': False,
             'allday': True,
             'activity_ids': [(6, False, activity_id.ids)],
         })
@@ -400,83 +258,104 @@ class TestCalendar(TransactionCase):
         """
         Check that mail are sent to the attendees on event creation
         Check that mail are sent to the added attendees on event edit
-        Check that mail are NOT sent to the attendees when detaching a recurring event
+        Check that mail are NOT sent to the attendees when the event date is past
         """
 
-        def _test_one_mail_per_attendee(self, m, partners):
+        def _test_one_mail_per_attendee(self, partners):
             # check that every attendee receive a (single) mail for the event
             for partner in partners:
-                mail = self.env['mail.mail'].search([
-                    ('recipient_ids', 'in', partner.id),
-                    ('subject', 'like', m.name),
+                mail = self.env['mail.message'].sudo().search([
+                    ('notified_partner_ids', 'in', partner.id),
                     ])
                 self.assertEqual(len(mail), 1)
 
         partners = [
-           self.env['res.partner'].create({'name':'testuser0','email': u'bob@example.com'}),
-           self.env['res.partner'].create({'name':'testuser1','email': u'alice@example.com'}),
-           ]
+            self.env['res.partner'].create({'name': 'testuser0', 'email': u'bob@example.com'}),
+            self.env['res.partner'].create({'name': 'testuser1', 'email': u'alice@example.com'}),
+        ]
         partner_ids = [(6, False, [p.id for p in partners]),]
-        now = fields.Datetime.now()
+        now = fields.Datetime.context_timestamp(partners[0], fields.Datetime.now())
         m = self.CalendarEvent.create({
             'name': "mailTest1",
             'allday': False,
             'rrule': u'FREQ=DAILY;INTERVAL=1;COUNT=5',
-            'duration': 0.5,
+            'recurrency': True,
             'partner_ids': partner_ids,
             'start': fields.Datetime.to_string(now + timedelta(days=10)),
             'stop': fields.Datetime.to_string(now + timedelta(days=15)),
             })
 
         # every partner should have 1 mail sent
-        _test_one_mail_per_attendee(self, m, partners)
+        _test_one_mail_per_attendee(self, partners)
 
         # adding more partners to the event
         partners.extend([
-            self.env['res.partner'].create({'name':'testuser2','email': u'marc@example.com'}),
-            self.env['res.partner'].create({'name':'testuser3','email': u'carl@example.com'}),
-            self.env['res.partner'].create({'name':'testuser4','email': u'alain@example.com'}),
+            self.env['res.partner'].create({'name': 'testuser2', 'email': u'marc@example.com'}),
+            self.env['res.partner'].create({'name': 'testuser3', 'email': u'carl@example.com'}),
+            self.env['res.partner'].create({'name': 'testuser4', 'email': u'alain@example.com'}),
             ])
         partner_ids = [(6, False, [p.id for p in partners]),]
-        m.write({'partner_ids': partner_ids})
+        m.write({
+            'partner_ids': partner_ids,
+            'recurrence_update': 'all_events',
+        })
 
         # more email should be sent
-        _test_one_mail_per_attendee(self, m, partners)
+        _test_one_mail_per_attendee(self, partners)
 
-        # calculate virtualid to detach one event
-        virtid = str(m.id) + '-' + ''.join(re.split('[\D]', fields.Datetime.to_string(now + timedelta(days=12))))
-
-        # detaching a virtual event in the chain
-        self.env['calendar.event'].browse(virtid).detach_recurring_event(values={'active':False})
-
-        # since the detach actually create an event in the backend
-        # we check that no mail notifications are sent to the attendees
-        _test_one_mail_per_attendee(self, m, partners)
-
-    def test_event_creation_mail_with_different_owner(self):
-        admin = self.env.ref('base.user_admin')
-        demo = self.env.ref('base.user_demo')
-        now = fields.Datetime.now()
-        partners = [
-           self.env['res.partner'].create({'name': 'testuser0', 'email': u'bob@example.com'}),
-           self.env['res.partner'].create({'name': 'testuser1', 'email': u'alice@example.com'}),
-           ]
-        partner_ids = [(6, False, [p.id for p in partners]), ]
-
-        m = self.CalendarEvent.with_user(admin).create({
-            'name': "mailTest1",
-            'partner_ids': partner_ids,
+        # create a new event in the past
+        self.CalendarEvent.create({
+            'name': "NOmailTest",
             'allday': False,
-            'duration': 0.5,
-            'user_id': demo.id,
-            'start': fields.Datetime.to_string(now + timedelta(days=10)),
-            'stop': fields.Datetime.to_string(now + timedelta(days=15)),
-            })
+            'recurrency': False,
+            'partner_ids': partner_ids,
+            'start': fields.Datetime.to_string(now - timedelta(days=10)),
+            'stop': fields.Datetime.to_string(now - timedelta(days=9)),
+        })
 
-        for partner in partners:
-            mail = self.env['mail.mail'].search([
-                ('recipient_ids', 'in', partner.id),
-                ('subject', 'like', m.name),
-                ])
-            self.assertEqual(len(mail), 1)
-            self.assertEqual(mail.author_id.id, demo.partner_id.id)
+        # no more email should be sent
+        _test_one_mail_per_attendee(self, partners)
+
+    def test_event_creation_sudo_other_company(self):
+        """ Check Access right issue when create event with sudo
+
+            Create a company, a user in that company
+            Create an event for someone else in another company as sudo
+            Should not failed for acces right check
+        """
+        now = fields.Datetime.context_timestamp(self.partner_demo, fields.Datetime.now())
+
+        web_company = self.env['res.company'].sudo().create({'name': "Website Company"})
+        web_user = self.env['res.users'].with_company(web_company).sudo().create({
+            'name': 'web user',
+            'login': 'web',
+            'company_id': web_company.id
+        })
+        self.CalendarEvent.with_user(web_user).with_company(web_company).sudo().create({
+            'name': "Test",
+            'allday': False,
+            'recurrency': False,
+            'partner_ids': [(6, 0, self.partner_demo.ids)],
+            'alarm_ids': [(0, 0, {
+                'name': 'Alarm',
+                'alarm_type': 'notification',
+                'interval': 'minutes',
+                'duration': 30,
+            })],
+            'user_id': self.user_demo.id,
+            'start': fields.Datetime.to_string(now + timedelta(hours=5)),
+            'stop': fields.Datetime.to_string(now + timedelta(hours=6)),
+        })
+
+    def test_meeting_creation_from_partner_form(self):
+        """ When going from a partner to the Calendar and adding a meeting, both current user and partner
+         should be attendees of the event """
+        calendar_action = self.partner_demo.schedule_meeting()
+        event = self.env['calendar.event'].with_context(calendar_action['context']).create({
+            'name': 'Super Meeting',
+            'start': datetime(2020, 12, 13, 17),
+            'stop': datetime(2020, 12, 13, 22),
+        })
+        self.assertEqual(len(event.attendee_ids), 2)
+        self.assertTrue(self.partner_demo in event.attendee_ids.mapped('partner_id'))
+        self.assertTrue(self.env.user.partner_id in event.attendee_ids.mapped('partner_id'))
