@@ -4,7 +4,8 @@
 
 {
     'name': 'Contacts',
-    'category': 'Tools',
+    'category': 'Sales/CRM',
+    'sequence': 150,
     'summary': 'Centralize your address book',
     'description': """
 This module gives you a quick view of your contacts directory, accessible from your home page.
