@@ -2,9 +2,9 @@
 
 {
     'name': 'Events Sales',
-    'version': '1.1',
+    'version': '1.2',
     'category': 'Marketing/Events',
-    'website': 'https://www.odoo.com/page/events',
+    'website': 'https://www.odoo.com/app/events',
     'description': """
 Creating registration with sales orders.
 ========================================
@@ -20,11 +20,12 @@ this event.
 """,
     'depends': ['event', 'sale_management'],
     'data': [
-        'views/assets.xml',
+        'views/event_ticket_views.xml',
+        'views/event_registration_views.xml',
         'views/event_views.xml',
-        'views/product_views.xml',
         'views/sale_order_views.xml',
         'data/event_sale_data.xml',
+        'data/mail_data.xml',
         'report/event_event_templates.xml',
         'security/ir.model.access.csv',
         'security/event_security.xml',
@@ -34,5 +35,16 @@ this event.
     'demo': ['data/event_demo.xml'],
     'installable': True,
     'auto_install': True,
+    'assets': {
+        'web.assets_backend': [
+            'event_sale/static/src/**/*',
+        ],
+        'web.assets_tests': [
+            'event_sale/static/tests/tours/**/*',
+        ],
+        'web.qunit_suite_tests': [
+            'event_sale/static/tests/event_configurator.test.js',
+        ],
+    },
     'license': 'LGPL-3',
 }
