@@ -24,7 +24,6 @@ class BadgeUser(models.Model):
     comment = fields.Text('Comment')
     badge_name = fields.Char(related='badge_id.name', string="Badge Name", readonly=False)
     level = fields.Selection(
-        [('bronze', 'Bronze'), ('silver', 'Silver'), ('gold', 'Gold')],
         string='Badge Level', related="badge_id.level", store=True, readonly=True)
 
     def _send_badge(self):
@@ -53,10 +52,11 @@ class BadgeUser(models.Model):
 
         return True
 
-    @api.model
-    def create(self, vals):
-        self.env['gamification.badge'].browse(vals['badge_id']).check_granting()
-        return super(BadgeUser, self).create(vals)
+    @api.model_create_multi
+    def create(self, vals_list):
+        for vals in vals_list:
+            self.env['gamification.badge'].browse(vals['badge_id']).check_granting()
+        return super().create(vals_list)
 
 
 class GamificationBadge(models.Model):
@@ -74,7 +74,7 @@ class GamificationBadge(models.Model):
 
     name = fields.Char('Badge', required=True, translate=True)
     active = fields.Boolean('Active', default=True)
-    description = fields.Text('Description', translate=True)
+    description = fields.Html('Description', translate=True)
     level = fields.Selection([
         ('bronze', 'Bronze'), ('silver', 'Silver'), ('gold', 'Gold')],
         string='Forum Badge Level', default='bronze')
@@ -107,8 +107,8 @@ class GamificationBadge(models.Model):
         'gamification.badge.user', 'badge_id',
         string='Owners', help='The list of instances of this badge granted to users')
 
-    stat_count = fields.Integer("Total", compute='_get_owners_info', help="The number of time this badge has been received.")
-    stat_count_distinct = fields.Integer("Number of users", compute='_get_owners_info', help="The number of time this badge has been received by unique users.")
+    granted_count = fields.Integer("Total", compute='_get_owners_info', help="The number of time this badge has been received.")
+    granted_users_count = fields.Integer("Number of users", compute='_get_owners_info', help="The number of time this badge has been received by unique users.")
     unique_owner_ids = fields.Many2many(
         'res.users', string="Unique Owners", compute='_get_owners_info',
         help="The list of unique users having received this badge.")
@@ -138,24 +138,39 @@ class GamificationBadge(models.Model):
             the total number of time this badge was granted
             the total number of users this badge was granted to
         """
-        self.env.cr.execute("""
-            SELECT badge_id, count(user_id) as stat_count,
-                count(distinct(user_id)) as stat_count_distinct,
-                array_agg(distinct(user_id)) as unique_owner_ids
-            FROM gamification_badge_user
-            WHERE badge_id in %s
-            GROUP BY badge_id
-            """, [tuple(self.ids)])
-
         defaults = {
-            'stat_count': 0,
-            'stat_count_distinct': 0,
+            'granted_count': 0,
+            'granted_users_count': 0,
             'unique_owner_ids': [],
         }
+        if not self.ids:
+            self.update(defaults)
+            return
+
+        Users = self.env["res.users"]
+        query = Users._where_calc([])
+        Users._apply_ir_rules(query)
+        badge_alias = query.join("res_users", "id", "gamification_badge_user", "user_id", "badges")
+
+        tables, where_clauses, where_params = query.get_sql()
+
+        self.env.cr.execute(
+            f"""
+              SELECT {badge_alias}.badge_id, count(res_users.id) as stat_count,
+                     count(distinct(res_users.id)) as stat_count_distinct,
+                     array_agg(distinct(res_users.id)) as unique_owner_ids
+                FROM {tables}
+               WHERE {where_clauses}
+                 AND {badge_alias}.badge_id IN %s
+            GROUP BY {badge_alias}.badge_id
+            """,
+            [*where_params, tuple(self.ids)]
+        )
+
         mapping = {
             badge_id: {
-                'stat_count': count,
-                'stat_count_distinct': distinct_count,
+                'granted_count': count,
+                'granted_users_count': distinct_count,
                 'unique_owner_ids': owner_ids,
             }
             for (badge_id, count, distinct_count, owner_ids) in self.env.cr._obj
