@@ -8,10 +8,8 @@ import logging
 import requests
 from werkzeug import urls
 
-from odoo import api, fields, models, registry, _
+from odoo import api, fields, models, _
 from odoo.exceptions import UserError
-from odoo.http import request
-
 
 _logger = logging.getLogger(__name__)
 
@@ -22,8 +20,7 @@ GOOGLE_TOKEN_ENDPOINT = 'https://accounts.google.com/o/oauth2/token'
 GOOGLE_API_BASE_URL = 'https://www.googleapis.com'
 
 
-# FIXME : this needs to become an AbstractModel, to be inhereted by google_calendar_service and google_drive_service
-class GoogleService(models.TransientModel):
+class GoogleService(models.AbstractModel):
     _name = 'google.service'
     _description = 'Google Service'
 
@@ -96,7 +93,7 @@ class GoogleService(models.TransientModel):
         return "%s?%s" % (GOOGLE_AUTH_ENDPOINT, encoded_params)
 
     @api.model
-    def _get_google_token_json(self, authorize_code, service):
+    def _get_google_tokens(self, authorize_code, service):
         """ Call Google API to exchange authorization code against token, with POST request, to
             not be redirected.
         """
@@ -114,73 +111,86 @@ class GoogleService(models.TransientModel):
             'redirect_uri': base_url + '/google_account/authentication'
         }
         try:
-            dummy, response, dummy = self._do_request(GOOGLE_TOKEN_ENDPOINT, params=data, headers=headers, type='POST', preuri='')
-            return response
+            dummy, response, dummy = self._do_request(GOOGLE_TOKEN_ENDPOINT, params=data, headers=headers, method='POST', preuri='')
+            access_token = response.get('access_token')
+            refresh_token = response.get('refresh_token')
+            ttl = response.get('expires_in')
+            return access_token, refresh_token, ttl
         except requests.HTTPError:
             error_msg = _("Something went wrong during your token generation. Maybe your Authorization Code is invalid")
             raise self.env['res.config.settings'].get_config_warning(error_msg)
 
-    # FIXME : this method update a field defined in google_calendar module. Since it is used only in that module, maybe it should be moved.
     @api.model
-    def _refresh_google_token_json(self, refresh_token, service):  # exchange_AUTHORIZATION vs Token (service = calendar)
+    def _get_access_token(self, refresh_token, service, scope):
+        """Fetch the access token thanks to the refresh token."""
         get_param = self.env['ir.config_parameter'].sudo().get_param
-        client_id = get_param('google_%s_client_id' % (service,), default=False)
-        client_secret = get_param('google_%s_client_secret' % (service,), default=False)
+        client_id = get_param('google_%s_client_id' % service, default=False)
+        client_secret = get_param('google_%s_client_secret' % service, default=False)
 
         if not client_id or not client_secret:
-            raise UserError(_("The account for the Google service '%s' is not configured.") % service)
+            raise UserError(_('Google %s is not yet configured.', service.title()))
 
-        headers = {"content-type": "application/x-www-form-urlencoded"}
-        data = {
-            'refresh_token': refresh_token,
-            'client_id': client_id,
-            'client_secret': client_secret,
-            'grant_type': 'refresh_token',
-        }
+        if not refresh_token:
+            raise UserError(_('The refresh token for authentication is not set.'))
 
         try:
-            dummy, response, dummy = self._do_request(GOOGLE_TOKEN_ENDPOINT, params=data, headers=headers, type='POST', preuri='')
-            return response
-        except requests.HTTPError as error:
-            if error.response.status_code == 400:  # invalid grant
-                with registry(request.session.db).cursor() as cur:
-                    self.env(cur)['res.users'].browse(self.env.uid).sudo().write({'google_%s_rtoken' % service: False})
-            error_key = error.response.json().get("error", "nc")
-            _logger.exception("Bad google request : %s !", error_key)
-            error_msg = _("Something went wrong during your token generation. Maybe your Authorization Code is invalid or already expired [%s]") % error_key
-            raise self.env['res.config.settings'].get_config_warning(error_msg)
+            result = requests.post(
+                GOOGLE_TOKEN_ENDPOINT,
+                data={
+                    'client_id': client_id,
+                    'client_secret': client_secret,
+                    'refresh_token': refresh_token,
+                    'grant_type': 'refresh_token',
+                    'scope': scope,
+                },
+                headers={'Content-type': 'application/x-www-form-urlencoded'},
+                timeout=TIMEOUT,
+            )
+            result.raise_for_status()
+        except requests.HTTPError:
+            raise UserError(
+                _('Something went wrong during the token generation. Please request again an authorization code.')
+            )
+
+        json_result = result.json()
+
+        return json_result.get('access_token'), json_result.get('expires_in')
 
-    # TODO JEM : remove preuri param, and rename type into method
     @api.model
-    def _do_request(self, uri, params={}, headers={}, type='POST', preuri="https://www.googleapis.com"):
+    def _do_request(self, uri, params=None, headers=None, method='POST', preuri="https://www.googleapis.com", timeout=TIMEOUT):
         """ Execute the request to Google API. Return a tuple ('HTTP_CODE', 'HTTP_RESPONSE')
             :param uri : the url to contact
             :param params : dict or already encoded parameters for the request to make
             :param headers : headers of request
-            :param type : the method to use to make the request
+            :param method : the method to use to make the request
             :param preuri : pre url to prepend to param uri.
         """
-        _logger.debug("Uri: %s - Type : %s - Headers: %s - Params : %s !", (uri, type, headers, params))
+        if params is None:
+            params = {}
+        if headers is None:
+            headers = {}
+
+        _logger.debug("Uri: %s - Type : %s - Headers: %s - Params : %s !", uri, method, headers, params)
 
         ask_time = fields.Datetime.now()
         try:
-            if type.upper() in ('GET', 'DELETE'):
-                res = requests.request(type.lower(), preuri + uri, params=params, timeout=TIMEOUT)
-            elif type.upper() in ('POST', 'PATCH', 'PUT'):
-                res = requests.request(type.lower(), preuri + uri, data=params, headers=headers, timeout=TIMEOUT)
+            if method.upper() in ('GET', 'DELETE'):
+                res = requests.request(method.lower(), preuri + uri, params=params, timeout=timeout)
+            elif method.upper() in ('POST', 'PATCH', 'PUT'):
+                res = requests.request(method.lower(), preuri + uri, data=params, headers=headers, timeout=timeout)
             else:
-                raise Exception(_('Method not supported [%s] not in [GET, POST, PUT, PATCH or DELETE]!') % (type))
+                raise Exception(_('Method not supported [%s] not in [GET, POST, PUT, PATCH or DELETE]!') % (method))
             res.raise_for_status()
             status = res.status_code
 
-            if int(status) in (204, 404):  # Page not found, no response
+            if int(status) == 204:  # Page not found, no response
                 response = False
             else:
                 response = res.json()
 
             try:
-                ask_time = datetime.strptime(res.headers.get('date'), "%a, %d %b %Y %H:%M:%S %Z")
-            except:
+                ask_time = datetime.strptime(res.headers.get('date', ''), "%a, %d %b %Y %H:%M:%S %Z")
+            except ValueError:
                 pass
         except requests.HTTPError as error:
             if error.response.status_code in (204, 404):
@@ -188,12 +198,5 @@ class GoogleService(models.TransientModel):
                 response = ""
             else:
                 _logger.exception("Bad google request : %s !", error.response.content)
-                if error.response.status_code in (400, 401, 403, 410):
-                    raise error
-                raise self.env['res.config.settings'].get_config_warning(_("Something went wrong with your request to google"))
+                raise error
         return (status, response, ask_time)
-
-    # TODO : remove me, it is only used in google calendar. Make google_calendar use the constants
-    @api.model
-    def get_client_id(self, service):
-        return self.env['ir.config_parameter'].sudo().get_param('google_%s_client_id' % (service,), default=False)
