@@ -42,7 +42,7 @@ var KioskMode = AbstractAction.extend({
     on_attach_callback: function () {
         // Stop polling to avoid notifications in kiosk mode
         this.call('bus_service', 'stopPolling');
-        $('body').find('.o_thread_window_close').click();
+        $('body').find('.o_ChatWindowHeader_commandClose').click();
     },
 
     _onBarcodeScanned: function(barcode) {
@@ -57,7 +57,7 @@ var KioskMode = AbstractAction.extend({
                 if (result.action) {
                     self.do_action(result.action);
                 } else if (result.warning) {
-                    self.do_warn(result.warning);
+                    self.displayNotification({ title: result.warning, type: 'danger' });
                     core.bus.on('barcode_scanned', self, self._onBarcodeScanned);
                 }
             }, function () {
@@ -75,7 +75,6 @@ var KioskMode = AbstractAction.extend({
         core.bus.off('barcode_scanned', this, this._onBarcodeScanned);
         clearInterval(this.clock_start);
         clearInterval(this._interval);
-        this.call('bus_service', 'startPolling');
         this._super.apply(this, arguments);
     },
 
