@@ -5,6 +5,7 @@
     'name': 'Employee Contracts',
     'version': '1.0',
     'category': 'Human Resources/Contracts',
+    'sequence': 335,
     'description': """
 Add all information on the employee form to manage contracts.
 =============================================================
@@ -16,17 +17,27 @@ Add all information on the employee form to manage contracts.
 
 You can assign several contracts per employee.
     """,
-    'website': 'https://www.odoo.com/page/employees',
+    'website': 'https://www.odoo.com/app/employees',
     'depends': ['hr'],
     'data': [
         'security/security.xml',
         'security/ir.model.access.csv',
         'data/hr_contract_data.xml',
+        'report/hr_contract_history_report_views.xml',
         'views/hr_contract_views.xml',
-        'views/assets.xml',
+        'views/hr_employee_views.xml',
+        'views/hr_employee_public_views.xml',
+        'views/resource_calendar_views.xml',
+        'wizard/hr_departure_wizard_views.xml',
     ],
     'demo': ['data/hr_contract_demo.xml'],
     'installable': True,
     'auto_install': False,
+    'application': True,
+    'assets': {
+        'web.assets_backend': [
+            'hr_contract/static/src/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
