@@ -2,6 +2,8 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import hr_employee
+from . import hr_employee_public
 from . import hr_contract
 from . import res_users
 from . import resource
+from . import hr_contract_type
