@@ -9,7 +9,6 @@ import logging
 from collections import defaultdict
 
 from odoo import api, fields, models
-from odoo.exceptions import ValidationError
 from odoo.osv import expression
 from odoo.tools.translate import _
 from odoo.tools.float_utils import float_round
@@ -20,15 +19,15 @@ _logger = logging.getLogger(__name__)
 class HolidaysType(models.Model):
     _name = "hr.leave.type"
     _description = "Time Off Type"
+    _order = 'sequence'
 
     @api.model
     def _model_sorting_key(self, leave_type):
         remaining = leave_type.virtual_remaining_leaves > 0
         taken = leave_type.leaves_taken > 0
-        return leave_type.allocation_type == 'fixed' and remaining, leave_type.allocation_type == 'fixed_allocation' and remaining, taken
+        return -1*leave_type.sequence, leave_type.employee_requests == 'no' and remaining, leave_type.employee_requests == 'yes' and remaining, taken
 
     name = fields.Char('Time Off Type', required=True, translate=True)
-    code = fields.Char('Code')
     sequence = fields.Integer(default=100,
                               help='The type with the smallest sequence is the default value in time off request')
     create_calendar_meeting = fields.Boolean(string="Display Time Off in Calendar", default=True)
@@ -49,7 +48,9 @@ class HolidaysType(models.Model):
         ('lavender', 'Lavender'),
         ('wheat', 'Wheat'),
         ('ivory', 'Ivory')], string='Color in Report', required=True, default='red',
-        help='This color will be used in the time off summary located in Reporting > Time off by Department.')
+         help='This color will be used in the time off summary located in Reporting > Time off by Department.')
+    color = fields.Integer(string='Color', help="The color selected here will be used in every screen with the time off type.")
+    icon_id = fields.Many2one('ir.attachment', string='Cover Image', domain="[('res_model', '=', 'hr.leave.type'), ('res_field', '=', 'icon_id')]")
     active = fields.Boolean('Active', default=True,
                             help="If the active field is set to false, it will allow you to hide the time off type without removing it.")
     max_leaves = fields.Float(compute='_compute_leaves', string='Maximum Allowed', search='_search_max_leaves',
@@ -63,70 +64,100 @@ class HolidaysType(models.Model):
     virtual_remaining_leaves = fields.Float(
         compute='_compute_leaves', search='_search_virtual_remaining_leaves', string='Virtual Remaining Time Off',
         help='Maximum Time Off Allowed - Time Off Already Taken - Time Off Waiting Approval')
+    virtual_leaves_taken = fields.Float(
+        compute='_compute_leaves', string='Virtual Time Off Already Taken',
+        help='Sum of validated and non validated time off requests.')
+    # KBA TODO in master: rename, change to int
     group_days_allocation = fields.Float(
         compute='_compute_group_days_allocation', string='Days Allocated')
     group_days_leave = fields.Float(
         compute='_compute_group_days_leave', string='Group Time Off')
     company_id = fields.Many2one('res.company', string='Company', default=lambda self: self.env.company)
-    responsible_id = fields.Many2one('res.users', 'Responsible',
+    responsible_id = fields.Many2one(
+        'res.users', 'Responsible Time Off Officer',
         domain=lambda self: [('groups_id', 'in', self.env.ref('hr_holidays.group_hr_holidays_user').id)],
-        help="This user will be responsible for approving this type of times off"
-        "This is only used when validation is 'hr' or 'both'",)
-    validation_type = fields.Selection([
+        help="Choose the Time Off Officer who will be notified to approve allocation or Time Off request")
+    leave_validation_type = fields.Selection([
         ('no_validation', 'No Validation'),
-        ('hr', 'Time Off Officer'),
-        ('manager', 'Team Leader'),
-        ('both', 'Team Leader and Time Off Officer')], default='hr', string='Validation')
-    allocation_type = fields.Selection([
-        ('no', 'No Allocation Needed'),
-        ('fixed_allocation', 'Free Allocation Request'),
-        ('fixed', 'Allocated by HR only')],
-        default='no', string='Mode',
-        help='\tNo Allocation Needed: no allocation by default, users can freely request time off;'
-             '\tFree Allocation Request: allocated by HR and users can request time off and allocations;'
-             '\tAllocated by HR only: allocated by HR and cannot be bypassed; users can request time off;')
-    validity_start = fields.Date("From",
-                                 help='Adding validity to types of time off so that it cannot be selected outside this time period')
-    validity_stop = fields.Date("To")
-    valid = fields.Boolean(compute='_compute_valid', search='_search_valid', help='This indicates if it is still possible to use this type of leave')
+        ('hr', 'By Time Off Officer'),
+        ('manager', "By Employee's Approver"),
+        ('both', "By Employee's Approver and Time Off Officer")], default='hr', string='Leave Validation')
+    requires_allocation = fields.Selection([
+        ('yes', 'Yes'),
+        ('no', 'No Limit')], default="yes", required=True, string='Requires allocation')
+    employee_requests = fields.Selection([
+        ('yes', 'Extra Days Requests Allowed'),
+        ('no', 'Not Allowed')], default="no", required=True, string="Employee Requests")
+    allocation_validation_type = fields.Selection([
+        ('no', 'No validation needed'),
+        ('officer', 'Approved by Time Off Officer'),
+        ('set', "Set by Time Off Officer")], default='officer', string='Approval')
+    has_valid_allocation = fields.Boolean(compute='_compute_valid', search='_search_valid', help='This indicates if it is still possible to use this type of leave')
     time_type = fields.Selection([('leave', 'Time Off'), ('other', 'Other')], default='leave', string="Kind of Leave",
                                  help="Whether this should be computed as a holiday or as work time (eg: formation)")
     request_unit = fields.Selection([
-        ('day', 'Day'), ('half_day', 'Half Day'), ('hour', 'Hours')],
-        default='day', string='Take Time Off in', required=True)
+        ('day', 'Day'),
+        ('half_day', 'Half Day'),
+        ('hour', 'Hours')], default='day', string='Take Time Off in', required=True)
     unpaid = fields.Boolean('Is Unpaid', default=False)
     leave_notif_subtype_id = fields.Many2one('mail.message.subtype', string='Time Off Notification Subtype', default=lambda self: self.env.ref('hr_holidays.mt_leave', raise_if_not_found=False))
     allocation_notif_subtype_id = fields.Many2one('mail.message.subtype', string='Allocation Notification Subtype', default=lambda self: self.env.ref('hr_holidays.mt_leave_allocation', raise_if_not_found=False))
+    support_document = fields.Boolean(string='Supporting Document')
+    accruals_ids = fields.One2many('hr.leave.accrual.plan', 'time_off_type_id')
+    accrual_count = fields.Float(compute="_compute_accrual_count", string="Accruals count")
 
-    @api.constrains('validity_start', 'validity_stop')
-    def _check_validity_dates(self):
-        for leave_type in self:
-            if leave_type.validity_start and leave_type.validity_stop and \
-               leave_type.validity_start > leave_type.validity_stop:
-                raise ValidationError(_("End of validity period should be greater than start of validity period"))
-
-    @api.depends('validity_start', 'validity_stop')
-    def _compute_valid(self):
-        dt = self._context.get('default_date_from') or fields.Date.context_today(self)
-
-        for holiday_type in self:
-            if holiday_type.validity_start and holiday_type.validity_stop:
-                holiday_type.valid = ((dt < holiday_type.validity_stop) and (dt > holiday_type.validity_start))
-            elif holiday_type.validity_start and (dt > holiday_type.validity_start):
-                holiday_type.valid = False
-            else:
-                holiday_type.valid = True
 
+    @api.model
     def _search_valid(self, operator, value):
-        dt = self._context.get('default_date_from', False)
+        """ Returns leave_type ids for which a valid allocation exists
+            or that don't need an allocation
+            return [('id', domain_operator, [x['id'] for x in res])]
+        """
+        date_to = self._context.get('default_date_from') or fields.Date.today().strftime('%Y-1-1')
+        date_from = self._context.get('default_date_to') or fields.Date.today().strftime('%Y-12-31')
+        employee_id = self._context.get('default_employee_id', self._context.get('employee_id')) or self.env.user.employee_id.id
 
-        if not dt:
-            return []
-        signs = ['>=', '<='] if operator == '=' else ['<=', '>=']
+        if not isinstance(value, bool):
+            raise ValueError('Invalid value: %s' % (value))
+        if operator not in ['=', '!=']:
+            raise ValueError('Invalid operator: %s' % (operator))
+        new_operator = 'in' if operator == '=' else 'not in'
 
-        return ['|', ('validity_stop', operator, False), '&',
-                ('validity_stop', signs[0] if value else signs[1], dt),
-                ('validity_start', signs[1] if value else signs[0], dt)]
+        query = '''
+        SELECT
+            holiday_status_id
+        FROM
+            hr_leave_allocation alloc
+        WHERE
+            alloc.employee_id = %s AND
+            alloc.active = True AND alloc.state = 'validate' AND
+            (alloc.date_to >= %s OR alloc.date_to IS NULL) AND
+            alloc.date_from <= %s 
+        '''
+
+        self._cr.execute(query, (employee_id or None, date_to, date_from))
+
+        return [('id', new_operator, [x['holiday_status_id'] for x in self._cr.dictfetchall()])]
+
+
+    @api.depends('requires_allocation')
+    def _compute_valid(self):
+        date_to = self._context.get('default_date_to', fields.Datetime.today())
+        date_from = self._context.get('default_date_from', fields.Datetime.today())
+        employee_id = self._context.get('default_employee_id', self._context.get('employee_id', self.env.user.employee_id.id))
+        for holiday_type in self:
+            if holiday_type.requires_allocation:
+                allocation = self.env['hr.leave.allocation'].search([
+                    ('holiday_status_id', '=', holiday_type.id),
+                    ('employee_id', '=', employee_id),
+                    '|',
+                    ('date_to', '>=', date_to),
+                    '&',
+                    ('date_to', '=', False),
+                    ('date_from', '<=', date_from)])
+                holiday_type.has_valid_allocation = bool(allocation)
+            else:
+                holiday_type.has_valid_allocation = True
 
     def _search_max_leaves(self, operator, value):
         value = float(value)
@@ -163,7 +194,7 @@ class HolidaysType(models.Model):
         valid_leave_types = self.env['hr.leave.type']
 
         for leave_type in leave_types:
-            if leave_type.allocation_type != 'no':
+            if leave_type.requires_allocation == "yes":
                 if operator == '>' and leave_type.virtual_remaining_leaves > value:
                     valid_leave_types |= leave_type
                 elif operator == '<' and leave_type.virtual_remaining_leaves < value:
@@ -181,11 +212,7 @@ class HolidaysType(models.Model):
 
         return [('id', 'in', valid_leave_types.ids)]
 
-    # YTI TODO: Remove me in master
-    def get_days(self, employee_id):
-        return self.get_employees_days([employee_id])[employee_id]
-
-    def get_employees_days(self, employee_ids):
+    def get_employees_days(self, employee_ids, date=None):
         result = {
             employee_id: {
                 leave_type.id: {
@@ -193,6 +220,7 @@ class HolidaysType(models.Model):
                     'leaves_taken': 0,
                     'remaining_leaves': 0,
                     'virtual_remaining_leaves': 0,
+                    'virtual_leaves_taken': 0,
                 } for leave_type in self
             } for employee_id in employee_ids
         }
@@ -203,53 +231,74 @@ class HolidaysType(models.Model):
             ('holiday_status_id', 'in', self.ids)
         ])
 
+        if not date:
+            date = self.env.context.get('default_date_from') or fields.Date.context_today(self)
         allocations = self.env['hr.leave.allocation'].search([
             ('employee_id', 'in', employee_ids),
             ('state', 'in', ['confirm', 'validate1', 'validate']),
-            ('holiday_status_id', 'in', self.ids)
+            ('holiday_status_id', 'in', self.ids),
+            ('date_from', '<=', date),
+            '|', ('date_to', '=', False),
+                 ('date_to', '>=', date),
         ])
 
         for request in requests:
             status_dict = result[request.employee_id.id][request.holiday_status_id.id]
-            status_dict['virtual_remaining_leaves'] -= (request.number_of_hours_display
+            if not request.holiday_allocation_id or request.holiday_allocation_id in allocations:
+                status_dict['virtual_remaining_leaves'] -= (request.number_of_hours_display
+                                                        if request.leave_type_request_unit == 'hour'
+                                                        else request.number_of_days)
+            if request.holiday_status_id.requires_allocation == 'no':
+                status_dict['virtual_leaves_taken'] += (request.number_of_hours_display
                                                     if request.leave_type_request_unit == 'hour'
                                                     else request.number_of_days)
-            if request.state == 'validate':
-                status_dict['leaves_taken'] += (request.number_of_hours_display
-                                            if request.leave_type_request_unit == 'hour'
-                                            else request.number_of_days)
-                status_dict['remaining_leaves'] -= (request.number_of_hours_display
+                if request.state == 'validate':
+                    status_dict['leaves_taken'] += (request.number_of_hours_display
                                                 if request.leave_type_request_unit == 'hour'
                                                 else request.number_of_days)
+                    status_dict['remaining_leaves'] -= (request.number_of_hours_display
+                                                    if request.leave_type_request_unit == 'hour'
+                                                    else request.number_of_days)
 
         for allocation in allocations.sudo():
             status_dict = result[allocation.employee_id.id][allocation.holiday_status_id.id]
             if allocation.state == 'validate':
-                # note: add only validated allocation even for the virtual
-                # count; otherwise pending then refused allocation allow
-                # the employee to create more leaves than possible
                 status_dict['virtual_remaining_leaves'] += (allocation.number_of_hours_display
-                                                          if allocation.type_request_unit == 'hour'
-                                                          else allocation.number_of_days)
-                status_dict['max_leaves'] += (allocation.number_of_hours_display
-                                            if allocation.type_request_unit == 'hour'
-                                            else allocation.number_of_days)
-                status_dict['remaining_leaves'] += (allocation.number_of_hours_display
-                                                  if allocation.type_request_unit == 'hour'
-                                                  else allocation.number_of_days)
+                                                        if allocation.type_request_unit == 'hour'
+                                                        else allocation.number_of_days)
+                if allocation.holiday_status_id.requires_allocation == 'no':
+                    # note: add only validated allocation even for the virtual
+                    # count; otherwise pending then refused allocation allow
+                    # the employee to create more leaves than possible
+                    status_dict['max_leaves'] += (allocation.number_of_hours_display
+                                                if allocation.type_request_unit == 'hour'
+                                                else allocation.number_of_days)
+                    status_dict['remaining_leaves'] += (allocation.number_of_hours_display
+                                                    if allocation.type_request_unit == 'hour'
+                                                    else allocation.number_of_days)
+                else:
+                    remaining_leaves = allocation.max_leaves - allocation.leaves_taken
+                    status_dict['max_leaves'] += allocation.max_leaves
+                    status_dict['remaining_leaves'] += remaining_leaves
+                    status_dict['leaves_taken'] += allocation.leaves_taken
         return result
 
     @api.model
     def get_days_all_request(self):
-        leave_types = sorted(self.search([]).filtered(lambda x: x.virtual_remaining_leaves or x.max_leaves), key=self._model_sorting_key, reverse=True)
-        return [(lt.name, {
-                    'remaining_leaves': ('%.2f' % lt.remaining_leaves).rstrip('0').rstrip('.'),
-                    'virtual_remaining_leaves': ('%.2f' % lt.virtual_remaining_leaves).rstrip('0').rstrip('.'),
-                    'max_leaves': ('%.2f' % lt.max_leaves).rstrip('0').rstrip('.'),
-                    'leaves_taken': ('%.2f' % lt.leaves_taken).rstrip('0').rstrip('.'),
-                    'request_unit': lt.request_unit,
-                }, lt.allocation_type)
-            for lt in leave_types]
+        leave_types = sorted(self.search([]).filtered(lambda x: ((x.virtual_remaining_leaves > 0 or x.max_leaves))), key=self._model_sorting_key, reverse=True)
+        return [lt._get_days_request() for lt in leave_types]
+
+    def _get_days_request(self):
+        self.ensure_one()
+        return (self.name, {
+                'remaining_leaves': ('%.2f' % self.remaining_leaves).rstrip('0').rstrip('.'),
+                'virtual_remaining_leaves': ('%.2f' % self.virtual_remaining_leaves).rstrip('0').rstrip('.'),
+                'max_leaves': ('%.2f' % self.max_leaves).rstrip('0').rstrip('.'),
+                'leaves_taken': ('%.2f' % self.leaves_taken).rstrip('0').rstrip('.'),
+                'virtual_leaves_taken': ('%.2f' % self.virtual_leaves_taken).rstrip('0').rstrip('.'),
+                'request_unit': self.request_unit,
+                'icon': self.sudo().icon_id.url,
+                }, self.requires_allocation, self.id)
 
     def _get_contextual_employee_id(self):
         if 'employee_id' in self._context:
@@ -260,12 +309,14 @@ class HolidaysType(models.Model):
             employee_id = self.env.user.employee_id.id
         return employee_id
 
+    @api.depends_context('employee_id', 'default_employee_id')
     def _compute_leaves(self):
         data_days = {}
         employee_id = self._get_contextual_employee_id()
 
         if employee_id:
-            data_days = self.get_employees_days([employee_id])[employee_id]
+            data_days = (self.get_employees_days(employee_id)[employee_id[0]] if isinstance(employee_id, list) else
+                         self.get_employees_days([employee_id])[employee_id])
 
         for holiday_status in self:
             result = data_days.get(holiday_status.id, {})
@@ -273,30 +324,21 @@ class HolidaysType(models.Model):
             holiday_status.leaves_taken = result.get('leaves_taken', 0)
             holiday_status.remaining_leaves = result.get('remaining_leaves', 0)
             holiday_status.virtual_remaining_leaves = result.get('virtual_remaining_leaves', 0)
+            holiday_status.virtual_leaves_taken = result.get('virtual_leaves_taken', 0)
 
     def _compute_group_days_allocation(self):
-        domain = [
-            ('holiday_status_id', 'in', self.ids),
-            ('holiday_type', '!=', 'employee'),
-            ('state', '=', 'validate'),
-        ]
-        domain2 = [
-            '|',
-            ('date_from', '>=', fields.Datetime.to_string(datetime.datetime.now().replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0))),
-            ('date_from', '=', False),
-        ]
         grouped_res = self.env['hr.leave.allocation'].read_group(
-            expression.AND([domain, domain2]),
-            ['holiday_status_id', 'number_of_days'],
+            [('holiday_status_id', 'in', self.ids), ],
+            ['holiday_status_id'],
             ['holiday_status_id'],
         )
-        grouped_dict = dict((data['holiday_status_id'][0], data['number_of_days']) for data in grouped_res)
+        grouped_dict = dict((data['holiday_status_id'][0], data['holiday_status_id_count']) for data in grouped_res)
         for allocation in self:
             allocation.group_days_allocation = grouped_dict.get(allocation.id, 0)
 
     def _compute_group_days_leave(self):
         grouped_res = self.env['hr.leave'].read_group(
-            [('holiday_status_id', 'in', self.ids), ('holiday_type', '=', 'employee'), ('state', '=', 'validate'),
+            [('holiday_status_id', 'in', self.ids),
              ('date_from', '>=', fields.Datetime.to_string(datetime.datetime.now().replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0)))],
             ['holiday_status_id'],
             ['holiday_status_id'],
@@ -305,6 +347,12 @@ class HolidaysType(models.Model):
         for allocation in self:
             allocation.group_days_leave = grouped_dict.get(allocation.id, 0)
 
+    def _compute_accrual_count(self):
+        accrual_allocations = self.env['hr.leave.accrual.plan'].read_group([('time_off_type_id', 'in', self.ids)], ['time_off_type_id'], ['time_off_type_id'])
+        mapped_data = dict((data['time_off_type_id'][0], data['time_off_type_id_count']) for data in accrual_allocations)
+        for leave_type in self:
+            leave_type.accrual_count = mapped_data.get(leave_type.id, 0)
+
     def name_get(self):
         if not self._context.get('employee_id'):
             # leave counts is based on employee_id, would be inaccurate if not based on correct employee
@@ -312,7 +360,7 @@ class HolidaysType(models.Model):
         res = []
         for record in self:
             name = record.name
-            if record.allocation_type != 'no':
+            if record.requires_allocation == "yes" and not self._context.get('from_manager_leave_form'):
                 name = "%(name)s (%(count)s)" % {
                     'name': name,
                     'count': _('%g remaining out of %g') % (
@@ -346,17 +394,10 @@ class HolidaysType(models.Model):
 
     def action_see_days_allocated(self):
         self.ensure_one()
-        action = self.env.ref('hr_holidays.hr_leave_allocation_action_all').read()[0]
-        domain = [
+        action = self.env["ir.actions.actions"]._for_xml_id("hr_holidays.hr_leave_allocation_action_all")
+        action['domain'] = [
             ('holiday_status_id', 'in', self.ids),
-            ('holiday_type', '!=', 'employee'),
         ]
-        domain2 = [
-            '|',
-            ('date_from', '>=', fields.Datetime.to_string(datetime.datetime.now().replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0))),
-            ('date_from', '=', False),
-        ]
-        action['domain'] = expression.AND([domain, domain2])
         action['context'] = {
             'default_holiday_type': 'department',
             'default_holiday_status_id': self.ids[0],
@@ -365,7 +406,7 @@ class HolidaysType(models.Model):
 
     def action_see_group_leaves(self):
         self.ensure_one()
-        action = self.env.ref('hr_holidays.hr_leave_action_all').read()[0]
+        action = self.env["ir.actions.actions"]._for_xml_id("hr_holidays.hr_leave_action_action_approve_department")
         action['domain'] = [
             ('holiday_status_id', '=', self.ids[0]),
             ('date_from', '>=', fields.Datetime.to_string(datetime.datetime.now().replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0)))
@@ -374,3 +415,14 @@ class HolidaysType(models.Model):
             'default_holiday_status_id': self.ids[0],
         }
         return action
+
+    def action_see_accrual_plans(self):
+        self.ensure_one()
+        action = self.env["ir.actions.actions"]._for_xml_id("hr_holidays.open_view_accrual_plans")
+        action['domain'] = [
+            ('time_off_type_id', '=', self.id),
+        ]
+        action['context'] = {
+            'default_time_off_type_id': self.id,
+        }
+        return action
