@@ -5,20 +5,19 @@ from datetime import datetime
 from dateutil.relativedelta import relativedelta
 
 from odoo import tests
-from odoo.addons.hr_holidays.tests.common import TestHrHolidaysBase
+from odoo.addons.hr_holidays.tests.common import TestHrHolidaysCommon
 from odoo.exceptions import AccessError, UserError, ValidationError
 from odoo.tools import mute_logger
 
 
 @tests.tagged('access_rights', 'post_install', '-at_install')
-class TestLeavesRights(TestHrHolidaysBase):
+class TestHrHolidaysAccessRightsCommon(TestHrHolidaysCommon):
     def setUp(self):
-        super(TestLeavesRights, self).setUp()
+        super(TestHrHolidaysAccessRightsCommon, self).setUp()
         self.leave_type = self.env['hr.leave.type'].create({
             'name': 'Unlimited',
-            'validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'leave_validation_type': 'hr',
+            'requires_allocation': 'no',
         })
         self.rd_dept.manager_id = False
         self.hr_dept.manager_id = False
@@ -35,30 +34,26 @@ class TestLeavesRights(TestHrHolidaysBase):
 
         self.lt_no_validation = self.env['hr.leave.type'].create({
             'name': 'Validation = no_validation',
-            'validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'leave_validation_type': 'hr',
+            'requires_allocation': 'no',
         })
 
         self.lt_validation_hr = self.env['hr.leave.type'].create({
             'name': 'Validation = HR',
-            'validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'leave_validation_type': 'hr',
+            'requires_allocation': 'no',
         })
 
         self.lt_validation_manager = self.env['hr.leave.type'].create({
             'name': 'Validation = manager',
-            'validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'leave_validation_type': 'hr',
+            'requires_allocation': 'no',
         })
 
         self.lt_validation_both = self.env['hr.leave.type'].create({
             'name': 'Validation = both',
-            'validation_type': 'hr',
-            'allocation_type': 'no',
-            'validity_start': False,
+            'leave_validation_type': 'hr',
+            'requires_allocation': 'no',
         })
 
         self.draft_status = [
@@ -77,14 +72,16 @@ class TestLeavesRights(TestHrHolidaysBase):
     def request_leave(self, user_id, date_from, number_of_days, values=None):
         values = dict(values or {}, **{
             'date_from': date_from,
+            'request_date_from': date_from,
             'date_to': date_from + relativedelta(days=number_of_days),
+            'request_date_to': date_from + relativedelta(days=number_of_days),
             'number_of_days': number_of_days,
         })
         return self.env['hr.leave'].with_user(user_id).create(values)
 
 
 @tests.tagged('access_rights', 'access_rights_states')
-class TestAcessRightsStates(TestLeavesRights):
+class TestAcessRightsStates(TestHrHolidaysAccessRightsCommon):
     # ******************************************************
     # Action draft
     # ******************************************************
@@ -107,9 +104,10 @@ class TestAcessRightsStates(TestLeavesRights):
                 'name': 'Ranoi',
                 'employee_id': self.employee_emp.id,
                 'holiday_status_id': status.id,
-                'state': 'draft'
             }
             leave = self.request_leave(1, datetime.today() + relativedelta(days=20 + i), 1, values)
+            # the state has to be set to draft in a write because it is initialized to confirm if it has validation
+            leave.write({'state': 'draft'})
             with self.assertRaises(UserError):
                 leave.action_draft()
 
@@ -207,7 +205,7 @@ class TestAcessRightsStates(TestLeavesRights):
     def test_holiday_user_draft_his_leave(self):
         """
             Should be able to draft his own leave
-            whatever the holidays_status_id
+            whatever the holiday_status_id
         """
         for i, status in enumerate(self.draft_status):
             values = {
@@ -377,7 +375,7 @@ class TestAcessRightsStates(TestLeavesRights):
             leave.with_user(self.user_hrmanager.id).action_draft()
 
 @tests.tagged('access_rights', 'access_rights_create')
-class TestAccessRightsCreate(TestLeavesRights):
+class TestAccessRightsCreate(TestHrHolidaysAccessRightsCommon):
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_base_user_create_self(self):
         """ A simple user can create a leave for himself """
@@ -399,17 +397,6 @@ class TestAccessRightsCreate(TestLeavesRights):
         with self.assertRaises(AccessError):
             self.request_leave(self.user_employee_id, datetime.today() + relativedelta(days=5), 1, values)
 
-    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
-    def test_base_user_create_validate(self):
-        """ A simple user cannot create a leave in validate state """
-        values = {
-            'name': 'Hol10',
-            'employee_id': self.employee_emp_id,
-            'holiday_status_id': self.leave_type.id,
-            'state': 'validate',
-        }
-        with self.assertRaises(ValidationError):
-            self.request_leave(self.user_employee_id, datetime.today() + relativedelta(days=5), 1, values)
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_base_user_create_batch(self):
@@ -445,34 +432,6 @@ class TestAccessRightsCreate(TestLeavesRights):
         }
         self.request_leave(self.user_hruser_id, datetime.today() + relativedelta(days=5), 1, values)
 
-    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
-    def test_holidays_user_create_validate(self):
-        """ A holidays user cannot create a leave in validate state """
-        values = {
-            'name': 'Hol10',
-            'employee_id': self.employee_emp_id,
-            'holiday_status_id': self.leave_type.id,
-            'state': 'validate',
-        }
-        with self.assertRaises(ValidationError):
-            self.request_leave(self.user_hruser_id, datetime.today() + relativedelta(days=5), 1, values)
-        values.update(employee_id=self.employee_hruser_id)
-        with self.assertRaises(ValidationError):
-            self.request_leave(self.user_hruser_id, datetime.today() + relativedelta(days=5), 1, values)
-
-
-    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
-    def test_holidays_user_create_batch(self):
-        """ A holidays user cannot create a leave in bacth mode (by company, by department, by tag)"""
-        values = {
-            'name': 'Hol10',
-            'holiday_status_id': self.leave_type.id,
-            'holiday_type': 'company',
-            'mode_company_id': 1,
-        }
-        with self.assertRaises(AccessError):
-            self.request_leave(self.user_hruser_id, datetime.today() + relativedelta(days=5), 1, values)
-
     # hr_holidays.group_hr_holidays_manager
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
@@ -495,21 +454,6 @@ class TestAccessRightsCreate(TestLeavesRights):
         }
         self.request_leave(self.user_hrmanager_id, datetime.today() + relativedelta(days=5), 1, values)
 
-    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
-    def test_holidays_manager_create_validate(self):
-        """ A holidays manager cannot create a leave in validate state """
-        values = {
-            'name': 'Hol10',
-            'employee_id': self.employee_emp_id,
-            'holiday_status_id': self.leave_type.id,
-            'state': 'validate',
-        }
-        with self.assertRaises(ValidationError):
-            self.request_leave(self.user_hrmanager_id, datetime.today() + relativedelta(days=5), 1, values)
-        values.update(employee_id=self.employee_hruser_id)
-        with self.assertRaises(ValidationError):
-            self.request_leave(self.user_hrmanager_id, datetime.today() + relativedelta(days=5), 1, values)
-
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_holidays_manager_create_batch(self):
         """ A holidays manager can create a leave in bacth mode (by company, by department, by tag)"""
@@ -523,7 +467,7 @@ class TestAccessRightsCreate(TestLeavesRights):
 
 
 @tests.tagged('access_rights', 'access_rights_read')
-class TestAccessRightsRead(TestLeavesRights):
+class TestAccessRightsRead(TestHrHolidaysAccessRightsCommon):
     # base.group_user
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
@@ -565,7 +509,7 @@ class TestAccessRightsRead(TestLeavesRights):
 
 
 @tests.tagged('access_rights', 'access_rights_write')
-class TestAccessRightsWrite(TestLeavesRights):
+class TestAccessRightsWrite(TestHrHolidaysAccessRightsCommon):
     # base.group_user
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
@@ -607,7 +551,7 @@ class TestAccessRightsWrite(TestLeavesRights):
         """ User may communicate on its own leaves, even if validated """
         self.employee_leave.with_user(self.user_employee_id).message_post(
             body='I haz messaging',
-            subtype='mail.mt_comment',
+            subtype_xmlid='mail.mt_comment',
             message_type='comment'
         )
 
@@ -615,7 +559,7 @@ class TestAccessRightsWrite(TestLeavesRights):
 
         self.employee_leave.with_user(self.user_employee_id).message_post(
             body='I still haz messaging',
-            subtype='mail.mt_comment',
+            subtype_xmlid='mail.mt_comment',
             message_type='comment'
         )
 
@@ -669,7 +613,7 @@ class TestAccessRightsWrite(TestLeavesRights):
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_leave_manager_to_validate_by_user(self):
         """ A simple user can validate in manager mode if he is leave_manager_id """
-        self.leave_type.write({'validation_type': 'manager'})
+        self.leave_type.write({'leave_validation_type': 'manager'})
         values = {
             'name': 'Hol HrUser',
             'employee_id': self.employee_hruser_id,
@@ -687,7 +631,7 @@ class TestAccessRightsWrite(TestLeavesRights):
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_leave_manager_to_validate_by_holiday_user(self):
         """ A holiday user can validate in manager mode """
-        self.leave_type.write({'validation_type': 'manager'})
+        self.leave_type.write({'leave_validation_type': 'manager'})
         values = {
             'name': 'Hol HrUser',
             'employee_id': self.employee_emp_id,
@@ -703,7 +647,7 @@ class TestAccessRightsWrite(TestLeavesRights):
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_leave_double_validate(self):
-        self.leave_type.write({'validation_type': 'both'})
+        self.leave_type.write({'leave_validation_type': 'both'})
         values = {
             'name': 'double HrManager',
             'employee_id': self.employee_hrmanager_id,
@@ -727,7 +671,7 @@ class TestAccessRightsWrite(TestLeavesRights):
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_leave_double_validate_holiday_manager(self):
-        self.leave_type.write({'validation_type': 'both'})
+        self.leave_type.write({'leave_validation_type': 'both'})
         values = {
             'name': 'double HrManager',
             'employee_id': self.employee_emp_id,
@@ -768,8 +712,58 @@ class TestAccessRightsWrite(TestLeavesRights):
 
     # TODO Can always cancel with great powers comes great responbilities
 
+class TestAccessRightsUnlink(TestHrHolidaysAccessRightsCommon):
 
-class TestMultiCompany(TestHrHolidaysBase):
+    # base.group_user
+
+    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
+    def test_leave_unlink_draft_by_user(self):
+        """ A simple user may delete its leave in draft state in the future"""
+        values = {
+            'name': 'Random Leave',
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.leave_type.id,
+            'state': 'draft',
+        }
+        leave = self.request_leave(self.user_employee_id, datetime.now() + relativedelta(days=6), 1, values)
+        leave.with_user(self.user_employee.id).unlink()
+
+    def test_leave_unlink_confirm_by_user(self):
+        """ A simple user may delete its leave in confirm state in the future"""
+        values = {
+            'name': 'Random Leave',
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.leave_type.id,
+            'state': 'confirm',
+        }
+        leave = self.request_leave(self.user_employee_id, datetime.now() + relativedelta(days=6), 1, values)
+        leave.with_user(self.user_employee.id).unlink()
+
+    def test_leave_unlink_confirm_in_past_by_user(self):
+        """ A simple user cannot delete its leave in the past"""
+        values = {
+            'name': 'Random Leave',
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.leave_type.id,
+            'state': 'confirm',
+        }
+        leave = self.request_leave(self.user_employee_id, datetime.now() + relativedelta(days=-4), 1, values)
+        with self.assertRaises(UserError), self.cr.savepoint():
+            leave.with_user(self.user_employee.id).unlink()
+
+    def test_leave_unlink_validate_by_user(self):
+        """ A simple user cannot delete its leave in validate state"""
+        values = {
+            'name': 'Random Leave',
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.leave_type.id,
+        }
+        leave = self.request_leave(self.user_employee_id, datetime.now() + relativedelta(days=6), 1, values)
+        leave.with_user(self.user_hrmanager_id).write({'state': 'validate'})
+        with self.assertRaises(UserError), self.cr.savepoint():
+            leave.with_user(self.user_employee.id).unlink()
+
+class TestMultiCompany(TestHrHolidaysCommon):
 
     def setUp(self):
         super(TestMultiCompany, self).setUp()
@@ -779,8 +773,8 @@ class TestMultiCompany(TestHrHolidaysBase):
         self.leave_type = self.env['hr.leave.type'].create({
             'name': 'Unlimited - Company New',
             'company_id': self.new_company.id,
-            'validation_type': 'hr',
-            'allocation_type': 'no',
+            'leave_validation_type': 'hr',
+            'requires_allocation': 'no',
         })
         self.rd_dept.manager_id = False
         self.hr_dept.manager_id = False
