@@ -4,11 +4,11 @@
 from datetime import date, datetime
 
 from odoo.tests import tagged
-from odoo.tests.common import SavepointCase
+from odoo.tests.common import TransactionCase
 
 
 @tagged('company_leave')
-class TestCompanyLeave(SavepointCase):
+class TestCompanyLeave(TransactionCase):
     """ Test leaves for a whole company, conflict resolutions """
 
     @classmethod
@@ -20,13 +20,15 @@ class TestCompanyLeave(SavepointCase):
             'name': 'Bank Holiday',
             'responsible_id': cls.env.user.id,
             'company_id': cls.company.id,
+            'requires_allocation': 'no',
         })
 
         cls.paid_time_off = cls.env['hr.leave.type'].create({
             'name': 'Paid Time Off',
             'request_unit': 'day',
-            'validation_type': 'both',
+            'leave_validation_type': 'both',
             'company_id': cls.company.id,
+            'requires_allocation': 'no',
         })
 
         cls.employee = cls.env['hr.employee'].create({
@@ -49,7 +51,7 @@ class TestCompanyLeave(SavepointCase):
             'date_to': date(2020, 1, 9),
             'number_of_days': 3,
         })
-        leave._onchange_request_parameters()
+        leave._compute_date_from_to()
 
         company_leave = self.env['hr.leave'].create({
             'name': 'Bank Holiday',
@@ -62,7 +64,7 @@ class TestCompanyLeave(SavepointCase):
             'request_date_to': date(2020, 1, 8),
             'number_of_days': 1,
         })
-        company_leave._onchange_request_parameters()
+        company_leave._compute_date_from_to()
 
         company_leave.action_validate()
 
@@ -103,7 +105,7 @@ class TestCompanyLeave(SavepointCase):
             'date_to': date(2020, 1, 9),
             'number_of_days': 3,
         })
-        leave._onchange_request_parameters()
+        leave._compute_date_from_to()
 
         company_leave = self.env['hr.leave'].create({
             'name': 'Bank Holiday',
@@ -116,7 +118,7 @@ class TestCompanyLeave(SavepointCase):
             'request_date_to': date(2020, 1, 8),
             'number_of_days': 1,
         })
-        company_leave._onchange_request_parameters()
+        company_leave._compute_date_from_to()
 
         company_leave.action_validate()
 
@@ -157,7 +159,7 @@ class TestCompanyLeave(SavepointCase):
             'request_date_from_period': 'am',
 
         })
-        leave._onchange_request_parameters()
+        leave._compute_date_from_to()
 
         company_leave = self.env['hr.leave'].create({
             'name': 'Bank Holiday',
@@ -170,7 +172,7 @@ class TestCompanyLeave(SavepointCase):
             'request_date_to': date(2020, 1, 7),
             'number_of_days': 1,
         })
-        company_leave._onchange_request_parameters()
+        company_leave._compute_date_from_to()
 
         company_leave.action_validate()
 
@@ -199,7 +201,7 @@ class TestCompanyLeave(SavepointCase):
             'number_of_days': 1,
 
         })
-        leave._onchange_request_parameters()
+        leave._compute_date_from_to()
 
         company_leave = self.env['hr.leave'].create({
             'name': 'Bank Holiday',
@@ -212,7 +214,7 @@ class TestCompanyLeave(SavepointCase):
             'request_date_to': date(2020, 1, 9),
             'number_of_days': 1,
         })
-        company_leave._onchange_request_parameters()
+        company_leave._compute_date_from_to()
 
         company_leave.action_validate()
 
@@ -249,7 +251,7 @@ class TestCompanyLeave(SavepointCase):
             'request_date_to': date(2020, 1, 10),
             'number_of_days': 3,
         })
-        leave._onchange_request_parameters()
+        leave._compute_date_from_to()
 
         company_leave = self.env['hr.leave'].create({
             'name': 'Bank Holiday',
@@ -262,7 +264,7 @@ class TestCompanyLeave(SavepointCase):
             'request_date_to': date(2020, 1, 10),
             'number_of_days': 1,
         })
-        company_leave._onchange_request_parameters()
+        company_leave._compute_date_from_to()
         company_leave.action_validate()
 
         all_leaves = self.env['hr.leave'].search([('employee_id', '=', self.employee.id)], order='id')
@@ -300,8 +302,7 @@ class TestCompanyLeave(SavepointCase):
             'date_to': datetime(2020, 4, 1, 19, 0, 0),
             'number_of_days': 3,
         } for employee in employees[0:15]])
-        for leave in leaves:
-            leave._onchange_request_parameters()
+        leaves._compute_date_from_to()
 
         company_leave = self.env['hr.leave'].create({
             'name': 'Bank Holiday',
@@ -314,9 +315,9 @@ class TestCompanyLeave(SavepointCase):
             'request_date_to': date(2020, 4, 1),
             'number_of_days': 1,
         })
-        company_leave._onchange_request_parameters()
+        company_leave._compute_date_from_to()
 
-        with self.assertQueryCount(__system__=846, admin=845):
+        with self.assertQueryCount(__system__=771, admin=865):  # 770 community
             # Original query count: 1987
             # Without tracking/activity context keys: 5154
             company_leave.action_validate()
