@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from datetime import date, datetime
+from datetime import datetime, date
+import time
 from dateutil.relativedelta import relativedelta
+from freezegun import freeze_time
 from pytz import timezone, UTC
 
 from odoo import fields
@@ -10,10 +12,11 @@ from odoo.exceptions import ValidationError
 from odoo.tools import mute_logger
 from odoo.tests.common import Form
 from odoo.tests import tagged
-from odoo.addons.hr_holidays.tests.common import TestHrHolidaysBase
+
+from odoo.addons.hr_holidays.tests.common import TestHrHolidaysCommon
 
 @tagged('leave_requests')
-class TestLeaveRequests(TestHrHolidaysBase):
+class TestLeaveRequests(TestHrHolidaysCommon):
 
     def _check_holidays_status(self, holiday_status, ml, lt, rl, vrl):
             self.assertEqual(holiday_status.max_leaves, ml,
@@ -33,22 +36,19 @@ class TestLeaveRequests(TestHrHolidaysBase):
 
         self.holidays_type_1 = LeaveType.create({
             'name': 'NotLimitedHR',
-            'allocation_type': 'no',
-            'validation_type': 'hr',
-            'validity_start': False,
+            'requires_allocation': 'no',
+            'leave_validation_type': 'hr',
         })
         self.holidays_type_2 = LeaveType.create({
             'name': 'Limited',
-            'allocation_type': 'fixed',
-            'validation_type': 'hr',
-            'validity_start': False,
+            'requires_allocation': 'yes',
+            'employee_requests': 'yes',
+            'leave_validation_type': 'hr',
         })
         self.holidays_type_3 = LeaveType.create({
             'name': 'TimeNotLimited',
-            'allocation_type': 'no',
-            'validation_type': 'manager',
-            'validity_start': fields.Datetime.from_string('2017-01-01 00:00:00'),
-            'validity_stop': fields.Datetime.from_string('2017-06-01 00:00:00'),
+            'requires_allocation': 'no',
+            'leave_validation_type': 'manager',
         })
 
         self.set_employee_create_date(self.employee_emp_id, '2010-02-03 00:00:00')
@@ -105,41 +105,56 @@ class TestLeaveRequests(TestHrHolidaysBase):
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_limited_type_days_left(self):
         """  Employee creates a leave request in a limited category and has enough days left  """
-        aloc1_user_group = self.env['hr.leave.allocation'].with_user(self.user_hruser_id).create({
-            'name': 'Days for limited category',
-            'employee_id': self.employee_emp_id,
-            'holiday_status_id': self.holidays_type_2.id,
-            'number_of_days': 2,
-        })
-        aloc1_user_group.action_approve()
+        with freeze_time('2022-01-05'):
+            aloc1_user_group = self.env['hr.leave.allocation'].with_user(self.user_hruser_id).create({
+                'name': 'Days for limited category',
+                'employee_id': self.employee_emp_id,
+                'holiday_status_id': self.holidays_type_2.id,
+                'number_of_days': 2,
+                'state': 'confirm',
+                'date_from': time.strftime('%Y-1-1'),
+                'date_to': time.strftime('%Y-12-31'),
+            })
+            aloc1_user_group.action_validate()
 
-        holiday_status = self.holidays_type_2.with_user(self.user_employee_id)
-        self._check_holidays_status(holiday_status, 2.0, 0.0, 2.0, 2.0)
+            holiday_status = self.holidays_type_2.with_user(self.user_employee_id)
+            self._check_holidays_status(holiday_status, 2.0, 0.0, 2.0, 2.0)
 
-        hol = self.env['hr.leave'].with_user(self.user_employee_id).create({
-            'name': 'Hol11',
-            'employee_id': self.employee_emp_id,
-            'holiday_status_id': self.holidays_type_2.id,
-            'date_from': (datetime.today() - relativedelta(days=2)),
-            'date_to': datetime.today(),
-            'number_of_days': 2,
-        })
+            hol = self.env['hr.leave'].with_user(self.user_employee_id).create({
+                'name': 'Hol11',
+                'employee_id': self.employee_emp_id,
+                'holiday_status_id': self.holidays_type_2.id,
+                'date_from': (datetime.today() - relativedelta(days=2)),
+                'date_to': datetime.today(),
+                'number_of_days': 2,
+            })
 
-        holiday_status.invalidate_cache()
-        self._check_holidays_status(holiday_status, 2.0, 0.0, 2.0, 0.0)
+            holiday_status.invalidate_cache()
+            self._check_holidays_status(holiday_status, 2.0, 0.0, 2.0, 0.0)
 
-        hol.with_user(self.user_hrmanager_id).action_approve()
+            hol.with_user(self.user_hrmanager_id).action_approve()
 
-        holiday_status.invalidate_cache(['max_leaves'])
-        self._check_holidays_status(holiday_status, 2.0, 2.0, 0.0, 0.0)
+            holiday_status.invalidate_cache(['max_leaves'])
+            self._check_holidays_status(holiday_status, 2.0, 2.0, 0.0, 0.0)
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_accrual_validity_time_valid(self):
         """  Employee ask leave during a valid validity time """
+
+        self.env['hr.leave.allocation'].with_user(self.user_hrmanager_id).create({
+                'name': 'Sick Time Off',
+                'holiday_status_id': self.holidays_type_2.id,
+                'employee_id': self.employee_emp.id,
+                'date_from': fields.Datetime.from_string('2017-01-01 00:00:00'),
+                'date_to': fields.Datetime.from_string('2017-06-01 00:00:00'),
+                'number_of_days': 10,
+                'state': 'validate',
+        })
+
         self.env['hr.leave'].with_user(self.user_employee_id).create({
             'name': 'Valid time period',
             'employee_id': self.employee_emp_id,
-            'holiday_status_id': self.holidays_type_3.id,
+            'holiday_status_id': self.holidays_type_2.id,
             'date_from': fields.Datetime.from_string('2017-03-03 06:00:00'),
             'date_to': fields.Datetime.from_string('2017-03-11 19:00:00'),
             'number_of_days': 1,
@@ -147,12 +162,12 @@ class TestLeaveRequests(TestHrHolidaysBase):
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
     def test_accrual_validity_time_not_valid(self):
-        """  Employee ask leav during a not valid validity time """
+        """  Employee ask leave when there's no valid allocation """
         with self.assertRaises(ValidationError):
-            self.env['hr.leave'].with_user(self.user_employee_id).create({
+            self.env['hr.leave'].with_user(self.user_employee).create({
                 'name': 'Sick Time Off',
                 'employee_id': self.employee_emp_id,
-                'holiday_status_id': self.holidays_type_3.id,
+                'holiday_status_id': self.holidays_type_2.id,
                 'date_from': fields.Datetime.from_string('2017-07-03 06:00:00'),
                 'date_to': fields.Datetime.from_string('2017-07-11 19:00:00'),
                 'number_of_days': 1,
@@ -167,6 +182,8 @@ class TestLeaveRequests(TestHrHolidaysBase):
         leave_form.holiday_type = 'department'
         leave_form.department_id = self.hr_dept
         leave_form.holiday_status_id = self.holidays_type_1
+        leave_form.request_date_from = date(2019, 5, 6)
+        leave_form.request_date_to = date(2019, 5, 6)
         leave = leave_form.save()
         leave.action_approve()
         member_ids = self.hr_dept.member_ids.ids
@@ -177,8 +194,10 @@ class TestLeaveRequests(TestHrHolidaysBase):
         """ Create an allocation request """
         # employee should be set to current user
         allocation_form = Form(self.env['hr.leave.allocation'].with_user(self.user_employee))
-        allocation_form.holiday_status_id = self.holidays_type_1
         allocation_form.name = 'New Allocation Request'
+        allocation_form.holiday_status_id = self.holidays_type_2
+        allocation_form.date_from = date(2019, 5, 6)
+        allocation_form.date_to = date(2019, 5, 6)
         allocation = allocation_form.save()
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
@@ -196,6 +215,82 @@ class TestLeaveRequests(TestHrHolidaysBase):
         self.assertTrue(self.employee_emp.is_absent, "He should be considered absent")
         self.assertFalse(self.employee_hrmanager.is_absent, "He should not be considered absent")
 
+    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
+    def test_timezone_employee_leave_request(self):
+        """ Create a leave request for an employee in another timezone """
+        self.employee_emp.tz = 'NZ'  # GMT+12
+        leave = self.env['hr.leave'].new({
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.holidays_type_1.id,
+            'request_unit_hours': True,
+            'request_date_from': date(2019, 5, 6),
+            'request_date_to': date(2019, 5, 6),
+            'request_hour_from': '8',  # 8:00 AM in the employee's timezone
+            'request_hour_to': '17',  # 5:00 PM in the employee's timezone
+        })
+        self.assertEqual(leave.date_from, datetime(2019, 5, 5, 20, 0, 0), "It should have been localized before saving in UTC")
+        self.assertEqual(leave.date_to, datetime(2019, 5, 6, 5, 0, 0), "It should have been localized before saving in UTC")
+
+    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
+    def test_timezone_company_leave_request(self):
+        """ Create a leave request for a company in another timezone """
+        company = self.env['res.company'].create({'name': "Hergé"})
+        company.resource_calendar_id.tz = 'NZ'  # GMT+12
+        leave = self.env['hr.leave'].new({
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.holidays_type_1.id,
+            'request_unit_hours': True,
+            'holiday_type': 'company',
+            'mode_company_id': company.id,
+            'request_date_from': date(2019, 5, 6),
+            'request_date_to': date(2019, 5, 6),
+            'request_hour_from': '8',  # 8:00 AM in the company's timezone
+            'request_hour_to': '17',  # 5:00 PM in the company's timezone
+        })
+        self.assertEqual(leave.date_from, datetime(2019, 5, 5, 20, 0, 0), "It should have been localized before saving in UTC")
+        self.assertEqual(leave.date_to, datetime(2019, 5, 6, 5, 0, 0), "It should have been localized before saving in UTC")
+
+    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
+    def test_timezone_company_validated(self):
+        """ Create a leave request for a company in another timezone and validate it """
+        self.env.user.tz = 'NZ' # GMT+12
+        company = self.env['res.company'].create({'name': "Hergé"})
+        employee = self.env['hr.employee'].create({'name': "Remi", 'company_id': company.id})
+        leave_form = Form(self.env['hr.leave'], view='hr_holidays.hr_leave_view_form_manager')
+        leave_form.holiday_type = 'company'
+        leave_form.mode_company_id = company
+        leave_form.holiday_status_id = self.holidays_type_1
+        leave_form.request_date_from = date(2019, 5, 6)
+        leave_form.request_date_to = date(2019, 5, 6)
+        leave = leave_form.save()
+        leave.state = 'confirm'
+        leave.action_validate()
+        employee_leave = self.env['hr.leave'].search([('employee_id', '=', employee.id)])
+        self.assertEqual(
+            employee_leave.request_date_from, date(2019, 5, 6),
+            "Timezone should be kept between company and employee leave"
+        )
+
+    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
+    def test_timezone_department_leave_request(self):
+        """ Create a leave request for a department in another timezone """
+        company = self.env['res.company'].create({'name': "Hergé"})
+        company.resource_calendar_id.tz = 'NZ'  # GMT+12
+        department = self.env['hr.department'].create({'name': "Museum", 'company_id': company.id})
+        leave = self.env['hr.leave'].new({
+            'employee_id': self.employee_emp.id,
+            'holiday_status_id': self.holidays_type_1.id,
+            'request_unit_hours': True,
+            'holiday_type': 'department',
+            'department_id': department.id,
+            'request_date_from': date(2019, 5, 6),
+            'request_date_to': date(2019, 5, 6),
+            'request_hour_from': '8',  # 8:00 AM in the department's timezone
+            'request_hour_to': '17',  # 5:00 PM in the department's timezone
+        })
+        self.assertEqual(leave.date_from, datetime(2019, 5, 5, 20, 0, 0), "It should have been localized before saving in UTC")
+        self.assertEqual(leave.date_to, datetime(2019, 5, 6, 5, 0, 0), "It should have been localized before saving in UTC")
+
     def test_number_of_hours_display(self):
         # Test that the field number_of_hours_dispay doesn't change
         # after time off validation, as it takes the attendances
@@ -221,15 +316,18 @@ class TestLeaveRequests(TestHrHolidaysBase):
         leave_type = self.env['hr.leave.type'].create({
             'name': 'Paid Time Off',
             'request_unit': 'hour',
-            'validation_type': 'both',
+            'leave_validation_type': 'both',
         })
         allocation = self.env['hr.leave.allocation'].create({
             'name': '20 days allocation',
             'holiday_status_id': leave_type.id,
             'number_of_days': 20,
             'employee_id': employee.id,
+            'state': 'confirm',
+            'date_from': time.strftime('2018-1-1'),
+            'date_to': time.strftime('%Y-1-1'),
         })
-        allocation.action_approve()
+        allocation.action_validate()
 
         leave1 = self.env['hr.leave'].create({
             'name': 'Holiday 1 week',
@@ -294,8 +392,8 @@ class TestLeaveRequests(TestHrHolidaysBase):
         leave_type = self.env['hr.leave.type'].create({
             'name': 'Sick',
             'request_unit': 'hour',
-            'validation_type': 'both',
-            'allocation_type': 'no',
+            'leave_validation_type': 'both',
+            'requires_allocation': 'no',
         })
         leave1 = self.env['hr.leave'].create({
             'name': 'Sick 1 week during christmas snif',
@@ -335,7 +433,6 @@ class TestLeaveRequests(TestHrHolidaysBase):
             'holiday_status_id': self.holidays_type_1.id,
         })
         leave = self.env['hr.leave'].with_user(self.user_employee_id).new(values)
-        leave._onchange_request_parameters()
         self.assertEqual(leave.number_of_days, number_of_days)
 
     @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
@@ -350,93 +447,237 @@ class TestLeaveRequests(TestHrHolidaysBase):
         # 12 13 14 15 16 17 18
         # 19 20 21 22 23 24 25
         # 26 27 28 29 30 31
-        local_date_from = datetime(2019, 1, 1, 7, 0, 0)
-        local_date_to = datetime(2019, 1, 1, 19, 0, 0)
+        local_date_from = datetime(2020, 1, 1, 7, 0, 0)
+        local_date_to = datetime(2020, 1, 1, 19, 0, 0)
         for tz in timezones_to_test:
             self._test_leave_with_tz(tz, local_date_from, local_date_to, 1)
 
         # We, Th, Fr, Mo, Tu, We => 6 days
-        local_date_from = datetime(2019, 1, 1, 7, 0, 0)
-        local_date_to = datetime(2019, 1, 8, 19, 0, 0)
+        local_date_from = datetime(2020, 1, 1, 7, 0, 0)
+        local_date_to = datetime(2020, 1, 8, 19, 0, 0)
         for tz in timezones_to_test:
-            self._test_leave_with_tz(tz, local_date_from, local_date_to, 6)
+            self._test_leave_with_tz(tz, local_date_from, local_date_to, 6)#TODO JUD check why this fails
 
-    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
-    def test_timezone_employee_leave_request(self):
-        """ Create a leave request for an employee in another timezone """
-        self.employee_emp.tz = 'NZ'  # GMT+12
-        leave = self.env['hr.leave'].new({
-            'employee_id': self.employee_emp.id,
-            'holiday_type': "employee",
-            'holiday_status_id': self.holidays_type_1.id,
-            'request_unit_hours': True,
-            'request_date_from': date(2019, 5, 6),
-            'request_date_to': date(2019, 5, 6),
-            'request_hour_from': '8',  # 8:00 AM in the employee's timezone
-            'request_hour_to': '17',  # 5:00 PM in the employee's timezone
+    def test_expired_allocation(self):
+        allocation = self.env['hr.leave.allocation'].create({
+            'name': 'Expired Allocation',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+            'number_of_days': 20,
+            'state': 'confirm',
+            'date_from': '2020-01-01',
+            'date_to': '2020-12-31',
         })
-        leave._onchange_request_parameters()
-        self.assertEqual(leave.date_from, datetime(2019, 5, 5, 20, 0, 0), "It should have been localized before saving in UTC")
-        self.assertEqual(leave.date_to, datetime(2019, 5, 6, 5, 0, 0), "It should have been localized before saving in UTC")
+        allocation.action_validate()
 
-    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
-    def test_timezone_company_leave_request(self):
-        """ Create a leave request for a company in another timezone """
-        company = self.env['res.company'].create({'name': "Hergé"})
-        company.resource_calendar_id.tz = 'NZ'  # GMT+12
-        leave = self.env['hr.leave'].new({
-            'employee_id': self.employee_emp.id,
-            'holiday_status_id': self.holidays_type_1.id,
-            'request_unit_hours': True,
+        with self.assertRaises(ValidationError):
+            self.env['hr.leave'].with_user(self.user_employee_id).create({
+                'name': 'Holiday Request',
+                'employee_id': self.employee_emp_id,
+                'holiday_status_id': self.holidays_type_2.id,
+                'date_from': '2021-09-01',
+                'date_to': '2021-09-02',
+                'number_of_days': 1,
+            })
+        self.env['hr.leave'].with_user(self.user_employee_id).create({
+            'name': 'Holiday Request',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+            'date_from': '2020-09-01',
+            'date_to': '2020-09-02',
+            'number_of_days': 1,
+        })
+
+    def test_no_days_expired(self):
+        # First expired allocation
+        allocation1 = self.env['hr.leave.allocation'].create({
+            'name': 'Expired Allocation',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+            'number_of_days': 20,
+            'state': 'confirm',
+            'date_from': '2020-01-01',
+            'date_to': '2020-12-31',
+        })
+        allocation2 = self.env['hr.leave.allocation'].create({
+            'name': 'Expired Allocation',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+            'number_of_days': 3,
+            'state': 'confirm',
+            'date_from': '2021-01-01',
+            'date_to': '2021-12-31',
+        })
+        allocation1.action_validate()
+        allocation2.action_validate()
+        # Try creating a request that could be validated if allocation1 was still valid
+        with self.assertRaises(ValidationError):
+            self.env['hr.leave'].with_user(self.user_employee_id).create({
+                'name': 'Holiday Request',
+                'employee_id': self.employee_emp_id,
+                'holiday_status_id': self.holidays_type_2.id,
+                'date_from': '2021-09-06',
+                'date_to': '2021-09-10',
+                'number_of_days': 5,
+            })
+        # This time we have enough days
+        self.env['hr.leave'].with_user(self.user_employee_id).create({
+            'name': 'Holiday Request',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+            'date_from': '2021-09-06',
+            'date_to': '2021-09-08',
+            'number_of_days': 3,
+        })
+
+    def test_company_leaves(self):
+        # First expired allocation
+        allocation = self.env['hr.leave.allocation'].create({
+            'name': 'Allocation',
             'holiday_type': 'company',
-            'mode_company_id': company.id,
-            'request_date_from': date(2019, 5, 6),
-            'request_date_to': date(2019, 5, 6),
-            'request_hour_from': '8',  # 8:00 AM in the company's timezone
-            'request_hour_to': '17',  # 5:00 PM in the company's timezone
+            'mode_company_id': self.env.company.id,
+            'holiday_status_id': self.holidays_type_1.id,
+            'number_of_days': 20,
+            'state': 'confirm',
+            'date_from': '2021-01-01',
         })
-        leave._onchange_request_parameters()
-        self.assertEqual(leave.date_from, datetime(2019, 5, 5, 20, 0, 0), "It should have been localized before saving in UTC")
-        self.assertEqual(leave.date_to, datetime(2019, 5, 6, 5, 0, 0), "It should have been localized before saving in UTC")
+        allocation.action_validate()
 
-    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
-    def test_timezone_department_validated(self):
-        """ Create a department leave """
-        self.env.user.tz = 'NZ' # GMT+12
-        company = self.env['res.company'].create({'name': "Hergé"})
-        employee = self.env['hr.employee'].create({'name': "Remi", 'company_id': company.id})
-        leave_form = Form(self.env['hr.leave'], view='hr_holidays.hr_leave_view_form_manager')
-        leave_form.holiday_type = 'company'
-        leave_form.mode_company_id = company
-        leave_form.holiday_status_id = self.holidays_type_1
-        leave_form.request_date_from = date(2019, 5, 6)
-        leave_form.request_date_to = date(2019, 5, 6)
-        leave = leave_form.save()
-        leave.state = 'confirm'
-        leave.action_validate()
-        employee_leave = self.env['hr.leave'].search([('employee_id', '=', employee.id)])
-        self.assertEqual(
-            employee_leave.request_date_from, date(2019, 5, 6),
-            "Timezone should be kept between company and employee leave"
-        )
+        req1_form = Form(self.env['hr.leave'].sudo())
+        req1_form.employee_ids.add(self.employee_emp)
+        req1_form.employee_ids.add(self.employee_hrmanager)
+        req1_form.holiday_status_id = self.holidays_type_1
+        req1_form.request_date_from = fields.Date.to_date('2021-12-06')
+        req1_form.request_date_to = fields.Date.to_date('2021-12-08')
 
-    @mute_logger('odoo.models.unlink', 'odoo.addons.mail.models.mail_mail')
-    def test_timezone_department_leave_request(self):
-        """ Create a leave request for a department in another timezone """
-        company = self.env['res.company'].create({'name': "Hergé"})
-        company.resource_calendar_id.tz = 'NZ'  # GMT+12
-        department = self.env['hr.department'].create({'name': "Museum", 'company_id': company.id})
-        leave = self.env['hr.leave'].new({
+        self.assertEqual(req1_form.number_of_days, 3)
+        req1_form.save().action_approve()
+
+        req2_form = Form(self.env['hr.leave'].sudo())
+        req2_form.employee_ids.add(self.employee_hruser)
+        req2_form.holiday_status_id = self.holidays_type_1
+        req2_form.request_date_from = fields.Date.to_date('2021-12-06')
+        req2_form.request_date_to = fields.Date.to_date('2021-12-08')
+
+        self.assertEqual(req2_form.number_of_days, 3)
+
+    def test_leave_with_public_holiday_other_company(self):
+        other_company = self.env['res.company'].create({
+            'name': 'Test Company',
+        })
+        # Create a public holiday for the second company
+        p_leave = self.env['resource.calendar.leaves'].create({
+            'date_from': datetime(2022, 3, 11),
+            'date_to': datetime(2022, 3, 11, 23, 59, 59),
+        })
+        p_leave.company_id = other_company
+
+        leave = self.env['hr.leave'].with_user(self.user_employee_id).create({
+            'name': 'Holiday Request',
+            'holiday_type': 'employee',
             'employee_id': self.employee_emp.id,
             'holiday_status_id': self.holidays_type_1.id,
-            'request_unit_hours': True,
-            'holiday_type': 'department',
-            'department_id': department.id,
-            'request_date_from': date(2019, 5, 6),
-            'request_date_to': date(2019, 5, 6),
-            'request_hour_from': '8',  # 8:00 AM in the department's timezone
-            'request_hour_to': '17',  # 5:00 PM in the department's timezone
+            'date_from': datetime(2022, 3, 11),
+            'date_to': datetime(2022, 3, 11, 23, 59, 59),
         })
-        leave._onchange_request_parameters()
-        self.assertEqual(leave.date_from, datetime(2019, 5, 5, 20, 0, 0), "It should have been localized before saving in UTC")
-        self.assertEqual(leave.date_to, datetime(2019, 5, 6, 5, 0, 0), "It should have been localized before saving in UTC")
+        self.assertEqual(leave.number_of_days, 1)
+
+    def test_several_allocations(self):
+        allocation_vals = {
+            'name': 'Allocation',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+            'number_of_days': 5,
+            'state': 'confirm',
+            'date_from': '2022-01-01',
+            'date_to': '2022-12-31',
+        }
+        allocation1 = self.env['hr.leave.allocation'].create(allocation_vals)
+        allocation2 = self.env['hr.leave.allocation'].create(allocation_vals)
+
+        allocation1.action_validate()
+        allocation2.action_validate()
+
+        # Neither allocation has enough days
+        with self.assertRaises(ValidationError):
+            self.env['hr.leave'].with_user(self.user_employee_id).create({
+                'name': 'Holiday Request',
+                'employee_id': self.employee_emp_id,
+                'holiday_status_id': self.holidays_type_2.id,
+                'date_from': '2022-01-01',
+                'date_to': '2022-01-15',
+                'number_of_days': 10,
+            })
+
+        allocation_vals.update({'number_of_days': 10})
+        allocation3 = self.env['hr.leave.allocation'].create(allocation_vals)
+        allocation3.action_validate()
+
+        # The 3rd allocation has enough days, so it should be picked
+        holiday = self.env['hr.leave'].with_user(self.user_employee_id).create({
+            'name': 'Holiday Request',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+            'date_from': '2022-01-01',
+            'date_to': '2022-01-15',
+            'number_of_days': 10,
+        })
+        holiday.sudo().action_validate()
+
+        self.assertEqual(allocation1.leaves_taken, 0.0, 'As this allocation does not have enough days, it should not be affected')
+        self.assertEqual(allocation2.leaves_taken, 0.0, 'As this allocation does not have enough days, it should not be affected')
+        self.assertEqual(allocation3.leaves_taken, 10.0, 'As this allocation has enough days, the leave days should be taken')
+
+    def test_several_allocations_split(self):
+        Allocation = self.env['hr.leave.allocation']
+        allocation_vals = {
+            'name': 'Allocation',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+            'state': 'confirm',
+            'date_from': '2022-01-01',
+            'date_to': '2022-12-31',
+        }
+        Leave = self.env['hr.leave'].with_user(self.user_employee_id).sudo()
+        leave_vals = {
+            'name': 'Holiday Request',
+            'employee_id': self.employee_emp_id,
+            'holiday_status_id': self.holidays_type_2.id,
+        }
+
+        for unit in ['hour', 'day']:
+            self.holidays_type_2.request_unit = unit
+
+            allocation_vals.update({'number_of_days': 4})
+            allocation_4days = Allocation.create(allocation_vals)
+            allocation_vals.update({'number_of_days': 1})
+            allocation_1day = Allocation.create(allocation_vals)
+            allocations = (allocation_4days + allocation_1day)
+            allocations.action_validate()
+
+            leave_vals.update({
+                'date_from': '2022-01-03 00:00:00',
+                'date_to': '2022-01-06 23:59:59',
+                'number_of_days': 4,
+            })
+            leave_draft = Leave.create(leave_vals)
+            leave_draft.action_refuse()
+            leave_vals.update({
+                'date_from': '2022-01-03 00:00:00',
+                'date_to': '2022-01-06 23:59:59',
+                'number_of_days': 4,
+            })
+            leave_4days = Leave.create(leave_vals)
+            leave_vals.update({
+                'date_from': '2022-01-07 00:00:00',
+                'date_to': '2022-01-07 23:59:59',
+                'number_of_days': 1,
+            })
+            leave_1day = Leave.create(leave_vals)
+            leaves = (leave_4days + leave_1day)
+            leaves.action_approve()
+
+            self.assertEqual(allocation_4days.leaves_taken, leave_4days['number_of_%ss_display' % unit], 'As 4 days were available in this allocation, they should have been taken')
+            self.assertEqual(allocation_1day.leaves_taken, leave_1day['number_of_%ss_display' % unit], 'As no days were available in previous allocation, they should have been taken in this one')
+            leaves.action_refuse()
+            allocations.action_refuse()
