@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import api, fields, models, SUPERUSER_ID
+from random import randint
+
+from odoo import api, fields, models, tools, SUPERUSER_ID
 from odoo.tools.translate import _
 from odoo.exceptions import UserError
 
@@ -20,7 +22,7 @@ class RecruitmentSource(models.Model):
 
     source_id = fields.Many2one('utm.source', "Source", ondelete='cascade', required=True)
     email = fields.Char(related='alias_id.display_name', string="Email", readonly=True)
-    job_id = fields.Many2one('hr.job', "Job ID")
+    job_id = fields.Many2one('hr.job', "Job", ondelete='cascade')
     alias_id = fields.Many2one('mail.alias', "Alias ID")
 
     def create_alias(self):
@@ -29,6 +31,8 @@ class RecruitmentSource(models.Model):
         for source in self:
             vals = {
                 'alias_parent_thread_id': source.job_id.id,
+                'alias_model_id': self.env['ir.model']._get('hr.applicant').id,
+                'alias_parent_model_id': self.env['ir.model']._get('hr.job').id,
                 'alias_name': "%s+%s" % (source.job_id.alias_name or source.job_id.name, source.name),
                 'alias_defaults': {
                     'job_id': source.job_id.id,
@@ -37,10 +41,9 @@ class RecruitmentSource(models.Model):
                     'source_id': source.source_id.id,
                 },
             }
-            source.alias_id = self.with_context(alias_model_name='hr.applicant', alias_parent_model_name='hr.job').env['mail.alias'].create(vals)
+            source.alias_id = self.env['mail.alias'].create(vals)
             source.name = source.source_id.name
 
-
 class RecruitmentStage(models.Model):
     _name = "hr.recruitment.stage"
     _description = "Recruitment Stages"
@@ -60,12 +63,15 @@ class RecruitmentStage(models.Model):
     fold = fields.Boolean(
         "Folded in Kanban",
         help="This stage is folded in the kanban view when there are no records in that stage to display.")
+    hired_stage = fields.Boolean('Hired Stage',
+        help="If checked, this stage is used to determine the hire date of an applicant")
     legend_blocked = fields.Char(
         'Red Kanban Label', default=lambda self: _('Blocked'), translate=True, required=True)
     legend_done = fields.Char(
         'Green Kanban Label', default=lambda self: _('Ready for Next Stage'), translate=True, required=True)
     legend_normal = fields.Char(
         'Grey Kanban Label', default=lambda self: _('In Progress'), translate=True, required=True)
+    is_warning_visible = fields.Boolean(compute='_compute_is_warning_visible')
 
     @api.model
     def default_get(self, fields):
@@ -75,6 +81,15 @@ class RecruitmentStage(models.Model):
             self = self.with_context(context)
         return super(RecruitmentStage, self).default_get(fields)
 
+    @api.depends('hired_stage')
+    def _compute_is_warning_visible(self):
+        applicant_data = self.env['hr.applicant'].read_group([('stage_id', 'in', self.ids)], ['stage_id'], 'stage_id')
+        applicants = dict((data['stage_id'][0], data['stage_id_count']) for data in applicant_data)
+        for stage in self:
+            if stage._origin.hired_stage and not stage.hired_stage and applicants.get(stage._origin.id):
+                stage.is_warning_visible = True
+            else:
+                stage.is_warning_visible = False
 
 class RecruitmentDegree(models.Model):
     _name = "hr.recruitment.degree"
@@ -92,66 +107,53 @@ class Applicant(models.Model):
     _description = "Applicant"
     _order = "priority desc, id desc"
     _inherit = ['mail.thread.cc', 'mail.activity.mixin', 'utm.mixin']
+    _mailing_enabled = True
 
-    def _default_stage_id(self):
-        if self._context.get('default_job_id'):
-            return self.env['hr.recruitment.stage'].search([
-                '|',
-                ('job_ids', '=', False),
-                ('job_ids', '=', self._context['default_job_id']),
-                ('fold', '=', False)
-            ], order='sequence asc', limit=1).id
-        return False
-
-    def _default_company_id(self):
-        company_id = False
-        if self._context.get('default_department_id'):
-            department = self.env['hr.department'].browse(self._context['default_department_id'])
-            company_id = department.company_id.id
-        if not company_id and self.job_id:
-            company_id = self.env['hr.job'].browse(self._context['default_job_id']).company_id.ids
-        if not company_id:
-            company_id = self.env.company
-        return company_id
-
-    name = fields.Char("Subject / Application Name", required=True)
+    name = fields.Char("Subject / Application Name", required=True, help="Email subject for applications sent via email")
     active = fields.Boolean("Active", default=True, help="If the active field is set to false, it will allow you to hide the case without removing it.")
-    description = fields.Text("Description")
-    email_from = fields.Char("Email", size=128, help="Applicant email")
+    description = fields.Html("Description")
+    email_from = fields.Char("Email", size=128, help="Applicant email", compute='_compute_partner_phone_email',
+        inverse='_inverse_partner_email', store=True)
     probability = fields.Float("Probability")
     partner_id = fields.Many2one('res.partner', "Contact", copy=False)
     create_date = fields.Datetime("Creation Date", readonly=True, index=True)
     stage_id = fields.Many2one('hr.recruitment.stage', 'Stage', ondelete='restrict', tracking=True,
+                               compute='_compute_stage', store=True, readonly=False,
                                domain="['|', ('job_ids', '=', False), ('job_ids', '=', job_id)]",
                                copy=False, index=True,
-                               group_expand='_read_group_stage_ids',
-                               default=_default_stage_id)
+                               group_expand='_read_group_stage_ids')
     last_stage_id = fields.Many2one('hr.recruitment.stage', "Last Stage",
                                     help="Stage of the applicant before being in the current stage. Used for lost cases analysis.")
     categ_ids = fields.Many2many('hr.applicant.category', string="Tags")
-    company_id = fields.Many2one('res.company', "Company", default=_default_company_id)
-    user_id = fields.Many2one('res.users', "Responsible", tracking=True, default=lambda self: self.env.uid)
-    date_closed = fields.Datetime("Closed", readonly=True, index=True)
+    company_id = fields.Many2one('res.company', "Company", compute='_compute_company', store=True, readonly=False, tracking=True)
+    user_id = fields.Many2one(
+        'res.users', "Recruiter", compute='_compute_user',
+        tracking=True, store=True, readonly=False)
+    date_closed = fields.Datetime("Hire Date", compute='_compute_date_closed', store=True, index=True, readonly=False, tracking=True)
     date_open = fields.Datetime("Assigned", readonly=True, index=True)
     date_last_stage_update = fields.Datetime("Last Stage Update", index=True, default=fields.Datetime.now)
     priority = fields.Selection(AVAILABLE_PRIORITIES, "Appreciation", default='0')
-    job_id = fields.Many2one('hr.job', "Applied Job", domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]")
-    salary_proposed_extra = fields.Char("Proposed Salary Extra", help="Salary Proposed by the Organisation, extra advantages")
-    salary_expected_extra = fields.Char("Expected Salary Extra", help="Salary Expected by Applicant, extra advantages")
-    salary_proposed = fields.Float("Proposed Salary", group_operator="avg", help="Salary Proposed by the Organisation")
-    salary_expected = fields.Float("Expected Salary", group_operator="avg", help="Salary Expected by Applicant")
-    availability = fields.Date("Availability", help="The date at which the applicant will be available to start working")
+    job_id = fields.Many2one('hr.job', "Applied Job", domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]", tracking=True, index=True)
+    salary_proposed_extra = fields.Char("Proposed Salary Extra", help="Salary Proposed by the Organisation, extra advantages", tracking=True)
+    salary_expected_extra = fields.Char("Expected Salary Extra", help="Salary Expected by Applicant, extra advantages", tracking=True)
+    salary_proposed = fields.Float("Proposed Salary", group_operator="avg", help="Salary Proposed by the Organisation", tracking=True)
+    salary_expected = fields.Float("Expected Salary", group_operator="avg", help="Salary Expected by Applicant", tracking=True)
+    availability = fields.Date("Availability", help="The date at which the applicant will be available to start working", tracking=True)
     partner_name = fields.Char("Applicant's Name")
-    partner_phone = fields.Char("Phone", size=32)
-    partner_mobile = fields.Char("Mobile", size=32)
+    partner_phone = fields.Char("Phone", size=32, compute='_compute_partner_phone_email',
+        inverse='_inverse_partner_phone', store=True)
+    partner_mobile = fields.Char("Mobile", size=32, compute='_compute_partner_phone_email',
+        inverse='_inverse_partner_mobile', store=True)
     type_id = fields.Many2one('hr.recruitment.degree', "Degree")
-    department_id = fields.Many2one('hr.department', "Department", domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]")
+    department_id = fields.Many2one(
+        'hr.department', "Department", compute='_compute_department', store=True, readonly=False,
+        domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]", tracking=True)
     day_open = fields.Float(compute='_compute_day', string="Days to Open", compute_sudo=True)
     day_close = fields.Float(compute='_compute_day', string="Days to Close", compute_sudo=True)
     delay_close = fields.Float(compute="_compute_day", string='Delay to Close', readonly=True, group_operator="avg", help="Number of days to close", store=True)
     color = fields.Integer("Color Index", default=0)
     emp_id = fields.Many2one('hr.employee', string="Employee", help="Employee linked to the applicant.", copy=False)
-    user_email = fields.Char(related='user_id.email', type="char", string="User Email", readonly=True)
+    user_email = fields.Char(related='user_id.email', string="User Email", readonly=True)
     attachment_number = fields.Integer(compute='_get_attachment_number', string="Number of Attachments")
     employee_name = fields.Char(related='emp_id.name', string="Employee Name", readonly=False, tracking=False)
     attachment_ids = fields.One2many('ir.attachment', 'res_id', domain=[('res_model', '=', 'hr.applicant')], string='Attachments')
@@ -160,11 +162,14 @@ class Applicant(models.Model):
         ('done', 'Green'),
         ('blocked', 'Red')], string='Kanban State',
         copy=False, default='normal', required=True)
-    legend_blocked = fields.Char(related='stage_id.legend_blocked', string='Kanban Blocked', readonly=False)
-    legend_done = fields.Char(related='stage_id.legend_done', string='Kanban Valid', readonly=False)
-    legend_normal = fields.Char(related='stage_id.legend_normal', string='Kanban Ongoing', readonly=False)
+    legend_blocked = fields.Char(related='stage_id.legend_blocked', string='Kanban Blocked')
+    legend_done = fields.Char(related='stage_id.legend_done', string='Kanban Valid')
+    legend_normal = fields.Char(related='stage_id.legend_normal', string='Kanban Ongoing')
     application_count = fields.Integer(compute='_compute_application_count', help='Applications with the same email')
-    meeting_count = fields.Integer(compute='_compute_meeting_count', help='Meeting Count')
+    refuse_reason_id = fields.Many2one('hr.applicant.refuse.reason', string='Refuse Reason', tracking=True)
+    meeting_ids = fields.One2many('calendar.event', 'applicant_id', 'Meetings')
+    meeting_display_text = fields.Char(compute='_compute_meeting_display')
+    meeting_display_date = fields.Date(compute='_compute_meeting_display')
 
     @api.depends('date_open', 'date_closed')
     def _compute_day(self):
@@ -194,9 +199,30 @@ class Applicant(models.Model):
             applicant.application_count = application_data_mapped.get(applicant.email_from, 1) - 1
         (self - applicants).application_count = False
 
-    def _compute_meeting_count(self):
-        for applicant in self:
-            applicant.meeting_count = self.env['calendar.event'].search_count([('applicant_id', '=', applicant.id)])
+    @api.depends_context('lang')
+    @api.depends('meeting_ids', 'meeting_ids.start')
+    def _compute_meeting_display(self):
+        applicant_with_meetings = self.filtered('meeting_ids')
+        (self - applicant_with_meetings).update({
+            'meeting_display_text': _('No Meeting'),
+            'meeting_display_date': ''
+        })
+        today = fields.Date.today()
+        for applicant in applicant_with_meetings:
+            count = len(applicant.meeting_ids)
+            dates = applicant.meeting_ids.mapped('start')
+            min_date, max_date = min(dates).date(), max(dates).date()
+            if min_date >= today:
+                applicant.meeting_display_date = min_date
+            else:
+                applicant.meeting_display_date = max_date
+            if count == 1:
+                applicant.meeting_display_text = _('1 Meeting')
+            elif applicant.meeting_display_date >= today:
+                applicant.meeting_display_text = _('Next Meeting')
+            else:
+                applicant.meeting_display_text = _('Last Meeting')
+
 
     def _get_attachment_number(self):
         read_group_res = self.env['ir.attachment'].read_group(
@@ -219,88 +245,74 @@ class Applicant(models.Model):
         stage_ids = stages._search(search_domain, order=order, access_rights_uid=SUPERUSER_ID)
         return stages.browse(stage_ids)
 
-    @api.onchange('job_id')
-    def onchange_job_id(self):
-        vals = self._onchange_job_id_internal(self.job_id.id)
-        self.department_id = vals['value']['department_id']
-        self.user_id = vals['value']['user_id']
-        self.stage_id = vals['value']['stage_id']
+    @api.depends('job_id', 'department_id')
+    def _compute_company(self):
+        for applicant in self:
+            company_id = False
+            if applicant.department_id:
+                company_id = applicant.department_id.company_id.id
+            if not company_id and applicant.job_id:
+                company_id = applicant.job_id.company_id.id
+            applicant.company_id = company_id or self.env.company.id
 
-    def _onchange_job_id_internal(self, job_id):
-        department_id = False
-        company_id = False
-        user_id = False
-        company_id = False
-        stage_id = self.stage_id.id or self._context.get('default_stage_id')
-        if job_id:
-            job = self.env['hr.job'].browse(job_id)
-            department_id = job.department_id.id
-            user_id = job.user_id.id
-            company_id = job.company_id.id
-            if not stage_id:
-                stage_ids = self.env['hr.recruitment.stage'].search([
-                    '|',
-                    ('job_ids', '=', False),
-                    ('job_ids', '=', job.id),
-                    ('fold', '=', False)
-                ], order='sequence asc', limit=1).ids
-                stage_id = stage_ids[0] if stage_ids else False
+    @api.depends('job_id')
+    def _compute_department(self):
+        for applicant in self:
+            applicant.department_id = applicant.job_id.department_id.id
 
-        return {'value': {
-            'department_id': department_id,
-            'company_id': company_id,
-            'user_id': user_id,
-            'stage_id': stage_id
-        }}
+    @api.depends('job_id')
+    def _compute_stage(self):
+        for applicant in self:
+            if applicant.job_id:
+                if not applicant.stage_id:
+                    stage_ids = self.env['hr.recruitment.stage'].search([
+                        '|',
+                        ('job_ids', '=', False),
+                        ('job_ids', '=', applicant.job_id.id),
+                        ('fold', '=', False)
+                    ], order='sequence asc', limit=1).ids
+                    applicant.stage_id = stage_ids[0] if stage_ids else False
+            else:
+                applicant.stage_id = False
 
-    @api.onchange('email_from')
-    def onchange_email_from(self):
-        if self.partner_id and self.email_from and not self.partner_id.email:
-            self.partner_id.email = self.email_from
+    @api.depends('job_id')
+    def _compute_user(self):
+        for applicant in self:
+            applicant.user_id = applicant.job_id.user_id.id or self.env.uid
 
-    @api.onchange('partner_phone')
-    def onchange_partner_phone(self):
-        if self.partner_id and self.partner_phone and not self.partner_id.phone:
-            self.partner_id.phone = self.partner_phone
+    @api.depends('partner_id')
+    def _compute_partner_phone_email(self):
+        for applicant in self:
+            applicant.partner_phone = applicant.partner_id.phone
+            applicant.partner_mobile = applicant.partner_id.mobile
+            applicant.email_from = applicant.partner_id.email
 
-    @api.onchange('partner_mobile')
-    def onchange_partner_mobile(self):
-        if self.partner_id and self.partner_mobile and not self.partner_id.mobile:
-            self.partner_id.mobile = self.partner_mobile
+    def _inverse_partner_email(self):
+        for applicant in self.filtered(lambda a: a.partner_id and a.email_from and not a.partner_id.email):
+            applicant.partner_id.email = applicant.email_from
 
-    @api.onchange('partner_id')
-    def onchange_partner_id(self):
-        self.partner_phone = self.partner_id.phone
-        self.partner_mobile = self.partner_id.mobile
-        self.email_from = self.partner_id.email
+    def _inverse_partner_phone(self):
+        for applicant in self.filtered(lambda a: a.partner_id and a.partner_phone and not a.partner_id.phone):
+            applicant.partner_id.phone = applicant.partner_phone
 
-    @api.onchange('stage_id')
-    def onchange_stage_id(self):
-        vals = self._onchange_stage_id_internal(self.stage_id.id)
-        if vals['value'].get('date_closed'):
-            self.date_closed = vals['value']['date_closed']
+    def _inverse_partner_mobile(self):
+        for applicant in self.filtered(lambda a: a.partner_id and a.partner_mobile and not a.partner_id.mobile):
+            applicant.partner_id.mobile = applicant.partner_mobile
 
-    def _onchange_stage_id_internal(self, stage_id):
-        if not stage_id:
-            return {'value': {}}
-        stage = self.env['hr.recruitment.stage'].browse(stage_id)
-        if stage.fold:
-            return {'value': {'date_closed': fields.datetime.now()}}
-        return {'value': {'date_closed': False}}
+    @api.depends('stage_id.hired_stage')
+    def _compute_date_closed(self):
+        for applicant in self:
+            if applicant.stage_id and applicant.stage_id.hired_stage and not applicant.date_closed:
+                applicant.date_closed = fields.datetime.now()
+            if not applicant.stage_id.hired_stage:
+                applicant.date_closed = False
 
     @api.model
     def create(self, vals):
         if vals.get('department_id') and not self._context.get('default_department_id'):
             self = self.with_context(default_department_id=vals.get('department_id'))
-        if vals.get('job_id') or self._context.get('default_job_id'):
-            job_id = vals.get('job_id') or self._context.get('default_job_id')
-            for key, value in self._onchange_job_id_internal(job_id)['value'].items():
-                if key not in vals:
-                    vals[key] = value
         if vals.get('user_id'):
             vals['date_open'] = fields.Datetime.now()
-        if 'stage_id' in vals:
-            vals.update(self._onchange_stage_id_internal(vals.get('stage_id'))['value'])
         if vals.get('email_from'):
             vals['email_from'] = vals['email_from'].strip()
         return super(Applicant, self).create(vals)
@@ -314,7 +326,6 @@ class Applicant(models.Model):
         # stage_id: track last stage before update
         if 'stage_id' in vals:
             vals['date_last_stage_update'] = fields.Datetime.now()
-            vals.update(self._onchange_stage_id_internal(vals.get('stage_id'))['value'])
             if 'kanban_state' not in vals:
                 vals['kanban_state'] = 'normal'
             for applicant in self:
@@ -324,11 +335,28 @@ class Applicant(models.Model):
             res = super(Applicant, self).write(vals)
         return res
 
-    @api.model
     def get_empty_list_help(self, help):
-        return super(Applicant, self.with_context(empty_list_help_model='hr.job',
-                                                  empty_list_help_id=self.env.context.get('default_job_id'),
-                                                  empty_list_help_document_name=_("job applicant"))).get_empty_list_help(help)
+        if 'active_id' in self.env.context and self.env.context.get('active_model') == 'hr.job':
+            alias_id = self.env['hr.job'].browse(self.env.context['active_id']).alias_id
+        else:
+            alias_id = False
+
+        nocontent_values = {
+            'help_title': _('No application yet'),
+            'para_1': _('Let people apply by email to save time.') ,
+            'para_2': _('Attachments, like resumes, get indexed automatically.'),
+        }
+        nocontent_body = """
+            <p class="o_view_nocontent_empty_folder">%(help_title)s</p>
+            <p>%(para_1)s<br/>%(para_2)s</p>"""
+
+        if alias_id and alias_id.alias_domain and alias_id.alias_name:
+            email = alias_id.display_name 
+            email_link = "<a href='mailto:%s'>%s</a>" % (email, email)
+            nocontent_values['email_link'] = email_link
+            nocontent_body += """<p class="o_copy_paste_email">%(email_link)s</p>"""
+
+        return nocontent_body % nocontent_values
 
     def action_makeMeeting(self):
         """ This opens Meeting's calendar view to schedule meeting on current applicant
@@ -338,7 +366,7 @@ class Applicant(models.Model):
         partners = self.partner_id | self.user_id.partner_id | self.department_id.manager_id.user_id.partner_id
 
         category = self.env.ref('hr_recruitment.categ_meet_interview')
-        res = self.env['ir.actions.act_window'].for_xml_id('calendar', 'action_calendar_event')
+        res = self.env['ir.actions.act_window']._for_xml_id('calendar.action_calendar_event')
         res['context'] = {
             'default_applicant_id': self.id,
             'default_partner_ids': partners.ids,
@@ -349,8 +377,7 @@ class Applicant(models.Model):
         return res
 
     def action_get_attachment_tree_view(self):
-        attachment_action = self.env.ref('base.action_attachment')
-        action = attachment_action.read()[0]
+        action = self.env['ir.actions.act_window']._for_xml_id('base.action_attachment')
         action['context'] = {'default_res_model': self._name, 'default_res_id': self.ids[0]}
         action['domain'] = str(['&', ('res_model', '=', self._name), ('res_id', 'in', self.ids)])
         action['search_view_id'] = (self.env.ref('hr_recruitment.ir_attachment_view_search_inherit_hr_recruitment').id, )
@@ -359,7 +386,7 @@ class Applicant(models.Model):
     def action_applications_email(self):
         return {
             'type': 'ir.actions.act_window',
-            'name': _('Applications'),
+            'name': _('Job Applications'),
             'res_model': self._name,
             'view_mode': 'kanban,tree,form,pivot,graph,calendar,activity',
             'domain': [('email_from', 'in', self.mapped('email_from'))],
@@ -374,7 +401,7 @@ class Applicant(models.Model):
         if 'stage_id' in changes and applicant.stage_id.template_id:
             res['stage_id'] = (applicant.stage_id.template_id, {
                 'auto_delete_message': True,
-                'subtype_id': self.env['ir.model.data'].xmlid_to_res_id('mail.mt_note'),
+                'subtype_id': self.env['ir.model.data']._xmlid_to_res_id('mail.mt_note'),
                 'email_layout_xmlid': 'mail.mail_notification_light'
             })
         return res
@@ -401,11 +428,11 @@ class Applicant(models.Model):
         recipients = super(Applicant, self)._message_get_suggested_recipients()
         for applicant in self:
             if applicant.partner_id:
-                applicant._message_add_suggested_recipient(recipients, partner=applicant.partner_id, reason=_('Contact'))
+                applicant._message_add_suggested_recipient(recipients, partner=applicant.partner_id.sudo(), reason=_('Contact'))
             elif applicant.email_from:
                 email_from = applicant.email_from
                 if applicant.partner_name:
-                    email_from = '%s<%s>' % (applicant.partner_name, email_from)
+                    email_from = tools.formataddr((applicant.partner_name, email_from))
                 applicant._message_add_suggested_recipient(recipients, email=email_from, reason=_('Contact Email'))
         return recipients
 
@@ -420,6 +447,9 @@ class Applicant(models.Model):
         # want the gateway user to be responsible if no other responsible is
         # found.
         self = self.with_context(default_user_id=False)
+        stage = False
+        if custom_values and 'job_id' in custom_values:
+            stage = self.env['hr.job'].browse(custom_values['job_id'])._get_first_stage()
         val = msg.get('from').split('<')[0]
         defaults = {
             'name': msg.get('subject') or _("No Subject"),
@@ -429,6 +459,8 @@ class Applicant(models.Model):
         }
         if msg.get('priority'):
             defaults['priority'] = msg.get('priority')
+        if stage and stage.id:
+            defaults['stage_id'] = stage.id
         if custom_values:
             defaults.update(custom_values)
         return super(Applicant, self).message_new(msg, custom_values=defaults)
@@ -441,7 +473,11 @@ class Applicant(models.Model):
             new_partner = message.partner_ids.filtered(lambda partner: partner.email == self.email_from)
             if new_partner:
                 if new_partner.create_date.date() == fields.Date.today():
-                    new_partner.type = 'private'
+                    new_partner.write({
+                        'type': 'private',
+                        'phone': self.partner_phone,
+                        'mobile': self.partner_mobile,
+                    })
                 self.search([
                     ('partner_id', '=', False),
                     ('email_from', '=', new_partner.email),
@@ -467,53 +503,84 @@ class Applicant(models.Model):
                     'phone': applicant.partner_phone,
                     'mobile': applicant.partner_mobile
                 })
+                applicant.partner_id = new_partner_id
                 address_id = new_partner_id.address_get(['contact'])['contact']
             if applicant.partner_name or contact_name:
-                employee = self.env['hr.employee'].create({
-                    'name': applicant.partner_name or contact_name,
-                    'job_id': applicant.job_id.id or False,
-                    'job_title': applicant.job_id.name,
-                    'address_home_id': address_id,
-                    'department_id': applicant.department_id.id or False,
-                    'address_id': applicant.company_id and applicant.company_id.partner_id
+                employee_data = {
+                    'default_name': applicant.partner_name or contact_name,
+                    'default_job_id': applicant.job_id.id,
+                    'default_job_title': applicant.job_id.name,
+                    'default_address_home_id': address_id,
+                    'default_department_id': applicant.department_id.id or False,
+                    'default_address_id': applicant.company_id and applicant.company_id.partner_id
                             and applicant.company_id.partner_id.id or False,
-                    'work_email': applicant.department_id and applicant.department_id.company_id
+                    'default_work_email': applicant.department_id and applicant.department_id.company_id
                             and applicant.department_id.company_id.email or False,
-                    'work_phone': applicant.department_id and applicant.department_id.company_id
-                            and applicant.department_id.company_id.phone or False})
-                applicant.write({'emp_id': employee.id})
-                if applicant.job_id:
-                    applicant.job_id.write({'no_of_hired_employee': applicant.job_id.no_of_hired_employee + 1})
-                    applicant.job_id.message_post(
-                        body=_('New Employee %s Hired') % applicant.partner_name if applicant.partner_name else applicant.name,
-                        subtype="hr_recruitment.mt_job_applicant_hired")
-                applicant.message_post_with_view(
-                    'hr_recruitment.applicant_hired_template',
-                    values={'applicant': applicant},
-                    subtype_id=self.env.ref("hr_recruitment.mt_applicant_hired").id)
-
-        employee_action = self.env.ref('hr.open_view_employee_list')
-        dict_act_window = employee_action.read([])[0]
-        dict_act_window['context'] = {'form_view_initial_mode': 'edit'}
-        dict_act_window['res_id'] = employee.id
+                    'default_work_phone': applicant.department_id.company_id.phone,
+                    'form_view_initial_mode': 'edit',
+                    'default_applicant_id': applicant.ids,
+                    }
+                    
+        dict_act_window = self.env['ir.actions.act_window']._for_xml_id('hr.open_view_employee_list')
+        dict_act_window['context'] = employee_data
         return dict_act_window
 
     def archive_applicant(self):
-        self.write({'active': False})
+        return {
+            'type': 'ir.actions.act_window',
+            'name': _('Refuse Reason'),
+            'res_model': 'applicant.get.refuse.reason',
+            'view_mode': 'form',
+            'target': 'new',
+            'context': {'default_applicant_ids': self.ids, 'active_test': False},
+            'views': [[False, 'form']]
+        }
 
     def reset_applicant(self):
         """ Reinsert the applicant into the recruitment pipe in the first stage"""
-        default_stage_id = self._default_stage_id()
-        self.write({'active': True, 'stage_id': default_stage_id})
+        default_stage = dict()
+        for job_id in self.mapped('job_id'):
+            default_stage[job_id.id] = self.env['hr.recruitment.stage'].search(
+                ['|',
+                    ('job_ids', '=', False),
+                    ('job_ids', '=', job_id.id),
+                    ('fold', '=', False)
+                ], order='sequence asc', limit=1).id
+        for applicant in self:
+            applicant.write(
+                {'stage_id': applicant.job_id.id and default_stage[applicant.job_id.id],
+                 'refuse_reason_id': False})
+
+    def toggle_active(self):
+        res = super(Applicant, self).toggle_active()
+        applicant_active = self.filtered(lambda applicant: applicant.active)
+        if applicant_active:
+            applicant_active.reset_applicant()
+        applicant_inactive = self.filtered(lambda applicant: not applicant.active)
+        if applicant_inactive:
+            return applicant_inactive.archive_applicant()
+        return res
 
 
 class ApplicantCategory(models.Model):
     _name = "hr.applicant.category"
     _description = "Category of applicant"
 
+    def _get_default_color(self):
+        return randint(1, 11)
+
     name = fields.Char("Tag Name", required=True)
-    color = fields.Integer(string='Color Index', default=10)
+    color = fields.Integer(string='Color Index', default=_get_default_color)
 
     _sql_constraints = [
             ('name_uniq', 'unique (name)', "Tag name already exists !"),
     ]
+
+
+class ApplicantRefuseReason(models.Model):
+    _name = "hr.applicant.refuse.reason"
+    _description = 'Refuse Reason of Applicant'
+
+    name = fields.Char('Description', required=True, translate=True)
+    template_id = fields.Many2one('mail.template', string='Email Template', domain="[('model', '=', 'hr.applicant')]")
+    active = fields.Boolean('Active', default=True)
