@@ -16,9 +16,13 @@
         'security/ir.model.access.csv',
         'data/hr_work_entry_data.xml',
         'views/hr_work_entry_views.xml',
+        'views/hr_employee_views.xml',
+        'views/resource_calendar_views.xml',
     ],
-    'qweb': [
-        "static/src/xml/work_entry_templates.xml",
-    ],
+    'assets': {
+        'web.assets_qweb': [
+            'hr_work_entry/static/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
