@@ -4,10 +4,10 @@
 {
     'name': 'OdooBot for livechat',
     'version': '1.0',
-    'category': 'Discuss',
+    'category': 'Productivity/Discuss',
     'summary': 'Add livechat support for OdooBot',
     'description': "",
-    'website': 'https://www.odoo.com/page/discuss',
+    'website': 'https://www.odoo.com/app/discuss',
     'depends': ['mail_bot', 'im_livechat'],
     'installable': True,
     'application': False,
