@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 {
-    'name': 'Argentinian Accounting',
+    'name': 'Argentina - Accounting',
+    'icon': '/base/static/img/country_flags/ar.png',
     'version': "3.5",
     'description': """
 Functional
 ----------
 
-This module add accounting features for the Argentinian localization, which represent the minimal configuration needed for a company  to operate in Argentina and under the AFIP (Administración Federal de Ingresos Públicos) regulations and guidelines.
+This module add accounting features for the Argentinean localization, which represent the minimal configuration needed for a company  to operate in Argentina and under the AFIP (Administración Federal de Ingresos Públicos) regulations and guidelines.
 
 Follow the next configuration steps for Production:
 
@@ -55,7 +56,7 @@ Master Data:
   * Exento (EX)
   * Monotributo (Mono)
 
-* Argentinian Taxes and Account Tax Groups (VAT taxes with the existing aliquots and other types)
+* Argentinean Taxes and Account Tax Groups (VAT taxes with the existing aliquots and other types)
 * AFIP Responsibility Types
 * Fiscal Positions (in order to map taxes)
 * Legal Documents Types in Argentina
@@ -66,7 +67,7 @@ Master Data:
 * Partners: Consumidor Final and AFIP
 """,
     'author': 'ADHOC SA',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'depends': [
         'l10n_latam_invoice_document',
         'l10n_latam_base',
@@ -75,11 +76,11 @@ Master Data:
         'security/ir.model.access.csv',
         'data/l10n_latam_identification_type_data.xml',
         'data/l10n_ar_afip_responsibility_type_data.xml',
-        'data/account_group_data.xml',
         'data/account_chart_template_data.xml',
+        'data/account.group.template.csv',
         'data/account.account.template.csv',
         'data/account_chart_template_data2.xml',
-        'data/account_tax_group.xml',
+        'data/account_tax_group_data.xml',
         'data/account_tax_template_data.xml',
         'data/account_fiscal_template.xml',
         'data/uom_uom_data.xml',
@@ -99,9 +100,10 @@ Master Data:
         'views/uom_uom_view.xml',
         'views/account_journal_view.xml',
         'views/l10n_latam_document_type_view.xml',
-        'views/ir_sequence_view.xml',
         'views/report_invoice.xml',
+        'views/res_config_settings_view.xml',
         'report/invoice_report_view.xml',
+        'data/account_chart_template_configure_data.xml',
     ],
     'demo': [
         # we create demo data on different companies (not main_company) to
@@ -116,11 +118,14 @@ Master Data:
         'demo/account_customer_refund_demo.xml',
         'demo/account_supplier_invoice_demo.xml',
         'demo/account_supplier_refund_demo.xml',
-        # restore
-        'demo/res_users_demo.xml',
     ],
     'installable': True,
     'auto_install': False,
     'application': False,
+    'assets': {
+        'web.assets_backend': [
+            'l10n_ar/static/src/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
