@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 {
     'name': 'Belgium - E-Invoicing (UBL 2.0, e-fff)',
+    'icon': '/l10n_be/static/description/icon.png',
     'version': '0.1',
-    'category': 'Hidden',
+    'category': 'Accounting/Localizations/EDI',
     'summary': 'E-Invoicing, Universal Business Language (UBL 2.0), e-fff protocol',
     'description': """
 Universal Business Language (UBL <http://ubl.xml.org/>`_) is a library of standard electronic XML business documents such as
@@ -11,8 +12,10 @@ invoices. The UBL standard became the `ISO/IEC 19845
 (cf the `official announce <http://www.prweb.com/releases/2016/01/prweb13186919.htm>`_).
 Belgian e-invoicing uses the UBL 2.0 using the e-fff protocol.
     """,
-    'depends': ['account', 'l10n_be', 'account_facturx'],
+    'depends': ['l10n_be', 'account_edi_ubl'],
     'data': [
+        'data/account_edi_data.xml',
+        'data/ubl_templates.xml',
     ],
     'installable': True,
     'auto_install': True,
