@@ -4,13 +4,11 @@
 import re
 
 from odoo import api, fields, models, _
+from odoo.addons.base.models.res_bank import sanitize_account_number
+from odoo.addons.base_iban.models.res_partner_bank import normalize_iban, pretty_iban, validate_iban
 from odoo.exceptions import ValidationError
 from odoo.tools.misc import mod10r
-from odoo.tools.image import image_data_uri
-import base64
 
-import werkzeug.urls
-import werkzeug.exceptions
 
 ISR_SUBSCRIPTION_CODE = {'CHF': '01', 'EUR': '03'}
 CLEARING = "09000"
@@ -39,18 +37,54 @@ def _is_l10n_ch_isr_issuer(account_ref, currency_code):
         return _is_l10n_ch_postal(account_ref)
     return False
 
+def validate_qr_iban(qr_iban):
+    # Check first if it's a valid IBAN.
+    validate_iban(qr_iban)
+
+    # We sanitize first so that _check_qr_iban_range() can extract correct IID from IBAN to validate it.
+    sanitized_qr_iban = sanitize_account_number(qr_iban)
+
+    if sanitized_qr_iban[:2] != 'CH':
+        raise ValidationError(_("QR-IBAN numbers are only available in Switzerland."))
+
+    # Now, check if it's valid QR-IBAN (based on its IID).
+    if not check_qr_iban_range(sanitized_qr_iban):
+        raise ValidationError(_("QR-IBAN '%s' is invalid.") % qr_iban)
+
+    return True
+
+def check_qr_iban_range(iban):
+    if not iban or len(iban) < 9:
+        return False
+    iid_start_index = 4
+    iid_end_index = 8
+    iid = iban[iid_start_index : iid_end_index+1]
+    return re.match(r'\d+', iid) and 30000 <= int(iid) <= 31999 # Those values for iid are reserved for QR-IBANs only
+
 
 class ResPartnerBank(models.Model):
     _inherit = 'res.partner.bank'
 
-    l10n_ch_postal = fields.Char(string='Swiss Postal Account', help='This field is used for the Swiss postal account number '
-                                                                     'on a vendor account and for the client number on your '
-                                                                     'own account.  The client number is mostly 6 numbers without '
-                                                                     '-, while the postal account number can be e.g. 01-162-8')
+    l10n_ch_postal = fields.Char(
+        string="Swiss Postal Account",
+        readonly=False, store=True,
+        compute='_compute_l10n_ch_postal',
+        help="This field is used for the Swiss postal account number on a vendor account and for the client number on "
+             "your own account. The client number is mostly 6 numbers without -, while the postal account number can "
+             "be e.g. 01-162-8")
+
+    l10n_ch_qr_iban = fields.Char(string='QR-IBAN',
+                                  compute='_compute_l10n_ch_qr_iban',
+                                  store=True,
+                                  readonly=False,
+                                  help="Put the QR-IBAN here for your own bank accounts.  That way, you can "
+                                       "still use the main IBAN in the Account Number while you will see the "
+                                       "QR-IBAN for the barcode.  ")
+
     # fields to configure ISR payment slip generation
     l10n_ch_isr_subscription_chf = fields.Char(string='CHF ISR Subscription Number', help='The subscription number provided by the bank or Postfinance to identify the bank, used to generate ISR in CHF. eg. 01-162-8')
     l10n_ch_isr_subscription_eur = fields.Char(string='EUR ISR Subscription Number', help='The subscription number provided by the bank or Postfinance to identify the bank, used to generate ISR in EUR. eg. 03-162-5')
-    l10n_ch_show_subscription = fields.Boolean(compute='_compute_l10n_ch_show_subscription', default=lambda self: self.env.company.country_id.code == 'CH')
+    l10n_ch_show_subscription = fields.Boolean(compute='_compute_l10n_ch_show_subscription', default=lambda self: self.env.company.account_fiscal_country_id.code == 'CH')
 
     def _is_isr_issuer(self):
         return (_is_l10n_ch_isr_issuer(self.l10n_ch_postal, 'CHF')
@@ -60,7 +94,7 @@ class ResPartnerBank(models.Model):
     def _check_postal_num(self):
         """Validate postal number format"""
         for rec in self:
-            if rec.l10n_ch_postal and not _is_l10n_ch_postal(self.l10n_ch_postal):
+            if rec.l10n_ch_postal and not _is_l10n_ch_postal(rec.l10n_ch_postal):
                 # l10n_ch_postal is used for the purpose of Client Number on your own accounts, so don't do the check there
                 if rec.partner_id and not rec.partner_id.ref_company_ids:
                     raise ValidationError(
@@ -88,11 +122,11 @@ class ResPartnerBank(models.Model):
     def _compute_l10n_ch_show_subscription(self):
         for bank in self:
             if bank.partner_id:
-                bank.l10n_ch_show_subscription = bank.partner_id.ref_company_ids.country_id.code =='CH'
+                bank.l10n_ch_show_subscription = bank.partner_id.ref_company_ids.country_id.code == 'CH'
             elif bank.company_id:
-                bank.l10n_ch_show_subscription = bank.company_id.country_id.code == 'CH'
+                bank.l10n_ch_show_subscription = bank.company_id.account_fiscal_country_id.code == 'CH'
             else:
-                bank.l10n_ch_show_subscription = self.env.company.country_id.code == 'CH'
+                bank.l10n_ch_show_subscription = self.env.company.account_fiscal_country_id.code == 'CH'
 
     @api.depends('acc_number', 'acc_type')
     def _compute_sanitized_acc_number(self):
@@ -102,6 +136,33 @@ class ResPartnerBank(models.Model):
             bank.sanitized_acc_number = bank.acc_number
         super(ResPartnerBank, self - postal_banks)._compute_sanitized_acc_number()
 
+    @api.depends('acc_number')
+    def _compute_l10n_ch_qr_iban(self):
+        for record in self:
+            try:
+                validate_qr_iban(record.acc_number)
+                valid_qr_iban = True
+            except ValidationError:
+                valid_qr_iban = False
+
+            if valid_qr_iban:
+                record.l10n_ch_qr_iban = record.sanitized_acc_number
+            else:
+                record.l10n_ch_qr_iban = None
+
+    @api.model
+    def create(self, vals):
+        if vals.get('l10n_ch_qr_iban'):
+            validate_qr_iban(vals['l10n_ch_qr_iban'])
+            vals['l10n_ch_qr_iban'] = pretty_iban(normalize_iban(vals['l10n_ch_qr_iban']))
+        return super().create(vals)
+
+    def write(self, vals):
+        if vals.get('l10n_ch_qr_iban'):
+            validate_qr_iban(vals['l10n_ch_qr_iban'])
+            vals['l10n_ch_qr_iban'] = pretty_iban(normalize_iban(vals['l10n_ch_qr_iban']))
+        return super().write(vals)
+
     @api.model
     def _get_supported_account_types(self):
         rslt = super(ResPartnerBank, self)._get_supported_account_types()
@@ -123,20 +184,21 @@ class ResPartnerBank(models.Model):
         else:
             return super(ResPartnerBank, self).retrieve_acc_type(acc_number)
 
-    @api.onchange('acc_number', 'partner_id', 'acc_type')
-    def _onchange_set_l10n_ch_postal(self):
-        if self.acc_type == 'iban':
-            self.l10n_ch_postal = self._retrieve_l10n_ch_postal(self.sanitized_acc_number)
-        elif self.acc_type == 'postal':
-            if self.acc_number and " " in self.acc_number:
-                self.l10n_ch_postal = self.acc_number.split(" ")[0]
-            else:
-                self.l10n_ch_postal = self.acc_number
-                # In case of ISR issuer, this number is not
-                # unique and we fill acc_number with partner
-                # name to give proper information to the user
-                if self.partner_id and self.acc_number[:2] in ["01", "03"]:
-                    self.acc_number = ("{} {}").format(self.acc_number, self.partner_id.name)
+    @api.depends('acc_number', 'partner_id', 'acc_type')
+    def _compute_l10n_ch_postal(self):
+        for record in self:
+            if record.acc_type == 'iban':
+                record.l10n_ch_postal = self._retrieve_l10n_ch_postal(record.sanitized_acc_number)
+            elif record.acc_type == 'postal':
+                if record.acc_number and " " in record.acc_number:
+                    record.l10n_ch_postal = record.acc_number.split(" ")[0]
+                else:
+                    record.l10n_ch_postal = record.acc_number
+                    # In case of ISR issuer, this number is not
+                    # unique and we fill acc_number with partner
+                    # name to give proper information to the user
+                    if record.partner_id and record.acc_number[:2] in ["01", "03"]:
+                        record.acc_number = ("{} {}").format(record.acc_number, record.partner_id.name)
 
     @api.model
     def _is_postfinance_iban(self, iban):
@@ -173,31 +235,34 @@ class ResPartnerBank(models.Model):
             return self._pretty_postal_num(iban[-9:])
         return None
 
-    def find_number(self, s):
-        # DEPRECATED FUNCTION: not used anymore. QR-bills don't use structured addresses
-        # this regex match numbers like 1bis 1a
-        lmo = re.findall('([0-9]+[^ ]*)',s)
-        # no number found
-        if len(lmo) == 0:
-            return ''
-        # Only one number or starts with a number return the first one
-        if len(lmo) == 1 or re.match(r'^\s*([0-9]+[^ ]*)',s):
-            return lmo[0]
-        # else return the last one
-        if len(lmo) > 1:
-            return lmo[-1]
-        else:
-            return ''
+    def _l10n_ch_get_qr_vals(self, amount, currency, debtor_partner, free_communication, structured_communication):
+        comment = ""
+        if free_communication:
+            comment = (free_communication[:137] + '...') if len(free_communication) > 140 else free_communication
 
-    def _prepare_swiss_code_url_vals(self, amount, currency_name, debtor_partner, reference_type, reference, comment):
         creditor_addr_1, creditor_addr_2 = self._get_partner_address_lines(self.partner_id)
         debtor_addr_1, debtor_addr_2 = self._get_partner_address_lines(debtor_partner)
 
+        # Compute reference type (empty by default, only mandatory for QR-IBAN,
+        # and must then be 27 characters-long, with mod10r check digit as the 27th one,
+        # just like ISR number for invoices)
+        reference_type = 'NON'
+        reference = ''
+        acc_number = self.sanitized_acc_number
+
+        if self.l10n_ch_qr_iban:
+            # _check_for_qr_code_errors ensures we can't have a QR-IBAN without a QR-reference here
+            reference_type = 'QRR'
+            reference = structured_communication
+            acc_number = sanitize_account_number(self.l10n_ch_qr_iban)
+
+        currency = currency or self.currency_id or self.company_id.currency_id
+
         return [
             'SPC',                                                # QR Type
             '0200',                                               # Version
             '1',                                                  # Coding Type
-            self.sanitized_acc_number,                            # IBAN
+            acc_number,                                           # IBAN / QR-IBAN
             'K',                                                  # Creditor Address Type
             (self.acc_holder_name or self.partner_id.name)[:70],  # Creditor Name
             creditor_addr_1,                                      # Creditor Address Line 1
@@ -213,7 +278,7 @@ class ResPartnerBank(models.Model):
             '',                                                   # Ultimate Creditor Town
             '',                                                   # Ultimate Creditor Country
             '{:.2f}'.format(amount),                              # Amount
-            currency_name,                                        # Currency
+            currency.name,                                        # Currency
             'K',                                                  # Ultimate Debtor Address Type
             debtor_partner.commercial_partner_id.name[:70],       # Ultimate Debtor Name
             debtor_addr_1,                                        # Ultimate Debtor Address Line 1
@@ -227,39 +292,24 @@ class ResPartnerBank(models.Model):
             'EPD',                                                # Mandatory trailer part
         ]
 
+    def _get_qr_vals(self, qr_method, amount, currency, debtor_partner, free_communication, structured_communication):
+        if qr_method == 'ch_qr':
+            return self._l10n_ch_get_qr_vals(amount, currency, debtor_partner, free_communication, structured_communication)
+        return super()._get_qr_vals(qr_method, amount, currency, debtor_partner, free_communication, structured_communication)
 
-    def _build_swiss_code_vals(self, amount, currency_name, debtor_partner, structured_communication, free_communication):
-        comment = ""
-        if free_communication:
-            comment = (free_communication[:137] + '...') if len(free_communication) > 140 else free_communication
-
-        # Compute reference type (empty by default, only mandatory for QR-IBAN,
-        # and must then be 27 characters-long, with mod10r check digit as the 27th one,
-        # just like ISR number for invoices)
-        reference_type = 'NON'
-        reference = ''
-        if self._is_qr_iban():
-            # _check_for_qr_code_errors ensures we can't have a QR-IBAN without a QR-reference here
-            reference_type = 'QRR'
-            reference = structured_communication
-
-        return self._prepare_swiss_code_url_vals(amount, currency_name, debtor_partner, reference_type, reference, comment)
-
-    @api.model
-    def build_swiss_code_url(self, amount, currency_name, not_used_anymore_1, debtor_partner, not_used_anymore_2, structured_communication, free_communication):
-        qr_code_vals = self._build_swiss_code_vals(amount, currency_name, debtor_partner, structured_communication, free_communication)
-        # use quiet to remove blank around the QR and make it easier to place it
-        return '/report/barcode/?type=%s&value=%s&width=%s&height=%s&quiet=1' % ('QR', werkzeug.urls.url_quote_plus('\n'.join(qr_code_vals)), 256, 256)
-
-    @api.model
-    def build_swiss_code_base64(self, amount, currency_name, not_used_anymore_1, debtor_partner, not_used_anymore_2, structured_communication, free_communication):
-        qr_code_vals = self._build_swiss_code_vals(amount, currency_name, debtor_partner, structured_communication, free_communication)
-        try:
-            barcode = self.env['ir.actions.report'].barcode('QR', '\n'.join(qr_code_vals), width=256, height=256, quiet=1)
-        except (ValueError, AttributeError):
-            raise werkzeug.exceptions.HTTPException(description='Cannot convert into barcode.')
-
-        return image_data_uri(base64.b64encode(barcode))
+    def _get_qr_code_generation_params(self, qr_method, amount, currency, debtor_partner, free_communication, structured_communication):
+        if qr_method == 'ch_qr':
+            return {
+                'barcode_type': 'QR',
+                'width': 256,
+                'height': 256,
+                'quiet': 1,
+                'mask': 'ch_cross',
+                'value': '\n'.join(self._get_qr_vals(qr_method, amount, currency, debtor_partner, free_communication, structured_communication)),
+                # Swiss QR code requires Error Correction Level = 'M' by specification
+                'barLevel': 'M',
+            }
+        return super()._get_qr_code_generation_params(qr_method, amount, currency, debtor_partner, free_communication, structured_communication)
 
     def _get_partner_address_lines(self, partner):
         """ Returns a tuple of two elements containing the address lines to use
@@ -271,26 +321,6 @@ class ResPartnerBank(models.Model):
         line_2 = partner.zip + ' ' + partner.city
         return line_1[:70], line_2[:70]
 
-    def _check_qr_iban_range(self, iban):
-        if not iban or len(iban) < 9:
-            return False
-        iid_start_index = 4
-        iid_end_index = 8
-        iid = iban[iid_start_index : iid_end_index+1]
-        return re.match('\d+', iid) \
-               and 30000 <= int(iid) <= 31999 # Those values for iid are reserved for QR-IBANs only
-
-    def _is_qr_iban(self):
-        """ Tells whether or not this bank account has a QR-IBAN account number.
-        QR-IBANs are specific identifiers used in Switzerland as references in
-        QR-codes. They are formed like regular IBANs, but are actually something
-        different.
-        """
-        self.ensure_one()
-        return self.sanitized_acc_number.startswith('CH')\
-               and self.acc_type == 'iban'\
-               and self._check_qr_iban_range(self.sanitized_acc_number)
-
     @api.model
     def _is_qr_reference(self, reference):
         """ Checks whether the given reference is a QR-reference, i.e. it is
@@ -301,18 +331,37 @@ class ResPartnerBank(models.Model):
                and re.match('\d+$', reference) \
                and reference == mod10r(reference[:-1])
 
-    def validate_swiss_code_arguments(self, currency, debtor_partner, reference_to_check=''):
-        # reference_to_check added as an optional parameter in order not to break our stability policy.
-        # For people having already installed the module, QRR won't be checked until
-        # they update the module (as a change in the pdf report's xml sets a value in reference_to_check).
-        # '' is used as default, as an empty field will pass None value,
-        # and we want to be able to distinguish between those cases
+    def _eligible_for_qr_code(self, qr_method, debtor_partner, currency):
+        if qr_method == 'ch_qr':
+
+            return self.acc_type == 'iban' and \
+                   self.partner_id.country_id.code == 'CH' and \
+                   (not debtor_partner or debtor_partner.country_id.code == 'CH') \
+                   and currency.name in ('EUR', 'CHF')
+
+        return super()._eligible_for_qr_code(qr_method, debtor_partner, currency)
+
+    def _check_for_qr_code_errors(self, qr_method, amount, currency, debtor_partner, free_communication, structured_communication):
         def _partner_fields_set(partner):
             return partner.zip and \
                    partner.city and \
                    partner.country_id.code and \
                    (partner.street or partner.street2)
 
-        return _partner_fields_set(self.partner_id) and \
-               _partner_fields_set(debtor_partner) and \
-               (reference_to_check == '' or not self._is_qr_iban() or self._is_qr_reference(reference_to_check))
+        if qr_method == 'ch_qr':
+            if not _partner_fields_set(self.partner_id):
+                return _("The partner set on the bank account meant to receive the payment (%s) must have a complete postal address (street, zip, city and country).", self.acc_number)
+
+            if debtor_partner and not _partner_fields_set(debtor_partner):
+                return _("The partner must have a complete postal address (street, zip, city and country).")
+
+            if self.l10n_ch_qr_iban and not self._is_qr_reference(structured_communication):
+                return _("When using a QR-IBAN as the destination account of a QR-code, the payment reference must be a QR-reference.")
+
+        return super()._check_for_qr_code_errors(qr_method, amount, currency, debtor_partner, free_communication, structured_communication)
+
+    @api.model
+    def _get_available_qr_methods(self):
+        rslt = super()._get_available_qr_methods()
+        rslt.append(('ch_qr', _("Swiss QR bill"), 10))
+        return rslt
