@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.addons.account.tests.account_test_savepoint import AccountTestInvoicingCommon
-from odoo.tests import common, Form
+from odoo.addons.account.tests.common import AccountTestInvoicingCommon
+from odoo.tests import tagged
 
 QR_IBAN = 'CH21 3080 8001 2345 6782 7'
 ISR_SUBS_NUMBER = "01-162-8"
 
 
+@tagged('post_install_l10n', 'post_install', '-at_install')
 class TestGenISRReference(AccountTestInvoicingCommon):
     """Check condition of generation of and content of the structured ref"""
 
@@ -36,11 +37,11 @@ class TestGenISRReference(AccountTestInvoicingCommon):
                 "partner_id": cls.partner_a.id,
             }
         )
-        cls.invoice = cls.init_invoice("out_invoice")
+        cls.invoice = cls.init_invoice("out_invoice", products=cls.product_a+cls.product_b)
 
     def test_isr(self):
 
-        self.invoice.invoice_partner_bank_id = self.bank_acc_isr
+        self.invoice.partner_bank_id = self.bank_acc_isr
         self.invoice.name = "INV/01234567890"
 
         expected_isr = "000000000000000012345678903"
@@ -51,7 +52,7 @@ class TestGenISRReference(AccountTestInvoicingCommon):
         self.assertEqual(self.invoice.l10n_ch_isr_optical_line, expected_optical_line)
 
     def test_qrr(self):
-        self.invoice.invoice_partner_bank_id = self.bank_acc_qriban
+        self.invoice.partner_bank_id = self.bank_acc_qriban
 
         self.invoice.name = "INV/01234567890"
 
@@ -62,7 +63,7 @@ class TestGenISRReference(AccountTestInvoicingCommon):
         # No need to check optical line, we have no use for it with QR-bill
 
     def test_isr_long_reference(self):
-        self.invoice.invoice_partner_bank_id = self.bank_acc_isr
+        self.invoice.partner_bank_id = self.bank_acc_isr
 
         self.invoice.name = "INV/123456789012345678901234567890"
 
@@ -76,7 +77,7 @@ class TestGenISRReference(AccountTestInvoicingCommon):
     def test_missing_isr_subscription_num(self):
         self.bank_acc_isr.l10n_ch_isr_subscription_chf = False
 
-        self.invoice.invoice_partner_bank_id = self.bank_acc_isr
+        self.invoice.partner_bank_id = self.bank_acc_isr
 
         self.assertFalse(self.invoice.l10n_ch_isr_number)
         self.assertFalse(self.invoice.l10n_ch_isr_number_spaced)
@@ -86,21 +87,21 @@ class TestGenISRReference(AccountTestInvoicingCommon):
         self.bank_acc_isr.l10n_ch_isr_subscription_chf = False
         self.bank_acc_isr.l10n_ch_postal = ISR_SUBS_NUMBER
 
-        self.invoice.invoice_partner_bank_id = self.bank_acc_isr
+        self.invoice.partner_bank_id = self.bank_acc_isr
 
         self.assertFalse(self.invoice.l10n_ch_isr_number)
         self.assertFalse(self.invoice.l10n_ch_isr_number_spaced)
         self.assertFalse(self.invoice.l10n_ch_isr_optical_line)
 
     def test_no_bank_account(self):
-        self.invoice.invoice_partner_bank_id = False
+        self.invoice.partner_bank_id = False
 
         self.assertFalse(self.invoice.l10n_ch_isr_number)
         self.assertFalse(self.invoice.l10n_ch_isr_number_spaced)
         self.assertFalse(self.invoice.l10n_ch_isr_optical_line)
 
     def test_wrong_currency(self):
-        self.invoice.invoice_partner_bank_id = self.bank_acc_isr
+        self.invoice.partner_bank_id = self.bank_acc_isr
         self.invoice.currency_id = self.env.ref("base.BTN")
 
         self.assertFalse(self.invoice.l10n_ch_isr_number)
