@@ -30,10 +30,11 @@ class ResPartner(models.Model):
     def _format_vat_cl(self, values):
         identification_types = [self.env.ref('l10n_latam_base.it_vat').id, self.env.ref('l10n_cl.it_RUT').id,
                                 self.env.ref('l10n_cl.it_RUN').id]
-        partner_country_is_chile = (values.get('country_id') == self.env.ref('base.cl').id) or (
-                    values.get('l10n_latam_identification_type_id') and
-                    self.env['l10n_latam.identification.type'].browse(
-                        values.get('l10n_latam_identification_type_id')).country_id == self.env.ref('base.cl'))
+        country = self.env["res.country"].browse(values.get('country_id'))
+        identification_type = self.env['l10n_latam.identification.type'].browse(
+            values.get('l10n_latam_identification_type_id')
+        )
+        partner_country_is_chile = country.code == "CL" or identification_type.country_id.code == "CL"
         if partner_country_is_chile and \
                 values.get('l10n_latam_identification_type_id') in identification_types and values.get('vat'):
             return stdnum.util.get_cc_module('cl', 'vat').format(values['vat']).replace('.', '').replace(
@@ -41,6 +42,11 @@ class ResPartner(models.Model):
         else:
             return values['vat']
 
+    def _format_dotted_vat_cl(self, vat):
+        vat_l = vat.split('-')
+        n_vat, n_dv = vat_l[0], vat_l[1]
+        return '%s-%s' % (format(int(n_vat), ',d').replace(',', '.'), n_dv)
+
     @api.model
     def create(self, values):
         if values.get('vat'):
@@ -48,12 +54,13 @@ class ResPartner(models.Model):
         return super().create(values)
 
     def write(self, values):
-        for record in self:
-            vat_values = {
-                'vat': values.get('vat', record.vat),
-                'l10n_latam_identification_type_id': values.get(
-                    'l10n_latam_identification_type_id', record.l10n_latam_identification_type_id.id),
-                'country_id': values.get('country_id', record.country_id.id)
-            }
-            values['vat'] = self._format_vat_cl(vat_values)
+        if any(field in values for field in ['vat', 'l10n_latam_identification_type_id', 'country_id']):
+            for record in self:
+                vat_values = {
+                    'vat': values.get('vat', record.vat),
+                    'l10n_latam_identification_type_id': values.get(
+                        'l10n_latam_identification_type_id', record.l10n_latam_identification_type_id.id),
+                    'country_id': values.get('country_id', record.country_id.id)
+                }
+                values['vat'] = self._format_vat_cl(vat_values)
         return super().write(values)
