@@ -5,7 +5,8 @@
 {
     'name': 'China - City Data',
     'version': '1.8',
-    'category': 'Localization',
+    'icon': '/l10n_cn/static/description/icon.png',
+    'category': 'Accounting/Localizations',
     'author': 'Jeffery Chen Fan<jeffery9@gmail.com>',
     'description': """
 Includes the following data for the Chinese localization
