@@ -33,10 +33,11 @@
 
 {
     'name': 'Germany SKR03 - Accounting',
+    'icon': '/l10n_de/static/description/icon.png',
     'version': '3.0',
     'author': 'openbig.org',
     'website': 'http://www.openbig.org',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'description': """
 Dieses  Modul beinhaltet einen deutschen Kontenrahmen basierend auf dem SKR03.
 ==============================================================================
@@ -48,11 +49,14 @@ German accounting chart and localization.
         'data/l10n_de_skr03_chart_data.xml',
         'data/account.account.template.csv',
         'data/l10n_de_skr03_chart_post_data.xml',
-        'data/account_data.xml',
+        'data/account_tax_group_data.xml',
         'data/account_tax_fiscal_position_data.xml',
         'data/account_reconcile_model_template.xml',
         'data/account_chart_template_data.xml',
     ],
+    'demo': [
+        'demo/demo_company.xml',
+    ],
     'auto_install': True,
     'license': 'LGPL-3',
 }
