@@ -16,14 +16,14 @@ Note: The Schedule VI has been revised by MCA and is applicable for all Balance
 31st March, 2011. The Format has done away with earlier two options of format of Balance
 Sheet, now only Vertical format has been permitted Which is Supported By Odoo.
   """,
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'depends': [
-        'account_tax_python',
+        'account_tax_python', 'base_vat',
     ],
     'data': [
         'security/l10n_in_security.xml',
         'security/ir.model.access.csv',
-        'data/account_data.xml',
+        'data/account_tax_group_data.xml',
         'data/l10n_in_chart_data.xml',
         'data/account.account.template.csv',
         'data/l10n_in_chart_post_data.xml',
@@ -47,10 +47,11 @@ Sheet, now only Vertical format has been permitted Which is Supported By Odoo.
         'data/account_chart_template_data.xml'
     ],
     'demo': [
-        'data/res_partner_demo.xml',
-        'data/product_demo.xml',
-        'data/account_payment_demo.xml',
-        'data/account_invoice_demo.xml'
+        'demo/demo_company.xml',
+        'demo/res_partner_demo.xml',
+        'demo/product_demo.xml',
+        'demo/account_payment_demo.xml',
+        'demo/account_invoice_demo.xml',
     ],
     'license': 'LGPL-3',
 }
