@@ -3,7 +3,7 @@
 
 {
     'name': 'Italy - Accounting',
-    'version': '0.2',
+    'version': '0.3',
     'depends': [
         'account',
         'base_iban',
@@ -16,17 +16,21 @@ Piano dei conti italiano di un'impresa generica.
 
 Italian accounting chart and localization.
     """,
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'website': 'http://www.odoo.com/',
     'data': [
-        'data/l10n_it_chart_data.xml',
+        'data/account_account_tag.xml',
+        'data/account_chart_template.xml',
         'data/account.account.template.csv',
+        'data/account_chart_template_accounts.xml',
         'data/account.tax.group.csv',
         'data/account_tax_report_data.xml',
         'data/account_tax_template.xml',
         'data/account.fiscal.position.template.csv',
-        'data/account.chart.template.csv',
         'data/account_chart_template_data.xml',
-        ],
+    ],
+    'demo': [
+        'demo/demo_company.xml',
+    ],
     'license': 'LGPL-3',
 }
