@@ -22,6 +22,15 @@ class AccountAccount(models.Model):
 
     name = fields.Char(translate=True)
 
+class AccountGroupTemplate(models.Model):
+    _inherit = 'account.group.template'
+
+    name = fields.Char(translate=True)
+
+class AccountGroup(models.Model):
+    _inherit = 'account.group'
+
+    name = fields.Char(translate=True)
 
 class AccountTax(models.Model):
     _inherit = 'account.tax'
@@ -49,7 +58,7 @@ class AccountFiscalPosition(models.Model):
     _inherit = 'account.fiscal.position'
 
     name = fields.Char(translate=True)
-    note = fields.Text(translate=True)
+    note = fields.Html(translate=True)
 
 
 class AccountFiscalPositionTemplate(models.Model):
