@@ -6,7 +6,7 @@
 {
     'name': 'Netherlands - Accounting',
     'version': '3.0',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'author': 'Onestein',
     'website': 'http://www.onestein.eu',
     'depends': [
@@ -20,7 +20,7 @@
         'data/account_chart_template.xml',
         'data/account.account.template.csv',
         'data/account_chart_template_post_data.xml',
-        'data/account_data.xml',
+        'data/account_tax_group_data.xml',
         'data/account_tax_report_data.xml',
         'data/account_tax_template.xml',
         'data/account_fiscal_position_template.xml',
@@ -28,8 +28,12 @@
         'data/account_fiscal_position_account_template.xml',
         'data/account_chart_template_data.xml',
         'data/menuitem.xml',
+        'views/res_partner_views.xml',
+        'views/res_company_views.xml',
+    ],
+    'demo': [
+        'demo/demo_company.xml',
     ],
-    'demo': [],
     'auto_install': False,
     'installable': True,
     'license': 'LGPL-3',
