@@ -8,7 +8,7 @@
     'version' : '2.0',
     'author' : 'Grzegorz Grzelak (OpenGLOBE)',
     'website': 'http://www.openglobe.pl',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'description': """
 This is the module to manage the accounting chart and taxes for Poland in Odoo.
 ==================================================================================
@@ -28,13 +28,17 @@ Wewnętrzny numer wersji OpenGLOBE 1.02
     'data': [
               'data/l10n_pl_chart_data.xml',
               'data/account.account.template.csv',
+              'data/res.country.state.csv',
               'data/l10n_pl_chart_post_data.xml',
-              'data/account_data.xml',
+              'data/account_tax_group_data.xml',
               'data/account_tax_report_data.xml',
               'data/account_tax_data.xml',
               'data/account_fiscal_position_data.xml',
               'data/account_chart_template_data.xml'
     ],
+    'demo': [
+        'demo/demo_company.xml',
+    ],
     'post_init_hook': '_preserve_tag_on_taxes',
     'license': 'LGPL-3',
 }
