@@ -7,7 +7,7 @@
     'name': 'Singapore - Accounting',
     'author': 'Tech Receptives',
     'version': '2.0',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'description': """
 Singapore accounting chart and localization.
 =======================================================
@@ -21,7 +21,7 @@ This module add, for accounting:
     'depends': ['base', 'account'],
     'data': [
         'data/l10n_sg_chart_data.xml',
-        'data/account_data.xml',
+        'data/account_tax_group_data.xml',
         'data/account_tax_report_data.xml',
         'data/account_tax_data.xml',
         'data/account_chart_template_data.xml',
