@@ -4,7 +4,7 @@
 {
     'name': 'Thailand - Accounting',
     'version': '2.0',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'description': """
 Chart of Accounts for Thailand.
 ===============================
@@ -15,7 +15,7 @@ Thai accounting chart and localization.
     'website': 'http://almacom.co.th/',
     'depends': ['account'],
     'data': [
-        'data/account_data.xml',
+        'data/account_tax_group_data.xml',
         'data/l10n_th_chart_data.xml',
         'data/account.account.template.csv',
         'data/l10n_th_chart_post_data.xml',
@@ -23,6 +23,9 @@ Thai accounting chart and localization.
         'data/account_tax_template_data.xml',
         'data/account_chart_template_data.xml',
     ],
+    'demo': [
+        'demo/demo_company.xml',
+    ],
     'post_init_hook': '_preserve_tag_on_taxes',
     'license': 'LGPL-3',
 }
