@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import models
+from . import tools
 from . import wizard
 from . import controllers
-from . import validation
