@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+import ast
 import base64
 import datetime
 import logging
@@ -14,7 +15,6 @@ from collections import defaultdict
 from odoo import _, api, fields, models
 from odoo import tools
 from odoo.addons.base.models.ir_mail_server import MailDeliveryException
-from odoo.tools.safe_eval import safe_eval
 
 _logger = logging.getLogger(__name__)
 
@@ -28,6 +28,16 @@ class MailMail(models.Model):
     _order = 'id desc'
     _rec_name = 'subject'
 
+    @api.model
+    def default_get(self, fields):
+        # protection for `default_type` values leaking from menu action context (e.g. for invoices)
+        # To remove when automatic context propagation is removed in web client
+        if self._context.get('default_type') not in type(self).message_type.base_field.selection:
+            self = self.with_context(dict(self._context, default_type=None))
+        if self._context.get('default_state') not in type(self).state.base_field.selection:
+            self = self.with_context(dict(self._context, default_state='outgoing'))
+        return super(MailMail, self).default_get(fields)
+
     # content
     mail_message_id = fields.Many2one('mail.message', 'Message', required=True, ondelete='cascade', index=True, auto_join=True)
     body_html = fields.Text('Rich-text Contents', help="Rich-text/HTML message")
@@ -35,7 +45,7 @@ class MailMail(models.Model):
     headers = fields.Text('Headers', copy=False)
     # Auto-detected based on create() - if 'mail_message_id' was passed then this mail is a notification
     # and during unlink() we will not cascade delete the parent and its attachments
-    notification = fields.Boolean('Is Notification', help='Mail has been created to notify people of an existing mail.message')
+    is_notification = fields.Boolean('Notification Email', help='Mail has been created to notify people of an existing mail.message')
     # recipients: include inactive partners (they may have been archived after
     # the message was sent, but they should remain visible in the relation)
     email_to = fields.Text('To', help='Message recipients (emails)')
@@ -50,24 +60,44 @@ class MailMail(models.Model):
         ('exception', 'Delivery Failed'),
         ('cancel', 'Cancelled'),
     ], 'Status', readonly=True, copy=False, default='outgoing')
+    failure_type = fields.Selection(selection=[
+        # generic
+        ("unknown", "Unknown error"),
+        # mail
+        ("mail_email_invalid", "Invalid email address"),
+        ("mail_email_missing", "Missing email"),
+        ("mail_smtp", "Connection failed (outgoing mail server problem)"),
+        # mass mode
+        ("mail_bl", "Blacklisted Address"),
+        ("mail_optout", "Opted Out"),
+        ("mail_dup", "Duplicated Email"),
+        ], string='Failure type')
+    failure_reason = fields.Text(
+        'Failure Reason', readonly=1, copy=False,
+        help="Failure reason. This is usually the exception thrown by the email server, stored to ease the debugging of mailing issues.")
     auto_delete = fields.Boolean(
         'Auto Delete',
-        help="Permanently delete this email after sending it, to save space")
-    failure_reason = fields.Text(
-        'Failure Reason', readonly=1,
-        help="Failure reason. This is usually the exception thrown by the email server, stored to ease the debugging of mailing issues.")
+        help="This option permanently removes any track of email after it's been sent, including from the Technical menu in the Settings, in order to preserve storage space of your Odoo database.")
     scheduled_date = fields.Char('Scheduled Send Date',
         help="If set, the queue manager will send the email after the date. If not set, the email will be send as soon as possible.")
 
-    @api.model
-    def create(self, values):
+    @api.model_create_multi
+    def create(self, values_list):
         # notification field: if not set, set if mail comes from an existing mail.message
-        if 'notification' not in values and values.get('mail_message_id'):
-            values['notification'] = True
-        new_mail = super(MailMail, self).create(values)
-        if values.get('attachment_ids'):
-            new_mail.attachment_ids.check(mode='read')
-        return new_mail
+        for values in values_list:
+            if 'is_notification' not in values and values.get('mail_message_id'):
+                values['is_notification'] = True
+
+        new_mails = super(MailMail, self).create(values_list)
+
+        new_mails_w_attach = self
+        for mail, values in zip(new_mails, values_list):
+            if values.get('attachment_ids'):
+                new_mails_w_attach += mail
+        if new_mails_w_attach:
+            new_mails_w_attach.mapped('attachment_ids').check(mode='read')
+
+        return new_mails
 
     def write(self, vals):
         res = super(MailMail, self).write(vals)
@@ -78,21 +108,14 @@ class MailMail(models.Model):
 
     def unlink(self):
         # cascade-delete the parent message for all mails that are not created for a notification
-        mail_msg_cascade_ids = [mail.mail_message_id.id for mail in self if not mail.notification]
+        mail_msg_cascade_ids = [mail.mail_message_id.id for mail in self if not mail.is_notification]
         res = super(MailMail, self).unlink()
         if mail_msg_cascade_ids:
             self.env['mail.message'].browse(mail_msg_cascade_ids).unlink()
         return res
 
-    @api.model
-    def default_get(self, fields):
-        # protection for `default_type` values leaking from menu action context (e.g. for invoices)
-        # To remove when automatic context propagation is removed in web client
-        if self._context.get('default_type') not in type(self).message_type.base_field.selection:
-            self = self.with_context(dict(self._context, default_type=None))
-        if self._context.get('default_state') not in type(self).state.base_field.selection:
-            self = self.with_context(dict(self._context, default_state='outgoing'))
-        return super(MailMail, self).default_get(fields)
+    def action_retry(self):
+        self.filtered(lambda mail: mail.state == 'exception').mark_outgoing()
 
     def mark_outgoing(self):
         return self.write({'state': 'outgoing'})
@@ -147,11 +170,11 @@ class MailMail(models.Model):
 
         :return: True
         """
-        notif_mails_ids = [mail.id for mail in self if mail.notification]
+        notif_mails_ids = [mail.id for mail in self if mail.is_notification]
         if notif_mails_ids:
             notifications = self.env['mail.notification'].search([
                 ('notification_type', '=', 'email'),
-                ('mail_id', 'in', notif_mails_ids),
+                ('mail_mail_id', 'in', notif_mails_ids),
                 ('notification_status', 'not in', ('sent', 'canceled'))
             ])
             if notifications:
@@ -159,19 +182,21 @@ class MailMail(models.Model):
                 failed = self.env['mail.notification']
                 if failure_type:
                     failed = notifications.filtered(lambda notif: notif.res_partner_id not in success_pids)
+                (notifications - failed).sudo().write({
+                    'notification_status': 'sent',
+                    'failure_type': '',
+                    'failure_reason': '',
+                })
+                if failed:
                     failed.sudo().write({
                         'notification_status': 'exception',
                         'failure_type': failure_type,
                         'failure_reason': failure_reason,
                     })
                     messages = notifications.mapped('mail_message_id').filtered(lambda m: m.is_thread_message())
-                    messages._notify_mail_failure_update()  # notify user that we have a failure
-                (notifications - failed).sudo().write({
-                    'notification_status': 'sent',
-                    'failure_type': '',
-                    'failure_reason': '',
-                })
-        if not failure_type or failure_type == 'RECIPIENT':  # if we have another error, we want to keep the mail.
+                    # TDE TODO: could be great to notify message-based, not notifications-based, to lessen number of notifs
+                    messages._notify_message_notification_update()  # notify user that we have a failure
+        if not failure_type or failure_type in ['mail_email_invalid', 'mail_email_missing']:  # if we have another error, we want to keep the mail.
             mail_to_delete_ids = [mail.id for mail in self if mail.auto_delete]
             self.browse(mail_to_delete_ids).sudo().unlink()
         return True
@@ -206,22 +231,47 @@ class MailMail(models.Model):
         }
         return res
 
-    def _split_by_server(self):
-        """Returns an iterator of pairs `(mail_server_id, record_ids)` for current recordset.
+    def _split_by_mail_configuration(self):
+        """Group the <mail.mail> based on their "email_from" and their "mail_server_id".
 
-        The same `mail_server_id` may repeat in order to limit batch size according to
-        the `mail.session.batch.size` system parameter.
+        The <mail.mail> will have the "same sending configuration" if they have the same
+        mail server or the same mail from. For performance purpose, we can use an SMTP
+        session in batch and therefore we need to group them by the parameter that will
+        influence the mail server used.
+
+        The same "sending configuration" may repeat in order to limit batch size
+        according to the `mail.session.batch.size` system parameter.
+
+        Return iterators over
+            mail_server_id, email_from, Records<mail.mail>.ids
         """
-        groups = defaultdict(list)
-        # Turn prefetch OFF to avoid MemoryError on very large mail queues, we only care
-        # about the mail server ids in this case.
-        for mail in self.with_context(prefetch_fields=False):
-            groups[mail.mail_server_id.id].append(mail.id)
+        mail_values = self.read(['id', 'email_from', 'mail_server_id'])
+
+        # First group the <mail.mail> per mail_server_id and per email_from
+        group_per_email_from = defaultdict(list)
+        for values in mail_values:
+            mail_server_id = values['mail_server_id'][0] if values['mail_server_id'] else False
+            group_per_email_from[(mail_server_id, values['email_from'])].append(values['id'])
+
+        # Then find the mail server for each email_from and group the <mail.mail>
+        # per mail_server_id and smtp_from
+        mail_servers = self.env['ir.mail_server'].sudo().search([], order='sequence')
+        group_per_smtp_from = defaultdict(list)
+        for (mail_server_id, email_from), mail_ids in group_per_email_from.items():
+            if not mail_server_id:
+                mail_server, smtp_from = self.env['ir.mail_server']._find_mail_server(email_from, mail_servers)
+                mail_server_id = mail_server.id if mail_server else False
+            else:
+                smtp_from = email_from
+
+            group_per_smtp_from[(mail_server_id, smtp_from)].extend(mail_ids)
+
         sys_params = self.env['ir.config_parameter'].sudo()
         batch_size = int(sys_params.get_param('mail.session.batch.size', 1000))
-        for server_id, record_ids in groups.items():
-            for mail_batch in tools.split_every(batch_size, record_ids):
-                yield server_id, mail_batch
+
+        for (mail_server_id, smtp_from), record_ids in group_per_smtp_from.items():
+            for batch_ids in tools.split_every(batch_size, record_ids):
+                yield mail_server_id, smtp_from, batch_ids
 
     def send(self, auto_commit=False, raise_exception=False):
         """ Sends the selected emails immediately, ignoring their current
@@ -238,10 +288,10 @@ class MailMail(models.Model):
                 email sending process has failed
             :return: True
         """
-        for server_id, batch_ids in self._split_by_server():
+        for mail_server_id, smtp_from, batch_ids in self._split_by_mail_configuration():
             smtp_session = None
             try:
-                smtp_session = self.env['ir.mail_server'].connect(mail_server_id=server_id)
+                smtp_session = self.env['ir.mail_server'].connect(mail_server_id=mail_server_id, smtp_from=smtp_from)
             except Exception as exc:
                 if raise_exception:
                     # To be consistent and backward compatible with mail_mail.send() raised
@@ -250,7 +300,7 @@ class MailMail(models.Model):
                 else:
                     batch = self.browse(batch_ids)
                     batch.write({'state': 'exception', 'failure_reason': exc})
-                    batch._postprocess_sent_message(success_pids=[], failure_type="SMTP")
+                    batch._postprocess_sent_message(success_pids=[], failure_type="mail_smtp")
             else:
                 self.browse(batch_ids)._send(
                     auto_commit=auto_commit,
@@ -258,7 +308,7 @@ class MailMail(models.Model):
                     smtp_session=smtp_session)
                 _logger.info(
                     'Sent batch %s emails via mail server ID #%s',
-                    len(batch_ids), server_id)
+                    len(batch_ids), mail_server_id)
             finally:
                 if smtp_session:
                     smtp_session.quit()
@@ -303,18 +353,12 @@ class MailMail(models.Model):
                 headers = {}
                 ICP = self.env['ir.config_parameter'].sudo()
                 bounce_alias = ICP.get_param("mail.bounce.alias")
-                bounce_alias_static = tools.str2bool(ICP.get_param("mail.bounce.alias.static", "False"))
                 catchall_domain = ICP.get_param("mail.catchall.domain")
                 if bounce_alias and catchall_domain:
-                    if bounce_alias_static:
-                        headers['Return-Path'] = '%s@%s' % (bounce_alias, catchall_domain)
-                    elif mail.mail_message_id.is_thread_message():
-                        headers['Return-Path'] = '%s+%d-%s-%d@%s' % (bounce_alias, mail.id, mail.model, mail.res_id, catchall_domain)
-                    else:
-                        headers['Return-Path'] = '%s+%d@%s' % (bounce_alias, mail.id, catchall_domain)
+                    headers['Return-Path'] = '%s@%s' % (bounce_alias, catchall_domain)
                 if mail.headers:
                     try:
-                        headers.update(safe_eval(mail.headers))
+                        headers.update(ast.literal_eval(mail.headers))
                     except Exception:
                         pass
 
@@ -330,14 +374,14 @@ class MailMail(models.Model):
                 # mail record.
                 notifs = self.env['mail.notification'].search([
                     ('notification_type', '=', 'email'),
-                    ('mail_id', 'in', mail.ids),
+                    ('mail_mail_id', 'in', mail.ids),
                     ('notification_status', 'not in', ('sent', 'canceled'))
                 ])
                 if notifs:
                     notif_msg = _('Error without exception. Probably due do concurrent access update of notification records. Please see with an administrator.')
                     notifs.sudo().write({
                         'notification_status': 'exception',
-                        'failure_type': 'UNKNOWN',
+                        'failure_type': 'unknown',
                         'failure_reason': notif_msg,
                     })
                     # `test_mail_bounce_during_send`, force immediate update to obtain the lock.
@@ -346,6 +390,8 @@ class MailMail(models.Model):
 
                 # build an RFC2822 email.message.Message object and send it without queuing
                 res = None
+                # TDE note: could be great to pre-detect missing to/cc and skip sending it
+                # to go directly to failed state update
                 for email in email_list:
                     msg = IrMailServer.build_email(
                         email_from=mail.email_from,
@@ -371,7 +417,11 @@ class MailMail(models.Model):
                         processing_pid = None
                     except AssertionError as error:
                         if str(error) == IrMailServer.NO_VALID_RECIPIENT:
-                            failure_type = "RECIPIENT"
+                            # if we have a list of void emails for email_list -> email missing, otherwise generic email failure
+                            if not email.get('email_to') and failure_type != "mail_email_invalid":
+                                failure_type = "mail_email_missing"
+                            else:
+                                failure_type = "mail_email_invalid"
                             # No valid recipient found for this particular
                             # mail item -> ignore error to avoid blocking
                             # delivery to next recipients, if any. If this is
@@ -380,7 +430,7 @@ class MailMail(models.Model):
                                          mail.message_id, email.get('email_to'))
                         else:
                             raise
-                if res:  # mail has been sent at least once, no major exception occured
+                if res:  # mail has been sent at least once, no major exception occurred
                     mail.write({'state': 'sent', 'message_id': res, 'failure_reason': False})
                     _logger.info('Mail with ID %r and Message-Id %r successfully sent', mail.id, mail.message_id)
                     # /!\ can't use mail.state here, as mail.refresh() will cause an error
@@ -405,16 +455,14 @@ class MailMail(models.Model):
                 failure_reason = tools.ustr(e)
                 _logger.exception('failed sending mail (id: %s) due to %s', mail.id, failure_reason)
                 mail.write({'state': 'exception', 'failure_reason': failure_reason})
-                mail._postprocess_sent_message(success_pids=success_pids, failure_reason=failure_reason, failure_type='UNKNOWN')
+                mail._postprocess_sent_message(success_pids=success_pids, failure_reason=failure_reason, failure_type='unknown')
                 if raise_exception:
                     if isinstance(e, (AssertionError, UnicodeEncodeError)):
                         if isinstance(e, UnicodeEncodeError):
                             value = "Invalid text: %s" % e.object
                         else:
-                            # get the args of the original error, wrap into a value and throw a MailDeliveryException
-                            # that is an except_orm, with name and value as arguments
                             value = '. '.join(e.args)
-                        raise MailDeliveryException(_("Mail Delivery Failed"), value)
+                        raise MailDeliveryException(value)
                     raise
 
             if auto_commit is True:
