@@ -12,8 +12,7 @@ class MailTracking(models.Model):
     _rec_name = 'field'
     _order = 'tracking_sequence asc'
 
-    # TDE CLEANME: why not a m2o to ir model field ?
-    field = fields.Char('Changed Field', required=True, readonly=1)
+    field = fields.Many2one('ir.model.fields', required=True, readonly=1, ondelete='cascade')
     field_desc = fields.Char('Field Description', required=True, readonly=1)
     field_type = fields.Char('Field Type')
     field_groups = fields.Char(compute='_compute_field_groups')
@@ -32,6 +31,9 @@ class MailTracking(models.Model):
     new_value_text = fields.Text('New Value Text', readonly=1)
     new_value_datetime = fields.Datetime('New Value Datetime', readonly=1)
 
+    currency_id = fields.Many2one('res.currency', 'Currency', readonly=True, ondelete='set null',
+        help="Used to display the currency when tracking monetary values")
+
     mail_message_id = fields.Many2one('mail.message', 'Message ID', required=True, index=True, ondelete='cascade')
 
     tracking_sequence = fields.Integer('Tracking field sequence', readonly=1, default=100)
@@ -39,13 +41,18 @@ class MailTracking(models.Model):
     def _compute_field_groups(self):
         for tracking in self:
             model = self.env[tracking.mail_message_id.model]
-            field = model._fields.get(tracking.field)
+            field = model._fields.get(tracking.field.name)
             tracking.field_groups = field.groups if field else 'base.group_system'
 
     @api.model
-    def create_tracking_values(self, initial_value, new_value, col_name, col_info, tracking_sequence):
+    def create_tracking_values(self, initial_value, new_value, col_name, col_info, tracking_sequence, model_name):
         tracked = True
-        values = {'field': col_name, 'field_desc': col_info['string'], 'field_type': col_info['type'], 'tracking_sequence': tracking_sequence}
+
+        field = self.env['ir.model.fields']._get(model_name, col_name)
+        if not field:
+            return
+
+        values = {'field': field.id, 'field_desc': col_info['string'], 'field_type': col_info['type'], 'tracking_sequence': tracking_sequence}
 
         if col_info['type'] in ['integer', 'float', 'char', 'text', 'datetime', 'monetary']:
             values.update({
