@@ -1,14 +1,21 @@
-odoo.define('mail.DocumentViewer', function (require) {
-"use strict";
+/** @odoo-module **/
 
-var core = require('web.core');
-var Widget = require('web.Widget');
+import core from 'web.core';
+import Widget from 'web.Widget';
+import { hidePDFJSButtons } from '@web/legacy/js/libs/pdfjs';
 
 var QWeb = core.qweb;
 
 var SCROLL_ZOOM_STEP = 0.1;
 var ZOOM_STEP = 0.5;
 
+/**
+ * This widget is deprecated, and should instead use AttachmentViewer component.
+ * @see `mail/static/src/components/attachment_viewer/attachment_viewer.js`
+ * TODO: remove this widget when it's not longer used
+ *
+ * @deprecated
+ */
 var DocumentViewer = Widget.extend({
     template: "DocumentViewer",
     events: {
@@ -68,6 +75,15 @@ var DocumentViewer = Widget.extend({
         this.modelName = 'ir.attachment';
         this._reset();
     },
+    /**
+     * Do some actions after the widget is appended to the DOM
+     * @override
+     */
+    setElement: function () {
+        const result = this._super(...arguments);
+        this._hidePdfButtonsIfPresent();
+        return result;
+    },
     /**
      * Open a modal displaying the active attachment
      * @override
@@ -86,7 +102,6 @@ var DocumentViewer = Widget.extend({
         if (this.isDestroyed()) {
             return;
         }
-        this.trigger_up('document_viewer_closed');
         this.$el.modal('hide');
         this.$el.remove();
         this._super.apply(this, arguments);
@@ -96,6 +111,15 @@ var DocumentViewer = Widget.extend({
     // Private
     //---------------------------------------------------------------------------
 
+    /**
+     * Hide some buttons in PDF.js
+     * @override
+     */
+    _hidePdfButtonsIfPresent: function () {
+        if (this.activeAttachment.mimetype === 'application/pdf') {
+            hidePDFJSButtons(this.el);
+        }
+    },
     /**
      * @private
      */
@@ -132,6 +156,7 @@ var DocumentViewer = Widget.extend({
             widget: this
         }));
         this.$('.o_viewer_img').on("load", _.bind(this._onImageLoaded, this));
+        this._hidePdfButtonsIfPresent();
         this.$('[data-toggle="tooltip"]').tooltip({delay: 0});
         this._reset();
     },
@@ -386,5 +411,5 @@ var DocumentViewer = Widget.extend({
         this._zoom(1);
     },
 });
-return DocumentViewer;
-});
+
+export default DocumentViewer;
