@@ -1,10 +1,16 @@
-odoo.define('mail.activity_view_tests', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var ActivityView = require('mail.ActivityView');
-var testUtils = require('web.test_utils');
+import ActivityRenderer from '@mail/js/views/activity/activity_renderer';
+import ActivityView from '@mail/js/views/activity/activity_view';
+import testUtils from 'web.test_utils';
+import domUtils from 'web.dom';
 
-var createActionManager = testUtils.createActionManager;
+import { legacyExtraNextTick, patchWithCleanup } from "@web/../tests/helpers/utils";
+import { createWebClient, doAction } from "@web/../tests/webclient/helpers";
+import { session } from '@web/session';
+import { click } from "@web/../tests/helpers/utils";
+
+let serverData;
 
 var createView = testUtils.createView;
 
@@ -109,6 +115,7 @@ QUnit.module('activity view', {
                 ],
             },
         };
+        serverData = { models: this.data };
     }
 });
 
@@ -189,12 +196,11 @@ QUnit.test('activity view: simple activity rendering', async function (assert) {
 QUnit.test('activity view: no content rendering', async function (assert) {
     assert.expect(2);
 
-    // remove activities from tasks
+    // reset incompatible setup
+    this.data['mail.activity'].records = [];
     this.data.task.records.forEach(function (task) {
         task.activity_ids = false;
     });
-
-    // remove activity types
     this.data['mail.activity.type'].records = [];
 
     var activity = await createView({
@@ -263,7 +269,8 @@ QUnit.test('activity view: batch send mail on activity', async function (assert)
 
 QUnit.test('activity view: activity widget', async function (assert) {
     assert.expect(16);
-    var activity = await createView({
+
+    const params = {
         View: ActivityView,
         model: 'task',
         data: this.data,
@@ -304,6 +311,7 @@ QUnit.test('activity view: activity widget', async function (assert) {
                     assert.step("do_action_compose");
                 } else if (action.res_model === 'mail.activity') {
                     assert.deepEqual({
+                        "default_activity_type_id": 2,
                         "default_res_id": 30,
                         "default_res_model": "task"
                     }, action.context);
@@ -313,7 +321,9 @@ QUnit.test('activity view: activity widget', async function (assert) {
                 }
             },
         },
-    });
+    };
+
+    var activity = await createView(params);
     var today = activity.$('table tbody tr:first td:nth-child(2).today');
     var dropdown = today.find('.dropdown-menu.o_activity');
 
@@ -349,49 +359,62 @@ QUnit.test('activity view: activity widget', async function (assert) {
 
     activity.destroy();
 });
-QUnit.test('activity view: no group_by_menu and no time_range_menu', async function (assert) {
+
+QUnit.test("activity view: no group_by_menu and no comparison_menu", async function (assert) {
     assert.expect(4);
 
-    var actionManager = await createActionManager({
-        actions: [{
+    serverData.actions = {
+        1: {
             id: 1,
-            name: 'Task Action',
-            res_model: 'task',
-            type: 'ir.actions.act_window',
-            views: [[false, 'activity']],
-        }],
-        archs: {
-            'task,false,activity': '<activity string="Task">' +
-                                    '<templates>' +
-                                        '<div t-name="activity-box">' +
-                                            '<field name="foo"/>' +
-                                        '</div>' +
-                                    '</templates>' +
-                                '</activity>',
-            'task,false,search': '<search></search>',
+            name: "Task Action",
+            res_model: "task",
+            type: "ir.actions.act_window",
+            views: [[false, "activity"]],
         },
-        data: this.data,
-        session: {
-            user_context: {lang: 'zz_ZZ'},
-        },
-        mockRPC: function(route, args) {
-            if (args.method === 'get_activity_data') {
-                assert.deepEqual(args.kwargs.context, {lang: 'zz_ZZ'},
-                    'The context should have been passed');
-            }
-            return this._super.apply(this, arguments);
-        },
-    });
+    };
 
-    await actionManager.doAction(1);
+    serverData.views = {
+        "task,false,activity":
+            '<activity string="Task">' +
+            "<templates>" +
+            '<div t-name="activity-box">' +
+            '<field name="foo"/>' +
+            "</div>" +
+            "</templates>" +
+            "</activity>",
+        "task,false,search": "<search></search>",
+    };
 
-    assert.containsN(actionManager, '.o_search_options .o_dropdown button:visible', 2,
-        "only two elements should be available in view search");
-    assert.isVisible(actionManager.$('.o_search_options .o_dropdown button.o_filters_menu_button'),
-        "filter should be available in view search");
-    assert.isVisible(actionManager.$('.o_search_options .o_dropdown button.o_favorites_menu_button'),
-        "favorites should be available in view search");
-    actionManager.destroy();
+    const mockRPC = (route, args) => {
+        if (args.method === "get_activity_data") {
+            assert.strictEqual(
+                args.kwargs.context.lang,
+                "zz_ZZ",
+                "The context should have been passed"
+            );
+        }
+    };
+
+    patchWithCleanup(session.user_context, { lang: "zz_ZZ" });
+
+    const webClient = await createWebClient({ serverData, mockRPC , legacyParams: {withLegacyMockServer: true}});
+
+    await doAction(webClient, 1);
+
+    assert.containsN(
+        webClient,
+        ".o_search_options .dropdown button:visible",
+        2,
+        "only two elements should be available in view search"
+    );
+    assert.isVisible(
+        $(webClient.el).find(".o_search_options .dropdown.o_filter_menu > button"),
+        "filter should be available in view search"
+    );
+    assert.isVisible(
+        $(webClient.el).find(".o_search_options .dropdown.o_favorite_menu > button"),
+        "favorites should be available in view search"
+    );
 });
 
 QUnit.test('activity view: search more to schedule an activity for a record of a respecting model', async function (assert) {
@@ -457,64 +480,264 @@ QUnit.test('activity view: search more to schedule an activity for a record of a
     activity.destroy();
 });
 
-QUnit.test('Activity view: discard an activity creation dialog', async function (assert) {
+QUnit.test("Activity view: discard an activity creation dialog", async function (assert) {
     assert.expect(2);
 
-    var actionManager = await createActionManager({
-        actions: [{
+    serverData.actions = {
+        1: {
+            id: 1,
+            name: "Task Action",
+            res_model: "task",
+            type: "ir.actions.act_window",
+            views: [[false, "activity"]],
+        },
+    };
+
+    serverData.views = {
+        "task,false,activity": `
+        <activity string="Task">
+            <templates>
+                <div t-name="activity-box">
+                    <field name="foo"/>
+                </div>
+            </templates>
+        </activity>`,
+        "task,false,search": "<search></search>",
+        "mail.activity,false,form": `
+        <form>
+            <field name="display_name"/>
+            <footer>
+                <button string="Discard" class="btn-secondary" special="cancel"/>
+            </footer>
+        </form>`,
+    };
+
+    const mockRPC = (route, args) => {
+        if (args.method === "check_access_rights") {
+            return true;
+        }
+    };
+
+    const webClient = await createWebClient({ serverData, mockRPC, legacyParams: {withLegacyMockServer: true} });
+    await doAction(webClient, 1);
+
+    await testUtils.dom.click(
+        $(webClient.el).find(".o_activity_view .o_data_row .o_activity_empty_cell")[0]
+    );
+    await legacyExtraNextTick();
+    assert.containsOnce($, ".modal.o_technical_modal", "Activity Modal should be opened");
+
+    await testUtils.dom.click($('.modal.o_technical_modal button[special="cancel"]'));
+    await legacyExtraNextTick();
+    assert.containsNone($, ".modal.o_technical_modal", "Activity Modal should be closed");
+});
+
+QUnit.test('Activity view: many2one_avatar_user widget in activity view', async function (assert) {
+    assert.expect(3);
+
+    const taskModel = serverData.models.task;
+
+    serverData.models['res.users'] = {
+        fields: {
+            display_name: { string: "Displayed name", type: "char" },
+            avatar_128: { string: "Image 128", type: 'image' },
+        },
+        records: [{
+            id: 1,
+            display_name: "first user",
+            avatar_128: "Atmaram Bhide",
+        }],
+    };
+    taskModel.fields.user_id = { string: "Related User", type: "many2one", relation: 'res.users' };
+    taskModel.records[0].user_id = 1;
+
+    serverData.actions = {
+        1: {
             id: 1,
             name: 'Task Action',
             res_model: 'task',
             type: 'ir.actions.act_window',
             views: [[false, 'activity']],
-        }],
-        archs: {
-            'task,false,activity': `
-                <activity string="Task">
-                    <templates>
-                        <div t-name="activity-box">
-                            <field name="foo"/>
-                        </div>
-                    </templates>
-                </activity>`,
-            'task,false,search': '<search></search>',
-            'mail.activity,false,form': `
-                <form>
-                    <field name="display_name"/>
-                    <footer>
-                        <button string="Discard" class="btn-secondary" special="cancel"/>
-                    </footer>
-                </form>`
-        },
+        }
+    };
+
+    serverData.views = {
+        'task,false,activity': `
+            <activity string="Task">
+                <templates>
+                    <div t-name="activity-box">
+                        <field name="user_id" widget="many2one_avatar_user"/>
+                        <field name="foo"/>
+                    </div>
+                </templates>
+            </activity>`,
+        'task,false,search': '<search></search>'
+    };
+
+    const webClient = await createWebClient({ serverData, legacyParams: { withLegacyMockServer: true } });
+    await doAction(webClient, 1);
+
+    await legacyExtraNextTick();
+    assert.containsN(webClient, '.o_m2o_avatar', 2);
+    assert.containsOnce(webClient, 'tr[data-res-id=13] .o_m2o_avatar > img[src="/web/image/res.users/1/avatar_128"]',
+        "should have m2o avatar image");
+    assert.containsNone(webClient, '.o_m2o_avatar > span',
+        "should not have text on many2one_avatar_user if onlyImage node option is passed");
+});
+
+QUnit.test("Activity view: on_destroy_callback doesn't crash", async function (assert) {
+    assert.expect(3);
+
+    const params = {
+        View: ActivityView,
+        model: 'task',
         data: this.data,
-        intercepts: {
-            do_action(ev) {
-                actionManager.doAction(ev.data.action, ev.data.options);
-            }
-        },
-        async mockRPC(route, args) {
-            if (args.method === 'check_access_rights') {
-                return true;
-            }
-            return this._super(...arguments);
+        arch: `<activity string="Task">
+                <templates>
+                    <div t-name="activity-box">
+                        <field name="foo"/>
+                    </div>
+                </templates>
+            </activity>`,
+    };
+
+    patchWithCleanup(ActivityRenderer.prototype, {
+        mounted() {
+            assert.step('mounted');
         },
+        willUnmount() {
+            assert.step('willUnmount');
+        }
     });
-    await actionManager.doAction(1);
 
-    await testUtils.dom.click(actionManager.$('.o_activity_view .o_data_row .o_activity_empty_cell')[0]);
-    assert.containsOnce(
-        $,
-        '.modal.o_technical_modal.show',
-        "Activity Modal should be opened");
+    const activity = await createView(params);
+    domUtils.detach([{ widget: activity }]);
 
-    await testUtils.dom.click($('.modal.o_technical_modal.show button[special="cancel"]'));
-    assert.containsNone(
-        $,
-        '.modal.o_technical_modal.show',
-        "Activity Modal should be closed");
+    assert.verifySteps([
+        'mounted',
+        'willUnmount'
+    ]);
 
-    actionManager.destroy();
+    activity.destroy();
+});
+
+QUnit.test("Schedule activity dialog uses the same search view as activity view", async function (assert) {
+    assert.expect(8);
+    serverData.models.task.records = [];
+    serverData.views = {
+        "task,false,activity": `
+            <activity string="Task">
+                <templates>
+                    <div t-name="activity-box">
+                        <field name="foo"/>
+                    </div>
+                </templates>
+            </activity>
+        `,
+        "task,false,list": `<list><field name="foo"/></list>`,
+        "task,false,search": `<search/>`,
+        'task,1,search': `<search/>`,
+    };
+
+    function mockRPC(route, args) {
+        if (args.method === "load_views") {
+            assert.step(JSON.stringify(args.kwargs.views));
+        } 
+    }
+
+    const webClient = await createWebClient({ serverData, mockRPC, legacyParams: {withLegacyMockServer: true} });
+
+    // open an activity view (with default search arch)
+    await doAction(webClient, {
+        name: 'Dashboard',
+        res_model: 'task',
+        type: 'ir.actions.act_window',
+        views: [[false, 'activity']],
+    });
+
+    assert.verifySteps([
+        '[[false,"activity"],[false,"search"]]',
+    ])
+
+    // click on "Schedule activity"
+    await click(webClient.el.querySelector(".o_activity_view .o_record_selector"));
+
+    assert.verifySteps([
+        '[[false,"list"],[false,"search"]]',
+    ])
+
+    // open an activity view (with search arch 1)
+    await doAction(webClient, {
+        name: 'Dashboard',
+        res_model: 'task',
+        type: 'ir.actions.act_window',
+        views: [[false, 'activity']],
+        search_view_id: [1,"search"],
+    });
+
+    assert.verifySteps([
+        '[[false,"activity"],[1,"search"]]',
+    ])
+
+    // click on "Schedule activity"
+    await click(webClient.el.querySelector(".o_activity_view .o_record_selector"));
+
+    assert.verifySteps([
+        '[[false,"list"],[1,"search"]]',
+    ])
+});
+
+QUnit.test('Activity view: apply progressbar filter', async function (assert) {
+    assert.expect(9);
+
+    serverData.actions = {
+        1: {
+            id: 1,
+            name: 'Task Action',
+            res_model: 'task',
+            type: 'ir.actions.act_window',
+            views: [[false, 'activity']],
+        }
+    };
+    serverData.views = {
+        'task,false,activity':
+            `<activity string="Task" >
+                <templates>
+                    <div t-name="activity-box">
+                        <field name="foo"/>
+                    </div>
+                </templates>
+            </activity>`,
+        'task,false,search': '<search></search>',
+    };
+
+    const webClient = await createWebClient({ serverData, legacyParams: { withLegacyMockServer: true } });
+
+    await doAction(webClient, 1);
+
+    assert.containsNone(webClient.el.querySelector('.o_activity_view thead'),
+        '.o_activity_filter_planned,.o_activity_filter_today,.o_activity_filter_overdue,.o_activity_filter___false',
+        "should not have active filter");
+    assert.containsNone(webClient.el.querySelector('.o_activity_view tbody'),
+        '.o_activity_filter_planned,.o_activity_filter_today,.o_activity_filter_overdue,.o_activity_filter___false',
+        "should not have active filter");
+    assert.strictEqual(webClient.el.querySelector('.o_activity_view tbody .o_activity_record').textContent,
+        'Office planning', "'Office planning' should be first record");
+    assert.containsOnce(webClient.el.querySelector('.o_activity_view tbody'), '.planned',
+        "other records should be available");
+
+    await testUtils.dom.click(webClient.el.querySelector('.o_kanban_counter_progress .progress-bar[data-filter="planned"]'));
+    assert.containsOnce(webClient.el.querySelector('.o_activity_view thead'), '.o_activity_filter_planned',
+        "planned should be active filter");
+    assert.containsN(webClient.el.querySelector('.o_activity_view tbody'), '.o_activity_filter_planned', 5,
+        "planned should be active filter");
+    assert.strictEqual(webClient.el.querySelector('.o_activity_view tbody .o_activity_record').textContent,
+        'Meeting Room Furnitures', "'Office planning' should be first record");
+    const tr = webClient.el.querySelectorAll('.o_activity_view tbody tr')[1];
+    assert.hasClass(tr.querySelectorAll('td')[1], 'o_activity_empty_cell',
+        "other records should be hidden");
+    assert.containsNone(webClient.el.querySelector('.o_activity_view tbody'), 'planned',
+        "other records should be hidden");
 });
 
 });
-});
