@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import api, fields, models, _
+from odoo import api, fields, models, _, Command
 from odoo.exceptions import UserError
 
 
@@ -29,20 +29,20 @@ class MailResendMessage(models.TransientModel):
         if message_id:
             mail_message_id = self.env['mail.message'].browse(message_id)
             notification_ids = mail_message_id.notification_ids.filtered(lambda notif: notif.notification_type == 'email' and notif.notification_status in ('exception', 'bounce'))
-            partner_ids = [(0, 0, {
+            partner_ids = [Command.create({
                 "partner_id": notif.res_partner_id.id,
                 "name": notif.res_partner_id.name,
                 "email": notif.res_partner_id.email,
                 "resend": True,
                 "message": notif.format_failure_reason(),
             }) for notif in notification_ids]
-            has_user = any([notif.res_partner_id.user_ids for notif in notification_ids])
+            has_user = any(notif.res_partner_id.user_ids for notif in notification_ids)
             if has_user:
                 partner_readonly = not self.env['res.users'].check_access_rights('write', raise_exception=False)
             else:
                 partner_readonly = not self.env['res.partner'].check_access_rights('write', raise_exception=False)
             rec['partner_readonly'] = partner_readonly
-            rec['notification_ids'] = [(6, 0, notification_ids.ids)]
+            rec['notification_ids'] = [Command.set(notification_ids.ids)]
             rec['mail_message_id'] = mail_message_id.id
             rec['partner_ids'] = partner_ids
         else:
@@ -63,7 +63,7 @@ class MailResendMessage(models.TransientModel):
                 record = self.env[message.model].browse(message.res_id) if message.is_thread_message() else self.env['mail.thread']
 
                 email_partners_data = []
-                for pid, cid, active, pshare, ctype, notif, groups in self.env['mail.followers']._get_recipient_data(None, 'comment', False, pids=to_send.ids):
+                for pid, active, pshare, notif, groups in self.env['mail.followers']._get_recipient_data(None, 'comment', False, pids=to_send.ids):
                     if pid and notif == 'email' or not notif:
                         pdata = {'id': pid, 'share': pshare, 'active': active, 'notif': 'email', 'groups': groups or []}
                         if not pshare and notif:  # has an user and is not shared, is therefore user
@@ -73,22 +73,22 @@ class MailResendMessage(models.TransientModel):
                         else:  # has no user, is therefore customer
                             email_partners_data.append(dict(pdata, type='customer'))
 
-                record._notify_record_by_email(message, {'partners': email_partners_data}, check_existing=True, send_after_commit=False)
+                record._notify_record_by_email(message, email_partners_data, check_existing=True, send_after_commit=False)
 
-            self.mail_message_id._notify_mail_failure_update()
+            self.mail_message_id._notify_message_notification_update()
         return {'type': 'ir.actions.act_window_close'}
 
     def cancel_mail_action(self):
         for wizard in self:
             for notif in wizard.notification_ids:
                 notif.filtered(lambda notif: notif.notification_type == 'email' and notif.notification_status in ('exception', 'bounce')).sudo().write({'notification_status': 'canceled'})
-            wizard.mail_message_id._notify_mail_failure_update()
+            wizard.mail_message_id._notify_message_notification_update()
         return {'type': 'ir.actions.act_window_close'}
 
 
 class PartnerResend(models.TransientModel):
     _name = 'mail.resend.partner'
-    _description = 'Partner with additionnal information for mail resend'
+    _description = 'Partner with additional information for mail resend'
 
     partner_id = fields.Many2one('res.partner', string='Partner', required=True, ondelete='cascade')
     name = fields.Char(related="partner_id.name", related_sudo=False, readonly=False)
