@@ -6,8 +6,6 @@ import werkzeug.urls
 
 from odoo import api, fields, models, tools
 
-from odoo.addons.link_tracker.models.link_tracker import URL_REGEX
-
 
 class MailMail(models.Model):
     """Add the mass mailing campaign data to mail"""
@@ -16,41 +14,19 @@ class MailMail(models.Model):
     mailing_id = fields.Many2one('mailing.mailing', string='Mass Mailing')
     mailing_trace_ids = fields.One2many('mailing.trace', 'mail_mail_id', string='Statistics')
 
-    @api.model
-    def create(self, values):
-        """ Override mail_mail creation to create an entry in mailing.trace """
+    @api.model_create_multi
+    def create(self, values_list):
+        """ Override mail_mail creation to create an entry in mail.mail.statistics """
         # TDE note: should be after 'all values computed', to have values (FIXME after merging other branch holding create refactoring)
-        mail = super(MailMail, self).create(values)
-        if values.get('mailing_trace_ids'):
-            mail_sudo = mail.sudo()
-            mail_sudo.mailing_trace_ids.write({'message_id': mail_sudo.message_id, 'state': 'outgoing'})
-        return mail
+        mails = super(MailMail, self).create(values_list)
+        for mail, values in zip(mails, values_list):
+            if values.get('mailing_trace_ids'):
+                mail.mailing_trace_ids.write({'message_id': mail.message_id})
+        return mails
 
     def _get_tracking_url(self):
-        base_url = self.env['ir.config_parameter'].sudo().get_param('web.base.url')
-        track_url = werkzeug.urls.url_join(
-            base_url, 'mail/track/%(mail_id)s/blank.gif?%(params)s' % {
-                'mail_id': self.id,
-                'params': werkzeug.urls.url_encode({'db': self.env.cr.dbname})
-            }
-        )
-        return '<img src="%s" alt=""/>' % track_url
-
-    def _get_unsubscribe_url(self, email_to):
-        base_url = self.env['ir.config_parameter'].sudo().get_param('web.base.url')
-        url = werkzeug.urls.url_join(
-            base_url, 'mail/mailing/%(mailing_id)s/unsubscribe?%(params)s' % {
-                'mailing_id': self.mailing_id.id,
-                'params': werkzeug.urls.url_encode({
-                    'db': self.env.cr.dbname,
-                    'res_id': self.res_id,
-                    'email': email_to,
-                    'token': self.mailing_id._unsubscribe_token(
-                        self.res_id, email_to),
-                }),
-            }
-        )
-        return url
+        token = tools.hmac(self.env(su=True), 'mass_mailing-mail_mail-open', self.id)
+        return werkzeug.urls.url_join(self.get_base_url(), 'mail/track/%s/%s/blank.gif' % (self.id, token))
 
     def _send_prepare_body(self):
         """ Override to add the tracking URL to the body and to add
@@ -60,7 +36,7 @@ class MailMail(models.Model):
         body = super(MailMail, self)._send_prepare_body()
 
         if self.mailing_id and body and self.mailing_trace_ids:
-            for match in re.findall(URL_REGEX, self.body_html):
+            for match in re.findall(tools.URL_REGEX, self.body_html):
                 href = match[0]
                 url = match[1]
 
@@ -72,24 +48,32 @@ class MailMail(models.Model):
 
             # generate tracking URL
             tracking_url = self._get_tracking_url()
-            if tracking_url:
-                body = tools.append_content_to_html(body, tracking_url, plaintext=False, container_tag='div')
+            body = tools.append_content_to_html(
+                body,
+                '<img src="%s"/>' % tracking_url,
+                plaintext=False,
+            )
 
-        body = self.env['mail.thread']._replace_local_links(body)
+        body = self.env['mail.render.mixin']._replace_local_links(body)
 
         return body
 
     def _send_prepare_values(self, partner=None):
         # TDE: temporary addition (mail was parameter) due to semi-new-API
         res = super(MailMail, self)._send_prepare_values(partner)
-        base_url = self.env['ir.config_parameter'].sudo().get_param('web.base.url').rstrip('/')
         if self.mailing_id and res.get('body') and res.get('email_to'):
+            base_url = self.mailing_id.get_base_url()
             emails = tools.email_split(res.get('email_to')[0])
             email_to = emails and emails[0] or False
-            unsubscribe_url = self._get_unsubscribe_url(email_to)
-            link_to_replace = base_url + '/unsubscribe_from_list'
-            if link_to_replace in res['body']:
-                res['body'] = res['body'].replace(link_to_replace, unsubscribe_url if unsubscribe_url else '#')
+
+            urls_to_replace = [
+                (base_url + '/unsubscribe_from_list', self.mailing_id._get_unsubscribe_url(email_to, self.res_id)),
+                (base_url + '/view', self.mailing_id._get_view_url(email_to, self.res_id))
+            ]
+
+            for url_to_replace, new_url in urls_to_replace:
+                if url_to_replace in res['body']:
+                    res['body'] = res['body'].replace(url_to_replace, new_url if new_url else '#')
         return res
 
     def _postprocess_sent_message(self, success_pids, failure_reason=False, failure_type=None):
@@ -97,7 +81,7 @@ class MailMail(models.Model):
         for mail in self:
             if mail.mailing_id:
                 if mail_sent is True and mail.mailing_trace_ids:
-                    mail.mailing_trace_ids.write({'sent': fields.Datetime.now(), 'exception': False})
+                    mail.mailing_trace_ids.set_sent()
                 elif mail_sent is False and mail.mailing_trace_ids:
-                    mail.mailing_trace_ids.write({'exception': fields.Datetime.now(), 'failure_type': failure_type})
+                    mail.mailing_trace_ids.set_failed(failure_type=failure_type)
         return super(MailMail, self)._postprocess_sent_message(success_pids, failure_reason=failure_reason, failure_type=failure_type)
