@@ -12,11 +12,20 @@ var _t = core._t;
 
 var MassMailingFieldHtml = FieldHtml.extend({
     xmlDependencies: (FieldHtml.prototype.xmlDependencies || []).concat(["/mass_mailing/static/src/xml/mass_mailing.xml"]),
-    jsLibs: [],
+    jsLibs: [
+        '/mass_mailing/static/src/js/mass_mailing_link_dialog_fix.js',
+        '/mass_mailing/static/src/js/mass_mailing_snippets.js',
+        '/mass_mailing/static/src/snippets/s_blockquote/options.js',
+        '/mass_mailing/static/src/snippets/s_masonry_block/options.js',
+        '/mass_mailing/static/src/snippets/s_media_list/options.js',
+        '/mass_mailing/static/src/snippets/s_showcase/options.js',
+        '/mass_mailing/static/src/snippets/s_rating/options.js',
+    ],
 
     custom_events: _.extend({}, FieldHtml.prototype.custom_events, {
         snippets_loaded: '_onSnippetsLoaded',
     }),
+    _wysiwygSnippetsActive: true,
 
     /**
      * @override
@@ -26,13 +35,6 @@ var MassMailingFieldHtml = FieldHtml.extend({
         if (!this.nodeOptions.snippets) {
             this.nodeOptions.snippets = 'mass_mailing.email_designer_snippets';
         }
-
-        // All the code related to this __extraAssetsForIframe variable is an
-        // ugly hack to restore mass mailing options in stable versions. The
-        // whole logic has to be refactored as soon as possible...
-        this.__extraAssetsForIframe = [{
-            jsLibs: ['/mass_mailing/static/src/js/mass_mailing_snippets.js'],
-        }];
     },
 
     //--------------------------------------------------------------------------
@@ -46,12 +48,8 @@ var MassMailingFieldHtml = FieldHtml.extend({
      *
      * @override
      */
-    commitChanges: function () {
+    commitChanges: async function () {
         var self = this;
-        if (config.isDebug() && this.mode === 'edit') {
-            var layoutInfo = $.summernote.core.dom.makeLayoutInfo(this.wysiwyg.$editor);
-            $.summernote.pluginEvents.codeview(undefined, undefined, layoutInfo, false);
-        }
         if (this.mode === 'readonly' || !this.isRendered) {
             return this._super();
         }
@@ -62,34 +60,27 @@ var MassMailingFieldHtml = FieldHtml.extend({
         }
 
         var $editable = this.wysiwyg.getEditable();
+        if (this.wysiwyg.snippetsMenu) {
+            await this.wysiwyg.snippetsMenu.cleanForSave();
+        }
+        return this.wysiwyg.saveModifiedImages(this.$content).then(function () {
+            self._isDirty = self.wysiwyg.isDirty();
+            self._doAction();
 
-        return this.wysiwyg.saveCroppedImages(this.$content).then(function () {
-            return self.wysiwyg.save().then(function (result) {
-                self._isDirty = result.isDirty;
+            const $editorEnable = $editable.closest('.editor_enable');
+            $editorEnable.removeClass('editor_enable');
+            convertInline.toInline($editable, self.cssRules, self.wysiwyg.$iframe);
+            $editorEnable.addClass('editor_enable');
 
-                convertInline.attachmentThumbnailToLinkImg($editable);
-                convertInline.fontToImg($editable);
-                convertInline.classToStyle($editable);
-
-                // fix outlook image rendering bug
-                _.each(['width', 'height'], function(attribute) {
-                    $editable.find('img[style*="width"], img[style*="height"]').attr(attribute, function(){
-                        return $(this)[attribute]();
-                    }).css(attribute, function(){
-                        return $(this).get(0).style[attribute] || 'auto';
-                    });
-                });
-
-                self.trigger_up('field_changed', {
-                    dataPointID: self.dataPointID,
-                    changes: _.object([fieldName], [self._unWrap($editable.html())])
-                });
-                self.wysiwyg.setValue(result.html);
-
-                if (self._isDirty && self.mode === 'edit') {
-                    return self._doAction();
-                }
+            self.trigger_up('field_changed', {
+                dataPointID: self.dataPointID,
+                changes: _.object([fieldName], [self._unWrap($editable.html())])
             });
+
+            $editable.html(self.value);
+            if (self._isDirty && self.mode === 'edit') {
+                return self._doAction();
+            }
         });
     },
     /**
@@ -113,15 +104,27 @@ var MassMailingFieldHtml = FieldHtml.extend({
     //--------------------------------------------------------------------------
 
     /**
-     * Returns true if must force the user to choose a theme.
+     * Adds automatic editor messages on drag&drop zone elements.
      *
      * @private
-     * @returns {Boolean}
      */
-    _checkIfMustForceThemeChoice: function () {
-        var firstChoice = this._editableAreaIsEmpty();
-        this.$content.closest('body').toggleClass("o_force_mail_theme_choice", firstChoice);
-        return firstChoice;
+     _addEditorMessages: function () {
+        const $editable = this.wysiwyg.getEditable().find('.o_editable');
+        this.$editorMessageElements = $editable
+            .not('[data-editor-message]')
+            .attr('data-editor-message', _t('DRAG BUILDING BLOCKS HERE'));
+        $editable.filter(':empty').attr('contenteditable', false);
+    },
+    /**
+     * @override
+     */
+     _createWysiwygIntance: async function () {
+        await this._super(...arguments);
+        // Data is removed on save but we need the mailing and its body to be
+        // named so they are handled properly by the snippets menu.
+        this.$content.find('.o_layout').addBack().data('name', 'Mailing');
+        // We don't want to drop snippets directly within the wysiwyg.
+        this.$content.removeClass('o_editable');
     },
     /**
      * Returns true if the editable area is empty.
@@ -152,19 +155,12 @@ var MassMailingFieldHtml = FieldHtml.extend({
      * @override
      */
     _renderEdit: function () {
-        this._isFromInline = !!this.value;
+        this._wysiwygSnippetsActive = !$(this.value).is('.o_layout.o_basic_theme');
         if (!this.value) {
             this.value = this.recordData[this.nodeOptions['inline-field']];
         }
         return this._super.apply(this, arguments);
     },
-    /**
-     * @override
-     */
-    _renderReadonly: function () {
-        this.value = this.recordData[this.nodeOptions['inline-field']];
-        return this._super.apply(this, arguments);
-    },
 
     /**
      * @override
@@ -235,23 +231,27 @@ var MassMailingFieldHtml = FieldHtml.extend({
 
             $img.attr("src", src);
         });
+        $container.find('.o_mail_block_cover .oe_img_bg').each(function () {
+            $(this).css('background-image', `url('/mass_mailing_themes/static/src/img/theme_${themeParams.name}/s_default_image_block_banner.jpg')`);
+        });
     },
     /**
      * Switch themes or import first theme.
      *
      * @private
-     * @param {Boolean} firstChoice true if this is the first chosen theme (going from no theme to a theme)
      * @param {Object} themeParams
      */
-    _switchThemes: function (firstChoice, themeParams) {
+    _switchThemes: function (themeParams) {
         if (!themeParams || this.switchThemeLast === themeParams) {
             return;
         }
         this.switchThemeLast = themeParams;
 
+        this.$lastContent = this.$content.find('.o_mail_wrapper_td').contents();
+
         this.$content.closest('body').removeClass(this._allClasses).addClass(themeParams.className);
 
-        var $old_layout = this.$content.find('.o_layout');
+        const old_layout = this.$content.find('.o_layout')[0];
 
         var $new_wrapper;
         var $newWrapperContent;
@@ -263,54 +263,65 @@ var MassMailingFieldHtml = FieldHtml.extend({
         } else {
             // This wrapper structure is the only way to have a responsive
             // and centered fixed-width content column on all mail clients
-            $new_wrapper = $('<table/>', {
-                class: 'o_mail_wrapper'
+            $new_wrapper = $('<div/>', {
+                class: 'container o_mail_wrapper o_mail_regular oe_unremovable',
             });
-            $newWrapperContent = $('<td/>', {
-                class: 'o_mail_no_options o_mail_wrapper_td oe_structure'
+            $newWrapperContent = $('<div/>', {
+                class: 'col o_mail_no_options o_mail_wrapper_td bg-white oe_structure o_editable'
             });
-            $new_wrapper.append($('<tr/>').append(
-                $('<td/>', {
-                    class: 'o_mail_no_resize o_not_editable',
-                    contenteditable: 'false'
-                }),
-                $newWrapperContent,
-                $('<td/>', {
-                    class: 'o_mail_no_resize o_not_editable',
-                    contenteditable: 'false'
-                })
-            ));
+            $new_wrapper.append($('<div class="row"/>').append($newWrapperContent));
         }
         var $newLayout = $('<div/>', {
-            class: 'o_layout ' + themeParams.className
+            class: 'o_layout oe_unremovable oe_unmovable bg-200 ' + themeParams.className,
+            'data-name': 'Mailing',
         }).append($new_wrapper);
 
-        var $contents;
-        if (firstChoice) {
-            $contents = themeParams.template;
-        } else if ($old_layout.length) {
-            $contents = ($old_layout.hasClass('oe_structure') ? $old_layout : $old_layout.find('.oe_structure').first()).contents();
-        } else {
-            $contents = this.$content.find('.o_editable').contents();
-        }
-
+        const $contents = themeParams.template;
         $newWrapperContent.append($contents);
         this._switchImages(themeParams, $newWrapperContent);
-        this.$content.find('.o_editable').empty().append($newLayout);
-        $old_layout.remove();
+        old_layout && old_layout.remove();
+        this.$content.empty().append($newLayout);
 
-        if (firstChoice) {
-            $newWrapperContent.find('*').addBack()
-                .contents()
-                .filter(function () {
-                    return this.nodeType === 3 && this.textContent.match(/\S/);
-                }).parent().addClass('o_default_snippet_text');
+        $newWrapperContent.find('*').addBack()
+            .contents()
+            .filter(function () {
+                return this.nodeType === 3 && this.textContent.match(/\S/);
+            }).parent().addClass('o_default_snippet_text');
 
-            if (themeParams.name == 'basic') {
-                this.$content.focusIn();
-            }
+        if (themeParams.name === 'basic') {
+            this.$content[0].focus();
         }
         this.wysiwyg.trigger('reload_snippet_dropzones');
+        this.trigger_up('iframe_updated', { $iframe: this.wysiwyg.$iframe });
+        this.wysiwyg.odooEditor.historyStep(true);
+    },
+
+    /**
+     * @private
+     * @override
+     */
+    _toggleCodeView: function ($codeview) {
+        this._super(...arguments);
+        const isFullWidth = !!$(window.top.document).find('.o_mass_mailing_form_full_width')[0];
+        $codeview.css('height', isFullWidth ? $(window).height() : '');
+        if ($codeview.hasClass('d-none')) {
+            this.trigger_up('iframe_updated', { $iframe: this.wysiwyg.$iframe });
+        }
+    },
+
+    /**
+     * @override
+     */
+    _getWysiwygOptions: function () {
+        const options = this._super.apply(this, arguments);
+        options.resizable = false;
+        options.defaultDataForLinkTools = { isNewWindow: true };
+        if (this._wysiwygSnippetsActive) {
+            options.wysiwygAlias = 'mass_mailing.wysiwyg';
+        } else {
+            delete options.snippets;
+        }
+        return options;
     },
 
     //--------------------------------------------------------------------------
@@ -321,13 +332,24 @@ var MassMailingFieldHtml = FieldHtml.extend({
      * @override
      */
     _onLoadWysiwyg: function () {
-        if (this._isFromInline) {
-            this._fromInline();
-        }
         if (this.snippetsLoaded) {
             this._onSnippetsLoaded(this.snippetsLoaded);
         }
         this._super();
+        this.wysiwyg.odooEditor.observerFlush();
+        this.wysiwyg.odooEditor.historyReset();
+        this.wysiwyg.$iframeBody.addClass('o_mass_mailing_iframe');
+        this.trigger_up('iframe_updated', { $iframe: this.wysiwyg.$iframe });
+    },
+    /**
+     * @private
+     * @param {boolean} activateSnippets
+     */
+    _restartWysiwygIntance: async function (activateSnippets = true) {
+        this.wysiwyg.destroy();
+        this.$el.empty();
+        this._wysiwygSnippetsActive = activateSnippets;
+        await this._createWysiwygIntance();
     },
     /**
      * @private
@@ -335,6 +357,15 @@ var MassMailingFieldHtml = FieldHtml.extend({
      */
     _onSnippetsLoaded: function (ev) {
         var self = this;
+        if (this.wysiwyg.snippetsMenu && $(window.top.document).find('.o_mass_mailing_form_full_width')[0]) {
+            // In full width form mode, ensure the snippets menu's scrollable is
+            // in the form view, not in the iframe.
+            this.wysiwyg.snippetsMenu.$scrollable = this.$el.closestScrollable();
+            // Ensure said scrollable keeps its scrollbar at all times to
+            // prevent the scrollbar from appearing at awkward moments (ie: when
+            // previewing an option)
+            this.wysiwyg.snippetsMenu.$scrollable.css('overflow-y', 'scroll');
+        }
         if (!this.$content) {
             this.snippetsLoaded = ev;
             return;
@@ -342,13 +373,23 @@ var MassMailingFieldHtml = FieldHtml.extend({
         var $snippetsSideBar = ev.data;
         var $themes = $snippetsSideBar.find("#email_designer_themes").children();
         var $snippets = $snippetsSideBar.find(".oe_snippet");
+        var selectorToKeep = '.o_we_external_history_buttons, .email_designer_top_actions';
+        // Overide `d-flex` class which style is `!important`
+        $snippetsSideBar.find(`.o_we_website_top_actions > *:not(${selectorToKeep})`).attr('style', 'display: none!important');
         var $snippets_menu = $snippetsSideBar.find("#snippets_menu");
+        var $selectTemplateBtn = $snippets_menu.find('.o_we_select_template');
 
         if (config.device.isMobile) {
             $snippetsSideBar.hide();
             this.$content.attr('style', 'padding-left: 0px !important');
         }
 
+        if (!odoo.debug) {
+            $snippetsSideBar.find('.o_codeview_btn').hide();
+        }
+        this._$codeview = this.wysiwyg.$iframe.contents().find('textarea.o_codeview');
+        $snippetsSideBar.on('click', '.o_codeview_btn', () => this._toggleCodeView(this._$codeview));
+
         if ($themes.length === 0) {
             return;
         }
@@ -391,40 +432,60 @@ var MassMailingFieldHtml = FieldHtml.extend({
          * Create theme selection screen and check if it must be forced opened.
          * Reforce it opened if the last snippet is removed.
          */
-        var $dropdown = $(core.qweb.render("mass_mailing.theme_selector", {
+        const $themeSelector = $(core.qweb.render("mass_mailing.theme_selector", {
             themes: themesParams
-        })).dropdown();
+        }));
+        const $themeSelectorNew = $(core.qweb.render("mass_mailing.theme_selector_new", {
+            themes: themesParams
+        }));
 
-        var firstChoice = this._checkIfMustForceThemeChoice();
+
+        let firstChoice = this._editableAreaIsEmpty();
+        if (firstChoice) {
+            $themeSelectorNew.appendTo(this.wysiwyg.$iframeBody);
+        }
 
         /**
          * Add proposition to install enterprise themes if not installed.
          */
-        var $mail_themes_upgrade = $dropdown.find(".o_mass_mailing_themes_upgrade");
+        var $mail_themes_upgrade = $themeSelector.find(".o_mass_mailing_themes_upgrade");
         $mail_themes_upgrade.on("click", function (e) {
             e.stopImmediatePropagation();
             e.preventDefault();
             self.do_action("mass_mailing.action_mass_mailing_configuration");
         });
 
+        $selectTemplateBtn.on('click', () => {
+            $snippetsSideBar.data('snippetMenu').activateCustomTab($themeSelector);
+            /**
+             * Ensure the parent of the theme selector is not used as parent for a
+             * tooltip as it is overflow auto and would result in the tooltip being
+             * hidden by the body of the mail.
+             */
+            $themeSelector.parent().addClass('o_forbidden_tooltip_parent');
+            $selectTemplateBtn.addClass('active');
+        });
+
         /**
          * Switch theme when a theme button is hovered. Confirm change if the theme button
          * is pressed.
          */
         var selectedTheme = false;
-        $dropdown.on("mouseenter", ".dropdown-item", function (e) {
-            if (firstChoice) {
-                return;
-            }
+        $themeSelector.on("mouseenter", ".dropdown-item", function (e) {
             e.preventDefault();
             var themeParams = themesParams[$(e.currentTarget).index()];
-            self._switchThemes(firstChoice, themeParams);
+            self.wysiwyg.odooEditor.automaticStepSkipStack();
+            self._switchThemes(themeParams);
         });
-        $dropdown.on("mouseleave", ".dropdown-item", function (e) {
-            self._switchThemes(false, selectedTheme);
+        $themeSelector.on("mouseleave", ".dropdown-item", function (e) {
+            if (self.$lastContent) {
+                self._switchThemes(Object.assign({}, selectedTheme, {template: self.$lastContent}));
+            } else {
+                self._switchThemes(selectedTheme);
+            }
         });
-        $dropdown.on("click", '[data-toggle="dropdown"]', function (e) {
-            var $menu = $dropdown.find('.dropdown-menu');
+        $themeSelector.on("click", '[data-toggle="dropdown"]', function (e) {
+            var $menu = $themeSelector.find('.dropdown-menu');
             var isVisible = $menu.hasClass('show');
             if (isVisible) {
                 e.preventDefault();
@@ -433,46 +494,47 @@ var MassMailingFieldHtml = FieldHtml.extend({
             }
         });
 
-        $dropdown.on("click", ".dropdown-item", function (e) {
+        const selectTheme = (e) => {
             e.preventDefault();
             e.stopImmediatePropagation();
-            var themeParams = themesParams[$(e.currentTarget).index()];
-            if (firstChoice) {
-                self._switchThemes(firstChoice, themeParams);
-                self.$content.closest('body').removeClass("o_force_mail_theme_choice");
-                firstChoice = false;
-
-                if ($mail_themes_upgrade.length) {
-                    $dropdown.remove();
-                    $snippets_menu.empty();
-                }
-            }
-
+            const themeParams = themesParams[$(e.currentTarget).index()];
             self._switchImages(themeParams, $snippets);
 
             selectedTheme = themeParams;
 
             // Notify form view
-            self.wysiwyg.getEditable().trigger('change');
-            $dropdown.find('.dropdown-menu').removeClass('show');
-            $dropdown.find('.dropdown-item.selected').removeClass('selected');
-            $dropdown.find('.dropdown-item:eq(' + themesParams.indexOf(selectedTheme) + ')').addClass('selected');
-        });
+            $themeSelector.find('.dropdown-item.selected').removeClass('selected');
+            $themeSelector.find('.dropdown-item:eq(' + themesParams.indexOf(selectedTheme) + ')').addClass('selected');
 
-        /**
-         * If the user opens the theme selection screen, indicates which one is active and
-         * saves the information...
-         * ... then when the user closes check if the user confirmed its choice and restore
-         * previous state if this is not the case.
-         */
-        $dropdown.on("shown.bs.dropdown", function () {
-            selectedTheme = self._getSelectedTheme(themesParams);
-            $dropdown.find(".dropdown-item").removeClass("selected").filter(function () {
-                return ($(this).has(".o_thumb[style=\"" + "background-image: url(" + (selectedTheme && selectedTheme.img) + "_small.png)" + "\"]").length > 0);
-            }).addClass("selected");
-        });
-        $dropdown.on("hidden.bs.dropdown", function () {
-            self._switchThemes(firstChoice, selectedTheme);
+            // Invalidate previous content.
+            self.$lastContent = undefined;
+        };
+
+        $themeSelector.on("click", ".dropdown-item", selectTheme);
+        $themeSelectorNew.on("click", ".dropdown-item", async (e) => {
+            e.preventDefault();
+            e.stopImmediatePropagation();
+            const themeParams = themesParams[$(e.currentTarget).index()];
+
+            if (themeParams.name === "basic") {
+                await this._restartWysiwygIntance(false);
+            }
+            this._switchThemes(themeParams);
+            this.$content.closest('body').removeClass("o_force_mail_theme_choice");
+
+            $themeSelectorNew.remove();
+
+            if ($mail_themes_upgrade.length) {
+                $snippets_menu.empty();
+            }
+
+            selectTheme(e);
+            this._addEditorMessages();
+            // Wait the next tick because some mutation have to be processed by
+            // the Odoo editor before resetting the history.
+            setTimeout(() => {
+                this.wysiwyg.historyReset();
+            }, 0);
         });
 
         /**
@@ -482,7 +544,7 @@ var MassMailingFieldHtml = FieldHtml.extend({
         selectedTheme = this._getSelectedTheme(themesParams);
         if (selectedTheme) {
             this.$content.closest('body').addClass(selectedTheme.className);
-            $dropdown.find('.dropdown-item:eq(' + themesParams.indexOf(selectedTheme) + ')').addClass('selected');
+            $themeSelector.find('.dropdown-item:eq(' + themesParams.indexOf(selectedTheme) + ')').addClass('selected');
             this._switchImages(selectedTheme, $snippets);
         } else if (this.$content.find('.o_layout').length) {
             themesParams.push({
@@ -495,8 +557,6 @@ var MassMailingFieldHtml = FieldHtml.extend({
             });
             selectedTheme = this._getSelectedTheme(themesParams);
         }
-
-        $dropdown.insertAfter($snippets_menu);
     },
     /**
      * @override
