@@ -29,9 +29,6 @@ invoice and send propositions for membership renewal.
         'views/partner_views.xml',
         'report/report_membership_views.xml',
     ],
-    'website': 'https://www.odoo.com/page/community-builder',
-    'test': [
-        '../account/test/account_minimal_test.xml',
-    ],
+    'website': 'https://www.odoo.com/app/forums',
     'license': 'LGPL-3',
 }
