@@ -5,10 +5,12 @@ from . import test_byproduct
 from . import test_cancel_mo
 from . import test_order
 from . import test_stock
+from . import test_stock_report
 from . import test_warehouse_multistep_manufacturing
-from . import test_workorder_operation
 from . import test_procurement
 from . import test_unbuild
 from . import test_oee
 from . import test_traceability
 from . import test_multicompany
+from . import test_backorder
+from . import test_smp
