@@ -10,6 +10,9 @@ class TestUnbuild(TestMrpCommon):
     def setUp(self):
         super(TestUnbuild, self).setUp()
         self.stock_location = self.env.ref('stock.stock_location_stock')
+        self.env.ref('base.group_user').write({
+            'implied_ids': [(4, self.env.ref('stock.group_production_lot').id)]
+        })
 
     def test_unbuild_standart(self):
         """ This test creates a MO and then creates 3 unbuild
@@ -24,14 +27,9 @@ class TestUnbuild(TestMrpCommon):
         self.env['stock.quant']._update_available_quantity(p2, self.stock_location, 5)
         mo.action_assign()
 
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 5.0
-        produce_wizard = produce_form.save()
-        produce_wizard.do_produce()
-
+        mo_form = Form(mo)
+        mo_form.qty_producing = 5.0
+        mo = mo_form.save()
         mo.button_mark_done()
         self.assertEqual(mo.state, 'done', "Production order should be in done state.")
 
@@ -48,7 +46,6 @@ class TestUnbuild(TestMrpCommon):
         x.product_id = p_final
         x.bom_id = bom
         x.product_qty = 3
-        x.product_uom_id = self.uom_unit
         x.save().action_unbuild()
 
 
@@ -60,7 +57,6 @@ class TestUnbuild(TestMrpCommon):
         x.product_id = p_final
         x.bom_id = bom
         x.product_qty = 2
-        x.product_uom_id = self.uom_unit
         x.save().action_unbuild()
 
         self.assertEqual(self.env['stock.quant']._get_available_quantity(p_final, self.stock_location), 0, 'You should have 0 finalproduct in stock')
@@ -71,7 +67,6 @@ class TestUnbuild(TestMrpCommon):
         x.product_id = p_final
         x.bom_id = bom
         x.product_qty = 5
-        x.product_uom_id = self.uom_unit
         x.save().action_unbuild()
 
         # Check quantity in stock after last unbuild.
@@ -98,15 +93,10 @@ class TestUnbuild(TestMrpCommon):
         self.env['stock.quant']._update_available_quantity(p2, self.stock_location, 5)
         mo.action_assign()
 
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 5.0
-        produce_form.finished_lot_id = lot
-        produce_wizard = produce_form.save()
-
-        produce_wizard.do_produce()
+        mo_form = Form(mo)
+        mo_form.qty_producing = 5.0
+        mo_form.lot_producing_id = lot
+        mo = mo_form.save()
 
         mo.button_mark_done()
         self.assertEqual(mo.state, 'done', "Production order should be in done state.")
@@ -126,14 +116,12 @@ class TestUnbuild(TestMrpCommon):
             x.product_id = p_final
             x.bom_id = bom
             x.product_qty = 3
-            x.product_uom_id = self.uom_unit
             unbuild_order = x.save()
 
         x = Form(self.env['mrp.unbuild'])
         x.product_id = p_final
         x.bom_id = bom
         x.product_qty = 3
-        x.product_uom_id = self.uom_unit
         x.lot_id = lot
         x.save().action_unbuild()
 
@@ -146,7 +134,6 @@ class TestUnbuild(TestMrpCommon):
         x.bom_id = bom
         x.product_qty = 2
         x.lot_id = lot
-        x.product_uom_id = self.uom_unit
         x.save().action_unbuild()
 
         self.assertEqual(self.env['stock.quant']._get_available_quantity(p_final, self.stock_location, lot_id=lot), 0, 'You should have 0 finalproduct in stock')
@@ -158,7 +145,6 @@ class TestUnbuild(TestMrpCommon):
         x.bom_id = bom
         x.product_qty = 5
         x.lot_id = lot
-        x.product_uom_id = self.uom_unit
         x.save().action_unbuild()
 
         self.assertEqual(self.env['stock.quant']._get_available_quantity(p_final, self.stock_location, lot_id=lot, allow_negative=True), -5, 'You should have negative quantity for final product in stock')
@@ -187,14 +173,15 @@ class TestUnbuild(TestMrpCommon):
             if ml.product_id.tracking != 'none':
                 self.assertEqual(ml.lot_id, lot, 'Wrong reserved lot.')
 
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 5.0
-        produce_wizard = produce_form.save()
-
-        produce_wizard.do_produce()
+        # FIXME sle: behavior change
+        mo_form = Form(mo)
+        mo_form.qty_producing = 5.0
+        mo = mo_form.save()
+        details_operation_form = Form(mo.move_raw_ids[1], view=self.env.ref('stock.view_stock_move_operations'))
+        with details_operation_form.move_line_ids.edit(0) as ml:
+            ml.lot_id = lot
+            ml.qty_done = 20
+        details_operation_form.save()
 
         mo.button_mark_done()
         self.assertEqual(mo.state, 'done', "Production order should be in done state.")
@@ -211,7 +198,6 @@ class TestUnbuild(TestMrpCommon):
         x.product_id = p_final
         x.bom_id = bom
         x.product_qty = 3
-        x.product_uom_id = self.uom_unit
         unbuild_order = x.save()
 
         # This should fail since we do not provide the MO that we wanted to unbuild. (without MO we do not know which consumed lot we have to restore)
@@ -230,7 +216,6 @@ class TestUnbuild(TestMrpCommon):
         x = Form(self.env['mrp.unbuild'])
         x.product_id = p_final
         x.bom_id = bom
-        x.product_uom_id = self.uom_unit
         x.mo_id = mo
         x.product_qty = 2
         x.save().action_unbuild()
@@ -242,7 +227,6 @@ class TestUnbuild(TestMrpCommon):
         x = Form(self.env['mrp.unbuild'])
         x.product_id = p_final
         x.bom_id = bom
-        x.product_uom_id = self.uom_unit
         x.mo_id = mo
         x.product_qty = 5
         x.save().action_unbuild()
@@ -280,15 +264,19 @@ class TestUnbuild(TestMrpCommon):
         self.env['stock.quant']._update_available_quantity(p2, self.stock_location, 5, lot_id=lot_2)
         mo.action_assign()
 
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 5.0
-        produce_form.finished_lot_id = lot_final
-        produce_wizard = produce_form.save()
-
-        produce_wizard.do_produce()
+        # FIXME sle: behavior change
+        mo_form = Form(mo)
+        mo_form.qty_producing = 5.0
+        mo_form.lot_producing_id = lot_final
+        mo = mo_form.save()
+        details_operation_form = Form(mo.move_raw_ids[0], view=self.env.ref('stock.view_stock_move_operations'))
+        with details_operation_form.move_line_ids.edit(0) as ml:
+            ml.qty_done = 5
+        details_operation_form.save()
+        details_operation_form = Form(mo.move_raw_ids[1], view=self.env.ref('stock.view_stock_move_operations'))
+        with details_operation_form.move_line_ids.edit(0) as ml:
+            ml.qty_done = 20
+        details_operation_form.save()
 
         mo.button_mark_done()
         self.assertEqual(mo.state, 'done', "Production order should be in done state.")
@@ -305,14 +293,12 @@ class TestUnbuild(TestMrpCommon):
         with self.assertRaises(AssertionError):
             x.product_id = p_final
             x.bom_id = bom
-            x.product_uom_id = self.uom_unit
             x.product_qty = 3
             x.save()
 
         with self.assertRaises(AssertionError):
             x.product_id = p_final
             x.bom_id = bom
-            x.product_uom_id = self.uom_unit
             x.product_qty = 3
             x.save()
 
@@ -321,7 +307,6 @@ class TestUnbuild(TestMrpCommon):
         with self.assertRaises(AssertionError):
             x.product_id = p_final
             x.bom_id = bom
-            x.product_uom_id = self.uom_unit
             x.mo_id = mo
             x.product_qty = 3
             x.save()
@@ -331,20 +316,18 @@ class TestUnbuild(TestMrpCommon):
         x = Form(self.env['mrp.unbuild'])
         x.product_id = p_final
         x.bom_id = bom
-        x.product_uom_id = self.uom_unit
         x.mo_id = mo
         x.product_qty = 3
         x.lot_id = lot_final
         x.save().action_unbuild()
 
         self.assertEqual(self.env['stock.quant']._get_available_quantity(p_final, self.stock_location, lot_id=lot_final), 2, 'You should have consumed 3 final product in stock')
-        self.assertEqual(self.env['stock.quant']._get_available_quantity(p1, self.stock_location, lot_id=lot_1), 92, 'You should have 80 products in stock')
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(p1, self.stock_location, lot_id=lot_1), 92, 'You should have 92 products in stock')
         self.assertEqual(self.env['stock.quant']._get_available_quantity(p2, self.stock_location, lot_id=lot_2), 3, 'You should have consumed all the 5 product in stock')
 
         x = Form(self.env['mrp.unbuild'])
         x.product_id = p_final
         x.bom_id = bom
-        x.product_uom_id = self.uom_unit
         x.mo_id = mo
         x.product_qty = 2
         x.lot_id = lot_final
@@ -357,7 +340,6 @@ class TestUnbuild(TestMrpCommon):
         x = Form(self.env['mrp.unbuild'])
         x.product_id = p_final
         x.bom_id = bom
-        x.product_uom_id = self.uom_unit
         x.mo_id = mo
         x.product_qty = 5
         x.lot_id = lot_final
@@ -395,14 +377,18 @@ class TestUnbuild(TestMrpCommon):
         self.env['stock.quant']._update_available_quantity(p2, self.stock_location, 2, lot_id=lot_3)
         mo.action_assign()
 
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 5.0
-        produce_wizard = produce_form.save()
+        mo_form = Form(mo)
+        mo_form.qty_producing = 5.0
+        mo = mo_form.save()
+        details_operation_form = Form(mo.move_raw_ids.filtered(lambda ml: ml.product_id == p2), view=self.env.ref('stock.view_stock_move_operations'))
+        with details_operation_form.move_line_ids.edit(0) as ml:
+            ml.qty_done = ml.product_uom_qty
+        with details_operation_form.move_line_ids.edit(1) as ml:
+            ml.qty_done = ml.product_uom_qty
+        with details_operation_form.move_line_ids.edit(2) as ml:
+            ml.qty_done = ml.product_uom_qty
+        details_operation_form.save()
 
-        produce_wizard.do_produce()
         mo.button_mark_done()
         self.assertEqual(mo.state, 'done', "Production order should be in done state.")
         # Check quantity in stock before unbuild.
@@ -415,7 +401,6 @@ class TestUnbuild(TestMrpCommon):
         x = Form(self.env['mrp.unbuild'])
         x.product_id = p_final
         x.bom_id = bom
-        x.product_uom_id = self.uom_unit
         x.mo_id = mo
         x.product_qty = 5
         x.save().action_unbuild()
@@ -430,6 +415,9 @@ class TestUnbuild(TestMrpCommon):
         """ This test produces an MO in two times and checks that the move lines are linked in a correct way
         """
         mo, bom, p_final, p1, p2 = self.generate_mo(tracking_final='lot', tracking_base_1='none', tracking_base_2='lot')
+        # Young Tom
+        #    \ Botox - 4 - p1
+        #    \ Old Tom - 1 - p2
         lot_1 = self.env['stock.production.lot'].create({
             'name': 'lot_1',
             'product_id': p2.id,
@@ -443,15 +431,20 @@ class TestUnbuild(TestMrpCommon):
             'company_id': self.env.company.id,
         })
 
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 3.0
-        produce_form.finished_lot_id = lot_finished_1
-        produce_wizard = produce_form.save()
-        produce_wizard._workorder_line_ids()[0].lot_id = lot_1
-        produce_wizard.do_produce()
+        self.assertEqual(mo.product_qty, 5)
+        mo_form = Form(mo)
+        mo_form.qty_producing = 3.0
+        mo_form.lot_producing_id = lot_finished_1
+        mo = mo_form.save()
+        self.assertEqual(mo.move_raw_ids[1].quantity_done, 12)
+        details_operation_form = Form(mo.move_raw_ids[0], view=self.env.ref('stock.view_stock_move_operations'))
+        with details_operation_form.move_line_ids.new() as ml:
+            ml.qty_done = 3
+            ml.lot_id = lot_1
+        details_operation_form.save()
+        action = mo.button_mark_done()
+        backorder = Form(self.env[action['res_model']].with_context(**action['context']))
+        backorder.save().action_backorder()
 
         lot_2 = self.env['stock.production.lot'].create({
             'name': 'lot_2',
@@ -466,21 +459,26 @@ class TestUnbuild(TestMrpCommon):
             'company_id': self.env.company.id,
         })
 
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 2.0
-        produce_form.finished_lot_id = lot_finished_2
+        mo = mo.procurement_group_id.mrp_production_ids[1]
+        # FIXME sle: issue in backorder?
+        mo.move_raw_ids.move_line_ids.unlink()
+        self.assertEqual(mo.product_qty, 2)
+        mo_form = Form(mo)
+        mo_form.qty_producing = 2
+        mo_form.lot_producing_id = lot_finished_2
+        mo = mo_form.save()
+        details_operation_form = Form(mo.move_raw_ids[0], view=self.env.ref('stock.view_stock_move_operations'))
+        with details_operation_form.move_line_ids.new() as ml:
+            ml.qty_done = 2
+            ml.lot_id = lot_2
+        details_operation_form.save()
+        action = mo.button_mark_done()
 
-        produce_wizard = produce_form.save()
-        produce_wizard._workorder_line_ids()[0].lot_id = lot_2
-        produce_wizard.do_produce()
-        mo.button_mark_done()
-        ml = mo.finished_move_line_ids[0].consume_line_ids.filtered(lambda m: m.product_id == p1 and lot_finished_1 in m.lot_produced_ids)
-        self.assertEqual(ml[0].qty_done, 12.0, 'Should have consumed 12 for the first lot')
-        ml = mo.finished_move_line_ids[1].consume_line_ids.filtered(lambda m: m.product_id == p1 and lot_finished_2 in m.lot_produced_ids)
-        self.assertEqual(ml[0].qty_done, 8.0, 'Should have consumed 8 for the second lot')
+        mo1 = mo.procurement_group_id.mrp_production_ids[0]
+        ml = mo1.finished_move_line_ids[0].consume_line_ids.filtered(lambda m: m.product_id == p1 and lot_finished_1 in m.produce_line_ids.lot_id)
+        self.assertEqual(sum(ml.mapped('qty_done')), 12.0, 'Should have consumed 12 for the first lot')
+        ml = mo.finished_move_line_ids[0].consume_line_ids.filtered(lambda m: m.product_id == p1 and lot_finished_2 in m.produce_line_ids.lot_id)
+        self.assertEqual(sum(ml.mapped('qty_done')), 8.0, 'Should have consumed 8 for the second lot')
 
     def test_unbuild_with_routes(self):
         """ This test creates a MO of a stockable product (Table). A new route for rule QC/Unbuild -> Stock
@@ -554,13 +552,9 @@ class TestUnbuild(TestMrpCommon):
         mo.action_assign()
 
         # Produce the final product
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 1.0
-        produce_wizard = produce_form.save()
-        produce_wizard.do_produce()
+        mo_form = Form(mo)
+        mo_form.qty_producing = 1.0
+        produce_wizard = mo_form.save()
 
         mo.button_mark_done()
         self.assertEqual(mo.state, 'done', "Production order should be in done state.")
@@ -578,7 +572,6 @@ class TestUnbuild(TestMrpCommon):
         x = Form(self.env['mrp.unbuild'])
         x.product_id = finshed_product
         x.bom_id = bom
-        x.product_uom_id = self.uom_unit
         x.mo_id = mo
         x.product_qty = 1
         x.location_id = self.stock_location
@@ -598,40 +591,98 @@ class TestUnbuild(TestMrpCommon):
         # Transfer it
         for ml in picking.move_ids_without_package:
             ml.quantity_done = 1
-        picking.action_done()
+        picking._action_done()
 
         # Check the available quantity of components and final product in stock
         self.assertEqual(StockQuant._get_available_quantity(finshed_product, self.stock_location), 0, 'Table should not be available in stock')
         self.assertEqual(StockQuant._get_available_quantity(component1, self.stock_location), 1, 'Table head should be available in stock as the picking is transferred')
         self.assertEqual(StockQuant._get_available_quantity(component2, self.stock_location), 1, 'Table stand should be available in stock as the picking is transferred')
 
-    def test_unbuild_an_updated_mo(self):
-        """ Suppose the user has a MO with qty = 1. On Produce step, he increases the quantity to 3.
-        Then the user unbuilds one product"""
-        mo, bom, p_final, p1, p2 = self.generate_mo(qty_final=1, qty_base_1=1, qty_base_2=1)
-        self.env['stock.quant']._update_available_quantity(p1, self.stock_location, 3)
-        self.env['stock.quant']._update_available_quantity(p2, self.stock_location, 3)
+    def test_unbuild_decimal_qty(self):
+        """
+        Use case:
+        - decimal accuracy of Product UoM > decimal accuracy of Units
+        - unbuild a product with a decimal quantity of component
+        """
+        self.env['decimal.precision'].search([('name', '=', 'Product Unit of Measure')]).digits = 4
+        self.uom_unit.rounding = 0.001
+
+        self.bom_1.product_qty = 3
+        self.bom_1.bom_line_ids.product_qty = 5
+        self.env['stock.quant']._update_available_quantity(self.product_2, self.stock_location, 3)
+
+        mo_form = Form(self.env['mrp.production'])
+        mo_form.product_id = self.bom_1.product_id
+        mo_form.bom_id = self.bom_1
+        mo = mo_form.save()
+        mo.action_confirm()
         mo.action_assign()
 
-        produce_form = Form(self.env['mrp.product.produce'].with_context({
-            'active_id': mo.id,
-            'active_ids': [mo.id],
-        }))
-        produce_form.qty_producing = 3
-        produce_wizard = produce_form.save()
-        produce_wizard.do_produce()
-
+        mo_form = Form(mo)
+        mo_form.qty_producing = 3
+        mo_form.save()
         mo.button_mark_done()
 
-        ub_form = Form(self.env['mrp.unbuild'])
-        ub_form.mo_id = mo
-        self.assertEqual(ub_form.product_qty, 3)
-        ub_form.product_qty = 1
-        ub = ub_form.save()
-        ub.action_unbuild()
+        uo_form = Form(self.env['mrp.unbuild'])
+        uo_form.mo_id = mo
+        # Unbuilding one product means a decimal quantity equal to 1 / 3 * 5 for each component
+        uo_form.product_qty = 1
+        uo = uo_form.save()
+        uo.action_unbuild()
+        self.assertEqual(uo.state, 'done')
 
-        for p, qty in [(p1, 1), (p2, 1), (p_final, 2)]:
-            self.assertEqual(p.qty_available, qty, 'Inorrect qty for product %s' % p.name)
-            ub_sm = self.env['stock.move'].search([('product_id', '=', p.id), ('name', 'like', 'UB%')])
-            self.assertEqual(len(ub_sm), 1, 'Incorrect nb of SM for product %s' % p.name)
-            self.assertEqual(ub_sm.product_uom_qty, 1, 'Incorrect qty for prodcut %s' % p.name)
+    def test_unbuild_similar_tracked_components(self):
+        """
+        Suppose a MO with, in the components, two lines for the same tracked-by-usn product
+        When unbuilding such an MO, all SN used in the MO should be back in stock
+        """
+        compo, finished = self.env['product.product'].create([{
+            'name': 'compo',
+            'type': 'product',
+            'tracking': 'serial',
+        }, {
+            'name': 'finished',
+            'type': 'product',
+        }])
+
+        lot01, lot02 = self.env['stock.production.lot'].create([{
+            'name': n,
+            'product_id': compo.id,
+            'company_id': self.env.company.id,
+        } for n in ['lot01', 'lot02']])
+        self.env['stock.quant']._update_available_quantity(compo, self.stock_location, 1, lot_id=lot01)
+        self.env['stock.quant']._update_available_quantity(compo, self.stock_location, 1, lot_id=lot02)
+
+        mo_form = Form(self.env['mrp.production'])
+        mo_form.product_id = finished
+        with mo_form.move_raw_ids.new() as line:
+            line.product_id = compo
+            line.product_uom_qty = 1
+        with mo_form.move_raw_ids.new() as line:
+            line.product_id = compo
+            line.product_uom_qty = 1
+        mo = mo_form.save()
+
+        mo.action_confirm()
+        mo_form = Form(mo)
+        mo_form.qty_producing = 1
+        mo = mo_form.save()
+        mo.action_assign()
+
+        details_operation_form = Form(mo.move_raw_ids[0], view=self.env.ref('stock.view_stock_move_operations'))
+        with details_operation_form.move_line_ids.edit(0) as ml:
+            ml.qty_done = 1
+        details_operation_form.save()
+        details_operation_form = Form(mo.move_raw_ids[1], view=self.env.ref('stock.view_stock_move_operations'))
+        with details_operation_form.move_line_ids.edit(0) as ml:
+            ml.qty_done = 1
+        details_operation_form.save()
+        mo.button_mark_done()
+
+        uo_form = Form(self.env['mrp.unbuild'])
+        uo_form.mo_id = mo
+        uo_form.product_qty = 1
+        uo = uo_form.save()
+        uo.action_unbuild()
+
+        self.assertEqual(uo.produce_line_ids.filtered(lambda sm: sm.product_id == compo).lot_ids, lot01 + lot02)
