@@ -7,6 +7,7 @@ from . import res_partner
 from . import stock_move
 from . import stock_move_line
 from . import stock_picking
+from . import stock_quant
 from . import stock_rule
 from . import stock_warehouse
-
+from . import mrp_production
