@@ -16,6 +16,7 @@ class TestSubcontractingDropshippingFlows(TestMrpSubcontractingCommon):
         - Delivery for the component to the subcontractor for the specified wh.
         - Po created for the component.
         """
+        self.env.ref('stock.route_warehouse0_mto').active = True
         mto_route = self.env['stock.location.route'].search([('name', '=', 'Replenish on Order (MTO)')])
         resupply_route = self.env['stock.location.route'].search([('name', '=', 'Resupply Subcontractor on Order')])
         buy_route = self.env['stock.location.route'].search([('name', '=', 'Buy')])
@@ -89,7 +90,7 @@ class TestSubcontractingDropshippingFlows(TestMrpSubcontractingCommon):
         (self.comp1).write({'route_ids': [(4, resupply_sub_on_order_route.id, None)]})
         # Create a supplier and set it to component
         vendor = self.env['res.partner'].create({'name': 'AAA', 'email': 'from.test@example.com'})
-        supplier_info1 = self.env['product.supplierinfo'].create({
+        self.env['product.supplierinfo'].create({
             'name': vendor.id,
             'price': 50,
         })
@@ -113,7 +114,7 @@ class TestSubcontractingDropshippingFlows(TestMrpSubcontractingCommon):
         origin = picking_delivery.origin
         self.assertEqual(len(picking_delivery), 1)
         self.assertEqual(len(picking_delivery.move_ids_without_package), 1)
-        self.assertEqual(picking_delivery.picking_type_id, wh.out_type_id)
+        self.assertEqual(picking_delivery.picking_type_id, wh.subcontracting_resupply_type_id)
         self.assertEqual(picking_delivery.partner_id, self.subcontractor_partner1)
 
         # Change the purchased quantity to 2
@@ -121,11 +122,11 @@ class TestSubcontractingDropshippingFlows(TestMrpSubcontractingCommon):
         # Check that two deliveries with 1 component for the subcontractor have been created
         picking_deliveries = self.env['stock.picking'].search([('origin', '=', origin)])
         self.assertEqual(len(picking_deliveries), 2)
-        self.assertEqual(picking_deliveries[0].picking_type_id, wh.out_type_id)
+        self.assertEqual(picking_deliveries[0].picking_type_id, wh.subcontracting_resupply_type_id)
         self.assertEqual(picking_deliveries[0].partner_id, self.subcontractor_partner1)
         self.assertTrue(picking_deliveries[0].state != 'cancel')
         move1 = picking_deliveries[0].move_ids_without_package
-        self.assertEqual(picking_deliveries[1].picking_type_id, wh.out_type_id)
+        self.assertEqual(picking_deliveries[1].picking_type_id, wh.subcontracting_resupply_type_id)
         self.assertEqual(picking_deliveries[1].partner_id, self.subcontractor_partner1)
         self.assertTrue(picking_deliveries[1].state != 'cancel')
         move2 = picking_deliveries[1].move_ids_without_package
