@@ -4,7 +4,7 @@
 {
     'name': 'Collaborative Pads',
     'version': '2.0',
-    'category': 'Tools',
+    'category': 'Hidden/Tools',
     'description': """
 Adds enhanced support for (Ether)Pad attachments in the web client.
 ===================================================================
@@ -14,11 +14,21 @@ pads (by default, http://etherpad.com/).
     """,
     'depends': ['web', 'base_setup'],
     'data': [
-        'views/pad.xml',
         'views/res_config_settings_views.xml',
     ],
     'demo': ['data/pad_demo.xml'],
     'web': True,
-    'qweb': ['static/src/xml/pad.xml'],
+    'assets': {
+        'web.assets_backend': [
+            'pad/static/src/css/etherpad.css',
+            'pad/static/src/js/pad.js',
+        ],
+        'web.qunit_suite_tests': [
+            'pad/static/tests/**/*',
+        ],
+        'web.assets_qweb': [
+            'pad/static/src/xml/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
