@@ -7,5 +7,5 @@ from odoo import fields, models
 class ResConfigSettings(models.TransientModel):
     _inherit = 'res.config.settings'
 
-    pad_server = fields.Char(related='company_id.pad_server', string="Pad Server *", readonly=False)
-    pad_key = fields.Char(related='company_id.pad_key', string="Pad Api Key *", readonly=False)
+    pad_server = fields.Char(config_parameter='pad.pad_server', string="Pad Server")
+    pad_key = fields.Char(config_parameter='pad.pad_key', string="Pad API Key")
