@@ -3,7 +3,7 @@
 
 {
     'name': 'Pad on tasks',
-    'category': 'Operations/Project',
+    'category': 'Services/Project',
     'description': """
 This module adds a PAD in all project form views.
 =================================================
@@ -14,8 +14,13 @@ This module adds a PAD in all project form views.
     ],
     'data': [
         'views/res_config_settings_views.xml',
-        'views/project_views.xml'
-    ],
+        'views/project_views.xml',
+        ],
     'auto_install': True,
+    'assets': {
+        'web.assets_frontend': [
+            'pad_project/static/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
