@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import api, models, fields, _
+from odoo import _, api, fields, models
 from odoo.exceptions import UserError
 
 
@@ -31,13 +30,18 @@ class PaymentWizard(models.TransientModel):
     acc_number = fields.Char("Account Number",  default=lambda self: self._get_default_payment_acquirer_onboarding_value('acc_number'))
     manual_post_msg = fields.Html("Payment Instructions")
 
+    _data_fetched = fields.Boolean(store=False)
+
     @api.onchange('journal_name', 'acc_number')
     def _set_manual_post_msg_value(self):
-        self.manual_post_msg = _('<h3>Please make a payment to: </h3><ul><li>Bank: %s</li><li>Account Number: %s</li><li>Account Holder: %s</li></ul>') %\
-                               (self.journal_name or _("Bank") , self.acc_number or _("Account"), self.env.company.name)
+        self.manual_post_msg = _(
+            '<h3>Please make a payment to: </h3><ul><li>Bank: %s</li><li>Account Number: %s</li><li>Account Holder: %s</li></ul>',
+            self.journal_name or _("Bank"),
+            self.acc_number or _("Account"),
+            self.env.company.name
+        )
 
     _payment_acquirer_onboarding_cache = {}
-    _data_fetched = False
 
     def _get_manual_payment_acquirer(self, env=None):
         if env is None:
@@ -94,6 +98,9 @@ class PaymentWizard(models.TransientModel):
     def add_payment_methods(self):
         """ Install required payment acquiers, configure them and mark the
             onboarding step as done."""
+        if self.payment_method == 'stripe' and not self.stripe_publishable_key:
+            self.env.company.payment_onboarding_payment_method = self.payment_method
+            return self._start_stripe_onboarding()
 
         if self.payment_method == 'paypal':
             self._install_module('payment_paypal')
@@ -127,7 +134,7 @@ class PaymentWizard(models.TransientModel):
                 manual_acquirer = self._get_manual_payment_acquirer(new_env)
                 if not manual_acquirer:
                     raise UserError(_(
-                        'No manual payment method could be found for this company. ' +
+                        'No manual payment method could be found for this company. '
                         'Please create one from the Payment Acquirer menu.'
                     ))
                 manual_acquirer.name = self.manual_name
@@ -138,11 +145,9 @@ class PaymentWizard(models.TransientModel):
                 if journal:
                     journal.name = self.journal_name
                     journal.bank_acc_number = self.acc_number
-                else:
-                    raise UserError(_("You have to set a journal for your payment acquirer %s." % (self.manual_name,)))
 
             # delete wizard data immediately to get rid of residual credentials
-            self.unlink()
+            self.sudo().unlink()
         # the user clicked `apply` and not cancel so we can assume this step is done.
         self._set_payment_acquirer_onboarding_step_done()
         return {'type': 'ir.actions.act_window_close'}
@@ -152,5 +157,10 @@ class PaymentWizard(models.TransientModel):
 
     def action_onboarding_other_payment_acquirer(self):
         self._set_payment_acquirer_onboarding_step_done()
-        action = self.env.ref('payment.action_payment_acquirer').read()[0]
+        action = self.env["ir.actions.actions"]._for_xml_id("payment.action_payment_acquirer")
         return action
+
+    def _start_stripe_onboarding(self):
+        """ Start Stripe Connect onboarding. """
+        menu_id = self.env.ref('payment.payment_acquirer_menu').id
+        return self.env.company._run_payment_onboarding_step(menu_id)
