@@ -1,10 +1,11 @@
-# -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 {
     'name': 'Buckaroo Payment Acquirer',
-    'category': 'Accounting/Payment',
+    'version': '2.0',
+    'category': 'Accounting/Payment Acquirers',
+    'sequence': 355,
     'summary': 'Payment Acquirer: Buckaroo Implementation',
-    'version': '1.0',
     'description': """Buckaroo Payment Acquirer""",
     'depends': ['payment'],
     'data': [
@@ -12,8 +13,7 @@
         'views/payment_buckaroo_templates.xml',
         'data/payment_acquirer_data.xml',
     ],
-    'installable': True,
-    'post_init_hook': 'create_missing_journal_for_acquirers',
+    'application': True,
     'uninstall_hook': 'uninstall_hook',
     'license': 'LGPL-3',
 }
