@@ -1,9 +1,10 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 {
     'name': 'PayuLatam Payment Acquirer',
-    'category': 'Accounting/Payment',
+    'version': '2.0',
+    'category': 'Accounting/Payment Acquirers',
+    'sequence': 370,
     'summary': 'Payment Acquirer: PayuLatam Implementation',
     'description': """Payulatam payment acquirer""",
     'depends': ['payment'],
@@ -12,6 +13,7 @@
         'views/payment_payulatam_templates.xml',
         'data/payment_acquirer_data.xml',
     ],
-    'post_init_hook': 'create_missing_journal_for_acquirers',
+    'application': True,
+    'uninstall_hook': 'uninstall_hook',
     'license': 'LGPL-3',
 }
