@@ -1,26 +1,24 @@
-# -*- encoding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
-# Copyright 2015 Eezee-It
+# Original Copyright 2015 Eezee-It, modified and maintained by Odoo.
 
 {
     'name': 'Worldline SIPS',
-    'version': '1.0',
-    'author': 'Eezee-It',
-    'category': 'Accounting/Payment',
+    'version': '2.0',
+    'category': 'Accounting/Payment Acquirers',
+    'sequence': 385,
     'description': """
 Worldline SIPS Payment Acquirer for online payments
 
-Works with Worldline keys version 2.0, contains implementation of
-payments acquirer using Worldline SIPS.""",
+Implements the Worldline SIPS API for payment acquirers.
+Other SIPS providers may be compatible, though this is
+not guaranteed.""",
     'depends': ['payment'],
     'data': [
         'views/payment_views.xml',
         'views/payment_sips_templates.xml',
         'data/payment_acquirer_data.xml',
     ],
-    'installable': True,
-    'post_init_hook': 'create_missing_journal_for_acquirers',
+    'application': True,
     'uninstall_hook': 'uninstall_hook',
     'license': 'LGPL-3',
 }
