@@ -24,7 +24,6 @@ class PosOrderReport(models.Model):
     price_sub_total = fields.Float(string='Subtotal w/o discount', readonly=True)
     total_discount = fields.Float(string='Total Discount', readonly=True)
     average_price = fields.Float(string='Average Price', readonly=True, group_operator="avg")
-    location_id = fields.Many2one('stock.location', string='Location', readonly=True)
     company_id = fields.Many2one('res.company', string='Company', readonly=True)
     nbr_lines = fields.Integer(string='Sale Line Count', readonly=True)
     product_qty = fields.Integer(string='Product Quantity', readonly=True)
@@ -36,6 +35,7 @@ class PosOrderReport(models.Model):
     pos_categ_id = fields.Many2one('pos.category', string='PoS Category', readonly=True)
     pricelist_id = fields.Many2one('product.pricelist', string='Pricelist', readonly=True)
     session_id = fields.Many2one('pos.session', string='Session', readonly=True)
+    margin = fields.Float(string='Margin', readonly=True)
 
     def _select(self):
         return """
@@ -56,7 +56,6 @@ class PosOrderReport(models.Model):
                 s.partner_id AS partner_id,
                 s.state AS state,
                 s.user_id AS user_id,
-                s.location_id AS location_id,
                 s.company_id AS company_id,
                 s.sale_journal AS journal_id,
                 l.product_id AS product_id,
@@ -66,7 +65,8 @@ class PosOrderReport(models.Model):
                 pt.pos_categ_id,
                 s.pricelist_id,
                 s.session_id,
-                s.account_move IS NOT NULL AS invoiced
+                s.account_move IS NOT NULL AS invoiced,
+                SUM(l.price_subtotal - l.total_cost / CASE COALESCE(s.currency_rate, 0) WHEN 0 THEN 1.0 ELSE s.currency_rate END) AS margin
         """
 
     def _from(self):
@@ -85,7 +85,7 @@ class PosOrderReport(models.Model):
         return """
             GROUP BY
                 s.id, s.date_order, s.partner_id,s.state, pt.categ_id,
-                s.user_id, s.location_id, s.company_id, s.sale_journal,
+                s.user_id, s.company_id, s.sale_journal,
                 s.pricelist_id, s.account_move, s.create_date, s.session_id,
                 l.product_id,
                 pt.categ_id, pt.pos_categ_id,
