@@ -1,41 +1,87 @@
+/* global html2canvas */
 odoo.define('point_of_sale.Printer', function (require) {
 "use strict";
 
 var Session = require('web.Session');
 var core = require('web.core');
+const { Gui } = require('point_of_sale.Gui');
 var _t = core._t;
 
+// IMPROVEMENT: This is too much. We can get away from this class.
+class PrintResult {
+    constructor({ successful, message }) {
+        this.successful = successful;
+        this.message = message;
+    }
+}
+
+class PrintResultGenerator {
+    IoTActionError() {
+        return new PrintResult({
+            successful: false,
+            message: {
+                title: _t('Connection to IoT Box failed'),
+                body: _t('Please check if the IoT Box is still connected.'),
+            },
+        });
+    }
+    IoTResultError() {
+        return new PrintResult({
+            successful: false,
+            message: {
+                title: _t('Connection to the printer failed'),
+                body: _t('Please check if the printer is still connected.'),
+            },
+        });
+    }
+    Successful() {
+        return new PrintResult({
+            successful: true,
+        });
+    }
+}
+
 var PrinterMixin = {
-    init: function () {
+    init: function (pos) {
         this.receipt_queue = [];
-        this.htmlToImgLetterRendering = false; // Whether to render each letter seperately. Necessary if letter-spacing is used.
+        this.printResultGenerator = new PrintResultGenerator();
+        this.pos = pos;
     },
 
     /**
      * Add the receipt to the queue of receipts to be printed and process it.
+     * We clear the print queue if printing is not successful.
      * @param {String} receipt: The receipt to be printed, in HTML
+     * @returns {PrintResult}
      */
-    print_receipt: function (receipt) {
-        var self = this;
+    print_receipt: async function(receipt) {
         if (receipt) {
             this.receipt_queue.push(receipt);
         }
-        function process_next_job() {
-            if (self.receipt_queue.length > 0) {
-                var r = self.receipt_queue.shift();
-                return self.htmlToImg(r)
-                    .then(self.send_printing_job.bind(self))
-                    .then(self._onIoTActionResult.bind(self))
-                    .then(process_next_job)
-                    .guardedCatch(self._onIoTActionFail.bind(self));
+        let image, sendPrintResult;
+        while (this.receipt_queue.length > 0) {
+            receipt = this.receipt_queue.shift();
+            image = await this.htmlToImg(receipt);
+            try {
+                sendPrintResult = await this.send_printing_job(image);
+            } catch (error) {
+                // Error in communicating to the IoT box.
+                this.receipt_queue.length = 0;
+                return this.printResultGenerator.IoTActionError();
+            }
+            // rpc call is okay but printing failed because
+            // IoT box can't find a printer.
+            if (!sendPrintResult || sendPrintResult.result === false) {
+                this.receipt_queue.length = 0;
+                return this.printResultGenerator.IoTResultError();
             }
         }
-        return process_next_job();
+        return this.printResultGenerator.Successful();
     },
 
     /**
      * Generate a jpeg image from a canvas
-     * @param {DOMElement} canvas 
+     * @param {DOMElement} canvas
      */
     process_canvas: function (canvas) {
         return canvas.toDataURL('image/jpeg').replace('data:image/jpeg;base64,','');
@@ -53,12 +99,13 @@ var PrinterMixin = {
             html2canvas(self.receipt[0], {
                 onparsed: function(queue) {
                     queue.stack.ctx.height = Math.ceil(self.receipt.outerHeight() + self.receipt.offset().top);
+                    queue.stack.ctx.width = Math.ceil(self.receipt.outerWidth() + 2 * self.receipt.offset().left);
                 },
                 onrendered: function (canvas) {
                     $('.pos-receipt-print').empty();
                     resolve(self.process_canvas(canvas));
                 },
-                letterRendering: self.htmlToImgLetterRendering,
+                letterRendering: self.pos.htmlToImgLetterRendering(),
             })
         });
         return promise;
@@ -66,7 +113,7 @@ var PrinterMixin = {
 
     _onIoTActionResult: function (data){
         if (this.pos && (data === false || data.result === false)) {
-            this.pos.gui.show_popup('error',{
+            Gui.showPopup('ErrorPopup',{
                 'title': _t('Connection to the printer failed'),
                 'body':  _t('Please check if the printer is still connected.'),
             });
@@ -75,7 +122,7 @@ var PrinterMixin = {
 
     _onIoTActionFail: function () {
         if (this.pos) {
-            this.pos.gui.show_popup('error',{
+            Gui.showPopup('ErrorPopup',{
                 'title': _t('Connection to IoT Box failed'),
                 'body':  _t('Please check if the IoT Box is still connected.'),
             });
@@ -85,9 +132,7 @@ var PrinterMixin = {
 
 var Printer = core.Class.extend(PrinterMixin, {
     init: function (url, pos) {
-        PrinterMixin.init.call(this, arguments);
-        this.pos = pos;
-        this.htmlToImgLetterRendering = pos.htmlToImgLetterRendering();
+        PrinterMixin.init.call(this, pos);
         this.connection = new Session(undefined, url || 'http://localhost:8069', { use_cors: true});
     },
 
@@ -123,5 +168,7 @@ var Printer = core.Class.extend(PrinterMixin, {
 return {
     PrinterMixin: PrinterMixin,
     Printer: Printer,
+    PrintResult,
+    PrintResultGenerator,
 }
 });
