@@ -7,10 +7,13 @@ from . import test_point_of_sale_ui
 from . import test_anglo_saxon
 from . import test_point_of_sale
 from . import test_pos_setup
+from . import test_pos_simple_orders
+from . import test_pos_simple_invoiced_orders
 from . import test_pos_basic_config
 from . import test_pos_products_with_tax
-from . import test_pos_multiple_sale_accounts
+from . import test_pos_margin
 from . import test_pos_multiple_receivable_accounts
 from . import test_pos_other_currency_config
 from . import test_pos_with_fiscal_position
 from . import test_pos_stock_account
+from . import test_js
