@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import odoo
 import time
@@ -14,9 +15,8 @@ class TestAngloSaxonCommon(common.TransactionCase):
         self.Statement = self.env['account.bank.statement']
         self.company = self.env.ref('base.main_company')
         self.warehouse = self.env['stock.warehouse'].search([('company_id', '=', self.env.company.id)], limit=1)
-        self.product = self.env.ref('product.product_product_3')
-        self.partner = self.env.ref('base.res_partner_1')
-        self.category = self.env.ref('product.product_category_1')
+        self.partner = self.env['res.partner'].create({'name': 'Partner 1'})
+        self.category = self.env.ref('product.product_category_all')
         self.category = self.category.copy({'name': 'New category','property_valuation': 'real_time'})
         account_type_rcv = self.env.ref('account.data_account_type_receivable')
         account_type_inc = self.env.ref('account.data_account_type_revenue')
@@ -35,8 +35,14 @@ class TestAngloSaxonCommon(common.TransactionCase):
         self.category.property_stock_journal = self.env['account.journal'].create({'name': 'Stock journal', 'type': 'sale', 'code': 'STK00'})
         self.pos_config = self.env.ref('point_of_sale.pos_config_main')
         self.pos_config = self.pos_config.copy({'name': 'New POS config'})
-        self.product = self.product.copy({'name': 'New product','standard_price': 100})
+        self.product = self.env['product.product'].create({
+            'name': 'New product',
+            'standard_price': 100,
+            'available_in_pos': True,
+            'type': 'product',
+        })
         self.company.anglo_saxon_accounting = True
+        self.company.point_of_sale_update_stock_quantities = 'real'
         self.product.categ_id = self.category
         self.product.property_account_expense_id = account_expense
         self.product.property_account_income_id = account_income
@@ -45,11 +51,9 @@ class TestAngloSaxonCommon(common.TransactionCase):
         self.cash_journal = self.env['account.journal'].create({'name': 'CASH journal', 'type': 'cash', 'code': 'CSH00'})
         self.sale_journal = self.env['account.journal'].create({'name': 'SALE journal', 'type': 'sale', 'code': 'INV00'})
         self.pos_config.invoice_journal_id = self.sale_journal
-        self.pos_config.journal_ids = [self.cash_journal.id]
         self.cash_payment_method = self.env['pos.payment.method'].create({
             'name': 'Cash Test',
-            'is_cash_count': True,
-            'cash_journal_id': self.cash_journal.id,
+            'journal_id': self.cash_journal.id,
             'receivable_account_id': self.account.id,
         })
         self.pos_config.write({'payment_method_ids': [(6, 0, self.cash_payment_method.ids)]})
@@ -61,9 +65,10 @@ class TestAngloSaxonFlow(TestAngloSaxonCommon):
     def test_create_account_move_line(self):
         # This test will check that the correct journal entries are created when a product in real time valuation
         # is sold in a company using anglo-saxon
-        self.pos_config.open_session_cb()
+        self.pos_config.open_session_cb(check_coa=False)
         current_session = self.pos_config.current_session_id
         self.cash_journal.loss_account_id = self.account
+        current_session.set_cashbox_pos(0, None)
 
         # I create a PoS order with 1 unit of New product at 450 EUR
         self.pos_order_pos0 = self.PosOrder.create({
@@ -104,7 +109,8 @@ class TestAngloSaxonFlow(TestAngloSaxonCommon):
         # I close the current session to generate the journal entries
         current_session_id = self.pos_config.current_session_id
         current_session_id._check_pos_session_balance()
-        current_session_id.action_pos_session_close()
+        current_session_id.post_closing_cash_details(450.0)
+        current_session_id.close_session_from_ui()
         self.assertEqual(current_session_id.state, 'closed', 'Check that session is closed')
 
         # Check if there is account_move in the order.
@@ -131,18 +137,20 @@ class TestAngloSaxonFlow(TestAngloSaxonCommon):
             'product_id': self.product.id,
             'inventory_quantity': 5.0,
             'location_id': self.warehouse.lot_stock_id.id,
-        })
+        }).action_apply_inventory()
         self.product.standard_price = 1.0
         self.env['stock.quant'].with_context(inventory_mode=True).create({
             'product_id': self.product.id,
             'inventory_quantity': 10.0,
             'location_id': self.warehouse.lot_stock_id.id,
-        })
+        }).action_apply_inventory()
         self.assertEqual(self.product.value_svl, 30, "Value should be (5*5 + 5*1) = 30")
         self.assertEqual(self.product.quantity_svl, 10)
 
         self.pos_config.module_account = True
-        self.pos_config.open_session_cb()
+        self.pos_config.open_session_cb(check_coa=False)
+        pos_session = self.pos_config.current_session_id
+        pos_session.set_cashbox_pos(0, None)
 
         pos_order_values = {
             'company_id': self.company.id,
@@ -182,7 +190,8 @@ class TestAngloSaxonFlow(TestAngloSaxonCommon):
 
         # validate the session
         current_session_id = self.pos_config.current_session_id
-        current_session_id.action_pos_session_validate()
+        current_session_id.post_closing_cash_details(7 * 450.0)
+        current_session_id.close_session_from_ui()
 
         # check the anglo saxon move lines
         # with uninvoiced orders, the account_move field of pos.order is empty.
@@ -209,10 +218,6 @@ class TestAngloSaxonFlow(TestAngloSaxonCommon):
         # Create the customer invoice
         pos_order_pos0.action_pos_order_invoice()
 
-        # validate the session
-        current_session_id = self.pos_config.current_session_id
-        current_session_id.action_pos_session_validate()
-
         # check the anglo saxon move lines
         line = pos_order_pos0.account_move.line_ids.filtered(lambda l: l.debit and l.account_id == self.category.property_account_expense_categ_id)
         self.assertEqual(pos_order_pos0.account_move.journal_id, self.pos_config.invoice_journal_id)
