@@ -3,5 +3,5 @@
 
 from . import pos_box
 from . import pos_details
-from . import pos_open_statement
 from . import pos_payment
+from . import pos_close_session_wizard
