@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-import werkzeug
 from werkzeug import urls
 from werkzeug.exceptions import NotFound, Forbidden
 
@@ -9,7 +8,7 @@ from odoo import http, _
 from odoo.http import request
 from odoo.osv import expression
 from odoo.tools import consteq, plaintext2html
-from odoo.addons.mail.controllers.main import MailController
+from odoo.addons.mail.controllers import mail
 from odoo.addons.portal.controllers.portal import CustomerPortal
 from odoo.exceptions import AccessError, MissingError, UserError
 
@@ -86,7 +85,7 @@ def _message_post_helper(res_model, res_id, message, token='', _hash=False, pid=
     message_post_args = dict(
         body=message,
         message_type=kw.pop('message_type', "comment"),
-        subtype=kw.pop('subtype', "mt_comment"),
+        subtype_xmlid=kw.pop('subtype_xmlid', "mail.mt_comment"),
         author_id=author_id,
         **kw
     )
@@ -111,26 +110,22 @@ class PortalChatter(http.Controller):
             try:
                 CustomerPortal._document_check_access(self, 'ir.attachment', attachment_id, access_token)
             except (AccessError, MissingError):
-                raise UserError(_("The attachment %s does not exist or you do not have the rights to access it.") % attachment_id)
+                raise UserError(_("The attachment %s does not exist or you do not have the rights to access it.", attachment_id))
 
-    @http.route(['/mail/chatter_post'], type='http', methods=['POST'], auth='public', website=True)
-    def portal_chatter_post(self, res_model, res_id, message, redirect=None, attachment_ids='', attachment_tokens='', **kw):
-        """Create a new `mail.message` with the given `message` and/or
-        `attachment_ids` and redirect the user to the newly created message.
+    @http.route(['/mail/chatter_post'], type='json', methods=['POST'], auth='public', website=True)
+    def portal_chatter_post(self, res_model, res_id, message, attachment_ids=None, attachment_tokens=None, **kw):
+        """Create a new `mail.message` with the given `message` and/or `attachment_ids` and return new message values.
 
         The message will be associated to the record `res_id` of the model
         `res_model`. The user must have access rights on this target document or
         must provide valid identifiers through `kw`. See `_message_post_helper`.
         """
-        url = redirect or (request.httprequest.referrer and request.httprequest.referrer + "#discussion") or '/my'
-
         res_id = int(res_id)
 
-        attachment_ids = [int(attachment_id) for attachment_id in attachment_ids.split(',') if attachment_id]
-        attachment_tokens = [attachment_token for attachment_token in attachment_tokens.split(',') if attachment_token]
         self._portal_post_check_attachments(attachment_ids, attachment_tokens)
 
         if message or attachment_ids:
+            result = {'default_message': message}
             # message is received in plaintext and saved in html
             if message:
                 message = plaintext2html(message)
@@ -139,12 +134,24 @@ class PortalChatter(http.Controller):
                 'res_id': res_id,
                 'message': message,
                 'send_after_commit': False,
-                'attachment_ids': attachment_ids,
+                'attachment_ids': False,  # will be added afterward
             }
             post_values.update((fname, kw.get(fname)) for fname in self._portal_post_filter_params())
             message = _message_post_helper(**post_values)
+            result.update({'default_message_id': message.id})
 
-        return request.redirect(url)
+            if attachment_ids:
+                # sudo write the attachment to bypass the read access
+                # verification in mail message
+                record = request.env[res_model].browse(res_id)
+                message_values = {'res_id': res_id, 'model': res_model}
+                attachments = record._message_post_process_attachments([], attachment_ids, message_values)
+
+                if attachments.get('attachment_ids'):
+                    message.sudo().write(attachments)
+
+                result.update({'default_attachment_ids': message.attachment_ids.sudo().read(['id', 'name', 'mimetype', 'file_size', 'access_token'])})
+            return result
 
     @http.route('/mail/chatter_init', type='json', auth='public', website=True)
     def portal_chatter_init(self, res_model, res_id, domain=False, limit=False, **kwargs):
@@ -158,6 +165,7 @@ class PortalChatter(http.Controller):
             'options': {
                 'message_count': message_data['message_count'],
                 'is_user_public': is_user_public,
+                'is_user_employee': request.env.user.has_group('base.group_user'),
                 'is_user_publisher': request.env.user.has_group('website.group_website_publisher'),
                 'display_composer': display_composer,
                 'partner_id': request.env.user.partner_id.id
@@ -183,15 +191,21 @@ class PortalChatter(http.Controller):
                 raise Forbidden()
             # Non-employee see only messages with not internal subtype (aka, no internal logs)
             if not request.env['res.users'].has_group('base.group_user'):
-                domain = expression.AND([Message._non_employee_message_domain(), domain])
+                domain = expression.AND([Message._get_search_domain_share(), domain])
             Message = request.env['mail.message'].sudo()
         return {
             'messages': Message.search(domain, limit=limit, offset=offset).portal_message_format(),
             'message_count': Message.search_count(domain)
         }
 
+    @http.route(['/mail/update_is_internal'], type='json', auth="user", website=True)
+    def portal_message_update_is_internal(self, message_id, is_internal):
+        message = request.env['mail.message'].browse(int(message_id))
+        message.write({'is_internal': is_internal})
+        return message.is_internal
 
-class MailController(MailController):
+
+class MailController(mail.MailController):
 
     @classmethod
     def _redirect_to_record(cls, model, res_id, access_token=None, **kwargs):
@@ -226,5 +240,5 @@ class MailController(MailController):
                             url_params = url.decode_query()
                             url_params.update([("pid", pid), ("hash", hash)])
                             url = url.replace(query=urls.url_encode(url_params)).to_url()
-                        return werkzeug.utils.redirect(url)
+                        return request.redirect(url)
         return super(MailController, cls)._redirect_to_record(model, res_id, access_token=access_token)
