@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import api, models
+from odoo import models
 
 
 class MailMessage(models.Model):
@@ -10,21 +10,17 @@ class MailMessage(models.Model):
     def portal_message_format(self):
         return self._portal_message_format([
             'id', 'body', 'date', 'author_id', 'email_from',  # base message fields
-            'message_type', 'subtype_id', 'subject',  # message specific
+            'message_type', 'subtype_id', 'is_internal', 'subject',  # message specific
             'model', 'res_id', 'record_name',  # document related
         ])
 
     def _portal_message_format(self, fields_list):
-        message_values = self.read(fields_list)
-        message_tree = dict((m.id, m) for m in self.sudo())
-        self._message_read_dict_postprocess(message_values, message_tree)
+        vals_list = self._message_format(fields_list)
+        message_subtype_note_id = self.env['ir.model.data']._xmlid_to_res_id('mail.mt_note')
         IrAttachmentSudo = self.env['ir.attachment'].sudo()
-        for message in message_values:
-            for attachment in message.get('attachment_ids', []):
+        for vals in vals_list:
+            vals['is_message_subtype_note'] = message_subtype_note_id and vals.get('subtype_id', [False])[0] == message_subtype_note_id
+            for attachment in vals.get('attachment_ids', []):
                 if not attachment.get('access_token'):
                     attachment['access_token'] = IrAttachmentSudo.browse(attachment['id']).generate_access_token()[0]
-        return message_values
-
-    @api.model
-    def _non_employee_message_domain(self):
-        return ['&', ('subtype_id', '!=', False), ('subtype_id.internal', '=', False)]
+        return vals_list
