@@ -3,17 +3,20 @@
 {
     'name': 'POS Adyen',
     'version': '1.0',
-    'category': 'Sales/Point Of Sale',
+    'category': 'Sales/Point of Sale',
     'sequence': 6,
     'summary': 'Integrate your POS with an Adyen payment terminal',
     'description': '',
     'data': [
         'views/pos_config_views.xml',
         'views/pos_payment_method_views.xml',
-        'views/point_of_sale_assets.xml',
     ],
     'depends': ['point_of_sale'],
-    'qweb': ['static/src/xml/pos.xml'],
     'installable': True,
+    'assets': {
+        'point_of_sale.assets': [
+            'pos_adyen/static/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
