@@ -5,7 +5,7 @@
 {
     'name': 'Point of Sale Discounts',
     'version': '1.0',
-    'category': 'Sales/Point Of Sale',
+    'category': 'Sales/Point of Sale',
     'sequence': 6,
     'summary': 'Simple Discounts in the Point of Sale ',
     'description': """
@@ -17,11 +17,15 @@ discount to a customer.
     'depends': ['point_of_sale'],
     'data': [
         'views/pos_discount_views.xml',
-        'views/pos_discount_templates.xml'
-    ],
-    'qweb': [
-        'static/src/xml/discount_templates.xml',
-    ],
+        ],
     'installable': True,
+    'assets': {
+        'point_of_sale.assets': [
+            'pos_discount/static/src/js/**/*',
+        ],
+        'web.assets_qweb': [
+            'pos_discount/static/src/xml/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
