@@ -5,6 +5,7 @@
 
 from . import res_config_settings
 from . import decimal_precision
+from . import uom_uom
 
 # don't try to be a good boy and sort imports alphabetically.
 # `product.template` should be initialised before `product.product`
@@ -14,4 +15,5 @@ from . import product
 from . import product_attribute
 from . import product_pricelist
 from . import res_company
+from . import res_currency
 from . import res_partner
