@@ -10,104 +10,110 @@ from odoo.tests.common import TransactionCase
 from odoo.tools import mute_logger
 
 
-class TestProductAttributeValueSetup(TransactionCase):
-    def setUp(self):
-        super(TestProductAttributeValueSetup, self).setUp()
+class TestProductAttributeValueCommon(TransactionCase):
+    @classmethod
+    def setUpClass(cls):
+        super(TestProductAttributeValueCommon, cls).setUpClass()
 
-        self.computer = self.env['product.template'].create({
+        cls.computer = cls.env['product.template'].create({
             'name': 'Super Computer',
             'price': 2000,
         })
 
-        self._add_ssd_attribute()
-        self._add_ram_attribute()
-        self._add_hdd_attribute()
+        cls._add_ssd_attribute()
+        cls._add_ram_attribute()
+        cls._add_hdd_attribute()
 
-        self.computer_case = self.env['product.template'].create({
+        cls.computer_case = cls.env['product.template'].create({
             'name': 'Super Computer Case'
         })
 
-        self._add_size_attribute()
+        cls._add_size_attribute()
 
-    def _add_ssd_attribute(self):
-        self.ssd_attribute = self.env['product.attribute'].create({'name': 'Memory', 'sequence': 1})
-        self.ssd_256 = self.env['product.attribute.value'].create({
+    @classmethod
+    def _add_ssd_attribute(cls):
+        cls.ssd_attribute = cls.env['product.attribute'].create({'name': 'Memory', 'sequence': 1})
+        cls.ssd_256 = cls.env['product.attribute.value'].create({
             'name': '256 GB',
-            'attribute_id': self.ssd_attribute.id,
+            'attribute_id': cls.ssd_attribute.id,
             'sequence': 1,
         })
-        self.ssd_512 = self.env['product.attribute.value'].create({
+        cls.ssd_512 = cls.env['product.attribute.value'].create({
             'name': '512 GB',
-            'attribute_id': self.ssd_attribute.id,
+            'attribute_id': cls.ssd_attribute.id,
             'sequence': 2,
         })
 
-        self._add_ssd_attribute_line()
+        cls._add_ssd_attribute_line()
 
-    def _add_ssd_attribute_line(self):
-        self.computer_ssd_attribute_lines = self.env['product.template.attribute.line'].create({
-            'product_tmpl_id': self.computer.id,
-            'attribute_id': self.ssd_attribute.id,
-            'value_ids': [(6, 0, [self.ssd_256.id, self.ssd_512.id])],
+    @classmethod
+    def _add_ssd_attribute_line(cls):
+        cls.computer_ssd_attribute_lines = cls.env['product.template.attribute.line'].create({
+            'product_tmpl_id': cls.computer.id,
+            'attribute_id': cls.ssd_attribute.id,
+            'value_ids': [(6, 0, [cls.ssd_256.id, cls.ssd_512.id])],
         })
-        self.computer_ssd_attribute_lines.product_template_value_ids[0].price_extra = 200
-        self.computer_ssd_attribute_lines.product_template_value_ids[1].price_extra = 400
+        cls.computer_ssd_attribute_lines.product_template_value_ids[0].price_extra = 200
+        cls.computer_ssd_attribute_lines.product_template_value_ids[1].price_extra = 400
 
-    def _add_ram_attribute(self):
-        self.ram_attribute = self.env['product.attribute'].create({'name': 'RAM', 'sequence': 2})
-        self.ram_8 = self.env['product.attribute.value'].create({
+    @classmethod
+    def _add_ram_attribute(cls):
+        cls.ram_attribute = cls.env['product.attribute'].create({'name': 'RAM', 'sequence': 2})
+        cls.ram_8 = cls.env['product.attribute.value'].create({
             'name': '8 GB',
-            'attribute_id': self.ram_attribute.id,
+            'attribute_id': cls.ram_attribute.id,
             'sequence': 1,
         })
-        self.ram_16 = self.env['product.attribute.value'].create({
+        cls.ram_16 = cls.env['product.attribute.value'].create({
             'name': '16 GB',
-            'attribute_id': self.ram_attribute.id,
+            'attribute_id': cls.ram_attribute.id,
             'sequence': 2,
         })
-        self.ram_32 = self.env['product.attribute.value'].create({
+        cls.ram_32 = cls.env['product.attribute.value'].create({
             'name': '32 GB',
-            'attribute_id': self.ram_attribute.id,
+            'attribute_id': cls.ram_attribute.id,
             'sequence': 3,
         })
-        self.computer_ram_attribute_lines = self.env['product.template.attribute.line'].create({
-            'product_tmpl_id': self.computer.id,
-            'attribute_id': self.ram_attribute.id,
-            'value_ids': [(6, 0, [self.ram_8.id, self.ram_16.id, self.ram_32.id])],
+        cls.computer_ram_attribute_lines = cls.env['product.template.attribute.line'].create({
+            'product_tmpl_id': cls.computer.id,
+            'attribute_id': cls.ram_attribute.id,
+            'value_ids': [(6, 0, [cls.ram_8.id, cls.ram_16.id, cls.ram_32.id])],
         })
-        self.computer_ram_attribute_lines.product_template_value_ids[0].price_extra = 20
-        self.computer_ram_attribute_lines.product_template_value_ids[1].price_extra = 40
-        self.computer_ram_attribute_lines.product_template_value_ids[2].price_extra = 80
+        cls.computer_ram_attribute_lines.product_template_value_ids[0].price_extra = 20
+        cls.computer_ram_attribute_lines.product_template_value_ids[1].price_extra = 40
+        cls.computer_ram_attribute_lines.product_template_value_ids[2].price_extra = 80
 
-    def _add_hdd_attribute(self):
-        self.hdd_attribute = self.env['product.attribute'].create({'name': 'HDD', 'sequence': 3})
-        self.hdd_1 = self.env['product.attribute.value'].create({
+    @classmethod
+    def _add_hdd_attribute(cls):
+        cls.hdd_attribute = cls.env['product.attribute'].create({'name': 'HDD', 'sequence': 3})
+        cls.hdd_1 = cls.env['product.attribute.value'].create({
             'name': '1 To',
-            'attribute_id': self.hdd_attribute.id,
+            'attribute_id': cls.hdd_attribute.id,
             'sequence': 1,
         })
-        self.hdd_2 = self.env['product.attribute.value'].create({
+        cls.hdd_2 = cls.env['product.attribute.value'].create({
             'name': '2 To',
-            'attribute_id': self.hdd_attribute.id,
+            'attribute_id': cls.hdd_attribute.id,
             'sequence': 2,
         })
-        self.hdd_4 = self.env['product.attribute.value'].create({
+        cls.hdd_4 = cls.env['product.attribute.value'].create({
             'name': '4 To',
-            'attribute_id': self.hdd_attribute.id,
+            'attribute_id': cls.hdd_attribute.id,
             'sequence': 3,
         })
 
-        self._add_hdd_attribute_line()
+        cls._add_hdd_attribute_line()
 
-    def _add_hdd_attribute_line(self):
-        self.computer_hdd_attribute_lines = self.env['product.template.attribute.line'].create({
-            'product_tmpl_id': self.computer.id,
-            'attribute_id': self.hdd_attribute.id,
-            'value_ids': [(6, 0, [self.hdd_1.id, self.hdd_2.id, self.hdd_4.id])],
+    @classmethod
+    def _add_hdd_attribute_line(cls):
+        cls.computer_hdd_attribute_lines = cls.env['product.template.attribute.line'].create({
+            'product_tmpl_id': cls.computer.id,
+            'attribute_id': cls.hdd_attribute.id,
+            'value_ids': [(6, 0, [cls.hdd_1.id, cls.hdd_2.id, cls.hdd_4.id])],
         })
-        self.computer_hdd_attribute_lines.product_template_value_ids[0].price_extra = 2
-        self.computer_hdd_attribute_lines.product_template_value_ids[1].price_extra = 4
-        self.computer_hdd_attribute_lines.product_template_value_ids[2].price_extra = 8
+        cls.computer_hdd_attribute_lines.product_template_value_ids[0].price_extra = 2
+        cls.computer_hdd_attribute_lines.product_template_value_ids[1].price_extra = 4
+        cls.computer_hdd_attribute_lines.product_template_value_ids[2].price_extra = 8
 
     def _add_ram_exclude_for(self):
         self._get_product_value_id(self.computer_ram_attribute_lines, self.ram_16).update({
@@ -117,27 +123,28 @@ class TestProductAttributeValueSetup(TransactionCase):
             })]
         })
 
-    def _add_size_attribute(self):
-        self.size_attribute = self.env['product.attribute'].create({'name': 'Size', 'sequence': 4})
-        self.size_m = self.env['product.attribute.value'].create({
+    @classmethod
+    def _add_size_attribute(cls):
+        cls.size_attribute = cls.env['product.attribute'].create({'name': 'Size', 'sequence': 4})
+        cls.size_m = cls.env['product.attribute.value'].create({
             'name': 'M',
-            'attribute_id': self.size_attribute.id,
+            'attribute_id': cls.size_attribute.id,
             'sequence': 1,
         })
-        self.size_l = self.env['product.attribute.value'].create({
+        cls.size_l = cls.env['product.attribute.value'].create({
             'name': 'L',
-            'attribute_id': self.size_attribute.id,
+            'attribute_id': cls.size_attribute.id,
             'sequence': 2,
         })
-        self.size_xl = self.env['product.attribute.value'].create({
+        cls.size_xl = cls.env['product.attribute.value'].create({
             'name': 'XL',
-            'attribute_id': self.size_attribute.id,
+            'attribute_id': cls.size_attribute.id,
             'sequence': 3,
         })
-        self.computer_case_size_attribute_lines = self.env['product.template.attribute.line'].create({
-            'product_tmpl_id': self.computer_case.id,
-            'attribute_id': self.size_attribute.id,
-            'value_ids': [(6, 0, [self.size_m.id, self.size_l.id, self.size_xl.id])],
+        cls.computer_case_size_attribute_lines = cls.env['product.template.attribute.line'].create({
+            'product_tmpl_id': cls.computer_case.id,
+            'attribute_id': cls.size_attribute.id,
+            'value_ids': [(6, 0, [cls.size_m.id, cls.size_l.id, cls.size_xl.id])],
         })
 
     def _get_product_value_id(self, product_template_attribute_lines, product_attribute_value):
@@ -171,7 +178,7 @@ class TestProductAttributeValueSetup(TransactionCase):
 
 
 @tagged('post_install', '-at_install')
-class TestProductAttributeValueConfig(TestProductAttributeValueSetup):
+class TestProductAttributeValueConfig(TestProductAttributeValueCommon):
 
     def test_product_template_attribute_values_creation(self):
         self.assertEqual(len(self.computer_ssd_attribute_lines.product_template_value_ids), 2,
@@ -207,7 +214,7 @@ class TestProductAttributeValueConfig(TestProductAttributeValueSetup):
     def test_product_filtered_exclude_for(self):
         """
             Super Computer has 18 variants total (2 ssd * 3 ram * 3 hdd)
-            RAM 16 excudes HDD 1, that matches 2 variants:
+            RAM 16 excludes HDD 1, that matches 2 variants:
             - SSD 256 RAM 16 HDD 1
             - SSD 512 RAM 16 HDD 1
 
@@ -223,8 +230,8 @@ class TestProductAttributeValueConfig(TestProductAttributeValueSetup):
         self._add_ram_exclude_for()
         self.assertEqual(len(self.computer._get_possible_variants()), 16)
         self.assertTrue(self.computer._get_variant_for_combination(computer_ssd_256 + computer_ram_8 + computer_hdd_1)._is_variant_possible())
-        self.assertFalse(self.computer._get_variant_for_combination(computer_ssd_256 + computer_ram_16 + computer_hdd_1)._is_variant_possible())
-        self.assertFalse(self.computer._get_variant_for_combination(computer_ssd_512 + computer_ram_16 + computer_hdd_1)._is_variant_possible())
+        self.assertFalse(self.computer._get_variant_for_combination(computer_ssd_256 + computer_ram_16 + computer_hdd_1))
+        self.assertFalse(self.computer._get_variant_for_combination(computer_ssd_512 + computer_ram_16 + computer_hdd_1))
 
     def test_children_product_filtered_exclude_for(self):
         """
@@ -345,8 +352,7 @@ class TestProductAttributeValueConfig(TestProductAttributeValueSetup):
         self.assertEqual(next(gen), computer_ssd_512 + computer_ram_32 + computer_hdd_1)
         self.assertEqual(next(gen), computer_ssd_512 + computer_ram_32 + computer_hdd_2)
         self.assertEqual(next(gen), computer_ssd_512 + computer_ram_32 + computer_hdd_4)
-        with self.assertRaises(StopIteration):
-            next(gen)
+        self.assertIsNone(next(gen, None))
 
         # Give priority to ram_16 but it is not allowed by hdd_1 so it should return hhd_2 instead
         # Test invalidate_cache on product.attribute.value write
@@ -381,12 +387,130 @@ class TestProductAttributeValueConfig(TestProductAttributeValueSetup):
         self._add_exclude(computer_ram_32, computer_ssd_256)
         self.assertEqual(self.computer._get_first_possible_combination(), computer_ssd_512 + computer_ram_32 + computer_hdd_4)
 
-        # No possible combination (test helper and iterator)
-        self._add_exclude(computer_ram_32, computer_hdd_4)
+        # Not possible to add an exclusion when only one variant is left -> it deletes the product template associated to it
+        with self.assertRaises(UserError), self.cr.savepoint():
+            self._add_exclude(computer_ram_32, computer_hdd_4)
+
+        # If an exclusion rule deletes all variants at once it does not delete the template.
+        # Here we can test `_get_first_possible_combination` with a product template with no variants
+        # Deletes all exclusions
+        for exclusion in computer_ram_32.exclude_for:
+            computer_ram_32.write({
+                'exclude_for': [(2, exclusion.id, 0)]
+            })
+
+        # Activates all exclusions at once
+        computer_ram_32.write({
+            'exclude_for': [(0, computer_ram_32.exclude_for.id, {
+                'product_tmpl_id': self.computer.id,
+                'value_ids': [(6, 0, [computer_hdd_1.id, computer_hdd_2.id, computer_hdd_4.id, computer_ssd_256.id, computer_ssd_512.id])]
+            })]
+        })
+
         self.assertEqual(self.computer._get_first_possible_combination(), self.env['product.template.attribute.value'])
         gen = self.computer._get_possible_combinations()
-        with self.assertRaises(StopIteration):
-            next(gen)
+        self.assertIsNone(next(gen, None))
+
+        # Testing parent case
+        mouse = self.env['product.template'].create({'name': 'Mouse'})
+        self.assertTrue(mouse._is_combination_possible(self.env['product.template.attribute.value']))
+
+        # prep work for the last part of the test
+        color_attribute = self.env['product.attribute'].create({'name': 'Color'})
+        color_red = self.env['product.attribute.value'].create({
+            'name': 'Red',
+            'attribute_id': color_attribute.id,
+        })
+        color_green = self.env['product.attribute.value'].create({
+            'name': 'Green',
+            'attribute_id': color_attribute.id,
+        })
+        self.env['product.template.attribute.line'].create({
+            'product_tmpl_id': mouse.id,
+            'attribute_id': color_attribute.id,
+            'value_ids': [(6, 0, [color_red.id, color_green.id])],
+        })
+
+        mouse_color_red = self._get_product_template_attribute_value(color_red, mouse)
+        mouse_color_green = self._get_product_template_attribute_value(color_green, mouse)
+
+        self._add_exclude(computer_ssd_256, mouse_color_red, mouse)
+        self.assertEqual(mouse._get_first_possible_combination(parent_combination=computer_ssd_256 + computer_ram_8 + computer_hdd_1), mouse_color_green)
+
+        # Test to see if several attribute_line for same attribute is well handled
+        color_blue = self.env['product.attribute.value'].create({
+            'name': 'Blue',
+            'attribute_id': color_attribute.id,
+        })
+        color_yellow = self.env['product.attribute.value'].create({
+            'name': 'Yellow',
+            'attribute_id': color_attribute.id,
+        })
+        self.env['product.template.attribute.line'].create({
+            'product_tmpl_id': mouse.id,
+            'attribute_id': color_attribute.id,
+            'value_ids': [(6, 0, [color_blue.id, color_yellow.id])],
+        })
+        mouse_color_yellow = self._get_product_template_attribute_value(color_yellow, mouse)
+        self.assertEqual(mouse._get_first_possible_combination(necessary_values=mouse_color_yellow), mouse_color_red + mouse_color_yellow)
+
+        # Making sure it's not extremely slow (has to discard invalid combinations early !)
+        product_template = self.env['product.template'].create({
+            'name': 'many combinations',
+        })
+
+        for i in range(10):
+            # create the attributes
+            product_attribute = self.env['product.attribute'].create({
+                'name': "att %s" % i,
+                'create_variant': 'dynamic',
+                'sequence': i,
+            })
+
+            for j in range(50):
+                # create the attribute values
+                value = self.env['product.attribute.value'].create([{
+                    'name': "val %s" % j,
+                    'attribute_id': product_attribute.id,
+                    'sequence': j,
+                }])
+
+            # set attribute and attribute values on the template
+            self.env['product.template.attribute.line'].create([{
+                'attribute_id': product_attribute.id,
+                'product_tmpl_id': product_template.id,
+                'value_ids': [(6, 0, product_attribute.value_ids.ids)]
+            }])
+
+        self._add_exclude(
+            self._get_product_template_attribute_value(product_template.attribute_line_ids[1].value_ids[0],
+                                                       model=product_template),
+            self._get_product_template_attribute_value(product_template.attribute_line_ids[0].value_ids[0],
+                                                       model=product_template),
+            product_template)
+        self._add_exclude(
+            self._get_product_template_attribute_value(product_template.attribute_line_ids[0].value_ids[0],
+                                                       model=product_template),
+            self._get_product_template_attribute_value(product_template.attribute_line_ids[1].value_ids[1],
+                                                       model=product_template),
+            product_template)
+
+        combination = self.env['product.template.attribute.value']
+        for idx, ptal in enumerate(product_template.attribute_line_ids):
+            if idx != 1:
+                value = ptal.product_template_value_ids[0]
+            else:
+                value = ptal.product_template_value_ids[2]
+            combination += value
+
+        started_at = time.time()
+        self.assertEqual(product_template._get_first_possible_combination(), combination)
+        elapsed = time.time() - started_at
+        # It should be about instantaneous, 0.5 to avoid false positives
+        self.assertLess(elapsed, 0.5)
+
+
+
 
     def test_get_closest_possible_combinations(self):
         computer_ssd_256 = self._get_product_template_attribute_value(self.ssd_256)
@@ -410,8 +534,7 @@ class TestProductAttributeValueConfig(TestProductAttributeValueSetup):
         self.assertEqual(next(gen), computer_ssd_256 + computer_ram_32 + computer_hdd_1)
         self.assertEqual(next(gen), computer_ssd_512 + computer_ram_8 + computer_hdd_1)
         self.assertEqual(next(gen), computer_ssd_512 + computer_ram_32 + computer_hdd_1)
-        with self.assertRaises(StopIteration):
-            next(gen)
+        self.assertIsNone(next(gen, None))
 
         # CASE contains computer_hdd_2
         self.assertEqual(self.computer._get_closest_possible_combination(computer_hdd_2),
