@@ -3,7 +3,10 @@
 
 import base64
 from collections import OrderedDict
+from datetime import timedelta
 import io
+import unittest.mock
+
 from PIL import Image
 
 from . import common
@@ -219,18 +222,18 @@ class TestVariants(common.TestProductCommon):
         company_b = self.env['res.company'].create({'name': 'CB', 'currency_id': self.env.ref('base.VEF').id})
 
         self.assertEqual(one_variant_product.cost_currency_id, company_a.currency_id)
-        self.assertEqual(one_variant_product.with_context(force_company=company_b.id).cost_currency_id, company_b.currency_id)
+        self.assertEqual(one_variant_product.with_company(company_b).cost_currency_id, company_b.currency_id)
 
         one_variant_template = one_variant_product.product_tmpl_id
         self.assertEqual(one_variant_product.standard_price, one_variant_template.standard_price)
-        one_variant_product.with_context(force_company=company_b.id).standard_price = 500.0
+        one_variant_product.with_company(company_b).standard_price = 500.0
         self.assertEqual(
-            one_variant_product.with_context(force_company=company_b.id).standard_price,
-            one_variant_template.with_context(force_company=company_b.id).standard_price
+            one_variant_product.with_company(company_b).standard_price,
+            one_variant_template.with_company(company_b).standard_price
         )
         self.assertEqual(
             500.0,
-            one_variant_template.with_context(force_company=company_b.id).standard_price
+            one_variant_template.with_company(company_b).standard_price
         )
 
     def test_archive_variant(self):
@@ -259,6 +262,46 @@ class TestVariants(common.TestProductCommon):
         self.assertTrue(variant_1.active)
         self.assertTrue(template.active)
 
+    def test_template_barcode(self):
+        template = self.env['product.template'].create({
+            'name': 'template',
+            'barcode': 'test',
+        })
+        self.assertEqual(len(template.product_variant_ids), 1)
+        self.assertEqual(template.barcode, 'test')
+
+        template.product_variant_ids.action_archive()
+        self.assertFalse(template.active)
+        template.invalidate_cache(['barcode'])
+        self.assertEqual(template.barcode, 'test')
+        template.product_variant_ids.action_unarchive()
+        template.action_unarchive()
+
+        template.write({
+            'attribute_line_ids': [(0, False, {
+                'attribute_id': self.size_attr.id,
+                'value_ids': [
+                    (4, self.size_attr.value_ids[0].id, self.size_attr_value_s),
+                    (4, self.size_attr.value_ids[1].id, self.size_attr_value_m)
+                ],
+            })]
+        })
+        self.assertFalse(template.barcode)  # 2 active variants --> no barcode on template
+
+        variant_1 = template.product_variant_ids[0]
+        variant_2 = template.product_variant_ids[1]
+
+        variant_1.barcode = 'v1_barcode'
+        variant_2.barcode = 'v2_barcode'
+
+        variant_1.action_archive()
+        template.invalidate_cache(['barcode'])
+        self.assertEqual(template.barcode, variant_2.barcode)  # 1 active variant --> barcode on template
+
+        variant_1.action_unarchive()
+        template.invalidate_cache(['barcode'])
+        self.assertFalse(template.barcode)  # 2 active variants --> no barcode on template
+
     def test_archive_all_variants(self):
         template = self.env['product.template'].create({
             'name': 'template'
@@ -622,6 +665,13 @@ class TestVariantsImages(common.TestProductCommon):
         """Check that on variant, the image used is the image_variant_1920 if set,
         and defaults to the template image otherwise.
         """
+        # Pretend setup happened in an older transaction by updating on the SQL layer and making sure it gets reloaded
+        # Using _write() instead of write() because write() only allows updating log access fields at boot time
+        self.variants._write({
+            'write_date': self.cr.now() - timedelta(milliseconds=1),
+        })
+        self.variants.invalidate_cache(['write_date'], self.variants.ids)
+
         f = io.BytesIO()
         Image.new('RGB', (800, 500), '#000000').save(f, 'PNG')
         f.seek(0)
@@ -629,10 +679,12 @@ class TestVariantsImages(common.TestProductCommon):
 
         images = self.variants.mapped('image_1920')
         self.assertEqual(len(set(images)), 4)
-
         variant_no_image = self.variants[0]
+        old_last_update = variant_no_image['__last_update']
+
         self.assertFalse(variant_no_image.image_1920)
         self.template.image_1920 = image_black
+        new_last_update = variant_no_image['__last_update']
 
         # the first has no image variant, all the others do
         self.assertFalse(variant_no_image.image_variant_1920)
@@ -643,6 +695,9 @@ class TestVariantsImages(common.TestProductCommon):
         # having changed the template image should not have changed these
         self.assertEqual(images[1:], self.variants.mapped('image_1920')[1:])
 
+        # last update changed for the variant without image
+        self.assertLess(old_last_update, new_last_update)
+
     def test_update_images_with_archived_variants(self):
         """Update images after variants have been archived"""
         self.variants[1:].write({'active': False})
@@ -1179,3 +1234,157 @@ class TestVariantsArchive(common.TestProductCommon):
         variants = variants or self.template.product_variant_ids
         self.assertEqual(len(variants), 1)
         self.assertFalse(variants[0].product_template_attribute_value_ids)
+
+
+class TestVariantWrite(TransactionCase):
+
+    def test_active_one2many(self):
+        template = self.env['product.template'].create({'name': 'Foo', 'description': 'Foo'})
+        self.assertEqual(len(template.product_variant_ids), 1)
+
+        # check the consistency of one2many field product_variant_ids w.r.t. active variants
+        variant1 = template.product_variant_ids
+        variant2 = self.env['product.product'].create({'product_tmpl_id': template.id})
+        self.assertEqual(template.product_variant_ids, variant1 + variant2)
+
+        variant2.active = False
+        self.assertEqual(template.product_variant_ids, variant1)
+
+        variant2.active = True
+        self.assertEqual(template.product_variant_ids, variant1 + variant2)
+
+        variant1.active = False
+        self.assertEqual(template.product_variant_ids, variant2)
+
+    def test_write_inherited_field(self):
+        product = self.env['product.product'].create({'name': 'Foo', 'sequence': 1})
+        self.assertEqual(product.name, 'Foo')
+        self.assertEqual(product.sequence, 1)
+
+        self.env['product.pricelist'].create({
+            'name': 'Foo',
+            'item_ids': [(0, 0, {'product_id': product.id, 'fixed_price': 1})],
+        })
+
+        # patch template.write to modify pricelist items, which causes some
+        # cache invalidation
+        Template = self.registry['product.template']
+        Template_write = Template.write
+
+        def write(self, vals):
+            result = Template_write(self, vals)
+            items = self.env['product.pricelist.item'].search([('product_id', '=', product.id)])
+            items.fixed_price = 2
+            return result
+
+        with unittest.mock.patch.object(Template, 'write', write):
+            # change both 'name' and 'sequence': due to some programmed cache
+            # invalidation, the second field may not be properly assigned
+            product.write({'name': 'Bar', 'sequence': 2})
+            self.assertEqual(product.name, 'Bar')
+            self.assertEqual(product.sequence, 2)
+
+
+class TestVariantsExclusion(common.TestProductCommon):
+    def setUp(self):
+        res = super(TestVariantsExclusion, self).setUp()
+        self.smartphone = self.env['product.template'].create({
+            'name': 'Smartphone',
+        })
+
+        self.size_attr = self.env['product.attribute'].create({'name': 'Size'})
+        self.size_attr_value_s = self.env['product.attribute.value'].create({'name': 'S', 'attribute_id': self.size_attr.id})
+        self.size_attr_value_xl = self.env['product.attribute.value'].create({'name': 'XL', 'attribute_id': self.size_attr.id})
+
+        self.storage_attr = self.env['product.attribute'].create({'name': 'Storage'})
+        self.storage_attr_value_128 = self.env['product.attribute.value'].create({'name': '128', 'attribute_id': self.storage_attr.id})
+        self.storage_attr_value_256 = self.env['product.attribute.value'].create({'name': '256', 'attribute_id': self.storage_attr.id})
+
+        # add attributes to product
+        self.smartphone_size_attribute_lines = self.env['product.template.attribute.line'].create({
+            'product_tmpl_id': self.smartphone.id,
+            'attribute_id': self.size_attr.id,
+            'value_ids': [(6, 0, [self.size_attr_value_s.id, self.size_attr_value_xl.id])],
+        })
+
+        self.smartphone_storage_attribute_lines = self.env['product.template.attribute.line'].create({
+            'product_tmpl_id': self.smartphone.id,
+            'attribute_id': self.storage_attr.id,
+            'value_ids': [(6, 0, [self.storage_attr_value_128.id, self.storage_attr_value_256.id])],
+        })
+
+        def get_ptav(model, att):
+            return model.valid_product_template_attribute_line_ids.filtered(
+                lambda l: l.attribute_id == att.attribute_id
+            ).product_template_value_ids.filtered(
+                lambda v: v.product_attribute_value_id == att
+            )
+        self.smartphone_s = get_ptav(self.smartphone, self.size_attr_value_s)
+        self.smartphone_256 = get_ptav(self.smartphone, self.storage_attr_value_256)
+        self.smartphone_128 = get_ptav(self.smartphone, self.storage_attr_value_128)
+        return res
+
+    def test_variants_1_exclusion(self):
+        # Create one exclusion for Smartphone S
+        self.smartphone_s.write({
+            'exclude_for': [(0, 0, {
+                'product_tmpl_id': self.smartphone.id,
+                'value_ids': [(6, 0, [self.smartphone_256.id])]
+            })]
+        })
+        self.assertEqual(len(self.smartphone.product_variant_ids), 3, 'With exclusion {s: [256]}, the smartphone should have 3 active different variants')
+
+        # Delete exclusion
+        self.smartphone_s.write({
+            'exclude_for': [(2, self.smartphone_s.exclude_for.id, 0)]
+        })
+        self.assertEqual(len(self.smartphone.product_variant_ids), 4, 'With no exclusion, the smartphone should have 4 active different variants')
+
+    def test_variants_2_exclusions_same_line(self):
+        # Create two exclusions for Smartphone S on the same line
+        self.smartphone_s.write({
+            'exclude_for': [(0, 0, {
+                'product_tmpl_id': self.smartphone.id,
+                'value_ids': [(6, 0, [self.smartphone_128.id, self.smartphone_256.id])]
+            })]
+        })
+        self.assertEqual(len(self.smartphone.product_variant_ids), 2, 'With exclusion {s: [128, 256]}, the smartphone should have 2 active different variants')
+
+        # Delete one exclusion of the line
+        self.smartphone_s.write({
+            'exclude_for': [(1, self.smartphone_s.exclude_for.id, {
+                'product_tmpl_id': self.smartphone.id,
+                'value_ids': [(6, 0, [self.smartphone_128.id])]
+            })]
+        })
+        self.assertEqual(len(self.smartphone.product_variant_ids), 3, 'With exclusion {s: [128]}, the smartphone should have 3 active different variants')
+
+        # Delete exclusion
+        self.smartphone_s.write({
+            'exclude_for': [(2, self.smartphone_s.exclude_for.id, 0)]
+        })
+        self.assertEqual(len(self.smartphone.product_variant_ids), 4, 'With no exclusion, the smartphone should have 4 active different variants')
+
+    def test_variants_2_exclusions_different_lines(self):
+        # add 1 exclusion
+        self.smartphone_s.write({
+            'exclude_for': [(0, 0, {
+                'product_tmpl_id': self.smartphone.id,
+                'value_ids': [(6, 0, [self.smartphone_128.id])]
+            })]
+        })
+
+        # add 1 exclusion on a different line
+        self.smartphone_s.write({
+            'exclude_for': [(0, 0, {
+                'product_tmpl_id': self.smartphone.id,
+                'value_ids': [(6, 0, [self.smartphone_256.id])]
+            })]
+        })
+        self.assertEqual(len(self.smartphone.product_variant_ids), 2, 'With exclusion {s: [128, 256]}, the smartphone should have 2 active different variants')
+
+        # delete one exclusion line
+        self.smartphone_s.write({
+            'exclude_for': [(2, self.smartphone_s.exclude_for.ids[0], 0)]
+        })
+        self.assertEqual(len(self.smartphone.product_variant_ids), 3, 'With one exclusion, the smartphone should have 3 active different variants')
