@@ -3,4 +3,7 @@
 
 from . import test_purchase
 from . import test_purchase_order_report
+from . import test_purchase_invoice
 from . import test_access_rights
+from . import test_accrued_purchase_orders
+from . import test_purchase_tax_totals
