@@ -2,9 +2,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.tests.common import Form, TransactionCase
+from odoo.tests import tagged
 from odoo import fields
 
 
+@tagged('post_install', '-at_install')
 class TestSaleMrpFlow(TransactionCase):
 
     def setUp(self):
@@ -145,7 +147,7 @@ class TestSaleMrpFlow(TransactionCase):
     def _create_product(self, name, uom_id, routes=()):
         p = Form(self.env['product.product'])
         p.name = name
-        p.type = 'product'
+        p.detailed_type = 'product'
         p.uom_id = uom_id
         p.uom_po_id = uom_id
         p.route_ids.clear()
@@ -178,7 +180,7 @@ class TestSaleMrpFlow(TransactionCase):
         """
         moves_to_process = moves.filtered(lambda m: m.product_id in quantities_to_process.keys())
         for move in moves_to_process:
-            self.assertEquals(move.product_uom_qty, quantities_to_process[move.product_id])
+            self.assertEqual(move.product_uom_qty, quantities_to_process[move.product_id])
 
     def _create_move_quantities(self, qty_to_process, components, warehouse):
         """ Helper to creates moves in order to update the quantities of components
@@ -224,7 +226,7 @@ class TestSaleMrpFlow(TransactionCase):
         #              |- component_e x1
 
         # Creation of a sale order for x7 kit_parent
-        partner = self.env.ref('base.res_partner_1')
+        partner = self.env['res.partner'].create({'name': 'My Test Partner'})
         f = Form(self.env['purchase.order'])
         f.partner_id = partner
         with f.order_line.new() as line:
@@ -238,7 +240,7 @@ class TestSaleMrpFlow(TransactionCase):
         # Check picking creation, its move lines should concern
         # only components. Also checks that the quantities are corresponding
         # to the PO
-        self.assertEquals(len(po.picking_ids), 1)
+        self.assertEqual(len(po.picking_ids), 1)
         order_line = po.order_line[0]
         picking_original = po.picking_ids[0]
         move_lines = picking_original.move_lines
@@ -256,7 +258,7 @@ class TestSaleMrpFlow(TransactionCase):
             self.component_g: 28.0
         }
 
-        self.assertEquals(len(move_lines), 7)
+        self.assertEqual(len(move_lines), 7)
         self.assertTrue(not any(kit in products for kit in kits))
         self.assertTrue(all(component in products for component in components))
         self._assert_quantities(move_lines, expected_quantities)
@@ -266,17 +268,18 @@ class TestSaleMrpFlow(TransactionCase):
         move_lines.write({'quantity_done': qty_to_process})
 
         # Create a backorder for the missing componenents
-        backorder_wizard = self.env['stock.backorder.confirmation'].create({'pick_ids': [(4, po.picking_ids[0].id)]})
-        backorder_wizard.process()
+        pick = po.picking_ids[0]
+        res = pick.button_validate()
+        Form(self.env[res['res_model']].with_context(res['context'])).save().process()
 
         # Check that a backorded is created
-        self.assertEquals(len(po.picking_ids), 2)
+        self.assertEqual(len(po.picking_ids), 2)
         backorder_1 = po.picking_ids - picking_original
-        self.assertEquals(backorder_1.backorder_id.id, picking_original.id)
+        self.assertEqual(backorder_1.backorder_id.id, picking_original.id)
 
         # Even if some components are received completely,
         # no KitParent should be received
-        self.assertEquals(order_line.qty_received, 0)
+        self.assertEqual(order_line.qty_received, 0)
 
         # Process just enough components to make 1 kit_parent
         qty_to_process = {
@@ -286,16 +289,16 @@ class TestSaleMrpFlow(TransactionCase):
         self._process_quantities(backorder_1.move_lines, qty_to_process)
 
         # Create a backorder for the missing componenents
-        backorder_wizard = self.env['stock.backorder.confirmation'].create({'pick_ids': [(4, backorder_1.id)]})
-        backorder_wizard.process()
+        res = backorder_1.button_validate()
+        Form(self.env[res['res_model']].with_context(res['context'])).save().process()
 
         # Only 1 kit_parent should be received at this point
-        self.assertEquals(order_line.qty_received, 1)
+        self.assertEqual(order_line.qty_received, 1)
 
         # Check that the second backorder is created
-        self.assertEquals(len(po.picking_ids), 3)
+        self.assertEqual(len(po.picking_ids), 3)
         backorder_2 = po.picking_ids - picking_original - backorder_1
-        self.assertEquals(backorder_2.backorder_id.id, backorder_1.id)
+        self.assertEqual(backorder_2.backorder_id.id, backorder_1.id)
 
         # Set the components quantities that backorder_2 should have
         expected_quantities = {
@@ -310,7 +313,7 @@ class TestSaleMrpFlow(TransactionCase):
         # Check that the computed quantities are matching the theorical ones.
         # Since component_e was totally processed, this componenent shouldn't be
         # present in backorder_2
-        self.assertEquals(len(backorder_2.move_lines), 6)
+        self.assertEqual(len(backorder_2.move_lines), 6)
         move_comp_e = backorder_2.move_lines.filtered(lambda m: m.product_id.id == self.component_e.id)
         self.assertFalse(move_comp_e)
         self._assert_quantities(backorder_2.move_lines, expected_quantities)
@@ -325,16 +328,16 @@ class TestSaleMrpFlow(TransactionCase):
         self._process_quantities(backorder_2.move_lines, qty_to_process)
 
         # Create a backorder for the missing componenents
-        backorder_wizard = self.env['stock.backorder.confirmation'].create({'pick_ids': [(4, backorder_2.id)]})
-        backorder_wizard.process()
+        res = backorder_2.button_validate()
+        Form(self.env[res['res_model']].with_context(res['context'])).save().process()
 
         # Check that x3 kit_parents are indeed received
-        self.assertEquals(order_line.qty_received, 3)
+        self.assertEqual(order_line.qty_received, 3)
 
         # Check that the third backorder is created
-        self.assertEquals(len(po.picking_ids), 4)
+        self.assertEqual(len(po.picking_ids), 4)
         backorder_3 = po.picking_ids - (picking_original + backorder_1 + backorder_2)
-        self.assertEquals(backorder_3.backorder_id.id, backorder_2.id)
+        self.assertEqual(backorder_3.backorder_id.id, backorder_2.id)
 
         # Check the components quantities that backorder_3 should have
         expected_quantities = {
@@ -353,7 +356,7 @@ class TestSaleMrpFlow(TransactionCase):
         # Validating the last backorder now it's complete.
         # All kits should be received
         backorder_3.button_validate()
-        self.assertEquals(order_line.qty_received, 7.0)
+        self.assertEqual(order_line.qty_received, 7.0)
 
         # Return all components processed by backorder_3
         stock_return_picking_form = Form(self.env['stock.return.picking']
@@ -370,11 +373,11 @@ class TestSaleMrpFlow(TransactionCase):
 
         # Process all components and validate the picking
         wiz_act = return_pick.button_validate()
-        wiz = self.env[wiz_act['res_model']].browse(wiz_act['res_id'])
+        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
         wiz.process()
 
         # Now quantity received should be 3 again
-        self.assertEquals(order_line.qty_received, 3)
+        self.assertEqual(order_line.qty_received, 3)
 
         stock_return_picking_form = Form(self.env['stock.return.picking']
             .with_context(active_ids=return_pick.ids, active_id=return_pick.ids[0],
@@ -392,25 +395,101 @@ class TestSaleMrpFlow(TransactionCase):
                 'to_refund': True
             })
 
-        backorder_wizard = self.env['stock.backorder.confirmation'].create({'pick_ids': [(4, return_of_return_pick.id)]})
-        backorder_wizard.process()
+        wiz_act = return_of_return_pick.button_validate()
+        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
+        wiz.process()
 
         # As one of each component is missing, only 6 kit_parents should be received
-        self.assertEquals(order_line.qty_received, 6)
+        self.assertEqual(order_line.qty_received, 6)
 
         # Check that the 4th backorder is created.
-        self.assertEquals(len(po.picking_ids), 7)
+        self.assertEqual(len(po.picking_ids), 7)
         backorder_4 = po.picking_ids - (
                     picking_original + backorder_1 + backorder_2 + backorder_3 + return_of_return_pick + return_pick)
-        self.assertEquals(backorder_4.backorder_id.id, return_of_return_pick.id)
+        self.assertEqual(backorder_4.backorder_id.id, return_of_return_pick.id)
 
         # Check the components quantities that backorder_4 should have
         for move in backorder_4.move_lines:
-            self.assertEquals(move.product_qty, 1)
+            self.assertEqual(move.product_qty, 1)
+
+    def test_concurent_procurements(self):
+        """ Check a production created to fulfill a procurement will not
+        replenish more that needed if others procurements have the same products
+        than the production component. """
+
+        warehouse = self.env.ref('stock.warehouse0')
+        buy_route = warehouse.buy_pull_id.route_id
+        manufacture_route = warehouse.manufacture_pull_id.route_id
+
+        vendor1 = self.env['res.partner'].create({'name': 'aaa', 'email': 'from.test@example.com'})
+        supplier_info1 = self.env['product.supplierinfo'].create({
+            'name': vendor1.id,
+            'price': 50,
+        })
+
+        component = self.env['product.product'].create({
+            'name': 'component',
+            'type': 'product',
+            'route_ids': [(4, buy_route.id)],
+            'seller_ids': [(6, 0, [supplier_info1.id])],
+        })
+        finished = self.env['product.product'].create({
+            'name': 'finished',
+            'type': 'product',
+            'route_ids': [(4, manufacture_route.id)],
+        })
+        self.env['stock.warehouse.orderpoint'].create({
+            'name': 'A RR',
+            'location_id': warehouse.lot_stock_id.id,
+            'product_id': component.id,
+            'route_id': buy_route.id,
+            'product_min_qty': 0,
+            'product_max_qty': 0,
+        })
+        self.env['stock.warehouse.orderpoint'].create({
+            'name': 'A RR',
+            'location_id': warehouse.lot_stock_id.id,
+            'product_id': finished.id,
+            'route_id': manufacture_route.id,
+            'product_min_qty': 0,
+            'product_max_qty': 0,
+        })
+
+        self.env['mrp.bom'].create({
+            'product_id': finished.id,
+            'product_tmpl_id': finished.product_tmpl_id.id,
+            'product_uom_id': self.uom_unit.id,
+            'product_qty': 1.0,
+            'consumption': 'flexible',
+            'operation_ids': [
+            ],
+            'type': 'normal',
+            'bom_line_ids': [
+                (0, 0, {'product_id': component.id, 'product_qty': 1}),
+            ]})
+
+        # Delivery to trigger replenishment
+        picking_form = Form(self.env['stock.picking'])
+        picking_form.picking_type_id = warehouse.out_type_id
+        with picking_form.move_ids_without_package.new() as move:
+            move.product_id = finished
+            move.product_uom_qty = 3
+        with picking_form.move_ids_without_package.new() as move:
+            move.product_id = component
+            move.product_uom_qty = 2
+        picking = picking_form.save()
+        picking.action_confirm()
+
+        # Find PO
+        purchase = self.env['purchase.order.line'].search([
+            ('product_id', '=', component.id),
+        ]).order_id
+        self.assertTrue(purchase)
+        self.assertEqual(purchase.order_line.product_qty, 5)
 
     def test_01_purchase_mrp_kit_qty_change(self):
         self.partner = self.env.ref('base.res_partner_1')
-        
+
         # Create a PO with one unit of the kit product
         self.po = self.env['purchase.order'].create({
             'partner_id': self.partner.id,
@@ -418,7 +497,7 @@ class TestSaleMrpFlow(TransactionCase):
         })
         # Validate the PO
         self.po.button_confirm()
-        
+
         # Check the component qty in the created picking
         self.assertEqual(self.po.picking_ids.move_ids_without_package[0].product_uom_qty,2, "The quantity of components must be created according to the BOM")
         self.assertEqual(self.po.picking_ids.move_ids_without_package[1].product_uom_qty,1, "The quantity of components must be created according to the BOM")
