@@ -3,8 +3,9 @@
 
 from datetime import date, datetime, timedelta
 
+from odoo.addons.mail.tests.common import mail_new_test_user
 from odoo.addons.product.tests import common
-from odoo.tests.common import Form
+from odoo.tests import Form
 from odoo.tools import DEFAULT_SERVER_DATETIME_FORMAT
 
 
@@ -12,18 +13,18 @@ class TestCreatePicking(common.TestProductCommon):
 
     def setUp(self):
         super(TestCreatePicking, self).setUp()
-        self.partner_id = self.env.ref('base.res_partner_1')
-        self.product_id_1 = self.env.ref('product.product_product_8')
-        self.product_id_2 = self.env.ref('product.product_product_11')
-        res_users_purchase_user = self.env.ref('purchase.group_purchase_user')
+        self.partner_id = self.env['res.partner'].create({'name': 'Wood Corner Partner'})
+        self.product_id_1 = self.env['product.product'].create({'name': 'Large Desk'})
+        self.product_id_2 = self.env['product.product'].create({'name': 'Conference Chair'})
 
-        Users = self.env['res.users'].with_context({'no_reset_password': True, 'mail_create_nosubscribe': True})
-        self.user_purchase_user = Users.create({
-            'name': 'Pauline Poivraisselle',
-            'login': 'pauline',
-            'email': 'pur@example.com',
-            'notification_type': 'inbox',
-            'groups_id': [(6, 0, [res_users_purchase_user.id])]})
+        self.user_purchase_user = mail_new_test_user(
+            self.env,
+            name='Pauline Poivraisselle',
+            login='pauline',
+            email='pur@example.com',
+            notification_type='inbox',
+            groups='purchase.group_purchase_user',
+        )
 
         self.po_vals = {
             'partner_id': self.partner_id.id,
@@ -47,7 +48,7 @@ class TestCreatePicking(common.TestProductCommon):
         # Purchase order confirm
         self.po.button_confirm()
         self.assertEqual(self.po.state, 'purchase', 'Purchase: PO state should be "Purchase')
-        self.assertEqual(self.po.picking_count, 1, 'Purchase: one picking should be created')
+        self.assertEqual(self.po.incoming_picking_count, 1, 'Purchase: one picking should be created')
         self.assertEqual(len(self.po.order_line.move_ids), 1, 'One move should be created')
         # Change purchase order line product quantity
         self.po.order_line.write({'product_qty': 7.0})
@@ -57,10 +58,9 @@ class TestCreatePicking(common.TestProductCommon):
         self.picking = self.po.picking_ids[0]
         for ml in self.picking.move_line_ids:
             ml.qty_done = ml.product_uom_qty
-        self.picking.action_done()
+        self.picking._action_done()
         self.assertEqual(self.po.order_line.mapped('qty_received'), [7.0], 'Purchase: all products should be received')
 
-
         # create new order line
         self.po.write({'order_line': [
             (0, 0, {
@@ -71,7 +71,7 @@ class TestCreatePicking(common.TestProductCommon):
                 'price_unit': 250.0,
                 'date_planned': datetime.today().strftime(DEFAULT_SERVER_DATETIME_FORMAT),
                 })]})
-        self.assertEqual(self.po.picking_count, 2, 'New picking should be created')
+        self.assertEqual(self.po.incoming_picking_count, 2, 'New picking should be created')
         moves = self.po.order_line.mapped('move_ids').filtered(lambda x: x.state not in ('done', 'cancel'))
         self.assertEqual(len(moves), 1, 'One moves should have been created')
 
@@ -89,6 +89,7 @@ class TestCreatePicking(common.TestProductCommon):
         self.assertEqual(self.po.state, 'to approve', 'Purchase: PO state should be "to approve".')
 
         # PO approved by manager
+        self.po.env.user.groups_id += self.env.ref("purchase.group_purchase_manager")
         self.po.button_approve()
         self.assertEqual(self.po.state, 'purchase', 'PO state should be "Purchase".')
 
@@ -98,8 +99,9 @@ class TestCreatePicking(common.TestProductCommon):
         following move to MTS in order to be able to link it to a
         manually created purchase order.
         """
-        stock_location = self.env['ir.model.data'].xmlid_to_object('stock.stock_location_stock')
-        customer_location = self.env['ir.model.data'].xmlid_to_object('stock.stock_location_customers')
+        stock_location = self.env.ref('stock.stock_location_stock')
+        customer_location = self.env.ref('stock.stock_location_customers')
+        picking_type_out = self.env.ref('stock.picking_type_out')
         # route buy should be there by default
         partner = self.env['res.partner'].create({
             'name': 'Jhon'
@@ -131,6 +133,7 @@ class TestCreatePicking(common.TestProductCommon):
             'product_uom': product.uom_id.id,
             'product_uom_qty': 100.0,
             'procure_method': 'make_to_order',
+            'picking_type_id': picking_type_out.id,
         })
 
         customer_move._action_confirm()
@@ -141,8 +144,8 @@ class TestCreatePicking(common.TestProductCommon):
         # Check purchase order line data.
         purchase_order_line = purchase_order.order_line
         self.assertEqual(purchase_order_line.product_id, product, 'The product on the purchase order line is not correct.')
-        self.assertEqual(purchase_order_line.price_unit, seller.price, 'The purchase order line price should be the same than the seller.')
-        self.assertEqual(purchase_order_line.product_qty, customer_move.product_uom_qty, 'The purchase order line qty should be the same than the move.')
+        self.assertEqual(purchase_order_line.price_unit, seller.price, 'The purchase order line price should be the same as the seller.')
+        self.assertEqual(purchase_order_line.product_qty, customer_move.product_uom_qty, 'The purchase order line qty should be the same as the move.')
         self.assertEqual(purchase_order_line.price_subtotal, 1200.0, 'The purchase order line subtotal should be equal to the move qty * seller price.')
 
         purchase_order.button_cancel()
@@ -172,10 +175,9 @@ class TestCreatePicking(common.TestProductCommon):
         picking.move_lines.quantity_done = 100.0
         picking.button_validate()
 
-        self.assertEqual(self.env['stock.quant']._get_available_quantity(product, stock_location), 100.0, 'Wrong quantity in stock.')
-
-        customer_move._action_assign()
-        self.assertEqual(customer_move.state, 'assigned', 'Reservation should work with the new quantity provided by the PO.')
+        # mts move will be automatically assigned
+        self.assertEqual(customer_move.state, 'assigned', 'Automatically assigned due to the incoming move makes it available.')
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(product, stock_location), 0.0, 'Wrong quantity in stock.')
 
     def test_03_uom(self):
         """ Buy a dozen of products stocked in units. Check that the quantities on the purchase order
@@ -235,9 +237,9 @@ class TestCreatePicking(common.TestProductCommon):
         and receipt the picking then try to reserve the delivery
         picking.
         """
-        stock_location = self.env['ir.model.data'].xmlid_to_object('stock.stock_location_stock')
-        customer_location = self.env['ir.model.data'].xmlid_to_object('stock.stock_location_customers')
-        picking_type_out = self.env['ir.model.data'].xmlid_to_object('stock.picking_type_out')
+        stock_location = self.env.ref('stock.stock_location_stock')
+        customer_location = self.env.ref('stock.stock_location_customers')
+        picking_type_out = self.env.ref('stock.picking_type_out')
         # route buy should be there by default
         partner = self.env['res.partner'].create({
             'name': 'Jhon'
@@ -283,8 +285,8 @@ class TestCreatePicking(common.TestProductCommon):
         # Check purchase order line data.
         purchase_order_line = purchase_order.order_line
         self.assertEqual(purchase_order_line.product_id, product, 'The product on the purchase order line is not correct.')
-        self.assertEqual(purchase_order_line.price_unit, seller.price, 'The purchase order line price should be the same than the seller.')
-        self.assertEqual(purchase_order_line.product_qty, customer_move.product_uom_qty, 'The purchase order line qty should be the same than the move.')
+        self.assertEqual(purchase_order_line.price_unit, seller.price, 'The purchase order line price should be the same as the seller.')
+        self.assertEqual(purchase_order_line.product_qty, customer_move.product_uom_qty, 'The purchase order line qty should be the same as the move.')
 
         purchase_order.button_confirm()
 
@@ -379,11 +381,11 @@ class TestCreatePicking(common.TestProductCommon):
         self.assertEqual(move2.product_uom.id, uom_dozen.id)
         self.assertEqual(move2.product_qty, 24)
 
-    def create_delivery_order(self, propagate_date, propagate_date_minimum_delta):
-        stock_location = self.env['ir.model.data'].xmlid_to_object('stock.stock_location_stock')
-        customer_location = self.env['ir.model.data'].xmlid_to_object('stock.stock_location_customers')
+    def create_delivery_order(self):
+        stock_location = self.env.ref('stock.stock_location_stock')
+        customer_location = self.env.ref('stock.stock_location_customers')
         unit = self.ref("uom.product_uom_unit")
-        picking_type_out = self.env['ir.model.data'].xmlid_to_object('stock.picking_type_out')
+        picking_type_out = self.env.ref('stock.picking_type_out')
         partner = self.env['res.partner'].create({'name': 'AAA', 'email': 'from.test@example.com'})
         supplier_info1 = self.env['product.supplierinfo'].create({
             'name': partner.id,
@@ -394,10 +396,6 @@ class TestCreatePicking(common.TestProductCommon):
         route_buy = warehouse1.buy_pull_id.route_id
         route_mto = warehouse1.mto_pull_id.route_id
 
-        # change propagete date and Reschedule if Higher Than on rules
-        route_buy.rule_ids.write({'propagate_date': propagate_date,
-            'propagate_date_minimum_delta': propagate_date_minimum_delta})
-
         product = self.env['product.product'].create({
             'name': 'Usb Keyboard',
             'type': 'product',
@@ -431,13 +429,11 @@ class TestCreatePicking(common.TestProductCommon):
 
         return delivery_order, purchase_order
 
-    def test_05_if_propagate_date(self):
-        """ In order to check scheduled date of the delivery order is changed based
-            stock rules with propagate date and minimum delta.
-        """
+    def test_05_propagate_deadline(self):
+        """ In order to check deadline date of the delivery order is changed and the planned date not."""
 
         # Create Delivery Order and with propagate date and minimum delta
-        delivery_order, purchase_order = self.create_delivery_order(True, 5)
+        delivery_order, purchase_order = self.create_delivery_order()
 
         # check po is created or not
         self.assertTrue(purchase_order, 'No purchase order created.')
@@ -447,39 +443,17 @@ class TestCreatePicking(common.TestProductCommon):
         # change scheduled date of po line.
         purchase_order_line.write({'date_planned': purchase_order_line.date_planned + timedelta(days=5)})
 
-        # Now check scheduled date of delivery order is changed or not.
-        self.assertEquals(purchase_order_line.date_planned, delivery_order.scheduled_date,
-            'Delivery order schedule date should be changed as we have set date propagate.')
+        # Now check scheduled date and deadline of delivery order.
+        self.assertNotEqual(
+            purchase_order_line.date_planned, delivery_order.scheduled_date,
+            'Scheduled delivery order date should not changed.')
+        self.assertEqual(
+            purchase_order_line.date_planned, delivery_order.date_deadline,
+            'Delivery deadline date should be changed.')
 
-    def test_06_no_propagate_date(self):
-        """ In order to check scheduled date of the delivery order is changed based
-            stock rules without propagate date and minimum delta.
-        """
-
-        # Create Delivery Order and without propagate date and minimum delta
-        delivery_order, purchase_order = self.create_delivery_order(False, 5)
-
-        # check po is created or not
-        self.assertTrue(purchase_order, 'No purchase order created.')
-
-        purchase_order_line = purchase_order.order_line
-
-        # change scheduled date of po line.
-        purchase_order_line.write({'date_planned': purchase_order_line.date_planned + timedelta(days=5)})
-
-        # Now check scheduled date of delivery order is changed or not.
-        self.assertNotEquals(purchase_order_line.date_planned, delivery_order.scheduled_date,
-            'Delivery order schedule date should not changed.')
-
-    def test_06_differed_schedule_date(self):
+    def test_07_differed_schedule_date(self):
         warehouse = self.env['stock.warehouse'].search([], limit=1)
 
-        # mark all rules as propagate_date so that push rules will use it
-        self.env['stock.rule'].search([]).write({
-            'propagate_date': True,
-            'propagate_date_minimum_delta': 5,
-        })
-
         with Form(warehouse) as w:
             w.reception_steps = 'three_steps'
         po_form = Form(self.env['purchase.order'])
@@ -494,12 +468,6 @@ class TestCreatePicking(common.TestProductCommon):
             line.product_qty = 1.0
         po = po_form.save()
 
-        # mark the po line as propagate_date so that the created moves will use it
-        po.order_line.write({
-            'propagate_date': True,
-            'propagate_date_minimum_delta': 5,
-        })
-
         po.button_approve()
 
         po.picking_ids.move_line_ids.write({
@@ -510,3 +478,124 @@ class TestCreatePicking(common.TestProductCommon):
         pickings = self.env['stock.picking'].search([('group_id', '=', po.group_id.id)])
         for picking in pickings:
             self.assertEqual(picking.scheduled_date.date(), date.today())
+
+    def test_update_quantity_and_return(self):
+        po = self.env['purchase.order'].create(self.po_vals)
+
+        po.order_line.product_qty = 10
+        po.button_confirm()
+
+        first_picking = po.picking_ids
+        first_picking.move_lines.quantity_done = 5
+        # create the backorder
+        backorder_wizard_dict = first_picking.button_validate()
+        backorder_wizard = Form(self.env[backorder_wizard_dict['res_model']].with_context(backorder_wizard_dict['context'])).save()
+        backorder_wizard.process()
+
+        self.assertEqual(len(po.picking_ids), 2)
+
+        # Create a partial return
+        stock_return_picking_form = Form(
+            self.env['stock.return.picking'].with_context(
+                active_ids=first_picking.ids,
+                active_id=first_picking.ids[0],
+                active_model='stock.picking'
+            )
+        )
+        stock_return_picking = stock_return_picking_form.save()
+        stock_return_picking.product_return_moves.quantity = 2.0
+        stock_return_picking_action = stock_return_picking.create_returns()
+        return_pick = self.env['stock.picking'].browse(stock_return_picking_action['res_id'])
+        return_pick.action_assign()
+        return_pick.move_lines.quantity_done = 2
+        return_pick._action_done()
+
+        self.assertEqual(po.order_line.qty_received, 3)
+
+        po.order_line.product_qty += 2
+        backorder = po.picking_ids.filtered(lambda picking: picking.state == 'assigned')
+        self.assertEqual(backorder.move_lines.product_uom_qty, 9)
+
+    def test_08_check_update_qty_mto_chain(self):
+        """ Simulate a mto chain with a purchase order. Updating the
+        initial demand should also impact the initial move and the
+        purchase order if it wasn't yet confirmed.
+        """
+        def create_run_procurement(product, product_qty, values=None):
+            if not values:
+                values = {
+                    'warehouse_id': picking_type_out.warehouse_id,
+                    'action': 'pull_push',
+                    'group_id': procurement_group,
+                }
+            return self.env['procurement.group'].run([self.env['procurement.group'].Procurement(
+                product, product_qty, self.uom_unit, vendor.property_stock_customer,
+                product.name, '/', self.env.company, values)
+            ])
+
+        # Prepare procurement that replicates a sale order.
+        picking_type_out = self.env.ref('stock.picking_type_out')
+        partner = self.env['res.partner'].create({
+            'name': 'Jhon'
+        })
+        seller = self.env['product.supplierinfo'].create({
+            'name': partner.id,
+            'price': 12.0,
+        })
+        vendor = self.env['res.partner'].create({
+            'name': 'Roger'
+        })
+        # This needs to be tried with MTO route activated
+        self.env['stock.location.route'].browse(self.ref('stock.route_warehouse0_mto')).action_unarchive()
+        product = self.env['product.product'].create({
+            'name': 'product',
+            'type': 'product',
+            'route_ids': [(4, self.ref('stock.route_warehouse0_mto')), (4, self.ref('purchase_stock.route_warehouse0_buy'))],
+            'seller_ids': [(6, 0, [seller.id])],
+            'categ_id': self.env.ref('product.product_category_all').id,
+            'supplier_taxes_id': [(6, 0, [])],
+        })
+
+        procurement_group = self.env['procurement.group'].create({
+            'move_type': 'direct',
+            'partner_id': vendor.id
+        })
+        # Create initial procurement that will generate the initial move and its picking.
+        create_run_procurement(product, 50, {
+            'group_id': procurement_group,
+            'warehouse_id': picking_type_out.warehouse_id,
+            'partner_id': vendor
+        })
+        customer_move = self.env['stock.move'].search([('group_id', '=', procurement_group.id)])
+        purchase_order = self.env['purchase.order'].search([('partner_id', '=', partner.id)])
+        self.assertTrue(purchase_order, 'No purchase order created.')
+
+        # Check purchase order line data.
+        purchase_order_line = purchase_order.order_line
+        self.assertEqual(purchase_order_line.product_id, product, 'The product on the purchase order line is not correct.')
+        self.assertEqual(purchase_order_line.product_qty, 50, 'The purchase order line qty should be the same as the move.')
+
+        # Create procurement to decrease quantity in the initial move and the related RFQ.
+        create_run_procurement(product, -10.00)
+        self.assertEqual(customer_move.product_uom_qty, 40, 'The demand on the initial move should have been decreased when merged with the procurement.')
+        self.assertEqual(purchase_order_line.product_qty, 40, 'The demand on the Purchase Order should have been decreased since it is still a RFQ.')
+
+        # Create procurement to increase quantity on the initial move and the related RFQ.
+        create_run_procurement(product, 5.00)
+        self.assertEqual(customer_move.product_uom_qty, 45, 'The demand on the initial move should have been increased when merged with the procurement.')
+        self.assertEqual(purchase_order_line.product_qty, 45, 'The demand on the Purchase Order should have been increased since it is still a RFQ.')
+
+        purchase_order.button_confirm()
+        # Create procurement to decrease quantity in the initial move but not the confirmed PO.
+        create_run_procurement(product, -10.00)
+        self.assertEqual(customer_move.product_uom_qty, 35, 'The demand on the initial move should have been decreased when merged with the procurement.')
+        self.assertEqual(purchase_order_line.product_qty, 45, 'The demand on the Purchase Order should not have been decreased since it is has been confirmed.')
+        purchase_orders = self.env['purchase.order'].search([('partner_id', '=', partner.id)])
+        self.assertEqual(len(purchase_orders), 1, 'No RFQ should have been created for a negative demand')
+
+        # Create procurement to increase quantity on the initial move that will create a new move and a new RFQ for missing demand.
+        create_run_procurement(product, 5.00)
+        self.assertEqual(customer_move.product_uom_qty, 35, 'The demand on the initial move should not have been increased since it should be a new move.')
+        self.assertEqual(purchase_order_line.product_qty, 45, 'The demand on the Purchase Order should not have been increased since it is has been confirmed.')
+        purchase_orders = self.env['purchase.order'].search([('partner_id', '=', partner.id)])
+        self.assertEqual(len(purchase_orders), 2, 'A new RFQ should have been created for missing demand.')
