@@ -5,8 +5,8 @@
 {
     'name': 'Repairs',
     'version': '1.0',
-    'sequence': 200,
-    'category': 'Manufacturing/Manufacturing',
+    'sequence': 230,
+    'category': 'Inventory/Inventory',
     'summary': 'Repair damaged products',
     'description': """
 The aim is to have a complete module to manage all products repairs.
@@ -25,14 +25,15 @@ The following topics are covered by this module:
     'data': [
         'security/ir.model.access.csv',
         'security/repair_security.xml',
-        'wizard/repair_cancel_views.xml',
         'wizard/repair_make_invoice_views.xml',
         'wizard/stock_warn_insufficient_qty_views.xml',
+        'views/stock_move_views.xml',
         'views/repair_views.xml',
+        'views/stock_production_lot_views.xml',
         'report/repair_reports.xml',
         'report/repair_templates_repair_order.xml',
         'data/ir_sequence_data.xml',
-        'data/repair_data.xml',
+        'data/mail_template_data.xml',
     ],
     'demo': ['data/repair_demo.xml'],
     'test': [],
