@@ -3,3 +3,6 @@
 
 from . import repair
 from . import stock_traceability
+from . import stock_production_lot
+from . import account_move
+from . import product
