@@ -13,13 +13,19 @@ A resource represent something that can be scheduled (a developer on a task or a
 work center on manufacturing orders). This module manages a resource calendar
 associated to every resource. It also manages the leaves of every resource.
     """,
-    'depends': ['base'],
+    'depends': ['base', 'web'],
     'data': [
         'data/resource_data.xml',
         'security/ir.model.access.csv',
         'security/resource_security.xml',
         'views/resource_views.xml',
-    ],
+        ],
     'demo': [
     ],
+    'assets': {
+        'web.assets_backend': [
+            'resource/static/**/*',
+        ],
+    },
+    'license': 'LGPL-3',
 }
