@@ -279,6 +279,20 @@ class TestCalendar(TestResourceCommon):
 
         leave.unlink()
 
+        # leave without calendar, should count for anyone in the company
+        leave = self.env['resource.calendar.leaves'].create({
+            'name': 'small leave',
+            'resource_id': False,
+            'date_from': datetime_str(2018, 4, 3, 9, 0, 0, tzinfo=self.patel.tz),
+            'date_to': datetime_str(2018, 4, 3, 12, 0, 0, tzinfo=self.patel.tz),
+        })
+
+        hours = self.calendar_patel.get_work_hours_count(
+            datetime_tz(2018, 4, 2, 0, 0, 0, tzinfo=self.patel.tz),
+            datetime_tz(2018, 4, 6, 23, 59, 59, tzinfo=self.patel.tz),
+        )
+        self.assertEqual(hours, 32)
+
     def test_calendar_working_hours_count(self):
         calendar = self.env.ref('resource.resource_calendar_std_35h')
         calendar.tz = 'UTC'
@@ -375,10 +389,162 @@ class TestCalendar(TestResourceCommon):
         time = self.calendar_jean.plan_days(3000, datetime_tz(2018, 4, 10, 0, 0, 0, tzinfo=self.jean.tz), compute_leaves=False)
         self.assertEqual(time, False)
 
+    def test_closest_time(self):
+        # Calendar:
+        # Tuesdays 8-16
+        # Fridays 8-13 and 16-23
+        dt = datetime_tz(2020, 4, 2, 7, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt)
+        self.assertFalse(calendar_dt, "It should not return any value for unattended days")
+
+        dt = datetime_tz(2020, 4, 3, 7, 0, 0, tzinfo=self.john.tz)
+        range_start = datetime_tz(2020, 4, 3, 8, 0, 0, tzinfo=self.john.tz)
+        range_end = datetime_tz(2020, 4, 3, 19, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt, search_range=(range_start, range_end))
+        self.assertFalse(calendar_dt, "It should not return any value if dt outside of range")
+
+        dt = datetime_tz(2020, 4, 3, 7, 0, 0, tzinfo=self.john.tz)  # before
+        start = datetime_tz(2020, 4, 3, 8, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt)
+        self.assertEqual(calendar_dt, start, "It should return the start of the day")
+
+        dt = datetime_tz(2020, 4, 3, 10, 0, 0, tzinfo=self.john.tz)  # after
+        start = datetime_tz(2020, 4, 3, 8, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt)
+        self.assertEqual(calendar_dt, start, "It should return the start of the closest attendance")
+
+        dt = datetime_tz(2020, 4, 3, 7, 0, 0, tzinfo=self.john.tz)  # before
+        end = datetime_tz(2020, 4, 3, 13, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt, match_end=True)
+        self.assertEqual(calendar_dt, end, "It should return the end of the closest attendance")
+
+        dt = datetime_tz(2020, 4, 3, 14, 0, 0, tzinfo=self.john.tz)  # after
+        end = datetime_tz(2020, 4, 3, 13, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt, match_end=True)
+        self.assertEqual(calendar_dt, end, "It should return the end of the closest attendance")
+
+        dt = datetime_tz(2020, 4, 3, 0, 0, 0, tzinfo=self.john.tz)
+        start = datetime_tz(2020, 4, 3, 8, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt)
+        self.assertEqual(calendar_dt, start, "It should return the start of the closest attendance")
+
+        dt = datetime_tz(2020, 4, 3, 23, 59, 59, tzinfo=self.john.tz)
+        end = datetime_tz(2020, 4, 3, 23, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt, match_end=True)
+        self.assertEqual(calendar_dt, end, "It should return the end of the closest attendance")
+
+        # with a resource specific attendance
+        self.env['resource.calendar.attendance'].create({
+            'name': 'Att4',
+            'calendar_id': self.calendar_john.id,
+            'dayofweek': '4',
+            'hour_from': 5,
+            'hour_to': 6,
+            'resource_id': self.john.resource_id.id,
+        })
+        dt = datetime_tz(2020, 4, 3, 5, 0, 0, tzinfo=self.john.tz)
+        start = datetime_tz(2020, 4, 3, 8, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt)
+        self.assertEqual(calendar_dt, start, "It should not take into account resouce specific attendances")
+
+        dt = datetime_tz(2020, 4, 3, 5, 0, 0, tzinfo=self.john.tz)
+        start = datetime_tz(2020, 4, 3, 5, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt, resource=self.john.resource_id)
+        self.assertEqual(calendar_dt, start, "It should have taken john's specific attendances")
+
+        dt = datetime_tz(2020, 4, 4, 1, 0, 0, tzinfo='UTC')  # The next day in UTC, but still the 3rd in john's timezone (America/Los_Angeles)
+        start = datetime_tz(2020, 4, 3, 16, 0, 0, tzinfo=self.john.tz)
+        calendar_dt = self.calendar_john._get_closest_work_time(dt, resource=self.john.resource_id)
+        self.assertEqual(calendar_dt, start, "It should have found the attendance on the 3rd April")
 
 class TestResMixin(TestResourceCommon):
-    def setUp(self):
-        super(TestResMixin, self).setUp()
+
+    def test_adjust_calendar(self):
+        # Calendar:
+        # Tuesdays 8-16
+        # Fridays 8-13 and 16-23
+        result = self.john._adjust_to_calendar(
+            datetime_tz(2020, 4, 3, 9, 0, 0, tzinfo=self.john.tz),
+            datetime_tz(2020, 4, 3, 14, 0, 0, tzinfo=self.john.tz),
+        )
+        self.assertEqual(result[self.john],(
+            datetime_tz(2020, 4, 3, 8, 0, 0, tzinfo=self.john.tz),
+            datetime_tz(2020, 4, 3, 13, 0, 0, tzinfo=self.john.tz),
+        ))
+
+        result = self.john._adjust_to_calendar(
+            datetime_tz(2020, 4, 3, 13, 1, 0, tzinfo=self.john.tz),
+            datetime_tz(2020, 4, 3, 14, 0, 0, tzinfo=self.john.tz),
+        )
+        self.assertEqual(result[self.john],(
+            datetime_tz(2020, 4, 3, 16, 0, 0, tzinfo=self.john.tz),
+            datetime_tz(2020, 4, 3, 23, 0, 0, tzinfo=self.john.tz),
+        ))
+
+        result = self.john._adjust_to_calendar(
+            datetime_tz(2020, 4, 4, 9, 0, 0, tzinfo=self.john.tz),  # both a day without attendance
+            datetime_tz(2020, 4, 4, 14, 0, 0, tzinfo=self.john.tz),
+        )
+        self.assertEqual(result[self.john], (None, None))
+
+        result = self.john._adjust_to_calendar(
+            datetime_tz(2020, 4, 3, 8, 0, 0, tzinfo=self.john.tz),
+            datetime_tz(2020, 4, 4, 14, 0, 0, tzinfo=self.john.tz),  # day without attendance
+        )
+        self.assertEqual(result[self.john], (
+            datetime_tz(2020, 4, 3, 8, 0, 0, tzinfo=self.john.tz),
+            None,
+        ))
+
+        result = self.john._adjust_to_calendar(
+            datetime_tz(2020, 4, 2, 8, 0, 0, tzinfo=self.john.tz),  # day without attendance
+            datetime_tz(2020, 4, 3, 14, 0, 0, tzinfo=self.john.tz),
+        )
+        self.assertEqual(result[self.john], (
+            None,
+            datetime_tz(2020, 4, 3, 13, 0, 0, tzinfo=self.john.tz),
+        ))
+
+        # It should find the start and end within the search range
+        result = self.paul._adjust_to_calendar(
+            datetime_tz(2020, 4, 2, 2, 0, 0, tzinfo='UTC'),
+            datetime_tz(2020, 4, 3, 1, 59, 59, tzinfo='UTC'),
+        )
+
+        self.assertEqual(result[self.paul], (
+            datetime_tz(2020, 4, 2, 4, 0, tzinfo='UTC'),
+            datetime_tz(2020, 4, 2, 18, 0, tzinfo='UTC')
+        ), "It should have found the start and end of the shift on the same day on April 2nd, 2020")
+
+    def test_adjust_calendar_timezone_before(self):
+        # Calendar:
+        # Every day 8-16
+        self.jean.tz = 'Japan'
+        self.calendar_jean.tz = 'Europe/Brussels'
+
+        result = self.jean._adjust_to_calendar(
+            datetime_tz(2020, 4, 1, 0, 0, 0, tzinfo='Japan'),
+            datetime_tz(2020, 4, 1, 23, 59, 59, tzinfo='Japan'),
+        )
+        self.assertEqual(result[self.jean], (
+            datetime_tz(2020, 4, 1, 8, 0, 0, tzinfo='Japan'),
+            datetime_tz(2020, 4, 1, 16, 0, 0, tzinfo='Japan'),
+        ), "It should have found a starting time the 1st")
+
+    def test_adjust_calendar_timezone_after(self):
+        # Calendar:
+        # Tuesdays 8-16
+        # Fridays 8-13 and 16-23
+        tz = 'Europe/Brussels'
+        self.john.tz = tz
+        result = self.john._adjust_to_calendar(
+            datetime(2020, 4, 2, 23, 0, 0),  # The previous day in UTC, but the 3rd in Europe/Brussels
+            datetime(2020, 4, 3, 20, 0, 0),
+        )
+        self.assertEqual(result[self.john], (
+            datetime(2020, 4, 3, 6, 0, 0),
+            datetime(2020, 4, 3, 21, 0, 0),
+        ), "It should have found a starting time the 3rd")
 
     def test_work_days_data(self):
         # Looking at Jean's calendar
