@@ -2,15 +2,16 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import analytic
-from . import account_invoice
-from . import account_reconciliation_widget
 from . import account_move
+from . import account_move_line
+from . import crm_team
+from . import payment_acquirer
+from . import payment_transaction
 from . import product_product
 from . import product_template
 from . import res_company
-from . import res_partner
-from . import sale
 from . import res_config_settings
-from . import sales_team
-from . import payment
-from . import utm
+from . import res_partner
+from . import sale_order
+from . import sale_order_line
+from . import utm_campaign
