@@ -6,8 +6,11 @@ from . import test_sale_to_invoice
 from . import test_sale_order
 from . import test_sale_pricelist
 from . import test_onchange
+from . import test_payment_flow
 from . import test_reinvoice
 from . import test_access_rights
 from . import test_sale_refund
 from . import test_sale_signature
-from . import test_sale_transaction
+from . import test_sale_flow
+from . import test_accrued_sale_orders
+from . import test_sale_tax_totals
