@@ -1,94 +1,86 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+from datetime import timedelta
+from freezegun import freeze_time
 
-import odoo
+from odoo import fields
 from odoo.exceptions import UserError, AccessError
-from odoo.tests import Form
+from odoo.tests import tagged, Form
 from odoo.tools import float_compare
 
-from .test_sale_common import TestCommonSaleNoChart
+from .common import TestSaleCommon
 
 
-class TestSaleOrder(TestCommonSaleNoChart):
+@tagged('post_install', '-at_install')
+class TestSaleOrder(TestSaleCommon):
 
     @classmethod
-    def setUpClass(cls):
-        super(TestSaleOrder, cls).setUpClass()
+    def setUpClass(cls, chart_template_ref=None):
+        super().setUpClass(chart_template_ref=chart_template_ref)
 
         SaleOrder = cls.env['sale.order'].with_context(tracking_disable=True)
 
         # set up users
-        cls.setUpUsers()
-        group_salemanager = cls.env.ref('sales_team.group_sale_manager')
-        group_salesman = cls.env.ref('sales_team.group_sale_salesman')
-        group_employee = cls.env.ref('base.group_user')
-        cls.user_manager.write({'groups_id': [(6, 0, [group_salemanager.id, group_employee.id])]})
-        cls.user_employee.write({'groups_id': [(6, 0, [group_salesman.id, group_employee.id])]})
         cls.crm_team0 = cls.env['crm.team'].create({
             'name': 'crm team 0',
-            'company_id': cls.env.user.company_id.id,
+            'company_id': cls.company_data['company'].id
         })
         cls.crm_team1 = cls.env['crm.team'].create({
             'name': 'crm team 1',
-            'company_id': cls.env.user.company_id.id,
+            'company_id': cls.company_data['company'].id
         })
-        cls.user_in_team = cls.env['res.users'].create({
+        cls.user_in_team = cls.env['res.users'].sudo().create({
             'email': 'team0user@example.com',
             'login': 'team0user',
             'name': 'User in Team 0',
-            'sale_team_id': cls.crm_team0.id
         })
-        cls.user_not_in_team = cls.env['res.users'].create({
+        cls.crm_team0.sudo().write({'member_ids': [4, cls.user_in_team.id]})
+        cls.user_not_in_team = cls.env['res.users'].sudo().create({
             'email': 'noteamuser@example.com',
             'login': 'noteamuser',
             'name': 'User Not In Team',
         })
 
-        # set up accounts and products and journals
-        cls.setUpAdditionalAccounts()
-        cls.setUpClassicProducts()
-        cls.setUpAccountJournal()
-
         # create a generic Sale Order with all classical products and empty pricelist
         cls.sale_order = SaleOrder.create({
-            'partner_id': cls.partner_customer_usd.id,
-            'partner_invoice_id': cls.partner_customer_usd.id,
-            'partner_shipping_id': cls.partner_customer_usd.id,
-            'pricelist_id': cls.pricelist_usd.id,
+            'partner_id': cls.partner_a.id,
+            'partner_invoice_id': cls.partner_a.id,
+            'partner_shipping_id': cls.partner_a.id,
+            'pricelist_id': cls.company_data['default_pricelist'].id,
         })
         cls.sol_product_order = cls.env['sale.order.line'].create({
-            'name': cls.product_order.name,
-            'product_id': cls.product_order.id,
+            'name': cls.company_data['product_order_no'].name,
+            'product_id': cls.company_data['product_order_no'].id,
             'product_uom_qty': 2,
-            'product_uom': cls.product_order.uom_id.id,
-            'price_unit': cls.product_order.list_price,
+            'product_uom': cls.company_data['product_order_no'].uom_id.id,
+            'price_unit': cls.company_data['product_order_no'].list_price,
             'order_id': cls.sale_order.id,
             'tax_id': False,
         })
         cls.sol_serv_deliver = cls.env['sale.order.line'].create({
-            'name': cls.service_deliver.name,
-            'product_id': cls.service_deliver.id,
+            'name': cls.company_data['product_service_delivery'].name,
+            'product_id': cls.company_data['product_service_delivery'].id,
             'product_uom_qty': 2,
-            'product_uom': cls.service_deliver.uom_id.id,
-            'price_unit': cls.service_deliver.list_price,
+            'product_uom': cls.company_data['product_service_delivery'].uom_id.id,
+            'price_unit': cls.company_data['product_service_delivery'].list_price,
             'order_id': cls.sale_order.id,
             'tax_id': False,
         })
         cls.sol_serv_order = cls.env['sale.order.line'].create({
-            'name': cls.service_order.name,
-            'product_id': cls.service_order.id,
+            'name': cls.company_data['product_service_order'].name,
+            'product_id': cls.company_data['product_service_order'].id,
             'product_uom_qty': 2,
-            'product_uom': cls.service_order.uom_id.id,
-            'price_unit': cls.service_order.list_price,
+            'product_uom': cls.company_data['product_service_order'].uom_id.id,
+            'price_unit': cls.company_data['product_service_order'].list_price,
             'order_id': cls.sale_order.id,
             'tax_id': False,
         })
         cls.sol_product_deliver = cls.env['sale.order.line'].create({
-            'name': cls.product_deliver.name,
-            'product_id': cls.product_deliver.id,
+            'name': cls.company_data['product_delivery_no'].name,
+            'product_id': cls.company_data['product_delivery_no'].id,
             'product_uom_qty': 2,
-            'product_uom': cls.product_deliver.uom_id.id,
-            'price_unit': cls.product_deliver.list_price,
+            'product_uom': cls.company_data['product_delivery_no'].uom_id.id,
+            'price_unit': cls.company_data['product_delivery_no'].list_price,
             'order_id': cls.sale_order.id,
             'tax_id': False,
         })
@@ -97,10 +89,10 @@ class TestSaleOrder(TestCommonSaleNoChart):
         """ Test the sales order flow (invoicing and quantity updates)
             - Invoice repeatedly while varrying delivered quantities and check that invoice are always what we expect
         """
-        # DBO TODO: validate invoice and register payments
+        # TODO?: validate invoice and register payments
         self.sale_order.order_line.read(['name', 'price_unit', 'product_uom_qty', 'price_total'])
 
-        self.assertEqual(self.sale_order.amount_total, sum([2 * p.list_price for p in self.product_map.values()]), 'Sale: total amount is wrong')
+        self.assertEqual(self.sale_order.amount_total, 1240.0, 'Sale: total amount is wrong')
         self.sale_order.order_line._compute_product_updatable()
         self.assertTrue(self.sale_order.order_line[0].product_updatable)
         # send quotation
@@ -119,7 +111,7 @@ class TestSaleOrder(TestCommonSaleNoChart):
         # create invoice: only 'invoice on order' products are invoiced
         invoice = self.sale_order._create_invoices()
         self.assertEqual(len(invoice.invoice_line_ids), 2, 'Sale: invoice is missing lines')
-        self.assertEqual(invoice.amount_total, sum([2 * p.list_price if p.invoice_policy == 'order' else 0 for p in self.product_map.values()]), 'Sale: invoice total amount is wrong')
+        self.assertEqual(invoice.amount_total, 740.0, 'Sale: invoice total amount is wrong')
         self.assertTrue(self.sale_order.invoice_status == 'no', 'Sale: SO status after invoicing should be "nothing to invoice"')
         self.assertTrue(len(self.sale_order.invoice_ids) == 1, 'Sale: invoice is missing')
         self.sale_order.order_line._compute_product_updatable()
@@ -131,7 +123,7 @@ class TestSaleOrder(TestCommonSaleNoChart):
         self.assertTrue(self.sale_order.invoice_status == 'to invoice', 'Sale: SO status after delivery should be "to invoice"')
         invoice2 = self.sale_order._create_invoices()
         self.assertEqual(len(invoice2.invoice_line_ids), 2, 'Sale: second invoice is missing lines')
-        self.assertEqual(invoice2.amount_total, sum([2 * p.list_price if p.invoice_policy == 'delivery' else 0 for p in self.product_map.values()]), 'Sale: second invoice total amount is wrong')
+        self.assertEqual(invoice2.amount_total, 500.0, 'Sale: second invoice total amount is wrong')
         self.assertTrue(self.sale_order.invoice_status == 'invoiced', 'Sale: SO status after invoicing everything should be "invoiced"')
         self.assertTrue(len(self.sale_order.invoice_ids) == 2, 'Sale: invoice is missing')
 
@@ -156,9 +148,34 @@ class TestSaleOrder(TestCommonSaleNoChart):
 
         invoice3 = self.sale_order._create_invoices()
         self.assertEqual(len(invoice3.invoice_line_ids), 1, 'Sale: third invoice is missing lines')
-        self.assertEqual(invoice3.amount_total, 8 * self.product_map['serv_order'].list_price, 'Sale: second invoice total amount is wrong')
+        self.assertEqual(invoice3.amount_total, 720.0, 'Sale: second invoice total amount is wrong')
         self.assertTrue(self.sale_order.invoice_status == 'invoiced', 'Sale: SO status after invoicing everything (including the upsel) should be "invoiced"')
 
+    def test_sale_order_send_to_self(self):
+        # when sender(logged in user) is also present in recipients of the mail composer,
+        # user should receive mail.
+        sale_order = self.env['sale.order'].with_user(self.company_data['default_user_salesman']).create({
+            'partner_id': self.company_data['default_user_salesman'].partner_id.id,
+            'order_line': [[0, 0, {
+                'name':  self.company_data['product_order_no'].name,
+                'product_id': self.company_data['product_order_no'].id,
+                'product_uom_qty': 1,
+                'price_unit': self.company_data['product_order_no'].list_price,
+            }]]
+        })
+        email_ctx = sale_order.action_quotation_send().get('context', {})
+        # We need to prevent auto mail deletion, and so we copy the template and send the mail with
+        # added configuration in copied template. It will allow us to check whether mail is being
+        # sent to to author or not (in case author is present in 'Recipients' of composer).
+        mail_template = self.env['mail.template'].browse(email_ctx.get('default_template_id')).copy({'auto_delete': False})
+        # send the mail with same user as customer
+        sale_order.with_context(**email_ctx).with_user(self.company_data['default_user_salesman']).message_post_with_template(mail_template.id)
+        self.assertTrue(sale_order.state == 'sent', 'Sale : state should be changed to sent')
+        mail_message = sale_order.message_ids[0]
+        self.assertEqual(mail_message.author_id, sale_order.partner_id, 'Sale: author should be same as customer')
+        self.assertEqual(mail_message.author_id, mail_message.partner_ids, 'Sale: author should be in composer recipients thanks to "partner_to" field set on template')
+        self.assertEqual(mail_message.partner_ids, mail_message.sudo().mail_ids.recipient_ids, 'Sale: author should receive mail due to presence in composer recipients')
+
     def test_sale_sequence(self):
         self.env['ir.sequence'].search([
             ('code', '=', 'sale.order'),
@@ -178,8 +195,8 @@ class TestSaleOrder(TestCommonSaleNoChart):
         # SO in state 'draft' can be deleted
         so_copy = self.sale_order.copy()
         with self.assertRaises(AccessError):
-            so_copy.with_user(self.user_employee).unlink()
-        self.assertTrue(so_copy.with_user(self.user_manager).unlink(), 'Sale: deleting a quotation should be possible')
+            so_copy.with_user(self.company_data['default_user_employee']).unlink()
+        self.assertTrue(so_copy.unlink(), 'Sale: deleting a quotation should be possible')
 
         # SO in state 'cancel' can be deleted
         so_copy = self.sale_order.copy()
@@ -188,25 +205,22 @@ class TestSaleOrder(TestCommonSaleNoChart):
         so_copy.action_cancel()
         self.assertTrue(so_copy.state == 'cancel', 'Sale: SO should be in state "cancel"')
         with self.assertRaises(AccessError):
-            so_copy.with_user(self.user_employee).unlink()
-        self.assertTrue(so_copy.with_user(self.user_manager).unlink(), 'Sale: deleting a cancelled SO should be possible')
+            so_copy.with_user(self.company_data['default_user_employee']).unlink()
+        self.assertTrue(so_copy.unlink(), 'Sale: deleting a cancelled SO should be possible')
 
         # SO in state 'sale' or 'done' cannot be deleted
         self.sale_order.action_confirm()
         self.assertTrue(self.sale_order.state == 'sale', 'Sale: SO should be in state "sale"')
         with self.assertRaises(UserError):
-            self.sale_order.with_user(self.user_manager).unlink()
+            self.sale_order.unlink()
 
         self.sale_order.action_done()
         self.assertTrue(self.sale_order.state == 'done', 'Sale: SO should be in state "done"')
         with self.assertRaises(UserError):
-            self.sale_order.with_user(self.user_manager).unlink()
+            self.sale_order.unlink()
 
     def test_cost_invoicing(self):
         """ Test confirming a vendor invoice to reinvoice cost on the so """
-        # force the pricelist to have the same currency as the company
-        self.pricelist_usd.currency_id = self.env.ref('base.main_company').currency_id
-
         serv_cost = self.env['product.product'].create({
             'name': "Ordered at cost",
             'standard_price': 160,
@@ -217,19 +231,20 @@ class TestSaleOrder(TestCommonSaleNoChart):
             'default_code': 'PROD_COST',
             'service_type': 'manual',
         })
-        prod_gap = self.service_order
+        prod_gap = self.company_data['product_service_order']
         so = self.env['sale.order'].create({
-            'partner_id': self.partner_customer_usd.id,
-            'partner_invoice_id': self.partner_customer_usd.id,
-            'partner_shipping_id': self.partner_customer_usd.id,
+            'partner_id': self.partner_a.id,
+            'partner_invoice_id': self.partner_a.id,
+            'partner_shipping_id': self.partner_a.id,
             'order_line': [(0, 0, {'name': prod_gap.name, 'product_id': prod_gap.id, 'product_uom_qty': 2, 'product_uom': prod_gap.uom_id.id, 'price_unit': prod_gap.list_price})],
-            'pricelist_id': self.pricelist_usd.id,
+            'pricelist_id': self.company_data['default_pricelist'].id,
         })
         so.action_confirm()
         so._create_analytic_account()
 
-        inv = self.env['account.move'].with_context(default_type='in_invoice').create({
-            'partner_id': self.partner_customer_usd.id,
+        inv = self.env['account.move'].with_context(default_move_type='in_invoice').create({
+            'partner_id': self.partner_a.id,
+            'invoice_date': so.date_order,
             'invoice_line_ids': [
                 (0, 0, {
                     'name': serv_cost.name,
@@ -241,10 +256,10 @@ class TestSaleOrder(TestCommonSaleNoChart):
                 }),
             ],
         })
-        inv.post()
+        inv.action_post()
         sol = so.order_line.filtered(lambda l: l.product_id == serv_cost)
         self.assertTrue(sol, 'Sale: cost invoicing does not add lines when confirming vendor invoice')
-        self.assertEquals((sol.price_unit, sol.qty_delivered, sol.product_uom_qty, sol.qty_invoiced), (160, 2, 0, 0), 'Sale: line is wrong after confirming vendor invoice')
+        self.assertEqual((sol.price_unit, sol.qty_delivered, sol.product_uom_qty, sol.qty_invoiced), (160, 2, 0, 0), 'Sale: line is wrong after confirming vendor invoice')
 
     def test_sale_with_taxes(self):
         """ Test SO with taxes applied on its lines and check subtotal applied on its lines and total applied on the SO """
@@ -277,113 +292,42 @@ class TestSaleOrder(TestCommonSaleNoChart):
             else:
                 price = line.price_unit * line.product_uom_qty
 
-            self.assertEquals(float_compare(line.price_subtotal, price, precision_digits=2), 0)
+            self.assertEqual(float_compare(line.price_subtotal, price, precision_digits=2), 0)
 
-        self.assertEquals(self.sale_order.amount_total,
+        self.assertEqual(self.sale_order.amount_total,
                           self.sale_order.amount_untaxed + self.sale_order.amount_tax,
                           'Taxes should be applied')
 
     def test_so_create_multicompany(self):
         """Check that only taxes of the right company are applied on the lines."""
-        user_demo = self.env.ref('base.user_demo')
-        company_1 = self.env.ref('base.main_company')
-        company_2 = self.env['res.company'].create({
-            'name': 'company 2',
-            'parent_id': company_1.id,
-        })
-        user_demo.company_ids = (company_1 | company_2).ids
-
-        tax_company_1 = self.env['account.tax'].create({
-            'name': 'T1',
-            'amount': 90,
-            'company_id': company_1.id,
-        })
-
-        tax_company_2 = self.env['account.tax'].create({
-            'name': 'T2',
-            'amount': 90,
-            'company_id': company_2.id,
-        })
 
+        # Preparing test Data
         product_shared = self.env['product.template'].create({
             'name': 'shared product',
-            'taxes_id': [(6, False, [tax_company_1.id, tax_company_2.id])],
-            'property_account_income_id': self.account_receivable.id,
+            'invoice_policy': 'order',
+            'taxes_id': [(6, False, (self.company_data['default_tax_sale'] + self.company_data_2['default_tax_sale']).ids)],
+            'property_account_income_id': self.company_data['default_account_revenue'].id,
         })
 
-        so_1 = self.env['sale.order'].with_user(user_demo.id).create({
-            'partner_id': self.env.ref('base.res_partner_2').id,
-            'company_id': company_1.id,
+        so_1 = self.env['sale.order'].with_user(self.company_data['default_user_salesman']).create({
+            'partner_id': self.env['res.partner'].create({'name': 'A partner'}).id,
+            'company_id': self.company_data['company'].id,
         })
         so_1.write({
             'order_line': [(0, False, {'product_id': product_shared.product_variant_id.id, 'order_id': so_1.id})],
         })
 
-        self.assertEqual(set(so_1.order_line.tax_id.ids), set([tax_company_1.id]),
+        self.assertEqual(so_1.order_line.tax_id, self.company_data['default_tax_sale'],
             'Only taxes from the right company are put by default')
         so_1.action_confirm()
         # i'm not interested in groups/acls, but in the multi-company flow only
         # the sudo is there for that and does not impact the invoice that gets created
         # the goal here is to invoice in company 1 (because the order is in company 1) while being
         # 'mainly' in company 2 (through the context), the invoice should be in company 1
-        inv=so_1.sudo().with_context(allowed_company_ids=[company_2.id, company_1.id])._create_invoices()
-        self.assertEqual(inv.company_id, company_1, 'invoices should be created in the company of the SO, not the main company of the context')
-
-
-    def test_reconciliation_with_so(self):
-        # create SO
-        so = self.env['sale.order'].create({
-            'name': 'SO/01/01',
-            'reference': 'Petit suisse',
-            'partner_id': self.partner_customer_usd.id,
-            'partner_invoice_id': self.partner_customer_usd.id,
-            'partner_shipping_id': self.partner_customer_usd.id,
-            'pricelist_id': self.pricelist_usd.id,
-        })
-        self.env['sale.order.line'].create({
-            'name': self.product_order.name,
-            'product_id': self.product_order.id,
-            'product_uom_qty': 2,
-            'product_uom': self.product_order.uom_id.id,
-            'price_unit': self.product_order.list_price,
-            'order_id': so.id,
-            'tax_id': False,
-        })
-        # Mark SO as sent otherwise we won't find any match
-        so.write({'state': 'sent'})
-        # Create bank statement
-        statement = self.env['account.bank.statement'].create({
-            'name': 'Test',
-            'journal_id': self.journal_purchase.id,
-            'user_id': self.user_employee.id,
-        })
-        st_line1 = self.env['account.bank.statement.line'].create({
-            'name': 'should not find anything',
-            'amount': 15,
-            'statement_id': statement.id
-        })
-        st_line2 = self.env['account.bank.statement.line'].create({
-            'name': 'Payment for SO/01/01',
-            'amount': 15,
-            'statement_id': statement.id
-        })
-        st_line3 = self.env['account.bank.statement.line'].create({
-            'name': 'Payment for Petit suisse',
-            'amount': 15,
-            'statement_id': statement.id
-        })
-        # Call get_bank_statement_line_data for st_line_1, should not find any sale order
-        res = self.env['account.reconciliation.widget'].get_bank_statement_line_data([st_line1.id])
-        line = res.get('lines', [{}])[0]
-        self.assertFalse(line.get('sale_order_ids', False))
-        # Call again for st_line_2, it should find sale_order
-        res = self.env['account.reconciliation.widget'].get_bank_statement_line_data([st_line2.id])
-        line = res.get('lines', [{}])[0]
-        self.assertEquals(line.get('sale_order_ids', []), [so.id])
-        # Call again for st_line_3, it should find sale_order based on reference
-        res = self.env['account.reconciliation.widget'].get_bank_statement_line_data([st_line3.id])
-        line = res.get('lines', [{}])[0]
-        self.assertEquals(line.get('sale_order_ids', []), [so.id])
+        inv=so_1.sudo()\
+            .with_context(allowed_company_ids=(self.company_data['company'] + self.company_data_2['company']).ids)\
+            ._create_invoices()
+        self.assertEqual(inv.company_id, self.company_data['company'], 'invoices should be created in the company of the SO, not the main company of the context')
 
     def test_group_invoice(self):
         """ Test that invoicing multiple sales order for the same customer works. """
@@ -403,6 +347,29 @@ class TestSaleOrder(TestCommonSaleNoChart):
         # Check that exactly 2 invoices are generated
         self.assertEqual(len(res['domain'][0][2]),2, "Grouping invoicing 3 orders for the same partner with 2 currencies should create exactly 2 invoices")
 
+    def test_so_note_to_invoice(self):
+        """Test that notes from SO are pushed into invoices"""
+
+        sol_note = self.env['sale.order.line'].create({
+            'name': 'This is a note',
+            'display_type': 'line_note',
+            'product_id': False,
+            'product_uom_qty': 0,
+            'product_uom': False,
+            'price_unit': 0,
+            'order_id': self.sale_order.id,
+            'tax_id': False,
+        })
+
+        # confirm quotation
+        self.sale_order.action_confirm()
+
+        # create invoice
+        invoice = self.sale_order._create_invoices()
+
+        # check note from SO has been pushed in invoice
+        self.assertEqual(len(invoice.invoice_line_ids.filtered(lambda line: line.display_type == 'line_note')), 1, 'Note SO line should have been pushed to the invoice')
+
     def test_multi_currency_discount(self):
         """Verify the currency used for pricelist price & discount computation."""
         products = self.env["product.product"].search([], limit=2)
@@ -412,7 +379,8 @@ class TestSaleOrder(TestCommonSaleNoChart):
         # Make sure the company is in USD
         main_company = self.env.ref('base.main_company')
         main_curr = main_company.currency_id
-        other_curr = (self.env.ref('base.USD') + self.env.ref('base.EUR')) - main_curr
+        current_curr = self.env.company.currency_id
+        other_curr = self.currency_data['currency']
         # main_company.currency_id = other_curr # product.currency_id when no company_id set
         other_company = self.env["res.company"].create({
             "name": "Test",
@@ -438,11 +406,11 @@ class TestSaleOrder(TestCommonSaleNoChart):
 
         self.assertEqual(product_1.currency_id, main_curr)
         self.assertEqual(product_2.currency_id, main_curr)
-        self.assertEqual(product_1.cost_currency_id, main_curr)
-        self.assertEqual(product_2.cost_currency_id, main_curr)
+        self.assertEqual(product_1.cost_currency_id, current_curr)
+        self.assertEqual(product_2.cost_currency_id, current_curr)
 
-        product_1_ctxt = product_1.with_env(self.env(user=user_in_other_company.id))
-        product_2_ctxt = product_2.with_env(self.env(user=user_in_other_company.id))
+        product_1_ctxt = product_1.with_user(user_in_other_company)
+        product_2_ctxt = product_2.with_user(user_in_other_company)
         self.assertEqual(product_1_ctxt.currency_id, main_curr)
         self.assertEqual(product_2_ctxt.currency_id, main_curr)
         self.assertEqual(product_1_ctxt.cost_currency_id, other_curr)
@@ -564,7 +532,7 @@ class TestSaleOrder(TestCommonSaleNoChart):
         """When we assign a sales person, change the team on the sales order to their team"""
         sale_order = self.env['sale.order'].create({
             'user_id': self.user_not_in_team.id,
-            'partner_id': self.partner_customer_usd.id,
+            'partner_id': self.partner_a.id,
             'team_id': self.crm_team1.id
         })
         sale_order.user_id = self.user_in_team
@@ -574,20 +542,172 @@ class TestSaleOrder(TestCommonSaleNoChart):
     def test_keep_sales_team_when_changing_user_with_no_team(self):
         """When we assign a sales person that has no team, do not reset the team to default"""
         sale_order = self.env['sale.order'].create({
-            'partner_id': self.partner_customer_usd.id,
+            'partner_id': self.partner_a.id,
             'team_id': self.crm_team1.id
         })
         sale_order.user_id = self.user_not_in_team
         sale_order.onchange_user_id()
         self.assertEqual(sale_order.team_id.id, self.crm_team1.id, 'Should not reset the team to default')
 
+    def test_onchange_packaging_00(self):
+        """Create a SO and use packaging. Check we suggested suitable packaging
+        according to the product_qty. Also check product_qty or product_packaging
+        are correctly calculated when one of them changed.
+        """
+        partner = self.env['res.partner'].create({'name': "I'm a partner"})
+        product_tmpl = self.env['product.template'].create({'name': "I'm a product"})
+        product = product_tmpl.product_variant_id
+        packaging_single = self.env['product.packaging'].create({
+            'name': "I'm a packaging",
+            'product_id': product.id,
+            'qty': 1.0,
+        })
+        packaging_dozen = self.env['product.packaging'].create({
+            'name': "I'm also a packaging",
+            'product_id': product.id,
+            'qty': 12.0,
+        })
+
+        so = self.env['sale.order'].create({
+            'partner_id': partner.id,
+        })
+        so_form = Form(so)
+        with so_form.order_line.new() as line:
+            line.product_id = product
+            line.product_uom_qty = 1.0
+        so_form.save()
+        self.assertEqual(so.order_line.product_packaging_id, packaging_single)
+        self.assertEqual(so.order_line.product_packaging_qty, 1.0)
+        with so_form.order_line.edit(0) as line:
+            line.product_packaging_qty = 2.0
+        so_form.save()
+        self.assertEqual(so.order_line.product_uom_qty, 2.0)
+
+
+        with so_form.order_line.edit(0) as line:
+            line.product_uom_qty = 24.0
+        so_form.save()
+        self.assertEqual(so.order_line.product_packaging_id, packaging_dozen)
+        self.assertEqual(so.order_line.product_packaging_qty, 2.0)
+        with so_form.order_line.edit(0) as line:
+            line.product_packaging_qty = 1.0
+        so_form.save()
+        self.assertEqual(so.order_line.product_uom_qty, 12)
+
+    def _create_sale_order(self):
+        """Create dummy sale order (without lines)"""
+        return self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'partner_invoice_id': self.partner_a.id,
+            'partner_shipping_id': self.partner_a.id,
+            'pricelist_id': self.company_data['default_pricelist'].id,
+        })
+
+    def test_invoicing_terms(self):
+        # Enable invoicing terms
+        self.env['ir.config_parameter'].sudo().set_param('account.use_invoice_terms', True)
+
+        # Plain invoice terms
+        self.env.company.terms_type = 'plain'
+        self.env.company.invoice_terms = "Coin coin"
+        sale_order = self._create_sale_order()
+        self.assertEqual(sale_order.note, "<p>Coin coin</p>")
+
+        # Html invoice terms (/terms page)
+        self.env.company.terms_type = 'html'
+        sale_order = self._create_sale_order()
+        self.assertTrue(sale_order.note.startswith("<p>Terms &amp; Conditions: "))
+
+    def test_validity_days(self):
+        self.env['ir.config_parameter'].sudo().set_param('sale.use_quotation_validity_days', True)
+        self.env.company.quotation_validity_days = 5
+        with freeze_time("2020-05-02"):
+            sale_order = self._create_sale_order()
+
+            self.assertEqual(sale_order.validity_date, fields.Date.today() + timedelta(days=5))
+        self.env.company.quotation_validity_days = 0
+        sale_order = self._create_sale_order()
+        self.assertFalse(
+            sale_order.validity_date,
+            "No validity date must be specified if the company validity duration is 0")
+
+    def test_update_prices(self):
+        """Test prices recomputation on SO's.
+
+        `update_prices` is shown as a button to update
+        prices when the pricelist was changed.
+        """
+        self.env.user.write({'groups_id': [(4, self.env.ref('product.group_discount_per_so_line').id)]})
+
+        sale_order = self.sale_order
+        so_amount = sale_order.amount_total
+        sale_order.update_prices()
+        self.assertEqual(
+            sale_order.amount_total, so_amount,
+            "Updating the prices of an unmodified SO shouldn't modify the amounts")
+
+        pricelist = sale_order.pricelist_id
+        pricelist.item_ids = [
+            fields.Command.create({
+                'percent_price': 5.0,
+                'compute_price': 'percentage'
+            })
+        ]
+        pricelist.discount_policy = "without_discount"
+        self.env['product.product'].invalidate_cache(['price'])
+        sale_order.update_prices()
+
+        self.assertTrue(all(line.discount == 5 for line in sale_order.order_line))
+        self.assertEqual(sale_order.amount_undiscounted, so_amount)
+        self.assertEqual(sale_order.amount_total, 0.95*so_amount)
+
+        pricelist.discount_policy = "with_discount"
+        self.env['product.product'].invalidate_cache(['price'])
+        sale_order.update_prices()
+
+        self.assertTrue(all(line.discount == 0 for line in sale_order.order_line))
+        self.assertEqual(sale_order.amount_undiscounted, so_amount)
+        self.assertEqual(sale_order.amount_total, 0.95*so_amount)
+
+    def test_so_names(self):
+        """Test custom context key for name_get & name_search.
+
+        Note: this key is used in sale_expense & sale_timesheet modules.
+        """
+        SaleOrder = self.env['sale.order'].with_context(sale_show_partner_name=True)
+
+        res = SaleOrder.name_search(name=self.sale_order.partner_id.name)
+        self.assertEqual(res[0][0], self.sale_order.id)
+
+        self.assertNotIn(self.sale_order.partner_id.name, self.sale_order.display_name)
+        self.assertIn(
+            self.sale_order.partner_id.name,
+            self.sale_order.with_context(sale_show_partner_name=True).name_get()[0][1])
+
+    def test_state_changes(self):
+        """Test some untested state changes methods & logic."""
+        self.sale_order.action_quotation_sent()
+
+        self.assertEqual(self.sale_order.state, 'sent')
+        self.assertIn(self.sale_order.partner_id, self.sale_order.message_follower_ids.partner_id)
+
+        self.env.user.groups_id += self.env.ref('sale.group_auto_done_setting')
+        self.sale_order.action_confirm()
+        self.assertEqual(self.sale_order.state, 'done', "The order wasn't automatically locked at confirmation.")
+        with self.assertRaises(UserError):
+            self.sale_order.action_confirm()
+
+        self.sale_order.action_unlock()
+        self.assertEqual(self.sale_order.state, 'sale')
+
     def test_discount_and_untaxed_subtotal(self):
         """When adding a discount on a SO line, this test ensures that the untaxed amount to invoice is
         equal to the untaxed subtotal"""
+        self.product_a.invoice_policy = 'delivery'
         sale_order = self.env['sale.order'].create({
-            'partner_id': self.partner_customer_usd.id,
+            'partner_id': self.partner_a.id,
             'order_line': [(0, 0, {
-                'product_id': self.product_deliver.id,
+                'product_id': self.product_a.id,
                 'product_uom_qty': 38,
                 'price_unit': 541.26,
                 'discount': 2.00,
@@ -618,3 +738,104 @@ class TestSaleOrder(TestCommonSaleNoChart):
         # (541,26 / 1,15) * ,98 * 38 = 17527,410782609 ~= 17527.41
         self.assertEqual(line.price_subtotal, 17527.41)
         self.assertEqual(line.untaxed_amount_to_invoice, line.price_subtotal)
+
+    def test_discount_and_amount_undiscounted(self):
+        """When adding a discount on a SO line, this test ensures that amount undiscounted is
+        consistent with the used tax"""
+        sale_order = self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'order_line': [(0, 0, {
+                'product_id': self.product_a.id,
+                'product_uom_qty': 1,
+                'price_unit': 100.0,
+                'discount': 1.00,
+            })]
+        })
+        sale_order.action_confirm()
+        line = sale_order.order_line
+
+        # test discount and qty 1
+        self.assertEqual(sale_order.amount_undiscounted, 100.0)
+        self.assertEqual(line.price_subtotal, 99.0)
+
+        # more quantity 1 -> 3
+        sale_form = Form(sale_order)
+        with sale_form.order_line.edit(0) as line_form:
+            line_form.product_uom_qty = 3.0
+            line_form.price_unit = 100.0
+        sale_order = sale_form.save()
+
+        self.assertEqual(sale_order.amount_undiscounted, 300.0)
+        self.assertEqual(line.price_subtotal, 297.0)
+
+        # undiscounted
+        with sale_form.order_line.edit(0) as line_form:
+            line_form.discount = 0.0
+        sale_order = sale_form.save()
+        self.assertEqual(line.price_subtotal, 300.0)
+        self.assertEqual(sale_order.amount_undiscounted, 300.0)
+
+        # Same with an included-in-price tax
+        sale_order = sale_order.copy()
+        line = sale_order.order_line
+        line.tax_id = [(0, 0, {
+            'name': 'Super Tax',
+            'amount_type': 'percent',
+            'amount': 10.0,
+            'price_include': True,
+        })]
+        line.discount = 50.0
+        sale_order.action_confirm()
+
+        # 300 with 10% incl tax -> 272.72 total tax excluded without discount
+        # 136.36 price tax excluded with discount applied
+        self.assertEqual(sale_order.amount_undiscounted, 272.72)
+        self.assertEqual(line.price_subtotal, 136.36)
+
+    def test_free_product_and_price_include_fixed_tax(self):
+        """ Check that fixed tax include are correctly computed while the price_unit is 0
+        """
+        # please ensure this test remains consistent with
+        # test_out_invoice_line_onchange_2_taxes_fixed_price_include_free_product in account module
+        sale_order = self.env['sale.order'].create({
+            'partner_id': self.partner_a.id,
+            'order_line': [(0, 0, {
+                'product_id': self.company_data['product_order_no'].id,
+                'product_uom_qty': 1,
+                'price_unit': 0.0,
+            })]
+        })
+        sale_order.action_confirm()
+        line = sale_order.order_line
+        line.tax_id = [
+            (0, 0, {
+                'name': 'BEBAT 0.05',
+                'type_tax_use': 'sale',
+                'amount_type': 'fixed',
+                'amount': 0.05,
+                'price_include': True,
+                'include_base_amount': True,
+            }),
+            (0, 0, {
+                'name': 'Recupel 0.25',
+                'type_tax_use': 'sale',
+                'amount_type': 'fixed',
+                'amount': 0.25,
+                'price_include': True,
+                'include_base_amount': True,
+            }),
+        ]
+        sale_order.action_confirm()
+        self.assertRecordValues(sale_order, [{
+            'amount_untaxed': -0.30,
+            'amount_tax': 0.30,
+            'amount_total': 0.0,
+        }])
+
+    def test_sol_name_search(self):
+        # Shouldn't raise
+        self.env['sale.order']._search([('order_line', 'ilike', 'acoustic')])
+
+        name_search_data = self.env['sale.order.line'].name_search(name=self.sale_order.name)
+        sol_ids_found = dict(name_search_data).keys()
+        self.assertEqual(list(sol_ids_found), self.sale_order.order_line.ids)
