@@ -2,6 +2,8 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import test_sale_mrp_flow
+from . import test_sale_mrp_kit_bom
 from . import test_sale_mrp_lead_time
 from . import test_sale_mrp_procurement
-from . import test_multistep_manufacturing
\ No newline at end of file
+from . import test_multistep_manufacturing
+from . import test_sale_mrp_report
