@@ -10,6 +10,7 @@ from odoo.tools import mute_logger
 class TestSaleMrpProcurement(TransactionCase):
 
     def test_sale_mrp(self):
+        self.env.ref('stock.route_warehouse0_mto').active = True
         warehouse0 = self.env.ref('stock.warehouse0')
         # In order to test the sale_mrp module in OpenERP, I start by creating a new product 'Slider Mobile'
         # I define product category Mobile Products Sellable.
@@ -31,7 +32,7 @@ class TestSaleMrpProcurement(TransactionCase):
         product.categ_id = product_category_allproductssellable0
         product.list_price = 200.0
         product.name = 'Slider Mobile'
-        product.type = 'product'
+        product.detailed_type = 'product'
         product.uom_id = uom_unit
         product.uom_po_id = uom_unit
         product.route_ids.clear()
@@ -39,9 +40,7 @@ class TestSaleMrpProcurement(TransactionCase):
         product.route_ids.add(warehouse0.mto_pull_id.route_id)
         product_template_slidermobile0 = product.save()
 
-        std_price_wiz = Form(self.env['stock.change.standard.price'].with_context(active_id=product_template_slidermobile0.id, active_model='product.template'))
-        std_price_wiz.new_price = 189
-        std_price_wiz.save()
+        product_template_slidermobile0.standard_price = 189
 
         product_component = Form(self.env['product.product'])
         product_component.name = 'Battery'
@@ -55,7 +54,7 @@ class TestSaleMrpProcurement(TransactionCase):
 
         # I create a sale order for product Slider mobile
         so_form = Form(self.env['sale.order'])
-        so_form.partner_id = self.env.ref('base.res_partner_4')
+        so_form.partner_id = self.env['res.partner'].create({'name': 'Another Test Partner'})
         with so_form.order_line.new() as line:
             line.product_id = product_template_slidermobile0.product_variant_ids
             line.price_unit = 200
@@ -70,25 +69,28 @@ class TestSaleMrpProcurement(TransactionCase):
         mo = self.env['mrp.production'].search([('origin', 'like', sale_order_so0.name)], limit=1)
         self.assertTrue(mo, 'Manufacturing order has not been generated')
 
+        # Check the mo is displayed on the so
+        self.assertEqual(mo.id, sale_order_so0.action_view_mrp_production()['res_id'])
+
     def test_sale_mrp_pickings(self):
         """ Test sale of multiple mrp products in MTO
         to avoid generating multiple deliveries
         to the customer location
         """
-
+        self.env.ref('stock.route_warehouse0_mto').active = True
         # Create warehouse
-        self.customer_location = self.env['ir.model.data'].xmlid_to_res_id('stock.stock_location_customers')
-        warehouse_form = Form(self.env['stock.warehouse'])
-        warehouse_form.name = 'Test Warehouse'
-        warehouse_form.code = 'TWH'
-        self.warehouse = warehouse_form.save()
+        self.customer_location = self.env['ir.model.data']._xmlid_to_res_id('stock.stock_location_customers')
+        self.warehouse = self.env['stock.warehouse'].create({
+            'name': 'Test Warehouse',
+            'code': 'TWH'
+        })
 
         self.uom_unit = self.env.ref('uom.product_uom_unit')
 
         # Create raw product for manufactured product
         product_form = Form(self.env['product.product'])
         product_form.name = 'Raw Stick'
-        product_form.type = 'product'
+        product_form.detailed_type = 'product'
         product_form.uom_id = self.uom_unit
         product_form.uom_po_id = self.uom_unit
         self.raw_product = product_form.save()
@@ -98,7 +100,7 @@ class TestSaleMrpProcurement(TransactionCase):
         product_form.name = 'Stick'
         product_form.uom_id = self.uom_unit
         product_form.uom_po_id = self.uom_unit
-        product_form.type = 'product'
+        product_form.detailed_type = 'product'
         product_form.route_ids.clear()
         product_form.route_ids.add(self.warehouse.manufacture_pull_id.route_id)
         product_form.route_ids.add(self.warehouse.mto_pull_id.route_id)
@@ -107,7 +109,7 @@ class TestSaleMrpProcurement(TransactionCase):
         # Create manifactured product which uses another manifactured
         product_form = Form(self.env['product.product'])
         product_form.name = 'Arrow'
-        product_form.type = 'product'
+        product_form.detailed_type = 'product'
         product_form.route_ids.clear()
         product_form.route_ids.add(self.warehouse.manufacture_pull_id.route_id)
         product_form.route_ids.add(self.warehouse.mto_pull_id.route_id)
@@ -116,7 +118,7 @@ class TestSaleMrpProcurement(TransactionCase):
         ## Create raw product for manufactured product
         product_form = Form(self.env['product.product'])
         product_form.name = 'Raw Iron'
-        product_form.type = 'product'
+        product_form.detailed_type = 'product'
         product_form.uom_id = self.uom_unit
         product_form.uom_po_id = self.uom_unit
         self.raw_product_2 = product_form.save()
@@ -150,7 +152,7 @@ class TestSaleMrpProcurement(TransactionCase):
             warehouse.manufacture_steps = 'pbm_sam'
 
         so_form = Form(self.env['sale.order'])
-        so_form.partner_id = self.env.ref('base.res_partner_4')
+        so_form.partner_id = self.env['res.partner'].create({'name': 'Another Test Partner'})
         with so_form.order_line.new() as line:
             line.product_id = self.complex_product
             line.price_unit = 1
@@ -163,6 +165,9 @@ class TestSaleMrpProcurement(TransactionCase):
 
         sale_order_so0.action_confirm()
 
+        # Verify buttons are working as expected
+        self.assertEqual(sale_order_so0.mrp_production_count, 2, "User should see the correct number of manufacture orders in smart button")
+
         pickings = sale_order_so0.picking_ids
 
         # One delivery...
