@@ -3,9 +3,9 @@
 
 {
     'name': 'Quotation Builder',
-    'category': 'Sales',
+    'category': 'Sales/Sales',
     'summary': 'Build great quotation templates',
-    'website': 'https://www.odoo.com/page/quote-builder',
+    'website': 'https://www.odoo.com/app/sales',
     'version': '1.0',
     'description': "Design great quotation templates with building blocks to significantly boost your success rate.",
     'depends': ['website', 'sale_management', 'website_mail'],
@@ -17,4 +17,5 @@
         'views/sale_order_views.xml',
     ],
     'installable': True,
+    'license': 'LGPL-3',
 }
