@@ -28,15 +28,25 @@ Preferences
         'views/res_config_settings_views.xml',
         'views/sale_stock_portal_template.xml',
         'views/stock_production_lot_views.xml',
-        'views/report_invoice.xml',
+        'views/res_users_views.xml',
+        'report/report_stock_forecasted.xml',
         'report/sale_order_report_templates.xml',
         'report/stock_report_deliveryslip.xml',
+        'data/mail_templates.xml',
         'data/sale_stock_data.xml',
         'wizard/stock_rules_report_views.xml',
+        'wizard/sale_order_cancel_views.xml',
     ],
     'demo': ['data/sale_order_demo.xml'],
-    'qweb': ['static/src/xml/qty_at_date.xml'],
     'installable': True,
     'auto_install': True,
+    'assets': {
+        'web.assets_backend': [
+            'sale_stock/static/src/js/**/*',
+        ],
+        'web.assets_qweb': [
+            'sale_stock/static/src/xml/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
