@@ -1,14 +1,37 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.addons.sale.tests.test_sale_common import TestCommonSaleNoChart, TestCommonSaleMultiCompanyNoChart
+from odoo.addons.mail.tests.common import mail_new_test_user
+from odoo.addons.sale.tests.common import TestSaleCommon
 
 
-class TestCommonSaleTimesheetNoChart(TestCommonSaleNoChart):
+class TestCommonSaleTimesheet(TestSaleCommon):
 
     @classmethod
-    def setUpEmployees(cls):
-        # Create employees
+    def setUpClass(cls, chart_template_ref=None):
+        super().setUpClass(chart_template_ref=chart_template_ref)
+
+        cls.user_employee_company_B = mail_new_test_user(
+            cls.env,
+            name='Gregor Clegane Employee',
+            login='gregor',
+            email='gregor@example.com',
+            notification_type='email',
+            groups='base.group_user',
+            company_id=cls.company_data_2['company'].id,
+            company_ids=[cls.company_data_2['company'].id],
+        )
+        cls.user_manager_company_B = mail_new_test_user(
+            cls.env,
+            name='Cersei Lannister Manager',
+            login='cersei',
+            email='cersei@example.com',
+            notification_type='email',
+            groups='base.group_user',
+            company_id=cls.company_data_2['company'].id,
+            company_ids=[cls.company_data_2['company'].id, cls.env.company.id],
+        )
+
         cls.employee_user = cls.env['hr.employee'].create({
             'name': 'Employee User',
             'timesheet_cost': 15,
@@ -18,28 +41,40 @@ class TestCommonSaleTimesheetNoChart(TestCommonSaleNoChart):
             'timesheet_cost': 45,
         })
 
-    @classmethod
-    def setUpServiceProducts(cls):
-        """ Create Service product for all kind, with each tracking policy. """
-        # Account and project
-        cls.account_sale = cls.env['account.account'].create({
-            'code': 'SERV-2020',
-            'name': 'Product Sales - (test)',
-            'reconcile': True,
-            'user_type_id': cls.env.ref('account.data_account_type_revenue').id,
+        cls.employee_company_B = cls.env['hr.employee'].create({
+            'name': 'Gregor Clegane',
+            'user_id': cls.user_employee_company_B.id,
+            'timesheet_cost': 15,
         })
+
+        cls.manager_company_B = cls.env['hr.employee'].create({
+            'name': 'Cersei Lannister',
+            'user_id': cls.user_manager_company_B.id,
+            'timesheet_cost': 45,
+        })
+
+        # Account and project
+        cls.account_sale = cls.company_data['default_account_revenue']
         cls.analytic_account_sale = cls.env['account.analytic.account'].create({
             'name': 'Project for selling timesheet - AA',
-            'code': 'AA-2030'
+            'code': 'AA-2030',
+            'company_id': cls.company_data['company'].id,
+        })
+        cls.analytic_account_sale_company_B = cls.env['account.analytic.account'].create({
+            'name': 'Project for selling timesheet Company B - AA',
+            'code': 'AA-2030',
+            'company_id': cls.company_data_2['company'].id,
         })
 
         # Create projects
-        cls.project_global = cls.env['project.project'].create({
+        Project = cls.env['project.project'].with_context(tracking_disable=True)
+        cls.project_global = Project.create({
             'name': 'Project for selling timesheets',
             'allow_timesheets': True,
             'analytic_account_id': cls.analytic_account_sale.id,
+            'allow_billable': True,
         })
-        cls.project_template = cls.env['project.project'].create({
+        cls.project_template = Project.create({
             'name': 'Project TEMPLATE for services',
             'allow_timesheets': True,
         })
@@ -48,6 +83,27 @@ class TestCommonSaleTimesheetNoChart(TestCommonSaleNoChart):
             'sequence': 1,
             'project_ids': [(4, cls.project_template.id)]
         })
+        # Projects: at least one per billable type
+        cls.project_task_rate = Project.create({
+            'name': 'Project with pricing_type="task_rate"',
+            'allow_timesheets': True,
+            'allow_billable': True,
+            'partner_id': cls.partner_b.id,
+            'analytic_account_id': cls.analytic_account_sale.id,
+        })
+
+        cls.project_subtask = Project.create({
+            'name': "Sub Task Project (non billable)",
+            'allow_timesheets': True,
+            'allow_billable': False,
+            'partner_id': False,
+        })
+        cls.project_non_billable = Project.create({
+            'name': "Non Billable Project",
+            'allow_timesheets': True,
+            'allow_billable': False,
+            'partner_id': False,
+        })
 
         # Create service products
         uom_hour = cls.env.ref('uom.product_uom_hour')
@@ -199,7 +255,7 @@ class TestCommonSaleTimesheetNoChart(TestCommonSaleNoChart):
             'invoice_policy': 'delivery',
             'uom_id': cls.env.ref('uom.product_uom_hour').id,
             'uom_po_id': cls.env.ref('uom.product_uom_hour').id,
-            'default_code': 'SERV-DELI4',
+            'default_code': 'SERV-DELI5',
             'service_type': 'timesheet',
             'service_tracking': 'project_only',
             'project_template_id': cls.project_template.id,
@@ -286,34 +342,30 @@ class TestCommonSaleTimesheetNoChart(TestCommonSaleNoChart):
             'property_account_income_id': cls.account_sale.id,
         })
 
-
-class TestCommonSaleTimesheetMultiCompanyNoChart(TestCommonSaleMultiCompanyNoChart, TestCommonSaleTimesheetNoChart):
-
-    @classmethod
-    def setUpEmployees(cls):
-        # Create employees
-        cls.setUpUsers()
-        super(TestCommonSaleTimesheetMultiCompanyNoChart, cls).setUpEmployees()
-
-        cls.employee_company_B = cls.env['hr.employee'].create({
-            'name': 'Gregor Clegane',
-            'user_id': cls.user_employee_company_B.id,
-            'timesheet_cost': 15,
-        })
-
-        cls.manager_company_B = cls.env['hr.employee'].create({
-            'name': 'Cersei Lannister',
-            'user_id': cls.user_manager_company_B.id,
-            'timesheet_cost': 45,
-        })
-
-    @classmethod
-    def setUpServiceProducts(cls):
-        """ Create Service product for all kind, with each tracking policy. """
-        super(TestCommonSaleTimesheetMultiCompanyNoChart, cls).setUpServiceProducts()
-        # Account and project
-        cls.analytic_account_sale_company_B = cls.env['account.analytic.account'].create({
-            'name': 'Project for selling timesheet Company B - AA',
-            'code': 'AA-2030',
-            'company_id': cls.company_B.id,
+    def setUp(self):
+        super().setUp()
+        self.so = self.env['sale.order'].with_context(mail_notrack=True, mail_create_nolog=True).create({
+            'partner_id': self.partner_b.id,
+            'partner_invoice_id': self.partner_b.id,
+            'partner_shipping_id': self.partner_b.id,
         })
+        self.env['sale.order.line'].create([{
+            'order_id': self.so.id,
+            'name': self.product_delivery_timesheet1.name,
+            'product_id': self.product_delivery_timesheet1.id,
+            'product_uom_qty': 10,
+            'price_unit': self.product_delivery_timesheet1.list_price
+        }, {
+            'order_id': self.so.id,
+            'product_id': self.product_delivery_timesheet2.id,
+            'product_uom_qty': 5,
+        }, {
+            'order_id': self.so.id,
+            'product_id': self.product_delivery_timesheet3.id,
+            'product_uom_qty': 5,
+        }, {
+            'order_id': self.so.id,
+            'product_id': self.product_order_timesheet1.id,
+            'product_uom_qty': 2,
+        }])
+        self.so.action_confirm()
