@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+
 {
     'name': 'SMS gateway',
-    'version': '2.0',
-    'category': 'Tools',
+    'version': '2.2',
+    'category': 'Hidden/Tools',
     'summary': 'SMS Text Messaging',
     'description': """
 This module gives a framework for SMS text messaging
@@ -11,7 +12,12 @@ This module gives a framework for SMS text messaging
 
 The service is provided by the In App Purchase Odoo platform.
 """,
-    'depends': ['base', 'iap', 'mail', 'phone_validation'],
+    'depends': [
+        'base',
+        'iap_mail',
+        'mail',
+        'phone_validation'
+    ],
     'data': [
         'data/ir_cron_data.xml',
         'wizard/sms_cancel_views.xml',
@@ -19,9 +25,9 @@ The service is provided by the In App Purchase Odoo platform.
         'wizard/sms_template_preview_views.xml',
         'wizard/sms_resend_views.xml',
         'views/ir_actions_views.xml',
+        'views/mail_notification_views.xml',
         'views/res_config_settings_views.xml',
         'views/res_partner_views.xml',
-        'views/assets.xml',
         'views/sms_sms_views.xml',
         'views/sms_template_views.xml',
         'security/ir.model.access.csv',
@@ -31,10 +37,26 @@ The service is provided by the In App Purchase Odoo platform.
         'data/sms_demo.xml',
         'data/mail_demo.xml',
     ],
-    'qweb': [
-        'static/src/xml/thread.xml',
-    ],
     'installable': True,
     'auto_install': True,
+    'assets': {
+        'mail.assets_discuss_public': [
+            'sms/static/src/components/*/*',
+            'sms/static/src/models/*/*.js',
+        ],
+        'web.assets_backend': [
+            'sms/static/src/js/fields_phone_widget.js',
+            'sms/static/src/js/fields_sms_widget.js',
+            'sms/static/src/components/*/*.js',
+            'sms/static/src/models/*/*.js',
+        ],
+        'web.qunit_suite_tests': [
+            'sms/static/tests/sms_widget_test.js',
+            'sms/static/src/components/*/tests/*.js',
+        ],
+        'web.assets_qweb': [
+            'sms/static/src/components/*/*.xml',
+        ],
+    },
     'license': 'LGPL-3',
 }
