@@ -6,10 +6,10 @@ from . import product_strategy
 from . import res_company
 from . import res_partner
 from . import res_config_settings
-from . import stock_inventory
 from . import stock_location
 from . import stock_move
 from . import stock_move_line
+from . import stock_orderpoint
 from . import stock_production_lot
 from . import stock_picking
 from . import stock_quant
@@ -18,3 +18,5 @@ from . import stock_warehouse
 from . import stock_scrap
 from . import product
 from . import stock_package_level
+from . import stock_package_type
+from . import stock_storage_category
