@@ -10,7 +10,7 @@ class ReportStockQuantity(models.Model):
     _description = 'Stock Quantity Report'
 
     date = fields.Date(string='Date', readonly=True)
-    product_tmpl_id = fields.Many2one('product.template', related='product_id.product_tmpl_id')
+    product_tmpl_id = fields.Many2one('product.template', readonly=True)
     product_id = fields.Many2one('product.product', string='Product', readonly=True)
     state = fields.Selection([
         ('forecast', 'Forecasted Stock'),
@@ -29,28 +29,29 @@ CREATE or REPLACE VIEW report_stock_quantity AS (
 SELECT
     MIN(id) as id,
     product_id,
+    product_tmpl_id,
     state,
     date,
     sum(product_qty) as product_qty,
     company_id,
     warehouse_id
-FROM (
-    SELECT
+FROM (SELECT
         m.id,
         m.product_id,
+        pt.id as product_tmpl_id,
         CASE
-            WHEN (whs.id IS NOT NULL AND whd.id IS NULL) OR ls.usage = 'transit' THEN 'out'
-            WHEN (whs.id IS NULL AND whd.id IS NOT NULL) OR ld.usage = 'transit' THEN 'in'
+            WHEN whs.id IS NOT NULL AND whd.id IS NULL THEN 'out'
+            WHEN whd.id IS NOT NULL AND whs.id IS NULL THEN 'in'
         END AS state,
-        m.date_expected::date AS date,
+        m.date::date AS date,
         CASE
-            WHEN (whs.id IS NOT NULL AND whd.id IS NULL) OR ls.usage = 'transit' THEN -m.product_qty
-            WHEN (whs.id IS NULL AND whd.id IS NOT NULL) OR ld.usage = 'transit' THEN m.product_qty
+            WHEN whs.id IS NOT NULL AND whd.id IS NULL THEN -m.product_qty
+            WHEN whd.id IS NOT NULL AND whs.id IS NULL THEN m.product_qty
         END AS product_qty,
         m.company_id,
         CASE
-            WHEN (whs.id IS NOT NULL AND whd.id IS NULL) OR ls.usage = 'transit' THEN whs.id
-            WHEN (whs.id IS NULL AND whd.id IS NOT NULL) OR ld.usage = 'transit' THEN whd.id
+            WHEN whs.id IS NOT NULL AND whd.id IS NULL THEN whs.id
+            WHEN whd.id IS NOT NULL AND whs.id IS NULL THEN whd.id
         END AS warehouse_id
     FROM
         stock_move m
@@ -63,12 +64,14 @@ FROM (
     WHERE
         pt.type = 'product' AND
         m.product_qty != 0 AND
-        (whs.id IS NULL or whd.id IS NULL OR whs.id != whd.id) AND
+        (whs.id IS NOT NULL OR whd.id IS NOT NULL) AND
+        (whs.id IS NULL OR whd.id IS NULL OR whs.id != whd.id) AND
         m.state NOT IN ('cancel', 'draft', 'done')
-    UNION
+    UNION ALL
     SELECT
         -q.id as id,
         q.product_id,
+        pp.product_tmpl_id,
         'forecast' as state,
         date.*::date,
         q.quantity as product_qty,
@@ -80,32 +83,35 @@ FROM (
         stock_quant q
     LEFT JOIN stock_location l on (l.id=q.location_id)
     LEFT JOIN stock_warehouse wh ON l.parent_path like concat('%/', wh.view_location_id, '/%')
+    LEFT JOIN product_product pp on pp.id=q.product_id
     WHERE
-        l.usage = 'internal'
-    UNION
+        (l.usage = 'internal' AND wh.id IS NOT NULL) OR
+        l.usage = 'transit'
+    UNION ALL
     SELECT
         m.id,
         m.product_id,
+        pt.id as product_tmpl_id,
         'forecast' as state,
         GENERATE_SERIES(
         CASE
             WHEN m.state = 'done' THEN (now() at time zone 'utc')::date - interval '3month'
-            ELSE m.date_expected::date
+            ELSE m.date::date
         END,
         CASE
             WHEN m.state != 'done' THEN (now() at time zone 'utc')::date + interval '3 month'
             ELSE m.date::date - interval '1 day'
         END, '1 day'::interval)::date date,
         CASE
-            WHEN ((whs.id IS NOT NULL AND whd.id IS NULL) OR ls.usage = 'transit') AND m.state = 'done' THEN m.product_qty
-            WHEN ((whs.id IS NULL AND whd.id IS NOT NULL) OR ld.usage = 'transit') AND m.state = 'done' THEN -m.product_qty
-            WHEN (whs.id IS NOT NULL AND whd.id IS NULL) OR ls.usage = 'transit' THEN -m.product_qty
-            WHEN (whs.id IS NULL AND whd.id IS NOT NULL) OR ld.usage = 'transit' THEN m.product_qty
+            WHEN whs.id IS NOT NULL AND whd.id IS NULL AND m.state = 'done' THEN m.product_qty
+            WHEN whd.id IS NOT NULL AND whs.id IS NULL AND m.state = 'done' THEN -m.product_qty
+            WHEN whs.id IS NOT NULL AND whd.id IS NULL THEN -m.product_qty
+            WHEN whd.id IS NOT NULL AND whs.id IS NULL THEN m.product_qty
         END AS product_qty,
         m.company_id,
         CASE
-            WHEN (whs.id IS NOT NULL AND whd.id IS NULL) OR ls.usage = 'transit' THEN whs.id
-            WHEN (whs.id IS NULL AND whd.id IS NOT NULL) OR ld.usage = 'transit' THEN whd.id
+            WHEN whs.id IS NOT NULL AND whd.id IS NULL THEN whs.id
+            WHEN whd.id IS NOT NULL AND whs.id IS NULL THEN whd.id
         END AS warehouse_id
     FROM
         stock_move m
@@ -118,9 +124,10 @@ FROM (
     WHERE
         pt.type = 'product' AND
         m.product_qty != 0 AND
+        (whs.id IS NOT NULL OR whd.id IS NOT NULL) AND
         (whs.id IS NULL or whd.id IS NULL OR whs.id != whd.id) AND
         m.state NOT IN ('cancel', 'draft')) as forecast_qty
-GROUP BY product_id, state, date, company_id, warehouse_id
+GROUP BY product_id, product_tmpl_id, state, date, company_id, warehouse_id
 );
 """
         self.env.cr.execute(query)
