@@ -2,10 +2,10 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.exceptions import UserError
-from odoo.tests.common import SavepointCase, Form
+from odoo.tests.common import TransactionCase, Form
 
 
-class TestMultiCompany(SavepointCase):
+class TestMultiCompany(TransactionCase):
     @classmethod
     def setUpClass(cls):
         super(TestMultiCompany, cls).setUpClass()
@@ -27,7 +27,7 @@ class TestMultiCompany(SavepointCase):
             'company_ids': [(6, 0, [cls.company_a.id, cls.company_b.id])]
         })
         cls.user_b = cls.env['res.users'].create({
-            'name': 'user company a with access to company b',
+            'name': 'user company b with access to company a',
             'login': 'user b',
             'groups_id': [(6, 0, [group_user.id, group_stock_manager.id])],
             'company_id': cls.company_b.id,
@@ -103,60 +103,45 @@ class TestMultiCompany(SavepointCase):
             shared_partner.with_user(self.user_b).property_stock_customer = self.stock_location_a
 
     def test_inventory_1(self):
-        """Create an inventory in Company A for a product limited to Company A and, as a user of company
-        B, start the inventory and set its counted quantity to 10 before validating. The inventory
-        lines and stock moves should belong to Company A. The inventory loss location used should be
-        the one of Company A.
+        """Create a quant (inventory adjustment) in Company A for a product limited to Company A and
+        as a user of company B, apply the inventory adjustment and set its counted quantity to 10
+        before validating. The quant and stock moves should belong to Company A.
         """
         product = self.env['product.product'].create({
             'type': 'product',
             'company_id': self.company_a.id,
             'name': 'Product limited to company A',
         })
-        inventory = self.env['stock.inventory'].with_user(self.user_a).create({})
-        self.assertEqual(inventory.company_id, self.company_a)
-        inventory.with_user(self.user_b).action_start()
-        inventory.with_user(self.user_b).line_ids = [(0, 0, {
-            'product_qty': 10,
-            'product_id': product.id,
+        inventory_quant = self.env['stock.quant'].with_user(self.user_a).with_context(inventory_mode=True).create({
             'location_id': self.stock_location_a.id,
-        })]
-        inventory.with_user(self.user_b).action_validate()
-        self.assertEqual(inventory.line_ids.company_id, self.company_a)
-        self.assertEqual(inventory.move_ids.company_id, self.company_a)
-        self.assertEqual(inventory.move_ids.location_id.company_id, self.company_a)
+            'product_id': product.id,
+            'inventory_quantity': 0
+        })
+        self.assertEqual(inventory_quant.company_id, self.company_a)
+        inventory_quant.with_user(self.user_b).inventory_quantity = 10
+        inventory_quant.with_user(self.user_b).action_apply_inventory()
+        last_move_id = self.env['stock.move'].search([('is_inventory', '=', True)])[-1]
+        self.assertEqual(inventory_quant.company_id, self.company_a)
+        self.assertEqual(last_move_id.company_id, self.company_a)
+        self.assertEqual(last_move_id.quantity_done, 10)
+        self.assertEqual(last_move_id.location_id.company_id, self.company_a)
 
     def test_inventory_2(self):
-        """Create an empty inventory in Company A and check it is not possible to use products limited
-        to Company B in it.
+        """Try to create a quant (inventory adjustment) in Company A and check it is not possible to use
+        products limited to Company B in it.
         """
         product = self.env['product.product'].create({
             'name': 'product limited to company b',
             'company_id': self.company_b.id,
             'type': 'product'
         })
-        inventory = self.env['stock.inventory'].with_user(self.user_a).create({})
-        inventory.with_user(self.user_a).action_start()
-        inventory.with_user(self.user_a).line_ids = [(0, 0, {
-            'product_id': product.id,
-            'product_qty': 10,
-            'location_id': self.stock_location_a.id,
-        })]
-        with self.assertRaises(UserError):
-            inventory.with_user(self.user_a).action_validate()
 
-    def test_inventory_3(self):
-        """As a user of Company A, check it is not possible to start an inventory adjustment for
-        a product limited to Company B.
-        """
-        product = self.env['product.product'].create({
-            'name': 'product limited to company b',
-            'company_id': self.company_b.id,
-            'type': 'product'
-        })
-        inventory = self.env['stock.inventory'].with_user(self.user_a).create({'product_ids': [(4, product.id)]})
         with self.assertRaises(UserError):
-            inventory.with_user(self.user_a).action_start()
+            self.env['stock.quant'].with_user(self.user_a).with_context(inventory_mode=True).create({
+                'location_id': self.stock_location_a.id,
+                'product_id': product.id,
+                'inventory_quantity': 10
+        })
 
     def test_picking_1(self):
         """As a user of Company A, create a picking and use a picking type of Company B, check the
@@ -386,3 +371,213 @@ class TestMultiCompany(SavepointCase):
         })
         with self.assertRaises(UserError):
             move._action_confirm()
+
+    def test_intercom_lot_push(self):
+        """ Create a push rule to transfer products received in inter company
+        transit location to company b. Move a lot product from company a to the
+        transit location. Check the move created by the push rule is not chained
+        with previous move, and no product are reserved from inter-company
+        transit. """
+        supplier_location = self.env.ref('stock.stock_location_suppliers')
+        intercom_location = self.env.ref('stock.stock_location_inter_wh')
+        intercom_location.write({'active': True})
+
+        product_lot = self.env['product.product'].create({
+            'type': 'product',
+            'tracking': 'lot',
+            'name': 'product lot',
+        })
+
+        picking_type_to_transit = self.env['stock.picking.type'].create({
+            'name': 'To Transit',
+            'sequence_code': 'TRANSIT',
+            'code': 'outgoing',
+            'company_id': self.company_a.id,
+            'warehouse_id': False,
+            'default_location_src_id': self.stock_location_a.id,
+            'default_location_dest_id': intercom_location.id,
+            'sequence_id': self.env['ir.sequence'].create({
+                'code': 'transit',
+                'name': 'transit sequence',
+                'company_id': self.company_a.id,
+            }).id,
+        })
+
+        route = self.env['stock.location.route'].create({
+            'name': 'Push',
+            'company_id': False,
+            'rule_ids': [(0, False, {
+                'name': 'create a move to company b',
+                'company_id': self.company_b.id,
+                'location_src_id': intercom_location.id,
+                'location_id': self.stock_location_b.id,
+                'action': 'push',
+                'auto': 'manual',
+                'picking_type_id': self.warehouse_b.in_type_id.id,
+            })],
+        })
+
+        move_from_supplier = self.env['stock.move'].create({
+            'company_id': self.company_a.id,
+            'name': 'test_from_supplier',
+            'location_id': supplier_location.id,
+            'location_dest_id': self.stock_location_a.id,
+            'product_id': product_lot.id,
+            'product_uom': product_lot.uom_id.id,
+            'product_uom_qty': 1.0,
+            'picking_type_id': self.warehouse_a.in_type_id.id,
+        })
+        move_from_supplier._action_confirm()
+        move_line_1 = move_from_supplier.move_line_ids[0]
+        move_line_1.lot_name = 'lot 1'
+        move_line_1.qty_done = 1.0
+        move_from_supplier._action_done()
+        lot_1 = move_line_1.lot_id
+
+        move_to_transit = self.env['stock.move'].create({
+            'company_id': self.company_a.id,
+            'name': 'test_to_transit',
+            'location_id': self.stock_location_a.id,
+            'location_dest_id': intercom_location.id,
+            'product_id': product_lot.id,
+            'product_uom': product_lot.uom_id.id,
+            'product_uom_qty': 1.0,
+            'picking_type_id': picking_type_to_transit.id,
+            'route_ids': [(4, route.id)],
+        })
+        move_to_transit._action_confirm()
+        move_to_transit._action_assign()
+        move_line_2 = move_to_transit.move_line_ids[0]
+        self.assertTrue(move_line_2.lot_id, move_line_1.lot_id)
+        move_line_2.qty_done = 1.0
+        move_to_transit._action_done()
+
+        move_push = self.env['stock.move'].search([('location_id', '=', intercom_location.id),
+                                                   ('product_id', '=', product_lot.id)])
+        self.assertTrue(move_push, 'No move created from push rules')
+        self.assertEqual(move_push.state, "assigned")
+        self.assertTrue(move_push.move_line_ids, "No move line created for the move")
+        self.assertFalse(move_push in move_to_transit.move_dest_ids,
+                         "Chained move created in transit location")
+        self.assertNotEqual(move_push.move_line_ids.lot_id, move_line_2.lot_id,
+                            "Reserved from transit location")
+        picking_receipt = move_push.picking_id
+        with self.assertRaises(UserError):
+            picking_receipt.button_validate()
+
+        move_line_3 = move_push.move_line_ids[0]
+        move_line_3.lot_name = 'lot 2'
+        move_line_3.qty_done = 1.0
+        picking_receipt.button_validate()
+        lot_2 = move_line_3.lot_id
+        self.assertEqual(lot_1.company_id, self.company_a)
+        self.assertEqual(lot_1.name, 'lot 1')
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, intercom_location, lot_1), 1.0)
+        self.assertEqual(lot_2.company_id, self.company_b)
+        self.assertEqual(lot_2.name, 'lot 2')
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, self.stock_location_b, lot_2), 1.0)
+
+    def test_intercom_lot_pull(self):
+        """Use warehouse of comany a to resupply warehouse of company b. Check
+        pull rule works correctly in two companies and moves are unchained from
+        inter-company transit location."""
+        customer_location = self.env.ref('stock.stock_location_customers')
+        supplier_location = self.env.ref('stock.stock_location_suppliers')
+        intercom_location = self.env.ref('stock.stock_location_inter_wh')
+        intercom_location.write({'active': True})
+        partner = self.env['res.partner'].create({'name': 'Deco Addict'})
+        self.warehouse_a.resupply_wh_ids = [(6, 0, [self.warehouse_b.id])]
+        resupply_route = self.env['stock.location.route'].search([('supplier_wh_id', '=', self.warehouse_b.id),
+                                                                  ('supplied_wh_id', '=', self.warehouse_a.id)])
+        self.assertTrue(resupply_route, "Resupply route not found")
+
+        product_lot = self.env['product.product'].create({
+            'type': 'product',
+            'tracking': 'lot',
+            'name': 'product lot',
+            'route_ids': [(4, resupply_route.id), (4, self.env.ref('stock.route_warehouse0_mto').id)],
+        })
+
+        move_sup_to_whb = self.env['stock.move'].create({
+            'company_id': self.company_b.id,
+            'name': 'from_supplier_to_whb',
+            'location_id': supplier_location.id,
+            'location_dest_id': self.warehouse_b.lot_stock_id.id,
+            'product_id': product_lot.id,
+            'product_uom': product_lot.uom_id.id,
+            'product_uom_qty': 1.0,
+            'picking_type_id': self.warehouse_b.in_type_id.id,
+        })
+        move_sup_to_whb._action_confirm()
+        move_line_1 = move_sup_to_whb.move_line_ids[0]
+        move_line_1.lot_name = 'lot b'
+        move_line_1.qty_done = 1.0
+        move_sup_to_whb._action_done()
+        lot_b = move_line_1.lot_id
+
+        picking_out = self.env['stock.picking'].create({
+            'company_id': self.company_a.id,
+            'partner_id': partner.id,
+            'picking_type_id': self.warehouse_a.out_type_id.id,
+            'location_id': self.stock_location_a.id,
+            'location_dest_id': customer_location.id,
+        })
+        move_wha_to_cus = self.env['stock.move'].create({
+            'name': "WH_A to Customer",
+            'product_id': product_lot.id,
+            'product_uom_qty': 1,
+            'product_uom': product_lot.uom_id.id,
+            'picking_id': picking_out.id,
+            'location_id': self.stock_location_a.id,
+            'location_dest_id': customer_location.id,
+            'warehouse_id': self.warehouse_a.id,
+            'procure_method': 'make_to_order',
+            'company_id': self.company_a.id,
+        })
+        picking_out.action_confirm()
+
+        move_whb_to_transit = self.env['stock.move'].search([('location_id', '=', self.stock_location_b.id),
+                                                             ('product_id', '=', product_lot.id)])
+        move_transit_to_wha = self.env['stock.move'].search([('location_id', '=', intercom_location.id),
+                                                             ('product_id', '=', product_lot.id)])
+        self.assertTrue(move_whb_to_transit, "No move created by pull rule")
+        self.assertTrue(move_transit_to_wha, "No move created by pull rule")
+        self.assertTrue(move_wha_to_cus in move_transit_to_wha.move_dest_ids,
+                        "Moves are not chained")
+        self.assertFalse(move_transit_to_wha in move_whb_to_transit.move_dest_ids,
+                         "Chained move created in transit location")
+        self.assertEqual(move_wha_to_cus.state, "waiting")
+        self.assertEqual(move_transit_to_wha.state, "waiting")
+        self.assertEqual(move_whb_to_transit.state, "assigned")
+
+        (move_wha_to_cus + move_whb_to_transit + move_transit_to_wha).picking_id.action_assign()
+        self.assertEqual(move_wha_to_cus.state, "waiting")
+        self.assertEqual(move_transit_to_wha.state, "assigned")
+        self.assertEqual(move_whb_to_transit.state, "assigned")
+
+        res_dict = move_whb_to_transit.picking_id.button_validate()
+        self.assertEqual(res_dict.get('res_model'), 'stock.immediate.transfer')
+        wizard = Form(self.env[res_dict['res_model']].with_context(res_dict['context'])).save()
+        wizard.process()
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, intercom_location, lot_b), 1.0)
+        with self.assertRaises(UserError):
+            move_transit_to_wha.picking_id.button_validate()
+
+        move_line_2 = move_transit_to_wha.move_line_ids[0]
+        move_line_2.lot_name = 'lot a'
+        move_line_2.qty_done = 1.0
+        move_transit_to_wha._action_done()
+        lot_a = move_line_2.lot_id
+
+        move_wha_to_cus._action_assign()
+        self.assertEqual(move_wha_to_cus.state, "assigned")
+        res_dict = move_wha_to_cus.picking_id.button_validate()
+        self.assertEqual(res_dict.get('res_model'), 'stock.immediate.transfer')
+        wizard = Form(self.env[res_dict['res_model']].with_context(res_dict['context'])).save()
+        wizard.process()
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, customer_location, lot_a), 1.0)
+
+        self.assertEqual(lot_a.company_id, self.company_a)
+        self.assertEqual(lot_a.name, 'lot a')
+        self.assertEqual(lot_b.company_id, self.company_b)
+        self.assertEqual(lot_b.name, 'lot b')
