@@ -1,20 +1,20 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.tests import Form, tagged
-from odoo.tests.common import SavepointCase
+from odoo.tests import Form
+from odoo.tests.common import TransactionCase
 from odoo.tools import float_round
 from odoo.exceptions import UserError
 
 
-@tagged('post_install', '-at_install')
-class TestPacking(SavepointCase):
+class TestPackingCommon(TransactionCase):
     @classmethod
     def setUpClass(cls):
-        super(TestPacking, cls).setUpClass()
+        super(TestPackingCommon, cls).setUpClass()
         cls.stock_location = cls.env.ref('stock.stock_location_stock')
         cls.warehouse = cls.env['stock.warehouse'].search([('lot_stock_id', '=', cls.stock_location.id)], limit=1)
         cls.warehouse.write({'delivery_steps': 'pick_pack_ship'})
+        cls.warehouse.int_type_id.reservation_method = 'manual'
         cls.pack_location = cls.warehouse.wh_pack_stock_loc_id
         cls.ship_location = cls.warehouse.wh_output_stock_loc_id
         cls.customer_location = cls.env.ref('stock.stock_location_customers')
@@ -22,6 +22,9 @@ class TestPacking(SavepointCase):
         cls.productA = cls.env['product.product'].create({'name': 'Product A', 'type': 'product'})
         cls.productB = cls.env['product.product'].create({'name': 'Product B', 'type': 'product'})
 
+
+class TestPacking(TestPackingCommon):
+
     def test_put_in_pack(self):
         """ In a pick pack ship scenario, create two packs in pick and check that
         they are correctly recognised and handled by the pack and ship picking.
@@ -76,12 +79,12 @@ class TestPacking(SavepointCase):
         pick_picking.move_line_ids.filtered(lambda ml: ml.product_id == self.productA).qty_done = 1.0
         pick_picking.move_line_ids.filtered(lambda ml: ml.product_id == self.productB).qty_done = 2.0
 
-        first_pack = pick_picking.put_in_pack()
-        self.assertEquals(len(pick_picking.package_level_ids), 1, 'Put some products in pack should create a package_level')
-        self.assertEquals(pick_picking.package_level_ids[0].state, 'new', 'A new pack should be in state "new"')
+        first_pack = pick_picking.action_put_in_pack()
+        self.assertEqual(len(pick_picking.package_level_ids), 1, 'Put some products in pack should create a package_level')
+        self.assertEqual(pick_picking.package_level_ids[0].state, 'new', 'A new pack should be in state "new"')
         pick_picking.move_line_ids.filtered(lambda ml: ml.product_id == self.productA and ml.qty_done == 0.0).qty_done = 4.0
         pick_picking.move_line_ids.filtered(lambda ml: ml.product_id == self.productB and ml.qty_done == 0.0).qty_done = 3.0
-        second_pack = pick_picking.put_in_pack()
+        second_pack = pick_picking.action_put_in_pack()
         self.assertEqual(len(pick_picking.move_ids_without_package), 0)
         self.assertEqual(len(packing_picking.move_ids_without_package), 2)
         pick_picking.button_validate()
@@ -91,7 +94,7 @@ class TestPacking(SavepointCase):
         packing_picking.action_assign()
         self.assertEqual(len(packing_picking.package_level_ids), 2, 'Two package levels must be created after assigning picking')
         packing_picking.package_level_ids.write({'is_done': True})
-        packing_picking.action_done()
+        packing_picking._action_done()
 
     def test_pick_a_pack_confirm(self):
         pack = self.env['stock.quant.package'].create({'name': 'The pack to pick'})
@@ -109,42 +112,42 @@ class TestPacking(SavepointCase):
             'location_dest_id': self.stock_location.id,
             'company_id': picking.company_id.id,
         })
-        self.assertEquals(package_level.state, 'draft',
+        self.assertEqual(package_level.state, 'draft',
                           'The package_level should be in draft as it has no moves, move lines and is not confirmed')
         picking.action_confirm()
         self.assertEqual(len(picking.move_ids_without_package), 0)
         self.assertEqual(len(picking.move_lines), 1,
                          'One move should be created when the package_level has been confirmed')
-        self.assertEquals(len(package_level.move_ids), 1,
+        self.assertEqual(len(package_level.move_ids), 1,
                           'The move should be in the package level')
-        self.assertEquals(package_level.state, 'confirmed',
+        self.assertEqual(package_level.state, 'confirmed',
                           'The package level must be state confirmed when picking is confirmed')
         picking.action_assign()
         self.assertEqual(len(picking.move_lines), 1,
                          'You still have only one move when the picking is assigned')
         self.assertEqual(len(picking.move_lines.move_line_ids), 1,
                          'The move  should have one move line which is the reservation')
-        self.assertEquals(picking.move_line_ids.package_level_id.id, package_level.id,
+        self.assertEqual(picking.move_line_ids.package_level_id.id, package_level.id,
                           'The move line created should be linked to the package level')
-        self.assertEquals(picking.move_line_ids.package_id.id, pack.id,
+        self.assertEqual(picking.move_line_ids.package_id.id, pack.id,
                           'The move line must have been reserved on the package of the package_level')
-        self.assertEquals(picking.move_line_ids.result_package_id.id, pack.id,
+        self.assertEqual(picking.move_line_ids.result_package_id.id, pack.id,
                           'The move line must have the same package as result package')
-        self.assertEquals(package_level.state, 'assigned', 'The package level must be in state assigned')
+        self.assertEqual(package_level.state, 'assigned', 'The package level must be in state assigned')
         package_level.write({'is_done': True})
-        self.assertEquals(len(package_level.move_line_ids), 1,
+        self.assertEqual(len(package_level.move_line_ids), 1,
                           'The package level should still keep one move line after have been set to "done"')
-        self.assertEquals(package_level.move_line_ids[0].qty_done, 20.0,
+        self.assertEqual(package_level.move_line_ids[0].qty_done, 20.0,
                           'All quantity in package must be procesed in move line')
         picking.button_validate()
         self.assertEqual(len(picking.move_lines), 1,
                          'You still have only one move when the picking is assigned')
         self.assertEqual(len(picking.move_lines.move_line_ids), 1,
                          'The move  should have one move line which is the reservation')
-        self.assertEquals(package_level.state, 'done', 'The package level must be in state done')
-        self.assertEquals(pack.location_id.id, picking.location_dest_id.id,
+        self.assertEqual(package_level.state, 'done', 'The package level must be in state done')
+        self.assertEqual(pack.location_id.id, picking.location_dest_id.id,
                           'The quant package must be in the destination location')
-        self.assertEquals(pack.quant_ids[0].location_id.id, picking.location_dest_id.id,
+        self.assertEqual(pack.quant_ids[0].location_id.id, picking.location_dest_id.id,
                           'The quant must be in the destination location')
 
     def test_multi_pack_reservation(self):
@@ -179,22 +182,22 @@ class TestPacking(SavepointCase):
             'company_id': picking.company_id.id,
         })
         picking.action_confirm()
-        self.assertEqual(picking.package_level_ids.mapped('location_id.id'), [self.stock_location.id],
+        self.assertEqual(picking.package_level_ids.mapped('location_id.id'), [shelf1_location.id],
                          'The package levels should still in the same location after confirmation.')
         picking.action_assign()
         package_level_reserved = picking.package_level_ids.filtered(lambda pl: pl.state == 'assigned')
         package_level_confirmed = picking.package_level_ids.filtered(lambda pl: pl.state == 'confirmed')
         self.assertEqual(package_level_reserved.location_id.id, shelf1_location.id, 'The reserved package level must be reserved in shelf1')
-        self.assertEqual(package_level_confirmed.location_id.id, self.stock_location.id, 'The not reserved package should keep its location')
+        self.assertEqual(package_level_confirmed.location_id.id, shelf1_location.id, 'The not reserved package should keep its location')
         picking.do_unreserve()
-        self.assertEqual(picking.package_level_ids.mapped('location_id.id'), [self.stock_location.id],
+        self.assertEqual(picking.package_level_ids.mapped('location_id.id'), [shelf1_location.id],
                          'The package levels should have back the original location.')
         picking.package_level_ids.write({'is_done': True})
         picking.action_assign()
         package_level_reserved = picking.package_level_ids.filtered(lambda pl: pl.state == 'assigned')
         package_level_confirmed = picking.package_level_ids.filtered(lambda pl: pl.state == 'confirmed')
         self.assertEqual(package_level_reserved.location_id.id, shelf1_location.id, 'The reserved package level must be reserved in shelf1')
-        self.assertEqual(package_level_confirmed.location_id.id, self.stock_location.id, 'The not reserved package should keep its location')
+        self.assertEqual(package_level_confirmed.location_id.id, shelf1_location.id, 'The not reserved package should keep its location')
         self.assertEqual(picking.package_level_ids.mapped('is_done'), [True, True], 'Both package should still done')
 
     def test_put_in_pack_to_different_location(self):
@@ -215,7 +218,7 @@ class TestPacking(SavepointCase):
         })
         picking = self.env['stock.picking'].create({
             'picking_type_id': self.warehouse.in_type_id.id,
-            'location_id': self.stock_location.id,
+            'location_id': self.customer_location.id,
             'location_dest_id': self.stock_location.id,
             'state': 'draft',
         })
@@ -232,7 +235,7 @@ class TestPacking(SavepointCase):
         picking.action_confirm()
         picking.action_assign()
         picking.move_line_ids.filtered(lambda ml: ml.product_id == self.productA).qty_done = 5.0
-        picking.put_in_pack()
+        picking.action_put_in_pack()
         pack1 = self.env['stock.quant.package'].search([])[-1]
         picking.write({
             'move_line_ids': [(0, 0, {
@@ -258,11 +261,11 @@ class TestPacking(SavepointCase):
                 'state': 'confirmed',
             })]
         })
-        wizard_values = picking.put_in_pack()
+        wizard_values = picking.action_put_in_pack()
         wizard = self.env[(wizard_values.get('res_model'))].browse(wizard_values.get('res_id'))
         wizard.location_dest_id = shelf2_location.id
         wizard.action_done()
-        picking.action_done()
+        picking._action_done()
         pack2 = self.env['stock.quant.package'].search([])[-1]
         self.assertEqual(pack2.location_id.id, shelf2_location.id, 'The package must be stored  in shelf2')
         self.assertEqual(pack1.location_id.id, shelf1_location.id, 'The package must be stored  in shelf1')
@@ -272,10 +275,10 @@ class TestPacking(SavepointCase):
 
     def test_move_picking_with_package(self):
         """
-        355.4 rounded with 0.001 precision is 355.40000000000003.
+        355.4 rounded with 0.01 precision is 355.40000000000003.
         check that nonetheless, moving a picking is accepted
         """
-        self.assertEqual(self.productA.uom_id.rounding, 0.001)
+        self.assertEqual(self.productA.uom_id.rounding, 0.01)
         self.assertEqual(
             float_round(355.4, precision_rounding=self.productA.uom_id.rounding),
             355.40000000000003,
@@ -312,7 +315,7 @@ class TestPacking(SavepointCase):
         picking.action_confirm()
         picking.action_assign()
         move.quantity_done = move.reserved_availability
-        picking.action_done()
+        picking._action_done()
         # if we managed to get there, there was not any exception
         # complaining that 355.4 is not 355.40000000000003. Good job!
 
@@ -356,7 +359,7 @@ class TestPacking(SavepointCase):
         })
         picking.action_confirm()
         with self.assertRaises(UserError):
-            picking.action_done()
+            picking._action_done()
 
     def test_pack_in_receipt_two_step_single_putway(self):
         """ Checks all works right in the following specific corner case:
@@ -421,7 +424,7 @@ class TestPacking(SavepointCase):
         with receipt_form.move_line_ids_without_package.edit(1) as move_line:
             move_line.qty_done = 1
         receipt = receipt_form.save()
-        receipt.put_in_pack()
+        receipt.action_put_in_pack()
         receipt.button_validate()
 
         receipt_package = receipt.package_level_ids_details[0]
@@ -560,7 +563,7 @@ class TestPacking(SavepointCase):
         with receipt_form.move_line_ids_without_package.edit(1) as move_line:
             move_line.qty_done = 1
         receipt = receipt_form.save()
-        receipt.put_in_pack()
+        receipt.action_put_in_pack()
         receipt.button_validate()
 
         receipt_package = receipt.package_level_ids_details[0]
@@ -662,7 +665,7 @@ class TestPacking(SavepointCase):
         pick_picking.action_assign()
 
         pick_picking.move_line_ids.qty_done = 3
-        first_pack = pick_picking.put_in_pack()
+        first_pack = pick_picking.action_put_in_pack()
 
     def test_action_assign_package_level(self):
         """calling _action_assign on move does not erase lines' "result_package_id"
@@ -758,7 +761,7 @@ class TestPacking(SavepointCase):
                 package_level.is_done = True
         action = picking.button_validate()
 
-        self.assertFalse(action, 'Should not open wizard')
+        self.assertEqual(action, True, 'Should not open wizard')
 
         for ml in picking.move_line_ids:
             self.assertEqual(ml.package_id, package, 'move_line.package')
@@ -870,21 +873,269 @@ class TestPacking(SavepointCase):
 
         picking, moveA, moveB = create_picking(pick_type, pick_type.default_location_src_id, pick_type.default_location_dest_id)
         moveA.move_line_ids.qty_done = 1
-        picking.put_in_pack()
+        picking.action_put_in_pack()
         moveB.move_line_ids.qty_done = 1
-        picking.put_in_pack()
+        picking.action_put_in_pack()
         picking.button_validate()
 
         picking, _, _ = create_picking(delivery_type, delivery_type.default_location_src_id, self.customer_location)
-        packA, packB = picking.package_level_ids
+        packB = picking.package_level_ids[1]
         with Form(picking) as picking_form:
             with picking_form.package_level_ids_details.edit(0) as package_level:
                 package_level.is_done = True
         action_data = picking.button_validate()
-        backorder_wizard = self.env[action_data['res_model']].browse(action_data['res_id'])
+        backorder_wizard = Form(self.env['stock.backorder.confirmation'].with_context(action_data['context'])).save()
         backorder_wizard.process()
         bo = self.env['stock.picking'].search([('backorder_id', '=', picking.id)])
 
         self.assertNotIn(packB, picking.package_level_ids)
         self.assertEqual(packB, bo.package_level_ids)
         self.assertEqual(bo.package_level_ids.state, 'assigned')
+
+    def test_package_and_sub_location(self):
+        """
+        Suppose there are some products P available in shelf1, a child location of the pack location.
+        When moving these P to another child location of pack location, the source location of the
+        related package level should be shelf1
+        """
+        shelf1_location = self.env['stock.location'].create({
+            'name': 'shelf1',
+            'usage': 'internal',
+            'location_id': self.pack_location.id,
+        })
+        shelf2_location = self.env['stock.location'].create({
+            'name': 'shelf2',
+            'usage': 'internal',
+            'location_id': self.pack_location.id,
+        })
+
+        pack = self.env['stock.quant.package'].create({'name': 'Super Package'})
+        self.env['stock.quant']._update_available_quantity(self.productA, shelf1_location, 20.0, package_id=pack)
+
+        picking = self.env['stock.picking'].create({
+            'picking_type_id': self.warehouse.in_type_id.id,
+            'location_id': self.pack_location.id,
+            'location_dest_id': shelf2_location.id,
+        })
+        package_level = self.env['stock.package_level'].create({
+            'package_id': pack.id,
+            'picking_id': picking.id,
+            'location_dest_id': picking.location_dest_id.id,
+            'company_id': picking.company_id.id,
+        })
+
+        self.assertEqual(package_level.location_id, shelf1_location)
+
+        picking.action_confirm()
+        package_level.is_done = True
+        picking.button_validate()
+
+        self.assertEqual(package_level.location_id, shelf1_location)
+
+    def test_pack_in_receipt_two_step_multi_putaway_02(self):
+        """
+        Suppose a product P, its weight is equal to 1kg
+        We have 100 x P on two pallets.
+        Receipt in two steps + Sub locations in WH/Stock + Storage Category
+        The Storage Category adds some constraints on weight/pallets capacity
+        """
+        warehouse = self.stock_location.warehouse_id
+        warehouse.reception_steps = "two_steps"
+        self.productA.weight = 1.0
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_stock_storage_categories').id)]})
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_stock_multi_locations').id)]})
+
+        package_type = self.env['stock.package.type'].create({
+            'name': "Super Pallet",
+        })
+        package_01, package_02 = self.env['stock.quant.package'].create([{
+            'name': 'Pallet %s' % i,
+            'package_type_id': package_type.id,
+        } for i in [1, 2]])
+
+        # max 100kg (so 100 x P) and max 1 pallet -> we will work with pallets,
+        # so the pallet capacity constraint should be the effective one
+        stor_category = self.env['stock.storage.category'].create({
+            'name': 'Super Storage Category',
+            'max_weight': 100,
+            'package_capacity_ids': [(0, 0, {
+                'package_type_id': package_type.id,
+                'quantity': 1,
+            })]
+        })
+
+        # 3 sub locations with the storage category
+        # (the third location should never be used)
+        sub_loc_01, sub_loc_02, dummy = self.env['stock.location'].create([{
+            'name': 'Sub Location %s' % i,
+            'usage': 'internal',
+            'location_id': self.stock_location.id,
+            'storage_category_id': stor_category.id,
+        } for i in [1, 2, 3]])
+
+        self.env['stock.putaway.rule'].create({
+            'location_in_id': self.stock_location.id,
+            'location_out_id': self.stock_location.id,
+            'package_type_ids': [(4, package_type.id)],
+            'storage_category_id': stor_category.id,
+        })
+
+        # Receive 100 x P
+        receipt_picking = self.env['stock.picking'].create({
+            'picking_type_id': warehouse.in_type_id.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': warehouse.wh_input_stock_loc_id.id,
+        })
+        self.env['stock.move'].create({
+            'name': self.productA.name,
+            'product_id': self.productA.id,
+            'product_uom': self.productA.uom_id.id,
+            'product_uom_qty': 100.0,
+            'picking_id': receipt_picking.id,
+            'location_id': receipt_picking.location_id.id,
+            'location_dest_id': receipt_picking.location_dest_id.id,
+        })
+        receipt_picking.action_confirm()
+
+        # Distribute the products on two pallets, one with 49 x P and a second
+        # one with 51 x P (to easy the debugging in case of trouble)
+        move_form = Form(receipt_picking.move_lines, view="stock.view_stock_move_operations")
+        with move_form.move_line_ids.edit(0) as line:
+            line.qty_done = 49
+            line.result_package_id = package_01
+        with move_form.move_line_ids.new() as line:
+            line.qty_done = 51
+            line.result_package_id = package_02
+        move_form.save()
+        receipt_picking.button_validate()
+
+        # We are in two-steps receipt -> check the internal picking
+        internal_picking = self.env['stock.picking'].search([], order='id desc', limit=1)
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'product_uom_qty': 49, 'qty_done': 0, 'result_package_id': package_01.id, 'location_dest_id': sub_loc_01.id},
+            {'product_uom_qty': 51, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+        ])
+
+        # Change the constraints of the storage category:
+        # max 75kg (so 75 x P) and max 2 pallet -> this time, the weight
+        # constraint should be the effective one
+        stor_category.max_weight = 75
+        stor_category.package_capacity_ids.quantity = 2
+        internal_picking.do_unreserve()
+        internal_picking.action_assign()
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'product_uom_qty': 49, 'qty_done': 0, 'result_package_id': package_01.id, 'location_dest_id': sub_loc_01.id},
+            {'product_uom_qty': 51, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+        ])
+
+        move_form = Form(internal_picking.move_lines, view="stock.view_stock_move_operations")
+        # lines order is reversed: [Pallet 02, Pallet 01]
+        with move_form.move_line_ids.edit(0) as line:
+            line.qty_done = 51
+        with move_form.move_line_ids.edit(1) as line:
+            line.qty_done = 49
+        move_form.save()
+        # lines order is reversed: [Pallet 02, Pallet 01]
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'product_uom_qty': 51, 'qty_done': 51, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+            {'product_uom_qty': 49, 'qty_done': 49, 'result_package_id': package_01.id, 'location_dest_id': sub_loc_01.id},
+        ])
+
+    def test_pack_in_receipt_two_step_multi_putaway_03(self):
+        """
+        Two sublocations (max 100kg, max 2 pallet)
+        Two products P1, P2, weight = 1kg
+        There are 10 x P1 on a pallet in the first sub location
+        Receive a pallet of 50 x P1 + 50 x P2 => because of weight constraint, should be redirected to the
+            second sub location
+        Then, same with max 200kg max 1 pallet => same result, this time because of pallet count constraint
+        """
+        warehouse = self.stock_location.warehouse_id
+        warehouse.reception_steps = "two_steps"
+        self.productA.weight = 1.0
+        self.productB.weight = 1.0
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_stock_storage_categories').id)]})
+        self.env.user.write({'groups_id': [(4, self.env.ref('stock.group_stock_multi_locations').id)]})
+
+        package_type = self.env['stock.package.type'].create({
+            'name': "Super Pallet",
+        })
+        package_01, package_02 = self.env['stock.quant.package'].create([{
+            'name': 'Pallet %s' % i,
+            'package_type_id': package_type.id,
+        } for i in [1, 2]])
+
+        # max 100kg and max 2 pallets
+        stor_category = self.env['stock.storage.category'].create({
+            'name': 'Super Storage Category',
+            'max_weight': 100,
+            'package_capacity_ids': [(0, 0, {
+                'package_type_id': package_type.id,
+                'quantity': 2,
+            })]
+        })
+
+        # 3 sub locations with the storage category
+        # (the third location should never be used)
+        sub_loc_01, sub_loc_02, dummy = self.env['stock.location'].create([{
+            'name': 'Sub Location %s' % i,
+            'usage': 'internal',
+            'location_id': self.stock_location.id,
+            'storage_category_id': stor_category.id,
+        } for i in [1, 2, 3]])
+
+        self.env['stock.quant']._update_available_quantity(self.productA, sub_loc_01, 10, package_id=package_01)
+
+        self.env['stock.putaway.rule'].create({
+            'location_in_id': self.stock_location.id,
+            'location_out_id': self.stock_location.id,
+            'package_type_ids': [(4, package_type.id)],
+            'storage_category_id': stor_category.id,
+        })
+
+        # Receive 50 x P_A and 50 x P_B
+        receipt_picking = self.env['stock.picking'].create({
+            'picking_type_id': warehouse.in_type_id.id,
+            'location_id': self.env.ref('stock.stock_location_suppliers').id,
+            'location_dest_id': warehouse.wh_input_stock_loc_id.id,
+        })
+        self.env['stock.move'].create([{
+            'name': p.name,
+            'product_id': p.id,
+            'product_uom': p.uom_id.id,
+            'product_uom_qty': 50,
+            'picking_id': receipt_picking.id,
+            'location_id': receipt_picking.location_id.id,
+            'location_dest_id': receipt_picking.location_dest_id.id,
+        } for p in [self.productA, self.productB]])
+        receipt_picking.action_confirm()
+
+        move_form = Form(receipt_picking.move_lines[0], view="stock.view_stock_move_operations")
+        with move_form.move_line_ids.edit(0) as line:
+            line.qty_done = 50
+            line.result_package_id = package_02
+        move_form.save()
+        move_form = Form(receipt_picking.move_lines[1], view="stock.view_stock_move_operations")
+        with move_form.move_line_ids.edit(0) as line:
+            line.qty_done = 50
+            line.result_package_id = package_02
+        move_form.save()
+        receipt_picking.button_validate()
+
+        # We are in two-steps receipt -> check the internal picking
+        internal_picking = self.env['stock.picking'].search([], order='id desc', limit=1)
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'product_id': self.productA.id, 'product_uom_qty': 50, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+            {'product_id': self.productB.id, 'product_uom_qty': 50, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+        ])
+
+        # Change the constraints of the storage category:
+        # max 200kg and max 1 pallet
+        stor_category.max_weight = 200
+        stor_category.package_capacity_ids.quantity = 1
+        internal_picking.do_unreserve()
+        internal_picking.action_assign()
+        self.assertRecordValues(internal_picking.move_line_ids, [
+            {'product_id': self.productA.id, 'product_uom_qty': 50, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+            {'product_id': self.productB.id, 'product_uom_qty': 50, 'qty_done': 0, 'result_package_id': package_02.id, 'location_dest_id': sub_loc_02.id},
+        ])
