@@ -1,17 +1,22 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import product_label_layout
 from . import stock_assign_serial_numbers
 from . import stock_picking_return
 from . import stock_change_product_qty
+from . import stock_inventory_conflict
+from . import stock_inventory_warning
+from . import stock_inventory_adjustment_name
 from . import stock_scheduler_compute
 from . import stock_immediate_transfer
 from . import stock_backorder_confirmation
-from . import stock_overprocessed_transfer
 from . import stock_quantity_history
 from . import stock_rules_report
 from . import stock_warn_insufficient_qty
 from . import product_replenish
 from . import stock_track_confirmation
 from . import stock_package_destination
-from . import stock_picking_responsible  # TODO: Remove this import in 13.1
+from . import stock_orderpoint_snooze
+from . import stock_request_count
+from . import stock_replenishment_info
