@@ -1,8 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-import re
 
-from odoo import api, fields, models, _
+from odoo import _, api, fields, models
 from odoo.exceptions import ValidationError
 
 
@@ -17,8 +16,8 @@ class StockAssignSerialNumbers(models.TransientModel):
             return len(filtered_move_lines)
 
     product_id = fields.Many2one('product.product', 'Product',
-        related='move_id.product_id', required=True)
-    move_id = fields.Many2one('stock.move', required=True)
+        related='move_id.product_id')
+    move_id = fields.Many2one('stock.move')
     next_serial_number = fields.Char('First SN', required=True)
     next_serial_count = fields.Integer('Number of SN',
         default=_default_next_serial_count, required=True)
