@@ -5,10 +5,10 @@
 
 from odoo.addons.stock_account.tests.test_stockvaluation import _create_accounting_data
 from odoo.tests import Form, tagged
-from odoo.tests.common import SavepointCase, TransactionCase
+from odoo.tests.common import TransactionCase
 
 
-class TestStockValuationCommon(SavepointCase):
+class TestStockValuationCommon(TransactionCase):
     @classmethod
     def setUpClass(cls):
         super(TestStockValuationCommon, cls).setUpClass()
@@ -23,6 +23,7 @@ class TestStockValuationCommon(SavepointCase):
         })
         cls.picking_type_in = cls.env.ref('stock.picking_type_in')
         cls.picking_type_out = cls.env.ref('stock.picking_type_out')
+        cls.env.ref('base.EUR').active = True
 
     def setUp(self):
         super(TestStockValuationCommon, self).setUp()
@@ -108,7 +109,7 @@ class TestStockValuationCommon(SavepointCase):
             'picking_type_id': self.picking_type_out.id,
         })
         if unit_cost:
-            dropshipped.unit_cost = unit_cost
+            dropshipped.price_unit = unit_cost
         dropshipped._action_confirm()
         dropshipped._action_assign()
         dropshipped.move_line_ids.qty_done = quantity
@@ -123,7 +124,7 @@ class TestStockValuationCommon(SavepointCase):
         stock_return_picking_action = stock_return_picking.create_returns()
         return_pick = self.env['stock.picking'].browse(stock_return_picking_action['res_id'])
         return_pick.move_lines[0].move_line_ids[0].qty_done = quantity_to_return
-        return_pick.action_done()
+        return_pick._action_done()
         return return_pick.move_lines
 
 
@@ -211,7 +212,7 @@ class TestStockValuationStandard(TestStockValuationCommon):
         move3 = self._make_out_move(self.product1, 15)
 
         # change cost from 10 to 15
-        self.product1._change_standard_price(15.0)
+        self.product1.standard_price = 15.0
 
         self.assertEqual(self.product1.value_svl, 75)
         self.assertEqual(self.product1.quantity_svl, 5)
@@ -276,7 +277,7 @@ class TestStockValuationStandard(TestStockValuationCommon):
             'location_dest_id': self.stock_location.id,
         })
         for product in (product1, product2):
-            product.unit_cost = 10
+            product.standard_price = 10
             in_move = self.env['stock.move'].create({
                 'name': 'in %s units @ %s per unit' % (2, str(10)),
                 'product_id': product.id,
@@ -293,7 +294,7 @@ class TestStockValuationStandard(TestStockValuationCommon):
         # set quantity done only on one move
         in_move.move_line_ids.qty_done = 2
         res_dict = picking.button_validate()
-        wizard = self.env[(res_dict.get('res_model'))].browse(res_dict.get('res_id'))
+        wizard = self.env[(res_dict.get('res_model'))].with_context(res_dict.get('context')).browse(res_dict.get('res_id'))
         res_dict_for_back_order = wizard.process()
 
         self.assertTrue(product2.stock_valuation_layer_ids)
@@ -486,9 +487,9 @@ class TestStockValuationAVCO(TestStockValuationCommon):
         self.assertEqual(self.product1.standard_price, 15)
 
     def test_rounding_slv_1(self):
-        move1 = self._make_in_move(self.product1, 1, unit_cost=1.00)
-        move2 = self._make_in_move(self.product1, 1, unit_cost=1.00)
-        move3 = self._make_in_move(self.product1, 1, unit_cost=1.01)
+        self._make_in_move(self.product1, 1, unit_cost=1.00)
+        self._make_in_move(self.product1, 1, unit_cost=1.00)
+        self._make_in_move(self.product1, 1, unit_cost=1.01)
 
         self.assertAlmostEqual(self.product1.value_svl, 3.01)
 
@@ -515,6 +516,16 @@ class TestStockValuationAVCO(TestStockValuationCommon):
         self.assertEqual(self.product1.quantity_svl, 0)
         self.assertEqual(self.product1.standard_price, 1.01)
 
+    def test_return_delivery_2(self):
+        self.product1.write({"standard_price": 1})
+        move1 = self._make_out_move(self.product1, 10, create_picking=True, force_assign=True)
+        self._make_in_move(self.product1, 10, unit_cost=2)
+        self._make_return(move1, 10)
+
+        self.assertEqual(self.product1.value_svl, 20)
+        self.assertEqual(self.product1.quantity_svl, 10)
+        self.assertEqual(self.product1.standard_price, 2)
+
 
 class TestStockValuationFIFO(TestStockValuationCommon):
     def setUp(self):
@@ -666,14 +677,33 @@ class TestStockValuationFIFO(TestStockValuationCommon):
         self.assertEqual(self.product1.quantity_svl, 1)
 
     def test_dropship_1(self):
-        orig_standard_price = self.product1.standard_price
         move1 = self._make_in_move(self.product1, 1, unit_cost=10)
         move2 = self._make_in_move(self.product1, 1, unit_cost=20)
         move3 = self._make_dropship_move(self.product1, 1, unit_cost=10)
 
         self.assertEqual(self.product1.value_svl, 30)
         self.assertEqual(self.product1.quantity_svl, 2)
-        self.assertEqual(orig_standard_price, self.product1.standard_price)
+        self.assertAlmostEqual(self.product1.standard_price, 10)
+
+    def test_return_delivery_2(self):
+        self._make_in_move(self.product1, 1, unit_cost=10)
+        self.product1.standard_price = 0
+        self._make_in_move(self.product1, 1, unit_cost=0)
+
+        self._make_out_move(self.product1, 1)
+        out_move02 = self._make_out_move(self.product1, 1, create_picking=True)
+
+        returned = self._make_return(out_move02, 1)
+        self.assertEqual(returned.stock_valuation_layer_ids.value, 0)
+
+    def test_return_delivery_3(self):
+        self.product1.write({"standard_price": 1})
+        move1 = self._make_out_move(self.product1, 10, create_picking=True, force_assign=True)
+        self._make_in_move(self.product1, 10, unit_cost=2)
+        self._make_return(move1, 10)
+
+        self.assertEqual(self.product1.value_svl, 20)
+        self.assertEqual(self.product1.quantity_svl, 10)
 
 
 class TestStockValuationChangeCostMethod(TestStockValuationCommon):
@@ -847,9 +877,9 @@ class TestStockValuationChangeValuation(TestStockValuationCommon):
             'property_stock_journal': self.stock_journal.id,
         })
 
-        # Try to change the product category with a `default_type` key in the context and
+        # Try to change the product category with a `default_detailed_type` key in the context and
         # check it doesn't break the account move generation.
-        self.product1.with_context(default_type='product').categ_id = cat2
+        self.product1.with_context(default_detailed_type='product').categ_id = cat2
         self.assertEqual(self.product1.categ_id, cat2)
 
         self.assertEqual(self.product1.value_svl, 100)
@@ -899,3 +929,61 @@ class TestStockValuationChangeValuation(TestStockValuationCommon):
         self.assertEqual(len(self.product1.stock_valuation_layer_ids.mapped('account_move_id')), 2)
         self.assertEqual(len(self.product1.stock_valuation_layer_ids), 3)
 
+@tagged('post_install', '-at_install')
+class TestAngloSaxonAccounting(TestStockValuationCommon):
+    @classmethod
+    def setUpClass(cls):
+        super(TestAngloSaxonAccounting, cls).setUpClass()
+        cls.env.company.anglo_saxon_accounting = True
+        cls.stock_input_account, cls.stock_output_account, cls.stock_valuation_account, cls.expense_account, cls.stock_journal = _create_accounting_data(cls.env)
+        cls.product1.write({
+            'property_account_expense_id': cls.expense_account.id,
+        })
+        cls.product1.categ_id.write({
+            'property_valuation': 'real_time',
+            'property_stock_account_input_categ_id': cls.stock_input_account.id,
+            'property_stock_account_output_categ_id': cls.stock_output_account.id,
+            'property_stock_valuation_account_id': cls.stock_valuation_account.id,
+            'property_stock_journal': cls.stock_journal.id,
+        })
+        cls.default_journal_purchase =  cls.env['account.journal'].search([
+            ('company_id', '=', cls.env.company.id),
+            ('type', '=', 'purchase')
+        ], limit=1)
+
+    def test_avco_and_credit_note(self):
+        """
+        When reversing an invoice that contains some anglo-saxo AML, the new anglo-saxo AML should have the same value
+        """
+        self.product1.categ_id.property_cost_method = 'average'
+
+        self._make_in_move(self.product1, 2, unit_cost=10)
+
+        invoice_form = Form(self.env['account.move'].with_context(default_move_type='out_invoice'))
+        invoice_form.partner_id = self.env['res.partner'].create({'name': 'Super Client'})
+        with invoice_form.invoice_line_ids.new() as invoice_line_form:
+            invoice_line_form.product_id = self.product1
+            invoice_line_form.quantity = 2
+            invoice_line_form.price_unit = 25
+            invoice_line_form.account_id = self.default_journal_purchase.default_account_id
+        invoice = invoice_form.save()
+        invoice.action_post()
+
+        self._make_in_move(self.product1, 2, unit_cost=20)
+        self.assertEqual(self.product1.standard_price, 15)
+
+        refund_wizard = self.env['account.move.reversal'].with_context(active_model="account.move", active_ids=invoice.ids).create({
+            'refund_method': 'refund',
+            'journal_id': invoice.journal_id.id,
+        })
+        action = refund_wizard.reverse_moves()
+        reverse_invoice = self.env['account.move'].browse(action['res_id'])
+        with Form(reverse_invoice) as reverse_invoice_form:
+            with reverse_invoice_form.invoice_line_ids.edit(0) as line:
+                line.quantity = 1
+        reverse_invoice.action_post()
+
+        anglo_lines = reverse_invoice.line_ids.filtered(lambda l: l.is_anglo_saxon_line)
+        self.assertEqual(len(anglo_lines), 2)
+        self.assertEqual(abs(anglo_lines[0].balance), 10)
+        self.assertEqual(abs(anglo_lines[1].balance), 10)
