@@ -8,7 +8,7 @@ from odoo.tools import mute_logger
 class TestDropship(common.TransactionCase):
     def test_change_qty(self):
         # enable the dropship and MTO route on the product
-        prod = self.env.ref('product.product_product_8')
+        prod = self.env['product.product'].create({'name': 'Large Desk'})
         dropshipping_route = self.env.ref('stock_dropshipping.route_drop_shipping')
         mto_route = self.env.ref('stock.route_warehouse0_mto')
         prod.write({'route_ids': [(6, 0, [dropshipping_route.id, mto_route.id])]})
@@ -41,6 +41,10 @@ class TestDropship(common.TransactionCase):
         po = self.env['purchase.order'].search([('group_id', '=', so.procurement_group_id.id)])
         po_line = po.order_line
 
+        # Check dropship count on SO and PO
+        self.assertEqual(po.incoming_picking_count, 0)
+        self.assertEqual(so.delivery_count, 0)
+
         # Check the qty on the P0
         self.assertAlmostEqual(po_line.product_qty, 1.00)
 
@@ -87,7 +91,7 @@ class TestDropship(common.TransactionCase):
 
         # Create a sales order with a line of 200 PCE incoming shipment, with route_id drop shipping
         so_form = Form(self.env['sale.order'])
-        so_form.partner_id = self.env.ref('base.res_partner_2')
+        so_form.partner_id = self.env['res.partner'].create({'name': 'My Test Partner'})
         so_form.payment_term_id = self.env.ref('account.account_payment_term_end_following_month')
         with mute_logger('odoo.tests.common.onchange'):
             # otherwise complains that there's not enough inventory and
@@ -109,8 +113,13 @@ class TestDropship(common.TransactionCase):
         purchase = self.env['purchase.order'].search([('partner_id', '=', supplier_dropship.id)])
         self.assertTrue(purchase, "an RFQ should have been created by the scheduler")
         purchase.button_confirm()
-        self.assertEquals(purchase.state, 'purchase', 'Purchase order should be in the approved state')
-        self.assertEquals(len(purchase.ids), 1, 'There should be one picking')
+        self.assertEqual(purchase.state, 'purchase', 'Purchase order should be in the approved state')
+
+        # Check dropship count on SO and PO
+        self.assertEqual(purchase.incoming_picking_count, 0)
+        self.assertEqual(sale_order_drp_shpng.delivery_count, 0)
+        self.assertEqual(sale_order_drp_shpng.dropship_picking_count, 1)
+        self.assertEqual(purchase.dropship_picking_count, 1)
 
         # Send the 200 pieces
         purchase.picking_ids.move_lines.quantity_done = purchase.picking_ids.move_lines.product_qty
@@ -120,4 +129,4 @@ class TestDropship(common.TransactionCase):
         move_line = self.env['stock.move.line'].search([
             ('location_dest_id', '=', self.env.ref('stock.stock_location_customers').id),
             ('product_id', '=', drop_shop_product.id)])
-        self.assertEquals(len(move_line.ids), 1, 'There should be exactly one move line')
+        self.assertEqual(len(move_line.ids), 1, 'There should be exactly one move line')
