@@ -2,5 +2,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import stock_picking_to_batch
-from . import stock_immediate_transfer
+from . import stock_add_to_wave
+from . import stock_package_destination
 from . import stock_backorder_confirmation
