@@ -3,6 +3,7 @@
 
 import logging
 import re
+import werkzeug
 
 from odoo import api, fields, models, tools, _
 from odoo.exceptions import UserError
@@ -14,6 +15,7 @@ emails_split = re.compile(r"[;,\n\r]+")
 
 class SurveyInvite(models.TransientModel):
     _name = 'survey.invite'
+    _inherit = 'mail.composer.mixin'
     _description = 'Survey Invitation Wizard'
 
     @api.model
@@ -27,14 +29,9 @@ class SurveyInvite(models.TransientModel):
         return self.env.user.partner_id
 
     # composer content
-    subject = fields.Char('Subject')
-    body = fields.Html('Contents', default='', sanitize_style=True)
     attachment_ids = fields.Many2many(
         'ir.attachment', 'survey_mail_compose_message_ir_attachments_rel', 'wizard_id', 'attachment_id',
         string='Attachments')
-    template_id = fields.Many2one(
-        'mail.template', 'Use template', index=True,
-        domain="[('model', '=', 'survey.user_input')]")
     # origin
     email_from = fields.Char('From', default=_get_default_from, help="Email address of the sender.")
     author_id = fields.Many2one(
@@ -43,7 +40,13 @@ class SurveyInvite(models.TransientModel):
         help="Author of the message.")
     # recipients
     partner_ids = fields.Many2many(
-        'res.partner', 'survey_invite_partner_ids', 'invite_id', 'partner_id', string='Recipients')
+        'res.partner', 'survey_invite_partner_ids', 'invite_id', 'partner_id', string='Recipients',
+        domain="""[
+            '|', (survey_users_can_signup, '=', 1),
+            '|', (not survey_users_login_required, '=', 1),
+                 ('user_ids', '!=', False),
+        ]"""
+    )
     existing_partner_ids = fields.Many2many(
         'res.partner', compute='_compute_existing_partner_ids', readonly=True, store=False)
     emails = fields.Text(string='Additional emails', help="This list of emails of recipients will not be converted in contacts.\
@@ -59,9 +62,10 @@ class SurveyInvite(models.TransientModel):
     mail_server_id = fields.Many2one('ir.mail_server', 'Outgoing mail server')
     # survey
     survey_id = fields.Many2one('survey.survey', string='Survey', required=True)
-    survey_url = fields.Char(related="survey_id.public_url", readonly=True)
+    survey_start_url = fields.Char('Survey URL', compute='_compute_survey_start_url')
     survey_access_mode = fields.Selection(related="survey_id.access_mode", readonly=True)
     survey_users_login_required = fields.Boolean(related="survey_id.users_login_required", readonly=True)
+    survey_users_can_signup = fields.Boolean(related='survey_id.users_can_signup')
     deadline = fields.Datetime(string="Answer deadline")
 
     @api.depends('partner_ids', 'survey_id')
@@ -92,6 +96,16 @@ class SurveyInvite(models.TransientModel):
 
         self.existing_text = existing_text
 
+    @api.depends('survey_id.access_token')
+    def _compute_survey_start_url(self):
+        for invite in self:
+            invite.survey_start_url = werkzeug.urls.url_join(invite.survey_id.get_base_url(), invite.survey_id.get_start_url()) if invite.survey_id else False
+
+    # Overrides of mail.composer.mixin
+    @api.depends('survey_id')  # fake trigger otherwise not computed in new mode
+    def _compute_render_model(self):
+        self.render_model = 'survey.user_input'
+
     @api.onchange('emails')
     def _onchange_emails(self):
         if self.emails and (self.survey_users_login_required and not self.survey_id.users_can_signup):
@@ -110,16 +124,6 @@ class SurveyInvite(models.TransientModel):
             raise UserError(_("Some emails you just entered are incorrect: %s") % (', '.join(error)))
         self.emails = '\n'.join(valid)
 
-    @api.onchange('survey_users_login_required')
-    def _onchange_survey_users_login_required(self):
-        if self.survey_users_login_required and not self.survey_id.users_can_signup:
-            return {'domain': {
-                    'partner_ids': [('user_ids', '!=', False)]
-                    }}
-        return {'domain': {
-                'partner_ids': []
-                }}
-
     @api.onchange('partner_ids')
     def _onchange_partner_ids(self):
         if self.survey_users_login_required and self.partner_ids:
@@ -129,30 +133,41 @@ class SurveyInvite(models.TransientModel):
                     ('id', 'in', self.partner_ids.ids)
                 ])
                 if invalid_partners:
-                    raise UserError(
-                        _('The following recipients have no user account: %s. You should create user accounts for them or allow external signup in configuration.' %
-                            (','.join(invalid_partners.mapped('name')))))
+                    raise UserError(_(
+                        'The following recipients have no user account: %s. You should create user accounts for them or allow external signup in configuration.',
+                        ', '.join(invalid_partners.mapped('name'))
+                    ))
 
-    @api.onchange('template_id')
-    def _onchange_template_id(self):
-        """ UPDATE ME """
-        if self.template_id:
-            self.subject = self.template_id.subject
-            self.body = self.template_id.body_html
+    @api.model_create_multi
+    def create(self, vals_list):
+        for values in vals_list:
+            if values.get('template_id') and not (values.get('body') or values.get('subject')):
+                template = self.env['mail.template'].browse(values['template_id'])
+                if not values.get('subject'):
+                    values['subject'] = template.subject
+                if not values.get('body'):
+                    values['body'] = template.body_html
+        return super().create(vals_list)
 
-    @api.model
-    def create(self, values):
-        if values.get('template_id') and not (values.get('body') or values.get('subject')):
-            template = self.env['mail.template'].browse(values['template_id'])
-            if not values.get('subject'):
-                values['subject'] = template.subject
-            if not values.get('body'):
-                values['body'] = template.body_html
-        return super(SurveyInvite, self).create(values)
+    @api.depends('template_id', 'partner_ids')
+    def _compute_subject(self):
+        for invite in self:
+            langs = set(invite.partner_ids.mapped('lang')) - {False}
+            if len(langs) == 1:
+                invite = invite.with_context(lang=langs.pop())
+            super(SurveyInvite, invite)._compute_subject()
 
-    #------------------------------------------------------
+    @api.depends('template_id', 'partner_ids')
+    def _compute_body(self):
+        for invite in self:
+            langs = set(invite.partner_ids.mapped('lang')) - {False}
+            if len(langs) == 1:
+                invite = invite.with_context(lang=langs.pop())
+            super(SurveyInvite, invite)._compute_body()
+
+    # ------------------------------------------------------
     # Wizard validation and send
-    #------------------------------------------------------
+    # ------------------------------------------------------
 
     def _prepare_answers(self, partners, emails):
         answers = self.env['survey.user_input']
@@ -190,14 +205,13 @@ class SurveyInvite(models.TransientModel):
 
     def _get_answers_values(self):
         return {
-            'input_type': 'link',
             'deadline': self.deadline,
         }
 
     def _send_mail(self, answer):
         """ Create mail specific for recipient containing notably its access token """
-        subject = self.env['mail.template'].with_context(safe=True)._render_template(self.subject, 'survey.user_input', answer.id, post_process=True)
-        body = self.env['mail.template']._render_template(self.body, 'survey.user_input', answer.id, post_process=True)
+        subject = self._render_field('subject', answer.ids, options={'render_safe': True})[answer.id]
+        body = self._render_field('body', answer.ids, post_process=True)[answer.id]
         # post the message
         mail_values = {
             'email_from': self.email_from,
@@ -227,8 +241,8 @@ class SurveyInvite(models.TransientModel):
                     'model_description': self.env['ir.model']._get('survey.survey').display_name,
                     'company': self.env.company,
                 }
-                body = template.render(template_ctx, engine='ir.qweb', minimal_qcontext=True)
-                mail_values['body_html'] = self.env['mail.thread']._replace_local_links(body)
+                body = template._render(template_ctx, engine='ir.qweb', minimal_qcontext=True)
+                mail_values['body_html'] = self.env['mail.render.mixin']._replace_local_links(body)
 
         return self.env['mail.mail'].sudo().create(mail_values)
 
@@ -240,6 +254,9 @@ class SurveyInvite(models.TransientModel):
 
         # compute partners and emails, try to find partners for given emails
         valid_partners = self.partner_ids
+        langs = set(valid_partners.mapped('lang')) - {False}
+        if len(langs) == 1:
+            self = self.with_context(lang=langs.pop())
         valid_emails = []
         for email in emails_split.split(self.emails or ''):
             partner = False
