@@ -5,7 +5,7 @@ import itertools
 import pstats
 from cProfile import Profile
 
-from odoo import fields
+from odoo import fields, Command
 from odoo.tests import common
 
 
@@ -285,7 +285,7 @@ class test_selection(CreatorCase):
             [[u"Bar"]])
 
     def test_localized_export(self):
-        self.env['res.lang'].load_lang('fr_FR')
+        self.env['res.lang']._activate_lang('fr_FR')
         ir_field = self.env['ir.model.fields']._get('export.selection', 'value')
         selection = ir_field.selection_ids
         translations = dict(self.translations_fr)
@@ -323,7 +323,7 @@ class test_m2o(CreatorCase):
     def test_empty(self):
         self.assertEqual(
             self.export(False),
-            [[False]])
+            [['']])
 
     def test_basic(self):
         """ Exported value is the name_get of the related object
@@ -370,11 +370,11 @@ class test_m2o(CreatorCase):
 class test_o2m(CreatorCase):
     model_name = 'export.one2many'
     commands = [
-        (0, False, {'value': 4, 'str': 'record1'}),
-        (0, False, {'value': 42, 'str': 'record2'}),
-        (0, False, {'value': 36, 'str': 'record3'}),
-        (0, False, {'value': 4, 'str': 'record4'}),
-        (0, False, {'value': 13, 'str': 'record5'}),
+        Command.create({'value': 4, 'str': 'record1'}),
+        Command.create({'value': 42, 'str': 'record2'}),
+        Command.create({'value': 36, 'str': 'record3'}),
+        Command.create({'value': 4, 'str': 'record4'}),
+        Command.create({'value': 13, 'str': 'record5'}),
     ]
     names = [
         u'export.one2many.child:%d' % d['value']
@@ -384,23 +384,23 @@ class test_o2m(CreatorCase):
     def test_empty(self):
         self.assertEqual(
             self.export(False),
-            [[False]])
+            [['']])
 
     def test_single(self):
         self.assertEqual(
-            self.export([(0, False, {'value': 42})]),
+            self.export([Command.create({'value': 42})]),
             # name_get result
             [[u'export.one2many.child:42']])
 
     def test_single_subfield(self):
         self.assertEqual(
-            self.export([(0, False, {'value': 42})],
+            self.export([Command.create({'value': 42})],
                         fields=['value', 'value/value']),
             [[u'export.one2many.child:42', 42]])
 
     def test_integrate_one_in_parent(self):
         self.assertEqual(
-            self.export([(0, False, {'value': 42})],
+            self.export([Command.create({'value': 42})],
                         fields=['const', 'value/value']),
             [[4, 42]])
 
@@ -499,37 +499,37 @@ class test_o2m_multiple(CreatorCase):
     def test_empty(self):
         self.assertEqual(
             self.export(child1=False, child2=False),
-            [[False, False]])
+            [['', '']])
 
     def test_single_per_side(self):
         self.assertEqual(
-            self.export(child1=False, child2=[(0, False, {'value': 42})]),
-            [[False, u'export.one2many.child.2:42']])
+            self.export(child1=False, child2=[Command.create({'value': 42})]),
+            [['', u'export.one2many.child.2:42']])
 
         self.assertEqual(
-            self.export(child1=[(0, False, {'value': 43})], child2=False),
-            [[u'export.one2many.child.1:43', False]])
+            self.export(child1=[Command.create({'value': 43})], child2=False),
+            [[u'export.one2many.child.1:43', '']])
 
         self.assertEqual(
-            self.export(child1=[(0, False, {'value': 43})],
-                        child2=[(0, False, {'value': 42})]),
+            self.export(child1=[Command.create({'value': 43})],
+                        child2=[Command.create({'value': 42})]),
             [[u'export.one2many.child.1:43', u'export.one2many.child.2:42']])
 
     def test_single_integrate_subfield(self):
         fields = ['const', 'child1/value', 'child2/value']
         self.assertEqual(
-            self.export(child1=False, child2=[(0, False, {'value': 42})],
+            self.export(child1=False, child2=[Command.create({'value': 42})],
                         fields=fields),
-            [[36, False, 42]])
+            [[36, '', 42]])
 
         self.assertEqual(
-            self.export(child1=[(0, False, {'value': 43})], child2=False,
+            self.export(child1=[Command.create({'value': 43})], child2=False,
                         fields=fields),
-            [[36, 43, False]])
+            [[36, 43, '']])
 
         self.assertEqual(
-            self.export(child1=[(0, False, {'value': 43})],
-                        child2=[(0, False, {'value': 42})],
+            self.export(child1=[Command.create({'value': 43})],
+                        child2=[Command.create({'value': 42})],
                         fields=fields),
             [[36, 43, 42]])
 
@@ -538,15 +538,15 @@ class test_o2m_multiple(CreatorCase):
         exports the rows for the first o2m, then the rows for the second o2m.
         """
         fields = ['const', 'child1/value', 'child2/value']
-        child1 = [(0, False, {'value': v, 'str': 'record%.02d' % index})
+        child1 = [Command.create({'value': v, 'str': 'record%.02d' % index})
                   for index, v in zip(itertools.count(), [4, 42, 36, 4, 13])]
-        child2 = [(0, False, {'value': v, 'str': 'record%.02d' % index})
+        child2 = [Command.create({'value': v, 'str': 'record%.02d' % index})
                   for index, v in zip(itertools.count(10), [8, 12, 8, 55, 33, 13])]
 
         self.assertEqual(
             self.export(child1=child1, child2=False, fields=fields),
             [
-                [36, 4, False],
+                [36, 4, ''],
                 ['', 42, ''],
                 ['', 36, ''],
                 ['', 4, ''],
@@ -555,7 +555,7 @@ class test_o2m_multiple(CreatorCase):
         self.assertEqual(
             self.export(child1=False, child2=child2, fields=fields),
             [
-                [36, False, 8],
+                [36, '', 8],
                 ['', '', 12],
                 ['', '', 8],
                 ['', '', 55],
@@ -581,11 +581,11 @@ class test_o2m_multiple(CreatorCase):
 class test_m2m(CreatorCase):
     model_name = 'export.many2many'
     commands = [
-        (0, False, {'value': 4, 'str': 'record000'}),
-        (0, False, {'value': 42, 'str': 'record001'}),
-        (0, False, {'value': 36, 'str': 'record010'}),
-        (0, False, {'value': 4, 'str': 'record011'}),
-        (0, False, {'value': 13, 'str': 'record100'}),
+        Command.create({'value': 4, 'str': 'record000'}),
+        Command.create({'value': 42, 'str': 'record001'}),
+        Command.create({'value': 36, 'str': 'record010'}),
+        Command.create({'value': 4, 'str': 'record011'}),
+        Command.create({'value': 13, 'str': 'record100'}),
     ]
     names = [
         u'export.many2many.other:%d' % d['value']
@@ -595,29 +595,31 @@ class test_m2m(CreatorCase):
     def test_empty(self):
         self.assertEqual(
             self.export(False),
-            [[False]])
+            [['']])
 
     def test_single(self):
         self.assertEqual(
-            self.export([(0, False, {'value': 42})]),
+            self.export([Command.create({'value': 42})]),
             # name_get result
             [[u'export.many2many.other:42']])
 
     def test_single_subfield(self):
         self.assertEqual(
-            self.export([(0, False, {'value': 42})],
-                        fields=['value', 'value/value']),
+            self.export([Command.create({'value': 42})],
+                        fields=['value', 'value/value'],
+                        context={'import_compat': False}),
             [[u'export.many2many.other:42', 42]])
 
     def test_integrate_one_in_parent(self):
         self.assertEqual(
-            self.export([(0, False, {'value': 42})],
-                        fields=['const', 'value/value']),
+            self.export([Command.create({'value': 42})],
+                        fields=['const', 'value/value'],
+                        context={'import_compat': False}),
             [[4, 42]])
 
     def test_multiple_records(self):
         self.assertEqual(
-            self.export(self.commands, fields=['const', 'value/value']),
+            self.export(self.commands, fields=['const', 'value/value'], context={'import_compat': False}),
             [
                 [4, 4],
                 [u'', 42],
@@ -629,6 +631,12 @@ class test_m2m(CreatorCase):
     def test_multiple_records_name(self):
         self.assertEqual(
             self.export(self.commands, fields=['const', 'value']),
+            [
+                [4, 'export.many2many.other:4,export.many2many.other:42,export.many2many.other:36,export.many2many.other:4,export.many2many.other:13'],
+            ])
+
+        self.assertEqual(
+            self.export(self.commands, fields=['const', 'value'], context={'import_compat': False}),
             [
                 [4, u'export.many2many.other:4'],
                 ['', u'export.many2many.other:42'],
@@ -664,6 +672,10 @@ class test_m2m(CreatorCase):
             r.with_context(import_compat=True)._export_rows([['value', 'id']]),
             [['__t__.record000,__t__.record001,__t__.record010,__t__.record011,__t__.record100']]
         )
+        self.assertEqual(
+            r.with_context(import_compat=True)._export_rows([['value'], ['value', 'id']]),
+            [['', '__t__.record000,__t__.record001,__t__.record010,__t__.record011,__t__.record100']]
+        )
 
         self.assertEqual(
             r.with_context(import_compat=False)._export_rows([['id'], ['value', 'id'], ['value', 'value']]),
