@@ -6,6 +6,7 @@ import pkgutil
 import re
 
 from odoo import fields
+from odoo.addons.base.tests.common import SavepointCaseWithUserDemo
 from odoo.tests import common
 from odoo.tools.misc import mute_logger
 
@@ -39,8 +40,13 @@ class ImporterCase(common.TransactionCase):
         super(ImporterCase, self).setUp()
         self.model = self.env[self.model_name]
         self.env['ir.model.data'].clear_caches()
+        self.cr.cache.clear()
 
     def import_(self, fields, rows, context=None):
+        context = context or {}
+        context.update({
+            'import_file': True
+        })
         return self.model.with_context(context or {}).load(fields, rows)
 
     def read(self, fields=('value',), domain=(), context=None):
@@ -72,7 +78,7 @@ class ImporterCase(common.TransactionCase):
         return '__test__.' + name
 
     def add_translations(self, name, type, code, *tnx):
-        self.env['res.lang'].load_lang(code)
+        self.env['res.lang']._activate_lang(code)
         Translations = self.env['ir.translation']
         for source, value in tnx:
             Translations.create({
@@ -167,20 +173,23 @@ class test_boolean_field(ImporterCase):
         self.assertEqual([False] * len(falses), values(self.read()))
 
     def test_trues(self):
+        # Since importing wrong boolean values is now returning error, import should not return any ids if an error is raised.
         trues = [['None'], ['nil'], ['()'], ['f'], ['#f'],
                   # Problem: OpenOffice (and probably excel) output localized booleans
                   ['VRAI'], ['ok'], ['true'], ['yes'], ['1'], ]
         result = self.import_(['value'], trues)
-        self.assertEqual(len(result['ids']), 10)
-        self.assertEqual(result['messages'], [
-            message(u"Unknown value '%s' for boolean field 'Value', assuming 'yes'" % v[0],
-                    moreinfo=u"Use '1' for yes and '0' for no",
-                    type='warning', from_=i, to_=i, record=i)
-            for i, v in enumerate(trues)
-            if v[0] not in ('true', 'yes', '1')
-        ])
+        self.assertEqual(result['ids'], False)
+        self.assertEqual(result['messages'], [{
+            'rows': {'from': i, 'to': i}, 'type': 'error', 'record': i, 'field': 'value',
+            'message': "Unknown value '%s' for boolean field 'Value'" % v[0],
+            'moreinfo': "Use '1' for yes and '0' for no", 'field_name': 'Value'
+        } for i, v in enumerate(trues) if v[0] not in ('true', 'yes', '1')])
+
+        # Only correct boolean values are accepted.
+        result = self.import_(['value'], [['1'], ['yes'], ['true']])
+        self.assertEqual(len(result['ids']), 3)
         self.assertEqual(
-            [True] * 10,
+            [True] * 3,
             values(self.read()))
 
 
@@ -263,6 +272,7 @@ class test_integer_field(ImporterCase):
         result = self.import_(['value'], [['zorglub']])
         self.assertIs(result['ids'], False)
         self.assertEqual(result['messages'], [{
+            'field_name': 'Value',
             'type': 'error',
             'rows': {'from': 0, 'to': 0},
             'record': 0,
@@ -334,7 +344,7 @@ class test_float_field(ImporterCase):
         result = self.import_(['value'], [['foobar']])
         self.assertIs(result['ids'], False)
         self.assertEqual(result['messages'], [
-            message(u"'foobar' does not seem to be a number for field 'Value'")])
+            message(u"'foobar' does not seem to be a number for field 'Value'", field_name='Value')])
 
 class test_string_field(ImporterCase):
     model_name = 'export.string.bounded'
@@ -478,13 +488,14 @@ class test_selection(ImporterCase):
         self.assertIs(result['ids'], False)
         self.assertEqual(result['messages'], [message(
             u"Value 'Baz' not found in selection field 'Value'",
-            moreinfo="Foo Bar Qux 4".split())])
+            moreinfo="Foo Bar Qux 4".split(), field_name='Value', field_path=['value'])])
 
         result = self.import_(['value'], [['42']])
         self.assertIs(result['ids'], False)
         self.assertEqual(result['messages'], [message(
             u"Value '42' not found in selection field 'Value'",
-            moreinfo="Foo Bar Qux 4".split())])
+            moreinfo="Foo Bar Qux 4".split(),
+            field_name='Value', field_path=['value'])])
 
 
 class test_selection_with_default(ImporterCase):
@@ -566,12 +577,27 @@ class test_m2o(ImporterCase):
         name1 = dict(record1.name_get())[record1.id]
         name2 = dict(record2.name_get())[record2.id]
 
-        result = self.import_(['value'], [
-            # import by name_get
-            [name1],
-            [name1],
-            [name2],
-        ])
+        # preheat the oven
+        for _ in range(5):
+            self.env.cr.execute('SAVEPOINT xxx')
+            self.import_(['value'], [[name1], [name1], [name2]])
+            self.env.cr.execute('ROLLBACK TO SAVEPOINT xxx')
+            self.env.cr.execute('RELEASE SAVEPOINT xxx')
+
+        # 1 x SAVEPOINT model_load
+        # 3 x name_search
+        # 1 x SAVEPOINT
+        # 3 x insert
+        # 1 x RELEASE SAVEPOINT
+        # => 9
+        with self.assertQueryCount(9):
+            result = self.import_(['value'], [
+                # import by name_get
+                [name1],
+                [name1],
+                [name2],
+            ])
+
         self.assertFalse(result['messages'])
         self.assertEqual(len(result['ids']), 3)
         # correct ids assigned to corresponding records
@@ -610,7 +636,7 @@ class test_m2o(ImporterCase):
         result = self.import_(['value'], [[name2]])
         self.assertEqual(
             result['messages'],
-            [message(u"Found multiple matches for field 'Value' (2 matches)",
+            [message(u"Found multiple matches for value 'export.integer:42' in field 'Value' (2 matches)",
                      type='warning')])
         self.assertEqual(len(result['ids']), 1)
         self.assertEqual([
@@ -634,7 +660,8 @@ class test_m2o(ImporterCase):
         self.assertEqual(result['messages'], [
             message(u"No matching record found for name '%s' in field 'Value'" % id,
                     from_=index, to_=index, record=index,
-                    moreinfo=moreaction(res_model='export.integer'))
+                    moreinfo=moreaction(res_model='export.integer'), field_name='Value', field_path=['value'],
+                    field_type='name', value=id)
             for index, id in enumerate([record1.id, record2.id, record1.id])])
         self.assertIs(result['ids'], False)
 
@@ -645,7 +672,8 @@ class test_m2o(ImporterCase):
         self.assertEqual(result['messages'], [
             message(u"Invalid database id 'foo' for the field 'Value'",
                     moreinfo=moreaction(res_model='ir.model.data',
-                                        domain=[('model','=','export.integer')]))
+                                        domain=[('model', '=', 'export.integer')]),
+                    field_name='Value', field_path=['value', '.id'])
         ])
         self.assertIs(result['ids'], False)
 
@@ -664,21 +692,25 @@ class test_m2o(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name 'nameisnoexist:3' "
             u"in field 'Value'", moreinfo=moreaction(
-                res_model='export.integer'))])
+                res_model='export.integer'),
+            field_name='Value', field_path=['value'], field_type='name', value='nameisnoexist:3'
+        )])
         self.assertIs(result['ids'], False)
 
         result = self.import_(['value/id'], [['noxidhere']])
         self.assertEqual(result['messages'], [message(
             u"No matching record found for external id 'noxidhere' "
             u"in field 'Value'", moreinfo=moreaction(
-                res_model='ir.model.data', domain=[('model','=','export.integer')]))])
+                res_model='ir.model.data', domain=[('model', '=', 'export.integer')]),
+            field_name='Value', field_path=['value', 'id'], field_type="external id", value="noxidhere")])
         self.assertIs(result['ids'], False)
 
         result = self.import_(['value/.id'], [['66']])
         self.assertEqual(result['messages'], [message(
             u"No matching record found for database id '66' "
             u"in field 'Value'", moreinfo=moreaction(
-                res_model='ir.model.data', domain=[('model','=','export.integer')]))])
+                res_model='ir.model.data', domain=[('model', '=', 'export.integer')]),
+            field_name='Value', field_path=['value', '.id'], field_type="database id", value="66")])
         self.assertIs(result['ids'], False)
 
     def test_fail_multiple(self):
@@ -695,7 +727,8 @@ class test_m2o(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name '101' "
             u"in field 'Value'", moreinfo=moreaction(
-                res_model='export.integer'))])
+                res_model='export.integer'),
+            field_name='Value', field_path=['value'], field_type='name', value=101)])
         self.assertIs(result['ids'], False)
         context = {
             'name_create_enabled_fields': {'value': True},
@@ -704,6 +737,20 @@ class test_m2o(ImporterCase):
         self.assertFalse(result['messages'])
         self.assertEqual(len(result['ids']), 1)
 
+class TestInvalidStrings(ImporterCase):
+    model_name = 'export.m2o.str'
+
+    @mute_logger('odoo.sql_db')
+    def test_fail_unpaired_surrogate(self):
+        result = self.import_(['child_id'], [['\uddff']])
+        self.assertTrue(result['messages'])
+        self.assertIn('surrogates', result['messages'][0]['message'])
+
+    @mute_logger('odoo.sql_db')
+    def test_fail_nul(self):
+        result = self.import_(['child_id'], [['\x00']])
+        self.assertTrue(result['messages'])
+        self.assertIn('NUL', result['messages'][0]['message'])
 
 class test_m2m(ImporterCase):
     model_name = 'export.many2many'
@@ -741,7 +788,8 @@ class test_m2m(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for database id '42' in field "
             u"'Value'", moreinfo=moreaction(
-                res_model='ir.model.data', domain=[('model','=','export.many2many.other')]))])
+                res_model='ir.model.data', domain=[('model', '=', 'export.many2many.other')]),
+            field_name='Value', field_path=['value', '.id'], field_type="database id", value='42')])
         self.assertIs(result['ids'], False)
 
     def test_xids(self):
@@ -767,7 +815,8 @@ class test_m2m(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for external id 'noxidforthat' in field"
             u" 'Value'", moreinfo=moreaction(
-                res_model='ir.model.data', domain=[('model','=','export.many2many.other')]))])
+                res_model='ir.model.data', domain=[('model', '=', 'export.many2many.other')]),
+            field_name='Value', field_path=['value', 'id'], field_type='external id', value='noxidforthat')])
         self.assertIs(result['ids'], False)
 
     def test_names(self):
@@ -795,7 +844,8 @@ class test_m2m(ImporterCase):
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name 'wherethem2mhavenonames' in "
             u"field 'Value'", moreinfo=moreaction(
-                res_model='export.many2many.other'))])
+                res_model='export.many2many.other'),
+            field_name='Value', field_path=['value'], field_type="name", value='wherethem2mhavenonames')])
         self.assertIs(result['ids'], False)
 
     def test_import_to_existing(self):
@@ -828,8 +878,9 @@ class test_o2m(ImporterCase):
             ['const', 'value'],
             [['5', s]])
         self.assertEqual(result['messages'], [message(
-            u"No matching record found for name '%s' in field 'Value'" % s,
-            moreinfo=moreaction(res_model='export.one2many.child'))])
+            u"No matching record found for name '%s' in field 'Value'" % s[:50],
+            moreinfo=moreaction(res_model='export.one2many.child'),
+            field_name='Value', field_path=['value'], field_type='name', value=s[:50])])
         self.assertIs(result['ids'], False)
 
     def test_single(self):
@@ -886,6 +937,14 @@ class test_o2m(ImporterCase):
             values(b.value.sorted(), 'str'),
             'this is the rhythm'.split())
 
+    def test_subfields_fail_by_implicit_id(self):
+        result = self.import_(['value/parent_id'], [['noxidforthat']])
+        self.assertEqual(result['messages'], [message(
+            u"No matching record found for name 'noxidforthat' in field 'Value/Parent'",
+            moreinfo=moreaction(res_model='export.one2many'),
+            field_name='Value', field_path=['value', 'parent_id'], field_type='name', value='noxidforthat')])
+        self.assertIs(result['ids'], False)
+
     def test_link_inline(self):
         """ m2m-style specification for o2ms
         """
@@ -939,12 +998,43 @@ class test_o2m(ImporterCase):
         self.assertEqual(set(values(b.value)), set([1, 2]))
         self.assertEqual(values(b.value, field='parent_id'), [b, b])
 
+    def test_o2m_repeated_with_xids(self):
+        # concern: formerly this would link existing records, and fault if
+        # the records did not exist. This is more in line with other XID uses,
+        # however it does make thing work where they'd previously fail for
+        # well-defined reasons.
+        result = self.import_(['id', 'const', 'value/id', 'value/value'], [
+            ['a', '5', 'aa', '11'],
+            ['', '', 'ab', '12'],
+            ['', '', 'ac', '13'],
+            ['', '', 'ad', '14'],
+            ['b', '10', 'ba', '15'],
+            ['', '', 'bb', '16'],
+        ])
+        self.assertFalse(result['messages'])
+        result = self.import_(['id', 'const', 'value/id', 'value/value'], [
+            ['a', '5', 'aa', '11'],
+            ['', '', 'ab', '12'],
+            ['', '', 'ac', '13'],
+            ['', '', 'ad', '14'],
+            ['b', '8', 'ba', '25'],
+            ['', '', 'bb', '16'],
+        ])
+        self.assertFalse(result['messages'])
+
+        [a, b] = self.browse().sorted(lambda r: r.const)
+        self.assertEqual(len(a.value), 4)
+        self.assertEqual(len(b.value), 2)
+        self.assertEqual(b.const, 8)
+        self.assertEqual(b.value.mapped('value'), [25, 16])
+
     def test_name_create_enabled_m2o_in_o2m(self):
         result = self.import_(['value/m2o'], [[101]])
         self.assertEqual(result['messages'], [message(
             u"No matching record found for name '101' "
-            u"in field 'Value'", moreinfo=moreaction(
-                res_model='export.integer'))])
+            u"in field 'Value/M2O'", moreinfo=moreaction(
+                res_model='export.integer'),
+            field_name='Value', field_path=['value', 'm2o'], field_type='name', value=101)])
         self.assertEqual(result['ids'], False)
         context = {
             'name_create_enabled_fields': {'value/m2o': True},
@@ -1008,7 +1098,13 @@ class test_o2m_multiple(ImporterCase):
         self.assertEqual(set(values(b.child2)), set([21, 22, 23]))
 
 
-class test_realworld(common.TransactionCase):
+class test_realworld(SavepointCaseWithUserDemo):
+
+    @classmethod
+    def setUpClass(cls):
+        super(test_realworld, cls).setUpClass()
+        cls._load_partners_set()
+
     def test_bigfile(self):
         data = json.loads(pkgutil.get_data(self.__module__, 'contacts_big.json').decode('utf-8'))
         result = self.env['res.partner'].load(['name', 'mobile', 'email', 'image_1920'], data)
@@ -1053,6 +1149,19 @@ class test_realworld(common.TransactionCase):
         self.assertEqual([child.value for child in b[1].child.sorted()[1].child2],
                          [12])
 
+    def test_o2m_subfields_fail_by_implicit_id(self):
+        self.env['ir.model.data'].clear_caches()
+        Model = self.env['export.one2many.recursive']
+        result = Model.with_context(import_file=True).load(
+            ['child/child1/parent_id'],
+            [['5'],],
+        )
+        self.assertEqual(result['messages'], [message(
+            u"No matching record found for name '5' in field 'Child/Child1/Parent'", field='child',
+            moreinfo=moreaction(res_model='export.one2many.multiple'),
+            field_name='Child', field_path=['child', 'child1', 'parent_id'], field_type='name', value='5')])
+        self.assertIs(result['ids'], False)
+
 
 class test_date(ImporterCase):
     model_name = 'export.date'
@@ -1072,7 +1181,7 @@ class test_date(ImporterCase):
         self.assertEqual(result['messages'], [
             message(u"'not really a date' does not seem to be a valid date "
                     u"for field 'Value'",
-                    moreinfo=u"Use the format '2012-12-31'")])
+                    moreinfo=u"Use the format '2012-12-31'", field_name='Value', field_path=['value'])])
         self.assertIs(result['ids'], False)
 
 
@@ -1094,7 +1203,7 @@ class test_datetime(ImporterCase):
         self.assertEqual(result['messages'], [
             message(u"'not really a datetime' does not seem to be a valid "
                     u"datetime for field 'Value'",
-                    moreinfo=u"Use the format '2012-12-31 23:59:59'")])
+                    moreinfo=u"Use the format '2012-12-31 23:59:59'", field_name='Value', field_path=['value'])])
         self.assertIs(result['ids'], False)
 
     def test_checktz1(self):
@@ -1200,3 +1309,159 @@ class test_unique(ImporterCase):
             m.group(2).split(', '),
             ['Value2', 'Value3']
         )
+
+class test_inherits(ImporterCase):
+    """ The import process should only assign a new xid (derived from the
+    childs') if the child is being created and triggers the creation of the
+    parent
+    """
+    model_name = 'export.inherits.child'
+    def test_create_no_parent(self):
+        r = self.import_(['id', 'value_parent', 'value'], [
+            ['xxx.child', '0', '1'],
+        ])
+        rec = self.env[self.model_name].browse(r['ids'])
+
+        self.assertEqual(rec.value_parent, 0)
+        self.assertEqual(rec.value, 1)
+        self.assertEqual(rec.parent_id.value_parent, 0)
+        self.assertEqual(
+            rec._get_external_ids()[rec.id],
+            ['xxx.child'],
+        )
+        self.assertEqual(
+            rec.parent_id._get_external_ids()[rec.parent_id.id],
+            ['xxx.child_export_inherits_parent'],
+        )
+
+    def test_create_parent_no_xid(self):
+        parent = self.env['export.inherits.parent'].create({'value_parent': 0})
+        r = self.import_(['id', 'parent_id/.id', 'value'], [
+            ['xxx.child', str(parent.id), '1'],
+        ])
+        rec = self.env[self.model_name].browse(r['ids'])
+        self.assertEqual(rec.value_parent, 0)
+        self.assertEqual(rec.parent_id, parent)
+
+        self.assertEqual(
+            rec._get_external_ids()[rec.id],
+            ['xxx.child'],
+        )
+        self.assertEqual(
+            rec.parent_id._get_external_ids()[rec.parent_id.id],
+            [],
+            "no xid should be created for the parent"
+        )
+
+    def test_create_parent_with_xid(self):
+        parent = self.env['export.inherits.parent'].create({'value_parent': 0})
+        pid = self.env['ir.model.data'].create({
+            'model': 'export.inherits.parent',
+            'res_id': parent.id,
+            'module': 'xxx',
+            'name': 'parent',
+        })
+        r = self.import_(['id', 'parent_id/.id', 'value'], [
+            ['xxx.child', str(parent.id), '1'],
+        ])
+        rec = self.env[self.model_name].browse(r['ids'])
+        self.assertEqual(rec.value_parent, 0)
+        self.assertEqual(rec.parent_id, parent)
+        self.assertTrue(pid.exists().res_id, parent.id)
+
+        self.assertEqual(
+            rec._get_external_ids()[rec.id],
+            ['xxx.child'],
+        )
+        self.assertEqual(
+            rec.parent_id._get_external_ids()[rec.parent_id.id],
+            ['xxx.parent'],
+        )
+
+    def test_create_parent_by_xid(self):
+        parent = self.env['export.inherits.parent'].create({'value_parent': 0})
+        pid = self.env['ir.model.data'].create({
+            'model': 'export.inherits.parent',
+            'res_id': parent.id,
+            'module': 'xxx',
+            'name': 'parent',
+        })
+        r = self.import_(['id', 'parent_id/id', 'value'], [
+            ['xxx.child', 'xxx.parent', '1'],
+        ])
+        rec = self.env[self.model_name].browse(r['ids'])
+        self.assertEqual(rec.value_parent, 0)
+        self.assertEqual(rec.parent_id, parent)
+        self.assertTrue(pid.exists().res_id, parent.id)
+
+        self.assertEqual(
+            rec._get_external_ids()[rec.id],
+            ['xxx.child'],
+        )
+        self.assertEqual(
+            rec.parent_id._get_external_ids()[rec.parent_id.id],
+            ['xxx.parent'],
+        )
+
+    def test_update_parent_no_xid(self):
+        parent = self.env['export.inherits.parent'].create({'value_parent': 0})
+        child = self.env[self.model_name].create({
+            'parent_id': parent.id,
+            'value': 1,
+        })
+        self.env['ir.model.data'].create({
+            'model': self.model_name,
+            'res_id': child.id,
+            'module': 'xxx',
+            'name': 'child'
+        })
+
+        self.import_(['id', 'value'], [
+            ['xxx.child', '42']
+        ])
+        self.assertEqual(child.value, 42)
+        self.assertEqual(child.parent_id, parent)
+
+        self.assertEqual(
+            child._get_external_ids()[child.id],
+            ['xxx.child'],
+        )
+        self.assertEqual(
+            parent._get_external_ids()[parent.id],
+            [],
+        )
+
+    def test_update_parent_with_xid(self):
+        parent = self.env['export.inherits.parent'].create({'value_parent': 0})
+        child = self.env[self.model_name].create({
+            'parent_id': parent.id,
+            'value': 1,
+        })
+        pid, cid = self.env['ir.model.data'].create([{
+            'model': 'export.inherits.parent',
+            'res_id': parent.id,
+            'module': 'xxx',
+            'name': 'parent',
+        }, {
+            'model': self.model_name,
+            'res_id': child.id,
+            'module': 'xxx',
+            'name': 'child'
+        }])
+
+        self.import_(['id', 'value'], [
+            ['xxx.child', '42']
+        ])
+        self.assertEqual(child.value, 42)
+        self.assertEqual(child.parent_id, parent)
+        self.assertEqual(pid.exists().res_id, parent.id)
+        self.assertEqual(cid.exists().res_id, child.id)
+
+        self.assertEqual(
+            child._get_external_ids()[child.id],
+            ['xxx.child'],
+        )
+        self.assertEqual(
+            parent._get_external_ids()[parent.id],
+            ['xxx.parent'],
+        )
