@@ -7,18 +7,17 @@
     'sequence': 9876,
     'summary': 'Mail Tests: performances and tests specific to mail',
     'description': """This module contains tests related to mail. Those are
-contained in a separate module as it contains models used only to perform
+present in a separate module as it contains models used only to perform
 tests independently to functional aspects of other models. """,
-    'depends': ['test_performance', 'mail', 'mail_bot'],
+    'depends': ['test_performance', 'mail'],
     'data': [
         'security/ir.model.access.csv',
         'security/test_mail_security.xml',
         'data/data.xml',
+        'data/mail_template_data.xml',
+        'data/subtype_data.xml',
     ],
     'demo': [
-        'data/demo.xml',
-        'data/subtype_demo.xml',
-        'data/template_demo.xml',
     ],
     'installable': True,
     'application': False,
