@@ -28,6 +28,29 @@ class MailTestGateway(models.Model):
     custom_field = fields.Char()
 
 
+class MailTestGatewayGroups(models.Model):
+    """ A model looking like discussion channels / groups (flat thread and
+    alias). Used notably for advanced gatewxay tests. """
+    _description = 'Channel/Group-like Chatter Model for Mail Gateway'
+    _name = 'mail.test.gateway.groups'
+    _inherit = ['mail.thread.blacklist', 'mail.alias.mixin']
+    _mail_flat_thread = False
+    _primary_email = 'email_from'
+
+    name = fields.Char()
+    email_from = fields.Char()
+    custom_field = fields.Char()
+    customer_id = fields.Many2one('res.partner', 'Customer')
+
+    def _alias_get_creation_values(self):
+        values = super(MailTestGatewayGroups, self)._alias_get_creation_values()
+        values['alias_model_id'] = self.env['ir.model']._get('mail.test.gateway.groups').id
+        if self.id:
+            values['alias_force_thread_id'] = self.id
+            values['alias_parent_thread_id'] = self.id
+        return values
+
+
 class MailTestStandard(models.Model):
     """ This model can be used in tests when automatic subscription and simple
     tracking is necessary. Most features are present in a simple way. """
@@ -38,9 +61,14 @@ class MailTestStandard(models.Model):
     name = fields.Char()
     email_from = fields.Char()
     user_id = fields.Many2one('res.users', 'Responsible', tracking=True)
-    umbrella_id = fields.Many2one('mail.test', tracking=True)
+    container_id = fields.Many2one('mail.test.container', tracking=True)
     company_id = fields.Many2one('res.company')
 
+    def _get_share_url(self, redirect, signup_partner, share_token):
+        """This function is required for a test on 'mail.mail_notification_paynow' template (test_message_post/test_mail_add_signature),
+        another model should be created in master"""
+        return '/mail/view'
+
 
 class MailTestActivity(models.Model):
     """ This model can be used to test activities in addition to simple chatter
@@ -50,6 +78,7 @@ class MailTestActivity(models.Model):
     _inherit = ['mail.thread', 'mail.activity.mixin']
 
     name = fields.Char()
+    date = fields.Date()
     email_from = fields.Char()
     active = fields.Boolean(default=True)
 
@@ -63,11 +92,11 @@ class MailTestActivity(models.Model):
         self.activity_feedback(['test_mail.mail_act_test_todo'], feedback=action_feedback)
 
 
-class MailTestFull(models.Model):
+class MailTestTicket(models.Model):
     """ This model can be used in tests when complex chatter features are
     required like modeling tasks or tickets. """
-    _description = 'Full Chatter Model'
-    _name = 'mail.test.full'
+    _description = 'Ticket-like model'
+    _name = 'mail.test.ticket'
     _inherit = ['mail.thread']
 
     name = fields.Char()
@@ -77,34 +106,34 @@ class MailTestFull(models.Model):
     mail_template = fields.Many2one('mail.template', 'Template')
     customer_id = fields.Many2one('res.partner', 'Customer', tracking=2)
     user_id = fields.Many2one('res.users', 'Responsible', tracking=1)
-    umbrella_id = fields.Many2one('mail.test', tracking=True)
+    container_id = fields.Many2one('mail.test.container', tracking=True)
 
     def _track_template(self, changes):
-        res = super(MailTestFull, self)._track_template(changes)
+        res = super(MailTestTicket, self)._track_template(changes)
         record = self[0]
         if 'customer_id' in changes and record.mail_template:
             res['customer_id'] = (record.mail_template, {'composition_mode': 'mass_mail'})
         elif 'datetime' in changes:
-            res['datetime'] = ('test_mail.mail_test_full_tracking_view', {'composition_mode': 'mass_mail'})
+            res['datetime'] = ('test_mail.mail_test_ticket_tracking_view', {'composition_mode': 'mass_mail'})
         return res
 
     def _creation_subtype(self):
-        if self.umbrella_id:
-            return self.env.ref('test_mail.st_mail_test_full_umbrella_upd')
-        return super(MailTestFull, self)._creation_subtype()
+        if self.container_id:
+            return self.env.ref('test_mail.st_mail_test_ticket_container_upd')
+        return super(MailTestTicket, self)._creation_subtype()
 
     def _track_subtype(self, init_values):
         self.ensure_one()
-        if 'umbrella_id' in init_values and self.umbrella_id:
-            return self.env.ref('test_mail.st_mail_test_full_umbrella_upd')
-        return super(MailTestFull, self)._track_subtype(init_values)
+        if 'container_id' in init_values and self.container_id:
+            return self.env.ref('test_mail.st_mail_test_ticket_container_upd')
+        return super(MailTestTicket, self)._track_subtype(init_values)
 
 
-class MailTestAlias(models.Model):
-    """ This model can be used in tests when umbrella records like projects
+class MailTestContainer(models.Model):
+    """ This model can be used in tests when container records like projects
     or teams are required. """
-    _description = 'Alias Chatter Model'
-    _name = 'mail.test'
+    _description = 'Project-like model with alias'
+    _name = 'mail.test.container'
     _mail_post_access = 'read'
     _inherit = ['mail.thread', 'mail.alias.mixin']
 
@@ -115,47 +144,24 @@ class MailTestAlias(models.Model):
         'mail.alias', 'Alias',
         delegate=True)
 
-    def get_alias_model_name(self, vals):
-        return vals.get('alias_model', 'mail.test')
+    def _alias_get_creation_values(self):
+        values = super(MailTestContainer, self)._alias_get_creation_values()
+        values['alias_model_id'] = self.env['ir.model']._get('mail.test.container').id
+        if self.id:
+            values['alias_force_thread_id'] = self.id
+            values['alias_parent_thread_id'] = self.id
+        return values
 
-    def get_alias_values(self):
-        self.ensure_one()
-        res = super(MailTestAlias, self).get_alias_values()
-        res['alias_force_thread_id'] = self.id
-        res['alias_parent_thread_id'] = self.id
-        return res
+class MailTestComposerMixin(models.Model):
+    """ A simple invite-like wizard using the composer mixin, rendering on
+    itself. """
+    _description = 'Invite-like Wizard'
+    _name = 'mail.test.composer.mixin'
+    _inherit = ['mail.composer.mixin']
 
+    name = fields.Char('Name')
+    author_id = fields.Many2one('res.partner')
+    description = fields.Html('Description', render_engine="qweb", render_options={"post_process": True}, sanitize=False)
 
-class MailModel(models.Model):
-    _name = 'test_performance.mail'
-    _description = 'Test Performance Mail'
-    _inherit = 'mail.thread'
-
-    name = fields.Char()
-    value = fields.Integer()
-    value_pc = fields.Float(compute="_value_pc", store=True)
-    track = fields.Char(default='test', tracking=True)
-    partner_id = fields.Many2one('res.partner', string='Customer')
-
-    @api.depends('value')
-    def _value_pc(self):
-        for record in self:
-            record.value_pc = float(record.value) / 100
-
-
-class MailCC(models.Model):
-    _name = 'mail.test.cc'
-    _description = "Test Email CC Thread"
-    _inherit = ['mail.thread.cc']
-
-    name = fields.Char()
-
-
-class MailMultiCompany(models.Model):
-    """ This model can be used in multi company tests"""
-    _name = 'mail.test.multi.company'
-    _description = "Test Multi Company Mail"
-    _inherit = 'mail.thread'
-
-    name = fields.Char()
-    company_id = fields.Many2one('res.company')
+    def _compute_render_model(self):
+        self.render_model = self._name
