@@ -2,31 +2,30 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.addons.base.tests.test_ir_actions import TestServerActionsBase
-from odoo.addons.test_mail.tests.common import BaseFunctionalTest
+from odoo.addons.test_mail.tests.common import TestMailCommon
 
 
-class TestServerActionsEmail(BaseFunctionalTest, TestServerActionsBase):
+class TestServerActionsEmail(TestMailCommon, TestServerActionsBase):
 
     def test_action_email(self):
         email_template = self._create_template('res.partner', {'partner_to': '%s' % self.test_partner.id})
         self.action.write({'state': 'email', 'template_id': email_template.id})
         self.action.with_context(self.context).run()
         # check an email is waiting for sending
-        mail = self.env['mail.mail'].search([('subject', '=', 'About TestingPartner')])
+        mail = self.env['mail.mail'].sudo().search([('subject', '=', 'About TestingPartner')])
         self.assertEqual(len(mail), 1)
         # check email content
         self.assertEqual(mail.body, '<p>Hello TestingPartner</p>')
 
     def test_action_followers(self):
         self.test_partner.message_unsubscribe(self.test_partner.message_partner_ids.ids)
+        random_partner = self.env['res.partner'].create({'name': 'Thierry Wololo'})
         self.action.write({
             'state': 'followers',
-            'partner_ids': [(4, self.env.ref('base.partner_admin').id), (4, self.env.ref('base.partner_demo').id)],
-            'channel_ids': [(4, self.env.ref('mail.channel_all_employees').id)]
+            'partner_ids': [(4, self.env.ref('base.partner_admin').id), (4, random_partner.id)],
         })
         self.action.with_context(self.context).run()
-        self.assertEqual(self.test_partner.message_partner_ids, self.env.ref('base.partner_admin') | self.env.ref('base.partner_demo'))
-        self.assertEqual(self.test_partner.message_channel_ids, self.env.ref('mail.channel_all_employees'))
+        self.assertEqual(self.test_partner.message_partner_ids, self.env.ref('base.partner_admin') | random_partner)
 
     def test_action_next_activity(self):
         self.action.write({
