@@ -1,24 +1,26 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+
 import base64
-
 from unittest.mock import patch
-from odoo.addons.test_mail.tests import common
-from odoo.addons.test_mail.tests.common import mail_new_test_user
+
+from odoo.addons.mail.tests.common import mail_new_test_user
+from odoo.addons.test_mail.tests.common import TestMailCommon
 from odoo.addons.test_mail.models.test_mail_models import MailTestSimple
-from odoo.exceptions import AccessError, except_orm
+from odoo.exceptions import AccessError
 from odoo.tools import mute_logger, formataddr
-from odoo.tests import tagged
+from odoo.tests import tagged, users
 
 
-class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
+@tagged('mail_message')
+class TestMessageValues(TestMailCommon):
 
     @classmethod
     def setUpClass(cls):
         super(TestMessageValues, cls).setUpClass()
 
         cls._init_mail_gateway()
-        cls.alias_record = cls.env['mail.test'].with_context(cls._test_context).create({
+        cls.alias_record = cls.env['mail.test.container'].with_context(cls._test_context).create({
             'name': 'Pigs',
             'alias_name': 'pigs',
             'alias_contact': 'followers',
@@ -26,7 +28,112 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         cls.Message = cls.env['mail.message'].with_user(cls.user_employee)
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_no_document_values(self):
+    def test_mail_message_format(self):
+        record1 = self.env['mail.test.simple'].create({'name': 'Test1'})
+        message = self.env['mail.message'].create([{
+            'model': 'mail.test.simple',
+            'res_id': record1.id,
+        }])
+        res = message.message_format()
+        self.assertEqual(res[0].get('record_name'), 'Test1')
+
+        record1.write({"name": "Test2"})
+        res = message.message_format()
+        self.assertEqual(res[0].get('record_name'), 'Test2')
+
+    @mute_logger('odoo.models.unlink')
+    def test_mail_message_format_access(self):
+        """
+        User that doesn't have access to a record should still be able to fetch
+        the record_name inside message_format.
+        """
+        company_2 = self.env['res.company'].create({'name': 'Second Test Company'})
+        record1 = self.env['mail.test.multi.company'].create({
+            'name': 'Test1',
+            'company_id': company_2.id,
+        })
+        message = record1.message_post(body='', partner_ids=[self.user_employee.partner_id.id])
+        # We need to flush and invalidate the ORM cache since the record_name
+        # is already cached from the creation. Otherwise it will leak inside
+        # message_format.
+        message.flush()
+        message.invalidate_cache()
+        res = message.with_user(self.user_employee).message_format()
+        self.assertEqual(res[0].get('record_name'), 'Test1')
+
+    def test_mail_message_values_body_base64_image(self):
+        msg = self.env['mail.message'].with_user(self.user_employee).create({
+            'body': 'taratata <img src="data:image/png;base64,iV/+OkI=" width="2"> <img src="data:image/png;base64,iV/+OkI=" width="2">',
+        })
+        self.assertEqual(len(msg.attachment_ids), 1)
+        self.assertEqual(
+            msg.body,
+            '<p>taratata <img src="/web/image/{attachment.id}?access_token={attachment.access_token}" alt="image0" width="2"> '
+            '<img src="/web/image/{attachment.id}?access_token={attachment.access_token}" alt="image0" width="2"></p>'.format(attachment=msg.attachment_ids[0])
+        )
+
+    @mute_logger('odoo.models.unlink')
+    @users('employee')
+    def test_mail_message_values_fromto_long_name(self):
+        """ Long headers may break in python if above 78 chars as folding is not
+        done correctly (see ``_notify_get_reply_to_formatted_email`` docstring
+        + commit linked to this test). """
+        # name would make it blow up: keep only email
+        test_record = self.env['mail.test.container'].browse(self.alias_record.ids)
+        test_record.write({
+            'name': 'Super Long Name That People May Enter "Even with an internal quoting of stuff"'
+        })
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        reply_to_email = f"{test_record.alias_name}@{self.alias_domain}"
+        self.assertEqual(msg.reply_to, reply_to_email,
+                         'Reply-To: use only email when formataddr > 78 chars')
+
+        # name + company_name would make it blow up: keep record_name in formatting
+        test_record.write({'name': 'Name that would be more than 78 with company name'})
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        self.assertEqual(msg.reply_to, formataddr((test_record.name, reply_to_email)),
+                         'Reply-To: use recordname as name in format if recordname + company > 78 chars')
+
+        # no record_name: keep company_name in formatting if ok
+        test_record.write({'name': ''})
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        self.assertEqual(msg.reply_to, formataddr((self.env.user.company_id.name, reply_to_email)),
+                         'Reply-To: use company as name in format when no record name and still < 78 chars')
+
+        # no record_name and company_name make it blow up: keep only email
+        self.env.user.company_id.write({'name': 'Super Long Name That People May Enter "Even with an internal quoting of stuff"'})
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        self.assertEqual(msg.reply_to, reply_to_email,
+                         'Reply-To: use only email when formataddr > 78 chars')
+
+        # whatever the record and company names, email is too long: keep only email
+        test_record.write({
+            'alias_name': 'Waaaay too long alias name that should make any reply-to blow the 78 characters limit',
+            'name': 'Short',
+        })
+        self.env.user.company_id.write({'name': 'Comp'})
+        sanitized_alias_name = 'waaaay-too-long-alias-name-that-should-make-any-reply-to-blow-the-78-characters-limit'
+        msg = self.env['mail.message'].create({
+            'model': test_record._name,
+            'res_id': test_record.id
+        })
+        self.assertEqual(msg.reply_to, f"{sanitized_alias_name}@{self.alias_domain}",
+                         'Reply-To: even a long email is ok as only formataddr is problematic')
+
+    @mute_logger('odoo.models.unlink')
+    def test_mail_message_values_fromto_no_document_values(self):
         msg = self.Message.create({
             'reply_to': 'test.reply@example.com',
             'email_from': 'test.from@example.com',
@@ -36,7 +143,7 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         self.assertEqual(msg.email_from, 'test.from@example.com')
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_no_document(self):
+    def test_mail_message_values_fromto_no_document(self):
         msg = self.Message.create({})
         self.assertIn('-private', msg.message_id.split('@')[0], 'mail_message: message_id for a void message should be a "private" one')
         reply_to_name = self.env.user.company_id.name
@@ -62,22 +169,22 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_document_alias(self):
+    def test_mail_message_values_fromto_document_alias(self):
         msg = self.Message.create({
-            'model': 'mail.test',
+            'model': 'mail.test.container',
             'res_id': self.alias_record.id
         })
         self.assertIn('-openerp-%d-mail.test' % self.alias_record.id, msg.message_id.split('@')[0])
         reply_to_name = '%s %s' % (self.env.user.company_id.name, self.alias_record.name)
         reply_to_email = '%s@%s' % (self.alias_record.alias_name, self.alias_domain)
         self.assertEqual(msg.reply_to, formataddr((reply_to_name, reply_to_email)))
-        self.assertEqual(msg.email_from, '"%s" <%s>' % (self.user_employee.name, self.user_employee.email))
+        self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
         # no alias domain -> author
         self.env['ir.config_parameter'].search([('key', '=', 'mail.catchall.domain')]).unlink()
 
         msg = self.Message.create({
-            'model': 'mail.test',
+            'model': 'mail.test.container',
             'res_id': self.alias_record.id
         })
         self.assertIn('-openerp-%d-mail.test' % self.alias_record.id, msg.message_id.split('@')[0])
@@ -89,7 +196,7 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         self.env['ir.config_parameter'].search([('key', '=', 'mail.catchall.alias')]).unlink()
 
         msg = self.Message.create({
-            'model': 'mail.test',
+            'model': 'mail.test.container',
             'res_id': self.alias_record.id
         })
         self.assertIn('-openerp-%d-mail.test' % self.alias_record.id, msg.message_id.split('@')[0])
@@ -99,7 +206,7 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_document_no_alias(self):
+    def test_mail_message_values_fromto_document_no_alias(self):
         test_record = self.env['mail.test.simple'].create({'name': 'Test', 'email_from': 'ignasse@example.com'})
 
         msg = self.Message.create({
@@ -113,7 +220,7 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
     @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_document_manual_alias(self):
+    def test_mail_message_values_fromto_document_manual_alias(self):
         test_record = self.env['mail.test.simple'].create({'name': 'Test', 'email_from': 'ignasse@example.com'})
         alias = self.env['mail.alias'].create({
             'alias_name': 'MegaLias',
@@ -134,27 +241,19 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         self.assertEqual(msg.reply_to, formataddr((reply_to_name, reply_to_email)))
         self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
-    def test_mail_message_values_no_auto_thread(self):
+    def test_mail_message_values_fromto_reply_to_force_new(self):
         msg = self.Message.create({
-            'model': 'mail.test',
+            'model': 'mail.test.container',
             'res_id': self.alias_record.id,
-            'no_auto_thread': True,
+            'reply_to_force_new': True,
         })
         self.assertIn('reply_to', msg.message_id.split('@')[0])
-        self.assertNotIn('mail.test', msg.message_id.split('@')[0])
+        self.assertNotIn('mail.test.container', msg.message_id.split('@')[0])
         self.assertNotIn('-%d-' % self.alias_record.id, msg.message_id.split('@')[0])
 
-    def test_mail_message_base64_image(self):
-        msg = self.env['mail.message'].with_user(self.user_employee).create({
-            'body': 'taratata <img src="data:image/png;base64,iV/+OkI=" width="2"> <img src="data:image/png;base64,iV/+OkI=" width="2">',
-        })
-        self.assertEqual(len(msg.attachment_ids), 1)
-        body = '<p>taratata <img src="/web/image/%s?access_token=%s" alt="image0" width="2"> <img src="/web/image/%s?access_token=%s" alt="image0" width="2"></p>'
-        body = body % (msg.attachment_ids[0].id, msg.attachment_ids[0].access_token, msg.attachment_ids[0].id, msg.attachment_ids[0].access_token)
-        self.assertEqual(msg.body, body)
 
-
-class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
+@tagged('mail_message')
+class TestMessageAccess(TestMailCommon):
 
     @classmethod
     def setUpClass(cls):
@@ -243,15 +342,12 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
 
     @mute_logger('odoo.addons.base.models.ir_model', 'odoo.models')
     def test_mail_message_access_read_crash(self):
-        # TODO: Change the except_orm to Warning ( Because here it's call check_access_rule
-        # which still generate exception in except_orm.So we need to change all
-        # except_orm to warning in mail module.)
-        with self.assertRaises(except_orm):
+        with self.assertRaises(AccessError):
             self.message.with_user(self.user_employee).read()
 
     @mute_logger('odoo.models')
     def test_mail_message_access_read_crash_portal(self):
-        with self.assertRaises(except_orm):
+        with self.assertRaises(AccessError):
             self.message.with_user(self.user_portal).read(['body', 'message_type', 'subtype_id'])
 
     def test_mail_message_access_read_ok_portal(self):
@@ -280,12 +376,6 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         # Test: Bert reads the message, ok because linked to a doc he is allowed to read
         self.message.with_user(self.user_employee).read()
 
-    def test_mail_message_access_read_crash_moderation(self):
-        # with self.assertRaises(AccessError):
-        self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
-        # Test: Bert reads the message, ok because linked to a doc he is allowed to read
-        self.message.with_user(self.user_employee).read()
-
     # --------------------------------------------------
     # CREATE
     # --------------------------------------------------
@@ -303,23 +393,24 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
     @mute_logger('odoo.models')
     def test_mail_message_access_create_crash(self):
         # Do: Bert create a private message -> ko, no creation rights
-        with self.assertRaises(except_orm):
+        with self.assertRaises(AccessError):
             self.env['mail.message'].with_user(self.user_employee).create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test'})
 
     @mute_logger('odoo.models')
     def test_mail_message_access_create_doc(self):
-        # TODO Change the except_orm to Warning
         Message = self.env['mail.message'].with_user(self.user_employee)
         # Do: Raoul creates a message on Jobs -> ok, write access to the related document
         Message.create({'model': 'mail.channel', 'res_id': self.group_public.id, 'body': 'Test'})
         # Do: Raoul creates a message on Priv -> ko, no write access to the related document
-        with self.assertRaises(except_orm):
+        with self.assertRaises(AccessError):
             Message.create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test'})
 
     def test_mail_message_access_create_private(self):
         self.env['mail.message'].with_user(self.user_employee).create({'body': 'Test'})
 
     def test_mail_message_access_create_reply(self):
+        # TDE FIXME: should it really work ? not sure - catchall makes crash (aka, post will crash also)
+        self.env['ir.config_parameter'].set_param('mail.catchall.domain', False)
         self.message.write({'partner_ids': [(4, self.user_employee.partner_id.id)]})
         self.env['mail.message'].with_user(self.user_employee).create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test', 'parent_id': self.message.id})
 
@@ -327,15 +418,21 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         """ Purpose is to test posting a message on a record whose first message / parent
         is not accessible by current user. """
         test_record = self.env['mail.test.simple'].with_context(self._test_context).create({'name': 'Test', 'email_from': 'ignasse@example.com'})
+        partner_1 = self.env['res.partner'].create({
+            'name': 'Jitendra Prajapati (jpr-odoo)',
+            'email': 'jpr@odoo.com',
+        })
+        test_record.message_subscribe((partner_1 | self.user_admin.partner_id).ids)
+
         message = test_record.message_post(
             body='<p>This is First Message</p>', subject='Subject',
-            message_type='comment', subtype='mail.mt_note')
+            message_type='comment', subtype_xmlid='mail.mt_note')
         # portal user have no rights to read the message
-        with self.assertRaises(except_orm):
+        with self.assertRaises(AccessError):
             message.with_user(self.user_portal).read(['subject, body'])
 
         with patch.object(MailTestSimple, 'check_access_rights', return_value=True):
-            with self.assertRaises(except_orm):
+            with self.assertRaises(AccessError):
                 message.with_user(self.user_portal).read(['subject, body'])
 
             # parent message is accessible to references notification mail values
@@ -344,175 +441,14 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
                 body='<p>This is Second Message</p>',
                 subject='Subject',
                 parent_id=message.id,
-                partner_ids=[self.user_admin.partner_id.id],
                 message_type='comment',
-                subtype='mail.mt_comment',
+                subtype_xmlid='mail.mt_comment',
                 mail_auto_delete=False)
 
-        new_mail = self.env['mail.mail'].search([
+        new_mail = self.env['mail.mail'].sudo().search([
             ('mail_message_id', '=', new_msg.id),
-            ('references', '=', message.message_id),
+            ('references', '=', f'{message.message_id} {new_msg.message_id}'),
         ])
 
         self.assertTrue(new_mail)
         self.assertEqual(new_msg.parent_id, message)
-
-    # --------------------------------------------------
-    # WRITE
-    # --------------------------------------------------
-
-    def test_mail_message_access_write_moderation(self):
-        """ Only moderators can modify pending messages """
-        self.group_public.write({
-            'email_send': True,
-            'moderation': True,
-            'channel_partner_ids': [(4, self.partner_employee.id)],
-            'moderator_ids': [(4, self.user_employee.id)],
-        })
-        self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
-        self.message.with_user(self.user_employee).write({'moderation_status': 'accepted'})
-
-    def test_mail_message_access_write_crash_moderation(self):
-        self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
-        with self.assertRaises(AccessError):
-            self.message.with_user(self.user_employee).write({'moderation_status': 'accepted'})
-
-    @mute_logger('openerp.addons.mail.models.mail_mail')
-    def test_mark_all_as_read(self):
-        self.user_employee.notification_type = 'inbox'
-        emp_partner = self.user_employee.partner_id.with_user(self.user_employee)
-
-        group_private = self.env['mail.channel'].with_context({
-            'mail_create_nolog': True,
-            'mail_create_nosubscribe': True,
-            'mail_channel_noautofollow': True,
-        }).create({
-            'name': 'Private',
-            'description': 'Private James R.',
-            'public': 'private',
-            'alias_name': 'private',
-            'alias_contact': 'followers'}
-        ).with_context({'mail_create_nosubscribe': False})
-
-        # mark all as read clear needactions
-        msg1 = group_private.message_post(body='Test', message_type='comment', subtype='mail.mt_comment', partner_ids=[emp_partner.id])
-        self._clear_bus()
-        emp_partner.env['mail.message'].mark_all_as_read(domain=[])
-        self.assertBusNotification([(self.cr.dbname, 'res.partner', emp_partner.id)], [{ 'type': 'mark_as_read', 'message_ids': [msg1.id] }])
-        na_count = emp_partner.get_needaction_count()
-        self.assertEqual(na_count, 0, "mark all as read should conclude all needactions")
-
-        # mark all as read also clear inaccessible needactions
-        msg2 = group_private.message_post(body='Zest', message_type='comment', subtype='mail.mt_comment', partner_ids=[emp_partner.id])
-        needaction_accessible = len(emp_partner.env['mail.message'].search([['needaction', '=', True]]))
-        self.assertEqual(needaction_accessible, 1, "a new message to a partner is readable to that partner")
-
-        msg2.sudo().partner_ids = self.env['res.partner']
-        emp_partner.env['mail.message'].search([['needaction', '=', True]])
-        needaction_length = len(emp_partner.env['mail.message'].search([['needaction', '=', True]]))
-        self.assertEqual(needaction_length, 1, "message should still be readable when notified")
-
-        na_count = emp_partner.get_needaction_count()
-        self.assertEqual(na_count, 1, "message not accessible is currently still counted")
-
-        self._clear_bus()
-        emp_partner.env['mail.message'].mark_all_as_read(domain=[])
-        self.assertBusNotification([(self.cr.dbname, 'res.partner', emp_partner.id)], [{ 'type': 'mark_as_read', 'message_ids': [msg2.id] }])
-        na_count = emp_partner.get_needaction_count()
-        self.assertEqual(na_count, 0, "mark all read should conclude all needactions even inacessible ones")
-
-    @mute_logger('openerp.addons.mail.models.mail_mail')
-    def test_mark_all_as_read_share(self):
-        self.user_portal.notification_type = 'inbox'
-        portal_partner = self.user_portal.partner_id.with_user(self.user_portal)
-
-        # mark all as read clear needactions
-        self.group_pigs.message_post(body='Test', message_type='comment', subtype='mail.mt_comment', partner_ids=[portal_partner.id])
-        portal_partner.env['mail.message'].mark_all_as_read(domain=[])
-        na_count = portal_partner.get_needaction_count()
-        self.assertEqual(na_count, 0, "mark all as read should conclude all needactions")
-
-        # mark all as read also clear inaccessible needactions
-        new_msg = self.group_pigs.message_post(body='Zest', message_type='comment', subtype='mail.mt_comment', partner_ids=[portal_partner.id])
-        needaction_accessible = len(portal_partner.env['mail.message'].search([['needaction', '=', True]]))
-        self.assertEqual(needaction_accessible, 1, "a new message to a partner is readable to that partner")
-
-        new_msg.sudo().partner_ids = self.env['res.partner']
-        needaction_length = len(portal_partner.env['mail.message'].search([['needaction', '=', True]]))
-        self.assertEqual(needaction_length, 1, "message should still be readable when notified")
-
-        na_count = portal_partner.get_needaction_count()
-        self.assertEqual(na_count, 1, "message not accessible is currently still counted")
-
-        portal_partner.env['mail.message'].mark_all_as_read(domain=[])
-        na_count = portal_partner.get_needaction_count()
-        self.assertEqual(na_count, 0, "mark all read should conclude all needactions even inacessible ones")
-
-
-@tagged('moderation')
-class TestMessageModeration(common.Moderation):
-
-    @classmethod
-    def setUpClass(cls):
-        super(TestMessageModeration, cls).setUpClass()
-
-        cls.msg_admin_pending_c1 = cls._create_new_message(cls, cls.channel_1.id, status='pending_moderation', author=cls.partner_admin)
-        cls.msg_admin_pending_c1_2 = cls._create_new_message(cls, cls.channel_1.id, status='pending_moderation', author=cls.partner_admin)
-        cls.msg_emp2_pending_c1 = cls._create_new_message(cls, cls.channel_1.id, status='pending_moderation', author=cls.partner_employee_2)
-
-    @mute_logger('odoo.models.unlink')
-    def test_moderate_accept(self):
-        self._clear_bus()
-        # A pending moderation message needs to have field channel_ids empty. Moderators
-        # need to be able to notify a pending moderation message (in a channel they moderate).
-        self.assertFalse(self.msg_admin_pending_c1.channel_ids)
-        self.msg_admin_pending_c1.with_user(self.user_employee)._moderate('accept')
-        self.assertEqual(self.msg_admin_pending_c1.channel_ids, self.channel_1)
-        self.assertEqual(self.msg_admin_pending_c1.moderation_status, 'accepted')
-        self.assertEqual(self.msg_admin_pending_c1_2.moderation_status, 'pending_moderation')
-        self.assertBusNotification([(self.cr.dbname, 'mail.channel', self.channel_1.id)])
-
-    @mute_logger('odoo.models.unlink')
-    def test_moderate_allow(self):
-        self._clear_bus()
-        # A pending moderation message needs to have field channel_ids empty. Moderators
-        # need to be able to notify a pending moderation message (in a channel they moderate).
-        self.assertFalse(self.msg_admin_pending_c1.channel_ids)
-        self.assertFalse(self.msg_admin_pending_c1_2.channel_ids)
-        self.msg_admin_pending_c1.with_user(self.user_employee)._moderate('allow')
-        self.assertEqual(self.msg_admin_pending_c1.channel_ids, self.channel_1)
-        self.assertEqual(self.msg_admin_pending_c1_2.channel_ids, self.channel_1)
-        self.assertEqual(self.msg_admin_pending_c1.moderation_status, 'accepted')
-        self.assertEqual(self.msg_admin_pending_c1_2.moderation_status, 'accepted')
-        self.assertBusNotification([
-            (self.cr.dbname, 'mail.channel', self.channel_1.id),
-            (self.cr.dbname, 'mail.channel', self.channel_1.id)])
-
-    @mute_logger('odoo.models.unlink')
-    def test_moderate_reject(self):
-        self._init_mock_build_email()
-        (self.msg_admin_pending_c1 | self.msg_emp2_pending_c1).with_user(self.user_employee)._moderate_send_reject_email('Title', 'Message to author')
-        self.env['mail.mail'].process_email_queue()
-        self.assertEmails(self.partner_employee, self.partner_employee_2 | self.partner_admin, subject='Title', body_content='Message to author')
-
-    def test_moderate_discard(self):
-        self._clear_bus()
-        id1, id2 = self.msg_admin_pending_c1.id, self.msg_emp2_pending_c1.id  # save ids because unlink will discard them
-        (self.msg_admin_pending_c1 | self.msg_emp2_pending_c1).with_user(self.user_employee)._moderate_discard()
-
-        self.assertBusNotification(
-            [(self.cr.dbname, 'res.partner', self.partner_admin.id),
-             (self.cr.dbname, 'res.partner', self.partner_employee_2.id),
-             (self.cr.dbname, 'res.partner', self.partner_employee.id)],
-            [{'type': 'deletion', 'message_ids': [id1]},  # admin: one message deleted because discarded
-             {'type': 'deletion', 'message_ids': [id2]},  # employee_2: one message delete because discarded
-             {'type': 'deletion', 'message_ids': [id1, id2]}]  # employee: two messages deleted because moderation done
-        )
-
-    @mute_logger('odoo.models.unlink')
-    def test_notify_moderators(self):
-        # create pending messages in another channel to have two notification to push
-        msg_emp_pending_c2 = self._create_new_message(self.channel_2.id, status='pending_moderation', author=self.partner_employee)
-
-        self.env['mail.message']._notify_moderators()
-        self.assertEmails(False, self.partner_employee | self.partner_employee_2, subject='Message are pending moderation', email_from=self.env.company.catchall or self.env.company.email)
