@@ -1,7 +1,10 @@
 # -*- coding: utf-8 -*-
 
 from . import common
+from . import test_odoobot
 from . import test_phone_blacklist
+from . import test_mass_mailing
+from . import test_mass_mailing_statistics
 from . import test_mass_sms
 from . import test_sms_composer
 from . import test_sms_management
