@@ -1,11 +1,10 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.addons.sms.tests import common as sms_common
-from odoo.addons.test_mail_full.tests import common as test_mail_full_common
+from odoo.addons.test_mail_full.tests.common import TestMailFullCommon, TestMailFullRecipients
 
 
-class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS, test_mail_full_common.TestRecipients):
+class TestSMSComposerComment(TestMailFullCommon, TestMailFullRecipients):
     """ TODO LIST
 
      * add test for default_res_model / default_res_id and stuff like that;
@@ -30,11 +29,11 @@ class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_commo
         cls.sms_template = cls.env['sms.template'].create({
             'name': 'Test Template',
             'model_id': cls.env['ir.model']._get('mail.test.sms').id,
-            'body': 'Dear ${object.display_name} this is an SMS.',
+            'body': 'Dear {{ object.display_name }} this is an SMS.',
         })
 
     def test_composer_comment_not_mail_thread(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             record = self.env['test_performance.base'].create({'name': 'TestBase'})
             composer = self.env['sms.composer'].with_context(
                 active_model='test_performance.base', active_id=record.id
@@ -46,10 +45,12 @@ class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_commo
             with self.mockSMSGateway():
                 composer._action_send_sms()
 
-        self.assertSMSSent(self.random_numbers_san, self._test_body)
+        # use sms.api directly, does not create sms.sms
+        self.assertNoSMS()
+        self.assertSMSIapSent(self.random_numbers_san, self._test_body)
 
     def test_composer_comment_default(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 active_model='mail.test.sms', active_id=self.test_record.id
             ).create({
@@ -62,7 +63,7 @@ class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_commo
         self.assertSMSNotification([{'partner': self.test_record.customer_id, 'number': self.test_numbers_san[1]}], self._test_body, messages)
 
     def test_composer_comment_field_1(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 active_model='mail.test.sms', active_id=self.test_record.id,
             ).create({
@@ -76,7 +77,7 @@ class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_commo
         self.assertSMSNotification([{'partner': self.test_record.customer_id, 'number': self.test_numbers_san[0]}], self._test_body, messages)
 
     def test_composer_comment_field_2(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 active_model='mail.test.sms', active_id=self.test_record.id,
             ).create({
@@ -90,7 +91,7 @@ class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_commo
         self.assertSMSNotification([{'partner': self.test_record.customer_id, 'number': self.test_numbers_san[1]}], self._test_body, messages)
 
     def test_composer_comment_field_w_numbers(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 active_model='mail.test.sms', active_id=self.test_record.id,
                 default_number_field_name='mobile_nbr',
@@ -107,7 +108,7 @@ class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_commo
             {'number': self.random_numbers_san[0]}, {'number': self.random_numbers_san[1]}], self._test_body, messages)
 
     def test_composer_comment_field_w_template(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 active_model='mail.test.sms', active_id=self.test_record.id,
                 default_template_id=self.sms_template.id,
@@ -119,8 +120,77 @@ class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_commo
 
         self.assertSMSNotification([{'partner': self.test_record.customer_id, 'number': self.test_record.mobile_nbr}], 'Dear %s this is an SMS.' % self.test_record.display_name, messages)
 
+    def test_composer_internals(self):
+        with self.with_user('employee'):
+            composer = self.env['sms.composer'].with_context(
+                default_res_model='mail.test.sms', default_res_id=self.test_record.id,
+            ).create({
+                'body': self._test_body,
+                'number_field_name': 'phone_nbr',
+            })
+
+        self.assertEqual(composer.res_model, self.test_record._name)
+        self.assertEqual(composer.res_id, self.test_record.id)
+        self.assertEqual(composer.number_field_name, 'phone_nbr')
+        self.assertTrue(composer.comment_single_recipient)
+        self.assertEqual(composer.recipient_single_description, self.test_record.customer_id.display_name)
+        self.assertEqual(composer.recipient_single_number, self.test_numbers[1])
+        self.assertEqual(composer.recipient_single_number_itf, self.test_numbers[1])
+        self.assertTrue(composer.recipient_single_valid)
+        self.assertEqual(composer.recipient_valid_count, 1)
+        self.assertEqual(composer.recipient_invalid_count, 0)
+
+        with self.with_user('employee'):
+            composer.update({'recipient_single_number_itf': '0123456789'})
+
+        self.assertFalse(composer.recipient_single_valid)
+
+        with self.with_user('employee'):
+            composer.update({'recipient_single_number_itf': self.random_numbers[0]})
+
+        self.assertTrue(composer.recipient_single_valid)
+
+        with self.with_user('employee'):
+            with self.mockSMSGateway():
+                composer.action_send_sms()
+
+        self.test_record.flush()
+        self.assertEqual(self.test_record.phone_nbr, self.random_numbers[0])
+
+    def test_composer_comment_wo_partner_wo_value_update(self):
+        """ Test record without partner and without phone values: should allow updating first found phone field """
+        self.test_record.write({
+            'customer_id': False,
+            'phone_nbr': False,
+            'mobile_nbr': False,
+        })
+        default_field_name = self.env['mail.test.sms']._sms_get_number_fields()[0]
+
+        with self.with_user('employee'):
+            composer = self.env['sms.composer'].with_context(
+                active_model='mail.test.sms', active_id=self.test_record.id,
+                default_composition_mode='comment',
+            ).create({
+                'body': self._test_body,
+            })
+            self.assertFalse(composer.recipient_single_number_itf)
+            self.assertFalse(composer.recipient_single_number)
+            self.assertEqual(composer.number_field_name, default_field_name)
+
+            composer.write({
+                'recipient_single_number_itf': self.random_numbers_san[0],
+            })
+            self.assertEqual(composer.recipient_single_number_itf, self.random_numbers_san[0])
+            self.assertFalse(composer.recipient_single_number)
+
+            with self.mockSMSGateway():
+                messages = composer._action_send_sms()
+
+        self.assertEqual(self.test_record[default_field_name], self.random_numbers_san[0])
+        self.assertSMSNotification([{'partner': self.env['res.partner'], 'number': self.random_numbers_san[0]}], self._test_body, messages)
+
     def test_composer_numbers_no_model(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='numbers'
             ).create({
@@ -130,10 +200,13 @@ class TestSMSComposerComment(test_mail_full_common.BaseFunctionalTest, sms_commo
 
             with self.mockSMSGateway():
                 composer._action_send_sms()
-        self.assertSMSSent(self.random_numbers_san, self._test_body)
+
+        # use sms.api directly, does not create sms.sms
+        self.assertNoSMS()
+        self.assertSMSIapSent(self.random_numbers_san, self._test_body)
 
 
-class TestSMSComposerBatch(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS):
+class TestSMSComposerBatch(TestMailFullCommon):
     @classmethod
     def setUpClass(cls):
         super(TestSMSComposerBatch, cls).setUpClass()
@@ -143,7 +216,7 @@ class TestSMSComposerBatch(test_mail_full_common.BaseFunctionalTest, sms_common.
         cls.sms_template = cls._create_sms_template('mail.test.sms')
 
     def test_composer_batch_active_domain(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='comment',
                 default_res_model='mail.test.sms',
@@ -160,7 +233,7 @@ class TestSMSComposerBatch(test_mail_full_common.BaseFunctionalTest, sms_common.
             self.assertSMSNotification([{'partner': r.customer_id} for r in self.records], 'Zizisse an SMS.', messages)
 
     def test_composer_batch_active_ids(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='comment',
                 default_res_model='mail.test.sms',
@@ -176,7 +249,7 @@ class TestSMSComposerBatch(test_mail_full_common.BaseFunctionalTest, sms_common.
             self.assertSMSNotification([{'partner': r.customer_id} for r in self.records], 'Zizisse an SMS.', messages)
 
     def test_composer_batch_domain(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='comment',
                 default_res_model='mail.test.sms',
@@ -193,7 +266,7 @@ class TestSMSComposerBatch(test_mail_full_common.BaseFunctionalTest, sms_common.
             self.assertSMSNotification([{'partner': r.customer_id} for r in self.records], 'Zizisse an SMS.', messages)
 
     def test_composer_batch_res_ids(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='comment',
                 default_res_model='mail.test.sms',
@@ -209,7 +282,7 @@ class TestSMSComposerBatch(test_mail_full_common.BaseFunctionalTest, sms_common.
             self.assertSMSNotification([{'partner': r.customer_id} for r in self.records], 'Zizisse an SMS.', messages)
 
 
-class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS):
+class TestSMSComposerMass(TestMailFullCommon):
 
     @classmethod
     def setUpClass(cls):
@@ -220,7 +293,7 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
         cls.sms_template = cls._create_sms_template('mail.test.sms')
 
     def test_composer_mass_active_domain(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -235,10 +308,10 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
                 composer.action_send_sms()
 
         for record in self.records:
-            self.assertSMSOutgoing(record.customer_id, None, self._test_body)
+            self.assertSMSOutgoing(record.customer_id, None, content=self._test_body)
 
     def test_composer_mass_active_domain_w_template(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -253,10 +326,10 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
                 composer.action_send_sms()
 
         for record in self.records:
-            self.assertSMSOutgoing(record.customer_id, None, 'Dear %s this is an SMS.' % record.display_name)
+            self.assertSMSOutgoing(record.customer_id, None, content='Dear %s this is an SMS.' % record.display_name)
 
     def test_composer_mass_active_ids(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -270,7 +343,7 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
                 composer.action_send_sms()
 
         for partner in self.partners:
-            self.assertSMSOutgoing(partner, None, self._test_body)
+            self.assertSMSOutgoing(partner, None, content=self._test_body)
 
     def test_composer_mass_active_ids_w_blacklist(self):
         self.env['phone.blacklist'].create([{
@@ -278,7 +351,7 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
             'active': True,
         } for p in self.partners[:5]])
 
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -295,7 +368,7 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
         for partner in self.partners[5:]:
             self.assertSMSOutgoing(partner, partner.phone_sanitized, content=self._test_body)
         for partner in self.partners[:5]:
-            self.assertSMSCanceled(partner, partner.phone_sanitized, 'sms_blacklist', content=self._test_body)
+            self.assertSMSCanceled(partner, partner.phone_sanitized, failure_type='sms_blacklist', content=self._test_body)
 
     def test_composer_mass_active_ids_wo_blacklist(self):
         self.env['phone.blacklist'].create([{
@@ -303,7 +376,7 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
             'active': True,
         } for p in self.partners[:5]])
 
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -329,7 +402,7 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
             p.mobile = self.partners[8].mobile
             self.assertEqual(p.phone_sanitized, self.partners[8].phone_sanitized)
 
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -346,12 +419,12 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
         for partner in self.partners[8:]:
             self.assertSMSOutgoing(partner, partner.phone_sanitized, content=self._test_body)
         for partner in self.partners[5:8]:
-            self.assertSMSCanceled(partner, partner.phone_sanitized, 'sms_duplicate', content=self._test_body)
+            self.assertSMSCanceled(partner, partner.phone_sanitized, failure_type='sms_duplicate', content=self._test_body)
         for partner in self.partners[:5]:
-            self.assertSMSCanceled(partner, partner.phone_sanitized, 'sms_blacklist', content=self._test_body)
+            self.assertSMSCanceled(partner, partner.phone_sanitized, failure_type='sms_blacklist', content=self._test_body)
 
     def test_composer_mass_active_ids_w_template(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -365,26 +438,26 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
                 composer.action_send_sms()
 
         for record in self.records:
-            self.assertSMSOutgoing(record.customer_id, None, 'Dear %s this is an SMS.' % record.display_name)
+            self.assertSMSOutgoing(record.customer_id, None, content='Dear %s this is an SMS.' % record.display_name)
 
     def test_composer_mass_active_ids_w_template_and_lang(self):
-        self.env.ref('base.lang_fr').write({'active': True})
+        self.env['res.lang']._activate_lang('fr_FR')
         self.env['ir.translation'].create({
             'type': 'model',
             'name': 'sms.template,body',
             'lang': 'fr_FR',
             'res_id': self.sms_template.id,
             'src': self.sms_template.body,
-            'value': 'Cher·e· ${object.display_name} ceci est un SMS.',
+            'value': 'Cher·e· {{ object.display_name }} ceci est un SMS.',
         })
         # set template to try to use customer lang
         self.sms_template.write({
-            'lang': '${object.customer_id.lang}',
+            'lang': '{{ object.customer_id.lang }}',
         })
         # set one customer as french speaking
         self.partners[2].write({'lang': 'fr_FR'})
 
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -399,12 +472,12 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
 
         for record in self.records:
             if record.customer_id == self.partners[2]:
-                self.assertSMSOutgoing(record.customer_id, None, 'Cher·e· %s ceci est un SMS.' % record.display_name)
+                self.assertSMSOutgoing(record.customer_id, None, content='Cher·e· %s ceci est un SMS.' % record.display_name)
             else:
-                self.assertSMSOutgoing(record.customer_id, None, 'Dear %s this is an SMS.' % record.display_name)
+                self.assertSMSOutgoing(record.customer_id, None, content='Dear %s this is an SMS.' % record.display_name)
 
     def test_composer_mass_active_ids_w_template_and_log(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
                 default_composition_mode='mass',
                 default_res_model='mail.test.sms',
@@ -418,25 +491,25 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
                 composer.action_send_sms()
 
         for record in self.records:
-            self.assertSMSOutgoing(record.customer_id, None, 'Dear %s this is an SMS.' % record.display_name)
+            self.assertSMSOutgoing(record.customer_id, None, content='Dear %s this is an SMS.' % record.display_name)
             self.assertSMSLogged(record, 'Dear %s this is an SMS.' % record.display_name)
 
     def test_composer_template_context_action(self):
         """ Test the context action from a SMS template (Add context action button)
         and the usage with the sms composer """
         # Create the lang info
-        self.env.ref('base.lang_fr').write({'active': True})
+        self.env['res.lang']._activate_lang('fr_FR')
         self.env['ir.translation'].create({
             'type': 'model',
             'name': 'sms.template,body',
             'lang': 'fr_FR',
             'res_id': self.sms_template.id,
             'src': self.sms_template.body,
-            'value': "Hello ${object.display_name} ceci est en français.",
+            'value': "Hello {{ object.display_name }} ceci est en français.",
         })
         # set template to try to use customer lang
         self.sms_template.write({
-            'lang': '${object.customer_id.lang}',
+            'lang': '{{ object.customer_id.lang }}',
         })
         # create a second record linked to a customer in another language
         self.partners[2].write({'lang': 'fr_FR'})
@@ -449,9 +522,9 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
             'customer_id': self.partners[1].id,
         })
         # Composer creation with context from a template context action (simulate) - comment (single recipient)
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
-                default_composition_mode='guess',
+                sms_composition_mode='guess',
                 default_res_ids=[test_record_2.id],
                 default_res_id=test_record_2.id,
                 active_ids=[test_record_2.id],
@@ -461,10 +534,8 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
             ).create({
                 'mass_keep_log': False,
             })
-            # Call manually the onchange
-            composer._onchange_template_id()
-            self.assertEquals(composer.composition_mode, "comment")
-            self.assertEquals(composer.body, "Hello %s ceci est en français." % test_record_2.display_name)
+            self.assertEqual(composer.composition_mode, "comment")
+            self.assertEqual(composer.body, "Hello %s ceci est en français." % test_record_2.display_name)
 
             with self.mockSMSGateway():
                 messages = composer._action_send_sms()
@@ -473,9 +544,9 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
         self.assertSMSNotification([{'partner': test_record_2.customer_id, 'number': number}], "Hello %s ceci est en français." % test_record_2.display_name, messages)
 
         # Composer creation with context from a template context action (simulate) - mass (multiple recipient)
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
-                default_composition_mode='guess',
+                sms_composition_mode='guess',
                 default_res_ids=[test_record_1.id, test_record_2.id],
                 default_res_id=test_record_1.id,
                 active_ids=[test_record_1.id, test_record_2.id],
@@ -485,14 +556,12 @@ class TestSMSComposerMass(test_mail_full_common.BaseFunctionalTest, sms_common.M
             ).create({
                 'mass_keep_log': True,
             })
-            # Call manually the onchange
-            composer._onchange_template_id()
-            self.assertEquals(composer.composition_mode, "mass")
+            self.assertEqual(composer.composition_mode, "mass")
             # In english because by default but when sinding depending of record
-            self.assertEquals(composer.body, "Dear ${object.display_name} this is an SMS.")
+            self.assertEqual(composer.body, "Dear {{ object.display_name }} this is an SMS.")
 
             with self.mockSMSGateway():
                 composer.action_send_sms()
 
-        self.assertSMSOutgoing(test_record_1.customer_id, None, 'Dear %s this is an SMS.' % test_record_1.display_name)
-        self.assertSMSOutgoing(test_record_2.customer_id, None, "Hello %s ceci est en français." % test_record_2.display_name)
+        self.assertSMSOutgoing(test_record_1.customer_id, None, content='Dear %s this is an SMS.' % test_record_1.display_name)
+        self.assertSMSOutgoing(test_record_2.customer_id, None, content="Hello %s ceci est en français." % test_record_2.display_name)
