@@ -9,7 +9,7 @@ from odoo.tools import mute_logger
 
 
 @tagged('mail_performance')
-class TestSMSPerformance(BaseMailPerformance, sms_common.MockSMS):
+class TestSMSPerformance(BaseMailPerformance, sms_common.SMSCase):
 
     def setUp(self):
         super(TestSMSPerformance, self).setUp()
@@ -44,23 +44,20 @@ class TestSMSPerformance(BaseMailPerformance, sms_common.MockSMS):
 
         self._init_mail_gateway()
 
-        # patch registry to simulate a ready environment
-        self.patch(self.env.registry, 'ready', True)
-
     @mute_logger('odoo.addons.sms.models.sms_sms')
     @users('employee')
     @warmup
     def test_message_sms_record_1_partner(self):
         record = self.test_record.with_user(self.env.user)
         pids = self.customer.ids
-        with self.mockSMSGateway(), self.assertQueryCount(employee=24):  # test_mail_enterprise: 24
+        with self.mockSMSGateway(sms_allow_unlink=True), self.assertQueryCount(employee=26):  # test_mail_enterprise: 26
             messages = record._message_sms(
                 body='Performance Test',
                 partner_ids=pids,
             )
 
         self.assertEqual(record.message_ids[0].body, '<p>Performance Test</p>')
-        self.assertSMSNotification([{'partner': self.customer}], 'Performance Test', messages)
+        self.assertSMSNotification([{'partner': self.customer}], 'Performance Test', messages, sent_unlink=True)
 
     @mute_logger('odoo.addons.sms.models.sms_sms')
     @users('employee')
@@ -68,27 +65,27 @@ class TestSMSPerformance(BaseMailPerformance, sms_common.MockSMS):
     def test_message_sms_record_10_partners(self):
         record = self.test_record.with_user(self.env.user)
         pids = self.partners.ids
-        with self.mockSMSGateway(), self.assertQueryCount(employee=42):  # test_mail_enterprise: 42
+        with self.mockSMSGateway(sms_allow_unlink=True), self.assertQueryCount(employee=44):
             messages = record._message_sms(
                 body='Performance Test',
                 partner_ids=pids,
             )
 
         self.assertEqual(record.message_ids[0].body, '<p>Performance Test</p>')
-        self.assertSMSNotification([{'partner': partner} for partner in self.partners], 'Performance Test', messages)
+        self.assertSMSNotification([{'partner': partner} for partner in self.partners], 'Performance Test', messages, sent_unlink=True)
 
     @mute_logger('odoo.addons.sms.models.sms_sms')
     @users('employee')
     @warmup
     def test_message_sms_record_default(self):
         record = self.test_record.with_user(self.env.user)
-        with self.mockSMSGateway(), self.assertQueryCount(employee=26):  # test_mail_enterprise: 26
+        with self.mockSMSGateway(sms_allow_unlink=True), self.assertQueryCount(employee=28):
             messages = record._message_sms(
                 body='Performance Test',
             )
 
         self.assertEqual(record.message_ids[0].body, '<p>Performance Test</p>')
-        self.assertSMSNotification([{'partner': self.customer}], 'Performance Test', messages)
+        self.assertSMSNotification([{'partner': self.customer}], 'Performance Test', messages, sent_unlink=True)
 
 
 @tagged('mail_performance')
@@ -127,9 +124,11 @@ class TestSMSMassPerformance(BaseMailPerformance, sms_common.MockSMS):
         self.sms_template = self.env['sms.template'].create({
             'name': 'Test Template',
             'model_id': self.env['ir.model']._get('mail.test.sms').id,
-            'body': 'Dear ${object.display_name} this is an SMS.',
+            'body': 'Dear {{ object.display_name }} this is an SMS.',
         })
 
+        self._init_mail_gateway()
+
     @mute_logger('odoo.addons.sms.models.sms_sms')
     @users('employee')
     @warmup
@@ -144,8 +143,8 @@ class TestSMSMassPerformance(BaseMailPerformance, sms_common.MockSMS):
             'mass_keep_log': False,
         })
 
-        with self.mockSMSGateway(), self.assertQueryCount(employee=106):
-                composer.action_send_sms()
+        with self.mockSMSGateway(sms_allow_unlink=True), self.assertQueryCount(employee=106):
+            composer.action_send_sms()
 
     @mute_logger('odoo.addons.sms.models.sms_sms')
     @users('employee')
@@ -161,5 +160,5 @@ class TestSMSMassPerformance(BaseMailPerformance, sms_common.MockSMS):
             'mass_keep_log': True,
         })
 
-        with self.mockSMSGateway(), self.assertQueryCount(employee=157):
+        with self.mockSMSGateway(sms_allow_unlink=True), self.assertQueryCount(employee=157):
             composer.action_send_sms()
