@@ -1,11 +1,10 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.addons.sms.tests import common as sms_common
-from odoo.addons.test_mail_full.tests import common as test_mail_full_common
+from odoo.addons.test_mail_full.tests.common import TestMailFullCommon, TestMailFullRecipients
 
 
-class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS, test_mail_full_common.TestRecipients):
+class TestSMSPost(TestMailFullCommon, TestMailFullRecipients):
     """ TODO
 
       * add tests for new mail.message and mail.thread fields;
@@ -25,7 +24,7 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
         cls.test_record = cls._reset_mail_context(cls.test_record)
 
     def test_message_sms_internals_body(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms('<p>Mega SMS<br/>Top moumoutte</p>', partner_ids=self.partner_1.ids)
 
@@ -34,26 +33,26 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
         self.assertSMSNotification([{'partner': self.partner_1}], 'Mega SMS\nTop moumoutte', messages)
 
     def test_message_sms_internals_check_existing(self):
-        with self.sudo('employee'), self.mockSMSGateway(sim_error='wrong_number_format'):
+        with self.with_user('employee'), self.mockSMSGateway(sim_error='wrong_number_format'):
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=self.partner_1.ids)
 
         self.assertSMSNotification([{'partner': self.partner_1, 'state': 'exception', 'failure_type': 'sms_number_format'}], self._test_body, messages)
 
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
-            test_record._notify_record_by_sms(messages, {'partners': [{'id': self.partner_1.id, 'notif': 'sms'}]}, check_existing=True)
+            test_record._notify_record_by_sms(messages, [{'id': self.partner_1.id, 'notif': 'sms'}], check_existing=True)
         self.assertSMSNotification([{'partner': self.partner_1}], self._test_body, messages)
 
     def test_message_sms_internals_sms_numbers(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=self.partner_1.ids, sms_numbers=self.random_numbers)
 
         self.assertSMSNotification([{'partner': self.partner_1}, {'number': self.random_numbers_san[0]}, {'number': self.random_numbers_san[1]}], self._test_body, messages)
 
     def test_message_sms_internals_subtype(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms('<p>Mega SMS<br/>Top moumoutte</p>', subtype_id=self.env.ref('mail.mt_comment').id, partner_ids=self.partner_1.ids)
 
@@ -66,7 +65,7 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
             self.partner_1.id: self.random_numbers[0],
             self.partner_2.id: self.random_numbers[1],
         }
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2).ids, sms_pid_to_number=pid_to_number)
 
@@ -76,7 +75,7 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
             self._test_body, messages)
 
     def test_message_sms_model_partner(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             messages = self.partner_1._message_sms(self._test_body)
             messages |= self.partner_2._message_sms(self._test_body)
         self.assertSMSNotification([{'partner': self.partner_1}, {'partner': self.partner_2}], self._test_body, messages)
@@ -91,8 +90,8 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
         self.assertSMSNotification([{'partner': self.partner_1, 'number': self.random_numbers_san[0]}, {'partner': self.partner_2}], self._test_body, messages)
 
     def test_message_sms_model_w_partner_only(self):
-        with self.sudo('employee'):
-            record = self.env['mail.test.sms.partner'].create({'partner_id': self.partner_1.id})
+        with self.with_user('employee'):
+            record = self.env['mail.test.sms.partner'].create({'customer_id': self.partner_1.id})
 
             with self.mockSMSGateway():
                 messages = record._message_sms(self._test_body)
@@ -100,18 +99,35 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
         self.assertSMSNotification([{'partner': self.partner_1}], self._test_body, messages)
 
     def test_message_sms_model_w_partner_only_void(self):
-        with self.sudo('employee'):
-            record = self.env['mail.test.sms.partner'].create({'partner_id': False})
+        with self.with_user('employee'):
+            record = self.env['mail.test.sms.partner'].create({'customer_id': False})
 
             with self.mockSMSGateway():
                 messages = record._message_sms(self._test_body)
 
-        # should not crash but no sms / no recipients
-        notifs = self.env['mail.notification'].search([('mail_message_id', 'in', messages.ids)])
-        self.assertFalse(notifs)
+        # should not crash but have a failed notification
+        self.assertSMSNotification([{'partner': self.env['res.partner'], 'number': False, 'state': 'exception', 'failure_type': 'sms_number_missing'}], self._test_body, messages)
+
+    def test_message_sms_model_w_partner_m2m_only(self):
+        with self.with_user('employee'):
+            record = self.env['mail.test.sms.partner.2many'].create({'customer_ids': [(4, self.partner_1.id)]})
+
+            with self.mockSMSGateway():
+                messages = record._message_sms(self._test_body)
+
+        self.assertSMSNotification([{'partner': self.partner_1}], self._test_body, messages)
+
+        # TDE: should take first found one according to partner ordering
+        with self.with_user('employee'):
+            record = self.env['mail.test.sms.partner.2many'].create({'customer_ids': [(4, self.partner_1.id), (4, self.partner_2.id)]})
+
+            with self.mockSMSGateway():
+                messages = record._message_sms(self._test_body)
+
+        self.assertSMSNotification([{'partner': self.partner_2}], self._test_body, messages)
 
     def test_message_sms_on_field_w_partner(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, number_field='mobile_nbr')
 
@@ -120,16 +136,31 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
     def test_message_sms_on_field_wo_partner(self):
         self.test_record.write({'customer_id': False})
 
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, number_field='mobile_nbr')
 
         self.assertSMSNotification([{'number': self.test_record.mobile_nbr}], self._test_body, messages)
 
+    def test_message_sms_on_field_wo_partner_wo_value(self):
+        """ Test record without a partner and without phone values. """
+        self.test_record.write({
+            'customer_id': False,
+            'phone_nbr': False,
+            'mobile_nbr': False,
+        })
+
+        with self.with_user('employee'), self.mockSMSGateway():
+            test_record = self.env['mail.test.sms'].browse(self.test_record.id)
+            messages = test_record._message_sms(self._test_body)
+
+        # should not crash but have a failed notification
+        self.assertSMSNotification([{'partner': self.env['res.partner'], 'number': False, 'state': 'exception', 'failure_type': 'sms_number_missing'}], self._test_body, messages)
+
     def test_message_sms_on_field_wo_partner_default_field(self):
         self.test_record.write({'customer_id': False})
 
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body)
 
@@ -138,40 +169,40 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
     def test_message_sms_on_field_wo_partner_default_field_2(self):
         self.test_record.write({'customer_id': False, 'phone_nbr': False})
 
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body)
 
         self.assertSMSNotification([{'number': self.test_numbers_san[0]}], self._test_body, messages)
 
     def test_message_sms_on_numbers(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, sms_numbers=self.random_numbers_san)
         self.assertSMSNotification([{'number': self.random_numbers_san[0]}, {'number': self.random_numbers_san[1]}], self._test_body, messages)
 
     def test_message_sms_on_numbers_sanitization(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, sms_numbers=self.random_numbers)
         self.assertSMSNotification([{'number': self.random_numbers_san[0]}, {'number': self.random_numbers_san[1]}], self._test_body, messages)
 
     def test_message_sms_on_partner_ids(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2).ids)
 
         self.assertSMSNotification([{'partner': self.partner_1}, {'partner': self.partner_2}], self._test_body, messages)
 
     def test_message_sms_on_partner_ids_default(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body)
 
         self.assertSMSNotification([{'partner': self.test_record.customer_id, 'number': self.test_numbers_san[1]}], self._test_body, messages)
 
     def test_message_sms_on_partner_ids_w_numbers(self):
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=self.partner_1.ids, sms_numbers=self.random_numbers[:1])
 
@@ -181,10 +212,10 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
         sms_template = self.env['sms.template'].create({
             'name': 'Test Template',
             'model_id': self.env['ir.model']._get('mail.test.sms').id,
-            'body': 'Dear ${object.display_name} this is an SMS.',
+            'body': 'Dear {{ object.display_name }} this is an SMS.',
         })
 
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             with self.mockSMSGateway():
                 test_record = self.env['mail.test.sms'].browse(self.test_record.id)
                 messages = test_record._message_sms_with_template(template=sms_template)
@@ -192,10 +223,10 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
         self.assertSMSNotification([{'partner': self.partner_1, 'number': self.test_numbers_san[1]}], 'Dear %s this is an SMS.' % self.test_record.display_name, messages)
 
     def test_message_sms_with_template_fallback(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             with self.mockSMSGateway():
                 test_record = self.env['mail.test.sms'].browse(self.test_record.id)
-                messages = test_record._message_sms_with_template(template_xmlid='test_mail_full.this_should_not_exists', template_fallback='Fallback for ${object.id}')
+                messages = test_record._message_sms_with_template(template_xmlid='test_mail_full.this_should_not_exists', template_fallback='Fallback for {{ object.id }}')
 
         self.assertSMSNotification([{'partner': self.partner_1, 'number': self.test_numbers_san[1]}], 'Fallback for %s' % self.test_record.id, messages)
 
@@ -203,7 +234,7 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
         sms_template = self.env['sms.template'].create({
             'name': 'Test Template',
             'model_id': self.env['ir.model']._get('mail.test.sms').id,
-            'body': 'Dear ${object.display_name} this is an SMS.',
+            'body': 'Dear {{ object.display_name }} this is an SMS.',
         })
         self.env['ir.model.data'].create({
             'name': 'this_should_exists',
@@ -212,7 +243,7 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
             'res_id': sms_template.id,
         })
 
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             with self.mockSMSGateway():
                 test_record = self.env['mail.test.sms'].browse(self.test_record.id)
                 messages = test_record._message_sms_with_template(template_xmlid='test_mail_full.this_should_exists')
@@ -220,7 +251,7 @@ class TestSMSPost(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS,
         self.assertSMSNotification([{'partner': self.partner_1, 'number': self.test_numbers_san[1]}], 'Dear %s this is an SMS.' % self.test_record.display_name, messages)
 
 
-class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS, test_mail_full_common.TestRecipients):
+class TestSMSPostException(TestMailFullCommon, TestMailFullRecipients):
 
     @classmethod
     def setUpClass(cls):
@@ -246,7 +277,7 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
 
     def test_message_sms_w_numbers_invalid(self):
         random_numbers = self.random_numbers + ['6988754']
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, sms_numbers=random_numbers)
 
@@ -259,7 +290,7 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
             'phone': self.random_numbers[1],
             'country_id': False,
         })
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=self.test_record.customer_id.ids)
 
@@ -271,7 +302,7 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
             'mobile': 'youpie',
             'phone': 'youpla',
         })
-        with self.sudo('employee'), self.mockSMSGateway():
+        with self.with_user('employee'), self.mockSMSGateway():
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=self.test_record.customer_id.ids)
 
@@ -282,12 +313,12 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
         # TDE FIXME: not sure
         # random_numbers = self.random_numbers + [self.random_numbers[1]]
         # random_numbers_san = self.random_numbers_san + [self.random_numbers_san[1]]
-        # with self.sudo('employee'), self.mockSMSGateway():
+        # with self.with_user('employee'), self.mockSMSGateway():
         #     messages = self.test_record._message_sms(self._test_body, sms_numbers=random_numbers)
         # self.assertSMSNotification({}, {random_numbers_san[0]: {}, random_numbers_san[1]: {}, random_numbers_san[2]: {}}, self._test_body, messages)
 
     def test_message_sms_crash_credit(self):
-        with self.sudo('employee'), self.mockSMSGateway(sim_error='credit'):
+        with self.with_user('employee'), self.mockSMSGateway(sim_error='credit'):
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2).ids)
 
@@ -297,7 +328,7 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
         ], self._test_body, messages)
 
     def test_message_sms_crash_credit_single(self):
-        with self.sudo('employee'), self.mockSMSGateway(nbr_t_error={self.partner_2.phone_get_sanitized_number(): 'credit'}):
+        with self.with_user('employee'), self.mockSMSGateway(nbr_t_error={self.partner_2.phone_get_sanitized_number(): 'credit'}):
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2 | self.partner_3).ids)
 
@@ -308,7 +339,7 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
         ], self._test_body, messages)
 
     def test_message_sms_crash_server_crash(self):
-        with self.sudo('employee'), self.mockSMSGateway(sim_error='jsonrpc_exception'):
+        with self.with_user('employee'), self.mockSMSGateway(sim_error='jsonrpc_exception'):
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2 | self.partner_3).ids)
 
@@ -318,8 +349,29 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
             {'partner': self.partner_3, 'state': 'exception', 'failure_type': 'sms_server'},
         ], self._test_body, messages)
 
+    def test_message_sms_crash_unregistered(self):
+        with self.with_user('employee'), self.mockSMSGateway(sim_error='unregistered'):
+            test_record = self.env['mail.test.sms'].browse(self.test_record.id)
+            messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2).ids)
+
+        self.assertSMSNotification([
+            {'partner': self.partner_1, 'state': 'exception', 'failure_type': 'sms_acc'},
+            {'partner': self.partner_2, 'state': 'exception', 'failure_type': 'sms_acc'},
+        ], self._test_body, messages)
+
+    def test_message_sms_crash_unregistered_single(self):
+        with self.with_user('employee'), self.mockSMSGateway(nbr_t_error={self.partner_2.phone_get_sanitized_number(): 'unregistered'}):
+            test_record = self.env['mail.test.sms'].browse(self.test_record.id)
+            messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2 | self.partner_3).ids)
+
+        self.assertSMSNotification([
+            {'partner': self.partner_1, 'state': 'sent'},
+            {'partner': self.partner_2, 'state': 'exception', 'failure_type': 'sms_acc'},
+            {'partner': self.partner_3, 'state': 'sent'},
+        ], self._test_body, messages)
+
     def test_message_sms_crash_wrong_number(self):
-        with self.sudo('employee'), self.mockSMSGateway(sim_error='wrong_number_format'):
+        with self.with_user('employee'), self.mockSMSGateway(sim_error='wrong_number_format'):
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2).ids)
 
@@ -329,7 +381,7 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
         ], self._test_body, messages)
 
     def test_message_sms_crash_wrong_number_single(self):
-        with self.sudo('employee'), self.mockSMSGateway(nbr_t_error={self.partner_2.phone_get_sanitized_number(): 'wrong_number_format'}):
+        with self.with_user('employee'), self.mockSMSGateway(nbr_t_error={self.partner_2.phone_get_sanitized_number(): 'wrong_number_format'}):
             test_record = self.env['mail.test.sms'].browse(self.test_record.id)
             messages = test_record._message_sms(self._test_body, partner_ids=(self.partner_1 | self.partner_2 | self.partner_3).ids)
 
@@ -340,7 +392,7 @@ class TestSMSPostException(test_mail_full_common.BaseFunctionalTest, sms_common.
         ], self._test_body, messages)
 
 
-class TestSMSApi(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS):
+class TestSMSApi(TestMailFullCommon):
 
     @classmethod
     def setUpClass(cls):
@@ -351,35 +403,35 @@ class TestSMSApi(test_mail_full_common.BaseFunctionalTest, sms_common.MockSMS):
         cls.sms_template = cls._create_sms_template('mail.test.sms')
 
     def test_message_schedule_sms(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             with self.mockSMSGateway():
                 self.env['mail.test.sms'].browse(self.records.ids)._message_sms_schedule_mass(body=self._test_body, mass_keep_log=False)
 
         for record in self.records:
-            self.assertSMSOutgoing(record.customer_id, None, self._test_body)
+            self.assertSMSOutgoing(record.customer_id, None, content=self._test_body)
 
     def test_message_schedule_sms_w_log(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             with self.mockSMSGateway():
                 self.env['mail.test.sms'].browse(self.records.ids)._message_sms_schedule_mass(body=self._test_body, mass_keep_log=True)
 
         for record in self.records:
-            self.assertSMSOutgoing(record.customer_id, None, self._test_body)
+            self.assertSMSOutgoing(record.customer_id, None, content=self._test_body)
             self.assertSMSLogged(record, self._test_body)
 
     def test_message_schedule_sms_w_template(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             with self.mockSMSGateway():
                 self.env['mail.test.sms'].browse(self.records.ids)._message_sms_schedule_mass(template=self.sms_template, mass_keep_log=False)
 
         for record in self.records:
-            self.assertSMSOutgoing(record.customer_id, None, 'Dear %s this is an SMS.' % record.display_name)
+            self.assertSMSOutgoing(record.customer_id, None, content='Dear %s this is an SMS.' % record.display_name)
 
     def test_message_schedule_sms_w_template_and_log(self):
-        with self.sudo('employee'):
+        with self.with_user('employee'):
             with self.mockSMSGateway():
                 self.env['mail.test.sms'].browse(self.records.ids)._message_sms_schedule_mass(template=self.sms_template, mass_keep_log=True)
 
         for record in self.records:
-            self.assertSMSOutgoing(record.customer_id, None, 'Dear %s this is an SMS.' % record.display_name)
+            self.assertSMSOutgoing(record.customer_id, None, content='Dear %s this is an SMS.' % record.display_name)
             self.assertSMSLogged(record, 'Dear %s this is an SMS.' % record.display_name)
