@@ -1,16 +1,17 @@
 # -*- coding: utf-8 -*-
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.tests import common
+from unittest.mock import patch
+
+from odoo.addons.base.tests.common import SavepointCaseWithUserDemo
+from odoo.tests import common, Form
+from odoo import Command
 
 def strip_prefix(prefix, names):
     size = len(prefix)
     return [name[size:] for name in names if name.startswith(prefix)]
 
-class TestOnChange(common.TransactionCase):
+class TestOnChange(SavepointCaseWithUserDemo):
 
     def setUp(self):
         super(TestOnChange, self).setUp()
@@ -130,10 +131,10 @@ class TestOnChange(common.TransactionCase):
             'moderator': False,
             'participants': [],
             'messages': [
-                (4, message1.id),
-                (4, message2.id),
-                (1, message2.id, {'body': "XYZ"}),
-                (0, 0, {
+                Command.link(message1.id),
+                Command.link(message2.id),
+                Command.update(message2.id, {'body': "XYZ"}),
+                Command.create({
                     'name': "[%s] %s" % ('', USER.name),
                     'body': "ABC",
                     'author': USER.id,
@@ -146,22 +147,22 @@ class TestOnChange(common.TransactionCase):
         result = self.Discussion.onchange(values, 'name', field_onchange)
         self.assertIn('messages', result['value'])
         self.assertEqual(result['value']['messages'], [
-            (5,),
-            (1, message1.id, {
+            Command.clear(),
+            Command.update(message1.id, {
                 'name': "[%s] %s" % ("Foo", USER.name),
                 'body': "ABC",
                 'author': USER.name_get()[0],
                 'size': 3,
                 'important': False,
             }),
-            (1, message2.id, {
+            Command.update(message2.id, {
                 'name': "[%s] %s" % ("Foo", USER.name),
                 'body': "XYZ",          # this must be sent back
                 'author': USER.name_get()[0],
                 'size': 3,
                 'important': False,
             }),
-            (0, 0, {
+            Command.create({
                 'name': "[%s] %s" % ("Foo", USER.name),
                 'body': "ABC",
                 'author': USER.name_get()[0],
@@ -175,11 +176,11 @@ class TestOnChange(common.TransactionCase):
         values = dict(values, name='{generate_dummy_message}')
         result = self.Discussion.with_context(generate_dummy_message=True).onchange(values, 'name', one_level_fields)
         self.assertEqual(result['value']['messages'], [
-            (5,),
-            (4, message1.id),
-            (4, message2.id),
-            (0, 0, {}),
-            (0, 0, {}),
+            Command.clear(),
+            Command.link(message1.id),
+            Command.link(message2.id),
+            Command.create({}),
+            Command.create({}),
         ])
 
     def test_onchange_one2many_reference(self):
@@ -217,7 +218,7 @@ class TestOnChange(common.TransactionCase):
         result = self.Discussion.onchange(values, 'name', field_onchange)
         self.assertIn('messages', result['value'])
         self.assertItemsEqual(result['value']['messages'], [
-            (5,),
+            (5, 0, 0),
             (0, REFERENCE, {
                 'name': "[%s] %s" % ("Foo", USER.name),
                 'body': BODY,
@@ -229,7 +230,7 @@ class TestOnChange(common.TransactionCase):
 
     def test_onchange_one2many_multi(self):
         """ test the effect of multiple onchange methods on one2many fields """
-        partner1 = self.env.ref('base.res_partner_1')
+        partner1 = self.env['res.partner'].create({'name': 'A partner'})
         multi = self.env['test_new_api.multi'].create({'partner': partner1.id})
         line1 = multi.lines.create({'multi': multi.id})
 
@@ -248,19 +249,19 @@ class TestOnChange(common.TransactionCase):
         self.assertEqual(values, {
             'name': partner1.name,
             'partner': partner1.id,
-            'lines': [(6, 0, [line1.id])],
+            'lines': [Command.set([line1.id])],
         })
 
         # modify 'partner'
         #   -> set 'partner' on all lines
         #   -> recompute 'name'
         #       -> set 'name' on all lines
-        partner2 = self.env.ref('base.res_partner_2')
+        partner2 = self.env['res.partner'].create({'name': 'A second partner'})
         values = {
             'name': partner1.name,
             'partner': partner2.id,             # this one just changed
-            'lines': [(6, 0, [line1.id]),
-                      (0, 0, {'name': False, 'partner': False, 'tags': [(5,)]})],
+            'lines': [Command.set([line1.id]),
+                      Command.create({'name': False, 'partner': False, 'tags': [Command.clear()]})],
         }
         self.env.cache.invalidate()
 
@@ -268,16 +269,16 @@ class TestOnChange(common.TransactionCase):
         self.assertEqual(result['value'], {
             'name': partner2.name,
             'lines': [
-                (5,),
-                (1, line1.id, {
+                Command.clear(),
+                Command.update(line1.id, {
                     'name': partner2.name,
                     'partner': (partner2.id, partner2.name),
-                    'tags': [(5,)],
+                    'tags': [Command.clear()],
                 }),
-                (0, 0, {
+                Command.create({
                     'name': partner2.name,
                     'partner': (partner2.id, partner2.name),
-                    'tags': [(5,)],
+                    'tags': [Command.clear()],
                 }),
             ],
         })
@@ -286,26 +287,26 @@ class TestOnChange(common.TransactionCase):
         values = {
             'name': partner1.name,
             'partner': partner2.id,             # this one just changed
-            'lines': [(6, 0, [line1.id]),
-                      (0, 0, {'name': False,
+            'lines': [Command.set([line1.id]),
+                      Command.create({'name': False,
                               'partner': False,
-                              'tags': [(5,), (0, 0, {'name': 'Tag'})]})],
+                              'tags': [Command.clear(), Command.create({'name': 'Tag'})]})],
         }
         self.env.cache.invalidate()
         result = multi.onchange(values, 'partner', field_onchange)
         expected_value = {
             'name': partner2.name,
             'lines': [
-                (5,),
-                (1, line1.id, {
+                Command.clear(),
+                Command.update(line1.id, {
                     'name': partner2.name,
                     'partner': (partner2.id, partner2.name),
-                    'tags': [(5,)],
+                    'tags': [Command.clear()],
                 }),
-                (0, 0, {
+                Command.create({
                     'name': partner2.name,
                     'partner': (partner2.id, partner2.name),
-                    'tags': [(5,), (0, 0, {'name': 'Tag'})],
+                    'tags': [Command.clear(), Command.create({'name': 'Tag'})],
                 }),
             ],
         }
@@ -326,7 +327,7 @@ class TestOnChange(common.TransactionCase):
     def test_onchange_specific(self):
         """ test the effect of field-specific onchange method """
         discussion = self.env.ref('test_new_api.discussion_0')
-        demo = self.env.ref('base.user_demo')
+        demo = self.user_demo
 
         field_onchange = self.Discussion._onchange_spec()
         self.assertEqual(field_onchange.get('moderator'), '1')
@@ -343,9 +344,9 @@ class TestOnChange(common.TransactionCase):
         values = {
             'name': discussion.name,
             'moderator': demo.id,
-            'categories': [(4, cat.id) for cat in discussion.categories],
-            'messages': [(4, msg.id) for msg in discussion.messages],
-            'participants': [(4, usr.id) for usr in discussion.participants],
+            'categories': [Command.link(cat.id) for cat in discussion.categories],
+            'messages': [Command.link(msg.id) for msg in discussion.messages],
+            'participants': [Command.link(usr.id) for usr in discussion.participants],
         }
         self.env.cache.invalidate()
         result = discussion.onchange(values, 'moderator', field_onchange)
@@ -353,7 +354,7 @@ class TestOnChange(common.TransactionCase):
         self.assertIn('participants', result['value'])
         self.assertItemsEqual(
             result['value']['participants'],
-            [(5,)] + [(4, user.id) for user in discussion.participants + demo],
+            [Command.clear()] + [Command.link(user.id) for user in discussion.participants + demo],
         )
 
     def test_onchange_default(self):
@@ -378,25 +379,38 @@ class TestOnChange(common.TransactionCase):
         result = Foo.onchange(values, 'value1', field_onchange)
         self.assertEqual(result['value'], {})
 
+    def test_onchange_one2many_first(self):
+        partner = self.env['res.partner'].create({
+            'name': 'X',
+            'country_id': self.env.ref('base.be').id,
+        })
+        with common.Form(self.env['test_new_api.multi']) as form:
+            form.partner = partner
+            self.assertEqual(form.partner, partner)
+            self.assertEqual(form.name, partner.name)
+            with form.lines.new() as line:
+                # the first onchange() must have computed partner
+                self.assertEqual(line.partner, partner)
+
     def test_onchange_one2many_value(self):
         """ test the value of the one2many field inside the onchange """
         discussion = self.env.ref('test_new_api.discussion_0')
-        demo = self.env.ref('base.user_demo')
+        demo = self.user_demo
 
         field_onchange = self.Discussion._onchange_spec()
         self.assertEqual(field_onchange.get('messages'), '1')
 
         self.assertEqual(len(discussion.messages), 3)
-        messages = [(4, msg.id) for msg in discussion.messages]
+        messages = [Command.link(msg.id) for msg in discussion.messages]
         messages[0] = (1, messages[0][1], {'body': 'test onchange'})
         lines = ["%s:%s" % (m.name, m.body) for m in discussion.messages]
         lines[0] = "%s:%s" % (discussion.messages[0].name, 'test onchange')
         values = {
             'name': discussion.name,
             'moderator': demo.id,
-            'categories': [(4, cat.id) for cat in discussion.categories],
+            'categories': [Command.link(cat.id) for cat in discussion.categories],
             'messages': messages,
-            'participants': [(4, usr.id) for usr in discussion.participants],
+            'participants': [Command.link(usr.id) for usr in discussion.participants],
             'message_concat': False,
         }
         result = discussion.onchange(values, 'messages', field_onchange)
@@ -406,7 +420,7 @@ class TestOnChange(common.TransactionCase):
     def test_onchange_one2many_with_domain_on_related_field(self):
         """ test the value of the one2many field when defined with a domain on a related field"""
         discussion = self.env.ref('test_new_api.discussion_0')
-        demo = self.env.ref('base.user_demo')
+        demo = self.user_demo
 
         # mimic UI behaviour, so we get subfields
         # (we need at least subfield: 'important_emails.important')
@@ -444,18 +458,18 @@ class TestOnChange(common.TransactionCase):
         values = {
             'name': "Foo Bar",
             'moderator': demo.id,
-            'categories': [(4, cat.id) for cat in discussion.categories],
-            'messages': [(4, msg.id) for msg in discussion.messages],
-            'participants': [(4, usr.id) for usr in discussion.participants],
-            'important_messages': [(4, msg.id) for msg in discussion.important_messages],
-            'important_emails': [(4, eml.id) for eml in discussion.important_emails],
+            'categories': [Command.link(cat.id) for cat in discussion.categories],
+            'messages': [Command.link(msg.id) for msg in discussion.messages],
+            'participants': [Command.link(usr.id) for usr in discussion.participants],
+            'important_messages': [Command.link(msg.id) for msg in discussion.important_messages],
+            'important_emails': [Command.link(eml.id) for eml in discussion.important_emails],
         }
         self.env.cache.invalidate()
         result = discussion.onchange(values, 'name', field_onchange)
 
         self.assertEqual(
             result['value']['important_emails'],
-            [(5,), (1, email.id, {
+            [Command.clear(), Command.update(email.id, {
                 'name': u'[Foo Bar] %s' % USER.name,
                 'body': BODY,
                 'author': USER.name_get()[0],
@@ -484,13 +498,13 @@ class TestOnChange(common.TransactionCase):
 
         self.env.cache.invalidate()
         Message = self.env['test_new_api.related']
-        result = Message.onchange(value, ['message', 'message_name', 'message_currency'], field_onchange)
+        result = Message.onchange(value, 'message', field_onchange)
 
         self.assertEqual(result['value'], onchange_result)
 
         self.env.cache.invalidate()
-        Message = self.env(user=self.env.ref('base.user_demo').id)['test_new_api.related']
-        result = Message.onchange(value, ['message', 'message_name', 'message_currency'], field_onchange)
+        Message = self.env(user=self.user_demo.id)['test_new_api.related']
+        result = Message.onchange(value, 'message', field_onchange)
 
         self.assertEqual(result['value'], onchange_result)
 
@@ -523,155 +537,382 @@ class TestOnChange(common.TransactionCase):
 
         self.assertFalse(called[0], "discussion.messages has been read")
 
+    def test_onchange_inherited(self):
+        """ Setting an inherited field should assign the field on the parent record. """
+        foo, bar = self.env['test_new_api.multi.tag'].create([{'name': 'Foo'}, {'name': 'Bar'}])
+        view = self.env['ir.ui.view'].create({
+            'name': 'Payment form view',
+            'model': 'test_new_api.payment',
+            'arch': """
+                <form>
+                    <field name="move_id" readonly="1" required="0"/>
+                    <field name="tag_id"/>
+                    <field name="tag_name"/>
+                    <field name="tag_repeat"/>
+                    <field name="tag_string"/>
+                </form>
+            """,
+        })
+
+        # both fields 'tag_id' and 'tag_name' are inherited through 'move_id';
+        # assigning 'tag_id' should modify 'move_id.tag_id' accordingly, which
+        # should in turn recompute `move.tag_name` and `tag_name`
+        form = Form(self.env['test_new_api.payment'], view)
+        self.assertEqual(form.tag_name, False)
+        form.tag_id = foo
+        self.assertEqual(form.tag_name, 'Foo')
+        self.assertEqual(form.tag_string, '')
+        form.tag_repeat = 2
+        self.assertEqual(form.tag_name, 'Foo')
+        self.assertEqual(form.tag_string, 'FooFoo')
+
+        payment = form.save()
+        self.assertEqual(payment.tag_id, foo)
+        self.assertEqual(payment.tag_name, 'Foo')
+        self.assertEqual(payment.tag_repeat, 2)
+        self.assertEqual(payment.tag_string, 'FooFoo')
+
+        with Form(payment, view) as form:
+            form.tag_id = bar
+            self.assertEqual(form.tag_name, 'Bar')
+            self.assertEqual(form.tag_string, 'BarBar')
+            form.tag_repeat = 3
+            self.assertEqual(form.tag_name, 'Bar')
+            self.assertEqual(form.tag_string, 'BarBarBar')
+
+        self.assertEqual(payment.tag_id, bar)
+        self.assertEqual(payment.tag_name, 'Bar')
+        self.assertEqual(payment.tag_repeat, 3)
+        self.assertEqual(payment.tag_string, 'BarBarBar')
+
 
 class TestComputeOnchange(common.TransactionCase):
 
     def test_create(self):
         model = self.env['test_new_api.compute.onchange']
 
+        # compute 'bar' (readonly) and 'baz' (editable)
+        record = model.create({'active': True})
+        self.assertEqual(record.bar, "r")
+        self.assertEqual(record.baz, "z")
+
         # compute 'bar' and 'baz'
         record = model.create({'active': True, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
-        self.assertEqual(record.baz, "foo")
+        self.assertEqual(record.bar, "foor")
+        self.assertEqual(record.baz, "fooz")
 
         # compute 'bar' but not 'baz'
         record = model.create({'active': True, 'foo': "foo", 'bar': "bar", 'baz': "baz"})
-        self.assertEqual(record.bar, "foo")
+        self.assertEqual(record.bar, "foor")
         self.assertEqual(record.baz, "baz")
 
         # compute 'bar' and 'baz', but do not change its value
         record = model.create({'active': False, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
+        self.assertEqual(record.bar, "foor")
         self.assertEqual(record.baz, False)
 
         # compute 'bar' but not 'baz'
         record = model.create({'active': False, 'foo': "foo", 'bar': "bar", 'baz': "baz"})
-        self.assertEqual(record.bar, "foo")
+        self.assertEqual(record.bar, "foor")
         self.assertEqual(record.baz, "baz")
 
+    def test_copy(self):
+        Model = self.env['test_new_api.compute.onchange']
+
+        # create tags
+        tag_foo, tag_bar = self.env['test_new_api.multi.tag'].create([
+            {'name': 'foo1'},
+            {'name': 'bar1'},
+        ])
+
+        # compute 'bar' (readonly), 'baz', 'line_ids' and 'tag_ids' (editable)
+        record = Model.create({'active': True, 'foo': "foo1"})
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "foo1z")
+        self.assertEqual(record.line_ids.mapped('foo'), ['foo1'])
+        self.assertEqual(record.tag_ids, tag_foo)
+
+        # manually update 'baz' and 'lines' to test copy attribute
+        record.write({
+            'baz': "baz1",
+            'line_ids': [Command.create({'foo': 'bar'})],
+            'tag_ids': [Command.link(tag_bar.id)],
+        })
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "baz1")
+        self.assertEqual(record.line_ids.mapped('foo'), ['foo1', 'bar'])
+        self.assertEqual(record.tag_ids, tag_foo + tag_bar)
+
+        # copy the record, and check results
+        copied = record.copy()
+        self.assertEqual(copied.foo, "foo1 (copy)")   # copied and modified
+        self.assertEqual(copied.bar, "foo1 (copy)r")  # computed
+        self.assertEqual(copied.baz, "baz1")          # copied
+        self.assertEqual(record.line_ids.mapped('foo'), ['foo1', 'bar'])  # copied
+        self.assertEqual(record.tag_ids, tag_foo + tag_bar)  # copied
+
     def test_write(self):
         model = self.env['test_new_api.compute.onchange']
         record = model.create({'active': True, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
-        self.assertEqual(record.baz, "foo")
+        self.assertEqual(record.bar, "foor")
+        self.assertEqual(record.baz, "fooz")
 
-        # recompute 'bar' and 'baz'
+        # recompute 'bar' (readonly) and 'baz' (editable)
         record.write({'foo': "foo1"})
-        self.assertEqual(record.bar, "foo1")
-        self.assertEqual(record.baz, "foo1")
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "foo1z")
 
         # recompute 'bar' but not 'baz'
         record.write({'foo': "foo2", 'bar': "bar2", 'baz': "baz2"})
-        self.assertEqual(record.bar, "foo2")
+        self.assertEqual(record.bar, "foo2r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'bar' and 'baz', but do not change its value
         record.write({'active': False, 'foo': "foo3"})
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'bar' but not 'baz'
         record.write({'active': False, 'foo': "foo4", 'bar': "bar4", 'baz': "baz4"})
-        self.assertEqual(record.bar, "foo4")
+        self.assertEqual(record.bar, "foo4r")
         self.assertEqual(record.baz, "baz4")
 
     def test_set(self):
         model = self.env['test_new_api.compute.onchange']
         record = model.create({'active': True, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
-        self.assertEqual(record.baz, "foo")
+        self.assertEqual(record.bar, "foor")
+        self.assertEqual(record.baz, "fooz")
 
-        # recompute 'bar' and 'baz'
+        # recompute 'bar' (readonly) and 'baz' (editable)
         record.foo = "foo1"
-        self.assertEqual(record.bar, "foo1")
-        self.assertEqual(record.baz, "foo1")
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "foo1z")
 
         # do not recompute 'baz'
         record.baz = "baz2"
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'baz', but do not change its value
         record.active = False
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'baz', but do not change its value
         record.foo = "foo3"
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz2")
 
         # do not recompute 'baz'
         record.baz = "baz4"
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz4")
 
     def test_set_new(self):
         model = self.env['test_new_api.compute.onchange']
-        record = model.new({'active': True, 'foo': "foo"})
-        self.assertEqual(record.bar, "foo")
-        self.assertEqual(record.baz, "foo")
+        record = model.new({'active': True})
+        self.assertEqual(record.bar, "r")
+        self.assertEqual(record.baz, "z")
 
-        # recompute 'bar' and 'baz'
+        # recompute 'bar' (readonly) and 'baz' (editable)
         record.foo = "foo1"
-        self.assertEqual(record.bar, "foo1")
-        self.assertEqual(record.baz, "foo1")
+        self.assertEqual(record.bar, "foo1r")
+        self.assertEqual(record.baz, "foo1z")
 
         # do not recompute 'baz'
         record.baz = "baz2"
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'baz', but do not change its value
         record.active = False
-        self.assertEqual(record.bar, "foo1")
+        self.assertEqual(record.bar, "foo1r")
         self.assertEqual(record.baz, "baz2")
 
         # recompute 'baz', but do not change its value
         record.foo = "foo3"
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz2")
 
         # do not recompute 'baz'
         record.baz = "baz4"
-        self.assertEqual(record.bar, "foo3")
+        self.assertEqual(record.bar, "foo3r")
         self.assertEqual(record.baz, "baz4")
 
     def test_onchange(self):
+        # check computations of 'bar' (readonly) and 'baz' (editable)
         form = common.Form(self.env['test_new_api.compute.onchange'])
+        self.assertEqual(form.bar, "r")
+        self.assertEqual(form.baz, False)
         form.active = True
+        self.assertEqual(form.bar, "r")
+        self.assertEqual(form.baz, "z")
         form.foo = "foo1"
-        self.assertEqual(form.bar, "foo1")
-        self.assertEqual(form.baz, "foo1")
+        self.assertEqual(form.bar, "foo1r")
+        self.assertEqual(form.baz, "foo1z")
         form.baz = "baz2"
-        self.assertEqual(form.bar, "foo1")
+        self.assertEqual(form.bar, "foo1r")
         self.assertEqual(form.baz, "baz2")
         form.active = False
-        self.assertEqual(form.bar, "foo1")
+        self.assertEqual(form.bar, "foo1r")
         self.assertEqual(form.baz, "baz2")
         form.foo = "foo3"
-        self.assertEqual(form.bar, "foo3")
+        self.assertEqual(form.bar, "foo3r")
         self.assertEqual(form.baz, "baz2")
         form.active = True
-        self.assertEqual(form.bar, "foo3")
-        self.assertEqual(form.baz, "foo3")
+        self.assertEqual(form.bar, "foo3r")
+        self.assertEqual(form.baz, "foo3z")
+
+        with form.line_ids.new() as line:
+            # check computation of 'bar' (readonly)
+            self.assertEqual(line.foo, False)
+            self.assertEqual(line.bar, "r")
+            line.foo = "foo"
+            self.assertEqual(line.foo, "foo")
+            self.assertEqual(line.bar, "foor")
 
         record = form.save()
-        self.assertEqual(record.bar, "foo3")
-        self.assertEqual(record.baz, "foo3")
+        self.assertEqual(record.bar, "foo3r")
+        self.assertEqual(record.baz, "foo3z")
 
         form = common.Form(record)
-        self.assertEqual(form.bar, "foo3")
-        self.assertEqual(form.baz, "foo3")
+        self.assertEqual(form.bar, "foo3r")
+        self.assertEqual(form.baz, "foo3z")
         form.foo = "foo4"
-        self.assertEqual(form.bar, "foo4")
-        self.assertEqual(form.baz, "foo4")
+        self.assertEqual(form.bar, "foo4r")
+        self.assertEqual(form.baz, "foo4z")
         form.baz = "baz5"
-        self.assertEqual(form.bar, "foo4")
+        self.assertEqual(form.bar, "foo4r")
         self.assertEqual(form.baz, "baz5")
         form.active = False
-        self.assertEqual(form.bar, "foo4")
+        self.assertEqual(form.bar, "foo4r")
         self.assertEqual(form.baz, "baz5")
         form.foo = "foo6"
-        self.assertEqual(form.bar, "foo6")
+        self.assertEqual(form.bar, "foo6r")
         self.assertEqual(form.baz, "baz5")
+
+    def test_onchange_default(self):
+        form = common.Form(self.env['test_new_api.compute.onchange'].with_context(
+            default_active=True, default_foo="foo", default_baz="baz",
+        ))
+        # 'baz' is computed editable, so when given a default value it should
+        # 'not be recomputed, even if a dependency also has a default value
+        self.assertEqual(form.foo, "foo")
+        self.assertEqual(form.bar, "foor")
+        self.assertEqual(form.baz, "baz")
+
+    def test_onchange_once(self):
+        """ Modifies `foo` field which will trigger an onchange method and
+        checks it was triggered only one time. """
+        form = Form(self.env['test_new_api.compute.onchange'].with_context(default_foo="oof"))
+        record = form.save()
+        self.assertEqual(record.foo, "oof")
+        self.assertEqual(record.count, 1, "value onchange must be called only one time")
+
+    def test_onchange_one2many(self):
+        record = self.env['test_new_api.model_parent_m2o'].create({
+            'name': 'Family',
+            'child_ids': [
+                Command.create({'name': 'W', 'cost': 10}),
+                Command.create({'name': 'X', 'cost': 10}),
+                Command.create({'name': 'Y'}),
+                Command.create({'name': 'Z'}),
+            ],
+        })
+        record.flush()
+        self.assertEqual(record.child_ids.mapped('name'), list('WXYZ'))
+        self.assertEqual(record.cost, 22)
+
+        # modifying a line should not recompute the cost on other lines
+        with common.Form(record) as form:
+            with form.child_ids.edit(1) as line:
+                line.name = 'XXX'
+            self.assertEqual(form.cost, 15)
+
+            with form.child_ids.edit(1) as line:
+                line.cost = 20
+            self.assertEqual(form.cost, 32)
+
+            with form.child_ids.edit(2) as line:
+                line.cost = 30
+            self.assertEqual(form.cost, 61)
+
+    def test_onchange_editable_compute_one2many(self):
+        # create a record with a computed editable field ('edit') on lines
+        record = self.env['test_new_api.compute_editable'].create({'line_ids': [(0, 0, {'value': 7})]})
+        record.flush()
+        line = record.line_ids
+        self.assertRecordValues(line, [{'value': 7, 'edit': 7, 'count': 0}])
+
+        # retrieve the onchange spec for calling 'onchange'
+        spec = Form(record)._view['onchange']
+
+        # The onchange on 'line_ids' should increment 'count' and keep the value
+        # of 'edit' (this field should not be recomputed), whatever the order of
+        # the fields in the dictionary.  This ensures that the value set by the
+        # user on a computed editable field on a line is not lost.
+        line_ids = [
+            Command.update(line.id, {'value': 8, 'edit': 9, 'count': 0}),
+            Command.create({'value': 8, 'edit': 9, 'count': 0}),
+        ]
+        result = record.onchange({'line_ids': line_ids}, 'line_ids', spec)
+        expected = {'value': {
+            'line_ids': [
+                Command.clear(),
+                Command.update(line.id, {'value': 8, 'edit': 9, 'count': 8}),
+                Command.create({'value': 8, 'edit': 9, 'count': 8}),
+            ],
+        }}
+        self.assertEqual(result, expected)
+
+        # change dict order in lines, and try again
+        line_ids = [
+            (op, id_, dict(reversed(list(vals.items()))))
+            for op, id_, vals in line_ids
+        ]
+        result = record.onchange({'line_ids': line_ids}, 'line_ids', spec)
+        self.assertEqual(result, expected)
+
+    def test_computed_editable_one2many_domain(self):
+        """ Test a computed, editable one2many field with a domain. """
+        record = self.env['test_new_api.one2many'].create({'name': 'foo'})
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+        ])
+
+        # trigger recomputation by changing name
+        record.name = 'bar'
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+        ])
+
+        # manually adding a line should not trigger recomputation
+        record.line_ids.create({'name': 'baz', 'container_id': record.id})
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+            {'name': 'baz', 'count': 1},
+        ])
+
+        # changing the field in the domain should not trigger recomputation...
+        record.line_ids[-1].count = 2
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+            {'name': 'baz', 'count': 2},
+        ])
+
+        # ...and may show cache inconsistencies
+        record.line_ids[-1].count = 0
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+            {'name': 'baz', 'count': 0},
+        ])
+        record.flush()
+        record.invalidate_cache()
+        self.assertRecordValues(record.line_ids, [
+            {'name': 'foo', 'count': 1},
+            {'name': 'bar', 'count': 1},
+        ])
