@@ -4,11 +4,45 @@
 from collections import defaultdict
 import json
 
+from odoo.addons.base.tests.common import SavepointCaseWithUserDemo
 from odoo.tests.common import TransactionCase, users, warmup, tagged
 from odoo.tools import mute_logger, json_default
+from odoo import Command
 
 
-class TestPerformance(TransactionCase):
+class TestPerformance(SavepointCaseWithUserDemo):
+
+    @classmethod
+    def setUpClass(cls):
+        super(TestPerformance, cls).setUpClass()
+        cls._load_partners_set()
+
+        partner3 = cls.env['res.partner'].search([('name', '=', 'AnalytIQ')], limit=1)
+        partner4 = cls.env['res.partner'].search([('name', '=', 'Urban Trends')], limit=1)
+        partner10 = cls.env['res.partner'].search([('name', '=', 'Ctrl-Alt-Fix')], limit=1)
+        partner12 = cls.env['res.partner'].search([('name', '=', 'Ignitive Labs')], limit=1)
+
+        cls.env['test_performance.base'].create([{
+            'name': 'Object 0',
+            'value': 0,
+            'partner_id': partner3.id,
+        }, {
+            'name': 'Object 1',
+            'value': 10,
+            'partner_id': partner3.id,
+        }, {
+            'name': 'Object 2',
+            'value': 20,
+            'partner_id': partner4.id,
+        }, {
+            'name': 'Object 3',
+            'value': 30,
+            'partner_id': partner10.id,
+        }, {
+            'name': 'Object 4',
+            'value': 40,
+            'partner_id': partner12.id,
+        }])
 
     @users('__system__', 'demo')
     @warmup
@@ -17,7 +51,7 @@ class TestPerformance(TransactionCase):
         records = self.env['test_performance.base'].search([])
         self.assertEqual(len(records), 5)
 
-        with self.assertQueryCount(__system__=3, demo=3):
+        with self.assertQueryCount(__system__=2, demo=2):
             # without cache
             for record in records:
                 record.partner_id.country_id.name
@@ -78,123 +112,125 @@ class TestPerformance(TransactionCase):
         rec1 = self.env['test_performance.base'].create({'name': 'X'})
 
         # create N lines on rec1: O(N) queries
-        rec1.invalidate_cache()
-        with self.assertQueryCount(2):
-            rec1.write({'line_ids': [(0, 0, {'value': 0})]})
+        with self.assertQueryCount(3):
+            rec1.invalidate_cache()
+            rec1.write({'line_ids': [Command.create({'value': 0})]})
         self.assertEqual(len(rec1.line_ids), 1)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(15):
-            rec1.write({'line_ids': [(0, 0, {'value': val}) for val in range(1, 12)]})
+            rec1.invalidate_cache()
+            rec1.write({'line_ids': [Command.create({'value': val}) for val in range(1, 12)]})
         self.assertEqual(len(rec1.line_ids), 12)
 
         lines = rec1.line_ids
 
         # update N lines: O(N) queries
-        rec1.invalidate_cache()
         with self.assertQueryCount(6):
-            rec1.write({'line_ids': [(1, line.id, {'value': 42}) for line in lines[0]]})
+            rec1.invalidate_cache()
+            rec1.write({'line_ids': [Command.update(line.id, {'value': 42}) for line in lines[0]]})
         self.assertEqual(rec1.line_ids, lines)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(26):
-            rec1.write({'line_ids': [(1, line.id, {'value': 42 + line.id}) for line in lines[1:]]})
+            rec1.invalidate_cache()
+            rec1.write({'line_ids': [Command.update(line.id, {'value': 42 + line.id}) for line in lines[1:]]})
         self.assertEqual(rec1.line_ids, lines)
 
         # delete N lines: O(1) queries
-        rec1.invalidate_cache()
-        with self.assertQueryCount(__system__=16, demo=16):
-            rec1.write({'line_ids': [(2, line.id) for line in lines[0]]})
+        with self.assertQueryCount(14):
+            rec1.invalidate_cache()
+            rec1.write({'line_ids': [Command.delete(line.id) for line in lines[0]]})
         self.assertEqual(rec1.line_ids, lines[1:])
 
-        rec1.invalidate_cache()
-        with self.assertQueryCount(__system__=14, demo=14):
-            rec1.write({'line_ids': [(2, line.id) for line in lines[1:]]})
+        with self.assertQueryCount(12):
+            rec1.invalidate_cache()
+            rec1.write({'line_ids': [Command.delete(line.id) for line in lines[1:]]})
         self.assertFalse(rec1.line_ids)
         self.assertFalse(lines.exists())
 
-        rec1.write({'line_ids': [(0, 0, {'value': val}) for val in range(12)]})
+        rec1.write({'line_ids': [Command.create({'value': val}) for val in range(12)]})
         lines = rec1.line_ids
 
         # unlink N lines: O(1) queries
-        rec1.invalidate_cache()
-        with self.assertQueryCount(__system__=13, demo=13):
-            rec1.write({'line_ids': [(3, line.id) for line in lines[0]]})
+        with self.assertQueryCount(14):
+            rec1.invalidate_cache()
+            rec1.write({'line_ids': [Command.unlink(line.id) for line in lines[0]]})
         self.assertEqual(rec1.line_ids, lines[1:])
 
-        rec1.invalidate_cache()
-        with self.assertQueryCount(__system__=14, demo=14):
-            rec1.write({'line_ids': [(3, line.id) for line in lines[1:]]})
+        with self.assertQueryCount(12):
+            rec1.invalidate_cache()
+            rec1.write({'line_ids': [Command.unlink(line.id) for line in lines[1:]]})
         self.assertFalse(rec1.line_ids)
         self.assertFalse(lines.exists())
 
-        rec1.write({'line_ids': [(0, 0, {'value': val}) for val in range(12)]})
+        rec1.write({'line_ids': [Command.create({'value': val}) for val in range(12)]})
         lines = rec1.line_ids
         rec2 = self.env['test_performance.base'].create({'name': 'X'})
 
         # link N lines from rec1 to rec2: O(1) queries
-        rec1.invalidate_cache()
-        with self.assertQueryCount(2):
-            rec2.write({'line_ids': [(4, line.id) for line in lines[0]]})
+        with self.assertQueryCount(8):
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.link(line.id) for line in lines[0]]})
         self.assertEqual(rec1.line_ids, lines[1:])
         self.assertEqual(rec2.line_ids, lines[0])
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(8):
-            rec2.write({'line_ids': [(4, line.id) for line in lines[1:]]})
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.link(line.id) for line in lines[1:]]})
         self.assertFalse(rec1.line_ids)
         self.assertEqual(rec2.line_ids, lines)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(4):
-            rec2.write({'line_ids': [(4, line.id) for line in lines[0]]})
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.link(line.id) for line in lines[0]]})
         self.assertEqual(rec2.line_ids, lines)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(4):
-            rec2.write({'line_ids': [(4, line.id) for line in lines[1:]]})
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.link(line.id) for line in lines[1:]]})
         self.assertEqual(rec2.line_ids, lines)
 
         # empty N lines in rec2: O(1) queries
-        rec1.invalidate_cache()
-        with self.assertQueryCount(__system__=15, demo=15):
-            rec2.write({'line_ids': [(5,)]})
+        with self.assertQueryCount(13):
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.clear()]})
         self.assertFalse(rec2.line_ids)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec2.write({'line_ids': [(5,)]})
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.clear()]})
         self.assertFalse(rec2.line_ids)
 
-        rec1.write({'line_ids': [(0, 0, {'value': val}) for val in range(12)]})
+        rec1.write({'line_ids': [Command.create({'value': val}) for val in range(12)]})
         lines = rec1.line_ids
 
         # set N lines in rec2: O(1) queries
-        rec1.invalidate_cache()
-        with self.assertQueryCount(4):
-            rec2.write({'line_ids': [(6, 0, lines[0].ids)]})
+        with self.assertQueryCount(8):
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.set(lines[0].ids)]})
         self.assertEqual(rec1.line_ids, lines[1:])
         self.assertEqual(rec2.line_ids, lines[0])
 
-        with self.assertQueryCount(5):
-            rec2.write({'line_ids': [(6, 0, lines.ids)]})
+        with self.assertQueryCount(6):
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.set(lines.ids)]})
         self.assertFalse(rec1.line_ids)
         self.assertEqual(rec2.line_ids, lines)
 
-        with self.assertQueryCount(3):
-            rec2.write({'line_ids': [(6, 0, lines.ids)]})
+        with self.assertQueryCount(4):
+            rec1.invalidate_cache()
+            rec2.write({'line_ids': [Command.set(lines.ids)]})
         self.assertEqual(rec2.line_ids, lines)
 
     @mute_logger('odoo.models.unlink')
     def test_write_base_one2many_with_constraint(self):
         """ Write on one2many field with lines being deleted and created. """
         rec = self.env['test_performance.base'].create({'name': 'Y'})
-        rec.write({'line_ids': [(0, 0, {'value': val}) for val in range(12)]})
+        rec.write({'line_ids': [Command.create({'value': val}) for val in range(12)]})
 
         # This write() will raise because of the unique index if the unlink() is
         # not performed before the create()
-        rec.write({'line_ids': [(5,)] + [(0, 0, {'value': val}) for val in range(6)]})
-        self.assertEquals(len(rec.line_ids), 6)
+        rec.write({'line_ids': [Command.clear()] + [Command.create({'value': val}) for val in range(6)]})
+        self.assertEqual(len(rec.line_ids), 6)
 
     @mute_logger('odoo.models.unlink')
     @users('__system__', 'demo')
@@ -204,110 +240,110 @@ class TestPerformance(TransactionCase):
         rec1 = self.env['test_performance.base'].create({'name': 'X'})
 
         # create N tags on rec1: O(N) queries
-        rec1.invalidate_cache()
         with self.assertQueryCount(4):
-            rec1.write({'tag_ids': [(0, 0, {'name': 0})]})
+            rec1.invalidate_cache()
+            rec1.write({'tag_ids': [Command.create({'name': 0})]})
         self.assertEqual(len(rec1.tag_ids), 1)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(14):
-            rec1.write({'tag_ids': [(0, 0, {'name': val}) for val in range(1, 12)]})
+            rec1.invalidate_cache()
+            rec1.write({'tag_ids': [Command.create({'name': val}) for val in range(1, 12)]})
         self.assertEqual(len(rec1.tag_ids), 12)
 
         tags = rec1.tag_ids
 
         # update N tags: O(N) queries
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec1.write({'tag_ids': [(1, tag.id, {'name': 'X'}) for tag in tags[0]]})
+            rec1.invalidate_cache()
+            rec1.write({'tag_ids': [Command.update(tag.id, {'name': 'X'}) for tag in tags[0]]})
         self.assertEqual(rec1.tag_ids, tags)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec1.write({'tag_ids': [(1, tag.id, {'name': 'X'}) for tag in tags[1:]]})
+            rec1.invalidate_cache()
+            rec1.write({'tag_ids': [Command.update(tag.id, {'name': 'X'}) for tag in tags[1:]]})
         self.assertEqual(rec1.tag_ids, tags)
 
         # delete N tags: O(1) queries
-        rec1.invalidate_cache()
-        with self.assertQueryCount(__system__=10, demo=10):
-            rec1.write({'tag_ids': [(2, tag.id) for tag in tags[0]]})
+        with self.assertQueryCount(__system__=8, demo=8):
+            rec1.invalidate_cache()
+            rec1.write({'tag_ids': [Command.delete(tag.id) for tag in tags[0]]})
         self.assertEqual(rec1.tag_ids, tags[1:])
 
-        rec1.invalidate_cache()
-        with self.assertQueryCount(__system__=10, demo=10):
-            rec1.write({'tag_ids': [(2, tag.id) for tag in tags[1:]]})
+        with self.assertQueryCount(__system__=8, demo=8):
+            rec1.invalidate_cache()
+            rec1.write({'tag_ids': [Command.delete(tag.id) for tag in tags[1:]]})
         self.assertFalse(rec1.tag_ids)
         self.assertFalse(tags.exists())
 
-        rec1.write({'tag_ids': [(0, 0, {'name': val}) for val in range(12)]})
+        rec1.write({'tag_ids': [Command.create({'name': val}) for val in range(12)]})
         tags = rec1.tag_ids
 
         # unlink N tags: O(1) queries
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec1.write({'tag_ids': [(3, tag.id) for tag in tags[0]]})
+            rec1.invalidate_cache()
+            rec1.write({'tag_ids': [Command.unlink(tag.id) for tag in tags[0]]})
         self.assertEqual(rec1.tag_ids, tags[1:])
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec1.write({'tag_ids': [(3, tag.id) for tag in tags[1:]]})
+            rec1.invalidate_cache()
+            rec1.write({'tag_ids': [Command.unlink(tag.id) for tag in tags[1:]]})
         self.assertFalse(rec1.tag_ids)
         self.assertTrue(tags.exists())
 
         rec2 = self.env['test_performance.base'].create({'name': 'X'})
 
         # link N tags from rec1 to rec2: O(1) queries
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec2.write({'tag_ids': [(4, tag.id) for tag in tags[0]]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.link(tag.id) for tag in tags[0]]})
         self.assertEqual(rec2.tag_ids, tags[0])
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec2.write({'tag_ids': [(4, tag.id) for tag in tags[1:]]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.link(tag.id) for tag in tags[1:]]})
         self.assertEqual(rec2.tag_ids, tags)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(2):
-            rec2.write({'tag_ids': [(4, tag.id) for tag in tags[1:]]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.link(tag.id) for tag in tags[1:]]})
         self.assertEqual(rec2.tag_ids, tags)
 
         # empty N tags in rec2: O(1) queries
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec2.write({'tag_ids': [(5,)]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.clear()]})
         self.assertFalse(rec2.tag_ids)
         self.assertTrue(tags.exists())
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(2):
-            rec2.write({'tag_ids': [(5,)]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.clear()]})
         self.assertFalse(rec2.tag_ids)
 
         # set N tags in rec2: O(1) queries
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec2.write({'tag_ids': [(6, 0, tags.ids)]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.set(tags.ids)]})
         self.assertEqual(rec2.tag_ids, tags)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec2.write({'tag_ids': [(6, 0, tags[:8].ids)]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.set(tags[:8].ids)]})
         self.assertEqual(rec2.tag_ids, tags[:8])
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(4):
-            rec2.write({'tag_ids': [(6, 0, tags[4:].ids)]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.set(tags[4:].ids)]})
         self.assertEqual(rec2.tag_ids, tags[4:])
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(3):
-            rec2.write({'tag_ids': [(6, 0, tags.ids)]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.set(tags.ids)]})
         self.assertEqual(rec2.tag_ids, tags)
 
-        rec1.invalidate_cache()
         with self.assertQueryCount(2):
-            rec2.write({'tag_ids': [(6, 0, tags.ids)]})
+            rec1.invalidate_cache()
+            rec2.write({'tag_ids': [Command.set(tags.ids)]})
         self.assertEqual(rec2.tag_ids, tags)
 
     @users('__system__', 'demo')
@@ -324,7 +360,7 @@ class TestPerformance(TransactionCase):
         with self.assertQueryCount(__system__=12, demo=12):
             self.env['test_performance.base'].create({
                 'name': 'X',
-                'line_ids': [(0, 0, {'value': val}) for val in range(10)],
+                'line_ids': [Command.create({'value': val}) for val in range(10)],
             })
 
     @users('__system__', 'demo')
@@ -338,7 +374,7 @@ class TestPerformance(TransactionCase):
         with self.assertQueryCount(13):
             self.env['test_performance.base'].create({
                 'name': 'X',
-                'tag_ids': [(0, 0, {'name': val}) for val in range(10)],
+                'tag_ids': [Command.create({'name': val}) for val in range(10)],
             })
 
         # link N tags: add O(1) queries
@@ -347,19 +383,19 @@ class TestPerformance(TransactionCase):
         with self.assertQueryCount(3):
             self.env['test_performance.base'].create({
                 'name': 'X',
-                'tag_ids': [(4, tag.id) for tag in tags],
+                'tag_ids': [Command.link(tag.id) for tag in tags],
             })
 
         with self.assertQueryCount(2):
             self.env['test_performance.base'].create({
                 'name': 'X',
-                'tag_ids': [(6, 0, [])],
+                'tag_ids': [Command.set([])],
             })
 
         with self.assertQueryCount(3):
             self.env['test_performance.base'].create({
                 'name': 'X',
-                'tag_ids': [(6, 0, tags.ids)],
+                'tag_ids': [Command.set(tags.ids)],
             })
 
     @users('__system__', 'demo')
@@ -378,12 +414,12 @@ class TestPerformance(TransactionCase):
         with self.assertQueryCount(__system__=2, demo=2):
             records.mapped('value')
 
-        records.invalidate_cache(['value'])
         with self.assertQueryCount(__system__=2, demo=2):
+            records.invalidate_cache(['value'])
             records.mapped('value')
 
-        records.invalidate_cache(['value'])
         with self.assertQueryCount(__system__=2, demo=2):
+            records.invalidate_cache(['value'])
             new_recs = records.browse(records.new(origin=record).id for record in records)
             new_recs.mapped('value')
 
@@ -395,9 +431,10 @@ class TestPerformance(TransactionCase):
 
     def expected_read_group(self):
         groups = defaultdict(list)
-        for record in self.env['test_performance.base'].search([]):
+        all_records = self.env['test_performance.base'].search([])
+        for record in all_records:
             groups[record.partner_id.id].append(record.value)
-        partners = self.env['res.partner'].search([('id', 'in', list(groups))])
+        partners = self.env['res.partner'].search([('id', 'in', all_records.mapped('partner_id').ids)])
         return [{
             '__domain': [('partner_id', '=', partner.id)],
             'partner_id': (partner.id, partner.display_name),
@@ -443,7 +480,7 @@ class TestIrPropertyOptimizations(TransactionCase):
         self.Eggs = self.env['test_performance.eggs']
 
     def test_with_falsy_default(self):
-        self.assertFalse(self.env['ir.property'].get('property_eggs', 'test_performance.bacon'))
+        self.assertFalse(self.env['ir.property']._get('property_eggs', 'test_performance.bacon'))
 
         # warmup
         eggs = self.Eggs.create({})
@@ -469,14 +506,9 @@ class TestIrPropertyOptimizations(TransactionCase):
 
     def test_with_truthy_default(self):
         eggs = self.Eggs.create({})
-        field = self.env['ir.model.fields']._get('test_performance.bacon', 'property_eggs')
-        self.env['ir.property'].create({
-            'name': 'property_eggs_with_bacon',
-            'fields_id': field.id,
-            'value': eggs,
-        })
+        self.env['ir.property']._set_default("property_eggs", "test_performance.bacon", eggs)
 
-        self.assertEqual(eggs, self.env['ir.property'].get('property_eggs', 'test_performance.bacon'))
+        self.assertEqual(eggs, self.env['ir.property']._get('property_eggs', 'test_performance.bacon'))
 
         # warmup
         self.Bacon.create({})
@@ -506,3 +538,21 @@ class TestIrPropertyOptimizations(TransactionCase):
 
         with self.assertQueryCount(3):
             self.Bacon.create({'property_eggs': False})
+
+
+@tagged('mapped_perf')
+class TestMapped(TransactionCase):
+
+    def test_relational_mapped(self):
+        # create 1000 records with one line each
+        recs = self.env['test_performance.base'].create([
+            {'name': 'foo%d' % index, 'line_ids': [Command.create({'value': index})]}
+            for index in range(1000)
+        ])
+        recs.flush()
+        recs.invalidate_cache()
+
+        # expected same performance as recs.line_ids.mapped('value')
+        with self.assertQueryCount(3):
+            for rec in recs:
+                rec.line_ids.mapped('value')
