@@ -3,3 +3,5 @@ from . import test_empty
 from . import test_group_expand
 from . import test_group_operator
 from . import test_fill_temporal
+from . import test_auto_join
+from . import test_m2m_grouping
