@@ -18,23 +18,16 @@ def post_init_hook(cr, registry):
     themes = themes.filtered(lambda t: not any([ex for ex in exclude_list if ex in t.name]))
     assert len(themes) == len(env.ref('base.module_test_themes').dependencies_id)
 
+    xmlids = []
     for theme in themes:
         website = env['website'].create({
             'name': theme.display_name,
-            'social_facebook': 'test_themes',
             'theme_id': theme.id,
         })
+        xmlids.append({
+            'xml_id': 'test_themes.%s' % theme.display_name.replace(' ', '_'),
+            'record': website,
+            'noupdate': True,  # Avoid unlink on -u
+        })
         theme._theme_get_stream_themes()._theme_load(website)
-
-
-def uninstall_hook(cr, registry):
-    ''' Remove the created websites. '''
-    env = api.Environment(cr, SUPERUSER_ID, {})
-    websites_themes = env['website'].search([('social_facebook', '=', 'test_themes')])
-    assert len(websites_themes) == len(env.ref('base.module_test_themes').dependencies_id)
-
-    # Bypass foreign key constraint
-    website_domain = [('website_id', 'in', websites_themes.ids)]
-    env['ir.ui.view'].with_context(active_test=False, _force_unlink=True).search(website_domain).unlink()
-    env['ir.attachment'].with_context(active_test=False).search(website_domain).unlink()
-    websites_themes.unlink()
+    env['ir.model.data']._update_xmlids(xmlids)
