@@ -10,18 +10,20 @@
     theme on one website.""",
     'depends': [
         # CE themes
-        'theme_bootswatch',
         'theme_default',
 
         # design-themes themes
         'theme_anelusia',
         'theme_artists',
         'theme_avantgarde',
+        'theme_aviato',
         'theme_beauty',
         'theme_bewise',
         'theme_bistro',
         'theme_bookstore',
+        'theme_buzzy',
         'theme_clean',
+        'theme_cobalt',
         'theme_enark',
         'theme_graphene',
         'theme_kea',
@@ -32,6 +34,7 @@
         'theme_notes',
         'theme_odoo_experts',
         'theme_orchid',
+        'theme_paptic',
         'theme_real_estate',
         'theme_treehouse',
         'theme_vehicle',
@@ -39,10 +42,15 @@
         'theme_zap',
     ],
     'data': [
+        'views/website_navbar_templates.xml',
     ],
     'installable': True,
     'application': False,
     'post_init_hook': 'post_init_hook',
-    'uninstall_hook': 'uninstall_hook',
+    'assets': {
+        'web.assets_frontend': [
+            'test_themes/static/src/js/navbar.js',
+        ],
+    },
     'license': 'LGPL-3',
 }
