@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import models
+from odoo import models, tools
 from odoo.http import request
 
 
@@ -12,7 +12,7 @@ class Http(models.AbstractModel):
     def _add_dispatch_parameters(cls, func):
         # Allow public user to use `fw` query string in test mode to ease tests
         force_website_id = request.httprequest.args.get('fw')
-        if request.registry.in_test_mode() and force_website_id:
+        if (request.registry.in_test_mode() or tools.config.options['test_enable']) and force_website_id:
             request.env['website']._force_website(force_website_id)
 
         super(Http, cls)._add_dispatch_parameters(func)
