@@ -7,7 +7,8 @@ import io
 import odoo
 from odoo.tests import common, tagged
 from odoo.tools.misc import file_open, mute_logger
-from odoo.tools.translate import _, _lt, TranslationFileReader
+from odoo.tools.translate import _, _lt, TranslationFileReader, TranslationModuleReader
+from odoo import Command
 
 
 TRANSLATED_TERM = _lt("Klingon")
@@ -18,7 +19,8 @@ class TestTermCount(common.TransactionCase):
         """
         Just make sure we have as many translation entries as we wanted.
         """
-        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', module_name='test_translation_import', verbose=False)
+        self.env['res.lang']._activate_lang('fr_FR')
+        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', verbose=False)
         translations = self.env['ir.translation'].search([
             ('lang', '=', 'fr_FR'),
             ('src', '=', '1XBUO5PUYH2RYZSA1FTLRYS8SPCNU1UYXMEYMM25ASV7JC2KTJZQESZYRV9L8CGB'),
@@ -40,6 +42,7 @@ class TestTermCount(common.TransactionCase):
         """
         Just make sure we have as many translation entries as we wanted and module deducted from file content
         """
+        self.env['res.lang']._activate_lang('fr_FR')
         odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', verbose=False)
         translations = self.env['ir.translation'].search([
             ('lang', '=', 'fr_FR'),
@@ -56,10 +59,11 @@ class TestTermCount(common.TransactionCase):
         menu = self.env.ref('test_translation_import.menu_test_translation_import')
         menu.name = "New Name"
         # install french and change translation content
-        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', module_name='test_translation_import', verbose=False)
+        self.env['res.lang']._activate_lang('fr_FR')
+        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', verbose=False)
         menu.with_context(lang='fr_FR').name = "Nouveau nom"
         # reload with overwrite
-        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', module_name='test_translation_import', verbose=False, context={'overwrite': True})
+        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', verbose=False, overwrite=True)
 
         # trans_load invalidates ormcache but not record cache
         menu.env.cache.invalidate()
@@ -67,8 +71,9 @@ class TestTermCount(common.TransactionCase):
         self.assertEqual(menu.with_context(lang='fr_FR').name, "Nouveau nom")
 
     def test_lang_with_base(self):
-        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_BE', module_name='test_translation_import', verbose=False)
-        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr_BE.po', 'fr_BE', module_name='test_translation_import', verbose=False, context={'overwrite': True})
+        self.env['res.lang']._activate_lang('fr_BE')
+        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_BE', verbose=False)
+        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr_BE.po', 'fr_BE', verbose=False, overwrite=True)
 
         # language override base language
         translations = self.env['ir.translation'].search([
@@ -95,7 +100,8 @@ class TestTermCount(common.TransactionCase):
         """
         Just make sure we do not create duplicated translation with 'code' type
         """
-        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', module_name='test_translation_import', verbose=False)
+        self.env['res.lang']._activate_lang('fr_FR')
+        odoo.tools.trans_load(self.cr, 'test_translation_import/i18n/fr.po', 'fr_FR', verbose=False)
         ids = self.env['ir.translation'].search([
             ('lang', '=', 'fr_FR'),
             ('src', '=', 'Test translation with two code lines'),
@@ -118,12 +124,15 @@ class TestTermCount(common.TransactionCase):
     def test_export_empty_string(self):
         """When the string and the translation is equal the translation is empty"""
         # Export the translations
-        def update_translations(context=None):
-            context = dict(context or {}, overwrite=True)
+        def update_translations(create_empty_translation=False):
+            self.env['res.lang']._activate_lang('fr_FR')
             with closing(io.BytesIO()) as bufferobj:
                 odoo.tools.trans_export('fr_FR', ['test_translation_import'], bufferobj, 'po', self.cr)
                 bufferobj.name = 'test_translation_import/i18n/fr.po'
-                odoo.tools.trans_load_data(self.cr, bufferobj, 'po', 'fr_FR', verbose=False, context=context)
+                odoo.tools.trans_load_data(self.cr, bufferobj, 'po', 'fr_FR',
+                                           verbose=False,
+                                           create_empty_translation=create_empty_translation,
+                                           overwrite=True)
 
         # Check that the not translated key is not created
         update_translations()
@@ -131,7 +140,7 @@ class TestTermCount(common.TransactionCase):
         self.assertFalse(translation, 'An empty translation is not imported')
 
         # Check that "Generate Missing Terms" create empty string for not translated key
-        update_translations({'create_empty_translation': True})
+        update_translations(create_empty_translation=True)
         translation = self.env['ir.translation'].search_count([('src', '=', 'Efgh'), ('value', '=', '')])
         self.assertTrue(translation, 'The translation of "Efgh" should be empty')
 
@@ -168,7 +177,7 @@ class TestTermCount(common.TransactionCase):
         trans_count = self.env['ir.translation'].search_count([('lang', '=', 'tlh')])
         self.assertEqual(trans_count, 1, "The imported translations were not created")
 
-        self.env.context = dict(self.env.context, lang="tlh")
+        self.env = self.env(context=dict(self.env.context, lang="tlh"))
         self.assertEqual(_("Klingon"), "tlhIngan", "The code translation was not applied")
 
     def test_lazy_translation(self):
@@ -254,7 +263,7 @@ class TestTermCount(common.TransactionCase):
         export = self.env["base.language.export"].create({
             'lang': 'dot',
             'format': 'po',
-            'modules': [(6, 0, [module.id])]
+            'modules': [Command.set([module.id])]
         })
         export.act_getfile()
         po_file = export.data
@@ -265,6 +274,72 @@ class TestTermCount(common.TransactionCase):
                 self.assertEqual(row['src'], "Accounting")
                 self.assertEqual(row['value'], "samva")
 
+    def test_translation_placeholder(self):
+        """Verify placeholder use in _()"""
+        context = {'lang': "fr_BE"}
+        self.env.ref("base.lang_fr_BE").active = True
+
+        # translation with positional placeholders
+        translation = self.env['ir.translation'].create({
+            'src': 'Text with %s placeholder',
+            'value': 'Text avec %s marqueur',
+            'type': 'code',
+            'name': 'addons/test_translation_import/tests/test_count_term.py',
+            'res_id': 0,
+            'lang': 'fr_BE',
+        })
+
+        # correctly translate
+        self.assertEqual(
+            _("Text with %s placeholder", 1),
+            "Text avec 1 marqueur",
+            "Translation placeholders were not applied"
+        )
+
+        # source error: wrong arguments
+        with self.assertRaises(TypeError), self.cr.savepoint():
+            _("Text with %s placeholder", 1, "🧀")
+
+        # translation error: log error and fallback on source
+        translation.value = "Text avec s% marqueur"
+        with self.assertLogs('odoo.tools.translate', 'ERROR'):
+            self.assertEqual(
+                _("Text with %s placeholder", 1),
+                "Text with 1 placeholder",
+                "Fallback to source was not used for bad translation"
+            )
+
+
+        # translation with named placeholders
+        translation = self.env['ir.translation'].create({
+            'src': 'Text with %(num)s placeholders %(symbol)s',
+            'value': 'Text avec %(num)s marqueurs %(symbol)s',
+            'type': 'code',
+            'name': 'addons/test_translation_import/tests/test_count_term.py',
+            'res_id': 0,
+            'lang': 'fr_BE',
+        })
+
+        # correctly translate
+        self.assertEqual(
+            _("Text with %(num)s placeholders %(symbol)s", num=2, symbol="🧀"),
+            "Text avec 2 marqueurs 🧀",
+            "Translation placeholders were not applied"
+        )
+
+        # source error: wrong arguments
+        with self.assertRaises(KeyError), self.cr.savepoint():
+            _("Text with %(num)s placeholders %(symbol)s", symbol="🧀")
+
+        # translation error: log error and fallback on source
+        translation.value = "Text avec %(num)s marqueurs %(symbole)s"
+        with self.assertLogs('odoo.tools.translate', 'ERROR'):
+            self.assertEqual(
+                _("Text with %(num)s placeholders %(symbol)s", num=2, symbol="🧀"),
+                "Text with 2 placeholders 🧀",
+                "Fallback to source was not used for bad translation"
+            )
+
 
 @tagged('post_install', '-at_install')
 class TestTranslationFlow(common.TransactionCase):
@@ -278,7 +353,8 @@ class TestTranslationFlow(common.TransactionCase):
 
         translations = self.env["ir.translation"].search([
             ('lang', '=', 'fr_FR'),
-            ('module', '=', 'test_translation_import')
+            ('module', '=', 'test_translation_import'),
+            ('value', '!=', ''),
         ])
 
         # minus 3 as the original fr.po contains 3 fake code translations (cf
@@ -289,7 +365,7 @@ class TestTranslationFlow(common.TransactionCase):
         export = self.env["base.language.export"].create({
             'lang': 'fr_FR',
             'format': 'po',
-            'modules': [(6, 0, [module.id])]
+            'modules': [Command.set([module.id])]
         })
         export.act_getfile()
         po_file = export.data
@@ -305,12 +381,14 @@ class TestTranslationFlow(common.TransactionCase):
             'overwrite': False,
         })
         with mute_logger('odoo.addons.base.models.res_lang'):
-            import_fr.with_context(create_empty_translation=True).import_lang()
+            import_fr.import_lang()
 
         import_translation = self.env["ir.translation"].search([
             ('lang', '=', 'fr_FR'),
-            ('module', '=', 'test_translation_import')
+            ('module', '=', 'test_translation_import'),
+            ('value', '!=', ''),
         ])
+
         self.assertEqual(init_translation_count, len(import_translation))
 
     def test_export_import_csv(self):
@@ -321,7 +399,7 @@ class TestTranslationFlow(common.TransactionCase):
         export = self.env["base.language.export"].create({
             'lang': 'fr_FR',
             'format': 'csv',
-            'modules': [(6, 0, [module.id])]
+            'modules': [Command.set([module.id])]
         })
         export.act_getfile()
         po_file = export.data
@@ -341,3 +419,17 @@ class TestTranslationFlow(common.TransactionCase):
         })
         with mute_logger('odoo.addons.base.models.res_lang'):
             import_fr.with_context(create_empty_translation=True).import_lang()
+
+    def test_export_static_templates(self):
+        trans_static = []
+        po_reader = TranslationModuleReader(self.env.cr, ['test_translation_import'])
+        for line in po_reader:
+            module, ttype, name, res_id, source, value, comments = line
+            if name == "addons/test_translation_import/static/src/xml/js_templates.xml":
+                trans_static.append(source)
+
+        self.assertNotIn('no export', trans_static)
+        self.assertIn('do export', trans_static)
+        self.assertIn('text node', trans_static)
+        self.assertIn('slot', trans_static)
+        self.assertIn('slot 2', trans_static)
