@@ -16,7 +16,7 @@ class WebsiteTest(Home):
     def test_view(self, **kwargs):
         return request.render('test_website.test_view')
 
-    @http.route('/ignore_args/converteronly/<string:a>/', type='http', auth="public", website=True, sitemap=False)
+    @http.route('/ignore_args/converteronly/<string:a>', type='http', auth="public", website=True, sitemap=False)
     def test_ignore_args_converter_only(self, a):
         return request.make_response(json.dumps(dict(a=a, kw=None)))
 
@@ -32,7 +32,7 @@ class WebsiteTest(Home):
     def test_ignore_args_kw(self, a, **kw):
         return request.make_response(json.dumps(dict(a=a, kw=kw)))
 
-    @http.route('/ignore_args/converter/<string:a>/', type='http', auth="public", website=True, sitemap=False)
+    @http.route('/ignore_args/converter/<string:a>', type='http', auth="public", website=True, sitemap=False)
     def test_ignore_args_converter(self, a, b='youhou', **kw):
         return request.make_response(json.dumps(dict(a=a, b=b, kw=kw)))
 
@@ -108,18 +108,37 @@ class WebsiteTest(Home):
     def test_denied_error_http(self, **kwargs):
         raise AccessDenied("This is an access denied http test")
 
-    @http.route(['/get'], type='http', auth="public", methods=['GET'], website=True)
+    @http.route(['/get'], type='http', auth="public", methods=['GET'], website=True, sitemap=False)
     def get_method(self, **kw):
         return request.make_response('get')
 
-    @http.route(['/post'], type='http', auth="public", methods=['POST'], website=True)
+    @http.route(['/post'], type='http', auth="public", methods=['POST'], website=True, sitemap=False)
     def post_method(self, **kw):
         return request.make_response('post')
 
-    @http.route(['/get_post'], type='http', auth="public", methods=['GET', 'POST'], website=True)
+    @http.route(['/get_post'], type='http', auth="public", methods=['GET', 'POST'], website=True, sitemap=False)
     def get_post_method(self, **kw):
         return request.make_response('get_post')
 
-    @http.route(['/get_post_nomultilang'], type='http', auth="public", methods=['GET', 'POST'], website=True, multilang=False)
+    @http.route(['/get_post_nomultilang'], type='http', auth="public", methods=['GET', 'POST'], website=True, multilang=False, sitemap=False)
     def get_post_method_no_multilang(self, **kw):
         return request.make_response('get_post_nomultilang')
+
+    # Test Perfs
+
+    @http.route(['/empty_controller_test'], type='http', auth='public', website=True, multilang=False, sitemap=False)
+    def empty_controller_test(self, **kw):
+        return 'Basic Controller Content'
+
+    # Test Redirects
+    @http.route(['/test_website/country/<model("res.country"):country>'], type='http', auth="public", website=True, sitemap=False)
+    def test_model_converter_country(self, country, **kw):
+        return request.render('test_website.test_redirect_view', {'country': country})
+
+    @http.route(['/test_website/200/<model("test.model"):rec>'], type='http', auth="public", website=True, sitemap=False)
+    def test_model_converter_seoname(self, rec, **kw):
+        return request.make_response('ok')
+
+    @http.route(['/test_website/test_redirect_view_qs'], type='http', auth="public", website=True, sitemap=False)
+    def test_redirect_view_qs(self, **kw):
+        return request.render('test_website.test_redirect_view_qs')
