@@ -2,9 +2,13 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import test_controller_args
-from . import test_is_multilang
-from . import test_reset_views
+from . import test_custom_snippet
 from . import test_error
-from . import test_views_during_module_operation
+from . import test_image_upload_progress
+from . import test_is_multilang
 from . import test_multi_company
+from . import test_performance
+from . import test_redirect
+from . import test_reset_views
 from . import test_session
+from . import test_views_during_module_operation
