@@ -7,16 +7,22 @@
     'description': """
 Enable management of UTM trackers: campaign, medium, source.
 """,
-    'version': '1.0',
+    'version': '1.1',
     'depends': ['base', 'web'],
     'data': [
         'data/utm_data.xml',
-        'views/assets.xml',
         'views/utm_campaign_views.xml',
         'views/utm_views.xml',
         'security/ir.model.access.csv',
     ],
-    'demo': [],
+    'demo': [
+        'data/utm_demo.xml',
+    ],
     'auto_install': False,
+    'assets': {
+        'web.assets_backend': [
+            'utm/static/src/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
