@@ -3,4 +3,6 @@
 
 from . import ir_qweb
 from . import ir_http
+from . import ir_ui_menu
 from . import models
+from . import base_document_layout
