@@ -52,14 +52,6 @@ var MediaDialog = Dialog.extend({
             save_text: _t("Add"),
         }, options));
 
-        this.trigger_up('getRecordInfo', {
-            recordInfo: options,
-            type: 'media',
-            callback: function (recordInfo) {
-                _.defaults(options, recordInfo);
-            },
-        });
-
         if (!options.noImages) {
             this.imageWidget = new MediaModules.ImageWidget(this, media, options);
         }
@@ -81,8 +73,8 @@ var MediaDialog = Dialog.extend({
             this.activeWidget = this.videoWidget;
         } else if (this.iconWidget && $media.is('span, i')) {
             this.activeWidget = this.iconWidget;
-        } else if (this.imageWidget) {
-            this.activeWidget = this.imageWidget;
+        } else {
+            this.activeWidget = [this.imageWidget, this.documentWidget, this.videoWidget, this.iconWidget].find(w => !!w);
         }
         this.initiallyActiveWidget = this.activeWidget;
     },
@@ -109,6 +101,8 @@ var MediaDialog = Dialog.extend({
             promises.push(this.videoWidget.appendTo(this.$("#editor-media-video")));
         }
 
+        this.opened(() => this.$('input.o_we_search:visible:first').focus());
+
         return Promise.all(promises);
     },
 
