@@ -5,8 +5,10 @@ var Dialog = require('wysiwyg.widgets.Dialog');
 var AltDialog = require('wysiwyg.widgets.AltDialog');
 var MediaDialog = require('wysiwyg.widgets.MediaDialog');
 var LinkDialog = require('wysiwyg.widgets.LinkDialog');
-var CropImageDialog = require('wysiwyg.widgets.CropImageDialog');
-var ColorpickerDialog = require('web.ColorpickerDialog');
+var LinkTools = require('wysiwyg.widgets.LinkTools');
+var ImageCropWidget = require('wysiwyg.widgets.ImageCropWidget');
+const LinkPopoverWidget = require('@web_editor/js/wysiwyg/widgets/link_popover_widget')[Symbol.for("default")];
+const {ColorpickerDialog} = require('web.Colorpicker');
 
 var media = require('wysiwyg.widgets.media');
 
@@ -15,7 +17,9 @@ return {
     AltDialog: AltDialog,
     MediaDialog: MediaDialog,
     LinkDialog: LinkDialog,
-    CropImageDialog: CropImageDialog,
+    LinkTools: LinkTools,
+    ImageCropWidget: ImageCropWidget,
+    LinkPopoverWidget: LinkPopoverWidget,
     ColorpickerDialog: ColorpickerDialog,
 
     MediaWidget: media.MediaWidget,
