@@ -9,7 +9,9 @@ class Http(models.AbstractModel):
     _inherit = 'ir.http'
 
     def session_info(self):
-        result = super(Http, self).session_info()
+        result = super().session_info()
         if result['is_admin']:
+            demo_modules_count = self.env['ir.module.module'].sudo().search_count([('demo', '=', True)])
             result['web_tours'] = request.env['web_tour.tour'].get_consumed_tours()
+            result['tour_disable'] = demo_modules_count > 0
         return result
