@@ -2,5 +2,5 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import backend
+from . import form
 from . import main
-from . import web_editor
