@@ -2,12 +2,14 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import logging
+import os
 import uuid
-from itertools import groupby
+import werkzeug
 
-from odoo import api, fields, models, _
+from odoo import api, fields, models
 from odoo import tools
-from odoo.addons.http_routing.models.ir_http import url_for
+from odoo.addons import website
+from odoo.exceptions import AccessError
 from odoo.osv import expression
 from odoo.http import request
 
@@ -19,24 +21,66 @@ class View(models.Model):
     _name = "ir.ui.view"
     _inherit = ["ir.ui.view", "website.seo.metadata"]
 
-    customize_show = fields.Boolean("Show As Optional Inherit", default=False)
     website_id = fields.Many2one('website', ondelete='cascade', string="Website")
     page_ids = fields.One2many('website.page', 'view_id')
     first_page_id = fields.Many2one('website.page', string='Website Page', help='First page linked to this view', compute='_compute_first_page_id')
     track = fields.Boolean(string='Track', default=False, help="Allow to specify for one page of the website to be trackable or not")
+    visibility = fields.Selection([('', 'All'), ('connected', 'Signed In'), ('restricted_group', 'Restricted Group'), ('password', 'With Password')], default='')
+    visibility_password = fields.Char(groups='base.group_system', copy=False)
+    visibility_password_display = fields.Char(compute='_get_pwd', inverse='_set_pwd', groups='website.group_website_designer')
+
+    @api.depends('visibility_password')
+    def _get_pwd(self):
+        for r in self:
+            r.visibility_password_display = r.sudo().visibility_password and '********' or ''
+
+    def _set_pwd(self):
+        crypt_context = self.env.user._crypt_context()
+        for r in self:
+            if r.type == 'qweb':
+                r.sudo().visibility_password = r.visibility_password_display and crypt_context.encrypt(r.visibility_password_display) or ''
+                r.visibility = r.visibility  # double check access
 
     def _compute_first_page_id(self):
         for view in self:
             view.first_page_id = self.env['website.page'].search([('view_id', '=', view.id)], limit=1)
 
+    @api.model_create_multi
+    def create(self, vals_list):
+        """
+        SOC for ir.ui.view creation. If a view is created without a website_id,
+        it should get one if one is present in the context. Also check that
+        an explicit website_id in create values matches the one in the context.
+        """
+        website_id = self.env.context.get('website_id', False)
+        if not website_id:
+            return super().create(vals_list)
+
+        for vals in vals_list:
+            if 'website_id' not in vals:
+                # Automatic addition of website ID during view creation if not
+                # specified but present in the context
+                vals['website_id'] = website_id
+            else:
+                # If website ID specified, automatic check that it is the same as
+                # the one in the context. Otherwise raise an error.
+                new_website_id = vals['website_id']
+                if not new_website_id:
+                    raise ValueError(f"Trying to create a generic view from a website {website_id} environment")
+                elif new_website_id != website_id:
+                    raise ValueError(f"Trying to create a view for website {new_website_id} from a website {website_id} environment")
+        return super().create(vals_list)
+
     def name_get(self):
-        if not self._context.get('display_website') and not self.env.user.has_group('website.group_multi_website'):
+        if not (self._context.get('display_key') or self._context.get('display_website')):
             return super(View, self).name_get()
 
         res = []
         for view in self:
             view_name = view.name
-            if view.website_id:
+            if self._context.get('display_key'):
+                view_name += ' <%s>' % view.key
+            if self._context.get('display_website') and view.website_id:
                 view_name += ' [%s]' % view.website_id.name
             res.append((view.id, view_name))
         return res
@@ -59,6 +103,16 @@ class View(models.Model):
             if not view.key and not vals.get('key'):
                 view.with_context(no_cow=True).key = 'website.key_%s' % str(uuid.uuid4())[:6]
 
+            pages = view.page_ids
+
+            # Disable cache of page if we guess some dynamic content (form with csrf, ...)
+            if vals.get('arch'):
+                to_invalidate = pages.filtered(
+                    lambda p: p.cache_time and not p._can_be_cached(vals['arch'])
+                )
+                to_invalidate and _logger.info('Disable cache for page %s', to_invalidate)
+                to_invalidate.cache_time = 0
+
             # No need of COW if the view is already specific
             if view.website_id:
                 super(View, view).write(vals)
@@ -70,7 +124,6 @@ class View(models.Model):
             # but in reality the values were only meant to go on the specific
             # page. Invalidate all fields and not only those in vals because
             # other fields could have been changed implicitly too.
-            pages = view.page_ids
             pages.flush(records=pages)
             pages.invalidate_cache(ids=pages.ids)
 
@@ -166,11 +219,11 @@ class View(models.Model):
 
         if current_website_id and not self._context.get('no_cow'):
             for view in self.filtered(lambda view: not view.website_id):
-                for website in self.env['website'].search([('id', '!=', current_website_id)]):
+                for w in self.env['website'].search([('id', '!=', current_website_id)]):
                     # reuse the COW mechanism to create
                     # website-specific copies, it will take
                     # care of creating pages and menus.
-                    view.with_context(website_id=website.id).write({'name': view.name})
+                    view.with_context(website_id=w.id).write({'name': view.name})
 
         specific_views = self.env['ir.ui.view']
         if self and self.pool._init:
@@ -184,10 +237,15 @@ class View(models.Model):
     def _create_website_specific_pages_for_view(self, new_view, website):
         for page in self.page_ids:
             # create new pages for this view
-            page.copy({
+            new_page = page.copy({
                 'view_id': new_view.id,
                 'is_published': page.is_published,
             })
+            page.menu_ids.filtered(lambda m: m.website_id.id == website.id).page_id = new_page.id
+
+    def _get_top_level_view(self):
+        self.ensure_one()
+        return self.inherit_id._get_top_level_view() if self.inherit_id else self
 
     @api.model
     def get_related_views(self, key, bundles=False):
@@ -245,41 +303,49 @@ class View(models.Model):
             return view_id if view_id._name == 'ir.ui.view' else self.env['ir.ui.view']
 
     @api.model
-    def _get_inheriting_views_arch_website(self, view_id):
-        return self.env['website'].browse(self._context.get('website_id'))
-
-    @api.model
-    def _get_inheriting_views_arch_domain(self, view_id, model):
-        domain = super(View, self)._get_inheriting_views_arch_domain(view_id, model)
-        current_website = self._get_inheriting_views_arch_website(view_id)
+    def _get_inheriting_views_domain(self):
+        domain = super(View, self)._get_inheriting_views_domain()
+        current_website = self.env['website'].browse(self._context.get('website_id'))
         website_views_domain = current_website.website_domain()
         # when rendering for the website we have to include inactive views
         # we will prefer inactive website-specific views over active generic ones
         if current_website:
             domain = [leaf for leaf in domain if 'active' not in leaf]
-
         return expression.AND([website_views_domain, domain])
 
     @api.model
-    def get_inheriting_views_arch(self, view_id, model):
+    def _get_inheriting_views(self):
         if not self._context.get('website_id'):
-            return super(View, self).get_inheriting_views_arch(view_id, model)
-
-        get_inheriting_self = self.with_context(active_test=False)
-        if self.pool._init and not self._context.get('load_all_views'):
-            view = self.browse(view_id)
-            if view.website_id:
-                original_view = view._get_original_view()
-                original_keys = self.with_context(website_id=False)._get_inheriting_views(original_view.id, model).mapped('key')
-                specific_views = self.search([('key', 'in', original_keys), ('website_id', '=', self._context.get('website_id'))])
-                check_view_ids = list(self._context.get('check_view_ids') or ()) + specific_views.ids
-                get_inheriting_self = self.with_context(check_view_ids=check_view_ids)
-        inheriting_views = super(View, get_inheriting_self).get_inheriting_views_arch(view_id, model)
+            return super(View, self)._get_inheriting_views()
 
+        views = super(View, self.with_context(active_test=False))._get_inheriting_views()
         # prefer inactive website-specific views over active generic ones
-        inheriting_views = self.browse([view[1] for view in inheriting_views]).filter_duplicate().filtered('active')
+        return views.filter_duplicate().filtered('active')
 
-        return [(view.arch, view.id) for view in inheriting_views]
+    @api.model
+    def _get_filter_xmlid_query(self):
+        """This method add some specific view that do not have XML ID
+        """
+        if not self._context.get('website_id'):
+            return super()._get_filter_xmlid_query()
+        else:
+            return """SELECT res_id
+                    FROM   ir_model_data
+                    WHERE  res_id IN %(res_ids)s
+                        AND model = 'ir.ui.view'
+                        AND module  IN %(modules)s
+                    UNION
+                    SELECT sview.id
+                    FROM   ir_ui_view sview
+                        INNER JOIN ir_ui_view oview USING (key)
+                        INNER JOIN ir_model_data d
+                                ON oview.id = d.res_id
+                                    AND d.model = 'ir.ui.view'
+                                    AND d.module  IN %(modules)s
+                    WHERE  sview.id IN %(res_ids)s
+                        AND sview.website_id IS NOT NULL
+                        AND oview.website_id IS NULL;
+                    """
 
     @api.model
     @tools.ormcache_context('self.env.uid', 'self.env.su', 'xml_id', keys=('website_id',))
@@ -299,24 +365,51 @@ class View(models.Model):
             current_website = self.env['website'].browse(self._context.get('website_id'))
             domain = ['&', ('key', '=', xml_id)] + current_website.website_domain()
 
-            view = self.search(domain, order='website_id', limit=1)
+            view = self.sudo().search(domain, order='website_id', limit=1)
             if not view:
                 _logger.warning("Could not find view object with xml_id '%s'", xml_id)
                 raise ValueError('View %r in website %r not found' % (xml_id, self._context['website_id']))
             return view.id
-        return super(View, self).get_view_id(xml_id)
+        return super(View, self.sudo()).get_view_id(xml_id)
 
-    def _get_original_view(self):
-        """Given a view, retrieve the original view it was COW'd from.
-        The given view might already be the original one. In that case it will
-        (and should) return itself.
+    def _handle_visibility(self, do_raise=True):
+        """ Check the visibility set on the main view and raise 403 if you should not have access.
+            Order is: Public, Connected, Has group, Password
+
+            It only check the visibility on the main content, others views called stay available in rpc.
         """
-        self.ensure_one()
-        domain = [('key', '=', self.key), ('model_data_id', '!=', None)]
-        return self.with_context(active_test=False).search(domain, limit=1)  # Useless limit has multiple xmlid should not be possible
+        error = False
 
-    def render(self, values=None, engine='ir.qweb', minimal_qcontext=False):
+        self = self.sudo()
+
+        if self.visibility and not request.env.user.has_group('website.group_website_designer'):
+            if (self.visibility == 'connected' and request.website.is_public_user()):
+                error = werkzeug.exceptions.Forbidden()
+            elif self.visibility == 'password' and \
+                    (request.website.is_public_user() or self.id not in request.session.get('views_unlock', [])):
+                pwd = request.params.get('visibility_password')
+                if pwd and self.env.user._crypt_context().verify(
+                        pwd, self.sudo().visibility_password):
+                    request.session.setdefault('views_unlock', list()).append(self.id)
+                else:
+                    error = werkzeug.exceptions.Forbidden('website_visibility_password_required')
+
+            if self.visibility not in ('password', 'connected'):
+                try:
+                    self._check_view_access()
+                except AccessError:
+                    error = werkzeug.exceptions.Forbidden()
+
+        if error:
+            if do_raise:
+                raise error
+            else:
+                return False
+        return True
+
+    def _render(self, values=None, engine='ir.qweb', minimal_qcontext=False):
         """ Render the template. If website is enabled on request, then extend rendering context with website values. """
+        self._handle_visibility(do_raise=True)
         new_context = dict(self._context)
         if request and getattr(request, 'is_frontend', False):
 
@@ -333,15 +426,11 @@ class View(models.Model):
             if values and 'main_object' in values:
                 if request.env.user.has_group('website.group_website_publisher'):
                     func = getattr(values['main_object'], 'get_backend_menu_id', False)
-                    values['backend_menu_id'] = func and func() or self.env.ref('website.menu_website_configuration').id
-
-                # Fallback incase main_object dont't inherit 'website.seo.metadata'
-                if not hasattr(values['main_object'], 'get_website_meta'):
-                    values['main_object'].get_website_meta = lambda: {}
+                    values['backend_menu_id'] = func and func() or self.env['ir.model.data']._xmlid_to_res_id('website.menu_website_configuration')
 
         if self._context != new_context:
             self = self.with_context(new_context)
-        return super(View, self).render(values, engine=engine, minimal_qcontext=minimal_qcontext)
+        return super(View, self)._render(values, engine=engine, minimal_qcontext=minimal_qcontext)
 
     @api.model
     def _prepare_qcontext(self):
@@ -372,17 +461,12 @@ class View(models.Model):
                 ]
 
             qcontext.update(dict(
-                self._context.copy(),
                 main_object=self,
                 website=request.website,
-                url_for=url_for,
-                res_company=request.website.company_id.sudo(),
-                default_lang_code=request.env['ir.http']._get_default_lang().code,
-                languages=request.env['res.lang'].get_available(),
+                is_view_active=request.website.is_view_active,
+                res_company=request.env['res.company'].browse(request.website._get_cached('company_id')).sudo(),
                 translatable=translatable,
                 editable=editable,
-                # retrocompatibility, remove me in master
-                menu_data={'children': []} if request.website.is_user() else None,
             ))
 
         return qcontext
@@ -438,3 +522,15 @@ class View(models.Model):
             if website_specific_view:
                 self = website_specific_view
         super(View, self).save(value, xpath=xpath)
+
+    # --------------------------------------------------------------------------
+    # Snippet saving
+    # --------------------------------------------------------------------------
+
+    @api.model
+    def _snippet_save_view_values_hook(self):
+        res = super()._snippet_save_view_values_hook()
+        website_id = self.env.context.get('website_id')
+        if website_id:
+            res['website_id'] = website_id
+        return res
