@@ -1,27 +1,40 @@
+/* global google, gapi */
 odoo.define('website.backend.dashboard', function (require) {
 'use strict';
 
 var AbstractAction = require('web.AbstractAction');
 var ajax = require('web.ajax');
-var ControlPanelMixin = require('web.ControlPanelMixin');
 var core = require('web.core');
 var Dialog = require('web.Dialog');
 var field_utils = require('web.field_utils');
+var pyUtils = require('web.py_utils');
 var session = require('web.session');
+var time = require('web.time');
 var web_client = require('web.web_client');
 
 var _t = core._t;
 var QWeb = core.qweb;
 
-var Dashboard = AbstractAction.extend(ControlPanelMixin, {
-    template: 'website.WebsiteDashboardMain',
-    cssLibs: [
-        '/web/static/lib/nvd3/nv.d3.css'
-    ],
+var COLORS = ["#1f77b4", "#aec7e8"];
+var FORMAT_OPTIONS = {
+    // allow to decide if utils.human_number should be used
+    humanReadable: function (value) {
+        return Math.abs(value) >= 1000;
+    },
+    // with the choices below, 1236 is represented by 1.24k
+    minDigits: 1,
+    decimals: 2,
+    // avoid comma separators for thousands in numbers when human_number is used
+    formatterCallback: function (str) {
+        return str;
+    },
+};
+
+var Dashboard = AbstractAction.extend({
+    hasControlPanel: true,
+    contentTemplate: 'website.WebsiteDashboardMain',
     jsLibs: [
-        '/web/static/lib/nvd3/d3.v3.js',
-        '/web/static/lib/nvd3/nv.d3.js',
-        '/web/static/src/js/libs/nvd3.js'
+        '/web/static/lib/Chart/Chart.js',
     ],
     events: {
         'click .js_link_analytics_settings': 'on_link_analytics_settings',
@@ -32,17 +45,19 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
     init: function(parent, context) {
         this._super(parent, context);
 
+        this.DATE_FORMAT = time.getLangDateFormat();
         this.date_range = 'week';  // possible values : 'week', 'month', year'
         this.date_from = moment.utc().subtract(1, 'week');
         this.date_to = moment.utc();
 
         this.dashboards_templates = ['website.dashboard_header', 'website.dashboard_content'];
         this.graphs = [];
+        this.chartIds = {};
     },
 
     willStart: function() {
         var self = this;
-        return $.when(ajax.loadLibs(this), this._super()).then(function() {
+        return Promise.all([ajax.loadLibs(this), this._super()]).then(function() {
             return self.fetch_data();
         }).then(function(){
             var website = _.findWhere(self.websites, {selected: true});
@@ -52,32 +67,42 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
 
     start: function() {
         var self = this;
+        this._computeControlPanelProps();
         return this._super().then(function() {
-            self.update_cp();
             self.render_graphs();
-            self.$el.parent().addClass('oe_background_grey');
         });
     },
 
+    on_attach_callback: function () {
+        this._isInDom = true;
+        this.render_graphs();
+        this._super.apply(this, arguments);
+    },
+    on_detach_callback: function () {
+        this._isInDom = false;
+        this._super.apply(this, arguments);
+    },
     /**
      * Fetches dashboard data
      */
     fetch_data: function() {
         var self = this;
-        return this._rpc({
+        var prom = this._rpc({
             route: '/website/fetch_dashboard_data',
             params: {
                 website_id: this.website_id || false,
                 date_from: this.date_from.year()+'-'+(this.date_from.month()+1)+'-'+this.date_from.date(),
                 date_to: this.date_to.year()+'-'+(this.date_to.month()+1)+'-'+this.date_to.date(),
             },
-        }).done(function(result) {
+        });
+        prom.then(function (result) {
             self.data = result;
             self.dashboards_data = result.dashboards;
             self.currency_id = result.currency_id;
             self.groups = result.groups;
             self.websites = result.websites;
         });
+        return prom;
     },
 
     on_link_analytics_settings: function(ev) {
@@ -86,7 +111,7 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
         var self = this;
         var dialog = new Dialog(this, {
             size: 'medium',
-            title: _t('Google Analytics'),
+            title: _t('Connect Google Analytics'),
             $content: QWeb.render('website.ga_dialog_content', {
                 ga_key: this.dashboards_data.visits.ga_client_id,
                 ga_analytics_key: this.dashboards_data.visits.ga_analytics_key,
@@ -110,6 +135,12 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
         }).open();
     },
 
+    on_go_to_website: function (ev) {
+        ev.preventDefault();
+        var website = _.findWhere(this.websites, {selected: true});
+        window.location.href = `/website/force/${website.id}`;
+    },
+
     on_save_ga_client_id: function(ga_client_id, ga_analytics_key) {
         var self = this;
         return this._rpc({
@@ -121,7 +152,7 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
             },
         }).then(function (result) {
             if (result.error) {
-                self.do_warn(result.error.title, result.error.message);
+                self.displayNotification({ title: result.error.title, message: result.error.message, type: 'danger' });
                 return;
             }
             self.on_date_range_button('week');
@@ -135,60 +166,110 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
         });
     },
 
-    render_graph: function(div_to_display, chart_values) {
-        this.$(div_to_display).empty();
-
+    render_graph: function(div_to_display, chart_values, chart_id) {
         var self = this;
-        nv.addGraph(function() {
-            var chart = nv.models.lineChart()
-                .x(function(d) { return self.getDate(d); })
-                .y(function(d) { return self.getValue(d); })
-                .forceY([0]);
-            chart
-                .useInteractiveGuideline(true)
-                .showLegend(false)
-                .showYAxis(true)
-                .showXAxis(true);
 
-            var tick_values = self.getPrunedTickValues(chart_values[0].values, 5);
+        this.$(div_to_display).empty();
+        var $canvasContainer = $('<div/>', {class: 'o_graph_canvas_container'});
+        this.$canvas = $('<canvas/>').attr('id', chart_id);
+        $canvasContainer.append(this.$canvas);
+        this.$(div_to_display).append($canvasContainer);
 
-            chart.xAxis
-                .tickFormat(function(d) { return d3.time.format("%m/%d/%y")(new Date(d)); })
-                .tickValues(_.map(tick_values, function(d) { return self.getDate(d); }))
-                .rotateLabels(-45);
+        var labels = chart_values[0].values.map(function (date) {
+            return moment(date[0], "YYYY-MM-DD", 'en');
+        });
 
-            chart.interactiveLayer.tooltip.contentGenerator(function(data) {
-                return QWeb.render('website.SalesChartTooltip', {
-                    format: field_utils.format,
-                    chartData: data,
-                    dateRange: self.date_range
-                });
-            });
+        var datasets = chart_values.map(function (group, index) {
+            return {
+                label: group.key,
+                data: group.values.map(function (value) {
+                    return value[1];
+                }),
+                dates: group.values.map(function (value) {
+                    return value[0];
+                }),
+                fill: false,
+                borderColor: COLORS[index],
+            };
+        });
 
-            chart.yAxis
-                .tickFormat(d3.format('.02f'));
-
-            var svg = d3.select(div_to_display)
-                .append("svg");
-
-            svg
-                .attr("height", '24em')
-                .datum(chart_values)
-                .call(chart);
-
-            nv.utils.windowResize(chart.update);
-            return chart;
+        var ctx = this.$canvas[0];
+        this.chart = new Chart(ctx, {
+            type: 'line',
+            data: {
+                labels: labels,
+                datasets: datasets,
+            },
+            options: {
+                legend: {
+                    display: false,
+                },
+                maintainAspectRatio: false,
+                scales: {
+                    yAxes: [{
+                        type: 'linear',
+                        ticks: {
+                            beginAtZero: true,
+                            callback: this.formatValue.bind(this),
+                        },
+                    }],
+                    xAxes: [{
+                        ticks: {
+                            callback: function (moment) {
+                                return moment.format(self.DATE_FORMAT);
+                            },
+                        }
+                    }],
+                },
+                tooltips: {
+                    mode: 'index',
+                    intersect: false,
+                    bodyFontColor: 'rgba(0,0,0,1)',
+                    titleFontSize: 13,
+                    titleFontColor: 'rgba(0,0,0,1)',
+                    backgroundColor: 'rgba(255,255,255,0.6)',
+                    borderColor: 'rgba(0,0,0,0.2)',
+                    borderWidth: 2,
+                    callbacks: {
+                        title: function (tooltipItems, data) {
+                            return data.datasets[0].label;
+                        },
+                        label: function (tooltipItem, data) {
+                            var moment = data.labels[tooltipItem.index];
+                            var date = tooltipItem.datasetIndex === 0 ?
+                                        moment :
+                                        moment.subtract(1, self.date_range);
+                            return date.format(self.DATE_FORMAT) + ': ' + self.formatValue(tooltipItem.yLabel);
+                        },
+                        labelColor: function (tooltipItem, chart) {
+                            var dataset = chart.data.datasets[tooltipItem.datasetIndex];
+                            return {
+                                borderColor: dataset.borderColor,
+                                backgroundColor: dataset.borderColor,
+                            };
+                        },
+                    }
+                }
+            }
         });
     },
 
     render_graphs: function() {
         var self = this;
-        _.each(this.graphs, function(e) {
-            if (self.groups[e.group]) {
-                self.render_graph('.o_graph_' + e.name, self.dashboards_data[e.name].graph);
-            }
-        });
-        this.render_graph_analytics(this.dashboards_data.visits.ga_client_id);
+        if (this._isInDom) {
+            _.each(this.graphs, function(e) {
+                var renderGraph = self.groups[e.group] &&
+                                    self.dashboards_data[e.name].summary.order_count;
+                if (!self.chartIds[e.name]) {
+                    self.chartIds[e.name] = _.uniqueId('chart_' + e.name);
+                }
+                var chart_id = self.chartIds[e.name];
+                if (renderGraph) {
+                    self.render_graph('.o_graph_' + e.name, self.dashboards_data[e.name].graph, chart_id);
+                }
+            });
+            this.render_graph_analytics(this.dashboards_data.visits.ga_client_id);
+        }
     },
 
     render_graph_analytics: function(client_id) {
@@ -243,7 +324,7 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
         }
 
         var self = this;
-        $.when(this.fetch_data()).then(function() {
+        Promise.resolve(this.fetch_data()).then(function () {
             self.$('.o_website_dashboard').empty();
             self.render_dashboards();
             self.render_graphs();
@@ -254,7 +335,7 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
     on_website_button: function(website_id) {
         var self = this;
         this.website_id = website_id;
-        $.when(this.fetch_data()).then(function() {
+        Promise.resolve(this.fetch_data()).then(function () {
             self.$('.o_website_dashboard').empty();
             self.render_dashboards();
             self.render_graphs();
@@ -264,7 +345,6 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
     on_reverse_breadcrumb: function() {
         var self = this;
         web_client.do_push_state({});
-        this.update_cp();
         this.fetch_data().then(function() {
             self.$('.o_website_dashboard').empty();
             self.render_dashboards();
@@ -274,6 +354,7 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
 
     on_dashboard_action: function (ev) {
         ev.preventDefault();
+        var self = this
         var $action = $(ev.currentTarget);
         var additional_context = {};
         if (this.date_range === 'week') {
@@ -283,9 +364,18 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
         } else if (this.date_range === 'year') {
             additional_context = {search_default_year: true};
         }
-        this.do_action($action.attr('name'), {
-            additional_context: additional_context,
-            on_reverse_breadcrumb: this.on_reverse_breadcrumb
+        this._rpc({
+            route: '/web/action/load',
+            params: {
+                'action_id': $action.attr('name'),
+            },
+        })
+        .then(function (action) {
+            action.domain = pyUtils.assembleDomains([action.domain, `[('website_id', '=', ${self.website_id})]`]);
+            return self.do_action(action, {
+                'additional_context': additional_context,
+                'on_reverse_breadcrumb': self.on_reverse_breadcrumb
+            });
         });
     },
 
@@ -303,29 +393,28 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
         });
     },
 
-    update_cp: function() {
-        var self = this;
-        if (!this.$searchview) {
-            this.$searchview = $(QWeb.render("website.DateRangeButtons", {
-                widget: this,
-            }));
-            this.$searchview.find('button.js_date_range').click(function(ev) {
-                self.$searchview.find('button.js_date_range.active').removeClass('active');
-                $(ev.target).addClass('active');
-                self.on_date_range_button($(ev.target).data('date'));
-            });
-            this.$searchview.find('button.js_website').click(function(ev) {
-                self.$searchview.find('button.js_website.active').removeClass('active');
-                $(ev.target).addClass('active');
-                self.on_website_button($(ev.target).data('website-id'));
-            });
-        }
-        this.update_control_panel({
-            cp_content: {
-                $searchview: this.$searchview,
-                $buttons: QWeb.render("website.GoToButtons"),
-            },
+    /**
+     * @private
+     */
+    _computeControlPanelProps() {
+        const $searchview = $(QWeb.render("website.DateRangeButtons", {
+            widget: this,
+        }));
+        $searchview.find('button.js_date_range').click((ev) => {
+            $searchview.find('button.js_date_range.active').removeClass('active');
+            $(ev.target).addClass('active');
+            this.on_date_range_button($(ev.target).data('date'));
         });
+        $searchview.find('button.js_website').click((ev) => {
+            $searchview.find('button.js_website.active').removeClass('active');
+            $(ev.target).addClass('active');
+            this.on_website_button($(ev.target).data('website-id'));
+        });
+
+        const $buttons = $(QWeb.render("website.GoToButtons"));
+        $buttons.on('click', this.on_go_to_website.bind(this));
+
+        this.controlPanelProps.cp_content = { $searchview, $buttons };
     },
 
     // Loads Analytics API
@@ -596,19 +685,10 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
 
     // Utility functions
     addLoader: function(selector) {
-        var loader = '<span class="fa fa-3x fa-spin fa-spinner fa-pulse"/>';
+        var loader = '<span class="fa fa-3x fa-spin fa-circle-o-notch fa-spin"/>';
         selector.html("<div class='o_loader'>" + loader + "</div>");
     },
-    getDate: function(d) { return new Date(d[0]); },
     getValue: function(d) { return d[1]; },
-    getPrunedTickValues: function(ticks, nb_desired_ticks) {
-        var nb_values = ticks.length;
-        var keep_one_of = Math.max(1, Math.floor(nb_values / nb_desired_ticks));
-
-        return _.filter(ticks, function(d, i) {
-            return i % keep_one_of === 0;
-        });
-    },
     format_number: function(value, type, digits, symbol) {
         if (type === 'currency') {
             return this.render_monetary_field(value, this.currency_id);
@@ -616,6 +696,11 @@ var Dashboard = AbstractAction.extend(ControlPanelMixin, {
             return field_utils.format[type](value || 0, {digits: digits}) + ' ' + symbol;
         }
     },
+    formatValue: function (value) {
+        var formatter = field_utils.format.float;
+        var formatedValue = formatter(value, undefined, FORMAT_OPTIONS);
+        return formatedValue;
+    },
     render_monetary_field: function(value, currency_id) {
         var currency = session.get_currency(currency_id);
         var formatted_value = field_utils.format.float(value || 0, {digits: currency && currency.digits});
