@@ -1,14 +1,15 @@
-odoo.define('website.contentMenu', function (require) {
-'use strict';
+/** @odoo-module alias=website.contentMenu */
 
-var Class = require('web.Class');
-var core = require('web.core');
-var Dialog = require('web.Dialog');
-var time = require('web.time');
-var weWidgets = require('wysiwyg.widgets');
-var websiteNavbarData = require('website.navbar');
-var websiteRootData = require('website.root');
-var Widget = require('web.Widget');
+import Class from 'web.Class';
+import core from 'web.core';
+import Dialog from 'web.Dialog';
+import time from 'web.time';
+import weWidgets from 'wysiwyg.widgets';
+import websiteNavbarData from 'website.navbar';
+import Widget from 'web.Widget';
+import { Markup } from 'web.utils';
+
+import { registry } from "@web/core/registry";
 
 var _t = core._t;
 var qweb = core.qweb;
@@ -22,6 +23,10 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
         'keyup input#page_name': '_onNameChanged',
         'keyup input#page_url': '_onUrlChanged',
         'change input#create_redirect': '_onCreateRedirectChanged',
+        'click input#visibility_password': '_onPasswordClicked',
+        'change input#visibility_password': '_onPasswordChanged',
+        'change select#visibility': '_onVisibilityChanged',
+        'error.datetimepicker': '_onDateTimePickerError',
     }),
 
     /**
@@ -43,22 +48,32 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
 
         var buttons = [
             {text: _t("Save"), classes: 'btn-primary', click: this.save},
-            {text: _t("Discard"), close: true},
+            {text: _t("Discard"), classes: 'mr-auto', close: true},
         ];
         if (options.fromPageManagement) {
             buttons.push({
                 text: _t("Go To Page"),
                 icon: 'fa-globe',
-                classes: 'btn-link float-right',
+                classes: 'btn-link',
                 click: function (e) {
                     window.location.href = '/' + self.page.url;
                 },
             });
         }
+        buttons.push({
+            text: _t("Duplicate Page"),
+            icon: 'fa-clone',
+            classes: 'btn-link',
+            click: function (e) {
+                // modal('hide') will break the rpc, so hide manually
+                this.$el.closest('.modal').addClass('d-none');
+                _clonePage.call(this, self.page_id);
+            },
+        });
         buttons.push({
             text: _t("Delete Page"),
             icon: 'fa-trash',
-            classes: 'btn-link float-right',
+            classes: 'btn-link',
             click: function (e) {
                 _deletePage.call(this, self.page_id, options.fromPageManagement);
             },
@@ -78,18 +93,12 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
 
         defs.push(this._rpc({
             model: 'website.page',
-            method: 'get_page_info',
+            method: 'get_page_properties',
             args: [this.page_id],
         }).then(function (page) {
-            page[0].url = _.str.startsWith(page[0].url, '/') ? page[0].url.substring(1) : page[0].url;
-            self.page = page[0];
-        }));
-
-        defs.push(this._rpc({
-            model: 'website.rewrite',
-            method: 'fields_get',
-        }).then(function (fields) {
-            self.fields = fields;
+            page.url = _.str.startsWith(page.url, '/') ? page.url.substring(1) : page.url;
+            page.hasSingleGroup = page.group_id !== undefined;
+            self.page = page;
         }));
 
         return Promise.all(defs);
@@ -105,6 +114,13 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
         this.$('.ask_for_redirect').addClass('d-none');
         this.$('.redirect_type').addClass('d-none');
         this.$('.warn_about_call').addClass('d-none');
+        if (this.page.visibility !== 'password') {
+            this.$('.show_visibility_password').addClass('d-none');
+        }
+        if (this.page.visibility !== 'restricted_group') {
+            this.$('.show_group_id').addClass('d-none');
+        }
+        this.autocompleteWithGroups(this.$('#group_id'));
 
         defs.push(this._getPageDependencies(this.page_id)
         .then(function (dependencies) {
@@ -116,8 +132,20 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
             });
             dep_text = dep_text.join(', ');
             self.$('#dependencies_redirect').html(qweb.render('website.show_page_dependencies', { dependencies: dependencies, dep_text: dep_text }));
-            self.$('#dependencies_redirect [data-toggle="popover"]').popover({
-                container: 'body',
+            self.$('a.o_dependencies_redirect_link').on('click', () => {
+                self.$('.o_dependencies_redirect_list_popover').popover({
+                    html: true,
+                    title: _t('Dependencies'),
+                    boundary: 'viewport',
+                    placement: 'right',
+                    trigger: 'focus',
+                    content: () => {
+                        return qweb.render('website.get_tooltip_dependencies', {
+                            dependencies: dependencies,
+                        });
+                    },
+                    template: qweb.render('website.page_dependencies_popover'),
+                }).popover('toggle');
             });
         }));
 
@@ -151,7 +179,7 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
                   }));
 
         var datepickersOptions = {
-            minDate: moment({y: 1900}),
+            minDate: moment({ y: 1000 }),
             maxDate: moment().add(200, 'y'),
             calendarWeeks: true,
             icons : {
@@ -174,7 +202,6 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
             datepickersOptions.defaultDate = time.str_to_datetime(this.page.date_publish);
         }
         this.$('#date_publish_container').datetimepicker(datepickersOptions);
-
         return Promise.all(defs);
     },
     /**
@@ -202,13 +229,13 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
         });
         var url = this.$('#page_url').val();
 
-        var $date_publish = this.$("#date_publish");
-        $date_publish.closest(".form-group").removeClass('o_has_error').find('.form-control, .custom-select').removeClass('is-invalid');
-        var date_publish = $date_publish.val();
-        if (date_publish !== "") {
-            date_publish = this._parse_date(date_publish);
-            if (!date_publish) {
-                $date_publish.closest(".form-group").addClass('o_has_error').find('.form-control, .custom-select').addClass('is-invalid');
+        var $datePublish = this.$("#date_publish");
+        $datePublish.closest(".form-group").removeClass('o_has_error').find('.form-control, .custom-select').removeClass('is-invalid');
+        var datePublish = $datePublish.val();
+        if (datePublish !== "") {
+            datePublish = this._parse_date(datePublish);
+            if (!datePublish) {
+                $datePublish.closest(".form-group").addClass('o_has_error').find('.form-control, .custom-select').addClass('is-invalid');
                 return;
             }
         }
@@ -223,8 +250,22 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
             create_redirect: this.$('#create_redirect').prop('checked'),
             redirect_type: this.$('#redirect_type').val(),
             website_indexed: this.$('#is_indexed').prop('checked'),
-            date_publish: date_publish,
+            visibility: this.$('#visibility').val(),
+            date_publish: datePublish,
         };
+        if (this.page.hasSingleGroup && this.$('#visibility').val() === 'restricted_group') {
+            params['group_id'] = this.$('#group_id').data('group-id');
+        }
+        if (this.$('#visibility').val() === 'password') {
+            var field_pwd = $('#visibility_password');
+            if (!field_pwd.get(0).reportValidity()) {
+                return;
+            }
+            if (field_pwd.data('dirty')) {
+                params['visibility_pwd'] = field_pwd.val();
+            }
+        }
+
         this._rpc({
             model: 'website.page',
             method: 'save_page_info',
@@ -326,6 +367,37 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
             return false;
         }
     },
+    /**
+     * Allows the given input to propose existing groups.
+     *
+     * @param {jQuery} $input
+     */
+    autocompleteWithGroups: function ($input) {
+        $input.autocomplete({
+            source: (request, response) => {
+                return this._rpc({
+                    model: 'res.groups',
+                    method: 'search_read',
+                    args: [[['name', 'ilike', request.term]], ['display_name']],
+                    kwargs: {
+                        limit: 15,
+                    },
+                }).then(founds => {
+                    founds = founds.map(g => ({'id': g['id'], 'label': g['display_name']}));
+                    response(founds);
+                });
+            },
+            change: (ev, ui) => {
+                var $target = $(ev.target);
+                if (!ui.item) {
+                    $target.val("");
+                    $target.removeData('group-id');
+                } else {
+                    $target.data('group-id', ui.item.id);
+                }
+            },
+        });
+    },
 
     //--------------------------------------------------------------------------
     // Handlers
@@ -358,13 +430,38 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
         var createRedirect = this.$('input#create_redirect').prop('checked');
         this.$('.redirect_type').toggleClass('d-none', !createRedirect);
     },
+    /**
+     * @private
+     */
+    _onVisibilityChanged: function (ev) {
+        this.$('.show_visibility_password').toggleClass('d-none', ev.target.value !== 'password');
+        this.$('.show_group_id').toggleClass('d-none', ev.target.value !== 'restricted_group');
+        this.$('#visibility_password').attr('required', ev.target.value === 'password');
+    },
+    /**
+     * Library clears the wrong date format so just ignore error
+     *
+     * @private
+     */
+    _onDateTimePickerError: function (ev) {
+        return false;
+    },
+    /**
+     * @private
+     */
+    _onPasswordClicked: function (ev) {
+        ev.target.value = '';
+        this._onPasswordChanged();
+    },
+    /**
+     * @private
+     */
+    _onPasswordChanged: function () {
+        this.$('#visibility_password').data('dirty', 1);
+    },
 });
 
 var MenuEntryDialog = weWidgets.LinkDialog.extend({
-    xmlDependencies: weWidgets.LinkDialog.prototype.xmlDependencies.concat(
-        ['/website/static/src/xml/website.contentMenu.xml']
-    ),
-
     /**
      * @constructor
      */
@@ -373,16 +470,34 @@ var MenuEntryDialog = weWidgets.LinkDialog.extend({
             title: _t("Add a menu item"),
         }, options || {}), editable, _.extend({
             needLabel: true,
-            text: data.name || '',
+            content: data.name || '',
             isNewWindow: data.new_window,
         }, data || {}));
 
+        this.linkWidget.xmlDependencies = this.linkWidget.xmlDependencies.concat(['/website/static/src/xml/website.contentMenu.xml']);
+
+        const oldSave = this.linkWidget.save;
+        /**
+         * @override
+         */
+        this.linkWidget.save = () => {
+            var $e = this.$('#o_link_dialog_label_input');
+            if (!$e.val() || !$e[0].checkValidity()) {
+                $e.closest('.form-group').addClass('o_has_error').find('.form-control, .custom-select').addClass('is-invalid');
+                $e.focus();
+                return Promise.reject();
+            }
+            return oldSave.bind(this.linkWidget)();
+        };
+
         this.menuType = data.menuType;
     },
     /**
      * @override
      */
-    start: function () {
+    start: async function () {
+        const res = await this._super(...arguments);
+
         // Remove style related elements
         this.$('.o_link_dialog_preview').remove();
         this.$('input[name="is_new_window"], .link-style').closest('.form-group').remove();
@@ -399,24 +514,7 @@ var MenuEntryDialog = weWidgets.LinkDialog.extend({
             $url.closest('.form-group').addClass('d-none');
         }
 
-        return this._super.apply(this, arguments);
-    },
-
-    //--------------------------------------------------------------------------
-    // Public
-    //--------------------------------------------------------------------------
-
-    /**
-     * @override
-     */
-    save: function () {
-        var $e = this.$('#o_link_dialog_label_input');
-        if (!$e.val() || !$e[0].checkValidity()) {
-            $e.closest('.form-group').addClass('o_has_error').find('.form-control, .custom-select').addClass('is-invalid');
-            $e.focus();
-            return;
-        }
-        return this._super.apply(this, arguments);
+        return res;
     },
 });
 
@@ -606,7 +704,7 @@ var EditMenuDialog = weWidgets.Dialog.extend({
             var newMenu = {
                 'fields': {
                     'id': _.uniqueId('new-'),
-                    'name': link.text,
+                    'name': _.unescape(link.content),
                     'url': link.url,
                     'new_window': link.isNewWindow,
                     'is_mega_menu': menuType === 'mega',
@@ -652,7 +750,7 @@ var EditMenuDialog = weWidgets.Dialog.extend({
             }, menu.fields));
             dialog.on('save', this, link => {
                 _.extend(menu.fields, {
-                    'name': link.text,
+                    'name': _.unescape(link.content),
                     'url': link.url,
                     'new_window': link.isNewWindow,
                 });
@@ -719,6 +817,12 @@ var ContentMenu = websiteNavbarData.WebsiteNavbarActionWidget.extend({
             $('#wrapwrap > header').removeClass(this.value)
                                    .addClass(value);
         },
+        header_visible: function (value) {
+            $('#wrapwrap > header').toggleClass('d-none o_snippet_invisible', !value);
+        },
+        footer_visible: function (value) {
+            $('#wrapwrap > footer').toggleClass('d-none o_snippet_invisible', !value);
+        },
     },
 
     /**
@@ -787,6 +891,7 @@ var ContentMenu = websiteNavbarData.WebsiteNavbarActionWidget.extend({
         return new Promise(function (resolve) {
             var dialog = new EditMenuDialog(self, {}, rootID);
             dialog.on('save', self, function () {
+                window.document.body.classList.add('o_wait_reload');
                 // Before reloading the page after menu modification, does the
                 // given action to do.
                 if (beforeReloadCallback) {
@@ -945,13 +1050,7 @@ var PageManagement = Widget.extend({
     },
     _onClonePageButtonClick: function (ev) {
         var pageId = $(ev.currentTarget).data('id');
-        this._rpc({
-            model: 'website.page',
-            method: 'clone_page',
-            args: [pageId],
-        }).then(function (path) {
-            window.location.href = path;
-        });
+        _clonePage.call(this, pageId);
     },
     _onDeletePageButtonClick: function (ev) {
         var pageId = $(ev.currentTarget).data('id');
@@ -969,46 +1068,67 @@ var PageManagement = Widget.extend({
  *                  It will affect redirect after page deletion: reload or '/'
  */
 // TODO: This function should be integrated in a widget in the future
-function _deletePage(pageId, fromPageManagement) {
-    var self = this;
-    new Promise(function (resolve, reject) {
-        // Search the page dependencies
-        self._getPageDependencies(pageId)
-        .then(function (dependencies) {
-            // Inform the user about those dependencies and ask him confirmation
-            return new Promise(function (confirmResolve, confirmReject) {
-                Dialog.safeConfirm(self, "", {
-                    title: _t("Delete Page"),
-                    $content: $(qweb.render('website.delete_page', {dependencies: dependencies})),
-                    confirm_callback: confirmResolve,
-                    cancel_callback: resolve,
-                });
-            });
-        }).then(function () {
-            // Delete the page if the user confirmed
-            return self._rpc({
-                model: 'website.page',
-                method: 'unlink',
-                args: [pageId],
-            });
-        }).then(function () {
+async function _deletePage(pageId, fromPageManagement) {
+    const dependencies = await this._getPageDependencies(pageId);
+    for (const locs of Object.values(dependencies)) {
+        for (const loc of locs) {
+            loc.text = Markup(loc.text);
+        }
+    }
+    Dialog.safeConfirm(this, "", {
+        title: _t("Delete Page"),
+        $content: $(qweb.render('website.delete_page', {dependencies: dependencies})),
+        async confirm_callback() {
+            await this._rpc({model: 'website.page', method: 'unlink', args: [pageId]});
             if (fromPageManagement) {
                 window.location.reload(true);
             } else {
                 window.location.href = '/';
             }
-        }, reject);
+        }
+    });
+}
+/**
+ * Duplicate the page after showing the wizard to enter new page name.
+ *
+ * @private
+ * @param {integer} pageId - The ID of the page to be duplicate
+ *
+ */
+function _clonePage(pageId) {
+    var self = this;
+    new Promise(function (resolve, reject) {
+        Dialog.confirm(this, undefined, {
+            title: _t("Duplicate Page"),
+            $content: $(qweb.render('website.duplicate_page_action_dialog')),
+            confirm_callback: function () {
+                var new_page_name =  this.$('#page_name').val();
+                return self._rpc({
+                    model: 'website.page',
+                    method: 'clone_page',
+                    args: [pageId, new_page_name],
+                }).then(function (path) {
+                    window.location.href = path;
+                }).guardedCatch(reject);
+            },
+            cancel_callback: reject,
+        }).on('closed', null, reject);
     });
 }
 
-websiteNavbarData.websiteNavbarRegistry.add(ContentMenu, '#content-menu');
-websiteRootData.websiteRootRegistry.add(PageManagement, '#list_website_pages');
+registry.category("website_navbar_widgets").add("ContentMenu", {
+    Widget: ContentMenu,
+    selector: '#content-menu',
+});
+registry.category("public_root_widgets").add("PageManagement", {
+    Widget: PageManagement,
+    selector: '#list_website_pages',
+});
 
-return {
+export default {
     PagePropertiesDialog: PagePropertiesDialog,
     ContentMenu: ContentMenu,
     EditMenuDialog: EditMenuDialog,
     MenuEntryDialog: MenuEntryDialog,
     SelectEditMenuDialog: SelectEditMenuDialog,
 };
-});
