@@ -1,13 +1,10 @@
-odoo.define("website.tour.carousel_content_removal", function (require) {
-"use strict";
+/** @odoo-module */
 
-var tour = require("web_tour.tour");
-var base = require("web_editor.base");
+import tour from 'web_tour.tour';
 
 tour.register("carousel_content_removal", {
     test: true,
     url: "/",
-    wait_for: base.ready(),
 }, [{
     trigger: "a[data-action=edit]",
     content: "Click the Edit button.",
@@ -21,7 +18,7 @@ tour.register("carousel_content_removal", {
     trigger: ".carousel .carousel-item.active .carousel-content",
     content: "Select the active carousel item.",
 }, {
-    trigger: ".oe_snippet_remove:last",
+    trigger: ".oe_overlay.oe_active .oe_snippet_remove",
     content: "Remove the active carousel item.",
 },
 {
@@ -29,5 +26,3 @@ tour.register("carousel_content_removal", {
     content: "Check for a carousel slide with an empty container tag",
     run: function () {},
 }]);
-
-});
