@@ -1,9 +1,15 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 from . import test_attachment
+from . import test_auth_signup_uninvited
+from . import test_automatic_editor
 from . import test_base_url
+from . import test_configurator
+from . import test_controllers
 from . import test_converter
 from . import test_crawl
+from . import test_disable_unused_snippets_assets
+from . import test_fuzzy
 from . import test_get_current_website
 from . import test_lang_url
 from . import test_menu
@@ -11,8 +17,12 @@ from . import test_page
 from . import test_performance
 from . import test_qweb
 from . import test_res_users
+from . import test_snippets
+from . import test_theme
 from . import test_ui
 from . import test_views
+from . import test_views_inherit_module_update
 from . import test_website_favicon
+from . import test_website_form_editor
 from . import test_website_reset_password
 from . import test_website_visitor
