@@ -1,5 +1,7 @@
 # coding: utf-8
+from odoo.addons.website.tools import MockRequest
 from odoo.tests import common, HttpCase, tagged
+from odoo.tools import mute_logger
 
 
 @tagged('-at_install', 'post_install')
@@ -60,7 +62,7 @@ class TestPage(common.TransactionCase):
         total_pages = Page.search_count([])
         total_menus = Menu.search_count([])
         # Copying a specific page should create a new page with an unique URL (suffixed by -X)
-        Page.clone_page(self.page_specific.id, True)
+        Page.clone_page(self.page_specific.id, clone_menu=True)
         cloned_page = Page.search([('url', '=', '/page_specific-1')])
         cloned_menu = Menu.search([('url', '=', '/page_specific-1')])
         self.assertEqual(len(cloned_page), 1, "A page with an URL /page_specific-1 should've been created")
@@ -69,19 +71,27 @@ class TestPage(common.TransactionCase):
         self.assertEqual(len(cloned_menu), 1, "A specific page (with a menu) being cloned should have it's menu also cloned")
         self.assertEqual(cloned_menu.page_id, cloned_page, "The new cloned menu and the new cloned page should be linked (m2o)")
         self.assertEqual(Menu.search_count([]), total_menus + 1, "Should have cloned the page menu")
+        Page.clone_page(self.page_specific.id, page_name="about-us", clone_menu=True)
+        cloned_page_about_us = Page.search([('url', '=', '/about-us')])
+        cloned_menu_about_us = Menu.search([('url', '=', '/about-us')])
+        self.assertEqual(len(cloned_page_about_us), 1, "A page with an URL /about-us should've been created")
+        self.assertEqual(len(cloned_menu_about_us), 1, "A specific page (with a menu) being cloned should have it's menu also cloned")
+        self.assertEqual(cloned_menu_about_us.page_id, cloned_page_about_us, "The new cloned menu and the new cloned page should be linked (m2o)")
+        # It should also copy its menu with new url/name/page_id (if the page has a menu)
+        self.assertEqual(Menu.search_count([]), total_menus + 2, "Should have cloned the page menu")
 
         total_pages = Page.search_count([])
         total_menus = Menu.search_count([])
 
         # Copying a generic page should create a specific page with same URL
-        Page.clone_page(self.page_1.id, True)
+        Page.clone_page(self.page_1.id, clone_menu=True)
         cloned_generic_page = Page.search([('url', '=', '/page_1'), ('id', '!=', self.page_1.id), ('website_id', '!=', False)])
         self.assertEqual(len(cloned_generic_page), 1, "A generic page being cloned should create a specific one for the current website")
         self.assertEqual(cloned_generic_page.url, self.page_1.url, "The URL of the cloned specific page should be the same as the generic page it has been cloned from")
         self.assertEqual(Page.search_count([]), total_pages + 1, "Should have cloned the generic page as a specific page for this website")
         self.assertEqual(Menu.search_count([]), total_menus, "It should not create a new menu as the generic page's menu belong to another website")
         # Except if the URL already exists for this website (its the case now that we already cloned it once)
-        Page.clone_page(self.page_1.id, True)
+        Page.clone_page(self.page_1.id, clone_menu=True)
         cloned_generic_page_2 = Page.search([('url', '=', '/page_1-1'), ('id', '!=', self.page_1.id)])
         self.assertEqual(len(cloned_generic_page_2), 1, "A generic page being cloned should create a specific page with a new URL if there is already a specific page with that URL")
 
@@ -167,6 +177,11 @@ class TestPage(common.TransactionCase):
         View = self.env['ir.ui.view']
         Website = self.env['website']
 
+        website2 = self.env['website'].create({
+            'name': 'My Second Website',
+            'domain': '',
+        })
+
         # currently the view unlink of website.page can't handle views with inherited views
         self.extension_view.unlink()
 
@@ -183,12 +198,13 @@ class TestPage(common.TransactionCase):
         self.assertTrue(website_id not in pages.mapped('website_id').ids, "The website from which we deleted the generic page should not have a specific one.")
         self.assertTrue(website_id not in View.search([('name', 'in', ('Base', 'Extension'))]).mapped('website_id').ids, "Same for views")
 
+@tagged('-at_install', 'post_install')
 class WithContext(HttpCase):
     def setUp(self):
         super().setUp()
         Page = self.env['website.page']
         View = self.env['ir.ui.view']
-        base_view = View.create({
+        self.base_view = View.create({
             'name': 'Base',
             'type': 'qweb',
             'arch': '''<t name="Homepage" t-name="website.base_view">
@@ -199,7 +215,7 @@ class WithContext(HttpCase):
             'key': 'test.base_view',
         })
         self.page = Page.create({
-            'view_id': base_view.id,
+            'view_id': self.base_view.id,
             'url': '/page_1',
             'is_published': True,
         })
@@ -208,6 +224,7 @@ class WithContext(HttpCase):
         specific_page = self.page.copy({'website_id': self.env['website'].get_current_website().id})
         specific_page.write({'is_published': False, 'arch': self.page.arch.replace('I am a generic page', 'I am a specific page')})
 
+        self.authenticate(None, None)
         r = self.url_open(specific_page.url)
         self.assertEqual(r.status_code, 404, "Restricted users should see a 404 and not the generic one as we unpublished the specific one")
 
@@ -225,13 +242,24 @@ class WithContext(HttpCase):
             dbname, admin_uid, 'admin',
             'website', 'search_pages', [website.id], 'info'
         )
-        self.assertEqual(robot, [{'loc': '/website/info'}])
+        self.assertIn({'loc': '/website/info'}, robot)
 
         pages = self.xmlrpc_object.execute(
             dbname, admin_uid, 'admin',
             'website', 'search_pages', [website.id], 'page'
         )
-        self.assertEqual(
-            [{'loc': p['loc']} for p in pages],
-            [{'loc': '/page_1'}]
+        self.assertIn(
+            '/page_1',
+            [p['loc'] for p in pages],
         )
+
+    @mute_logger('odoo.addons.http_routing.models.ir_http')
+    def test_03_error_page_debug(self):
+        with MockRequest(self.env, website=self.env['website'].browse(1)):
+            self.base_view.arch = self.base_view.arch.replace('I am a generic page', '<t t-esc="15/0"/>')
+            r = self.url_open(self.page.url)
+            self.assertEqual(r.status_code, 500, "15/0 raise a 500 error page")
+            self.assertNotIn('ZeroDivisionError: division by zero', r.text, "Error should not be shown when not in debug.")
+            r = self.url_open(self.page.url + '?debug=1')
+            self.assertEqual(r.status_code, 500, "15/0 raise a 500 error page (2)")
+            self.assertIn('ZeroDivisionError: division by zero', r.text, "Error should be shown in debug.")
