@@ -1,14 +1,12 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch
-from odoo.exceptions import AccessError
+from unittest.mock import patch
 
+from odoo.exceptions import AccessError
 from odoo.tests.common import TransactionCase
-from odoo.addons.crm.tests.common import TestCrmCases
+from odoo.addons.crm.tests.common import TestCrmCommon
+from odoo.addons.mail.tests.common import mail_new_test_user
 
 
 class TestPartnerAssign(TransactionCase):
@@ -63,7 +61,8 @@ class TestPartnerAssign(TransactionCase):
 
         # In order to test find nearest Partner functionality and assign to opportunity,
         # I Set Geo Lattitude and Longitude according to partner address.
-        partner_be.geo_localize()
+        # YTI Note: We should probably mock the call
+        partner_be.with_context(force_geo_localize=True).geo_localize()
 
         # I check Geo Latitude and Longitude of partner after set
         self.assertTrue(50 < partner_be.partner_latitude < 51, "Latitude is wrong: 50 < %s < 51" % partner_be.partner_latitude)
@@ -76,6 +75,7 @@ class TestPartnerAssign(TransactionCase):
         self.assertEqual(lead.partner_assigned_id, partner_uk, "Opportuniy is not assigned nearest partner")
         self.assertTrue(50 < lead.partner_latitude < 55, "Latitude is wrong: 50 < %s < 55" % lead.partner_latitude)
         self.assertTrue(-4 < lead.partner_longitude < -1, "Longitude is wrong: -4 < %s < -1" % lead.partner_longitude)
+        self.assertTrue(lead.date_partner_assign, "Partner Assignment Date should be set")
 
         # I forward this opportunity to its nearest partner.
         context = dict(self.env.context, default_model='crm.lead', default_res_id=lead.id, active_ids=lead.ids)
@@ -86,7 +86,7 @@ class TestPartnerAssign(TransactionCase):
             pass
 
 
-class TestPartnerLeadPortal(TestCrmCases):
+class TestPartnerLeadPortal(TestCrmCommon):
 
     def setUp(self):
         super(TestPartnerLeadPortal, self).setUp()
@@ -97,48 +97,37 @@ class TestPartnerLeadPortal(TestCrmCases):
             'sequence': 3,
         })
         # Integrating user/partner, having a salesman
-        self.portal_user = self.env['res.users'].with_context({'no_reset_password': True, 'mail_notrack': True}).create({
-            'name': 'Super Customer Odoo Intregrating Partner',
-            'email': 'super.partner@ododo.com',
-            'login': 'superpartner',
-            'groups_id': [(6, 0, [self.env.ref('base.group_portal').id])],
-            'user_id': self.crm_salesman.id,
-            'grade_id': self.grade.id,
-        })
-        self.portal_partner = self.portal_user.partner_id
+        self.user_portal = mail_new_test_user(
+            self.env, login='user_portal',
+            name='Patrick Portal', email='portal@test.example.com',
+            company_id=self.env.ref("base.main_company").id,
+            grade_id=self.grade.id,
+            user_id=self.user_sales_manager.id,
+            notification_type='inbox',
+            groups='base.group_portal',
+        )
+
         # New lead, assigned to the new portal
-        self.lead = self.env['crm.lead'].with_context(mail_notrack=True).create({
+        self.lead_portal = self.env['crm.lead'].with_context(mail_notrack=True).create({
             'type': "lead",
             'name': "Test lead new",
             'user_id': False,
             'team_id': False,
             'description': "This is the description of the test new lead.",
-            'partner_assigned_id': self.portal_partner.id
-        })
-        # Sales Team of crm_salesman
-        self.team = self.env['crm.team'].with_context(mail_notrack=True).create({
-            'name': 'Test Team FOR THE WIN',
-            'use_leads': True,
-            'member_ids': [(6, 0, [self.crm_salesman.id])],
+            'partner_assigned_id': self.user_portal.partner_id.id
         })
 
     def test_partner_lead_accept(self):
         """ Test an integrating partner accepting the lead """
-        team_before = self.lead.team_id
-        user_before = self.lead.user_id
-
-        self.lead.with_user(self.portal_user).partner_interested(comment="Oh yeah, I take that lead !")
-
-        self.assertEqual(self.lead.type, 'opportunity', 'Bad Type: accepted lead by portal user should become an opportunity.')
-        self.assertEqual(self.lead.team_id, team_before, 'Accepting lead does not change the sales team.')
-        self.assertEqual(self.lead.user_id, user_before, 'Accepting lead does not change the salesman.')
+        self.lead_portal.with_user(self.user_portal).partner_interested(comment="Oh yeah, I take that lead !")
+        self.assertEqual(self.lead_portal.type, 'opportunity')
 
     def test_partner_lead_decline(self):
         """ Test an integrating partner decline the lead """
-        self.lead.with_user(self.portal_user).partner_desinterested(comment="No thanks, I have enough leads !", contacted=True, spam=False)
+        self.lead_portal.with_user(self.user_portal).partner_desinterested(comment="No thanks, I have enough leads !", contacted=True, spam=False)
 
-        self.assertFalse(self.lead.partner_assigned_id.id, 'The partner_assigned_id of the declined lead should be False.')
-        self.assertTrue(self.portal_user.partner_id in self.lead.sudo().partner_declined_ids, 'Partner who has declined the lead should be in the declined_partner_ids.')
+        self.assertFalse(self.lead_portal.partner_assigned_id.id, 'The partner_assigned_id of the declined lead should be False.')
+        self.assertTrue(self.user_portal.partner_id in self.lead_portal.sudo().partner_declined_ids, 'Partner who has declined the lead should be in the declined_partner_ids.')
 
     def test_lead_access_right(self):
         """ Test another portal user can not write on every leads """
@@ -151,22 +140,22 @@ class TestPartnerLeadPortal(TestCrmCases):
         })
         # try to accept a lead that is not mine
         with self.assertRaises(AccessError):
-            self.lead.with_user(poor_portal_user).partner_interested(comment="Oh yeah, I take that lead !")
+            self.lead_portal.with_user(poor_portal_user).partner_interested(comment="Oh yeah, I take that lead !")
 
     def test_lead_creation(self):
         """ Test the opportinuty creation from portal """
-        data = self.env['crm.lead'].with_user(self.portal_user).create_opp_portal({
+        data = self.env['crm.lead'].with_user(self.user_portal).create_opp_portal({
             'title': "L'ours bleu",
             'description': 'A good joke',
             'contact_name': 'Renaud Rutten',
         })
         opportunity = self.env['crm.lead'].browse(data['id'])
-        salesmanteam = self.env['crm.team']._get_default_team_id(user_id=self.portal_user.user_id.id)
+        salesmanteam = self.env['crm.team']._get_default_team_id(user_id=self.user_portal.user_id.id)
 
         self.assertEqual(opportunity.team_id, salesmanteam, 'The created opportunity should have the same team as the salesman default team of the opportunity creator.')
-        self.assertEqual(opportunity.partner_assigned_id, self.portal_partner, 'Assigned Partner of created opportunity is the (portal) creator.')
+        self.assertEqual(opportunity.partner_assigned_id, self.user_portal.partner_id, 'Assigned Partner of created opportunity is the (portal) creator.')
 
     def test_portal_mixin_url(self):
-        record_action = self.lead.get_access_action(self.portal_user.id)
-        self.assertEqual(record_action['url'], '/my/opportunity/%s' % self.lead.id)
+        record_action = self.lead_portal.get_access_action(self.user_portal.id)
+        self.assertEqual(record_action['url'], '/my/opportunity/%s' % self.lead_portal.id)
         self.assertEqual(record_action['type'], 'ir.actions.act_url')
