@@ -18,13 +18,11 @@ Publish your customers as business references on your website to attract new pot
         'data/res_partner_demo.xml',
     ],
     'data': [
-        'data/website_customer_data.xml',
         'views/website_customer_templates.xml',
         'views/res_partner_views.xml',
         'security/ir.model.access.csv',
         'security/ir_rule.xml',
     ],
-    'qweb': [],
     'installable': True,
     'license': 'LGPL-3',
 }
