@@ -2,6 +2,7 @@
 
 from odoo import api, models, _
 from odoo.exceptions import UserError
+from odoo.osv import expression
 
 
 class SaleOrder(models.Model):
@@ -17,12 +18,12 @@ class SaleOrder(models.Model):
             domain.append(('event_ticket_id', '=', self.env.context.get("event_ticket_id")))
         return self.env['sale.order.line'].sudo().search(domain)
 
-    def _website_product_id_change(self, order_id, product_id, qty=0):
+    def _website_product_id_change(self, order_id, product_id, qty=0, **kwargs):
         order = self.env['sale.order'].sudo().browse(order_id)
         if self._context.get('pricelist') != order.pricelist_id.id:
             self = self.with_context(pricelist=order.pricelist_id.id)
 
-        values = super(SaleOrder, self)._website_product_id_change(order_id, product_id, qty=qty)
+        values = super(SaleOrder, self)._website_product_id_change(order_id, product_id, qty=qty, **kwargs)
         event_ticket_id = None
         if self.env.context.get("event_ticket_id"):
             event_ticket_id = self.env.context.get("event_ticket_id")
@@ -43,7 +44,7 @@ class SaleOrder(models.Model):
                 values['price_unit'] = ticket.price
             else:
                 values['price_unit'] = ticket.price_reduce
-            values['name'] = ticket.get_ticket_multiline_description_sale()
+            values['name'] = ticket._get_ticket_multiline_description()
 
         # avoid writing related values that end up locking the product record
         values.pop('event_ok', None)
@@ -71,20 +72,22 @@ class SaleOrder(models.Model):
             if ticket.id:
                 self = self.with_context(event_ticket_id=ticket.id, fixed_price=1)
         else:
-            line = None
-            ticket = self.env['event.event.ticket'].search([('product_id', '=', product_id)], limit=1)
+            ticket_domain = [('product_id', '=', product_id)]
+            if self.env.context.get("event_ticket_id"):
+                ticket_domain = expression.AND([ticket_domain, [('id', '=', self.env.context['event_ticket_id'])]])
+            ticket = self.env['event.event.ticket'].search(ticket_domain, limit=1)
             old_qty = 0
         new_qty = set_qty if set_qty else (add_qty or 0 + old_qty)
 
         # case: buying tickets for a sold out ticket
         values = {}
-        if ticket and ticket.seats_availability == 'limited' and ticket.seats_available <= 0:
+        if ticket and ticket.seats_limited and ticket.seats_available <= 0:
             values['warning'] = _('Sorry, The %(ticket)s tickets for the %(event)s event are sold out.') % {
                 'ticket': ticket.name,
                 'event': ticket.event_id.name}
             new_qty, set_qty, add_qty = 0, 0, -old_qty
         # case: buying tickets, too much attendees
-        elif ticket and ticket.seats_availability == 'limited' and new_qty > ticket.seats_available:
+        elif ticket and ticket.seats_limited and new_qty > ticket.seats_available:
             values['warning'] = _('Sorry, only %(remaining_seats)d seats are still available for the %(ticket)s ticket for the %(event)s event.') % {
                 'remaining_seats': ticket.seats_available,
                 'ticket': ticket.name,
@@ -99,13 +102,11 @@ class SaleOrder(models.Model):
                 ('sale_order_id', 'in', self.ids),  # To avoid break on multi record set
                 ('event_ticket_id', '=', ticket.id),
             ], offset=new_qty, limit=(old_qty - new_qty), order='create_date asc')
-            attendees.button_reg_cancel()
+            attendees.action_cancel()
         # adding attendees
         elif ticket and new_qty > old_qty:
-            line = OrderLine.browse(values['line_id'])
-            line._update_registrations(confirm=False, cancel_to_draft=True, registration_data=kwargs.get('registration_data', []))
-            # add in return values the registrations, to display them on website (or not)
-            values['attendee_ids'] = self.env['event.registration'].search([('sale_order_line_id', '=', line.id), ('state', '!=', 'cancel')]).ids
+            # do not do anything, attendees will be created at SO confirmation if not given previously
+            pass
         return values
 
 
