@@ -1,8 +1,8 @@
 odoo.define('website_forum.website_forum', function (require) {
 'use strict';
 
+const dom = require('web.dom');
 var core = require('web.core');
-var weDefaultOptions = require('web_editor.wysiwyg.default_options');
 var wysiwygLoader = require('web_editor.loader');
 var publicWidget = require('web.public.widget');
 var session = require('web.session');
@@ -12,7 +12,14 @@ var _t = core._t;
 
 publicWidget.registry.websiteForum = publicWidget.Widget.extend({
     selector: '.website_forum',
-    xmlDependencies: ['/website_forum/static/src/xml/website_forum_share_templates.xml'],
+    xmlDependencies: [
+        // TODO Move the toolbar template out of the website_forum_templates
+        // file in master, as explained by the comment in the creation dialog
+        // widget.
+        '/web_editor/static/src/xml/editor.xml',
+        '/website_forum/static/src/xml/website_forum_templates.xml',
+        '/website_forum/static/src/xml/website_forum_share_templates.xml',
+    ],
     events: {
         'click .karma_required': '_onKarmaRequiredClick',
         'mouseenter .o_js_forum_tag_follow': '_onTagFollowBoxMouseEnter',
@@ -27,6 +34,7 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
         'click .o_wforum_favourite_toggle': '_onFavoriteQuestionClick',
         'click .comment_delete': '_onDeleteCommentClick',
         'click .js_close_intro': '_onCloseIntroClick',
+        'submit .js_wforum_submit_form:has(:not(.karma_required).o_wforum_submit_post)': '_onSubmitForm',
     },
 
     /**
@@ -117,32 +125,16 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
             var editorKarma = $textarea.data('karma') || 0; // default value for backward compatibility
             var $form = $textarea.closest('form');
             var hasFullEdit = parseInt($("#karma").val()) >= editorKarma;
-            // Warning: Do not activate any option that adds inline style.
-            // Because the style is deleted after save.
-            var toolbar = [
-                ['style', ['style']],
-                ['font', ['bold', 'italic', 'underline', 'clear']],
-                ['para', ['ul', 'ol', 'paragraph']],
-                ['table', ['table']],
-            ];
-            if (hasFullEdit) {
-                toolbar.push(['insert', ['link', 'picture']]);
-            }
-            toolbar.push(['history', ['undo', 'redo']]);
 
             var options = {
-                height: 200,
-                minHeight: 80,
-                toolbar: toolbar,
-                styleWithSpan: false,
-                styleTags: _.without(weDefaultOptions.styleTags, 'h1', 'h2', 'h3'),
+                toolbarTemplate: 'website_forum.web_editor_toolbar',
                 recordInfo: {
                     context: self._getContext(),
                     res_model: 'forum.post',
                     res_id: +window.location.pathname.split('-').pop(),
                 },
-                disableFullMediaDialog: true,
-                disableResizeImage: true,
+                resizable: true,
+                userGeneratedContent: true,
             };
             if (!hasFullEdit) {
                 options.plugins = {
@@ -150,16 +142,12 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
                     MediaPlugin: false,
                 };
             }
-            wysiwygLoader.load(self, $textarea[0], options).then(wysiwyg => {
+            wysiwygLoader.loadFromTextarea(self, $textarea[0], options).then(wysiwyg => {
+                if (!hasFullEdit) {
+                    wysiwyg.toolbar.$el.find('#link, #media').remove();
+                }
                 // float-left class messes up the post layout OPW 769721
                 $form.find('.note-editable').find('img.float-left').removeClass('float-left');
-                // o_we_selected_image has not always been removed when
-                // saving a post so we need the line below to remove it if it is present.
-                $form.find('.note-editable').find('img.o_we_selected_image').removeClass('o_we_selected_image');
-                $form.on('click', 'button, .a-submit', () => {
-                    $form.find('.note-editable').find('img.o_we_selected_image').removeClass('o_we_selected_image');
-                    wysiwyg.save();
-                });
             });
         });
 
@@ -169,6 +157,14 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
                 offset: 10,
                 animation: false,
                 html: true,
+            }).popover('hide').data('bs.popover').tip.classList.add('o_wforum_bio_popover_container');
+        });
+
+        this.$('#post_reply').on('shown.bs.collapse', function (e) {
+            const replyEl = document.querySelector('#post_reply');
+            const scrollingElement = dom.closestScrollable(replyEl.parentNode);
+            dom.scrollTo(replyEl, {
+                forcedOffset: $(scrollingElement).innerHeight() - $(replyEl).innerHeight(),
             });
         });
 
@@ -179,33 +175,93 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
     // Handlers
     //--------------------------------------------------------------------------
 
+    /**
+     *
+     * @override
+     * @param {Event} ev
+     */
+    _onSubmitForm: function (ev) {
+        let validForm = true;
+
+        let $form = $(ev.currentTarget);
+        let $title = $form.find('input[name=post_name]');
+        let $textarea = $form.find('textarea[name=content]');
+        // It's not really in the textarea that the user write at first
+        let textareaContent = $form.find('.o_wysiwyg_textarea_wrapper').text().trim();
+
+        if ($title.length && $title[0].required) {
+            if ($title.val()) {
+                $title.removeClass('is-invalid');
+            } else {
+                $title.addClass('is-invalid');
+                validForm = false;
+            }
+        }
+
+        // Because the textarea is hidden, we add the red or green border to its container
+        if ($textarea[0] && $textarea[0].required) {
+            let $textareaContainer = $form.find('.o_wysiwyg_textarea_wrapper');
+            if (!textareaContent.length) {
+                $textareaContainer.addClass('border border-danger rounded-top');
+                validForm = false;
+            } else {
+                $textareaContainer.removeClass('border border-danger rounded-top');
+            }
+        }
+
+        if (validForm) {
+            // Stores social share data to display modal on next page.
+            if ($form.has('.oe_social_share_call').length) {
+                sessionStorage.setItem('social_share', JSON.stringify({
+                    targetType: $(ev.currentTarget).find('.o_wforum_submit_post').data('social-target-type'),
+                }));
+            }
+        } else {
+            ev.preventDefault();
+            setTimeout(function() {
+                var $buttons = $(ev.currentTarget).find('button[type="submit"], a.a-submit');
+                _.each($buttons, function (btn) {
+                    let $btn = $(btn);
+                    $btn.find('i').remove();
+                    $btn.prop('disabled', false);
+                });
+            }, 0);
+        }
+    },
     /**
      * @private
      * @param {Event} ev
      */
     _onKarmaRequiredClick: function (ev) {
-        var $karma = $(ev.currentTarget);
-        var karma = $karma.data('karma');
-        var forum_id = $('#wrapwrap').data('forum_id');
+        const karma = parseInt(ev.currentTarget.dataset.karma);
         if (!karma) {
             return;
         }
+
         ev.preventDefault();
-        var msg = karma + ' ' + _t("karma is required to perform this action. ");
-        var title = _t("Karma Error");
-        if (forum_id) {
-            msg += '<a class="alert-link" href="/forum/' + forum_id + '/faq">' + _t("Read the guidelines to know how to gain karma.") + '</a>';
-        }
-        if (session.is_website_user) {
-            msg = _t("Sorry you must be logged in to perform this action");
-            title = _t("Access Denied");
-        }
-        this.call('crash_manager', 'show_warning', {
-            message: msg,
-            title: title,
-        }, {
+        const forumID = parseInt(document.getElementById('wrapwrap').dataset.forum_id);
+        const notifOptions = {
+            type: "warning",
             sticky: false,
-        });
+        };
+        if (session.is_website_user) {
+            notifOptions.title = _t("Access Denied");
+            notifOptions.message = _t("Sorry you must be logged in to perform this action");
+        } else {
+            notifOptions.title = _t("Karma Error");
+            // FIXME this translation is bad, the number should be part of the
+            // translation, to fix in the appropriate version
+            notifOptions.message = `${karma} ${_t("karma is required to perform this action. ")}`;
+            if (forumID) {
+                notifOptions.messageIsHtml = true;
+                const linkLabel = _.escape(_t("Read the guidelines to know how to gain karma."));
+                notifOptions.message = `
+                    ${_.escape(notifOptions.message)}<br/>
+                    <a class="alert-link" href="/forum/${forumID}/faq">${linkLabel}</a>
+                `;
+            }
+        }
+        this.displayNotification(notifOptions);
     },
     /**
      * @private
@@ -262,11 +318,11 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
                 } else if (data.error === 'post_non_flaggable') {
                     message = _t("This post can not be flagged");
                 }
-                self.call('crash_manager', 'show_warning', {
+                self.displayNotification({
                     message: message,
                     title: _t("Access Denied"),
-                }, {
                     sticky: false,
+                    type: "warning",
                 });
             } else if (data.success) {
                 var elem = $link;
@@ -302,11 +358,11 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
                 } else if (data.error === 'anonymous_user') {
                     message = _t('Sorry you must be logged to vote');
                 }
-                self.call('crash_manager', 'show_warning', {
+                self.displayNotification({
                     message: message,
                     title: _t("Access Denied"),
-                }, {
                     sticky: false,
+                    type: "warning",
                 });
             } else {
                 var $container = $btn.closest('.vote');
@@ -325,10 +381,20 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
                 if (userVote === 1) {
                     $voteUp.addClass('text-success');
                     $voteCount.addClass('text-success');
+                    $voteDown.removeClass('karma_required');
                 }
                 if (userVote === -1) {
                     $voteDown.addClass('text-danger');
                     $voteCount.addClass('text-danger');
+                    $voteUp.removeClass('karma_required');
+                }
+                if (userVote === 0) {
+                    if (!$voteDown.data('can-downvote')) {
+                        $voteDown.addClass('karma_required');
+                    }
+                    if (!$voteUp.data('can-upvote')) {
+                        $voteUp.addClass('karma_required');
+                    }
                 }
                 $voteCount.html(data['vote_count']).addClass('o_forum_vote_animate');
             }
@@ -371,11 +437,11 @@ publicWidget.registry.websiteForum = publicWidget.Widget.extend({
                 if (data.error === 'anonymous_user') {
                     var message = _t("Sorry, anonymous users cannot choose correct answer.");
                 }
-                this.call('crash_manager', 'show_warning', {
+                this.displayNotification({
                     message: message,
                     title: _t("Access Denied"),
-                }, {
                     sticky: false,
+                    type: "warning",
                 });
             } else {
                 _.each(this.$('.forum_answer'), answer => {
