@@ -4,11 +4,11 @@
 {
     'name': 'Online Jobs',
     'category': 'Website/Website',
-    'sequence': 142,
+    'sequence': 310,
     'version': '1.0',
     'summary': 'Manage your online hiring process',
     'description': "This module allows to publish your available job positions on your website and keep track of application submissions easily. It comes as an add-on of *Recruitment* app.",
-    'depends': ['website_partner', 'hr_recruitment', 'website_mail', 'website_form'],
+    'depends': ['hr_recruitment', 'website_mail'],
     'data': [
         'security/ir.model.access.csv',
         'security/website_hr_recruitment_security.xml',
@@ -22,5 +22,17 @@
     ],
     'installable': True,
     'application': True,
+    'auto_install': ['hr_recruitment', 'website_mail'],
+    'assets': {
+        'web.assets_frontend': [
+            'website_hr_recruitment/static/src/scss/**/*',
+        ],
+        'website.assets_editor': [
+            'website_hr_recruitment/static/src/js/**/*',
+        ],
+        'web.assets_tests': [
+            'website_hr_recruitment/static/tests/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
