@@ -30,7 +30,7 @@ class WebsiteHrRecruitment(http.Controller):
 
         # List jobs available to current UID
         domain = request.website.website_domain()
-        job_ids = Jobs.search(domain, order="is_published desc, no_of_recruitment desc").ids
+        job_ids = Jobs.search(domain, order="is_published desc, sequence, no_of_recruitment desc").ids
         # Browse jobs as superuser, because address is restricted
         jobs = Jobs.sudo().browse(job_ids)
 
@@ -80,21 +80,15 @@ class WebsiteHrRecruitment(http.Controller):
         })
         return request.redirect("/jobs/detail/%s?enable_editor=1" % slug(job))
 
-    @http.route('''/jobs/detail/<model("hr.job", "[('website_id', 'in', (False, current_website_id))]"):job>''', type='http', auth="public", website=True)
+    @http.route('''/jobs/detail/<model("hr.job"):job>''', type='http', auth="public", website=True, sitemap=True)
     def jobs_detail(self, job, **kwargs):
-        if not job.can_access_from_current_website():
-            raise NotFound()
-
         return request.render("website_hr_recruitment.detail", {
             'job': job,
             'main_object': job,
         })
 
-    @http.route('''/jobs/apply/<model("hr.job", "[('website_id', 'in', (False, current_website_id))]"):job>''', type='http', auth="public", website=True)
+    @http.route('''/jobs/apply/<model("hr.job"):job>''', type='http', auth="public", website=True, sitemap=True)
     def jobs_apply(self, job, **kwargs):
-        if not job.can_access_from_current_website():
-            raise NotFound()
-
         error = {}
         default = {}
         if 'website_hr_recruitment_error' in request.session:
