@@ -1,15 +1,24 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-from odoo import api, SUPERUSER_ID
+from odoo import api, SUPERUSER_ID, _
 from . import controllers
 from . import models
 from . import wizard
 from . import report
 
+def _post_init_hook(cr, registry):
+    env = api.Environment(cr, SUPERUSER_ID, {})
+    terms_conditions = env['ir.config_parameter'].get_param('account.use_invoice_terms')
+    if not terms_conditions:
+        env['ir.config_parameter'].set_param('account.use_invoice_terms', True)
+    companies = env['res.company'].search([])
+    for company in companies:
+        company.terms_type = 'html'
+
 
 def uninstall_hook(cr, registry):
     ''' Need to reenable the `product` pricelist multi-company rule that were
-        disabled to be 'overriden' for multi-website purpose
+        disabled to be 'overridden' for multi-website purpose
     '''
     env = api.Environment(cr, SUPERUSER_ID, {})
     pl_rule = env.ref('product.product_pricelist_comp_rule', raise_if_not_found=False)
