@@ -147,15 +147,15 @@ odoo.define('website_sale.website_sale', function (require) {
 
 var core = require('web.core');
 var config = require('web.config');
-var concurrency = require('web.concurrency');
 var publicWidget = require('web.public.widget');
-var VariantMixin = require('sale.VariantMixin');
+var VariantMixin = require('website_sale.VariantMixin');
 var wSaleUtils = require('website_sale.utils');
+const cartHandlerMixin = wSaleUtils.cartHandlerMixin;
 require("web.zoomodoo");
+const {extraMenuUpdateCallbacks} = require('website.content.menu');
+const dom = require('web.dom');
 
-var qweb = core.qweb;
-
-publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
+publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, cartHandlerMixin, {
     selector: '.oe_website_sale',
     events: _.extend({}, VariantMixin.events || {}, {
         'change form .js_product:first input[name="add_qty"]': '_onChangeAddQuantity',
@@ -173,10 +173,11 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
         'change select[name="country_id"]': '_onChangeCountry',
         'change #shipping_use_same': '_onChangeShippingUseSame',
         'click .toggle_summary': '_onToggleSummary',
-        'click #add_to_cart, #buy_now, #products_grid .o_wsale_product_btn .a-submit': 'async _onClickAdd',
+        'click #add_to_cart, .o_we_buy_now, #products_grid .o_wsale_product_btn .a-submit': 'async _onClickAdd',
         'click input.js_product_change': 'onChangeVariant',
         'change .js_main_product [data-attribute_exclusions]': 'onChangeVariant',
-        'change oe_optional_products_modal [data-attribute_exclusions]': 'onChangeVariant',
+        'change oe_advanced_configurator_modal [data-attribute_exclusions]': 'onChangeVariant',
+        'click .o_product_page_reviews_link': '_onClickReviewsLink',
     }),
 
     /**
@@ -196,11 +197,10 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
     /**
      * @override
      */
-    start: function () {
-        var self = this;
-        var def = this._super.apply(this, arguments);
+    start() {
+        const def = this._super(...arguments);
 
-        this._applyHash();
+        this._applyHashFromSearch();
 
         _.each(this.$('div.js_product'), function (product) {
             $('input.js_product_change', product).first().trigger('change');
@@ -219,11 +219,12 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
 
         this._startZoom();
 
-        window.addEventListener('hashchange', function (e) {
-            self._applyHash();
-            self.triggerVariantChange($(self.el));
+        window.addEventListener('hashchange', () => {
+            this._applyHash();
+            this.triggerVariantChange(this.$el);
         });
 
+        this.getRedirectOption();
         return def;
     },
     /**
@@ -260,7 +261,7 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
                         $toSelect.prop('selected', true);
                     }
                 });
-                this._changeColorAttribute();
+                this._changeAttribute(['.css_attribute_color', '.o_variant_pills']);
             }
         }
     },
@@ -278,14 +279,17 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
         history.replaceState(undefined, undefined, '#attr=' + attributeIds.join(','));
     },
     /**
-     * Set the checked color active.
+     * Set the checked values active.
      *
      * @private
+     * @param {Array} valueSelectors Selectors
      */
-    _changeColorAttribute: function () {
-        $('.css_attribute_color').removeClass("active")
-                                 .filter(':has(input:checked)')
-                                 .addClass("active");
+    _changeAttribute: function (valueSelectors) {
+        _.each(valueSelectors, function (selector) {
+            $(selector).removeClass("active")
+                       .filter(':has(input:checked)')
+                       .addClass("active");
+        });
     },
     /**
      * @private
@@ -319,7 +323,7 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
             }
             wSaleUtils.updateCartNavBar(data);
             $input.val(data.quantity);
-            $('.js_quantity[data-line-id='+line_id+']').val(data.quantity).html(data.quantity);
+            $('.js_quantity[data-line-id='+line_id+']').val(data.quantity).text(data.quantity);
 
             if (data.warning) {
                 var cart_alert = $('.oe_cart').parent().find('#data_warning');
@@ -344,17 +348,17 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
         this._rpc({
             route: "/shop/country_infos/" + $("#country_id").val(),
             params: {
-                mode: 'shipping',
+                mode: $("#country_id").attr('mode'),
             },
         }).then(function (data) {
             // placeholder phone_code
-            //$("input[name='phone']").attr('placeholder', data.phone_code !== 0 ? '+'+ data.phone_code : '');
+            $("input[name='phone']").attr('placeholder', data.phone_code !== 0 ? '+'+ data.phone_code : '');
 
             // populate states and display
             var selectStates = $("select[name='state_id']");
             // dont reload state at first loading (done in qweb)
             if (selectStates.data('init')===0 || selectStates.find('option').length===1) {
-                if (data.states.length) {
+                if (data.states.length || data.state_required) {
                     selectStates.html('');
                     _.each(data.states, function (x) {
                         var opt = $('<option>').text(x[1])
@@ -383,6 +387,15 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
                     $(".checkout_autoformat .div_" + field.split('_')[0]).toggle($.inArray(field, data.fields)>=0);
                 });
             }
+
+            if ($("label[for='zip']").length) {
+                $("label[for='zip']").toggleClass('label-optional', !data.zip_required);
+                $("label[for='zip']").get(0).toggleAttribute('required', !!data.zip_required);
+            }
+            if ($("label[for='zip']").length) {
+                $("label[for='state_id']").toggleClass('label-optional', !data.state_required);
+                $("label[for='state_id']").get(0).toggleAttribute('required', !!data.state_required);
+            }
         });
     },
     /**
@@ -466,7 +479,7 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
     _onClickAdd: function (ev) {
         ev.preventDefault();
         var def = () => {
-            this.isBuyNow = $(ev.currentTarget).attr('id') === 'buy_now';
+            this.getCartHandlerOptions(ev);
             return this._handleAdd($(ev.currentTarget).closest('form'));
         };
         if ($('.js_add_cart_variants').children().length) {
@@ -520,35 +533,27 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
 
     /**
      * Add custom variant values and attribute values that do not generate variants
-     * in the form data and trigger submit.
+     * in the params to submit form if 'stay on page' option is disabled, or call
+     * '_addToCartInPage' otherwise.
      *
      * @private
-     * @returns {Promise} never resolved
+     * @returns {Promise}
      */
     _submitForm: function () {
-        var $productCustomVariantValues = $('<input>', {
-            name: 'product_custom_attribute_values',
-            type: "hidden",
-            value: JSON.stringify(this.rootProduct.product_custom_attribute_values)
-        });
-        this.$form.append($productCustomVariantValues);
+        const params = this.rootProduct;
 
-        var $productNoVariantAttributeValues = $('<input>', {
-            name: 'no_variant_attribute_values',
-            type: "hidden",
-            value: JSON.stringify(this.rootProduct.no_variant_attribute_values)
-        });
-        this.$form.append($productNoVariantAttributeValues);
-
-        if (this.isBuyNow) {
-            this.$form.append($('<input>', {name: 'express', type: "hidden", value: true}));
+        const $product = $('#product_detail');
+        const productTrackingInfo = $product.data('product-tracking-info');
+        if (productTrackingInfo) {
+            productTrackingInfo.quantity = params.quantity;
+            $product.trigger('add_to_cart_event', [productTrackingInfo]);
         }
 
-        this.$form.trigger('submit', [true]);
-
-        return new Promise(function () {});
+        params.add_qty = params.quantity;
+        params.product_custom_attribute_values = JSON.stringify(params.product_custom_attribute_values);
+        params.no_variant_attribute_values = JSON.stringify(params.no_variant_attribute_values);
+        return this.addToCart(params);
     },
-
     /**
      * @private
      * @param {MouseEvent} ev
@@ -699,7 +704,7 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
     _toggleDisable: function ($parent, isCombinationPossible) {
         VariantMixin._toggleDisable.apply(this, arguments);
         $parent.find("#add_to_cart").toggleClass('disabled', !isCombinationPossible);
-        $parent.find("#buy_now").toggleClass('disabled', !isCombinationPossible);
+        $parent.find(".o_we_buy_now").toggleClass('disabled', !isCombinationPossible);
     },
     /**
      * Write the properties of the form elements in the DOM to prevent the
@@ -729,6 +734,33 @@ publicWidget.registry.WebsiteSale = publicWidget.Widget.extend(VariantMixin, {
         $('.toggle_summary_div').toggleClass('d-none');
         $('.toggle_summary_div').removeClass('d-xl-block');
     },
+    /**
+     * @private
+     */
+    _applyHashFromSearch() {
+        const params = $.deparam(window.location.search.slice(1));
+        if (params.attrib) {
+            const dataValueIds = [];
+            for (const attrib of [].concat(params.attrib)) {
+                const attribSplit = attrib.split('-');
+                const attribValueSelector = `.js_variant_change[name="ptal-${attribSplit[0]}"][value="${attribSplit[1]}"]`;
+                const attribValue = this.el.querySelector(attribValueSelector);
+                if (attribValue !== null) {
+                    dataValueIds.push(attribValue.dataset.value_id);
+                }
+            }
+            if (dataValueIds.length) {
+                history.replaceState(undefined, undefined, `#attr=${dataValueIds.join(',')}`);
+            }
+        }
+        this._applyHash();
+    },
+    /**
+     * @private
+     */
+    _onClickReviewsLink: function () {
+        $('#o_product_page_reviews_content').collapse('show');
+    },
 });
 
 publicWidget.registry.WebsiteSaleLayout = publicWidget.Widget.extend({
@@ -747,6 +779,10 @@ publicWidget.registry.WebsiteSaleLayout = publicWidget.Widget.extend({
      * @param {Event} ev
      */
     _onApplyShopLayoutChange: function (ev) {
+        const wysiwyg = this.options.wysiwyg;
+        if (wysiwyg) {
+            wysiwyg.odooEditor.observerUnactive('_onApplyShopLayoutChange');
+        }
         var switchToList = $(ev.currentTarget).find('.o_wsale_apply_list input').is(':checked');
         if (!this.editableMode) {
             this._rpc({
@@ -765,6 +801,9 @@ publicWidget.registry.WebsiteSaleLayout = publicWidget.Widget.extend({
         $grid.toggleClass('o_wsale_layout_list', switchToList);
         void $grid[0].offsetWidth;
         $grid.find('*').css('transition', '');
+        if (wysiwyg) {
+            wysiwyg.odooEditor.observerActive('_onApplyShopLayoutChange');
+        }
     },
 });
 
@@ -816,47 +855,33 @@ publicWidget.registry.websiteSaleCart = publicWidget.Widget.extend({
     },
 });
 
-/**
- * @todo maybe the custom autocomplete logic could be extract to be reusable
- */
-publicWidget.registry.productsSearchBar = publicWidget.Widget.extend({
-    selector: '.o_wsale_products_searchbar_form',
-    xmlDependencies: ['/website_sale/static/src/xml/website_sale_utils.xml'],
+publicWidget.registry.websiteSaleCarouselProduct = publicWidget.Widget.extend({
+    selector: '#o-carousel-product',
+    disabledInEditableMode: false,
     events: {
-        'input .search-query': '_onInput',
-        'focusout': '_onFocusOut',
-        'keydown .search-query': '_onKeydown',
+        'wheel .o_carousel_product_indicators': '_onMouseWheel',
     },
-    autocompleteMinWidth: 300,
 
     /**
-     * @constructor
+     * @override
      */
-    init: function () {
-        this._super.apply(this, arguments);
-
-        this._dp = new concurrency.DropPrevious();
-
-        this._onInput = _.debounce(this._onInput, 400);
-        this._onFocusOut = _.debounce(this._onFocusOut, 100);
+    async start() {
+        await this._super(...arguments);
+        this._updateCarouselPosition();
+        extraMenuUpdateCallbacks.push(this._updateCarouselPosition.bind(this));
+        if (this.$target.find('.carousel-indicators').length > 0) {
+            this.$target.on('slide.bs.carousel.carousel_product_slider', this._onSlideCarouselProduct.bind(this));
+            $(window).on('resize.carousel_product_slider', _.throttle(this._onSlideCarouselProduct.bind(this), 150));
+            this._updateJustifyContent();
+        }
     },
     /**
      * @override
      */
-    start: function () {
-        this.$input = this.$('.search-query');
-
-        this.order = this.$('.o_wsale_search_order_by').val();
-        this.limit = parseInt(this.$input.data('limit'));
-        this.displayDescription = !!this.$input.data('displayDescription');
-        this.displayPrice = !!this.$input.data('displayPrice');
-        this.displayImage = !!this.$input.data('displayImage');
-
-        if (this.limit) {
-            this.$input.attr('autocomplete', 'off');
-        }
-
-        return this._super.apply(this, arguments);
+    destroy() {
+        this.$target.css('top', '');
+        this.$target.off('.carousel_product_slider');
+        this._super(...arguments);
     },
 
     //--------------------------------------------------------------------------
@@ -866,41 +891,8 @@ publicWidget.registry.productsSearchBar = publicWidget.Widget.extend({
     /**
      * @private
      */
-    _fetch: function () {
-        return this._rpc({
-            route: '/shop/products/autocomplete',
-            params: {
-                'term': this.$input.val(),
-                'options': {
-                    'order': this.order,
-                    'limit': this.limit,
-                    'display_description': this.displayDescription,
-                    'display_price': this.displayPrice,
-                    'max_nb_chars': Math.round(Math.max(this.autocompleteMinWidth, parseInt(this.$el.width())) * 0.22),
-                },
-            },
-        });
-    },
-    /**
-     * @private
-     */
-    _render: function (res) {
-        var $prevMenu = this.$menu;
-        this.$el.toggleClass('dropdown show', !!res);
-        if (res) {
-            var products = res['products'];
-            this.$menu = $(qweb.render('website_sale.productsSearchBar.autocomplete', {
-                products: products,
-                hasMoreProducts: products.length < res['products_count'],
-                currency: res['currency'],
-                widget: this,
-            }));
-            this.$menu.css('min-width', this.autocompleteMinWidth);
-            this.$el.append(this.$menu);
-        }
-        if ($prevMenu) {
-            $prevMenu.remove();
-        }
+    _updateCarouselPosition() {
+        this.$target.css('top', dom.scrollFixedOffset() + 5);
     },
 
     //--------------------------------------------------------------------------
@@ -908,39 +900,130 @@ publicWidget.registry.productsSearchBar = publicWidget.Widget.extend({
     //--------------------------------------------------------------------------
 
     /**
+     * Center the selected indicator to scroll the indicators list when it
+     * overflows.
+     * 
      * @private
+     * @param {Event} ev
      */
-    _onInput: function () {
-        if (!this.limit) {
-            return;
-        }
-        this._dp.add(this._fetch()).then(this._render.bind(this));
+    _onSlideCarouselProduct: function (ev) {
+        const isReversed = this.$target.css('flex-direction') === "column-reverse";
+        const isLeftIndicators = this.$target.hasClass('o_carousel_product_left_indicators');
+        const $indicatorsDiv = isLeftIndicators ? this.$target.find('.o_carousel_product_indicators') : this.$target.find('.carousel-indicators');
+        let indicatorIndex = $(ev.relatedTarget).index();
+        indicatorIndex = indicatorIndex > -1 ? indicatorIndex : this.$target.find('li.active').index();
+        const $indicator = $indicatorsDiv.find('[data-slide-to=' + indicatorIndex + ']');
+        const indicatorsDivSize = isLeftIndicators && !isReversed ? $indicatorsDiv.outerHeight() : $indicatorsDiv.outerWidth();
+        const indicatorSize = isLeftIndicators && !isReversed ? $indicator.outerHeight() : $indicator.outerWidth();
+        const indicatorPosition = isLeftIndicators && !isReversed ? $indicator.position().top : $indicator.position().left;
+        const scrollSize = isLeftIndicators && !isReversed ? $indicatorsDiv[0].scrollHeight : $indicatorsDiv[0].scrollWidth;
+        let indicatorsPositionDiff = (indicatorPosition + (indicatorSize/2)) - (indicatorsDivSize/2);
+        indicatorsPositionDiff = Math.min(indicatorsPositionDiff, scrollSize - indicatorsDivSize);
+        this._updateJustifyContent();
+        const indicatorsPositionX = isLeftIndicators && !isReversed ? '0' : '-' + indicatorsPositionDiff;
+        const indicatorsPositionY = isLeftIndicators && !isReversed ? '-' + indicatorsPositionDiff : '0';
+        const translate3D = indicatorsPositionDiff > 0 ? "translate3d(" + indicatorsPositionX + "px," + indicatorsPositionY + "px,0)" : '';
+        $indicatorsDiv.css("transform", translate3D);
     },
     /**
      * @private
      */
-    _onFocusOut: function () {
-        if (!this.$el.has(document.activeElement).length) {
-            this._render();
+     _updateJustifyContent: function () {
+        const $indicatorsDiv = this.$target.find('.carousel-indicators');
+        $indicatorsDiv.css('justify-content', 'start');
+        if (config.device.size_class <= config.device.SIZES.MD) {
+            if (($indicatorsDiv.children().last().position().left + this.$target.find('li').outerWidth()) < $indicatorsDiv.outerWidth()) {
+                $indicatorsDiv.css('justify-content', 'center');
+            }
         }
     },
     /**
      * @private
+     * @param {Event} ev
      */
-    _onKeydown: function (ev) {
-        switch (ev.which) {
-            case $.ui.keyCode.ESCAPE:
-                this._render();
-                break;
-            case $.ui.keyCode.UP:
-            case $.ui.keyCode.DOWN:
-                ev.preventDefault();
-                if (this.$menu) {
-                    let $element = ev.which === $.ui.keyCode.UP ? this.$menu.children().last() : this.$menu.children().first();
-                    $element.focus();
-                }
-                break;
+    _onMouseWheel: function (ev) {
+        ev.preventDefault();
+        if (ev.originalEvent.deltaY > 0) {
+            this.$target.carousel('next');
+        } else {
+            this.$target.carousel('prev');
         }
     },
 });
+
+publicWidget.registry.websiteSaleProductPageReviews = publicWidget.Widget.extend({
+    selector: '#o_product_page_reviews',
+    disabledInEditableMode: false,
+
+    /**
+     * @override
+     */
+    async start() {
+        await this._super(...arguments);
+        this._updateChatterComposerPosition();
+        extraMenuUpdateCallbacks.push(this._updateChatterComposerPosition.bind(this));
+    },
+    /**
+     * @override
+     */
+    destroy() {
+        this.$target.find('.o_portal_chatter_composer').css('top', '');
+        this._super(...arguments);
+    },
+
+    //--------------------------------------------------------------------------
+    // Private
+    //--------------------------------------------------------------------------
+
+    /**
+     * @private
+     */
+    _updateChatterComposerPosition() {
+        this.$target.find('.o_portal_chatter_composer').css('top', dom.scrollFixedOffset() + 20);
+    },
+});
+
+return {
+    WebsiteSale: publicWidget.registry.WebsiteSale,
+    WebsiteSaleLayout: publicWidget.registry.WebsiteSaleLayout,
+    websiteSaleCart: publicWidget.registry.websiteSaleCart,
+    WebsiteSaleCarouselProduct: publicWidget.registry.websiteSaleCarouselProduct,
+    WebsiteSaleProductPageReviews: publicWidget.registry.websiteSaleProductPageReviews,
+};
+
+});
+
+odoo.define('website_sale.price_range_option', function (require) {
+'use strict';
+
+const publicWidget = require('web.public.widget');
+
+publicWidget.registry.multirangePriceSelector = publicWidget.Widget.extend({
+    selector: '#o_wsale_price_range_option',
+    events: {
+        'newRangeValue input[type="range"]': '_onPriceRangeSelected',
+    },
+
+    //----------------------------------------------------------------------
+    // Handlers
+    //----------------------------------------------------------------------
+
+    /**
+     * @private
+     * @param {Event} ev
+     */
+    _onPriceRangeSelected(ev) {
+        const range = ev.currentTarget;
+        const search = $.deparam(window.location.search.substring(1));
+        delete search.min_price;
+        delete search.max_price;
+        if (parseFloat(range.min) !== range.valueLow) {
+            search['min_price'] = range.valueLow;
+        }
+        if (parseFloat(range.max) !== range.valueHigh) {
+            search['max_price'] = range.valueHigh;
+        }
+        window.location.search = $.param(search);
+    },
+});
 });
