@@ -2,15 +2,16 @@ odoo.define("website_sale.tour_shop_custom_attribute_value", function (require)
     "use strict";
 
     var tour = require("web_tour.tour");
+    const tourUtils = require('website_sale.tour_utils');
 
     tour.register("shop_custom_attribute_value", {
         url: "/shop?search=Customizable Desk",
         test: true,
     }, [{
         content: "click on Customizable Desk",
-        trigger: '.oe_product_cart a:contains("Customizable Desk")',
+        trigger: '.oe_product_cart a:contains("Customizable Desk (TEST)")',
     }, {
-        trigger: 'li.js_attribute_value span:contains(Custom)',
+        trigger: 'li.js_attribute_value span:contains(Custom TEST)',
         extra_trigger: 'li.js_attribute_value',
         run: 'click',
     }, {
@@ -18,10 +19,12 @@ odoo.define("website_sale.tour_shop_custom_attribute_value", function (require)
         run: 'text Wood',
     }, {
         id: 'add_cart_step',
-        trigger: 'a:contains(Add to Cart)',
+        trigger: 'a:contains(ADD TO CART)',
         run: 'click',
-    }, {
-        trigger: 'span:contains(Custom: Wood)',
+    },
+        tourUtils.goToCart(), 
+    {
+        trigger: 'span:contains(Custom TEST: Wood)',
         extra_trigger: '#cart_products',
         run: function (){}, // check
     }]);
