@@ -1,6 +1,10 @@
+# -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
+
 from . import test_customize
 from . import test_sale_process
 from . import test_sitemap
+from . import test_website_sale_cart_abandoned
 from . import test_website_sale_cart_recovery
 from . import test_website_sale_mail
 from . import test_website_sale_pricelist
