@@ -1,10 +1,8 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch
+from unittest.mock import patch
 
+from odoo.addons.base.tests.common import TransactionCaseWithUserDemo, HttpCaseWithUserPortal
 from odoo.addons.website.tools import MockRequest
 from odoo.tests import tagged
 from odoo.tests.common import HttpCase, TransactionCase
@@ -39,15 +37,58 @@ class TestWebsitePriceList(TransactionCase):
     def setUp(self):
         super(TestWebsitePriceList, self).setUp()
         self.env.user.partner_id.country_id = False  # Remove country to avoid property pricelist computed.
-        self.website = self.env['website'].browse(1)
+        self.website = self.env.ref('website.default_website')
         self.website.user_id = self.env.user
 
-        self.env['product.pricelist'].search([]).write({'website_id': False})
-        website_pls = ('list_benelux', 'list_christmas', 'list_europe')
-        for pl in website_pls:
-            self.env.ref('website_sale.' + pl).website_id = self.website.id
+        (self.env['product.pricelist'].search([]) - self.env.ref('product.list0')).write({'website_id': False, 'active': False})
+        self.benelux = self.env['res.country.group'].create({
+            'name': 'BeNeLux',
+            'country_ids': [(6, 0, (self.env.ref('base.be') + self.env.ref('base.lu') + self.env.ref('base.nl')).ids)]
+        })
+        self.list_benelux = self.env['product.pricelist'].create({
+            'name': 'Benelux',
+            'selectable': True,
+            'website_id': self.website.id,
+            'country_group_ids': [(4, self.benelux.id)],
+            'sequence': 2,
+        })
+        item_benelux = self.env['product.pricelist.item'].create({
+            'pricelist_id': self.list_benelux.id,
+            'compute_price': 'percentage',
+            'base': 'list_price',
+            'percent_price': 10,
+            'currency_id': self.env.ref('base.EUR').id,
+        })
+
+
+        self.list_christmas = self.env['product.pricelist'].create({
+            'name': 'Christmas',
+            'selectable': False,
+            'website_id': self.website.id,
+            'country_group_ids': [(4, self.env.ref('base.europe').id)],
+            'sequence': 20,
+        })
+        item_christmas = self.env['product.pricelist.item'].create({
+            'pricelist_id': self.list_christmas.id,
+            'compute_price': 'formula',
+            'base': 'list_price',
+            'price_discount': 20,
+        })
+
+        list_europe = self.env['product.pricelist'].create({
+            'name': 'EUR',
+            'selectable': True,
+            'website_id': self.website.id,
+            'country_group_ids': [(4, self.env.ref('base.europe').id)],
+            'sequence': 3,
+            'currency_id': self.env.ref('base.EUR').id,
+        })
+        item_europe = self.env['product.pricelist.item'].create({
+            'pricelist_id': list_europe.id,
+            'compute_price': 'formula',
+            'base': 'list_price',
+        })
         self.env.ref('product.list0').website_id = self.website.id
-        self.env.ref('website_sale.list_benelux').selectable = True
         self.website.pricelist_id = self.ref('product.list0')
 
         ca_group = self.env['res.country.group'].create({
@@ -93,7 +134,7 @@ class TestWebsitePriceList(TransactionCase):
         }
         for country, result in country_list.items():
             pls = self.get_pl(show, current_pl, country)
-            self.assertEquals(len(set(pls.mapped('name')) & set(result)), len(pls), 'Test failed for %s (%s %s vs %s %s)'
+            self.assertEqual(len(set(pls.mapped('name')) & set(result)), len(pls), 'Test failed for %s (%s %s vs %s %s)'
                               % (country, len(pls), pls.mapped('name'), len(result), result))
 
     def test_get_pricelist_available_not_show(self):
@@ -110,11 +151,11 @@ class TestWebsitePriceList(TransactionCase):
 
         for country, result in country_list.items():
             pls = self.get_pl(show, current_pl, country)
-            self.assertEquals(len(set(pls.mapped('name')) & set(result)), len(pls), 'Test failed for %s (%s %s vs %s %s)'
+            self.assertEqual(len(set(pls.mapped('name')) & set(result)), len(pls), 'Test failed for %s (%s %s vs %s %s)'
                               % (country, len(pls), pls.mapped('name'), len(result), result))
 
     def test_get_pricelist_available_promocode(self):
-        christmas_pl = self.ref('website_sale.list_christmas')
+        christmas_pl = self.list_christmas.id
 
         country_list = {
             False: True,
@@ -147,7 +188,7 @@ class TestWebsitePriceList(TransactionCase):
         }
         for country, result in country_list.items():
             pls = self.get_pl(show, current_pl, country)
-            self.assertEquals(len(set(pls.mapped('name')) & set(result)), len(pls), 'Test failed for %s (%s %s vs %s %s)'
+            self.assertEqual(len(set(pls.mapped('name')) & set(result)), len(pls), 'Test failed for %s (%s %s vs %s %s)'
                               % (country, len(pls), pls.mapped('name'), len(result), result))
 
     def test_pricelist_combination(self):
@@ -355,7 +396,11 @@ class TestWebsitePriceListAvailableGeoIP(TestWebsitePriceListAvailable):
 
         # set different country groups on pricelists
         c_EUR = self.env.ref('base.europe')
-        c_BENELUX = self.env.ref('website_sale.benelux')
+        c_BENELUX = self.env['res.country.group'].create({
+            'name': 'BeNeLux',
+            'country_ids': [(6, 0, (self.env.ref('base.be') + self.env.ref('base.lu') + self.env.ref('base.nl')).ids)]
+        })
+
         self.BE = self.env.ref('base.be')
         NL = self.env.ref('base.nl')
         c_BE = self.env['res.country.group'].create({'name': 'Belgium', 'country_ids': [(6, 0, [self.BE.id])]})
@@ -414,7 +459,7 @@ class TestWebsitePriceListAvailableGeoIP(TestWebsitePriceListAvailable):
 
 
 @tagged('post_install', '-at_install')
-class TestWebsitePriceListHttp(HttpCase):
+class TestWebsitePriceListHttp(HttpCaseWithUserPortal):
     def test_get_pricelist_available_multi_company(self):
         ''' Test that the `property_product_pricelist` of `res.partner` is not
             computed as SUPERUSER_ID.
@@ -425,6 +470,7 @@ class TestWebsitePriceListHttp(HttpCase):
             reading that `property_product_pricelist`.
         '''
         test_company = self.env['res.company'].create({'name': 'Test Company'})
+        test_company.flush()
         self.env['product.pricelist'].create({
             'name': 'Backend Pricelist For "Test Company"',
             'website_id': False,
@@ -438,7 +484,7 @@ class TestWebsitePriceListHttp(HttpCase):
 
 
 @tagged('post_install', '-at_install')
-class TestWebsitePriceListMultiCompany(TransactionCase):
+class TestWebsitePriceListMultiCompany(TransactionCaseWithUserDemo):
     def setUp(self):
         ''' Create a basic multi-company pricelist environment:
         - Set up 2 companies with their own company-restricted pricelist each.
@@ -449,48 +495,46 @@ class TestWebsitePriceListMultiCompany(TransactionCase):
         '''
         super(TestWebsitePriceListMultiCompany, self).setUp()
 
-        self.demo_user = self.env.ref('base.user_demo')
+        self.demo_user = self.user_demo
 
         # Create and add demo user to 2 companies
         self.company1 = self.demo_user.company_id
         self.company2 = self.env['res.company'].create({'name': 'Test Company'})
         self.demo_user.company_ids += self.company2
         # Set company2 as current company for demo user
-        self.website = self.env['website'].browse(1)
+        Website = self.env['website']
+        self.website = self.env.ref('website.default_website')
         self.website.company_id = self.company2
+        # Delete unused website, it will make PL manipulation easier, avoiding
+        # UserError being thrown when a website wouldn't have any PL left.
+        Website.search([('id', '!=', self.website.id)]).unlink()
+        self.website2 = Website.create({
+            'name': 'Website 2',
+            'company_id': self.company1.id,
+        })
 
         # Create a company pricelist for each company and set it to demo user
         self.c1_pl = self.env['product.pricelist'].create({
             'name': 'Company 1 Pricelist',
             'company_id': self.company1.id,
+            # The `website_id` field will default to the company's website,
+            # in this case `self.website2`.
+
         })
         self.c2_pl = self.env['product.pricelist'].create({
             'name': 'Company 2 Pricelist',
             'company_id': self.company2.id,
             'website_id': False,
         })
-        self.demo_user.partner_id.property_product_pricelist = self.c1_pl
-        # Switch env.user company to create ir.property in company2
-        self.env.user.company_id = self.company2
-        self.demo_user.partner_id.property_product_pricelist = self.c2_pl
+        self.demo_user.partner_id.with_company(self.company1.id).property_product_pricelist = self.c1_pl
+        self.demo_user.partner_id.with_company(self.company2.id).property_product_pricelist = self.c2_pl
 
         # Ensure everything was done correctly
-        self.assertEqual(self.demo_user.partner_id.with_context(force_company=self.company1.id).property_product_pricelist, self.c1_pl)
-        self.assertEqual(self.demo_user.partner_id.with_context(force_company=self.company2.id).property_product_pricelist, self.c2_pl)
-        irp1 = self.env['ir.property'].search([
-            ('name', '=', 'property_product_pricelist'),
-            ('company_id', '=', self.company1.id),
-            ('res_id', '=', 'res.partner,%s' % self.demo_user.partner_id.id),
-            ('value_reference', '=', 'product.pricelist,%s' % self.c1_pl.id),
-        ])
-        irp2 = self.env['ir.property'].search([
-            ('name', '=', 'property_product_pricelist'),
-            ('company_id', '=', self.company2.id),
-            ('res_id', '=', 'res.partner,%s' % self.demo_user.partner_id.id),
-            ('value_reference', '=', 'product.pricelist,%s' % self.c2_pl.id),
-        ])
-        self.assertEqual(len(irp1 + irp2), 2, "Ensure there is an `ir.property` for demo partner for every company, and that the pricelist is the company specific one.")
-        simulate_frontend_context(self)
+        self.assertEqual(self.demo_user.partner_id.with_company(self.company1.id).property_product_pricelist, self.c1_pl)
+        self.assertEqual(self.demo_user.partner_id.with_company(self.company2.id).property_product_pricelist, self.c2_pl)
+        irp1 = self.env['ir.property'].with_company(self.company1)._get("property_product_pricelist", "res.partner", self.demo_user.partner_id.id)
+        irp2 = self.env['ir.property'].with_company(self.company2)._get("property_product_pricelist", "res.partner", self.demo_user.partner_id.id)
+        self.assertEqual((irp1, irp2), (self.c1_pl, self.c2_pl), "Ensure there is an `ir.property` for demo partner for every company, and that the pricelist is the company specific one.")
         # ---------------------------------- IR.PROPERTY -------------------------------------
         # id |            name              |     res_id    | company_id |   value_reference
         # ------------------------------------------------------------------------------------
@@ -510,10 +554,12 @@ class TestWebsitePriceListMultiCompany(TransactionCase):
             for the company1 as we should get the website's company pricelist
             and not the demo user's current company pricelist.
         '''
+        simulate_frontend_context(self, self.website.id)
+
         # First check: It should return ir.property,4 as company_id is
         # website.company_id and not env.user.company_id
         company_id = self.website.company_id.id
-        partner = self.demo_user.partner_id.with_context(force_company=company_id)
+        partner = self.demo_user.partner_id.with_company(company_id)
         demo_pl = partner.property_product_pricelist
         self.assertEqual(demo_pl, self.c2_pl)
 
@@ -521,4 +567,29 @@ class TestWebsitePriceListMultiCompany(TransactionCase):
         # Indeed, the ir.rule for pricelists rights about company should allow to
         # also read a pricelist from another company if that company is the one
         # from the currently visited website.
-        self.env(user=self.env.ref('base.user_demo'))['product.pricelist'].browse(demo_pl.id).name
+        self.env(user=self.user_demo)['product.pricelist'].browse(demo_pl.id).name
+
+    def test_archive_pricelist_1(self):
+        ''' Test that when a pricelist is archived, the check that verify that
+            all website have at least one pricelist have access to all
+            pricelists (considering all companies).
+        '''
+
+        self.c2_pl.website_id = self.website
+        c2_pl2 = self.c2_pl.copy({'name': 'Copy of c2_pl'})
+        self.env['product.pricelist'].search([
+            ('id', 'not in', (self.c2_pl + self.c1_pl + c2_pl2).ids)
+        ]).write({'active': False})
+
+        # ---------------- PRICELISTS ----------------
+        #    name    |   website_id  |  company_id   |
+        # --------------------------------------------
+        # self.c1_pl | self.website2 | self.company1 |
+        # self.c2_pl | self.website  | self.company2 |
+        # c2_pl2     | self.website  | self.company2 |
+
+        self.demo_user.groups_id += self.env.ref('sales_team.group_sale_manager')
+
+        # The test is here: while having access only to self.company2 records,
+        # archive should not raise an error
+        self.c2_pl.with_user(self.demo_user).with_context(allowed_company_ids=self.company2.ids).write({'active': False})
