@@ -3,7 +3,7 @@
 
 {
     'name': 'Product Availability',
-    'category': 'Website',
+    'category': 'Website/Website',
     'summary': 'Manage product inventory & availability',
     'description': """
 Manage the inventory of your products and display their availability status in your eCommerce store.
@@ -25,4 +25,10 @@ Then it can be made specific at the product level.
         'data/website_sale_stock_demo.xml',
     ],
     'auto_install': True,
+    'assets': {
+        'web.assets_frontend': [
+            'website_sale_stock/static/src/js/**/*',
+        ],
+    },
+    'license': 'LGPL-3',
 }
