@@ -1,20 +1,15 @@
-var onYouTubeIframeAPIReady = undefined;
+/** @odoo-module **/
 
-odoo.define('website_slides.fullscreen', function (require) {
-    'use strict';
+/* global YT */
 
-    var publicWidget = require('web.public.widget');
-    var core = require('web.core');
-    var config = require('web.config');
-    var QWeb = core.qweb;
-    var _t = core._t;
-
-    var session = require('web.session');
-
-    var Quiz = require('website_slides.quiz');
-
-    var Dialog = require('web.Dialog');
+    import publicWidget from 'web.public.widget';
+    import  { qweb as QWeb, _t } from 'web.core';
+    import config from 'web.config';
 
+    import session from 'web.session';
+    import { Quiz } from '@website_slides/js/slides_course_quiz';
+    import Dialog from 'web.Dialog';
+    import '@website_slides/js/slides_course_join';
 
     /**
      * Helper: Get the slide dict matching the given criteria
@@ -58,7 +53,7 @@ odoo.define('website_slides.fullscreen', function (require) {
 
                     // function called when the Youtube asset is loaded
                     // see https://developers.google.com/youtube/iframe_api_reference#Requirements
-                    onYouTubeIframeAPIReady = function () {
+                    window.onYouTubeIframeAPIReady = function () {
                         resolve();
                     };
                 } else {
@@ -74,7 +69,6 @@ odoo.define('website_slides.fullscreen', function (require) {
          */
         _setupYoutubePlayer: function (){
             this.player = new YT.Player('youtube-player' + this.slide.id, {
-                host: 'https://www.youtube.com',
                 playerVars: {
                     'autoplay': 1,
                     'origin': window.location.origin
@@ -440,9 +434,7 @@ odoo.define('website_slides.fullscreen', function (require) {
         start: function (){
             var self = this;
             this.on('change:slide', this, this._onChangeSlide);
-            if  (config.device.size_class > config.device.SIZES.MD) {
-                this._toggleSidebar();
-            }
+            this._toggleSidebar();
             return this._super.apply(this, arguments).then(function () {
                 return self._onChangeSlide(); // trigger manually once DOM ready, since slide content is not rendered server side
             });
@@ -613,10 +605,12 @@ odoo.define('website_slides.fullscreen', function (require) {
         // Handlers
         //--------------------------------------------------------------------------
         /**
-         * Method called whenever the user changes slides.
-         * When the current slide is changed, it will autoimatically update the widget
-         * and allow it to fetch the content if needs be, render it, update the url and set the
-         * slide as "completed" if its type makes it necessary
+         * Triggered whenever the user changes slides.
+         * When the current slide is changed, widget will be automatically updated
+         * and allowed to: fetch the content if needed, render it, update the url,
+         * and set slide as "completed" according to its type requirements. In
+         * mobile case (i.e. limited screensize), sidebar will be toggled since 
+         * sidebar will block most or all of new slide visibility.
          *
          * @private
          */
@@ -627,6 +621,9 @@ odoo.define('website_slides.fullscreen', function (require) {
             return this._fetchSlideContent().then(function() { // render content
                 var websiteName = document.title.split(" | ")[1]; // get the website name from title
                 document.title =  (websiteName) ? slide.name + ' | ' + websiteName : slide.name;
+                if  (config.device.size_class < config.device.SIZES.MD) {
+                    self._toggleSidebar(); // hide sidebar when small device screen
+                }
                 return self._renderSlide();
             }).then(function() {
                 if (slide._autoSetDone && !session.is_website_user) {  // no useless RPC call
@@ -708,11 +705,32 @@ odoo.define('website_slides.fullscreen', function (require) {
         selector: '.o_wslides_fs_main',
         xmlDependencies: ['/website_slides/static/src/xml/website_slides_fullscreen.xml', '/website_slides/static/src/xml/website_slides_share.xml'],
         start: function (){
+            var self = this;
             var proms = [this._super.apply(this, arguments)];
             var fullscreen = new Fullscreen(this, this._getSlides(), this._getCurrentSlideID(), this._extractChannelData());
             proms.push(fullscreen.attachTo(".o_wslides_fs_main"));
-            return Promise.all(proms);
+            return Promise.all(proms).then(function () {
+                $('#edit-page-menu a[data-action="edit"]').on('click', self._onWebEditorClick.bind(self));
+            });
         },
+
+        /**
+         * The web editor does not work well with the e-learning fullscreen view.
+         * It actually completely closes the fullscreen view and opens the edition on a blank page.
+         *
+         * To avoid this, we intercept the click on the 'edit' button and redirect to the
+         * non-fullscreen view of this slide with the editor enabled, which is more suited to edit
+         * in-place anyway.
+         *
+         * @param {MouseEvent} e
+         */
+        _onWebEditorClick: function (e) {
+            e.preventDefault();
+            e.stopPropagation();
+
+            window.location = `${window.location.pathname}?fullscreen=0&enable_editor=1`;
+        },
+
         _extractChannelData: function (){
             return this.$el.data();
         },
@@ -734,5 +752,4 @@ odoo.define('website_slides.fullscreen', function (require) {
         },
     });
 
-    return Fullscreen;
-});
+    export default Fullscreen;
