@@ -1,10 +1,8 @@
-odoo.define('website_slides.slides_share', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var publicWidget = require('web.public.widget');
-require('website_slides.slides');
-var core = require('web.core');
-var _t = core._t;
+import publicWidget from 'web.public.widget';
+import '@website_slides/js/slides';
+import { _t } from 'web.core';
 
 var ShareMail = publicWidget.Widget.extend({
     events: {
@@ -79,12 +77,13 @@ publicWidget.registry.websiteSlidesShare = publicWidget.Widget.extend({
 
     _onShareLinkCopy: function (ev) {
         ev.preventDefault();
-        var $clipboardBtn = this.$('.o_clipboard_button');
+        var $clipboardBtn = $(ev.currentTarget);
         $clipboardBtn.tooltip({title: "Copied !", trigger: "manual", placement: "bottom"});
         var self = this;
-        var clipboard = new ClipboardJS('.o_clipboard_button', {
+        var clipboard = new ClipboardJS('#' + $clipboardBtn[0].id, {
             target: function () {
-                return self.$('.o_wslides_js_share_link')[0];
+                var share_link_el = self.$('#wslides_share_link_id_' + $clipboardBtn[0].id.split('id_')[1]);
+                return share_link_el[0];
             },
             container: this.el
         });
@@ -101,4 +100,3 @@ publicWidget.registry.websiteSlidesShare = publicWidget.Widget.extend({
         })
     },
 });
-});
