@@ -3,7 +3,7 @@
 
 {
     'name': 'Forum on Courses',
-    'category': 'Hidden',
+    'category': 'Website/eLearning',
     'version': '1.0',
     'summary': 'Allows to link forum on a course',
     'description': """A Slide channel can be linked to forum. Also, profiles from slide and forum are regrouped together""",
@@ -13,10 +13,10 @@
     ],
     'data': [
         'security/ir.model.access.csv',
+        'security/website_slides_forum_security.xml',
         'views/forum_views.xml',
         'views/slide_channel_views.xml',
         'views/website_slides_menu_views.xml',
-        'views/assets.xml',
         'views/website_slides_templates.xml',
         'views/website_slides_forum_templates.xml'
     ],
@@ -24,5 +24,10 @@
         'data/slide_channel_demo.xml',
     ],
     'auto_install': True,
+    'assets': {
+        'web.assets_frontend': [
+            'website_slides_forum/static/src/js/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
