@@ -4,13 +4,13 @@
     'name': "Course Certifications",
     'summary': 'Add certification capabilities to your courses',
     'description': """This module lets you use the full power of certifications within your courses.""",
-    'category': 'Hidden',
+    'category': 'Website/eLearning',
     'version': '1.0',
     'depends': ['website_slides', 'survey'],
     'installable': True,
     'auto_install': True,
     'data': [
-        'views/assets.xml',
+        'security/ir.model.access.csv',
         'views/slide_channel_views.xml',
         'views/slide_slide_views.xml',
         'views/survey_survey_views.xml',
@@ -27,7 +27,18 @@
     'demo': [
         'data/survey_demo.xml',
         'data/slide_slide_demo.xml',
-        'data/survey.user_input_line.csv',
+        'data/survey.user_input.line.csv',
     ],
+    'assets': {
+        'web.assets_frontend': [
+            'website_slides_survey/static/src/scss/website_slides_survey.scss',
+            'website_slides_survey/static/src/js/slides_upload.js',
+            'website_slides_survey/static/src/js/slides_course_fullscreen_player.js',
+            'website_slides_survey/static/src/js/slides_certification_upload_toast.js',
+        ],
+        'survey.survey_assets': [
+            'website_slides_survey/static/src/scss/website_slides_survey_result.scss',
+        ],
+    },
     'license': 'LGPL-3',
 }
