@@ -17,4 +17,14 @@ This module adds a Twitter scroller building block to the website builder, so th
         'views/website_twitter_snippet_templates.xml'
     ],
     'installable': True,
+    'assets': {
+        'web.assets_frontend': [
+            'website_twitter/static/src/scss/website_twitter.scss',
+            'website_twitter/static/src/js/website.twitter.animation.js',
+        ],
+        'website.assets_editor': [
+            'website_twitter/static/src/js/website.twitter.editor.js',
+        ],
+    },
+    'license': 'LGPL-3',
 }
