@@ -15,20 +15,19 @@ __all__ = [
 ]
 
 import logging
-from collections import defaultdict, Mapping
+import warnings
+from collections import defaultdict
+from collections.abc import Mapping
 from contextlib import contextmanager
-from copy import deepcopy
-from inspect import getargspec
+from inspect import signature
 from pprint import pformat
 from weakref import WeakSet
 
-from decorator import decorate, decorator
-from werkzeug.local import Local, release_local
+from decorator import decorate
 
-import odoo
+from .exceptions import CacheMiss
+from .tools import frozendict, classproperty, lazy_property, StackMap
 from .tools.translate import _
-from odoo.tools import frozendict, classproperty, lazy_property, StackMap
-from odoo.exceptions import CacheMiss
 
 _logger = logging.getLogger(__name__)
 
@@ -38,6 +37,7 @@ _logger = logging.getLogger(__name__)
 #  - method._returns: set by @returns, specifies return model
 #  - method._onchange: set by @onchange, specifies onchange fields
 #  - method.clear_cache: set by @ormcache, used to clear the cache
+#  - method._ondelete: set by @ondelete, used to raise errors for unlink operations
 #
 # On wrapping method only:
 #  - method._api: decorator function, used for re-applying decorator
@@ -127,10 +127,74 @@ def constrains(*args):
         sure a constraint will always be triggered (e.g. to test the absence of
         value).
 
+    One may also pass a single function as argument.  In that case, the field
+    names are given by calling the function with a model instance.
+
     """
+    if args and callable(args[0]):
+        args = args[0]
     return attrsetter('_constrains', args)
 
 
+def ondelete(*, at_uninstall):
+    """
+    Mark a method to be executed during :meth:`~odoo.models.BaseModel.unlink`.
+
+    The goal of this decorator is to allow client-side errors when unlinking
+    records if, from a business point of view, it does not make sense to delete
+    such records. For instance, a user should not be able to delete a validated
+    sales order.
+
+    While this could be implemented by simply overriding the method ``unlink``
+    on the model, it has the drawback of not being compatible with module
+    uninstallation. When uninstalling the module, the override could raise user
+    errors, but we shouldn't care because the module is being uninstalled, and
+    thus **all** records related to the module should be removed anyway.
+
+    This means that by overriding ``unlink``, there is a big chance that some
+    tables/records may remain as leftover data from the uninstalled module. This
+    leaves the database in an inconsistent state. Moreover, there is a risk of
+    conflicts if the module is ever reinstalled on that database.
+
+    Methods decorated with ``@ondelete`` should raise an error following some
+    conditions, and by convention, the method should be named either
+    ``_unlink_if_<condition>`` or ``_unlink_except_<not_condition>``.
+
+    .. code-block:: python
+
+        @api.ondelete(at_uninstall=False)
+        def _unlink_if_user_inactive(self):
+            if any(user.active for user in self):
+                raise UserError("Can't delete an active user!")
+
+        # same as above but with _unlink_except_* as method name
+        @api.ondelete(at_uninstall=False)
+        def _unlink_except_active_user(self):
+            if any(user.active for user in self):
+                raise UserError("Can't delete an active user!")
+
+    :param bool at_uninstall: Whether the decorated method should be called if
+        the module that implements said method is being uninstalled. Should
+        almost always be ``False``, so that module uninstallation does not
+        trigger those errors.
+
+    .. danger::
+        The parameter ``at_uninstall`` should only be set to ``True`` if the
+        check you are implementing also applies when uninstalling the module.
+
+        For instance, it doesn't matter if when uninstalling ``sale``, validated
+        sales orders are being deleted because all data pertaining to ``sale``
+        should be deleted anyway, in that case ``at_uninstall`` should be set to
+        ``False``.
+
+        However, it makes sense to prevent the removal of the default language
+        if no other languages are installed, since deleting the default language
+        will break a lot of basic behavior. In this case, ``at_uninstall``
+        should be set to ``True``.
+    """
+    return attrsetter('_ondelete', at_uninstall)
+
+
 def onchange(*args):
     """Return a decorator to decorate an onchange method for given fields.
 
@@ -148,7 +212,6 @@ def onchange(*args):
     .. code-block:: python
 
         return {
-            'domain': {'other_id': [('partner_id', '=', partner_id)]},
             'warning': {'title': "Warning", 'message': "What is this?", 'type': 'notification'},
         }
 
@@ -225,7 +288,7 @@ def depends_context(*args):
     All dependencies must be hashable.  The following keys have special
     support:
 
-    * `force_company` (value in context or current company id),
+    * `company` (value in context or current company id),
     * `uid` (current user id and superuser flag),
     * `active_test` (value in env.context or value in field.context).
     """
@@ -275,7 +338,7 @@ def downgrade(method, value, self, args, kwargs):
     if not spec:
         return value
     _, convert, _ = spec
-    if convert and len(getargspec(convert).args) > 1:
+    if convert and len(signature(convert).parameters) > 1:
         return convert(self, value, *args, **kwargs)
     elif convert:
         return convert(value)
@@ -293,6 +356,17 @@ def split_context(method, args, kwargs):
     return kwargs.pop('context', None), args, kwargs
 
 
+def autovacuum(method):
+    """
+    Decorate a method so that it is called by the daily vacuum cron job (model
+    ``ir.autovacuum``).  This is typically used for garbage-collection-like
+    tasks that do not deserve a specific cron job.
+    """
+    assert method.__name__.startswith('_'), "%s: autovacuum methods must be private" % method.__name__
+    method._autovacuum = True
+    return method
+
+
 def model(method):
     """ Decorate a record-style method where ``self`` is a recordset, but its
         contents is not relevant, only the model is. Such a method::
@@ -404,31 +478,25 @@ class Environment(Mapping):
         names to new api models. It also holds a cache for records, and a data
         structure to manage recomputations.
     """
-    _local = Local()
-
     @classproperty
     def envs(cls):
-        return getattr(cls._local, 'environments', ())
+        raise NotImplementedError(
+            "Since Odoo 15.0, Environment.envs no longer works; "
+            "use cr.transaction or env.transaction instead."
+        )
 
     @classmethod
     @contextmanager
     def manage(cls):
-        """ Context manager for a set of environments. """
-        if hasattr(cls._local, 'environments'):
-            yield
-        else:
-            try:
-                cls._local.environments = Environments()
-                yield
-            finally:
-                release_local(cls._local)
+        warnings.warn(
+            "Since Odoo 15.0, Environment.manage() is useless.",
+            DeprecationWarning, stacklevel=2,
+        )
+        yield
 
-    @classmethod
-    def reset(cls):
-        """ Clear the set of environments.
-            This may be useful when recreating a registry inside a transaction.
-        """
-        cls._local.environments = Environments()
+    def reset(self):
+        """ Reset the transaction, see :meth:`Transaction.reset`. """
+        self.transaction.reset()
 
     def __new__(cls, cr, uid, context, su=False):
         if uid == SUPERUSER_ID:
@@ -436,9 +504,13 @@ class Environment(Mapping):
         assert context is not None
         args = (cr, uid, context, su)
 
+        # determine transaction object
+        transaction = cr.transaction
+        if transaction is None:
+            transaction = cr.transaction = Transaction(Registry(cr.dbname))
+
         # if env already exists, return it
-        env, envs = None, cls.envs
-        for env in envs:
+        for env in transaction.envs:
             if env.args == args:
                 return env
 
@@ -446,11 +518,13 @@ class Environment(Mapping):
         self = object.__new__(cls)
         args = (cr, uid, frozendict(context), su)
         self.cr, self.uid, self.context, self.su = self.args = args
-        self.registry = Registry(cr.dbname)
-        self.cache = envs.cache
-        self._protected = envs.protected        # proxy to shared data structure
-        self.all = envs
-        envs.add(self)
+
+        self.transaction = self.all = transaction
+        self.registry = transaction.registry
+        self.cache = transaction.cache
+        self._cache_key = {}                    # memo {field: cache_key}
+        self._protected = transaction.protected
+        transaction.envs.add(self)
         return self
 
     #
@@ -501,7 +575,17 @@ class Environment(Mapping):
 
     def ref(self, xml_id, raise_if_not_found=True):
         """Return the record corresponding to the given ``xml_id``."""
-        return self['ir.model.data'].xmlid_to_object(xml_id, raise_if_not_found=raise_if_not_found)
+        res_model, res_id = self['ir.model.data']._xmlid_to_res_model_res_id(
+            xml_id, raise_if_not_found=raise_if_not_found
+        )
+
+        if res_model and res_id:
+            record = self[res_model].browse(res_id)
+            if record.exists():
+                return record
+            if raise_if_not_found:
+                raise ValueError('No record found for unique ID %s. It may have been deleted.' % (xml_id))
+        return None
 
     def is_superuser(self):
         """ Return whether the environment is in superuser mode. """
@@ -521,6 +605,7 @@ class Environment(Mapping):
     def user(self):
         """Return the current user (as an instance).
 
+        :returns: current user - sudoed
         :rtype: :class:`~odoo.addons.base.models.res_users`"""
         return self(su=True)['res.users'].browse(self.uid)
 
@@ -532,7 +617,7 @@ class Environment(Mapping):
         fallback on current user main company.
 
         :raise AccessError: invalid or unauthorized `allowed_company_ids` context key content.
-        :return: current company (default=`self.user.company_id`)
+        :return: current company (default=`self.user.company_id`), with the current environment
         :rtype: res.company
 
         .. warning::
@@ -552,7 +637,7 @@ class Environment(Mapping):
                 if any(cid not in user_company_ids for cid in company_ids):
                     raise AccessError(_("Access to unauthorized or invalid companies."))
             return self['res.company'].browse(company_ids[0])
-        return self.user.company_id
+        return self.user.company_id.with_env(self)
 
     @lazy_property
     def companies(self):
@@ -562,7 +647,7 @@ class Environment(Mapping):
         fallback on current user companies.
 
         :raise AccessError: invalid or unauthorized `allowed_company_ids` context key content.
-        :return: current companies (default=`self.user.company_ids`)
+        :return: current companies (default=`self.user.company_ids`), with the current environment
         :rtype: res.company
 
         .. warning::
@@ -592,7 +677,7 @@ class Environment(Mapping):
         #   - when printing a report for several records from several companies
         #   - when accessing to a record from the notification email template
         #   - when loading an binary image on a template
-        return self.user.company_ids
+        return self.user.company_ids.with_env(self)
 
     @property
     def lang(self):
@@ -606,35 +691,18 @@ class Environment(Mapping):
         """ Clear all record caches, and discard all fields to recompute.
             This may be useful when recovering from a failed ORM operation.
         """
-        self.cache.invalidate()
-        self.all.tocompute.clear()
-        self.all.towrite.clear()
+        lazy_property.reset_all(self)
+        self.transaction.clear()
 
-    @contextmanager
     def clear_upon_failure(self):
-        """ Context manager that clears the environments (caches and fields to
-            recompute) upon exception.
+        """ Context manager that rolls back the environments (caches and pending
+            computations and updates) upon exception.
         """
-        tocompute = {
-            field: set(ids)
-            for field, ids in self.all.tocompute.items()
-        }
-        towrite = {
-            model: {
-                record_id: dict(values)
-                for record_id, values in id_values.items()
-            }
-            for model, id_values in self.all.towrite.items()
-        }
-        try:
-            yield
-        except Exception:
-            self.clear()
-            self.all.tocompute.update(tocompute)
-            for model, id_values in towrite.items():
-                for record_id, values in id_values.items():
-                    self.all.towrite[model][record_id].update(values)
-            raise
+        warnings.warn(
+            "Since Odoo 15.0, use cr.savepoint() instead of env.clear_upon_failure().",
+            DeprecationWarning, stacklevel=2,
+        )
+        return self.cr.savepoint()
 
     def is_protected(self, field, record):
         """ Return whether `record` is protected against invalidation or
@@ -705,50 +773,117 @@ class Environment(Mapping):
         """ Delay recomputations (deprecated: this is not the default behavior). """
         yield
 
+    def cache_key(self, field):
+        """ Return the cache key of the given ``field``. """
+        try:
+            return self._cache_key[field]
 
-class Environments(object):
-    """ A common object for all environments in a request. """
-    def __init__(self):
-        self.envs = WeakSet()                   # weak set of environments
-        self.cache = Cache()                    # cache for all records
-        self.protected = StackMap()             # fields to protect {field: ids, ...}
-        self.tocompute = defaultdict(set)       # recomputations {field: ids}
-        # updates {model: {id: {field: value}}}
+        except KeyError:
+            def get(key, get_context=self.context.get):
+                if key == 'company':
+                    return self.company.id
+                elif key == 'uid':
+                    return (self.uid, self.su)
+                elif key == 'active_test':
+                    return get_context('active_test', field.context.get('active_test', True))
+                else:
+                    val = get_context(key)
+                    if type(val) is list:
+                        val = tuple(val)
+                    try:
+                        hash(val)
+                    except TypeError:
+                        raise TypeError(
+                            "Can only create cache keys from hashable values, "
+                            "got non-hashable value {!r} at context key {!r} "
+                            "(dependency of field {})".format(val, key, field)
+                        ) from None  # we don't need to chain the exception created 2 lines above
+                    else:
+                        return val
+
+            result = tuple(get(key) for key in self.registry.field_depends_context[field])
+            self._cache_key[field] = result
+            return result
+
+
+class Transaction:
+    """ A object holding ORM data structures for a transaction. """
+    def __init__(self, registry):
+        self.registry = registry
+        # weak set of environments
+        self.envs = WeakSet()
+        # cache for all records
+        self.cache = Cache()
+        # fields to protect {field: ids}
+        self.protected = StackMap()
+        # pending computations {field: ids}
+        self.tocompute = defaultdict(set)
+        # pending updates {model: {id: {field: value}}}
         self.towrite = defaultdict(lambda: defaultdict(dict))
 
-    def add(self, env):
-        """ Add the environment ``env``. """
-        self.envs.add(env)
+    def flush(self):
+        """ Flush pending computations and updates in the transaction. """
+        env_to_flush = None
+        for env in self.envs:
+            if isinstance(env.uid, int) or env.uid is None:
+                env_to_flush = env
+                if env.uid is not None:
+                    break
+        if env_to_flush is not None:
+            env_to_flush['base'].flush()
 
-    def __iter__(self):
-        """ Iterate over environments. """
-        return iter(self.envs)
+    def clear(self):
+        """ Clear the caches and pending computations and updates in the translations. """
+        self.cache.invalidate()
+        self.tocompute.clear()
+        self.towrite.clear()
+
+    def reset(self):
+        """ Reset the transaction.  This clears the transaction, and reassigns
+            the registry on all its environments.  This operation is strongly
+            recommended after reloading the registry.
+        """
+        self.registry = Registry(self.registry.db_name)
+        for env in self.envs:
+            env.registry = self.registry
+            lazy_property.reset_all(env)
+        self.clear()
 
 
 # sentinel value for optional parameters
 NOTHING = object()
+EMPTY_DICT = frozendict()
 
 
 class Cache(object):
     """ Implementation of the cache of records. """
     def __init__(self):
-        # {field: {record_id: value}}
+        # {field: {record_id: value}, field: {context_key: {record_id: value}}}
         self._data = defaultdict(dict)
 
+    def _get_field_cache(self, model, field):
+        """ Return the field cache of the given field, but not for modifying it. """
+        field_cache = self._data.get(field, EMPTY_DICT)
+        if field_cache and model.pool.field_depends_context[field]:
+            field_cache = field_cache.get(model.env.cache_key(field), EMPTY_DICT)
+        return field_cache
+
+    def _set_field_cache(self, model, field):
+        """ Return the field cache of the given field for modifying it. """
+        field_cache = self._data[field]
+        if model.pool.field_depends_context[field]:
+            field_cache = field_cache.setdefault(model.env.cache_key(field), {})
+        return field_cache
+
     def contains(self, record, field):
         """ Return whether ``record`` has a value for ``field``. """
-        if field.depends_context:
-            key = field.cache_key(record.env)
-            return key in self._data.get(field, {}).get(record.id, {})
-        return record.id in self._data.get(field, ())
+        return record.id in self._get_field_cache(record, field)
 
     def get(self, record, field, default=NOTHING):
         """ Return the value of ``field`` for ``record``. """
         try:
-            value = self._data[field][record._ids[0]]
-            if field.depends_context:
-                value = value[field.cache_key(record.env)]
-            return value
+            field_cache = self._get_field_cache(record, field)
+            return field_cache[record._ids[0]]
         except KeyError:
             if default is NOTHING:
                 raise CacheMiss(record, field)
@@ -756,53 +891,49 @@ class Cache(object):
 
     def set(self, record, field, value):
         """ Set the value of ``field`` for ``record``. """
-        if field.depends_context:
-            key = field.cache_key(record.env)
-            self._data[field].setdefault(record._ids[0], {})[key] = value
-        else:
-            self._data[field][record._ids[0]] = value
+        field_cache = self._set_field_cache(record, field)
+        field_cache[record._ids[0]] = value
 
     def update(self, records, field, values):
         """ Set the values of ``field`` for several ``records``. """
-        if field.depends_context:
-            key = field.cache_key(records.env)
-            field_cache = self._data[field]
-            for record_id, value in zip(records._ids, values):
-                field_cache.setdefault(record_id, {})[key] = value
-        else:
-            self._data[field].update(zip(records._ids, values))
+        field_cache = self._set_field_cache(records, field)
+        field_cache.update(zip(records._ids, values))
 
     def remove(self, record, field):
         """ Remove the value of ``field`` for ``record``. """
         try:
-            del self._data[field][record.id]
+            field_cache = self._set_field_cache(record, field)
+            del field_cache[record._ids[0]]
         except KeyError:
             pass
 
     def get_values(self, records, field):
         """ Return the cached values of ``field`` for ``records``. """
-        field_cache = self._data[field]
-        key = field.cache_key(records.env) if field.depends_context else None
+        field_cache = self._get_field_cache(records, field)
         for record_id in records._ids:
             try:
-                if key is not None:
-                    yield field_cache[record_id][key]
-                else:
-                    yield field_cache[record_id]
+                yield field_cache[record_id]
             except KeyError:
                 pass
 
+    def get_until_miss(self, records, field):
+        """ Return the cached values of ``field`` for ``records`` until a value is not found. """
+        field_cache = self._get_field_cache(records, field)
+        vals = []
+        for record_id in records._ids:
+            try:
+                vals.append(field_cache[record_id])
+            except KeyError:
+                break
+        return vals
+
     def get_records_different_from(self, records, field, value):
         """ Return the subset of ``records`` that has not ``value`` for ``field``. """
-        field_cache = self._data[field]
-        key = field.cache_key(records.env) if field.depends_context else None
+        field_cache = self._get_field_cache(records, field)
         ids = []
         for record_id in records._ids:
             try:
-                if key is not None:
-                    val = field_cache[record_id][key]
-                else:
-                    val = field_cache[record_id]
+                val = field_cache[record_id]
             except KeyError:
                 ids.append(record_id)
             else:
@@ -813,37 +944,20 @@ class Cache(object):
     def get_fields(self, record):
         """ Return the fields with a value for ``record``. """
         for name, field in record._fields.items():
-            if name == 'id':
-                continue
-            values = self._data.get(field, {})
-            if record.id not in values:
-                continue
-            if field.depends_context and field.cache_key(record.env) not in values[record.id]:
-                continue
-            yield field
+            if name != 'id' and record.id in self._get_field_cache(record, field):
+                yield field
 
     def get_records(self, model, field):
         """ Return the records of ``model`` that have a value for ``field``. """
-        field_cache = self._data[field]
-        if field.depends_context:
-            key = field.cache_key(model.env)
-            ids = [id_ for id_, value in field_cache.items() if key in value]
-        else:
-            ids = list(field_cache)
-        return model.browse(ids)
+        field_cache = self._get_field_cache(model, field)
+        return model.browse(field_cache)
 
     def get_missing_ids(self, records, field):
         """ Return the ids of ``records`` that have no value for ``field``. """
-        field_cache = self._data[field]
-        if field.depends_context:
-            key = field.cache_key(records.env)
-            for record_id in records._ids:
-                if key not in field_cache.get(record_id, ()):
-                    yield record_id
-        else:
-            for record_id in records._ids:
-                if record_id not in field_cache:
-                    yield record_id
+        field_cache = self._get_field_cache(records, field)
+        for record_id in records._ids:
+            if record_id not in field_cache:
+                yield record_id
 
     def invalidate(self, spec=None):
         """ Invalidate the cache, partially or totally depending on ``spec``. """
@@ -853,52 +967,53 @@ class Cache(object):
             for field, ids in spec:
                 if ids is None:
                     self._data.pop(field, None)
-                else:
-                    field_cache = self._data.get(field)
-                    if field_cache:
-                        for id in ids:
-                            field_cache.pop(id, None)
+                    continue
+                cache = self._data.get(field)
+                if not cache:
+                    continue
+                caches = cache.values() if isinstance(next(iter(cache)), tuple) else [cache]
+                for field_cache in caches:
+                    for id_ in ids:
+                        field_cache.pop(id_, None)
 
     def check(self, env):
         """ Check the consistency of the cache for the given environment. """
         # flush fields to be recomputed before evaluating the cache
         env['res.partner'].recompute()
 
-        # make a full copy of the cache, and invalidate it
-        dump = defaultdict(dict)
-        key_cache = self._data
-        for field, field_cache in key_cache.items():
-            for record_id, value in field_cache.items():
-                if record_id:
-                    dump[field][record_id] = value
-
+        # make a copy of the cache, and invalidate it
+        dump = dict(self._data)
         self.invalidate()
 
+        depends_context = env.registry.field_depends_context
+
         # re-fetch the records, and compare with their former cache
         invalids = []
-        for field, field_dump in dump.items():
+
+        def check(model, field, field_dump):
             records = env[field.model_name].browse(field_dump)
             for record in records:
+                if not record.id:
+                    continue
                 try:
                     cached = field_dump[record.id]
-                    if field.depends_context:
-                        for context_keys, value in cached.items():
-                            context = dict(zip(field.depends_context, context_keys))
-                            value = field.convert_to_record(value, record)
-                            fetched = record.with_context(context)[field.name]
-                            if fetched != value:
-                                info = {'cached': value, 'fetched': fetched}
-                                invalids.append((record, field, info))
-                    else:
-                        cached = field_dump[record.id]
-                        fetched = record[field.name]
-                        value = field.convert_to_record(cached, record)
-                        if fetched != value:
-                            info = {'cached': value, 'fetched': fetched}
-                            invalids.append((record, field, info))
+                    value = field.convert_to_record(cached, record)
+                    fetched = record[field.name]
+                    if fetched != value:
+                        info = {'cached': value, 'fetched': fetched}
+                        invalids.append((record, field, info))
                 except (AccessError, MissingError):
                     pass
 
+        for field, field_dump in dump.items():
+            model = env[field.model_name]
+            if depends_context[field]:
+                for context_keys, field_cache in field_dump.items():
+                    context = dict(zip(depends_context[field], context_keys))
+                    check(model.with_context(context), field, field_cache)
+            else:
+                check(model, field, field_dump)
+
         if invalids:
             raise UserError('Invalid cache for fields\n' + pformat(invalids))
 
