@@ -4,20 +4,24 @@
 """ Models registries.
 
 """
-from collections import Mapping, defaultdict, deque
+from collections import defaultdict, deque
+from collections.abc import Mapping
 from contextlib import closing, contextmanager
 from functools import partial
 from operator import attrgetter
-from weakref import WeakValueDictionary
 import logging
 import os
 import threading
+import time
+
+import psycopg2
 
 import odoo
 from .. import SUPERUSER_ID
 from odoo.sql_db import TestCursor
-from odoo.tools import (assertion_report, config, existing_tables, ignore,
-                        lazy_classproperty, lazy_property, OrderedSet)
+from odoo.tools import (config, existing_tables, ignore,
+                        lazy_classproperty, lazy_property, sql,
+                        Collector, OrderedSet)
 from odoo.tools.lru import LRU
 
 _logger = logging.getLogger(__name__)
@@ -34,9 +38,6 @@ class Registry(Mapping):
     _lock = threading.RLock()
     _saved_lock = None
 
-    # a cache for model classes, indexed by their base classes
-    model_cache = WeakValueDictionary()
-
     @lazy_classproperty
     def registries(cls):
         """ A mapping from database names to registries. """
@@ -68,50 +69,52 @@ class Registry(Mapping):
     @classmethod
     def new(cls, db_name, force_demo=False, status=None, update_module=False):
         """ Create and return a new registry for the given database name. """
+        t0 = time.time()
         with cls._lock:
-            with odoo.api.Environment.manage():
-                registry = object.__new__(cls)
-                registry.init(db_name)
+            registry = object.__new__(cls)
+            registry.init(db_name)
 
-                # Initializing a registry will call general code which will in
-                # turn call Registry() to obtain the registry being initialized.
-                # Make it available in the registries dictionary then remove it
-                # if an exception is raised.
-                cls.delete(db_name)
-                cls.registries[db_name] = registry
+            # Initializing a registry will call general code which will in
+            # turn call Registry() to obtain the registry being initialized.
+            # Make it available in the registries dictionary then remove it
+            # if an exception is raised.
+            cls.delete(db_name)
+            cls.registries[db_name] = registry  # pylint: disable=unsupported-assignment-operation
+            try:
+                registry.setup_signaling()
+                # This should be a method on Registry
                 try:
-                    registry.setup_signaling()
-                    # This should be a method on Registry
-                    try:
-                        odoo.modules.load_modules(registry._db, force_demo, status, update_module)
-                    except Exception:
-                        odoo.modules.reset_modules_state(db_name)
-                        raise
+                    odoo.modules.load_modules(registry, force_demo, status, update_module)
                 except Exception:
-                    _logger.exception('Failed to load registry')
-                    del cls.registries[db_name]
+                    odoo.modules.reset_modules_state(db_name)
                     raise
+            except Exception:
+                _logger.error('Failed to load registry')
+                del cls.registries[db_name]     # pylint: disable=unsupported-delete-operation
+                raise
 
-                # load_modules() above can replace the registry by calling
-                # indirectly new() again (when modules have to be uninstalled).
-                # Yeah, crazy.
-                registry = cls.registries[db_name]
+            # load_modules() above can replace the registry by calling
+            # indirectly new() again (when modules have to be uninstalled).
+            # Yeah, crazy.
+            registry = cls.registries[db_name]  # pylint: disable=unsubscriptable-object
 
             registry._init = False
             registry.ready = True
             registry.registry_invalidated = bool(update_module)
             registry.new = registry.init = registry.registries = None
 
+        _logger.info("Registry loaded in %.3fs", time.time() - t0)
         return registry
 
     def init(self, db_name):
         self.models = {}    # model name/model instance mapping
         self._sql_constraints = set()
         self._init = True
-        self._assertion_report = assertion_report.assertion_report()
+        self._assertion_report = odoo.tests.runner.OdooTestResult()
         self._fields_by_model = None
-        self._post_init_queue = deque()
+        self._ordinary_tables = None
         self._constraint_queue = deque()
+        self.__cache = LRU(8192)
 
         # modules fully loaded (maintained during init phase by `loading` module)
         self._init_modules = set()
@@ -129,6 +132,11 @@ class Registry(Mapping):
         self.loaded = False             # whether all modules are loaded
         self.ready = False              # whether everything is set up
 
+        # field dependencies
+        self.field_depends = Collector()
+        self.field_depends_context = Collector()
+        self.field_inverses = Collector()
+
         # Inter-process signaling:
         # The `base_registry_signaling` sequence indicates the whole registry
         # must be reloaded.
@@ -141,24 +149,21 @@ class Registry(Mapping):
         self._invalidation_flags = threading.local()
 
         with closing(self.cursor()) as cr:
-            has_unaccent = odoo.modules.db.has_unaccent(cr)
-            if odoo.tools.config['unaccent'] and not has_unaccent:
-                _logger.warning("The option --unaccent was given but no unaccent() function was found in database.")
-            self.has_unaccent = odoo.tools.config['unaccent'] and has_unaccent
+            self.has_unaccent = odoo.modules.db.has_unaccent(cr)
+            self.has_trigram = odoo.modules.db.has_trigram(cr)
 
     @classmethod
     def delete(cls, db_name):
         """ Delete the registry linked to a given database. """
         with cls._lock:
             if db_name in cls.registries:
-                cls.registries.pop(db_name)
+                del cls.registries[db_name]
 
     @classmethod
     def delete_all(cls):
         """ Delete all the registries. """
         with cls._lock:
-            for db_name in list(cls.registries.keys()):
-                cls.delete(db_name)
+            cls.registries.clear()
 
     #
     # Mapping abstract methods implementation
@@ -184,6 +189,13 @@ class Registry(Mapping):
         """ Add or replace a model in the registry."""
         self.models[model_name] = model
 
+    def __delitem__(self, model_name):
+        """ Remove a (custom) model from the registry. """
+        del self.models[model_name]
+        # the custom model can inherit from mixins ('mail.thread', ...)
+        for Model in self.models.values():
+            Model._inherit_children.discard(model_name)
+
     def descendants(self, model_names, *kinds):
         """ Return the models corresponding to ``model_names`` and all those
         that inherit/inherits from them.
@@ -206,12 +218,15 @@ class Registry(Mapping):
 
         At the Python level, the modules are already loaded, but not yet on a
         per-registry level. This method populates a registry with the given
-        modules, i.e. it instanciates all the classes of a the given module
+        modules, i.e. it instantiates all the classes of a the given module
         and registers them in the registry.
 
         """
         from .. import models
 
+        # clear cache to ensure consistency, but do not signal it
+        self.__cache.clear()
+
         lazy_property.reset_all(self)
 
         # Instantiate registered classes (via the MetaModel automatic discovery
@@ -236,7 +251,11 @@ class Registry(Mapping):
             for model in env.values():
                 model._unregister_hook()
 
+        # clear cache to ensure consistency, but do not signal it
+        self.__cache.clear()
+
         lazy_property.reset_all(self)
+        self.registry_invalidated = True
 
         if env.all.tocompute:
             _logger.error(
@@ -244,6 +263,11 @@ class Registry(Mapping):
                 env.all.tocompute, stack_info=True,
             )
 
+        # we must setup ir.model before adding manual fields because _add_manual_models may
+        # depend on behavior that is implemented through overrides, such as is_mail_thread which
+        # is implemented through an override to env['ir.model']._instanciate
+        env['ir.model']._prepare_setup()
+
         # add manual models
         if self._init_modules:
             env['ir.model']._add_manual_models()
@@ -253,24 +277,64 @@ class Registry(Mapping):
         for model in models:
             model._prepare_setup()
 
-        # do the actual setup from a clean state
-        self._m2m = defaultdict(list)
+        self.field_depends.clear()
+        self.field_depends_context.clear()
+        self.field_inverses.clear()
+
+        # do the actual setup
         for model in models:
             model._setup_base()
 
+        self._m2m = defaultdict(list)
         for model in models:
             model._setup_fields()
+        del self._m2m
 
+        for model in models:
+            model._setup_complete()
+
+        # determine field_depends and field_depends_context
+        for model in models:
+            for field in model._fields.values():
+                depends, depends_context = field.get_depends(model)
+                self.field_depends[field] = tuple(depends)
+                self.field_depends_context[field] = tuple(depends_context)
+
+        # Reinstall registry hooks. Because of the condition, this only happens
+        # on a fully loaded registry, and not on a registry being loaded.
+        if self.ready:
+            for model in env.values():
+                model._register_hook()
+            env['base'].flush()
+
+    @lazy_property
+    def field_computed(self):
+        """ Return a dict mapping each field to the fields computed by the same method. """
+        computed = {}
+        for model_name, Model in self.models.items():
+            groups = defaultdict(list)
+            for field in Model._fields.values():
+                if field.compute:
+                    computed[field] = group = groups[field.compute]
+                    group.append(field)
+            for fields in groups.values():
+                if len({field.compute_sudo for field in fields}) > 1:
+                    _logger.warning("%s: inconsistent 'compute_sudo' for computed fields: %s",
+                                    model_name, ", ".join(field.name for field in fields))
+        return computed
+
+    @lazy_property
+    def field_triggers(self):
         # determine field dependencies
         dependencies = {}
-        for model in models:
-            if model._abstract:
+        for Model in self.models.values():
+            if Model._abstract:
                 continue
-            for field in model._fields.values():
+            for field in Model._fields.values():
                 # dependencies of custom fields may not exist; ignore that case
                 exceptions = (Exception,) if field.base_field.manual else ()
                 with ignore(*exceptions):
-                    dependencies[field] = set(field.resolve_depends(model))
+                    dependencies[field] = OrderedSet(field.resolve_depends(self))
 
         # determine transitive dependencies
         def transitive_dependencies(field, seen=[]):
@@ -297,21 +361,9 @@ class Registry(Mapping):
                     tree = triggers
                     for label in reversed(path):
                         tree = tree.setdefault(label, {})
-                    tree.setdefault(None, set()).add(field)
+                    tree.setdefault(None, OrderedSet()).add(field)
 
-        self.field_triggers = triggers
-
-        for model in models:
-            model._setup_complete()
-
-        self.registry_invalidated = True
-
-        # Reinstall registry hooks. Because of the condition, this only happens
-        # on a fully loaded registry, and not on a registry being loaded.
-        if self.ready:
-            for model in env.values():
-                model._register_hook()
-            env['base'].flush()
+        return triggers
 
     def post_init(self, func, *args, **kwargs):
         """ Register a function to call at the end of :meth:`~.init_models`. """
@@ -355,6 +407,9 @@ class Registry(Mapping):
              - ``module``: the name of the module being installed/updated, if any;
              - ``update_custom_fields``: whether custom fields should be updated.
         """
+        if not model_names:
+            return
+
         if 'module' in context:
             _logger.info('module %s: creating or updating database tables', context['module'])
         elif context.get('models_to_check', False):
@@ -363,29 +418,123 @@ class Registry(Mapping):
         env = odoo.api.Environment(cr, SUPERUSER_ID, context)
         models = [env[model_name] for model_name in model_names]
 
-        # make sure the queue does not contain some leftover from a former call
-        self._post_init_queue.clear()
-        self._is_install = install
+        try:
+            self._post_init_queue = deque()
+            self._foreign_keys = {}
+            self._is_install = install
 
-        for model in models:
-            model._auto_init()
-            model.init()
+            for model in models:
+                model._auto_init()
+                model.init()
 
-        while self._post_init_queue:
-            func = self._post_init_queue.popleft()
-            func()
+            env['ir.model']._reflect_models(model_names)
+            env['ir.model.fields']._reflect_fields(model_names)
+            env['ir.model.fields.selection']._reflect_selections(model_names)
+            env['ir.model.constraint']._reflect_constraints(model_names)
 
-        env['base'].flush()
+            self._ordinary_tables = None
 
-        # make sure all tables are present
-        self.check_tables_exist(cr)
+            while self._post_init_queue:
+                func = self._post_init_queue.popleft()
+                func()
+
+            self.check_indexes(cr, model_names)
+            self.check_foreign_keys(cr)
+
+            env['base'].flush()
+
+            # make sure all tables are present
+            self.check_tables_exist(cr)
+
+        finally:
+            del self._post_init_queue
+            del self._foreign_keys
+            del self._is_install
+
+    def check_indexes(self, cr, model_names):
+        """ Create or drop column indexes for the given models. """
+        expected = [
+            ("%s_%s_index" % (Model._table, field.name), Model._table, field.name, field.index)
+            for model_name in model_names
+            for Model in [self.models[model_name]]
+            if Model._auto and not Model._abstract
+            for field in Model._fields.values()
+            if field.column_type and field.store
+        ]
+        if not expected:
+            return
+
+        cr.execute("SELECT indexname FROM pg_indexes WHERE indexname IN %s",
+                   [tuple(row[0] for row in expected)])
+        existing = {row[0] for row in cr.fetchall()}
+
+        for indexname, tablename, columnname, index in expected:
+            if index and indexname not in existing:
+                try:
+                    with cr.savepoint(flush=False):
+                        sql.create_index(cr, indexname, tablename, ['"%s"' % columnname])
+                except psycopg2.OperationalError:
+                    _schema.error("Unable to add index for %s", self)
+            elif not index and indexname in existing:
+                _schema.info("Keep unexpected index %s on table %s", indexname, tablename)
+
+    def add_foreign_key(self, table1, column1, table2, column2, ondelete,
+                        model, module, force=True):
+        """ Specify an expected foreign key. """
+        key = (table1, column1)
+        val = (table2, column2, ondelete, model, module)
+        if force:
+            self._foreign_keys[key] = val
+        else:
+            self._foreign_keys.setdefault(key, val)
+
+    def check_foreign_keys(self, cr):
+        """ Create or update the expected foreign keys. """
+        if not self._foreign_keys:
+            return
+
+        # determine existing foreign keys on the tables
+        query = """
+            SELECT fk.conname, c1.relname, a1.attname, c2.relname, a2.attname, fk.confdeltype
+            FROM pg_constraint AS fk
+            JOIN pg_class AS c1 ON fk.conrelid = c1.oid
+            JOIN pg_class AS c2 ON fk.confrelid = c2.oid
+            JOIN pg_attribute AS a1 ON a1.attrelid = c1.oid AND fk.conkey[1] = a1.attnum
+            JOIN pg_attribute AS a2 ON a2.attrelid = c2.oid AND fk.confkey[1] = a2.attnum
+            WHERE fk.contype = 'f' AND c1.relname IN %s
+        """
+        cr.execute(query, [tuple({table for table, column in self._foreign_keys})])
+        existing = {
+            (table1, column1): (name, table2, column2, deltype)
+            for name, table1, column1, table2, column2, deltype in cr.fetchall()
+        }
+
+        # create or update foreign keys
+        for key, val in self._foreign_keys.items():
+            table1, column1 = key
+            table2, column2, ondelete, model, module = val
+            deltype = sql._CONFDELTYPES[ondelete.upper()]
+            spec = existing.get(key)
+            if spec is None:
+                sql.add_foreign_key(cr, table1, column1, table2, column2, ondelete)
+                conname = sql.get_foreign_keys(cr, table1, column1, table2, column2, ondelete)[0]
+                model.env['ir.model.constraint']._reflect_constraint(model, conname, 'f', None, module)
+            elif (spec[1], spec[2], spec[3]) != (table2, column2, deltype):
+                sql.drop_constraint(cr, table1, spec[0])
+                sql.add_foreign_key(cr, table1, column1, table2, column2, ondelete)
+                conname = sql.get_foreign_keys(cr, table1, column1, table2, column2, ondelete)[0]
+                model.env['ir.model.constraint']._reflect_constraint(model, conname, 'f', None, module)
 
     def check_tables_exist(self, cr):
         """
         Verify that all tables are present and try to initialize those that are missing.
         """
         env = odoo.api.Environment(cr, SUPERUSER_ID, {})
-        table2model = {model._table: name for name, model in env.items() if not model._abstract}
+        table2model = {
+            model._table: name
+            for name, model in env.items()
+            if not model._abstract and model.__class__._table_query is None
+        }
         missing_tables = set(table2model).difference(existing_tables(cr, table2model))
 
         if missing_tables:
@@ -401,15 +550,9 @@ class Registry(Mapping):
             for table in missing_tables:
                 _logger.error("Model %s has no table.", table2model[table])
 
-    @lazy_property
-    def cache(self):
-        """ A cache for model methods. """
-        # this lazy_property is automatically reset by lazy_property.reset_all()
-        return LRU(8192)
-
     def _clear_cache(self):
         """ Clear the cache and mark it as invalidated. """
-        self.cache.clear()
+        self.__cache.clear()
         self.cache_invalidated = True
 
     def clear_caches(self):
@@ -419,6 +562,24 @@ class Registry(Mapping):
         for model in self.models.values():
             model.clear_caches()
 
+    def is_an_ordinary_table(self, model):
+        """ Return whether the given model has an ordinary table. """
+        if self._ordinary_tables is None:
+            cr = model.env.cr
+            query = """
+                SELECT c.relname
+                  FROM pg_class c
+                  JOIN pg_namespace n ON (n.oid = c.relnamespace)
+                 WHERE c.relname IN %s
+                   AND c.relkind = 'r'
+                   AND n.nspname = 'public'
+            """
+            tables = tuple(m._table for m in self.models.values())
+            cr.execute(query, [tables])
+            self._ordinary_tables = {row[0] for row in cr.fetchall()}
+
+        return model._table in self._ordinary_tables
+
     @property
     def registry_invalidated(self):
         """ Determine whether the current thread has modified the registry. """
@@ -482,9 +643,12 @@ class Registry(Mapping):
             # Check if the model caches must be invalidated.
             elif self.cache_sequence != c:
                 _logger.info("Invalidating all model caches after database signaling.")
-                # Bypass self.clear_caches() to avoid invalidation loops in multi-threaded
-                # configs due to the `cache_invalidated` flag being set, causing more signaling.
-                self.cache.clear()
+                self.clear_caches()
+
+            # prevent re-signaling the clear_caches() above, or any residual one that
+            # would be inherited from the master process (first request in pre-fork mode)
+            self.cache_invalidated = False
+
             self.registry_sequence = r
             self.cache_sequence = c
 
@@ -516,7 +680,7 @@ class Registry(Mapping):
                 self.setup_models(cr)
                 self.registry_invalidated = False
         if self.cache_invalidated:
-            self.cache.clear()
+            self.__cache.clear()
             self.cache_invalidated = False
 
     @contextmanager
@@ -556,8 +720,7 @@ class Registry(Mapping):
             as a context manager to commit/rollback and close automatically.
         """
         if self.test_cr is not None:
-            # When in test mode, we use a proxy object that uses 'self.test_cr'
-            # underneath.
+            # in test mode we use a proxy object that uses 'self.test_cr' underneath
             return TestCursor(self.test_cr, self.test_lock)
         return self._db.cursor()
 
