@@ -10,6 +10,7 @@ import traceback
 from xml.etree import ElementTree as ET
 import zipfile
 
+from psycopg2 import sql
 from pytz import country_timezones
 from functools import wraps
 from contextlib import closing
@@ -61,7 +62,7 @@ def _initialize_db(id, db_name, demo, lang, user_password, login='admin', countr
 
         registry = odoo.modules.registry.Registry.new(db_name, demo, None, update_module=True)
 
-        with closing(db.cursor()) as cr:
+        with closing(registry.cursor()) as cr:
             env = odoo.api.Environment(cr, SUPERUSER_ID, {})
 
             if lang:
@@ -102,14 +103,25 @@ def _create_empty_database(name):
         if cr.fetchall():
             raise DatabaseExists("database %r already exists!" % (name,))
         else:
-            cr.autocommit(True)     # avoid transaction block
+            # database-altering operations cannot be executed inside a transaction
+            cr.rollback()
+            cr._cnx.autocommit = True
 
             # 'C' collate is only safe with template0, but provides more useful indexes
-            collate = "LC_COLLATE 'C'" if chosen_template == 'template0' else ""
+            collate = sql.SQL("LC_COLLATE 'C'" if chosen_template == 'template0' else "")
             cr.execute(
-                """CREATE DATABASE "%s" ENCODING 'unicode' %s TEMPLATE "%s" """ %
-                (name, collate, chosen_template)
-            )
+                sql.SQL("CREATE DATABASE {} ENCODING 'unicode' {} TEMPLATE {}").format(
+                sql.Identifier(name), collate, sql.Identifier(chosen_template)
+            ))
+
+    if odoo.tools.config['unaccent']:
+        try:
+            db = odoo.sql_db.db_connect(name)
+            with closing(db.cursor()) as cr:
+                cr.execute("CREATE EXTENSION IF NOT EXISTS unaccent")
+                cr.commit()
+        except psycopg2.Error:
+            pass
 
 @check_db_management_enabled
 def exp_create_database(db_name, demo, lang, user_password='admin', login='admin', country_code=None, phone=None):
@@ -125,9 +137,13 @@ def exp_duplicate_database(db_original_name, db_name):
     odoo.sql_db.close_db(db_original_name)
     db = odoo.sql_db.db_connect('postgres')
     with closing(db.cursor()) as cr:
-        cr.autocommit(True)     # avoid transaction block
+        # database-altering operations cannot be executed inside a transaction
+        cr._cnx.autocommit = True
         _drop_conn(cr, db_original_name)
-        cr.execute("""CREATE DATABASE "%s" ENCODING 'unicode' TEMPLATE "%s" """ % (db_name, db_original_name))
+        cr.execute(sql.SQL("CREATE DATABASE {} ENCODING 'unicode' TEMPLATE {}").format(
+            sql.Identifier(db_name),
+            sql.Identifier(db_original_name)
+        ))
 
     registry = odoo.modules.registry.Registry.new(db_name)
     with registry.cursor() as cr:
@@ -166,11 +182,12 @@ def exp_drop(db_name):
 
     db = odoo.sql_db.db_connect('postgres')
     with closing(db.cursor()) as cr:
-        cr.autocommit(True) # avoid transaction block
+        # database-altering operations cannot be executed inside a transaction
+        cr._cnx.autocommit = True
         _drop_conn(cr, db_name)
 
         try:
-            cr.execute('DROP DATABASE "%s"' % db_name)
+            cr.execute(sql.SQL('DROP DATABASE {}').format(sql.Identifier(db_name)))
         except Exception as e:
             _logger.info('DROP DB: %s failed:\n%s', db_name, e)
             raise Exception("Couldn't drop database %s: %s" % (db_name, e))
@@ -216,7 +233,7 @@ def dump_db(db_name, stream, backup_format='zip'):
     cmd.append(db_name)
 
     if backup_format == 'zip':
-        with odoo.tools.osutil.tempdir() as dump_dir:
+        with tempfile.TemporaryDirectory() as dump_dir:
             filestore = odoo.tools.config.filestore(db_name)
             if os.path.exists(filestore):
                 shutil.copytree(filestore, os.path.join(dump_dir, 'filestore'))
@@ -266,7 +283,7 @@ def restore_db(db, dump_file, copy=False):
     _create_empty_database(db)
 
     filestore_path = None
-    with odoo.tools.osutil.tempdir() as dump_dir:
+    with tempfile.TemporaryDirectory() as dump_dir:
         if zipfile.is_zipfile(dump_file):
             # v8 format
             with zipfile.ZipFile(dump_file, 'r') as z:
@@ -302,13 +319,6 @@ def restore_db(db, dump_file, copy=False):
                 filestore_dest = env['ir.attachment']._filestore()
                 shutil.move(filestore_path, filestore_dest)
 
-            if odoo.tools.config['unaccent']:
-                try:
-                    with cr.savepoint(flush=False):
-                        cr.execute("CREATE EXTENSION unaccent")
-                except psycopg2.Error:
-                    pass
-
     _logger.info('RESTORE DB: %s', db)
 
 @check_db_management_enabled
@@ -318,10 +328,11 @@ def exp_rename(old_name, new_name):
 
     db = odoo.sql_db.db_connect('postgres')
     with closing(db.cursor()) as cr:
-        cr.autocommit(True)     # avoid transaction block
+        # database-altering operations cannot be executed inside a transaction
+        cr._cnx.autocommit = True
         _drop_conn(cr, old_name)
         try:
-            cr.execute('ALTER DATABASE "%s" RENAME TO "%s"' % (old_name, new_name))
+            cr.execute(sql.SQL('ALTER DATABASE {} RENAME TO {}').format(sql.Identifier(old_name), sql.Identifier(new_name)))
             _logger.info('RENAME DB: %s -> %s', old_name, new_name)
         except Exception as e:
             _logger.info('RENAME DB: %s -> %s failed:\n%s', old_name, new_name, e)
