@@ -1,13 +1,19 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+
+__all__ = [
+    'convert_file', 'convert_sql_import',
+    'convert_csv_import', 'convert_xml_import'
+]
+
 import base64
 import io
 import logging
 import os.path
+import pprint
 import re
 import subprocess
-import sys
-import time
+import warnings
 
 from datetime import datetime, timedelta
 from dateutil.relativedelta import relativedelta
@@ -20,27 +26,20 @@ except ImportError:
     jingtrang = None
 
 import odoo
-from . import assertion_report, pycompat
+from . import pycompat
 from .config import config
 from .misc import file_open, unquote, ustr, SKIPPED_ELEMENT_TYPES
 from .translate import _
 from odoo import SUPERUSER_ID, api
+from odoo.exceptions import ValidationError
 
 _logger = logging.getLogger(__name__)
 
-from .safe_eval import safe_eval as s_eval
+from .safe_eval import safe_eval as s_eval, pytz, time
 safe_eval = lambda expr, ctx={}: s_eval(expr, ctx, nocopy=True)
 
 class ParseError(Exception):
-    def __init__(self, msg, text, filename, lineno):
-        self.msg = msg
-        self.text = text
-        self.filename = filename
-        self.lineno = lineno
-
-    def __str__(self):
-        return '"%s" while parsing %s:%s, near\n%s' \
-            % (self.msg, self.filename, self.lineno, self.text)
+    ...
 
 class RecordDictWrapper(dict):
     """
@@ -56,6 +55,7 @@ class RecordDictWrapper(dict):
 
 def _get_idref(self, env, model_str, idref):
     idref2 = dict(idref,
+                  Command=odoo.fields.Command,
                   time=time,
                   DateTime=datetime,
                   datetime=datetime,
@@ -296,6 +296,7 @@ form: module.record_id""" % (xml_id,)
 
         xml_id = rec.get('id','')
         self._test_xml_id(xml_id)
+        warnings.warn(f"The <report> tag is deprecated, use a <record> tag for {xml_id!r}.", DeprecationWarning)
 
         if rec.get('groups'):
             g_names = rec.get('groups','').split(',')
@@ -303,10 +304,10 @@ form: module.record_id""" % (xml_id,)
             for group in g_names:
                 if group.startswith('-'):
                     group_id = self.id_get(group[1:])
-                    groups_value.append((3, group_id))
+                    groups_value.append(odoo.Command.unlink(group_id))
                 else:
                     group_id = self.id_get(group)
-                    groups_value.append((4, group_id))
+                    groups_value.append(odoo.Command.link(group_id))
             res['groups_id'] = groups_value
         if rec.get('paperformat'):
             pf_name = rec.get('paperformat')
@@ -335,6 +336,7 @@ form: module.record_id""" % (xml_id,)
         name = rec.get('name')
         xml_id = rec.get('id','')
         self._test_xml_id(xml_id)
+        warnings.warn(f"The <act_window> tag is deprecated, use a <record> for {xml_id!r}.", DeprecationWarning)
         view_id = False
         if rec.get('view_id'):
             view_id = self.id_get(rec.get('view_id'))
@@ -406,10 +408,10 @@ form: module.record_id""" % (xml_id,)
             for group in g_names:
                 if group.startswith('-'):
                     group_id = self.id_get(group[1:])
-                    groups_value.append((3, group_id))
+                    groups_value.append(odoo.Command.unlink(group_id))
                 else:
                     group_id = self.id_get(group)
-                    groups_value.append((4, group_id))
+                    groups_value.append(odoo.Command.link(group_id))
             res['groups_id'] = groups_value
 
         if rec.get('target'):
@@ -424,7 +426,7 @@ form: module.record_id""" % (xml_id,)
         data = dict(xml_id=xid, values=res, noupdate=self.noupdate)
         self.env['ir.actions.act_window']._load_records([data], self.mode == 'update')
 
-    def _tag_menuitem(self, rec):
+    def _tag_menuitem(self, rec, parent=None):
         rec_id = rec.attrib["id"]
         self._test_xml_id(rec_id)
 
@@ -438,11 +440,12 @@ form: module.record_id""" % (xml_id,)
         if rec.get('sequence'):
             values['sequence'] = int(rec.get('sequence'))
 
-        if rec.get('parent'):
+        if parent is not None:
+            values['parent_id'] = parent
+        elif rec.get('parent'):
             values['parent_id'] = self.id_get(rec.attrib['parent'])
-        else:
-            if rec.get('web_icon'):
-                values['web_icon'] = rec.attrib['web_icon']
+        elif rec.get('web_icon'):
+            values['web_icon'] = rec.attrib['web_icon']
 
 
         if rec.get('name'):
@@ -467,10 +470,10 @@ form: module.record_id""" % (xml_id,)
         for group in rec.get('groups', '').split(','):
             if group.startswith('-'):
                 group_id = self.id_get(group[1:])
-                groups.append((3, group_id))
+                groups.append(odoo.Command.unlink(group_id))
             elif group:
                 group_id = self.id_get(group)
-                groups.append((4, group_id))
+                groups.append(odoo.Command.link(group_id))
         if groups:
             values['groups_id'] = groups
 
@@ -480,7 +483,9 @@ form: module.record_id""" % (xml_id,)
             'values': values,
             'noupdate': self.noupdate,
         }
-        self.env['ir.ui.menu']._load_records([data], self.mode == 'update')
+        menu = self.env['ir.ui.menu']._load_records([data], self.mode == 'update')
+        for child in rec.iterchildren('menuitem'):
+            self._tag_menuitem(child, parent=menu.id)
 
     def _tag_record(self, rec):
         rec_model = rec.get("model")
@@ -492,7 +497,8 @@ form: module.record_id""" % (xml_id,)
         if self.xml_filename and rec_id:
             model = model.with_context(
                 install_module=self.module,
-                install_filename=self.xml_filename
+                install_filename=self.xml_filename,
+                install_xmlid=rec_id,
             )
 
         self._test_xml_id(rec_id)
@@ -548,7 +554,7 @@ form: module.record_id""" % (xml_id,)
                 _fields = env[rec_model]._fields
                 # if the current field is many2many
                 if (f_name in _fields) and _fields[f_name].type == 'many2many':
-                    f_val = [(6, 0, [x[f_use] for x in s])]
+                    f_val = [odoo.Command.set([x[f_use] for x in s])]
                 elif len(s):
                     # otherwise (we are probably in a many2one field),
                     # take the first element of the search
@@ -634,7 +640,7 @@ form: module.record_id""" % (xml_id,)
         groups = el.attrib.pop('groups', None)
         if groups:
             grp_lst = [("ref('%s')" % x) for x in groups.split(',')]
-            record.append(Field(name="groups_id", eval="[(6, 0, ["+', '.join(grp_lst)+"])]"))
+            record.append(Field(name="groups_id", eval="[Command.set(["+', '.join(grp_lst)+"])]"))
         if el.get('primary') == 'True':
             # Pseudo clone mode, we'll set the t-name to the full canonical xmlid
             el.append(
@@ -660,7 +666,7 @@ form: module.record_id""" % (xml_id,)
     def model_id_get(self, id_str, raise_if_not_found=True):
         if '.' not in id_str:
             id_str = '%s.%s' % (self.module, id_str)
-        return self.env['ir.model.data'].xmlid_to_res_model_res_id(id_str, raise_if_not_found=raise_if_not_found)
+        return self.env['ir.model.data']._xmlid_to_res_model_res_id(id_str, raise_if_not_found=raise_if_not_found)
 
     def _tag_root(self, el):
         for rec in el:
@@ -672,6 +678,23 @@ form: module.record_id""" % (xml_id,)
             self._noupdate.append(nodeattr2bool(el, 'noupdate', self.noupdate))
             try:
                 f(rec)
+            except ParseError:
+                raise
+            except ValidationError as err:
+                msg = "while parsing {file}:{viewline}\n{err}\n\nView error context:\n{context}\n".format(
+                    file=rec.getroottree().docinfo.URL,
+                    viewline=rec.sourceline,
+                    context=pprint.pformat(getattr(err, 'context', None) or '-no context-'),
+                    err=err.args[0],
+                )
+                _logger.debug(msg, exc_info=True)
+                raise ParseError(msg) from None  # Restart with "--log-handler odoo.tools.convert:DEBUG" for complete traceback
+            except Exception as e:
+                raise ParseError('while parsing %s:%s, somewhere inside\n%s' % (
+                    rec.getroottree().docinfo.URL,
+                    rec.sourceline,
+                    etree.tostring(rec, encoding='unicode').rstrip()
+                )) from e
             finally:
                 self._noupdate.pop()
                 self.envs.pop()
@@ -684,14 +707,11 @@ form: module.record_id""" % (xml_id,)
     def noupdate(self):
         return self._noupdate[-1]
 
-    def __init__(self, cr, module, idref, mode, report=None, noupdate=False, xml_filename=None):
+    def __init__(self, cr, module, idref, mode, noupdate=False, xml_filename=None):
         self.mode = mode
         self.module = module
         self.envs = [odoo.api.Environment(cr, SUPERUSER_ID, {})]
         self.idref = {} if idref is None else idref
-        if report is None:
-            report = assertion_report.assertion_report()
-        self.assertion_report = report
         self._noupdate = [noupdate]
         self.xml_filename = xml_filename
         self._tags = {
@@ -708,21 +728,10 @@ form: module.record_id""" % (xml_id,)
 
     def parse(self, de):
         assert de.tag in self.DATA_ROOTS, "Root xml tag must be <openerp>, <odoo> or <data>."
-        try:
-            self._tag_root(de)
-        except Exception as e:
-            exc_info = sys.exc_info()
-            pycompat.reraise(
-                ParseError,
-                ParseError(
-                    ustr(e),
-                    etree.tostring(de, encoding='unicode').rstrip(),
-                    de.getroottree().docinfo.URL, de.sourceline),
-                exc_info[2]
-            )
+        self._tag_root(de)
     DATA_ROOTS = ['odoo', 'data', 'openerp']
 
-def convert_file(cr, module, filename, idref, mode='update', noupdate=False, kind=None, report=None, pathname=None):
+def convert_file(cr, module, filename, idref, mode='update', noupdate=False, kind=None, pathname=None):
     if pathname is None:
         pathname = os.path.join(module, filename)
     ext = os.path.splitext(filename)[1].lower()
@@ -733,14 +742,14 @@ def convert_file(cr, module, filename, idref, mode='update', noupdate=False, kin
         elif ext == '.sql':
             convert_sql_import(cr, fp)
         elif ext == '.xml':
-            convert_xml_import(cr, module, fp, idref, mode, noupdate, report)
+            convert_xml_import(cr, module, fp, idref, mode, noupdate)
         elif ext == '.js':
             pass # .js files are valid but ignored here.
         else:
             raise ValueError("Can't load unknown file type %s.", filename)
 
 def convert_sql_import(cr, fp):
-    cr.execute(fp.read())
+    cr.execute(fp.read()) # pylint: disable=sql-injection
 
 def convert_csv_import(cr, module, fname, csvcontent, idref=None, mode='init',
         noupdate=False):
@@ -750,7 +759,6 @@ def convert_csv_import(cr, module, fname, csvcontent, idref=None, mode='init',
         encoding: utf-8'''
     filename, _ext = os.path.splitext(os.path.basename(fname))
     model = filename.split('-')[0]
-
     reader = pycompat.csv_reader(io.BytesIO(csvcontent), quotechar='"', delimiter=',')
     fields = next(reader)
 
@@ -788,10 +796,10 @@ def convert_xml_import(cr, module, xmlfile, idref=None, mode='init', noupdate=Fa
         _logger.exception("The XML file '%s' does not fit the required schema !", xmlfile.name)
         if jingtrang:
             p = subprocess.run(['pyjing', schema, xmlfile.name], stdout=subprocess.PIPE)
-            _logger.warn(p.stdout.decode())
+            _logger.warning(p.stdout.decode())
         else:
             for e in relaxng.error_log:
-                _logger.warn(e)
+                _logger.warning(e)
             _logger.info("Install 'jingtrang' for more precise and useful validation messages.")
         raise
 
@@ -799,5 +807,5 @@ def convert_xml_import(cr, module, xmlfile, idref=None, mode='init', noupdate=Fa
         xml_filename = xmlfile
     else:
         xml_filename = xmlfile.name
-    obj = xml_import(cr, module, idref, mode, report=report, noupdate=noupdate, xml_filename=xml_filename)
+    obj = xml_import(cr, module, idref, mode, noupdate=noupdate, xml_filename=xml_filename)
     obj.parse(doc.getroot())
