@@ -4,16 +4,19 @@
 import base64
 import collections
 import logging
-from lxml.html import clean
 import random
 import re
 import socket
 import threading
 import time
-
-from email.header import decode_header, Header
 from email.utils import getaddresses
+from urllib.parse import urlparse
+
+import idna
+import markupsafe
 from lxml import etree
+from lxml.html import clean
+from werkzeug import urls
 
 import odoo
 from odoo.loglevels import ustr
@@ -25,32 +28,20 @@ _logger = logging.getLogger(__name__)
 # HTML Sanitizer
 #----------------------------------------------------------
 
-tags_to_kill = ["script", "head", "meta", "title", "link", "style", "frame", "iframe", "base", "object", "embed"]
+tags_to_kill = ['base', 'embed', 'frame', 'head', 'iframe', 'link', 'meta',
+                'noscript', 'object', 'script', 'style', 'title']
+
 tags_to_remove = ['html', 'body']
 
 # allow new semantic HTML5 tags
-allowed_tags = frozenset({
-    'a', 'abbr', 'acronym', 'address', 'applet', 'area', 'article', 'aside',
-    'audio', 'b', 'basefont', 'bdi', 'bdo', 'big', 'blink', 'blockquote', 'body', 'br',
-    'button', 'canvas', 'caption', 'center', 'cite', 'code', 'col', 'colgroup',
-    'command', 'datalist', 'dd', 'del', 'details', 'dfn', 'dir', 'div', 'dl',
-    'dt', 'em', 'fieldset', 'figcaption', 'figure', 'font', 'footer', 'form',
-    'frameset', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'header', 'hgroup', 'hr', 'html',
-    'i', 'img', 'input', 'ins', 'isindex', 'kbd', 'keygen', 'label', 'legend',
-    'li', 'main', 'map', 'mark', 'marquee', 'math', 'menu', 'meter', 'nav',
-    'ol', 'optgroup', 'option', 'output', 'p', 'param', 'pre', 'progress', 'q',
-    'rp', 'rt', 'ruby', 's', 'samp', 'section', 'select', 'small', 'source',
-    'span', 'strike', 'strong', 'sub', 'summary', 'sup', 'svg', 'table', 'tbody',
-    'td', 'textarea', 'tfoot', 'th', 'thead', 'time', 'tr', 'track', 'tt', 'u',
-    'ul', 'var', 'video', 'wbr'
-}) | frozenset([etree.Comment])
-
+allowed_tags = clean.defs.tags | frozenset('article bdi section header footer hgroup nav aside figure main'.split() + [etree.Comment])
 safe_attrs = clean.defs.safe_attrs | frozenset(
     ['style',
      'data-o-mail-quote',  # quote detection
      'data-oe-model', 'data-oe-id', 'data-oe-field', 'data-oe-type', 'data-oe-expression', 'data-oe-translation-id', 'data-oe-nodeid',
      'data-publish', 'data-id', 'data-res_id', 'data-interval', 'data-member_id', 'data-scroll-background-ratio', 'data-view-id',
-     'data-class', 'data-mimetype',
+     'data-class', 'data-mimetype', 'data-original-src', 'data-original-id', 'data-gl-filter', 'data-quality', 'data-resize-width',
+     'data-shape', 'data-shape-colors', 'data-file-name', 'data-original-mimetype',
      ])
 
 
@@ -132,13 +123,20 @@ class _Cleaner(clean.Cleaner):
         el_id = el.get('id', '') or ''
 
         # gmail or yahoo // # outlook, html // # msoffice
-        if ('gmail_extra' in el_class or 'yahoo_quoted' in el_class) or \
-                (el.tag == 'hr' and ('stopSpelling' in el_class or 'stopSpelling' in el_id)) or \
+        if 'gmail_extra' in el_class or \
+                'divRplyFwdMsg' in el_id or \
                 ('SkyDrivePlaceholder' in el_class or 'SkyDrivePlaceholder' in el_class):
             el.set('data-o-mail-quote', '1')
             if el.getparent() is not None:
                 el.getparent().set('data-o-mail-quote-container', '1')
 
+        if (el.tag == 'hr' and ('stopSpelling' in el_class or 'stopSpelling' in el_id)) or \
+           'yahoo_quoted' in el_class:
+            # Quote all elements after this one
+            el.set('data-o-mail-quote', '1')
+            for sibling in el.itersiblings(preceding=False):
+                sibling.set('data-o-mail-quote', '1')
+
         # html signature (-- <br />blah)
         signature_begin = re.compile(r"((?:(?:^|\n)[-]{2}[\s]?$))")
         if el.text and el.find('br') is not None and re.search(signature_begin, el.text):
@@ -177,7 +175,7 @@ class _Cleaner(clean.Cleaner):
                 del el.attrib['style']
 
 
-def html_sanitize(src, silent=True, sanitize_tags=True, sanitize_attributes=False, sanitize_style=False, strip_style=False, strip_classes=False, sanitize_form=True):
+def html_sanitize(src, silent=True, sanitize_tags=True, sanitize_attributes=False, sanitize_style=False, sanitize_form=True, strip_style=False, strip_classes=False):
     if not src:
         return src
     src = ustr(src, errors='replace')
@@ -260,11 +258,37 @@ def html_sanitize(src, silent=True, sanitize_tags=True, sanitize_attributes=Fals
     if cleaned.startswith(u'<div>') and cleaned.endswith(u'</div>'):
         cleaned = cleaned[5:-6]
 
-    return cleaned
+    return markupsafe.Markup(cleaned)
 
-#----------------------------------------------------------
+# ----------------------------------------------------------
 # HTML/Text management
-#----------------------------------------------------------
+# ----------------------------------------------------------
+
+URL_REGEX = r'(\bhref=[\'"](?!mailto:|tel:|sms:)([^\'"]+)[\'"])'
+TEXT_URL_REGEX = r'https?://[a-zA-Z0-9@:%._\+~#=/-]+(?:\?\S+)?'
+# retrieve inner content of the link
+HTML_TAG_URL_REGEX = URL_REGEX + r'([^<>]*>([^<>]+)<\/)?'
+
+
+def validate_url(url):
+    if urls.url_parse(url).scheme not in ('http', 'https', 'ftp', 'ftps'):
+        return 'http://' + url
+
+    return url
+
+
+def is_html_empty(html_content):
+    """Check if a html content is empty. If there are only formatting tags with style
+    attributes or a void content  return True. Famous use case if a
+    '<p style="..."><br></p>' added by some web editor.
+
+    :param str html_content: html content, coming from example from an HTML field
+    :returns: bool, True if no content found or if containing only void formatting tags
+    """
+    if not html_content:
+        return True
+    tag_re = re.compile(r'\<\s*\/?(?:p|div|span|br|b|i|font)(?:(?=\s+\w*)[^/>]*|\s*)/?\s*\>')
+    return not bool(re.sub(tag_re, '', html_content).strip())
 
 def html_keep_url(text):
     """ Transform the url into clickable link with <a/> tag """
@@ -278,6 +302,7 @@ def html_keep_url(text):
     final += text[idx:]
     return final
 
+
 def html2plaintext(html, body_id=None, encoding='utf-8'):
     """ From an HTML text, convert the HTML to plain text.
     If @param body_id is provided then this is the tag where the
@@ -355,8 +380,7 @@ def plaintext2html(text, container_tag=False):
     text = misc.html_escape(ustr(text))
 
     # 1. replace \n and \r
-    text = text.replace('\n', '<br/>')
-    text = text.replace('\r', '<br/>')
+    text = re.sub(r'(\r\n|\r|\n)', '<br/>', text)
 
     # 2. clickable links
     text = html_keep_url(text)
@@ -364,16 +388,16 @@ def plaintext2html(text, container_tag=False):
     # 3-4: form paragraphs
     idx = 0
     final = '<p>'
-    br_tags = re.compile(r'(([<]\s*[bB][rR]\s*\/?[>]\s*){2,})')
+    br_tags = re.compile(r'(([<]\s*[bB][rR]\s*/?[>]\s*){2,})')
     for item in re.finditer(br_tags, text):
         final += text[idx:item.start()] + '</p><p>'
         idx = item.end()
     final += text[idx:] + '</p>'
 
     # 5. container
-    if container_tag:
+    if container_tag: # FIXME: validate that container_tag is just a simple tag?
         final = '<%s>%s</%s>' % (container_tag, final, container_tag)
-    return ustr(final)
+    return markupsafe.Markup(final)
 
 def append_content_to_html(html, content, plaintext=True, preserve=False, container_tag=False):
     """ Append extra content at the end of an HTML snippet, trying
@@ -410,8 +434,19 @@ def append_content_to_html(html, content, plaintext=True, preserve=False, contai
     if insert_location == -1:
         insert_location = html.find('</html>')
     if insert_location == -1:
-        return '%s%s' % (html, content)
-    return '%s%s%s' % (html[:insert_location], content, html[insert_location:])
+        return markupsafe.Markup('%s%s' % (html, content))
+    return markupsafe.Markup('%s%s%s' % (html[:insert_location], content, html[insert_location:]))
+
+
+def prepend_html_content(html_body, html_content):
+    """Prepend some HTML content at the beginning of an other HTML content."""
+    html_content = type(html_content)(re.sub(r'(?i)(</?(?:html|body|head|!\s*DOCTYPE)[^>]*>)', '', html_content))
+    html_content = html_content.strip()
+
+    body_match = re.search(r'<body[^>]*>', html_body) or re.search(r'<html[^>]*>', html_body)
+    insert_index = body_match.end() if body_match else 0
+
+    return html_body[:insert_index] + html_content + html_body[insert_index:]
 
 #----------------------------------------------------------
 # Emails
@@ -441,48 +476,10 @@ def generate_tracking_message_id(res_id):
     rndstr = ("%.15f" % rnd)[2:]
     return "<%s.%.15f-openerp-%s@%s>" % (rndstr, time.time(), res_id, socket.gethostname())
 
-def email_send(email_from, email_to, subject, body, email_cc=None, email_bcc=None, reply_to=False,
-               attachments=None, message_id=None, references=None, openobject_id=False, debug=False, subtype='plain', headers=None,
-               smtp_server=None, smtp_port=None, ssl=False, smtp_user=None, smtp_password=None, cr=None, uid=None):
-    """Low-level function for sending an email (deprecated).
-
-    :deprecate: since OpenERP 6.1, please use ir.mail_server.send_email() instead.
-    :param email_from: A string used to fill the `From` header, if falsy,
-                       config['email_from'] is used instead.  Also used for
-                       the `Reply-To` header if `reply_to` is not provided
-    :param email_to: a sequence of addresses to send the mail to.
-    """
-
-    # If not cr, get cr from current thread database
-    local_cr = None
-    if not cr:
-        db_name = getattr(threading.currentThread(), 'dbname', None)
-        if db_name:
-            local_cr = cr = odoo.registry(db_name).cursor()
-        else:
-            raise Exception("No database cursor found, please pass one explicitly")
-
-    # Send Email
-    try:
-        mail_server_pool = odoo.registry(cr.dbname)['ir.mail_server']
-        res = False
-        # Pack Message into MIME Object
-        email_msg = mail_server_pool.build_email(email_from, email_to, subject, body, email_cc, email_bcc, reply_to,
-                   attachments, message_id, references, openobject_id, subtype, headers=headers)
-
-        res = mail_server_pool.send_email(cr, uid or 1, email_msg, mail_server_id=None,
-                       smtp_server=smtp_server, smtp_port=smtp_port, smtp_user=smtp_user, smtp_password=smtp_password,
-                       smtp_encryption=('ssl' if ssl else None), smtp_debug=debug)
-    except Exception:
-        _logger.exception("tools.email_send failed to deliver email")
-        return False
-    finally:
-        if local_cr:
-            cr.close()
-    return res
-
 def email_split_tuples(text):
-    """ Return a list of (name, email) addresse tuples found in ``text``"""
+    """ Return a list of (name, email) address tuples found in ``text`` . Note
+    that text should be an email header or a stringified email list as it may
+    give broader results than expected on actual text. """
     if not text:
         return []
     return [(addr[0], addr[1]) for addr in getaddresses([text])
@@ -520,64 +517,55 @@ def email_normalize(text):
         return False
     return emails[0].lower()
 
+
+def email_domain_extract(email):
+    """ Extract the company domain to be used by IAP services notably. Domain
+    is extracted from email information e.g:
+        - info@proximus.be -> proximus.be
+    """
+    normalized_email = email_normalize(email)
+    if normalized_email:
+        return normalized_email.split('@')[1]
+    return False
+
+def email_domain_normalize(domain):
+    """Return the domain normalized or False if the domain is invalid."""
+    if not domain or '@' in domain:
+        return False
+
+    return domain.lower()
+
+def url_domain_extract(url):
+    """ Extract the company domain to be used by IAP services notably. Domain
+    is extracted from an URL e.g:
+        - www.info.proximus.be -> proximus.be
+    """
+    parser_results = urlparse(url)
+    company_hostname = parser_results.hostname
+    if company_hostname and '.' in company_hostname:
+        return '.'.join(company_hostname.split('.')[-2:])  # remove subdomains
+    return False
+
 def email_escape_char(email_address):
     """ Escape problematic characters in the given email address string"""
     return email_address.replace('\\', '\\\\').replace('%', '\\%').replace('_', '\\_')
 
-
-def email_domain_extract(email):
-    """Return the domain of the given email."""
-    if not email:
-        return
-
-    email_split = getaddresses([email])
-    if not email_split or not email_split[0]:
-        return
-
-    _, _, domain = email_split[0][1].rpartition('@')
-    return domain
-
-# was mail_message.decode()
-def decode_smtp_header(smtp_header, quoted=False):
-    """Returns unicode() string conversion of the given encoded smtp header
-    text. email.header decode_header method return a decoded string and its
-    charset for each decoded part of the header. This method unicodes the
-    decoded header and join them in a complete string.
-
-    :param bool quoted: when True, encoded words in the header will be turned into RFC822
-        quoted-strings after decoding, which is appropriate for address headers
-    """
-    if isinstance(smtp_header, Header):
-        smtp_header = ustr(smtp_header)
-    if smtp_header:
-        pairs = decode_header(smtp_header.replace('\r', ''))
-        tokens = []
-        for token, enc in pairs:
-            token = ustr(token, enc)
-            if enc and quoted:
-                # re-quote the encoded word to form an RFC822 quoted-string
-                token = email_addr_escapes_re.sub(r'\\\g<0>', token)
-                tokens.append('"%s"' % token)
-            else:
-                # plain word
-                tokens.append(token)
-        return ''.join(tokens)
-    return ''
-
 # was mail_thread.decode_header()
-def decode_message_header(message, header, separator=' ', quoted=False):
-    return separator.join(decode_smtp_header(h, quoted=quoted) for h in message.get_all(header, []) if h)
+def decode_message_header(message, header, separator=' '):
+    return separator.join(h for h in message.get_all(header, []) if h)
 
 def formataddr(pair, charset='utf-8'):
     """Pretty format a 2-tuple of the form (realname, email_address).
 
-    Set the charset to ascii to get a RFC-2822 compliant email.
-
-    The email address is considered valid and is left unmodified.
-
     If the first element of pair is falsy then only the email address
     is returned.
 
+    Set the charset to ascii to get a RFC-2822 compliant email. The
+    realname will be base64 encoded (if necessary) and the domain part
+    of the email will be punycode encoded (if necessary). The local part
+    is left unchanged thus require the SMTPUTF8 extension when there are
+    non-ascii characters.
+
     >>> formataddr(('John Doe', 'johndoe@example.com'))
     '"John Doe" <johndoe@example.com>'
 
@@ -585,24 +573,29 @@ def formataddr(pair, charset='utf-8'):
     'johndoe@example.com'
     """
     name, address = pair
-    address.encode('ascii')
+    local, _, domain = address.rpartition('@')
+
+    try:
+        domain.encode(charset)
+    except UnicodeEncodeError:
+        # rfc5890 - Internationalized Domain Names for Applications (IDNA)
+        domain = idna.encode(domain).decode('ascii')
+
     if name:
         try:
             name.encode(charset)
         except UnicodeEncodeError:
             # charset mismatch, encode as utf-8/base64
             # rfc2047 - MIME Message Header Extensions for Non-ASCII Text
-            return "=?utf-8?b?{name}?= <{addr}>".format(
-                name=base64.b64encode(name.encode('utf-8')).decode('ascii'),
-                addr=address)
+            name = base64.b64encode(name.encode('utf-8')).decode('ascii')
+            return f"=?utf-8?b?{name}?= <{local}@{domain}>"
         else:
             # ascii name, escape it if needed
             # rfc2822 - Internet Message Format
             #   #section-3.4 - Address Specification
-            return '"{name}" <{addr}>'.format(
-                name=email_addr_escapes_re.sub(r'\\\g<0>', name),
-                addr=address)
-    return address
+            name = email_addr_escapes_re.sub(r'\\\g<0>', name)
+            return f'"{name}" <{local}@{domain}>'
+    return f"{local}@{domain}"
 
 
 def encapsulate_email(old_email, new_email):
@@ -611,7 +604,7 @@ def encapsulate_email(old_email, new_email):
     e.g.
     * Old From: "Admin" <admin@gmail.com>
     * New From: notifications@odoo.com
-    * Output:   "Admin (admin@gmail.com)" <notifications@odoo.com>
+    * Output: "Admin" <notifications@odoo.com>
     """
     old_email_split = getaddresses([old_email])
     if not old_email_split or not old_email_split[0]:
@@ -621,10 +614,11 @@ def encapsulate_email(old_email, new_email):
     if not new_email_split or not new_email_split[0]:
         return
 
-    if old_email_split[0][0]:
-        name_part = '%s (%s)' % old_email_split[0]
+    old_name, old_email = old_email_split[0]
+    if old_name:
+        name_part = old_name
     else:
-        name_part = old_email_split[0][1]
+        name_part = old_email.split("@")[0]
 
     return formataddr((
         name_part,
