@@ -5,7 +5,7 @@ from babel.dates import format_datetime, format_date
 from odoo import models, api, _, fields
 from odoo.osv import expression
 from odoo.release import version
-from odoo.tools import DEFAULT_SERVER_DATE_FORMAT as DF, safe_eval
+from odoo.tools import DEFAULT_SERVER_DATE_FORMAT as DF
 from odoo.tools.misc import formatLang, format_date as odoo_format_date, get_lang
 import random
 
@@ -41,6 +41,7 @@ class account_journal(models.Model):
                     act_type.category as activity_category,
                     act.date_deadline,
                     m.date,
+                    m.ref,
                     CASE WHEN act.date_deadline < CURRENT_DATE THEN 'late' ELSE 'future' END as status
                 FROM account_move m
                     LEFT JOIN mail_activity act ON act.res_id = m.id
@@ -50,7 +51,7 @@ class account_journal(models.Model):
             '''
             self.env.cr.execute(sql_query, (journal.id,))
             for activity in self.env.cr.dictfetchall():
-                activities.append({
+                act = {
                     'id': activity.get('id'),
                     'res_id': activity.get('res_id'),
                     'res_model': activity.get('res_model'),
@@ -58,7 +59,11 @@ class account_journal(models.Model):
                     'name': (activity.get('summary') or activity.get('act_type_name')),
                     'activity_category': activity.get('activity_category'),
                     'date': odoo_format_date(self.env, activity.get('date_deadline'))
-                })
+                }
+                if activity.get('activity_category') == 'tax_report' and activity.get('res_model') == 'account.move':
+                    act['name'] = activity.get('ref')
+
+                activities.append(act)
             journal.json_activity_data = json.dumps({'activities': activities})
 
     kanban_dashboard = fields.Text(compute='_kanban_dashboard')
@@ -66,6 +71,19 @@ class account_journal(models.Model):
     json_activity_data = fields.Text(compute='_get_json_activity_data')
     show_on_dashboard = fields.Boolean(string='Show journal on dashboard', help="Whether this journal should be displayed on the dashboard or not", default=True)
     color = fields.Integer("Color Index", default=0)
+    entries_count = fields.Integer(compute='_compute_entries_count')
+
+    def _compute_entries_count(self):
+        res = {
+            r['journal_id'][0]: r['journal_id_count']
+            for r in self.env['account.move'].read_group(
+                domain=[('journal_id', 'in', self.ids)],
+                fields=['journal_id'],
+                groupby=['journal_id'],
+            )
+        }
+        for journal in self:
+            journal.entries_count = res.get(journal.id, 0)
 
     def _graph_title_and_key(self):
         if self.type in ['sale', 'purchase']:
@@ -94,7 +112,7 @@ class account_journal(models.Model):
         locale = get_lang(self.env).code
 
         #starting point of the graph is the last statement
-        last_stmt = BankStatement.search([('journal_id', '=', self.id), ('date', '<=', today.strftime(DF))], order='date desc, id desc', limit=1)
+        last_stmt = self._get_last_bank_statement(domain=[('move_id.state', '=', 'posted')])
 
         last_balance = last_stmt and last_stmt.balance_end_real or 0
         data.append(build_graph_data(today, last_balance))
@@ -103,15 +121,16 @@ class account_journal(models.Model):
         #(graph is drawn backward)
         date = today
         amount = last_balance
-        query = """SELECT l.date, sum(l.amount) as amount
-                        FROM account_bank_statement_line l
-                        RIGHT JOIN account_bank_statement st ON l.statement_id = st.id
-                        WHERE st.journal_id = %s
-                          AND l.date > %s
-                          AND l.date <= %s
-                        GROUP BY l.date
-                        ORDER BY l.date desc
-                        """
+        query = '''
+            SELECT move.date, sum(st_line.amount) as amount
+            FROM account_bank_statement_line st_line
+            JOIN account_move move ON move.id = st_line.move_id
+            WHERE move.journal_id = %s
+            AND move.date > %s
+            AND move.date <= %s
+            GROUP BY move.date
+            ORDER BY move.date desc
+        '''
         self.env.cr.execute(query, (self.id, last_month, today))
         query_result = self.env.cr.dictfetchall()
         for val in query_result:
@@ -204,8 +223,8 @@ class account_journal(models.Model):
             FROM account_move move
             WHERE move.journal_id = %(journal_id)s
             AND move.state = 'posted'
-            AND move.invoice_payment_state = 'not_paid'
-            AND move.type IN %(invoice_types)s
+            AND move.payment_state in ('not_paid', 'partial')
+            AND move.move_type IN %(invoice_types)s
         ''', {
             'invoice_types': tuple(self.env['account.move'].get_invoice_types(True)),
             'journal_id': self.id
@@ -213,41 +232,41 @@ class account_journal(models.Model):
 
     def get_journal_dashboard_datas(self):
         currency = self.currency_id or self.company_id.currency_id
-        number_to_reconcile = number_to_check = last_balance = account_sum = 0
+        number_to_reconcile = number_to_check = last_balance = 0
+        has_at_least_one_statement = False
+        bank_account_balance = nb_lines_bank_account_balance = 0
+        outstanding_pay_account_balance = nb_lines_outstanding_pay_account_balance = 0
         title = ''
         number_draft = number_waiting = number_late = to_check_balance = 0
         sum_draft = sum_waiting = sum_late = 0.0
-        if self.type in ['bank', 'cash']:
-            last_bank_stmt = self.env['account.bank.statement'].search([('journal_id', 'in', self.ids)], order="date desc, id desc", limit=1)
-            last_balance = last_bank_stmt and last_bank_stmt[0].balance_end or 0
-            #Get the number of items to reconcile for that bank journal
-            self.env.cr.execute("""SELECT COUNT(DISTINCT(line.id))
-                            FROM account_bank_statement_line AS line
-                            LEFT JOIN account_bank_statement AS st
-                            ON line.statement_id = st.id
-                            WHERE st.journal_id IN %s AND st.state = 'open' AND line.amount != 0.0 AND line.account_id IS NULL
-                            AND not exists (select 1 from account_move_line aml where aml.statement_line_id = line.id)
-                        """, (tuple(self.ids),))
+        if self.type in ('bank', 'cash'):
+            last_statement = self._get_last_bank_statement(
+                domain=[('move_id.state', '=', 'posted')])
+            last_balance = last_statement.balance_end
+            has_at_least_one_statement = bool(last_statement)
+            bank_account_balance, nb_lines_bank_account_balance = self._get_journal_bank_account_balance(
+                domain=[('parent_state', '=', 'posted')])
+            outstanding_pay_account_balance, nb_lines_outstanding_pay_account_balance = self._get_journal_outstanding_payments_account_balance(
+                domain=[('parent_state', '=', 'posted')])
+
+            self._cr.execute('''
+                SELECT COUNT(st_line.id)
+                FROM account_bank_statement_line st_line
+                JOIN account_move st_line_move ON st_line_move.id = st_line.move_id
+                JOIN account_bank_statement st ON st_line.statement_id = st.id
+                WHERE st_line_move.journal_id IN %s
+                AND st.state = 'posted'
+                AND NOT st_line.is_reconciled
+            ''', [tuple(self.ids)])
             number_to_reconcile = self.env.cr.fetchone()[0]
+
             to_check_ids = self.to_check_ids()
             number_to_check = len(to_check_ids)
             to_check_balance = sum([r.amount for r in to_check_ids])
-            # optimization to read sum of balance from account_move_line
-            account_ids = tuple(ac for ac in [self.default_debit_account_id.id, self.default_credit_account_id.id] if ac)
-            if account_ids:
-                amount_field = 'aml.balance' if (not self.currency_id or self.currency_id == self.company_id.currency_id) else 'aml.amount_currency'
-                query = """SELECT sum(%s) FROM account_move_line aml
-                           LEFT JOIN account_move move ON aml.move_id = move.id
-                           WHERE aml.account_id in %%s
-                           AND move.date <= %%s AND move.state = 'posted';""" % (amount_field,)
-                self.env.cr.execute(query, (account_ids, fields.Date.context_today(self),))
-                query_results = self.env.cr.dictfetchall()
-                if query_results and query_results[0].get('sum') != None:
-                    account_sum = query_results[0].get('sum')
         #TODO need to check if all invoices are in the same currency than the journal!!!!
         elif self.type in ['sale', 'purchase']:
             title = _('Bills to pay') if self.type == 'purchase' else _('Invoices owed to you')
-            self.env['account.move'].flush(['amount_residual', 'currency_id', 'type', 'invoice_date', 'company_id', 'journal_id', 'date', 'state', 'invoice_payment_state'])
+            self.env['account.move'].flush(['amount_residual', 'currency_id', 'move_type', 'invoice_date', 'company_id', 'journal_id', 'date', 'state', 'payment_state'])
 
             (query, query_args) = self._get_open_bills_to_pay_query()
             self.env.cr.execute(query, query_args)
@@ -257,23 +276,10 @@ class account_journal(models.Model):
             self.env.cr.execute(query, query_args)
             query_results_drafts = self.env.cr.dictfetchall()
 
-            today = fields.Date.today()
-            query = '''
-                SELECT
-                    (CASE WHEN type IN ('out_refund', 'in_refund') THEN -1 ELSE 1 END) * amount_residual AS amount_total,
-                    currency_id AS currency,
-                    type,
-                    invoice_date,
-                    company_id
-                FROM account_move move
-                WHERE journal_id = %s
-                AND invoice_date_due <= %s
-                AND state = 'posted'
-                AND invoice_payment_state = 'not_paid'
-                AND type IN ('out_invoice', 'out_refund', 'in_invoice', 'in_refund', 'out_receipt', 'in_receipt');
-            '''
-            self.env.cr.execute(query, (self.id, today))
+            (query, query_args) = self._get_late_bills_query()
+            self.env.cr.execute(query, query_args)
             late_query_results = self.env.cr.dictfetchall()
+
             curr_cache = {}
             (number_waiting, sum_waiting) = self._count_results_and_sum_amounts(query_results_to_pay, currency, curr_cache=curr_cache)
             (number_draft, sum_draft) = self._count_results_and_sum_amounts(query_results_drafts, currency, curr_cache=curr_cache)
@@ -288,17 +294,18 @@ class account_journal(models.Model):
                 number_to_check = read[0]['__count']
                 to_check_balance = read[0]['amount_total']
 
-        difference = currency.round(last_balance-account_sum) + 0.0
-
         is_sample_data = self.kanban_dashboard_graph and any(data.get('is_sample_data', False) for data in json.loads(self.kanban_dashboard_graph))
 
         return {
             'number_to_check': number_to_check,
             'to_check_balance': formatLang(self.env, to_check_balance, currency_obj=currency),
             'number_to_reconcile': number_to_reconcile,
-            'account_balance': formatLang(self.env, currency.round(account_sum) + 0.0, currency_obj=currency),
+            'account_balance': formatLang(self.env, currency.round(bank_account_balance), currency_obj=currency),
+            'has_at_least_one_statement': has_at_least_one_statement,
+            'nb_lines_bank_account_balance': nb_lines_bank_account_balance,
+            'outstanding_pay_account_balance': formatLang(self.env, currency.round(outstanding_pay_account_balance), currency_obj=currency),
+            'nb_lines_outstanding_pay_account_balance': nb_lines_outstanding_pay_account_balance,
             'last_balance': formatLang(self.env, currency.round(last_balance) + 0.0, currency_obj=currency),
-            'difference': formatLang(self.env, difference, currency_obj=currency) if difference else False,
             'number_draft': number_draft,
             'number_waiting': number_waiting,
             'number_late': number_late,
@@ -309,6 +316,7 @@ class account_journal(models.Model):
             'bank_statements_source': self.bank_statements_source,
             'title': title,
             'is_sample_data': is_sample_data,
+            'company_count': len(self.env.companies)
         }
 
     def _get_open_bills_to_pay_query(self):
@@ -319,16 +327,16 @@ class account_journal(models.Model):
         """
         return ('''
             SELECT
-                (CASE WHEN move.type IN ('out_refund', 'in_refund') THEN -1 ELSE 1 END) * move.amount_residual AS amount_total,
+                (CASE WHEN move.move_type IN ('out_refund', 'in_refund') THEN -1 ELSE 1 END) * move.amount_residual AS amount_total,
                 move.currency_id AS currency,
-                move.type,
+                move.move_type,
                 move.invoice_date,
                 move.company_id
             FROM account_move move
             WHERE move.journal_id = %(journal_id)s
             AND move.state = 'posted'
-            AND move.invoice_payment_state = 'not_paid'
-            AND move.type IN ('out_invoice', 'out_refund', 'in_invoice', 'in_refund', 'out_receipt', 'in_receipt');
+            AND move.payment_state in ('not_paid', 'partial')
+            AND move.move_type IN ('out_invoice', 'out_refund', 'in_invoice', 'in_refund', 'out_receipt', 'in_receipt');
         ''', {'journal_id': self.id})
 
     def _get_draft_bills_query(self):
@@ -339,18 +347,34 @@ class account_journal(models.Model):
         """
         return ('''
             SELECT
-                (CASE WHEN move.type IN ('out_refund', 'in_refund') THEN -1 ELSE 1 END) * move.amount_total AS amount_total,
+                (CASE WHEN move.move_type IN ('out_refund', 'in_refund') THEN -1 ELSE 1 END) * move.amount_total AS amount_total,
                 move.currency_id AS currency,
-                move.type,
+                move.move_type,
                 move.invoice_date,
                 move.company_id
             FROM account_move move
             WHERE move.journal_id = %(journal_id)s
             AND move.state = 'draft'
-            AND move.invoice_payment_state = 'not_paid'
-            AND move.type IN ('out_invoice', 'out_refund', 'in_invoice', 'in_refund', 'out_receipt', 'in_receipt');
+            AND move.payment_state in ('not_paid', 'partial')
+            AND move.move_type IN ('out_invoice', 'out_refund', 'in_invoice', 'in_refund', 'out_receipt', 'in_receipt');
         ''', {'journal_id': self.id})
 
+    def _get_late_bills_query(self):
+        return """
+            SELECT
+                (CASE WHEN move_type IN ('out_refund', 'in_refund') THEN -1 ELSE 1 END) * amount_residual AS amount_total,
+                currency_id AS currency,
+                move_type,
+                invoice_date,
+                company_id
+            FROM account_move move
+            WHERE journal_id = %(journal_id)s
+            AND invoice_date_due < %(today)s
+            AND state = 'posted'
+            AND payment_state in ('not_paid', 'partial')
+            AND move_type IN ('out_invoice', 'out_refund', 'in_invoice', 'in_refund', 'out_receipt', 'in_receipt');
+        """, {'journal_id': self.id, 'today': fields.Date.context_today(self)}
+
     def _count_results_and_sum_amounts(self, results_dict, target_currency, curr_cache=None):
         """ Loops on a query result to count the total number of invoices and sum
         their amount_total field (expressed in the given target currency).
@@ -366,7 +390,7 @@ class account_journal(models.Model):
             cur = self.env['res.currency'].browse(result.get('currency'))
             company = self.env['res.company'].browse(result.get('company_id')) or self.env.company
             rslt_count += 1
-            date = result.get('invoice_date') or fields.Date.today()
+            date = result.get('invoice_date') or fields.Date.context_today(self)
 
             amount = result.get('amount_total', 0) or 0
             if cur != target_currency:
@@ -383,11 +407,11 @@ class account_journal(models.Model):
         ctx = self._context.copy()
         ctx['default_journal_id'] = self.id
         if self.type == 'sale':
-            ctx['default_type'] = 'out_refund' if ctx.get('refund') else 'out_invoice'
+            ctx['default_move_type'] = 'out_refund' if ctx.get('refund') else 'out_invoice'
         elif self.type == 'purchase':
-            ctx['default_type'] = 'in_refund' if ctx.get('refund') else 'in_invoice'
+            ctx['default_move_type'] = 'in_refund' if ctx.get('refund') else 'in_invoice'
         else:
-            ctx['default_type'] = 'entry'
+            ctx['default_move_type'] = 'entry'
             ctx['view_no_maturity'] = True
         return {
             'name': _('Create invoice/bill'),
@@ -421,46 +445,23 @@ class account_journal(models.Model):
             })
         return action
 
-    def action_open_reconcile(self):
-        if self.type in ['bank', 'cash']:
-            # Open reconciliation view for bank statements belonging to this journal
-            limit = int(self.env["ir.config_parameter"].sudo().get_param("account.reconcile.batch", 1000))
-            bank_stmt = self.env['account.bank.statement.line'].search([
-                ('journal_id', 'in', self.ids),
-                # take not reconciled lines only. See _check_lines_reconciled method
-                ('account_id', '=', False),
-                ('journal_entry_ids', '=', False),
-                ('amount', '!=', 0),
-            ], limit=limit)
+    def action_create_vendor_bill(self):
+        """ This function is called by the "Import" button of Vendor Bills,
+        visible on dashboard if no bill has been created yet.
+        """
+        self.env.company.sudo().set_onboarding_step_done('account_setup_bill_state')
 
-            return {
-                'type': 'ir.actions.client',
-                'tag': 'bank_statement_reconciliation_view',
-                'context': {'statement_line_ids': bank_stmt.ids, 'company_ids': self.mapped('company_id').ids},
-            }
-        else:
-            # Open reconciliation view for customers/suppliers
-            action_context = {'show_mode_selector': False, 'company_ids': self.mapped('company_id').ids}
-            if self.type == 'sale':
-                action_context.update({'mode': 'customers'})
-            elif self.type == 'purchase':
-                action_context.update({'mode': 'suppliers'})
-            return {
-                'type': 'ir.actions.client',
-                'tag': 'manual_reconciliation_view',
-                'context': action_context,
-            }
+        new_wizard = self.env['account.tour.upload.bill'].create({})
+        view_id = self.env.ref('account.account_tour_upload_bill').id
 
-    def action_open_to_check(self):
-        self.ensure_one()
-        ids = self.to_check_ids().ids
-        action_context = {'show_mode_selector': False, 'company_ids': self.mapped('company_id').ids}
-        action_context.update({'suspense_moves_mode': True})
-        action_context.update({'statement_line_ids': ids})
         return {
-            'type': 'ir.actions.client',
-            'tag': 'bank_statement_reconciliation_view',
-            'context': action_context,
+            'type': 'ir.actions.act_window',
+            'name': _('Import your first bill'),
+            'view_mode': 'form',
+            'res_model': 'account.tour.upload.bill',
+            'target': 'new',
+            'res_id': new_wizard.id,
+            'views': [[view_id, 'form']],
         }
 
     def to_check_ids(self):
@@ -470,28 +471,32 @@ class account_journal(models.Model):
         statement_line_ids = self.env['account.move.line'].search(domain).mapped('statement_line_id')
         return statement_line_ids
 
+    def _select_action_to_open(self):
+        self.ensure_one()
+        if self._context.get('action_name'):
+            return self._context.get('action_name')
+        elif self.type == 'bank':
+            return 'action_bank_statement_tree'
+        elif self.type == 'cash':
+            return 'action_view_bank_statement_tree'
+        elif self.type == 'sale':
+            return 'action_move_out_invoice_type'
+        elif self.type == 'purchase':
+            return 'action_move_in_invoice_type'
+        else:
+            return 'action_move_journal_line'
+
     def open_action(self):
         """return action based on type for related journals"""
-        action_name = self._context.get('action_name')
-
-        # Find action based on journal.
-        if not action_name:
-            if self.type == 'bank':
-                action_name = 'action_bank_statement_tree'
-            elif self.type == 'cash':
-                action_name = 'action_view_bank_statement_tree'
-            elif self.type == 'sale':
-                action_name = 'action_move_out_invoice_type'
-            elif self.type == 'purchase':
-                action_name = 'action_move_in_invoice_type'
-            else:
-                action_name = 'action_move_journal_line'
+        self.ensure_one()
+        action_name = self._select_action_to_open()
 
         # Set 'account.' prefix if missing.
-        if '.' not in action_name:
+        if not action_name.startswith("account."):
             action_name = 'account.%s' % action_name
 
-        action = self.env.ref(action_name).read()[0]
+        action = self.env["ir.actions.act_window"]._for_xml_id(action_name)
+
         context = self._context.copy()
         if 'context' in action and type(action['context']) == str:
             context.update(ast.literal_eval(action['context']))
@@ -503,7 +508,7 @@ class account_journal(models.Model):
             'search_default_journal_id': self.id,
         })
 
-        domain_type_field = action['res_model'] == 'account.move.line' and 'move_id.type' or 'type' # The model can be either account.move or account.move.line
+        domain_type_field = action['res_model'] == 'account.move.line' and 'move_id.move_type' or 'move_type' # The model can be either account.move or account.move.line
 
         # Override the domain only if the action was not explicitly specified in order to keep the
         # original action domain.
@@ -511,7 +516,7 @@ class account_journal(models.Model):
             if self.type == 'sale':
                 action['domain'] = [(domain_type_field, 'in', ('out_invoice', 'out_refund', 'out_receipt'))]
             elif self.type == 'purchase':
-                action['domain'] = [(domain_type_field, 'in', ('in_invoice', 'in_refund', 'in_receipt'))]
+                action['domain'] = [(domain_type_field, 'in', ('in_invoice', 'in_refund', 'in_receipt', 'entry'))]
 
         return action
 
@@ -531,8 +536,13 @@ class account_journal(models.Model):
             action_ref = 'account.action_account_payments_transfer'
         else:
             action_ref = 'account.action_account_payments'
-        [action] = self.env.ref(action_ref).read()
-        action['context'] = dict(safe_eval(action.get('context')), default_journal_id=self.id, search_default_journal_id=self.id)
+        action = self.env['ir.actions.act_window']._for_xml_id(action_ref)
+        action['context'] = dict(ast.literal_eval(action.get('context')), default_journal_id=self.id, search_default_journal_id=self.id)
+        if payment_type == 'transfer':
+            action['context'].update({
+                'default_partner_id': self.company_id.partner_id.id,
+                'default_is_internal_transfer': True,
+            })
         if mode == 'form':
             action['views'] = [[False, 'form']]
         return action
@@ -544,19 +554,22 @@ class account_journal(models.Model):
         ctx = dict(self.env.context, default_journal_id=self.id)
         if ctx.get('search_default_journal', False):
             ctx.update(search_default_journal_id=self.id)
+            ctx['search_default_journal'] = False  # otherwise it will do a useless groupby in bank statements
         ctx.pop('group_by', None)
-        ir_model_obj = self.env['ir.model.data']
-        model, action_id = ir_model_obj.get_object_reference('account', action_name)
-        [action] = self.env[model].browse(action_id).read()
+        action = self.env['ir.actions.act_window']._for_xml_id(f"account.{action_name}")
         action['context'] = ctx
         if ctx.get('use_domain', False):
             action['domain'] = isinstance(ctx['use_domain'], list) and ctx['use_domain'] or ['|', ('journal_id', '=', self.id), ('journal_id', '=', False)]
-            action['name'] += ' for journal ' + self.name
+            action['name'] = _(
+                "%(action)s for journal %(journal)s",
+                action=action["name"],
+                journal=self.name,
+            )
         return action
 
     def create_bank_statement(self):
         """return action to create a bank statements. This button should be called only on journals with type =='bank'"""
-        action = self.env.ref('account.action_bank_statement_tree').read()[0]
+        action = self.env["ir.actions.actions"]._for_xml_id("account.action_bank_statement_tree")
         action.update({
             'views': [[False, 'form']],
             'context': "{'default_journal_id': " + str(self.id) + "}",
