@@ -25,6 +25,7 @@ class AccountDebitNote(models.TransientModel):
     # computed fields
     move_type = fields.Char(compute="_compute_from_moves")
     journal_type = fields.Char(compute="_compute_from_moves")
+    country_code = fields.Char(related='move_ids.company_id.country_id.code')
 
     @api.model
     def default_get(self, fields):
@@ -39,14 +40,14 @@ class AccountDebitNote(models.TransientModel):
     def _compute_from_moves(self):
         for record in self:
             move_ids = record.move_ids
-            record.move_type = move_ids[0].type if len(move_ids) == 1 or not any(m.type != move_ids[0].type for m in move_ids) else False
+            record.move_type = move_ids[0].move_type if len(move_ids) == 1 or not any(m.move_type != move_ids[0].move_type for m in move_ids) else False
             record.journal_type = record.move_type in ['in_refund', 'in_invoice'] and 'purchase' or 'sale'
 
     def _prepare_default_values(self, move):
-        if move.type in ('in_refund', 'out_refund'):
-            type = 'in_invoice' if move.type == 'in_refund' else 'out_invoice'
+        if move.move_type in ('in_refund', 'out_refund'):
+            type = 'in_invoice' if move.move_type == 'in_refund' else 'out_invoice'
         else:
-            type = move.type
+            type = move.move_type
         default_values = {
                 'ref': '%s, %s' % (move.name, self.reason) if self.reason else move.name,
                 'date': self.date or move.date,
@@ -54,9 +55,9 @@ class AccountDebitNote(models.TransientModel):
                 'journal_id': self.journal_id and self.journal_id.id or move.journal_id.id,
                 'invoice_payment_term_id': None,
                 'debit_origin_id': move.id,
-                'type': type,
+                'move_type': type,
             }
-        if not self.copy_lines or move.type in [('in_refund', 'out_refund')]:
+        if not self.copy_lines or move.move_type in [('in_refund', 'out_refund')]:
             default_values['line_ids'] = [(5, 0, 0)]
         return default_values
 
@@ -65,7 +66,7 @@ class AccountDebitNote(models.TransientModel):
         new_moves = self.env['account.move']
         for move in self.move_ids.with_context(include_business_fields=True): #copy sale/purchase links
             default_values = self._prepare_default_values(move)
-            new_move = move.with_context(internal_type='debit_note').copy(default=default_values) # Context key is used for l10n_latam_invoice_document for ar/cl/pe
+            new_move = move.copy(default=default_values)
             move_msg = _(
                 "This debit note was created from:") + " <a href=# data-oe-model=account.move data-oe-id=%d>%s</a>" % (
                        move.id, move.name)
@@ -76,8 +77,8 @@ class AccountDebitNote(models.TransientModel):
             'name': _('Debit Notes'),
             'type': 'ir.actions.act_window',
             'res_model': 'account.move',
-            'context': {'default_type': default_values['type']},
-            }
+            'context': {'default_move_type': default_values['move_type']},
+        }
         if len(new_moves) == 1:
             action.update({
                 'view_mode': 'form',
