@@ -5,6 +5,7 @@ import logging
 import lxml
 import os
 import sys
+import tempfile
 import zipfile
 from os.path import join as opj
 
@@ -13,7 +14,6 @@ from odoo import api, fields, models, _
 from odoo.exceptions import UserError
 from odoo.modules import load_information_from_description_file
 from odoo.tools import convert_file, exception_to_unicode
-from odoo.tools.osutil import tempdir
 
 _logger = logging.getLogger(__name__)
 
@@ -104,7 +104,52 @@ class IrModule(models.Model):
                     if attachment:
                         attachment.write(values)
                     else:
-                        IrAttachment.create(values)
+                        attachment = IrAttachment.create(values)
+                        self.env['ir.model.data'].create({
+                            'name': f"attachment_{url_path}".replace('.', '_'),
+                            'model': 'ir.attachment',
+                            'module': module,
+                            'res_id': attachment.id,
+                        })
+
+        IrAsset = self.env['ir.asset']
+        assets_vals = []
+
+        # Generate 'ir.asset' record values for each asset delared in the manifest
+        for bundle, commands in terp.get('assets', {}).items():
+            for command in commands:
+                directive, target, path = IrAsset._process_command(command)
+                path = path if path.startswith('/') else '/' + path # Ensures a '/' at the start
+                assets_vals.append({
+                    'name': f'{module}.{bundle}.{path}',
+                    'directive': directive,
+                    'target': target,
+                    'path': path,
+                    'bundle': bundle,
+                })
+
+        # Look for existing assets
+        existing_assets = {
+            asset.name: asset
+            for asset in IrAsset.search([('name', 'in', [vals['name'] for vals in assets_vals])])
+        }
+        assets_to_create = []
+
+        # Update existing assets and generate the list of new assets values
+        for values in assets_vals:
+            if values['name'] in existing_assets:
+                existing_assets[values['name']].write(values)
+            else:
+                assets_to_create.append(values)
+
+        # Create new assets and attach 'ir.model.data' records to them
+        created_assets = IrAsset.create(assets_to_create)
+        self.env['ir.model.data'].create([{
+            'name': f"{asset['bundle']}_{asset['path']}".replace(".", "_"),
+            'model': 'ir.asset',
+            'module': module,
+            'res_id': asset.id,
+        } for asset in created_assets])
 
         return True
 
@@ -121,9 +166,9 @@ class IrModule(models.Model):
         with zipfile.ZipFile(module_file, "r") as z:
             for zf in z.filelist:
                 if zf.file_size > MAX_FILE_SIZE:
-                    raise UserError(_("File '%s' exceed maximum allowed file size") % zf.filename)
+                    raise UserError(_("File '%s' exceed maximum allowed file size", zf.filename))
 
-            with tempdir() as module_dir:
+            with tempfile.TemporaryDirectory() as module_dir:
                 import odoo.modules.module as module
                 try:
                     odoo.addons.__path__.append(module_dir)
@@ -146,6 +191,29 @@ class IrModule(models.Model):
             r.append("Error while importing module '%s'.\n\n %s \n Make sure those modules are installed and try again." % (mod, error))
         return '\n'.join(r), module_names
 
+    def module_uninstall(self):
+        # Delete an ir_module_module record completely if it was an imported
+        # one. The rationale behind this is that an imported module *cannot* be
+        # reinstalled anyway, as it requires the data files. Any attempt to
+        # install it again will simply fail without trace.
+        # /!\ modules_to_delete must be calculated before calling super().module_uninstall(),
+        # because when uninstalling `base_import_module` the `imported` column will no longer be
+        # in the database but we'll still have an old registry that runs this code.
+        modules_to_delete = self.filtered('imported')
+        res = super().module_uninstall()
+        if modules_to_delete:
+            deleted_modules_names = modules_to_delete.mapped('name')
+            assets_data = self.env['ir.model.data'].search([
+                ('model', '=', 'ir.asset'),
+                ('module', 'in', deleted_modules_names),
+            ])
+            assets = self.env['ir.asset'].search([('id', 'in', assets_data.mapped('res_id'))])
+            assets.unlink()
+            _logger.info("deleting imported modules upon uninstallation: %s",
+                         ", ".join(deleted_modules_names))
+            modules_to_delete.unlink()
+        return res
+
 
 def _is_studio_custom(path):
     """
