@@ -1,4 +1,6 @@
-# -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
+
+import json
 
 from odoo import exceptions, _
 from odoo.http import Controller, request, route
@@ -6,23 +8,14 @@ from odoo.addons.bus.models.bus import dispatch
 
 
 class BusController(Controller):
-    """ Examples:
-    openerp.jsonRpc('/longpolling/poll','call',{"channels":["c1"],last:0}).then(function(r){console.log(r)});
-    openerp.jsonRpc('/longpolling/send','call',{"channel":"c1","message":"m1"});
-    openerp.jsonRpc('/longpolling/send','call',{"channel":"c2","message":"m2"});
-    """
-
-    @route('/longpolling/send', type="json", auth="public")
-    def send(self, channel, message):
-        if not isinstance(channel, str):
-            raise Exception("bus.Bus only string channels are allowed.")
-        return request.env['bus.bus'].sendone(channel, message)
 
     # override to add channels
     def _poll(self, dbname, channels, last, options):
+        channels = list(channels)  # do not alter original list
+        channels.append('broadcast')
         # update the user presence
         if request.session.uid and 'bus_inactivity' in options:
-            request.env['bus.presence'].update(options.get('bus_inactivity'))
+            request.env['bus.presence'].update(inactivity_period=options.get('bus_inactivity'), identity_field='user_id', identity_value=request.session.uid)
         request.cr.close()
         request._cr = None
         return dispatch.poll(dbname, channels, last, options)
@@ -41,4 +34,13 @@ class BusController(Controller):
 
     @route('/longpolling/im_status', type="json", auth="user")
     def im_status(self, partner_ids):
-        return request.env['res.partner'].search_read([['id', 'in', partner_ids]], ['id', 'im_status'])
+        return request.env['res.partner'].with_context(active_test=False).search([('id', 'in', partner_ids)]).read(['im_status'])
+
+    @route('/longpolling/health', type='http', auth='none', save_session=False)
+    def health(self):
+        data = json.dumps({
+            'status': 'pass',
+        })
+        headers = [('Content-Type', 'application/json'),
+                   ('Cache-Control', 'no-store')]
+        return request.make_response(data, headers)
