@@ -4,7 +4,7 @@
 {
     'name': 'Google Spreadsheet',
     'version': '1.0',
-    'category': 'Tools',
+    'category': 'Hidden/Tools',
     'description': """
 The module adds the possibility to display data from Odoo in Google Spreadsheets in real time.
 =================================================================================================
@@ -13,12 +13,18 @@ The module adds the possibility to display data from Odoo in Google Spreadsheets
     'data': [
         'data/google_spreadsheet_data.xml',
         'views/google_spreadsheet_views.xml',
-        'views/google_spreadsheet_templates.xml',
         'views/res_config_settings_views.xml',
     ],
-    'qweb': ['static/src/xml/*.xml'],
     'demo': [],
     'installable': True,
     'auto_install': False,
+    'assets': {
+        'web.assets_backend': [
+            'google_spreadsheet/static/src/**/*.js',
+        ],
+        'web.assets_qweb': [
+            'google_spreadsheet/static/src/**/*.xml',
+        ],
+    },
     'license': 'LGPL-3',
 }
