@@ -5,7 +5,7 @@
     'name': 'Germany - Accounting',
     'author': 'openbig.org',
     'website': 'http://www.openbig.org',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations',
     'description': """
 Dieses  Modul beinhaltet einen deutschen Kontenrahmen basierend auf dem SKR03.
 ==============================================================================
@@ -21,8 +21,14 @@ German accounting chart and localization.
         'data/account_account_tags_data.xml',
         'data/menuitem_data.xml',
         'views/account_view.xml',
+        'views/res_company_views.xml',
         'report/din5008_report.xml',
         'data/report_layout.xml',
     ],
+    'assets': {
+        'web.report_assets_common': [
+            'l10n_de/static/src/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
