@@ -3,9 +3,10 @@
 
 {
     'name': 'Indian - Stock Report(GST)',
+    'icon': '/l10n_in/static/description/icon.png',
     'version': '1.0',
     'description': """GST Stock Report""",
-    'category': 'Accounting/Accounting',
+    'category': 'Accounting/Localizations',
     'depends': [
         'l10n_in',
         'stock',
