@@ -3,22 +3,27 @@
 
 {
     'name': 'Italy - E-invoicing',
+    'icon': '/l10n_it/static/description/icon.png',
     'version': '0.3',
-    'depends': ['l10n_it'],
+    'depends': [
+        'l10n_it',
+        'fetchmail',
+        'account_edi'
+    ],
     'author': 'Odoo',
     'description': """
 E-invoice implementation
     """,
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/EDI',
     'website': 'http://www.odoo.com/',
     'data': [
         'security/ir.model.access.csv',
+        'data/account_edi_data.xml',
         'data/invoice_it_template.xml',
         'views/l10n_it_view.xml',
         ],
     'demo': [
         'data/account_invoice_demo.xml',
     ],
-    'auto_install': True,
     'license': 'LGPL-3',
 }
