@@ -9,7 +9,7 @@
     'version': '0.011',
     'author': 'ThinkOpen Solutions',
     'website': 'http://www.thinkopensolutions.com/',
-    'category': 'Localization',
+    'category': 'Accounting/Localizations/Account Charts',
     'description': 'Plano de contas SNC para Portugal',
     'depends': ['base',
                 'account',
@@ -19,7 +19,8 @@
            'data/l10n_pt_chart_data.xml',
            'data/account_chart_template_data.xml',
            'data/account_fiscal_position_template_data.xml',
-           'data/account_data.xml',
+           'data/account_tax_group_data.xml',
+           'data/account_tax_report.xml',
            'data/account_tax_data.xml',
            'data/account_chart_template_configure_data.xml',
            ],
