@@ -4,7 +4,7 @@ odoo.define('lunch.lunchKanbanTests', function (require) {
 const LunchKanbanView = require('lunch.LunchKanbanView');
 
 const testUtils = require('web.test_utils');
-const {createLunchKanbanView, mockLunchRPC} = require('lunch.test_utils');
+const {createLunchView, mockLunchRPC} = require('lunch.test_utils');
 
 QUnit.module('Views');
 
@@ -37,19 +37,14 @@ QUnit.module('LunchKanbanView', {
                 fields: {},
                 records: [],
             },
-            'ir.model.data': {
-                fields: {},
-                xmlid_to_res_id() {
-                    return Promise.resolve(PORTAL_GROUP_ID);
-                },
-            },
             'lunch.location': {
                 fields: {
                     name: {string: 'Name', type: 'char'},
+                    company_id: {string: 'Company', type: 'many2one', relation: 'res.company'},
                 },
                 records: [
-                    {id: 1, name: "Office 1"},
-                    {id: 2, name: "Office 2"},
+                    {id: 1, name: "Office 1", company_id: false},
+                    {id: 2, name: "Office 2", company_id: false},
                 ],
             },
             'res.users': {
@@ -63,11 +58,19 @@ QUnit.module('LunchKanbanView', {
                     {id: 3, name: "Jean-Luc Portal", groups_id: [PORTAL_GROUP_ID]},
                 ],
             },
+            'res.company': {
+                fields: {
+                    name: {string: 'Name', type: 'char'},
+                }, records: [
+                    {id: 1, name: "Dunder Trade Company"},
+                ]
+            }
         };
         this.regularInfos = {
             username: "Marc Demo",
             wallet: 36.5,
             is_manager: false,
+            group_portal_id: PORTAL_GROUP_ID,
             currency: {
                 symbol: "\u20ac",
                 position: "after"
@@ -78,6 +81,7 @@ QUnit.module('LunchKanbanView', {
             username: "Mitchell Admin",
             wallet: 47.6,
             is_manager: true,
+            group_portal_id: PORTAL_GROUP_ID,
             currency: {
                 symbol: "\u20ac",
                 position: "after"
@@ -89,7 +93,7 @@ QUnit.module('LunchKanbanView', {
     QUnit.test('basic rendering', async function (assert) {
         assert.expect(7);
 
-        const kanban = await createLunchKanbanView({
+        const kanban = await createLunchView({
             View: LunchKanbanView,
             model: 'product',
             data: this.data,
@@ -116,14 +120,14 @@ QUnit.module('LunchKanbanView', {
             "should have 2 columns");
         assert.containsOnce(kanban, '.o_content > div.o_search_panel',
             "should have a 'lunch filters' column");
-        assert.containsOnce(kanban, '.o_content > .o_lunch_kanban',
-            "should have a 'kanban lunch wrapper' column");
-        assert.containsOnce(kanban, '.o_lunch_kanban > .o_kanban_view',
+        assert.containsOnce(kanban, '.o_content > .o_lunch_content',
+            "should have a 'lunch wrapper' column");
+        assert.containsOnce(kanban, '.o_lunch_content > .o_kanban_view',
             "should have a 'classical kanban view' column");
         assert.hasClass(kanban.$('.o_kanban_view'), 'o_lunch_kanban_view',
             "should have classname 'o_lunch_kanban_view'");
-        assert.containsOnce(kanban, '.o_lunch_kanban > span > .o_lunch_kanban_banner',
-            "should have a 'lunch kanban' banner");
+        assert.containsOnce(kanban, '.o_lunch_content > span > .o_lunch_banner',
+            "should have a 'lunch' banner");
 
         kanban.destroy();
     });
@@ -133,7 +137,7 @@ QUnit.module('LunchKanbanView', {
 
         const self = this;
         let infosProm = Promise.resolve();
-        const kanban = await createLunchKanbanView({
+        const kanban = await createLunchView({
             View: LunchKanbanView,
             model: 'product',
             data: this.data,
@@ -175,12 +179,12 @@ QUnit.module('LunchKanbanView', {
         kanban.destroy();
     });
 
-    QUnit.module('LunchKanbanWidget', function () {
+    QUnit.module('LunchWidget', function () {
 
         QUnit.test('empty cart', async function (assert) {
             assert.expect(3);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -212,12 +216,12 @@ QUnit.module('LunchKanbanView', {
         });
 
         QUnit.test('search panel domain location', async function (assert) {
-            assert.expect(20);
+            assert.expect(18);
             let expectedLocation = 1;
             let locationId = this.data['lunch.location'].records[0].id;
             const regularInfos = _.extend({}, this.regularInfos);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -266,7 +270,6 @@ QUnit.module('LunchKanbanView', {
                 '/web/dataset/call_kw/product/search_panel_select_multi_range',
                 '/web/dataset/search_read',
                 '/lunch/infos',
-                '/web/dataset/call_kw/ir.model.data/xmlid_to_res_id',
                 // Click m2o
                 '/web/dataset/call_kw/lunch.location/name_search',
                 // Click new location
@@ -275,17 +278,16 @@ QUnit.module('LunchKanbanView', {
                 '/web/dataset/call_kw/product/search_panel_select_multi_range',
                 '/web/dataset/search_read',
                 '/lunch/infos',
-                '/web/dataset/call_kw/ir.model.data/xmlid_to_res_id',
             ]);
 
             kanban.destroy();
         });
 
         QUnit.test('search panel domain location false: fetch products in all locations', async function (assert) {
-            assert.expect(10);
+            assert.expect(9);
             const regularInfos = _.extend({}, this.regularInfos);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -324,7 +326,6 @@ QUnit.module('LunchKanbanView', {
                 '/web/dataset/call_kw/product/search_panel_select_multi_range',
                 '/web/dataset/search_read',
                 '/lunch/infos',
-                '/web/dataset/call_kw/ir.model.data/xmlid_to_res_id',
             ])
 
             kanban.destroy();
@@ -333,7 +334,7 @@ QUnit.module('LunchKanbanView', {
         QUnit.test('non-empty cart', async function (assert) {
             assert.expect(17);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -415,7 +416,7 @@ QUnit.module('LunchKanbanView', {
         QUnit.test('ordered cart', async function (assert) {
             assert.expect(15);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -491,7 +492,7 @@ QUnit.module('LunchKanbanView', {
         QUnit.test('confirmed cart', async function (assert) {
             assert.expect(15);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -567,7 +568,7 @@ QUnit.module('LunchKanbanView', {
         QUnit.test('regular user', async function (assert) {
             assert.expect(11);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -626,7 +627,7 @@ QUnit.module('LunchKanbanView', {
         QUnit.test('manager user', async function (assert) {
             assert.expect(12);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -690,7 +691,7 @@ QUnit.module('LunchKanbanView', {
         QUnit.test('add a product', async function (assert) {
             assert.expect(1);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -711,13 +712,12 @@ QUnit.module('LunchKanbanView', {
                     do_action: function (ev) {
                         assert.deepEqual(ev.data.action, {
                             name: "Configure Your Order",
-                            res_model: 'lunch.order.temp',
+                            res_model: 'lunch.order',
                             type: 'ir.actions.act_window',
                             views: [[false, 'form']],
                             target: 'new',
                             context: {
                                 default_product_id: 1,
-                                line_id: false,
                             },
                         },
                         "should open the wizard");
@@ -733,7 +733,7 @@ QUnit.module('LunchKanbanView', {
         QUnit.test('add product quantity', async function (assert) {
             assert.expect(3);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: Object.assign({}, this.data, {
@@ -785,7 +785,7 @@ QUnit.module('LunchKanbanView', {
         QUnit.test('remove product quantity', async function (assert) {
             assert.expect(3);
 
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: Object.assign({}, this.data, {
@@ -838,7 +838,7 @@ QUnit.module('LunchKanbanView', {
             assert.expect(1);
 
             const self = this;
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -884,7 +884,7 @@ QUnit.module('LunchKanbanView', {
             assert.expect(1);
 
             const self = this;
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
@@ -930,7 +930,7 @@ QUnit.module('LunchKanbanView', {
             assert.expect(5);
 
             const self = this;
-            const kanban = await createLunchKanbanView({
+            const kanban = await createLunchView({
                 View: LunchKanbanView,
                 model: 'product',
                 data: this.data,
