@@ -1,10 +1,11 @@
-odoo.define('mail.ActivityController', function (require) {
-"use strict";
+/** @odoo-module **/
 
-var BasicController = require('web.BasicController');
-var core = require('web.core');
-var field_registry = require('web.field_registry');
-var ViewDialogs = require('web.view_dialogs');
+import '@mail/js/activity';
+
+import BasicController from 'web.BasicController';
+import core from 'web.core';
+import field_registry from 'web.field_registry';
+import ViewDialogs from 'web.view_dialogs';
 
 var KanbanActivity = field_registry.get('kanban_activity');
 var _t = core._t;
@@ -20,13 +21,31 @@ var ActivityController = BasicController.extend({
     // Public
     //--------------------------------------------------------------------------
 
+    /**
+     * @override
+     * @param parent
+     * @param model
+     * @param renderer
+     * @param {Object} params
+     * @param {String} params.title The title used in schedule activity dialog
+     */
+    init: function (parent, model, renderer, params) {
+        this._super.apply(this, arguments);
+        this.title = params.title;
+        this.searchViewId = params.searchViewId;
+    },
+
+    //--------------------------------------------------------------------------
+    // Private
+    //--------------------------------------------------------------------------
+
     /**
      * Overridden to remove the pager as it makes no sense in this view.
      *
      * @override
      */
-    renderPager: function () {
-        return Promise.resolve();
+    _getPagingInfo: function () {
+        return null;
     },
 
     //--------------------------------------------------------------------------
@@ -38,17 +57,17 @@ var ActivityController = BasicController.extend({
      */
     _onScheduleActivity: function () {
         var self = this;
-
         var state = this.model.get(this.handle);
         new ViewDialogs.SelectCreateDialog(this, {
             res_model: state.model,
+            searchViewId: this.searchViewId,
             domain: this.model.originalDomain,
-            title: _.str.sprintf(_t("Search: %s"), this.renderer.arch.attrs.string),
+            title: _.str.sprintf(_t("Search: %s"), this.title),
             no_create: !this.activeActions.create,
             disable_multiple_selection: true,
             context: state.context,
             on_selected: function (record) {
-                var fakeRecord = self.renderer.getKanbanActivityData({}, record[0]);
+                var fakeRecord = state.getKanbanActivityData({}, record[0]);
                 var widget = new KanbanActivity(self, 'activity_ids', fakeRecord, {});
                 widget.scheduleActivity();
             },
@@ -79,7 +98,7 @@ var ActivityController = BasicController.extend({
     },
     /**
      * @private
-     * @param {OdooEvent} ev
+     * @param {CustomEvent} ev
      */
     _onSendMailTemplate: function (ev) {
         var templateID = ev.data.templateID;
@@ -102,6 +121,4 @@ var ActivityController = BasicController.extend({
     },
 });
 
-return ActivityController;
-
-});
+export default ActivityController;
