@@ -1,7 +1,6 @@
-odoo.define('mail.mail_utils_tests', function (require) {
-"use strict";
+/** @odoo-module **/
 
-var utils = require('mail.utils');
+import * as utils from '@mail/js/utils';
 
 QUnit.module('mail', {}, function () {
 
@@ -35,7 +34,7 @@ QUnit.test('add_link utility function', function (assert) {
     });
 });
 
-QUnit.test('addLink: linkify inside text node (1 occurence)', function (assert) {
+QUnit.test('addLink: linkify inside text node (1 occurrence)', function (assert) {
     assert.expect(5);
 
     const content = '<p>some text https://somelink.com</p>';
@@ -73,7 +72,7 @@ QUnit.test('addLink: linkify inside text node (1 occurence)', function (assert)
     );
 });
 
-QUnit.test('addLink: linkify inside text node (2 occurences)', function (assert) {
+QUnit.test('addLink: linkify inside text node (2 occurrences)', function (assert) {
     assert.expect(4);
 
     // linkify may add some attributes. Since we do not care of their exact
@@ -108,4 +107,3 @@ QUnit.test('addLink: linkify inside text node (2 occurences)', function (assert)
 });
 
 });
-});
