@@ -1,25 +1,30 @@
-odoo.define('mail.systray.ActivityMenuTests', function (require) {
-"use strict";
+/** @odoo-module **/
 
-var ActivityMenu = require('mail.systray.ActivityMenu');
-var mailTestUtils = require('mail.testUtils');
+import ActivityMenu from '@mail/js/systray/systray_activity_menu';
+import {
+    afterEach,
+    afterNextRender,
+    beforeEach,
+    start,
+} from '@mail/utils/test_utils';
 
-var testUtils = require('web.test_utils');
+import testUtils from 'web.test_utils';
 
 QUnit.module('mail', {}, function () {
 QUnit.module('ActivityMenu', {
-    beforeEach: function () {
-        this.services = mailTestUtils.getMailServices();
-        this.data = {
+    beforeEach() {
+        beforeEach(this);
+
+        Object.assign(this.data, {
             'mail.activity.menu': {
                 fields: {
                     name: { type: "char" },
                     model: { type: "char" },
                     type: { type: "char" },
-                    planned_count: { type: "integer"},
-                    today_count: { type: "integer"},
-                    overdue_count: { type: "integer"},
-                    total_count: { type: "integer"},
+                    planned_count: { type: "integer" },
+                    today_count: { type: "integer" },
+                    overdue_count: { type: "integer" },
+                    total_count: { type: "integer" },
                     actions: [{
                         icon: { type: "char" },
                         name: { type: "char" },
@@ -52,7 +57,7 @@ QUnit.module('ActivityMenu', {
                         today_count: 1,
                         overdue_count: 1,
                         total_count: 3,
-                        actions : [{
+                        actions: [{
                             icon: "fa-clock-o",
                             name: "summary",
                         }],
@@ -70,40 +75,44 @@ QUnit.module('ActivityMenu', {
                             name: "summary",
                             action_xmlid: "mail.mail_activity_type_view_tree",
                         }],
-                    }],
-                },
-            };
-            this.session = {
-                uid: 10,
-            };
-        }
-    });
+                    }
+                ],
+            },
+        });
+        this.session = {
+            uid: 10,
+        };
+    },
+    afterEach() {
+        afterEach(this);
+    },
+});
 
 QUnit.test('activity menu widget: menu with no records', async function (assert) {
     assert.expect(1);
 
-    var activityMenu = new ActivityMenu();
-    testUtils.mock.addMockEnvironment(activityMenu, {
-            services: this.services,
-            mockRPC: function (route, args) {
-                if (args.method === 'systray_get_activities') {
-                    return Promise.resolve([]);
-                }
-                return this._super(route, args);
-            },
-        });
+    const { widget } = await start({
+        data: this.data,
+        mockRPC: function (route, args) {
+            if (args.method === 'systray_get_activities') {
+                return Promise.resolve([]);
+            }
+            return this._super(route, args);
+        },
+    });
+    const activityMenu = new ActivityMenu(widget);
     await activityMenu.appendTo($('#qunit-fixture'));
     await testUtils.nextTick();
     assert.containsOnce(activityMenu, '.o_no_activity');
-    activityMenu.destroy();
+    widget.destroy();
 });
 
 QUnit.test('activity menu widget: activity menu with 3 records', async function (assert) {
     assert.expect(10);
     var self = this;
-    var activityMenu = new ActivityMenu();
-    testUtils.mock.addMockEnvironment(activityMenu, {
-        services: this.services,
+
+    const { widget } = await start({
+        data: this.data,
         mockRPC: function (route, args) {
             if (args.method === 'systray_get_activities') {
                 return Promise.resolve(self.data['mail.activity.menu']['records']);
@@ -111,6 +120,7 @@ QUnit.test('activity menu widget: activity menu with 3 records', async function
             return this._super(route, args);
         },
     });
+    var activityMenu = new ActivityMenu(widget);
     await activityMenu.appendTo($('#qunit-fixture'));
     await testUtils.nextTick();
     assert.hasClass(activityMenu.$el, 'o_mail_systray_item', 'should be the instance of widget');
@@ -156,23 +166,24 @@ QUnit.test('activity menu widget: activity menu with 3 records', async function
     await testUtils.dom.click(activityMenu.$('.dropdown-toggle'));
     await testUtils.dom.click(activityMenu.$(".o_mail_systray_dropdown_items > div[data-model_name='Issue']"));
 
-    activityMenu.destroy();
+    widget.destroy();
 });
 
 QUnit.test('activity menu widget: activity view icon', async function (assert) {
     assert.expect(12);
     var self = this;
-    var activityMenu = new ActivityMenu();
-    testUtils.mock.addMockEnvironment(activityMenu, {
-        services: this.services,
-        session: this.session,
+
+    const { widget } = await start({
+        data: this.data,
         mockRPC: function (route, args) {
             if (args.method === 'systray_get_activities') {
                 return Promise.resolve(self.data['mail.activity.menu'].records);
             }
             return this._super(route, args);
         },
+        session: this.session,
     });
+    var activityMenu = new ActivityMenu(widget);
     await activityMenu.appendTo($('#qunit-fixture'));
     await testUtils.nextTick();
     assert.containsN(activityMenu, '.o_mail_activity_action', 2,
@@ -182,17 +193,17 @@ QUnit.test('activity menu widget: activity view icon', async function (assert) {
     var $second = activityMenu.$('.o_mail_activity_action').eq(1);
     assert.strictEqual($first.data('model_name'), "Issue",
                        "first activity action should link to 'Issue'");
-    assert.hasClass($first,'fa-clock-o', "should display the activity action icon");
+    assert.hasClass($first, 'fa-clock-o', "should display the activity action icon");
 
     assert.strictEqual($second.data('model_name'), "Note",
                        "Second activity action should link to 'Note'");
-    assert.hasClass($second,'fa-clock-o', "should display the activity action icon");
+    assert.hasClass($second, 'fa-clock-o', "should display the activity action icon");
 
     testUtils.mock.intercept(activityMenu, 'do_action', function (ev) {
         if (ev.data.action.name) {
             assert.ok(ev.data.action.domain, "should define a domain on the action");
             assert.deepEqual(ev.data.action.domain, [["activity_ids.user_id", "=", 10]],
-                "should set domain to user's activity only")
+                "should set domain to user's activity only");
             assert.step('do_action:' + ev.data.action.name);
         } else {
             assert.step('do_action:' + ev.data.action);
@@ -217,7 +228,43 @@ QUnit.test('activity menu widget: activity view icon', async function (assert) {
         'do_action:mail.mail_activity_type_view_tree'
     ]);
 
-    activityMenu.destroy();
+    widget.destroy();
 });
+
+QUnit.test('activity menu widget: close on messaging menu click', async function (assert) {
+    assert.expect(2);
+
+    const { createMessagingMenuComponent, widget } = await start({
+        data: this.data,
+        async mockRPC(route, args) {
+            if (args.method === 'systray_get_activities') {
+                return [];
+            }
+            return this._super(route, args);
+        },
+    });
+    await createMessagingMenuComponent();
+    const activityMenu = new ActivityMenu(widget);
+    await activityMenu.appendTo($('#qunit-fixture'));
+    await testUtils.nextTick();
+
+    await testUtils.dom.click(activityMenu.$('.dropdown-toggle'));
+    assert.hasClass(
+        activityMenu.el.querySelector('.o_mail_systray_dropdown'),
+        'show',
+        "activity menu should be shown after click on itself"
+    );
+
+    await afterNextRender(() =>
+        document.querySelector(`.o_MessagingMenu_toggler`).click()
+    );
+    assert.doesNotHaveClass(
+        activityMenu.el.querySelector('.o_mail_systray_dropdown'),
+        'show',
+        "activity menu should be hidden after click on messaging menu"
+    );
+
+    widget.destroy();
 });
+
 });
