@@ -7,6 +7,7 @@ import werkzeug
 from odoo import _, exceptions, http, tools
 from odoo.http import request
 from odoo.tools import consteq
+from werkzeug.exceptions import BadRequest
 
 
 class MassMailController(http.Controller):
@@ -91,16 +92,48 @@ class MassMailController(http.Controller):
             return True
         return 'error'
 
-    @http.route('/mail/track/<int:mail_id>/blank.gif', type='http', auth='public')
-    def track_mail_open(self, mail_id, **post):
+    @http.route('/mail/track/<int:mail_id>/<string:token>/blank.gif', type='http', auth='public')
+    def track_mail_open(self, mail_id, token, **post):
         """ Email tracking. """
-        request.env['mailing.trace'].sudo().set_opened(mail_mail_ids=[mail_id])
+        if not consteq(token, tools.hmac(request.env(su=True), 'mass_mailing-mail_mail-open', mail_id)):
+            raise BadRequest()
+
+        request.env['mailing.trace'].sudo().set_opened(domain=[('mail_mail_id_int', 'in', [mail_id])])
         response = werkzeug.wrappers.Response()
         response.mimetype = 'image/gif'
         response.data = base64.b64decode(b'R0lGODlhAQABAIAAANvf7wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==')
 
         return response
 
+    @http.route(['/mailing/<int:mailing_id>/view'], type='http', website=True, auth='public')
+    def view(self, mailing_id, email=None, res_id=None, token=""):
+        mailing = request.env['mailing.mailing'].sudo().browse(mailing_id)
+        if mailing.exists():
+            res_id = int(res_id) if res_id else False
+            if not self._valid_unsubscribe_token(mailing_id, res_id, email, str(token)) and not request.env.user.has_group('mass_mailing.group_mass_mailing_user'):
+                raise exceptions.AccessDenied()
+
+            res = mailing.convert_links()
+            base_url = mailing.get_base_url().rstrip('/')
+            urls_to_replace = [
+                (base_url + '/unsubscribe_from_list', mailing._get_unsubscribe_url(email, res_id)),
+                (base_url + '/view', mailing._get_view_url(email, res_id))
+            ]
+            for url_to_replace, new_url in urls_to_replace:
+                if url_to_replace in res[mailing_id]:
+                    res[mailing_id] = res[mailing_id].replace(url_to_replace, new_url if new_url else '#')
+
+            res[mailing_id] = res[mailing_id].replace(
+                'class="o_snippet_view_in_browser"',
+                'class="o_snippet_view_in_browser" style="display: none;"'
+            )
+
+            return request.render('mass_mailing.view', {
+                    'body': res[mailing_id],
+                })
+
+        return request.redirect('/web')
+
     @http.route('/r/<string:code>/m/<int:mailing_trace_id>', type='http', auth="public")
     def full_url_redirect(self, code, mailing_trace_id, **post):
         # don't assume geoip is set, it is part of the website module
@@ -113,7 +146,7 @@ class MassMailController(http.Controller):
             country_code=country_code,
             mailing_trace_id=mailing_trace_id
         )
-        return werkzeug.utils.redirect(request.env['link.tracker'].get_url_from_code(code), 301)
+        return request.redirect(request.env['link.tracker'].get_url_from_code(code), code=301, local=False)
 
     @http.route('/mailing/blacklist/check', type='json', auth='public')
     def blacklist_check(self, mailing_id, res_id, email, token):
@@ -159,6 +192,6 @@ class MassMailController(http.Controller):
             model = request.env[mailing.mailing_model_real]
             records = model.sudo().search([('email_normalized', '=', tools.email_normalize(email))])
             for record in records:
-                record.sudo().message_post(body=_("Feedback from %s: %s" % (email, feedback)))
+                record.sudo().message_post(body=_("Feedback from %(email)s: %(feedback)s", email=email, feedback=feedback))
             return bool(records)
         return 'error'
