@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import exceptions
+from odoo import exceptions, Command
 from odoo.tests import Form
 from odoo.addons.mrp.tests.common import TestMrpCommon
-from odoo.tools import float_compare, float_round
+from odoo.tools import float_compare, float_round, float_repr
 
 
 class TestBoM(TestMrpCommon):
@@ -22,30 +22,69 @@ class TestBoM(TestMrpCommon):
 
     def test_10_variants(self):
         test_bom = self.env['mrp.bom'].create({
-            'product_id': self.product_7_3.id,
             'product_tmpl_id': self.product_7_template.id,
             'product_uom_id': self.uom_unit.id,
             'product_qty': 4.0,
-            'routing_id': self.routing_2.id,
             'type': 'normal',
+            'operation_ids': [
+                Command.create({
+                    'name': 'Cutting Machine',
+                    'workcenter_id': self.workcenter_1.id,
+                    'time_cycle': 12,
+                    'sequence': 1
+                }),
+                Command.create({
+                    'name': 'Weld Machine',
+                    'workcenter_id': self.workcenter_1.id,
+                    'time_cycle': 18,
+                    'sequence': 2,
+                    'bom_product_template_attribute_value_ids': [Command.link(self.product_7_attr1_v1.id)]
+                }),
+                Command.create({
+                    'name': 'Taking a coffee',
+                    'workcenter_id': self.workcenter_1.id,
+                    'time_cycle': 5,
+                    'sequence': 3,
+                    'bom_product_template_attribute_value_ids': [Command.link(self.product_7_attr1_v2.id)]
+                })
+            ],
+            'byproduct_ids': [
+                Command.create({
+                    'product_id': self.product_1.id,
+                    'product_uom_id': self.product_1.uom_id.id,
+                    'product_qty': 1,
+                }),
+                Command.create({
+                    'product_id': self.product_2.id,
+                    'product_uom_id': self.product_2.uom_id.id,
+                    'product_qty': 1,
+                    'bom_product_template_attribute_value_ids': [Command.link(self.product_7_attr1_v1.id)]
+                }),
+                Command.create({
+                    'product_id': self.product_3.id,
+                    'product_uom_id': self.product_3.uom_id.id,
+                    'product_qty': 1,
+                    'bom_product_template_attribute_value_ids': [Command.link(self.product_7_attr1_v2.id)]
+                }),
+            ],
+            'bom_line_ids': [
+                Command.create({
+                    'product_id': self.product_2.id,
+                    'product_qty': 2,
+                }),
+                Command.create({
+                    'product_id': self.product_3.id,
+                    'product_qty': 2,
+                    'bom_product_template_attribute_value_ids': [Command.link(self.product_7_attr1_v1.id)],
+                }),
+                Command.create({
+                    'product_id': self.product_4.id,
+                    'product_qty': 2,
+                    'bom_product_template_attribute_value_ids': [Command.link(self.product_7_attr1_v2.id)],
+                }),
+            ]
         })
-        test_bom_l1 = self.env['mrp.bom.line'].create({
-            'bom_id': test_bom.id,
-            'product_id': self.product_2.id,
-            'product_qty': 2,
-        })
-        test_bom_l2 = self.env['mrp.bom.line'].create({
-            'bom_id': test_bom.id,
-            'product_id': self.product_3.id,
-            'product_qty': 2,
-            'bom_product_template_attribute_value_ids': [(4, self.product_7_attr1_v1.id)],
-        })
-        test_bom_l3 = self.env['mrp.bom.line'].create({
-            'bom_id': test_bom.id,
-            'product_id': self.product_4.id,
-            'product_qty': 2,
-            'bom_product_template_attribute_value_ids': [(4, self.product_7_attr1_v2.id)],
-        })
+        test_bom_l1, test_bom_l2, test_bom_l3 = test_bom.bom_line_ids
         boms, lines = test_bom.explode(self.product_7_3, 4)
         self.assertIn(test_bom, [b[0]for b in boms])
         self.assertIn(test_bom_l1, [l[0] for l in lines])
@@ -64,6 +103,37 @@ class TestBoM(TestMrpCommon):
         self.assertNotIn(test_bom_l2, [l[0] for l in lines])
         self.assertIn(test_bom_l3, [l[0] for l in lines])
 
+        mrp_order_form = Form(self.env['mrp.production'])
+        mrp_order_form.product_id = self.product_7_3
+        mrp_order = mrp_order_form.save()
+        self.assertEqual(mrp_order.bom_id, test_bom)
+        self.assertEqual(len(mrp_order.workorder_ids), 1)
+        self.assertEqual(mrp_order.workorder_ids.operation_id, test_bom.operation_ids[0])
+        self.assertEqual(len(mrp_order.move_byproduct_ids), 1)
+        self.assertEqual(mrp_order.move_byproduct_ids.product_id, self.product_1)
+
+        mrp_order_form = Form(self.env['mrp.production'])
+        mrp_order_form.product_id = self.product_7_1
+        mrp_order_form.product_id = self.env['product.product']  # Check form
+        mrp_order_form.product_id = self.product_7_1
+        mrp_order_form.bom_id = self.env['mrp.bom']  # Check form
+        mrp_order_form.bom_id = test_bom
+        mrp_order = mrp_order_form.save()
+        self.assertEqual(mrp_order.bom_id, test_bom)
+        self.assertEqual(len(mrp_order.workorder_ids), 2)
+        self.assertEqual(mrp_order.workorder_ids.operation_id, test_bom.operation_ids[:2])
+        self.assertEqual(len(mrp_order.move_byproduct_ids), 2)
+        self.assertEqual(mrp_order.move_byproduct_ids.product_id, self.product_1 | self.product_2)
+
+        mrp_order_form = Form(self.env['mrp.production'])
+        mrp_order_form.product_id = self.product_7_2
+        mrp_order = mrp_order_form.save()
+        self.assertEqual(mrp_order.bom_id, test_bom)
+        self.assertEqual(len(mrp_order.workorder_ids), 2)
+        self.assertEqual(mrp_order.workorder_ids.operation_id, test_bom.operation_ids[0] | test_bom.operation_ids[2])
+        self.assertEqual(len(mrp_order.move_byproduct_ids), 2)
+        self.assertEqual(mrp_order.move_byproduct_ids.product_id, self.product_1 | self.product_3)
+
     def test_11_multi_level_variants(self):
         tmp_picking_type = self.env['stock.picking.type'].create({
             'name': 'Manufacturing',
@@ -78,9 +148,13 @@ class TestBoM(TestMrpCommon):
             'product_tmpl_id': self.product_5.product_tmpl_id.id,
             'product_uom_id': self.product_5.uom_id.id,
             'product_qty': 1.0,
-            'routing_id': self.routing_1.id,
             'type': 'phantom'
         })
+        test_bom_1.write({
+            'operation_ids': [
+                (0, 0, {'name': 'Gift Wrap Maching', 'workcenter_id': self.workcenter_1.id, 'time_cycle': 15, 'sequence': 1}),
+            ],
+        })
         test_bom_1_l1 = self.env['mrp.bom.line'].create({
             'bom_id': test_bom_1.id,
             'product_id': self.product_3.id,
@@ -92,9 +166,14 @@ class TestBoM(TestMrpCommon):
             'product_tmpl_id': self.product_7_template.id,
             'product_uom_id': self.uom_unit.id,
             'product_qty': 4.0,
-            'routing_id': self.routing_2.id,
             'type': 'normal',
         })
+        test_bom_2.write({
+            'operation_ids': [
+                (0, 0, {'name': 'Cutting Machine', 'workcenter_id': self.workcenter_1.id, 'time_cycle': 12, 'sequence': 1}),
+                (0, 0, {'name': 'Weld Machine', 'workcenter_id': self.workcenter_1.id, 'time_cycle': 18, 'sequence': 2}),
+            ]
+        })
         test_bom_2_l1 = self.env['mrp.bom.line'].create({
             'bom_id': test_bom_2.id,
             'product_id': self.product_2.id,
@@ -143,6 +222,7 @@ class TestBoM(TestMrpCommon):
             'product_tmpl_id': self.product_9.product_tmpl_id.id,
             'product_uom_id': self.product_9.uom_id.id,
             'product_qty': 1.0,
+            'consumption': 'flexible',
             'type': 'normal'
         })
         test_bom_4 = self.env['mrp.bom'].create({
@@ -150,6 +230,7 @@ class TestBoM(TestMrpCommon):
             'product_tmpl_id': self.product_10.product_tmpl_id.id,
             'product_uom_id': self.product_10.uom_id.id,
             'product_qty': 1.0,
+            'consumption': 'flexible',
             'type': 'phantom'
         })
         test_bom_3_l1 = self.env['mrp.bom.line'].create({
@@ -246,6 +327,104 @@ class TestBoM(TestMrpCommon):
             # Check consumed materials in production order.
             self.assertEqual(mrp_order.move_raw_ids.product_id, consumed_products)
 
+    def test_13_bom_kit_qty(self):
+        self.env['mrp.bom'].create({
+            'product_id': self.product_7_3.id,
+            'product_tmpl_id': self.product_7_template.id,
+            'product_uom_id': self.uom_unit.id,
+            'product_qty': 4.0,
+            'type': 'phantom',
+            'bom_line_ids': [
+                (0, 0, {
+                    'product_id': self.product_2.id,
+                    'product_qty': 2,
+                }),
+                (0, 0, {
+                    'product_id': self.product_3.id,
+                    'product_qty': 2,
+                })
+            ]
+        })
+        location = self.env.ref('stock.stock_location_stock')
+        self.env['stock.quant']._update_available_quantity(self.product_2, location, 4.0)
+        self.env['stock.quant']._update_available_quantity(self.product_3, location, 8.0)
+        # Force the kit product available qty to be computed at the same time than its component quantities
+        # Because `qty_available` of a bom kit "recurse" on `qty_available` of its component,
+        # and this is a tricky thing for the ORM:
+        # `qty_available` gets called for `product_7_3`, `product_2` and `product_3`
+        # which then recurse on calling `qty_available` for `product_2` and `product_3` to compute the quantity of
+        # the kit `product_7_3`. `product_2` and `product_3` gets protected at the first call of the compute method,
+        # ending the recurse call to not call the compute method and just left the Falsy value `0.0`
+        # for the components available qty.
+        kit_product_qty, _, _ = (self.product_7_3 + self.product_2 + self.product_3).mapped("qty_available")
+        self.assertEqual(kit_product_qty, 2)
+
+    def test_14_bom_kit_qty_multi_uom(self):
+        uom_dozens = self.env.ref('uom.product_uom_dozen')
+        uom_unit = self.env.ref('uom.product_uom_unit')
+        product_unit = self.env['product.product'].create({
+            'name': 'Test units',
+            'type': 'product',
+            'uom_id': uom_unit.id,
+        })
+        product_dozens = self.env['product.product'].create({
+            'name': 'Test dozens',
+            'type': 'product',
+            'uom_id': uom_dozens.id,
+        })
+
+        self.env['mrp.bom'].create({
+            'product_tmpl_id': product_unit.product_tmpl_id.id,
+            'product_uom_id': self.uom_unit.id,
+            'product_qty': 1.0,
+            'type': 'phantom',
+            'bom_line_ids': [
+                (0, 0, {
+                    'product_id': product_dozens.id,
+                    'product_qty': 1,
+                    'product_uom_id': uom_unit.id,
+                })
+            ]
+        })
+        location = self.env.ref('stock.stock_location_stock')
+        self.env['stock.quant']._update_available_quantity(product_dozens, location, 1.0)
+        self.assertEqual(product_unit.qty_available, 12.0)
+
+    def test_13_negative_on_hand_qty(self):
+        # We set the Product Unit of Measure digits to 5.
+        # Because float_round(-384.0, 5) = -384.00000000000006
+        # And float_round(-384.0, 2) = -384.0
+        precision = self.env.ref('product.decimal_product_uom')
+        precision.digits = 5
+
+        # We set the Unit(s) rounding to 0.0001 (digit = 4)
+        uom_unit = self.env.ref('uom.product_uom_unit')
+        uom_unit.rounding = 0.0001
+
+        _ = self.env['mrp.bom'].create({
+            'product_id': self.product_2.id,
+            'product_tmpl_id': self.product_2.product_tmpl_id.id,
+            'product_uom_id': uom_unit.id,
+            'product_qty': 1.00,
+            'type': 'phantom',
+            'bom_line_ids': [
+                (0, 0, {
+                    'product_id': self.product_3.id,
+                    'product_qty': 1.000,
+                }),
+            ]
+        })
+
+        self.env['stock.quant']._update_available_quantity(self.product_3, self.env.ref('stock.stock_location_stock'), -384.0)
+
+        kit_product_qty = self.product_2.qty_available  # Without product_3 in the prefetch
+        # Use the float_repr to remove extra small decimal (and represent the front-end behavior)
+        self.assertEqual(float_repr(float_round(kit_product_qty, precision_digits=precision.digits), precision_digits=precision.digits), '-384.00000')
+
+        self.product_2.invalidate_cache(fnames=['qty_available'], ids=self.product_2.ids)
+        kit_product_qty, _ = (self.product_2 + self.product_3).mapped("qty_available")  # With product_3 in the prefetch
+        self.assertEqual(float_repr(float_round(kit_product_qty, precision_digits=precision.digits), precision_digits=precision.digits), '-384.00000')
+
     def test_20_bom_report(self):
         """ Simulate a crumble receipt with mrp and open the bom structure
         report and check that data insde are correct.
@@ -278,6 +457,11 @@ class TestBoM(TestMrpCommon):
         bom_form_crumble.product_uom_id = uom_kg
         bom_crumble = bom_form_crumble.save()
 
+        workcenter = self.env['mrp.workcenter'].create({
+            'costs_hour': 10,
+            'name': 'Deserts Table'
+        })
+
         with Form(bom_crumble) as bom:
             with bom.bom_line_ids.new() as line:
                 line.product_id = butter
@@ -287,39 +471,28 @@ class TestBoM(TestMrpCommon):
                 line.product_id = biscuit
                 line.product_uom_id = uom_kg
                 line.product_qty = 6
-
-        workcenter = self.env['mrp.workcenter'].create({
-            'costs_hour': 10,
-            'name': 'Deserts Table'
-        })
-
-        routing_form = Form(self.env['mrp.routing'])
-        routing_form.name = "Crumble process"
-        routing_crumble = routing_form.save()
-
-        with Form(routing_crumble) as routing:
-            with routing.operation_ids.new() as operation:
+            with bom.operation_ids.new() as operation:
                 operation.workcenter_id = workcenter
                 operation.name = 'Prepare biscuits'
                 operation.time_cycle_manual = 5
-            with routing.operation_ids.new() as operation:
+                operation.bom_id = bom_crumble  # Can't handle by the testing env
+            with bom.operation_ids.new() as operation:
                 operation.workcenter_id = workcenter
                 operation.name = 'Prepare butter'
                 operation.time_cycle_manual = 3
-            with routing.operation_ids.new() as operation:
+                operation.bom_id = bom_crumble
+            with bom.operation_ids.new() as operation:
                 operation.workcenter_id = workcenter
                 operation.name = 'Mix manually'
                 operation.time_cycle_manual = 5
-
-        bom_crumble.routing_id = routing_crumble.id
+                operation.bom_id = bom_crumble
 
         # TEST BOM STRUCTURE VALUE WITH BOM QUANTITY
         report_values = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_crumble.id, searchQty=11, searchVariant=False)
-        # 5 min 'Prepare biscuits' + 3 min 'Prepare butter' + 5 min 'Mix manually' = 13 minutes
-        self.assertEqual(report_values['lines']['operations_time'], 13.0, 'Operation time should be the same for 1 unit or for the batch')
+        # 5 min 'Prepare biscuits' + 3 min 'Prepare butter' + 5 min 'Mix manually' = 13 minutes for 1 biscuits so 13 * 11 = 143 minutes
+        self.assertEqual(report_values['lines']['operations_time'], 143.0, 'Operation time should be the same for 1 unit or for the batch')
         # Operation cost is the sum of operation line.
-        operation_cost = float_round(5 / 60 * 10, precision_digits=2) * 2 + float_round(3 / 60 * 10, precision_digits=2)
-        self.assertEqual(float_compare(report_values['lines']['operations_cost'], operation_cost, precision_digits=2), 0, '13 minute for 10$/hours -> 2.16')
+        self.assertEqual(float_compare(report_values['lines']['operations_cost'], 23.84, precision_digits=2), 0, '143 minute for 10$/hours -> 23.84')
 
         for component_line in report_values['lines']['components']:
             # standard price * bom line quantity * current quantity / bom finished product quantity
@@ -329,9 +502,9 @@ class TestBoM(TestMrpCommon):
             if component_line['prod_id'] == biscuit.id:
                 # 6 kg of biscuits at 1.50$ for 11kg of crumble -> 9$
                 self.assertEqual(float_compare(component_line['total'], (1.5 * 6), precision_digits=2), 0)
-        # total price = 35.05 + 9 + operation_cost(0.83 + 0.83 + 0.5 = 2.16) = 46,21
-        self.assertEqual(float_compare(report_values['lines']['total'], 46.21, precision_digits=2), 0, 'Product Bom Price is not correct')
-        self.assertEqual(float_compare(report_values['lines']['total'] / 11.0, 4.20, precision_digits=2), 0, 'Product Unit Bom Price is not correct')
+        # total price = 35.05 + 9 + operation_cost(23.84) = 67.89
+        self.assertEqual(float_compare(report_values['lines']['total'], 67.89, precision_digits=2), 0, 'Product Bom Price is not correct')
+        self.assertEqual(float_compare(report_values['lines']['total'] / 11.0, 6.17, precision_digits=2), 0, 'Product Unit Bom Price is not correct')
 
         # TEST BOM STRUCTURE VALUE BY UNIT
         report_values = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_crumble.id, searchQty=1, searchVariant=False)
@@ -355,13 +528,9 @@ class TestBoM(TestMrpCommon):
         # TEST OPERATION COST WHEN PRODUCED QTY > BOM QUANTITY
         report_values_12 = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_crumble.id, searchQty=12, searchVariant=False)
         report_values_22 = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_crumble.id, searchQty=22, searchVariant=False)
-        operation_cost = float_round(10 / 60 * 10, precision_digits=2) * 2 + float_round(6 / 60 * 10, precision_digits=2)
-        # Both needs 2 operation cycle
-        self.assertEqual(report_values_12['lines']['operations_cost'], report_values_22['lines']['operations_cost'])
-        self.assertEqual(report_values_22['lines']['operations_cost'], operation_cost)
-        report_values_23 = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_crumble.id, searchQty=23, searchVariant=False)
-        operation_cost = float_round(15 / 60 * 10, precision_digits=2) * 2 + float_round(9 / 60 * 10, precision_digits=2)
-        self.assertEqual(report_values_23['lines']['operations_cost'], operation_cost)
+
+        #Operation cost = 47.66 € = 256 (min) * 10€/h
+        self.assertEqual(float_compare(report_values_22['lines']['operations_cost'], 47.66,precision_digits=2),0, 'Operation cost is not correct')
 
         # Create a more complex BoM with a sub product
         cheese_cake = self.env['product.product'].create({
@@ -381,6 +550,13 @@ class TestBoM(TestMrpCommon):
         bom_form_cheese_cake.product_uom_id = self.uom_unit
         bom_cheese_cake = bom_form_cheese_cake.save()
 
+        workcenter_2 = self.env['mrp.workcenter'].create({
+            'name': 'cake mounting',
+            'costs_hour': 20,
+            'time_start': 10,
+            'time_stop': 15
+        })
+
         with Form(bom_cheese_cake) as bom:
             with bom.bom_line_ids.new() as line:
                 line.product_id = cream
@@ -390,36 +566,23 @@ class TestBoM(TestMrpCommon):
                 line.product_id = crumble
                 line.product_uom_id = uom_kg
                 line.product_qty = 5.4
-
-        workcenter_2 = self.env['mrp.workcenter'].create({
-            'name': 'cake mounting',
-            'costs_hour': 20,
-            'time_start': 10,
-            'time_stop': 15
-        })
-
-        routing_form = Form(self.env['mrp.routing'])
-        routing_form.name = "Cheese cake process"
-        routing_cheese = routing_form.save()
-
-        with Form(routing_cheese) as routing:
-            with routing.operation_ids.new() as operation:
+            with bom.operation_ids.new() as operation:
                 operation.workcenter_id = workcenter
                 operation.name = 'Mix cheese and crumble'
                 operation.time_cycle_manual = 10
-            with routing.operation_ids.new() as operation:
+                operation.bom_id = bom_cheese_cake
+            with bom.operation_ids.new() as operation:
                 operation.workcenter_id = workcenter_2
                 operation.name = 'Cake mounting'
                 operation.time_cycle_manual = 5
-
-        bom_cheese_cake.routing_id = routing_cheese.id
+                operation.bom_id = bom_cheese_cake
 
         # TEST CHEESE BOM STRUCTURE VALUE WITH BOM QUANTITY
         report_values = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_cheese_cake.id, searchQty=60, searchVariant=False)
-        self.assertEqual(report_values['lines']['operations_time'], 40.0, 'Operation time should be the same for 1 unit or for the batch')
-        # Operation cost is the sum of operation line.
-        operation_cost = float_round(10 / 60 * 10, precision_digits=2) + float_round(30 / 60 * 20, precision_digits=2)
-        self.assertEqual(float_compare(report_values['lines']['operations_cost'], operation_cost, precision_digits=2), 0)
+        #Operation time = 15 min * 60 + time_start + time_stop = 925
+        self.assertEqual(report_values['lines']['operations_time'], 925.0, 'Operation time should be the same for 1 unit or for the batch')
+        # Operation cost is the sum of operation line : (60 * 10)/60 * 10€ + (10 + 15 + 60 * 5)/60 * 20€ = 208,33€
+        self.assertEqual(float_compare(report_values['lines']['operations_cost'], 208.33, precision_digits=2), 0)
 
         for component_line in report_values['lines']['components']:
             # standard price * bom line quantity * current quantity / bom finished product quantity
@@ -430,8 +593,55 @@ class TestBoM(TestMrpCommon):
                 # 5.4 kg of crumble at the cost of a batch.
                 crumble_cost = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_crumble.id, searchQty=5.4, searchVariant=False)['lines']['total']
                 self.assertEqual(float_compare(component_line['total'], crumble_cost, precision_digits=2), 0)
-        # total price = 15.51 + crumble_cost + operation_cost(10 + 1.67 = 11.67) = 27.18 + crumble_cost
-        self.assertEqual(float_compare(report_values['lines']['total'], 27.18 + crumble_cost, precision_digits=2), 0, 'Product Bom Price is not correct')
+        # total price = Cream (15.51€) + crumble_cost (34.63 €) + operation_cost(208,33) = 258.47€
+        self.assertEqual(float_compare(report_values['lines']['total'], 258.47, precision_digits=2), 0, 'Product Bom Price is not correct')
+
+    def test_bom_report_dozens(self):
+        """ Simulate a drawer bom with dozens as bom units
+        """
+        uom_dozen = self.env.ref('uom.product_uom_dozen')
+        uom_unit = self.env.ref('uom.product_uom_unit')
+        drawer = self.env['product.product'].create({
+            'name': 'drawer',
+            'type': 'product',
+            'uom_id': uom_unit.id,
+            'uom_po_id': uom_unit.id,
+        })
+        screw = self.env['product.product'].create({
+            'name': 'screw',
+            'type': 'product',
+            'uom_id': uom_unit.id,
+            'uom_po_id': uom_unit.id,
+            'standard_price': 7.01
+        })
+
+        bom_form_drawer = Form(self.env['mrp.bom'])
+        bom_form_drawer.product_tmpl_id = drawer.product_tmpl_id
+        bom_form_drawer.product_qty = 11
+        bom_form_drawer.product_uom_id = uom_dozen
+        bom_drawer = bom_form_drawer.save()
+
+        workcenter = self.env['mrp.workcenter'].create({
+            'costs_hour': 10,
+            'name': 'Deserts Table'
+        })
+
+        with Form(bom_drawer) as bom:
+            with bom.bom_line_ids.new() as line:
+                line.product_id = screw
+                line.product_uom_id = uom_unit
+                line.product_qty = 5
+            with bom.operation_ids.new() as operation:
+                operation.workcenter_id = workcenter
+                operation.name = 'Screw drawer'
+                operation.time_cycle_manual = 5
+                operation.bom_id = bom_drawer
+
+        # TEST BOM STRUCTURE VALUE WITH BOM QUANTITY
+        report_values = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_drawer.id, searchQty=11, searchVariant=False)
+        # 5 min 'Prepare biscuits' + 3 min 'Prepare butter' + 5 min 'Mix manually' = 13 minutes
+        self.assertEqual(report_values['lines']['operations_time'], 660.0, 'Operation time should be the same for 1 unit or for the batch')
+
 
     def test_21_bom_report_variant(self):
         """ Test a sub BoM process with multiple variants.
@@ -727,7 +937,7 @@ class TestBoM(TestMrpCommon):
 
         report_values = self.env['report.mrp.report_bom_structure']._get_report_data(bom_id=bom_finished.id, searchQty=80)
 
-        self.assertAlmostEqual(report_values['lines']['total'], 2.92)
+        self.assertAlmostEqual(report_values['lines']['total'], 0.58)
 
     def test_validate_no_bom_line_with_same_product(self):
         """
@@ -797,3 +1007,51 @@ class TestBoM(TestMrpCommon):
             line.product_uom_id = uom_unit
             line.product_qty = 5
         bom_finished = bom_finished.save()
+
+    def test_replenishment(self):
+        """ Tests the auto generation of manual orderpoints.
+            The multiple quantity of the orderpoint should be the
+            quantity of the BoM in the UoM of the product.
+        """
+
+        uom_kg = self.env.ref('uom.product_uom_kgm')
+        uom_gram = self.env.ref('uom.product_uom_gram')
+
+        product_gram = self.env['product.product'].create({
+            'name': 'Product sold in grams',
+            'type': 'product',
+            'uom_id': uom_gram.id,
+            'uom_po_id': uom_gram.id,
+        })
+        # We create a BoM that manufactures 2kg of product
+        self.env['mrp.bom'].create({
+            'product_id': product_gram.id,
+            'product_tmpl_id': product_gram.product_tmpl_id.id,
+            'product_uom_id': uom_kg.id,
+            'product_qty': 2.0,
+            'type': 'normal',
+        })
+        # We create a delivery order of 2300 grams
+        picking_form = Form(self.env['stock.picking'])
+        picking_form.picking_type_id = self.env.ref('stock.picking_type_out')
+        with picking_form.move_ids_without_package.new() as move:
+            move.product_id = product_gram
+            move.product_uom_qty = 2300.0
+        customer_picking = picking_form.save()
+        customer_picking.action_confirm()
+
+        # We check the created orderpoint without manufacturing route manufacturing
+        self.env['stock.warehouse.orderpoint']._get_orderpoint_action()
+        self.env['stock.warehouse.orderpoint']._get_orderpoint_action()
+        orderpoint = self.env['stock.warehouse.orderpoint'].search([('product_id', '=', product_gram.id)])
+        self.assertEqual(orderpoint.qty_multiple, 0.0)
+        self.assertEqual(orderpoint.qty_to_order, 2300.0)
+
+        # We select the manufacturing route and check the impact on the quantities
+        manufacturing_route_id = self.ref('mrp.route_warehouse0_manufacture')
+        manufacturing_route = self.env['stock.location.route'].search([('id', '=', manufacturing_route_id)])
+        orderpoint_form = Form(orderpoint)
+        orderpoint_form.route_id = manufacturing_route
+        orderpoint_form.save()
+        self.assertEqual(orderpoint.qty_multiple, 2000.0)
+        self.assertEqual(orderpoint.qty_to_order, 4000.0)
