@@ -6,7 +6,7 @@
     'version': '0.1',
     'summary': "Subcontract Productions",
     'description': "",
-    'website': 'https://www.odoo.com/page/manufacturing',
+    'website': 'https://www.odoo.com/app/manufacturing',
     'category': 'Manufacturing/Manufacturing',
     'depends': ['mrp'],
     'data': [
@@ -15,8 +15,13 @@
         'views/res_partner_views.xml',
         'views/stock_warehouse_views.xml',
         'views/stock_move_views.xml',
+        'views/stock_quant_views.xml',
         'views/stock_picking_views.xml',
         'views/supplier_info_views.xml',
+        'views/product_views.xml',
+        'views/mrp_production_views.xml',
+        'wizard/stock_picking_return_views.xml',
+        'report/mrp_report_bom_structure.xml',
     ],
     'demo': [
         'data/mrp_subcontracting_demo.xml',
