@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import mrp_product_produce
 from . import stock_picking_return
+from . import mrp_consumption_warning
