@@ -7,6 +7,7 @@ import re
 import string
 
 import requests
+from markupsafe import Markup
 
 from odoo import api, models, _
 from odoo.exceptions import UserError
@@ -20,17 +21,18 @@ class PadCommon(models.AbstractModel):
     _name = 'pad.common'
     _description = 'Pad Common'
 
+    def _valid_field_parameter(self, field, name):
+        return name == 'pad_content_field' or super()._valid_field_parameter(field, name)
+
     @api.model
     def pad_is_configured(self):
-        return bool(self.env.company.pad_server)
+        return bool(self.env['ir.config_parameter'].sudo().get_param('pad.pad_server'))
 
     @api.model
     def pad_generate_url(self):
-        company = self.env.company.sudo()
-
         pad = {
-            "server": company.pad_server,
-            "key": company.pad_key,
+            "server": self.env['ir.config_parameter'].sudo().get_param('pad.pad_server'),
+            "key": self.env['ir.config_parameter'].sudo().get_param('pad.pad_key'),
         }
 
         # make sure pad server in the form of http://hostname
@@ -62,10 +64,12 @@ class PadCommon(models.AbstractModel):
             field = model._fields[self.env.context['field_name']]
             real_field = field.pad_content_field
 
+            res_id = self.env.context.get("object_id")
+            record = model.browse(res_id)
             # get content of the real field
-            for record in model.browse(self.env.context.get("object_id")):
-                    if record[real_field]:
-                        myPad.setHtmlFallbackText(path, record[real_field])
+            real_field_value = record[real_field] or self.env.context.get('record', {}).get(real_field, '')
+            if real_field_value:
+                myPad.setHtmlFallbackText(path, real_field_value)
 
         return {
             "server": pad["server"],
@@ -75,8 +79,11 @@ class PadCommon(models.AbstractModel):
 
     @api.model
     def pad_get_content(self, url):
-        company = self.env.company.sudo()
-        myPad = EtherpadLiteClient(company.pad_key, (company.pad_server or '') + '/api')
+        pad = {
+            "server": self.env['ir.config_parameter'].sudo().get_param('pad.pad_server'),
+            "key": self.env['ir.config_parameter'].sudo().get_param('pad.pad_key'),
+        }
+        myPad = EtherpadLiteClient(pad['key'], (pad['server'] or '') + '/api')
         content = ''
         if url:
             split_url = url.split('/p/')
@@ -95,7 +102,7 @@ class PadCommon(models.AbstractModel):
                     if mo:
                         content = mo.group(1)
 
-        return content
+        return Markup(content)
 
     # TODO
     # reverse engineer protocol to be setHtml without using the api key
@@ -131,8 +138,11 @@ class PadCommon(models.AbstractModel):
         # Update the pad if the `pad_content_field` is modified
         for k, field in self._fields.items():
             if hasattr(field, 'pad_content_field') and vals.get(field.pad_content_field) and self[k]:
-                company = self.env.user.sudo().company_id
-                myPad = EtherpadLiteClient(company.pad_key, (company.pad_server or '') + '/api')
+                pad = {
+                    "server": self.env['ir.config_parameter'].sudo().get_param('pad.pad_server'),
+                    "key": self.env['ir.config_parameter'].sudo().get_param('pad.pad_key'),
+                }
+                myPad = EtherpadLiteClient(pad['key'], (pad['server'] or '') + '/api')
                 path = self[k].split('/p/')[1]
                 myPad.setHtmlFallbackText(path, vals[field.pad_content_field])
 
