@@ -1,10 +1,11 @@
-# -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 {
     'name': 'Authorize.Net Payment Acquirer',
-    'category': 'Accounting/Payment',
+    'version': '2.0',
+    'category': 'Accounting/Payment Acquirers',
+    'sequence': 350,
     'summary': 'Payment Acquirer: Authorize.net Implementation',
-    'version': '1.0',
     'description': """Authorize.Net Payment Acquirer""",
     'depends': ['payment'],
     'data': [
@@ -12,8 +13,13 @@
         'views/payment_authorize_templates.xml',
         'data/payment_acquirer_data.xml',
     ],
-    'installable': True,
-    'post_init_hook': 'create_missing_journal_for_acquirers',
+    'application': True,
     'uninstall_hook': 'uninstall_hook',
+    'assets': {
+        'web.assets_frontend': [
+            'payment_authorize/static/src/scss/payment_authorize.scss',
+            'payment_authorize/static/src/js/payment_form.js',
+        ],
+    },
     'license': 'LGPL-3',
 }
