@@ -1,10 +1,11 @@
-# -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 {
     'name': 'Paypal Payment Acquirer',
-    'category': 'Accounting/Payment',
+    'version': '2.0',
+    'category': 'Accounting/Payment Acquirers',
+    'sequence': 365,
     'summary': 'Payment Acquirer: Paypal Implementation',
-    'version': '1.0',
     'description': """Paypal Payment Acquirer""",
     'depends': ['payment'],
     'data': [
@@ -13,8 +14,7 @@
         'data/payment_acquirer_data.xml',
         'data/payment_paypal_email_data.xml',
     ],
-    'installable': True,
-    'post_init_hook': 'create_missing_journal_for_acquirers',
+    'application': True,
     'uninstall_hook': 'uninstall_hook',
     'license': 'LGPL-3',
 }
