@@ -10,7 +10,7 @@ class AccountTax(models.Model):
     def write(self, vals):
         forbidden_fields = set([
             'amount_type', 'amount', 'type_tax_use', 'tax_group_id', 'price_include',
-            'include_base_amount'
+            'include_base_amount', 'is_base_affected',
         ])
         if forbidden_fields & set(vals.keys()):
             tax_ids = self.env['pos.order.line'].sudo().search([
@@ -20,7 +20,7 @@ class AccountTax(models.Model):
             tax_ids = set([i for sl in [t['tax_ids'] for t in tax_ids] for i in sl])
             if tax_ids & set(self.ids):
                 raise UserError(_(
-                    'It is forbidden to modify a tax used in a POS order not posted. ' +
+                    'It is forbidden to modify a tax used in a POS order not posted. '
                     'You must close the POS sessions before modifying the tax.'
                 ))
         return super(AccountTax, self).write(vals)
