@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
+
 import time
 
 import odoo
-from odoo import fields
+from odoo import fields, tools
 from odoo.tools import float_compare, mute_logger, test_reports
 from odoo.tests.common import Form
 from odoo.addons.point_of_sale.tests.common import TestPointOfSaleCommon
@@ -13,18 +15,18 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
 
     def compute_tax(self, product, price, qty=1, taxes=None):
         if not taxes:
-            taxes = product.taxes_id.filtered(lambda t: t.company_id.id == self.env.user.id)
+            taxes = product.taxes_id.filtered(lambda t: t.company_id.id == self.env.company.id)
         currency = self.pos_config.pricelist_id.currency_id
         res = taxes.compute_all(price, currency, qty, product=product)
         untax = res['total_excluded']
         return untax, sum(tax.get('amount', 0.0) for tax in res['taxes'])
 
     def test_order_refund(self):
-        self.pos_config.open_session_cb()
+        self.pos_config.open_session_cb(check_coa=False)
         current_session = self.pos_config.current_session_id
         # I create a new PoS order with 2 lines
         order = self.PosOrder.create({
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'session_id': current_session.id,
             'partner_id': self.partner1.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
@@ -80,7 +82,9 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         self.assertEqual(refund.state, 'paid', "The refund is not marked as paid")
         self.assertTrue(refund.payment_ids.payment_method_id.is_cash_count, msg='There should only be one payment and paid in cash.')
 
-        current_session.action_pos_session_closing_control()
+        total_cash_payment = sum(current_session.mapped('order_ids.payment_ids').filtered(lambda payment: payment.payment_method_id.type == 'cash').mapped('amount'))
+        current_session.post_closing_cash_details(total_cash_payment)
+        current_session.close_session_from_ui()
         self.assertEqual(current_session.state, 'closed', msg='State of current session should be closed.')
 
     def test_order_refund_lots(self):
@@ -89,7 +93,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         current_session = self.pos_config.current_session_id
 
         # set up product iwith SN tracing and create two lots (1001, 1002)
-        self.stock_location = self.env.ref('stock.stock_location_stock')
+        self.stock_location = self.company_data['default_warehouse'].lot_stock_id
         self.product2 = self.env['product.product'].create({
             'name': 'Product A',
             'type': 'product',
@@ -97,13 +101,6 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             'categ_id': self.env.ref('product.product_category_all').id,
         })
 
-        inventory = self.env['stock.inventory'].create({
-            'name': 'add product2',
-            'location_ids': [(4, self.stock_location.id)],
-            'product_ids': [(4, self.product2.id)],
-        })
-        inventory.action_start()
-
         lot1 = self.env['stock.production.lot'].create({
             'name': '1001',
             'product_id': self.product2.id,
@@ -115,29 +112,23 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             'company_id': self.env.company.id,
         })
 
-        self.env['stock.inventory.line'].create([
-            {
-            'inventory_id': inventory.id,
-            'location_id': self.stock_location.id,
+        self.env['stock.quant'].with_context(inventory_mode=True).create({
             'product_id': self.product2.id,
-            'prod_lot_id': lot1.id,
-            'product_qty': 1
-            },
-            {
-            'inventory_id': inventory.id,
+            'inventory_quantity': 1,
             'location_id': self.stock_location.id,
+            'lot_id': lot1.id
+        }).action_apply_inventory()
+        self.env['stock.quant'].with_context(inventory_mode=True).create({
             'product_id': self.product2.id,
-            'prod_lot_id': lot2.id,
-            'product_qty': 1
-            },
-        ])
-
-        inventory.action_validate()
+            'inventory_quantity': 1,
+            'location_id': self.stock_location.id,
+            'lot_id': lot2.id
+        }).action_apply_inventory()
 
         # create pos order with the two SN created before
 
         order = self.PosOrder.create({
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'session_id': current_session.id,
             'partner_id': self.partner1.id,
             'lines': [(0, 0, {
@@ -203,7 +194,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         """
 
         # I click on create a new session button
-        self.pos_config.open_session_cb()
+        self.pos_config.open_session_cb(check_coa=False)
         current_session = self.pos_config.current_session_id
 
         # I create a PoS order with 2 units of PCSC234 at 450 EUR
@@ -211,7 +202,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         untax1, atax1 = self.compute_tax(self.product3, 450, 2)
         untax2, atax2 = self.compute_tax(self.product4, 300, 3)
         self.pos_order_pos1 = self.PosOrder.create({
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'session_id': current_session.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'partner_id': self.partner1.id,
@@ -262,12 +253,12 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         # I test that the pickings are created as expected during payment
         # One picking attached and having all the positive move lines in the correct state
         self.assertEqual(
-            self.pos_order_pos1.picking_id.state,
+            self.pos_order_pos1.picking_ids[0].state,
             'done',
             'Picking should be in done state.'
         )
         self.assertEqual(
-            self.pos_order_pos1.picking_id.move_lines.mapped('state'),
+            self.pos_order_pos1.picking_ids[0].move_lines.mapped('state'),
             ['done', 'done'],
             'Move Lines should be in done state.'
         )
@@ -276,7 +267,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         untax1, atax1 = self.compute_tax(self.product3, 450, -2)
         untax2, atax2 = self.compute_tax(self.product4, 300, -3)
         self.pos_order_pos2 = self.PosOrder.create({
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'session_id': current_session.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'partner_id': self.partner1.id,
@@ -326,14 +317,13 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
 
         # I test that the pickings are created as expected
         # One picking attached and having all the positive move lines in the correct state
-        self.pos_order_pos2.create_picking()
         self.assertEqual(
-            self.pos_order_pos2.picking_id.state,
+            self.pos_order_pos2.picking_ids[0].state,
             'done',
             'Picking should be in done state.'
         )
         self.assertEqual(
-            self.pos_order_pos2.picking_id.move_lines.mapped('state'),
+            self.pos_order_pos2.picking_ids[0].move_lines.mapped('state'),
             ['done', 'done'],
             'Move Lines should be in done state.'
         )
@@ -341,7 +331,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         untax1, atax1 = self.compute_tax(self.product3, 450, -2)
         untax2, atax2 = self.compute_tax(self.product4, 300, 3)
         self.pos_order_pos3 = self.PosOrder.create({
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'session_id': current_session.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'partner_id': self.partner1.id,
@@ -392,12 +382,12 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         # I test that the pickings are created as expected
         # One picking attached and having all the positive move lines in the correct state
         self.assertEqual(
-            self.pos_order_pos3.picking_id.state,
+            self.pos_order_pos3.picking_ids[0].state,
             'done',
             'Picking should be in done state.'
         )
         self.assertEqual(
-            self.pos_order_pos3.picking_id.move_lines.mapped('state'),
+            self.pos_order_pos3.picking_ids[0].move_lines.mapped('state'),
             ['done'],
             'Move Lines should be in done state.'
         )
@@ -405,16 +395,22 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         self.pos_config.current_session_id.action_pos_session_closing_control()
 
     def test_order_to_picking02(self):
-        """ This test is similar to test_order_to_picking except that this time, the product is tracked and its
-         location is a sublocation of the main warehouse
+        """ This test is similar to test_order_to_picking except that this time, there are two products:
+            - One tracked by lot
+            - One untracked
+            - Both are in a sublocation of the main warehouse
         """
-        product = self.env['product.product'].create({
-            'name': 'SuperProduct',
+        tracked_product, untracked_product = self.env['product.product'].create([{
+            'name': 'SuperProduct Tracked',
             'type': 'product',
             'tracking': 'lot',
             'available_in_pos': True,
-        })
-        wh_location = self.env['ir.model.data'].xmlid_to_object('stock.warehouse0').lot_stock_id
+        }, {
+            'name': 'SuperProduct Untracked',
+            'type': 'product',
+            'available_in_pos': True,
+        }])
+        wh_location = self.company_data['default_warehouse'].lot_stock_id
         shelf1_location = self.env['stock.location'].create({
             'name': 'shelf1',
             'usage': 'internal',
@@ -422,35 +418,46 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         })
         lot = self.env['stock.production.lot'].create({
             'name': 'SuperLot',
-            'product_id': product.id,
-            'company_id': self.company.id,
+            'product_id': tracked_product.id,
+            'company_id': self.env.company.id,
         })
         qty = 2
-        self.env['stock.quant']._update_available_quantity(product, shelf1_location, qty, lot_id=lot)
+        self.env['stock.quant']._update_available_quantity(tracked_product, shelf1_location, qty, lot_id=lot)
+        self.env['stock.quant']._update_available_quantity(untracked_product, shelf1_location, qty)
 
         self.pos_config.open_session_cb()
+        self.pos_config.current_session_id.update_stock_at_closing = False
 
-        untax, atax = self.compute_tax(product, 1.15, 1)
+        untax, atax = self.compute_tax(tracked_product, 1.15, 1)
 
-        for i in range(qty):
+        for dummy in range(qty):
             pos_order = self.PosOrder.create({
-                'company_id': self.company_id,
+                'company_id': self.env.company.id,
                 'session_id': self.pos_config.current_session_id.id,
                 'pricelist_id': self.partner1.property_product_pricelist.id,
                 'partner_id': self.partner1.id,
                 'lines': [(0, 0, {
                     'name': "OL/0001",
-                    'product_id': product.id,
-                    'price_unit': untax + atax,
+                    'product_id': tracked_product.id,
+                    'price_unit': 1.15,
                     'discount': 0.0,
                     'qty': 1.0,
-                    'tax_ids': [(6, 0, product.taxes_id.ids)],
+                    'tax_ids': [(6, 0, tracked_product.taxes_id.ids)],
                     'price_subtotal': untax,
                     'price_subtotal_incl': untax + atax,
                     'pack_lot_ids': [[0, 0, {'lot_name': lot.name}]],
+                }), (0, 0, {
+                    'name': "OL/0002",
+                    'product_id': untracked_product.id,
+                    'price_unit': 1.15,
+                    'discount': 0.0,
+                    'qty': 1.0,
+                    'tax_ids': [(6, 0, untracked_product.taxes_id.ids)],
+                    'price_subtotal': untax,
+                    'price_subtotal_incl': untax + atax,
                 })],
-                'amount_tax': atax,
-                'amount_total': untax + atax,
+                'amount_tax': 2 * atax,
+                'amount_total': 2 * (untax + atax),
                 'amount_paid': 0,
                 'amount_return': 0,
             })
@@ -460,27 +467,31 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
                 "active_id": pos_order.id,
             }
             pos_make_payment = self.PosMakePayment.with_context(context_make_payment).create({
-                'amount': untax + atax,
+                'amount': 2 * (untax + atax),
             })
             context_payment = {'active_id': pos_order.id}
             pos_make_payment.with_context(context_payment).check()
 
             self.assertEqual(pos_order.state, 'paid')
-            self.assertEqual(pos_order.picking_id.move_line_ids.lot_id, lot)
-            self.assertEqual(pos_order.picking_id.move_line_ids.location_id, shelf1_location)
+            tracked_line = pos_order.picking_ids.move_line_ids.filtered(lambda ml: ml.product_id.id == tracked_product.id)
+            untracked_line = pos_order.picking_ids.move_line_ids - tracked_line
+            self.assertEqual(tracked_line.lot_id, lot)
+            self.assertFalse(untracked_line.lot_id)
+            self.assertEqual(tracked_line.location_id, shelf1_location)
+            self.assertEqual(untracked_line.location_id, shelf1_location)
 
         self.pos_config.current_session_id.action_pos_session_closing_control()
 
     def test_order_to_invoice(self):
 
-        self.pos_config.open_session_cb()
+        self.pos_config.open_session_cb(check_coa=False)
         current_session = self.pos_config.current_session_id
 
         untax1, atax1 = self.compute_tax(self.product3, 450*0.95, 2)
         untax2, atax2 = self.compute_tax(self.product4, 300*0.95, 3)
         # I create a new PoS order with 2 units of PC1 at 450 EUR (Tax Incl) and 3 units of PCSC349 at 300 EUR. (Tax Excl)
         self.pos_order_pos1 = self.PosOrder.create({
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'session_id': current_session.id,
             'partner_id': self.partner1.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
@@ -490,7 +501,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
                 'price_unit': 450,
                 'discount': 5.0,
                 'qty': 2.0,
-                'tax_ids': [(6, 0, self.product3.taxes_id.filtered(lambda t: t.company_id.id == self.company_id).ids)],
+                'tax_ids': [(6, 0, self.product3.taxes_id.filtered(lambda t: t.company_id.id == self.env.company.id).ids)],
                 'price_subtotal': untax1,
                 'price_subtotal_incl': untax1 + atax1,
             }), (0, 0, {
@@ -499,7 +510,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
                 'price_unit': 300,
                 'discount': 5.0,
                 'qty': 3.0,
-                'tax_ids': [(6, 0, self.product4.taxes_id.filtered(lambda t: t.company_id.id == self.company_id).ids)],
+                'tax_ids': [(6, 0, self.product4.taxes_id.filtered(lambda t: t.company_id.id == self.env.company.id).ids)],
                 'price_subtotal': untax2,
                 'price_subtotal_incl': untax2 + atax2,
             })],
@@ -529,6 +540,8 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
 
         # I test that the total of the attached invoice is correct
         invoice = self.env['account.move'].browse(res['res_id'])
+        if invoice.state != 'posted':
+            invoice.action_post()
         self.assertAlmostEqual(
             invoice.amount_total, self.pos_order_pos1.amount_total, places=2, msg="Invoice not correct")
 
@@ -546,7 +559,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             'name': 'Bank Test',
             'code': 'BNKT',
             'type': 'bank',
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
         })
         # I create a bank statement with Opening and Closing balance 0.
         account_statement = self.AccountBankStatement.create({
@@ -554,7 +567,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             'balance_end_real': 0.0,
             'date': time.strftime('%Y-%m-%d'),
             'journal_id': journal.id,
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'name': 'pos session test',
         })
         # I create bank statement line
@@ -562,7 +575,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             'amount': 1000,
             'partner_id': self.partner4.id,
             'statement_id': account_statement.id,
-            'name': 'EXT001'
+            'payment_ref': 'EXT001'
         })
         # I modify the bank statement and set the Closing Balance.
         account_statement.write({
@@ -577,11 +590,9 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             'debit': 0.0,
         }]
 
-        self.env['account.reconciliation.widget'].process_bank_statement_line(account_statement_line.ids, [{'new_aml_dicts': new_aml_dicts}])
-
         # I confirm the bank statement using Confirm button
 
-        self.AccountBankStatement.button_confirm_bank()
+        self.AccountBankStatement.button_validate()
 
     def test_create_from_ui(self):
         """
@@ -589,11 +600,13 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         """
 
         # I click on create a new session button
-        self.pos_config.open_session_cb()
+        self.pos_config.open_session_cb(check_coa=False)
 
         current_session = self.pos_config.current_session_id
         num_starting_orders = len(current_session.order_ids)
 
+        current_session.set_cashbox_pos(0, None)
+
         untax, atax = self.compute_tax(self.led_lamp, 0.9)
         carrot_order = {'data':
           {'amount_paid': untax + atax,
@@ -649,7 +662,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
               'qty': 1,
               'tax_ids': [(6, 0, self.whiteboard_pen.taxes_id.ids)]}]],
            'name': 'Order 00043-003-0014',
-           'partner_id': False,
+           'partner_id': self.partner1.id,
            'pos_session_id': current_session.id,
            'sequence_number': self.pos_config.journal_id.id,
            'statement_ids': [[0,
@@ -701,7 +714,9 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         self.assertEqual(num_starting_orders + 1, len(current_session.order_ids), "Submitted order not encoded")
 
         # I close the session
-        current_session.action_pos_session_closing_control()
+        total_cash_payment = sum(current_session.mapped('order_ids.payment_ids').filtered(lambda payment: payment.payment_method_id.type == 'cash').mapped('amount'))
+        current_session.post_closing_cash_details(total_cash_payment)
+        current_session.close_session_from_ui()
         self.assertEqual(current_session.state, 'closed', "Session was not properly closed")
         self.assertFalse(self.pos_config.current_session_id, "Current session not properly recomputed")
 
@@ -718,7 +733,9 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         self.assertEqual(len(rescue_session.order_ids), 2, "Rescue session does not contain both orders")
 
         # I close the rescue session
-        rescue_session.action_pos_session_closing_control()
+        total_cash_payment = sum(rescue_session.mapped('order_ids.payment_ids').filtered(lambda payment: payment.payment_method_id.type == 'cash').mapped('amount'))
+        rescue_session.post_closing_cash_details(total_cash_payment)
+        rescue_session.close_session_from_ui()
         self.assertEqual(rescue_session.state, 'closed', "Rescue session was not properly closed")
 
     def test_order_to_payment_currency(self):
@@ -728,6 +745,8 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             The order will be in a different currency than the company currency.
         """
         # Make sure the company is in USD
+        self.env.ref('base.USD').active = True
+        self.env.ref('base.EUR').active = True
         self.env.cr.execute(
             "UPDATE res_company SET currency_id = %s WHERE id = %s",
             [self.env.ref('base.USD').id, self.env.company.id])
@@ -761,7 +780,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         })
 
         # I click on create a new session button
-        eur_config.open_session_cb()
+        eur_config.open_session_cb(check_coa=False)
         current_session = eur_config.current_session_id
 
         # I create a PoS order with 2 units of PCSC234 at 450 EUR (Tax Incl)
@@ -770,7 +789,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         untax1, atax1 = self.compute_tax(self.product3, 450, 2)
         untax2, atax2 = self.compute_tax(self.product4, 300, 3)
         self.pos_order_pos0 = self.PosOrder.create({
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'session_id': current_session.id,
             'pricelist_id': eur_pricelist.id,
             'partner_id': self.partner1.id,
@@ -780,7 +799,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
                 'price_unit': 450,
                 'discount': 0.0,
                 'qty': 2.0,
-                'tax_ids': [(6, 0, self.product3.taxes_id.ids)],
+                'tax_ids': [(6, 0, self.product3.taxes_id.filtered(lambda t: t.company_id == self.env.company).ids)],
                 'price_subtotal': untax1,
                 'price_subtotal_incl': untax1 + atax1,
             }), (0, 0, {
@@ -789,7 +808,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
                 'price_unit': 300,
                 'discount': 0.0,
                 'qty': 3.0,
-                'tax_ids': [(6, 0, self.product4.taxes_id.ids)],
+                'tax_ids': [(6, 0, self.product4.taxes_id.filtered(lambda t: t.company_id == self.env.company).ids)],
                 'price_subtotal': untax2,
                 'price_subtotal_incl': untax2 + atax2,
             })],
@@ -860,13 +879,12 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             self.assertAlmostEqual(a, b)
 
     def test_order_to_invoice_no_tax(self):
-
-        self.pos_config.open_session_cb()
+        self.pos_config.open_session_cb(check_coa=False)
         current_session = self.pos_config.current_session_id
 
         # I create a new PoS order with 2 units of PC1 at 450 EUR (Tax Incl) and 3 units of PCSC349 at 300 EUR. (Tax Excl)
         self.pos_order_pos1 = self.PosOrder.create({
-            'company_id': self.company_id,
+            'company_id': self.env.company.id,
             'session_id': current_session.id,
             'partner_id': self.partner1.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
@@ -913,6 +931,8 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
 
         # I test that the total of the attached invoice is correct
         invoice = self.env['account.move'].browse(res['res_id'])
+        if invoice.state != 'posted':
+            invoice.action_post()
         self.assertAlmostEqual(
             invoice.amount_total, self.pos_order_pos1.amount_total, places=2, msg="Invoice not correct")
 
@@ -939,7 +959,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         })
 
         # sell product thru pos
-        self.pos_config.open_session_cb()
+        self.pos_config.open_session_cb(check_coa=False)
         pos_session = self.pos_config.current_session_id
         untax, atax = self.compute_tax(product5, 10.0)
         product5_order = {'data':
@@ -979,9 +999,150 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         # delete tax
         dummy_50_perc_tax.unlink()
 
+        total_cash_payment = sum(pos_session.mapped('order_ids.payment_ids').filtered(lambda payment: payment.payment_method_id.type == 'cash').mapped('amount'))
+        pos_session.post_closing_cash_details(total_cash_payment)
+
         # close session (should not fail here)
-        pos_session.action_pos_session_closing_control()
+        # We don't call `action_pos_session_closing_control` to force the failed
+        # closing which will return the action because the internal rollback call messes
+        # with the rollback of the test runner. So instead, we directly call the method
+        # that returns the action by specifying the imbalance amount.
+        action = pos_session._close_session_action(5.0)
+        wizard = self.env['pos.close.session.wizard'].browse(action['res_id'])
+        wizard.with_context(action['context']).close_session()
 
         # check the difference line
         diff_line = pos_session.move_id.line_ids.filtered(lambda line: line.name == 'Difference at closing PoS session')
         self.assertAlmostEqual(diff_line.credit, 5.0, msg="Missing amount of 5.0")
+
+    def test_order_and_ship_later(self):
+        """ In case the product is tracked and the "Ship Later" feature is enabled, this test ensures
+        that the lots defined on the POS order are the same that the ones on the SMLs of the associated
+        picking"""
+        self.env['stock.picking.type'].search([('code', '=', 'outgoing')]).write({
+            'use_create_lots': True,
+            'use_existing_lots': True,
+        })
+
+        tracked_product = self.env['product.product'].create({
+            'name': 'SuperProduct Tracked',
+            'type': 'product',
+            'tracking': 'lot',
+            'available_in_pos': True,
+        })
+
+        lot01, lot02 = self.env['stock.production.lot'].create([{
+            'name': name,
+            'product_id': tracked_product.id,
+            'company_id': self.env.company.id,
+        } for name in ['Lot01', 'Lot02']])
+        stock_location = self.company_data['default_warehouse'].lot_stock_id
+        self.env['stock.quant']._update_available_quantity(tracked_product, stock_location, 10, lot_id=lot01)
+        self.env['stock.quant']._update_available_quantity(tracked_product, stock_location, 10, lot_id=lot02)
+
+        self.pos_config.ship_later = True
+        self.pos_config.open_session_cb()
+
+        # Create a POS order with:
+        #   1 x Lot01
+        #   1 x Lot02
+        #   1 x Lot03 (-> nonexistent lot!)
+        pos_order = self.PosOrder.create({
+            'company_id': self.env.company.id,
+            'session_id': self.pos_config.current_session_id.id,
+            'pricelist_id': self.partner1.property_product_pricelist.id,
+            'partner_id': self.partner1.id,
+            'lines': [(0, 0, {
+                'name': "OL/0001",
+                'product_id': tracked_product.id,
+                'price_unit': 5,
+                'discount': 0.0,
+                'qty': 1.0,
+                'tax_ids': [(6, 0, [])],
+                'price_subtotal': 5,
+                'price_subtotal_incl': 5,
+                'pack_lot_ids': [[0, 0, {'lot_name': lot_name}]],
+            }) for lot_name in [lot01.name, lot02.name, 'Lot03']],
+            'amount_tax': 10,
+            'amount_total': 10,
+            'amount_paid': 0,
+            'amount_return': 0,
+            'to_ship': True,
+        })
+
+        context_make_payment = {
+            "active_ids": [pos_order.id],
+            "active_id": pos_order.id,
+        }
+        pos_make_payment = self.PosMakePayment.with_context(context_make_payment).create({
+            'amount': 10,
+        })
+        pos_make_payment.with_context(active_id=pos_order.id).check()
+
+        picking = pos_order.picking_ids
+        picking_mls = picking.move_line_ids
+        lot01_ml = picking_mls.filtered(lambda ml: ml.lot_id == lot01)
+        lot02_ml = picking_mls.filtered(lambda ml: ml.lot_id == lot02)
+        lot03_ml = picking_mls - lot01_ml - lot02_ml
+        self.assertEqual(lot01_ml.product_qty, 1)
+        self.assertEqual(lot02_ml.product_qty, 1)
+        self.assertEqual(lot03_ml.product_qty, 1)
+        self.assertEqual(lot03_ml.lot_id.name, "Lot03")
+
+    def test_order_multi_step_route(self):
+        """ Test that orders in sessions with "Ship Later" enabled and "Specific Route" set to a
+            multi-step (2/3) route can be validated. This config implies multiple picking types
+            and multiple move_lines.
+        """
+        tracked_product = self.env['product.product'].create({
+            'name': 'SuperProduct Tracked',
+            'type': 'product',
+            'tracking': 'lot',
+            'available_in_pos': True
+        })
+        warehouse_id = self.company_data['default_warehouse']
+        warehouse_id.delivery_steps = 'pick_ship'
+
+        self.pos_config.ship_later = True
+        self.pos_config.warehouse_id = warehouse_id
+        self.pos_config.route_id = warehouse_id.route_ids[-1]
+        self.pos_config.open_session_cb()
+        self.pos_config.current_session_id.update_stock_at_closing = False
+
+        untax, tax = self.compute_tax(tracked_product, 1.15, 1)
+
+        pos_order = self.PosOrder.create({
+            'company_id': self.env.company.id,
+            'session_id': self.pos_config.current_session_id.id,
+            'pricelist_id': self.partner1.property_product_pricelist.id,
+            'partner_id': self.partner1.id,
+            'lines': [(0, 0, {
+                'name': "OL/0001",
+                'product_id': tracked_product.id,
+                'price_unit': 1.15,
+                'qty': 1.0,
+                'price_subtotal': untax,
+                'price_subtotal_incl': untax + tax,
+            })],
+            'amount_tax': tax,
+            'amount_total': untax+tax,
+            'amount_paid': 0,
+            'amount_return': 0,
+            'to_ship': True,
+        })
+
+        context_make_payment = {
+            "active_ids": [pos_order.id],
+            "active_id": pos_order.id,
+        }
+        pos_make_payment = self.PosMakePayment.with_context(context_make_payment).create({
+            'amount': untax+tax,
+        })
+        context_payment = {'active_id': pos_order.id}
+        pos_make_payment.with_context(context_payment).check()
+
+        pickings = pos_order.picking_ids
+        picking_mls = pickings.move_line_ids
+        self.assertEqual(pos_order.state, 'paid')
+        self.assertEqual(len(picking_mls), 2)
+        self.assertEqual(len(pickings.picking_type_id), 2)
