@@ -4,7 +4,7 @@
 {
     'name': 'Vantiv Payment Services',
     'version': '1.0',
-    'category': 'Sales/Point Of Sale',
+    'category': 'Sales/Point of Sale',
     'sequence': 6,
     'summary': 'Credit card support for Point Of Sale',
     'description': """
@@ -25,19 +25,28 @@ following:
     'data': [
         'data/pos_mercury_data.xml',
         'security/ir.model.access.csv',
-        'views/pos_mercury_templates.xml',
         'views/pos_mercury_views.xml',
         'views/pos_mercury_transaction_templates.xml',
         'views/pos_config_setting_views.xml',
-        'views/pos_config_views.xml',
     ],
     'demo': [
         'data/pos_mercury_demo.xml',
     ],
-    'qweb': [
-        'static/src/xml/pos_mercury.xml',
-    ],
     'installable': True,
     'auto_install': False,
+    'assets': {
+        'point_of_sale.assets': [
+            'pos_mercury/static/src/js/pos_mercury.js',
+            'pos_mercury/static/src/js/OrderReceipt.js',
+            'pos_mercury/static/src/js/PaymentScreen.js',
+            'pos_mercury/static/src/js/PaymentScreenPaymentLines.js',
+            'pos_mercury/static/src/js/PaymentTransactionPopup.js',
+            'pos_mercury/static/src/js/ProductScreen.js',
+            'pos_mercury/static/src/css/pos_mercury.css',
+        ],
+        'web.assets_qweb': [
+            'pos_mercury/static/src/xml/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
