@@ -8,6 +8,7 @@ from odoo import fields, SUPERUSER_ID
 
 from odoo.tests import common, new_test_user
 from odoo.addons.hr_timesheet.tests.test_timesheet import TestCommonTimesheet
+import time
 
 
 class TestTimesheetHolidaysCreate(common.TransactionCase):
@@ -16,11 +17,11 @@ class TestTimesheetHolidaysCreate(common.TransactionCase):
         """Ensure that when a status is created, it fullfills the project and task constrains"""
         status = self.env['hr.leave.type'].create({
             'name': 'A nice Leave Type',
-            'allocation_type': 'no'
+            'requires_allocation': 'no'
         })
 
         company = self.env.company
-        self.assertEqual(status.timesheet_project_id, company.leave_timesheet_project_id, 'The default project linked to the status should be the same as the company')
+        self.assertEqual(status.timesheet_project_id, company.internal_project_id, 'The default project linked to the status should be the same as the company')
         self.assertEqual(status.timesheet_task_id, company.leave_timesheet_task_id, 'The default task linked to the status should be the same as the company')
 
     def test_company_create(self):
@@ -31,9 +32,9 @@ class TestTimesheetHolidaysCreate(common.TransactionCase):
                              company_ids=[(6, 0, main_company.ids)])
         Company = self.env['res.company']
         Company = Company.with_user(user)
-        Company = Company.with_context(allowed_company_ids=main_company.ids)
+        Company = Company.with_company(main_company)
         company = Company.create({'name': "Wall Company"})
-        self.assertEqual(company.leave_timesheet_project_id.sudo().company_id, company, "It should have created a project for the company")
+        self.assertEqual(company.internal_project_id.sudo().company_id, company, "It should have created a project for the company")
 
 class TestTimesheetHolidays(TestCommonTimesheet):
 
@@ -48,24 +49,22 @@ class TestTimesheetHolidays(TestCommonTimesheet):
         self.leave_end_datetime = self.leave_start_datetime + relativedelta(days=3)
 
         # all company have those internal project/task (created by default)
-        self.internal_project = self.env.company.leave_timesheet_project_id
+        self.internal_project = self.env.company.internal_project_id
         self.internal_task_leaves = self.env.company.leave_timesheet_task_id
 
         self.hr_leave_type_with_ts = self.env['hr.leave.type'].create({
             'name': 'Leave Type with timesheet generation',
-            'allocation_type': 'no',
+            'requires_allocation': 'no',
             'timesheet_generate': True,
             'timesheet_project_id': self.internal_project.id,
             'timesheet_task_id': self.internal_task_leaves.id,
-            'validity_start': False,
         })
         self.hr_leave_type_no_ts = self.env['hr.leave.type'].create({
             'name': 'Leave Type without timesheet generation',
-            'allocation_type': 'no',
+            'requires_allocation': 'no',
             'timesheet_generate': False,
             'timesheet_project_id': False,
             'timesheet_task_id': False,
-            'validity_start': False,
         })
 
         # HR Officer allocates some leaves to the employee 1
@@ -76,15 +75,21 @@ class TestTimesheetHolidays(TestCommonTimesheet):
             'employee_id': self.empl_employee.id,
             'holiday_status_id': self.hr_leave_type_with_ts.id,
             'number_of_days': 10,
+            'state': 'confirm',
+            'date_from': time.strftime('%Y-01-01'),
+            'date_to': time.strftime('%Y-12-31'),
         })
-        self.hr_leave_allocation_with_ts.action_approve()
+        self.hr_leave_allocation_with_ts.action_validate()
         self.hr_leave_allocation_no_ts = self.Allocations.sudo().create({
             'name': 'Days for limited category without timesheet',
             'employee_id': self.empl_employee.id,
             'holiday_status_id': self.hr_leave_type_no_ts.id,
             'number_of_days': 10,
+            'state': 'confirm',
+            'date_from': time.strftime('%Y-01-01'),
+            'date_to': time.strftime('%Y-12-31'),
         })
-        self.hr_leave_allocation_no_ts.action_approve()
+        self.hr_leave_allocation_no_ts.action_validate()
 
     def test_validate_with_timesheet(self):
         # employee creates a leave request
@@ -98,11 +103,11 @@ class TestTimesheetHolidays(TestCommonTimesheet):
             'number_of_days': number_of_days,
         })
         holiday.with_user(SUPERUSER_ID).action_validate()
-        self.assertEquals(len(holiday.timesheet_ids), number_of_days, 'Number of generated timesheets should be the same as the leave duration (1 per day between %s and %s)' % (fields.Datetime.to_string(self.leave_start_datetime), fields.Datetime.to_string(self.leave_end_datetime)))
+        self.assertEqual(len(holiday.timesheet_ids), number_of_days, 'Number of generated timesheets should be the same as the leave duration (1 per day between %s and %s)' % (fields.Datetime.to_string(self.leave_start_datetime), fields.Datetime.to_string(self.leave_end_datetime)))
 
         # manager refuse the leave
         holiday.with_user(SUPERUSER_ID).action_refuse()
-        self.assertEquals(len(holiday.timesheet_ids), 0, 'Number of linked timesheets should be zero, since the leave is refused.')
+        self.assertEqual(len(holiday.timesheet_ids), 0, 'Number of linked timesheets should be zero, since the leave is refused.')
 
     def test_validate_without_timesheet(self):
         # employee creates a leave request
@@ -116,4 +121,4 @@ class TestTimesheetHolidays(TestCommonTimesheet):
             'number_of_days': number_of_days,
         })
         holiday.with_user(SUPERUSER_ID).action_validate()
-        self.assertEquals(len(holiday.timesheet_ids), 0, 'Number of generated timesheets should be zero since the leave type does not generate timesheet')
+        self.assertEqual(len(holiday.timesheet_ids), 0, 'Number of generated timesheets should be zero since the leave type does not generate timesheet')
