@@ -3,6 +3,7 @@ odoo.define('sale.sales_team_dashboard', function (require) {
 
 var core = require('web.core');
 var KanbanRecord = require('web.KanbanRecord');
+var session = require('web.session');
 var _t = core._t;
 
 KanbanRecord.include({
@@ -23,8 +24,20 @@ KanbanRecord.include({
 
         this.$target_input = $('<input>');
         this.$('.o_kanban_primary_bottom:last').html(this.$target_input);
-        this.$('.o_kanban_primary_bottom:last').prepend(_t("Set an invoicing target: "));
         this.$target_input.focus();
+        let preLabel = _t("Set an invoicing target: ");
+        let postLabel = _t(" / Month");
+        const kanbanBottomBlock = this.$el.find('.o_kanban_primary_bottom.bottom_block:last-child');
+        if (this.recordData.currency_id) {
+            const currency = session.get_currency(this.recordData.currency_id.res_id);
+            if (currency.position === "after") {
+                postLabel = ` ${' ' + currency.symbol}${postLabel}`;
+            } else {
+               preLabel = `${preLabel} ${currency.symbol + ' '}`;
+            }
+        }
+        kanbanBottomBlock.prepend($('<span/>').text(preLabel));
+        kanbanBottomBlock.append($('<span/>').text(postLabel));
 
         this.$target_input.on({
             blur: this._onSalesTeamTargetSet.bind(this),
@@ -45,7 +58,7 @@ KanbanRecord.include({
         var self = this;
         var value = Number(this.$target_input.val());
         if (isNaN(value)) {
-            this.do_warn(_t("Wrong value entered!"), _t("Only Integer Value should be valid."));
+            this.displayNotification({ message: _t("Please enter an integer value"), type: 'danger' });
         } else {
             this.trigger_up('kanban_record_update', {
                 invoiced_target: value,
