@@ -12,11 +12,32 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
     def setUp(self):
         super(TestSaleCouponProgramNumbers, self).setUp()
 
-        self.largeCabinet = self.env.ref('product.product_product_6')
-        self.conferenceChair = self.env.ref('product.product_product_11')
-        self.pedalBin = self.env.ref('product.product_product_9')
-        self.drawerBlack = self.env.ref('product.product_product_16')
-        self.largeMeetingTable = self.env.ref('product.consu_delivery_02')
+        self.largeCabinet = self.env['product.product'].create({
+            'name': 'Large Cabinet',
+            'list_price': 320.0,
+            'taxes_id': False,
+        })
+        self.conferenceChair = self.env['product.product'].create({
+            'name': 'Conference Chair',
+            'list_price': 16.5,
+            'taxes_id': False,
+        })
+        self.pedalBin = self.env['product.product'].create({
+            'name': 'Pedal Bin',
+            'list_price': 47.0,
+            'taxes_id': False,
+        })
+        self.drawerBlack = self.env['product.product'].create({
+            'name': 'Drawer Black',
+            'list_price': 25.0,
+            'taxes_id': False,
+        })
+        self.largeMeetingTable = self.env['product.product'].create({
+            'name': 'Large Meeting Table',
+            'list_price': 40000.0,
+            'taxes_id': False,
+        })
+
         self.steve = self.env['res.partner'].create({
             'name': 'Steve Bucknor',
             'email': 'steve.bucknor@example.com',
@@ -25,7 +46,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'partner_id': self.steve.id
         })
 
-        self.p1 = self.env['sale.coupon.program'].create({
+        self.p1 = self.env['coupon.program'].create({
             'name': 'Code for 10% on orders',
             'promo_code_usage': 'code_needed',
             'promo_code': 'test_10pc',
@@ -33,7 +54,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'discount_percentage': 10.0,
             'program_type': 'promotion_program',
         })
-        self.p2 = self.env['sale.coupon.program'].create({
+        self.p2 = self.env['coupon.program'].create({
             'name': 'Buy 3 cabinets, get one for free',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'product',
@@ -42,7 +63,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'rule_min_quantity': 3,
             'rule_products_domain': '[["name","ilike","large cabinet"]]',
         })
-        self.p3 = self.env['sale.coupon.program'].create({
+        self.p3 = self.env['coupon.program'].create({
             'name': 'Buy 1 drawer black, get a free Large Meeting Table',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'product',
@@ -50,7 +71,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'reward_product_id': self.largeMeetingTable.id,
             'rule_products_domain': '[["name","ilike","drawer black"]]',
         })
-        self.discount_coupon_program = self.env['sale.coupon.program'].create({
+        self.discount_coupon_program = self.env['coupon.program'].create({
             'name': '$100 coupon',
             'program_type': 'coupon_program',
             'reward_type': 'discount',
@@ -144,7 +165,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'amount': 15,
             'price_include': True,
         })
-        p_specific_product = self.env['sale.coupon.program'].create({
+        p_specific_product = self.env['coupon.program'].create({
             'name': '20% reduction on Large Cabinet in cart',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'discount',
@@ -296,7 +317,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
 
         # Create needed programs
         self.p2.active = False
-        self.p_large_cabinet = self.env['sale.coupon.program'].create({
+        self.p_large_cabinet = self.env['coupon.program'].create({
             'name': 'Buy 1 large cabinet, get one for free',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'product',
@@ -304,7 +325,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'reward_product_id': self.largeCabinet.id,
             'rule_products_domain': '[["name","ilike","large cabinet"]]',
         })
-        self.p_conference_chair = self.env['sale.coupon.program'].create({
+        self.p_conference_chair = self.env['coupon.program'].create({
             'name': 'Buy 1 chair, get one for free',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'product',
@@ -312,7 +333,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'reward_product_id': self.conferenceChair.id,
             'rule_products_domain': '[["name","ilike","conference chair"]]',
         })
-        self.p_pedal_bin = self.env['sale.coupon.program'].create({
+        self.p_pedal_bin = self.env['coupon.program'].create({
             'name': 'Buy 1 bin, get one for free',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'product',
@@ -380,7 +401,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         # -- End test inside the test
 
         # Now we want to apply a 20% discount only on Large Cabinet
-        self.env['sale.coupon.program'].create({
+        self.env['coupon.program'].create({
             'name': '20% reduction on Large Cabinet in cart',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'discount',
@@ -474,7 +495,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
     def test_program_fixed_price(self):
         # Check fixed amount discount
         order = self.empty_order
-        fixed_amount_program = self.env['sale.coupon.program'].create({
+        fixed_amount_program = self.env['coupon.program'].create({
             'name': '$249 discount',
             'promo_code_usage': 'no_code_needed',
             'program_type': 'promotion_program',
@@ -509,7 +530,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
 
     def test_program_next_order(self):
         order = self.empty_order
-        self.env['sale.coupon.program'].create({
+        self.env['coupon.program'].create({
             'name': 'Free Pedal Bin if at least 1 article',
             'promo_code_usage': 'no_code_needed',
             'promo_applicability': 'on_next_order',
@@ -560,7 +581,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'order_id': order.id,
         })
         self.assertEqual(order.amount_total, 165.0, "The order amount is not correct")
-        self.env['sale.coupon.generate'].with_context(active_id=self.discount_coupon_program.id).create({}).generate_coupon()
+        self.env['coupon.generate.wizard'].with_context(active_id=self.discount_coupon_program.id).create({}).generate_coupon()
         coupon = self.discount_coupon_program.coupon_ids[0]
         self.env['sale.coupon.apply.code'].with_context(active_id=order.id).create({
             'coupon_code': coupon.code
@@ -583,17 +604,17 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         })
         self.assertEqual(order.amount_total, 165.0, "The order amount is not correct")
 
-        discount_promotion_program = self.env['sale.coupon.program'].create({
-             'name': '$100 promotion program',
-             'program_type': 'promotion_program',
-             'promo_code_usage': 'code_needed',
-             'promo_code': 'testpromo',
-             'reward_type': 'discount',
-             'discount_type': 'fixed_amount',
-             'discount_fixed_amount': 100,
-             'active': True,
-             'discount_apply_on': 'on_order',
-             'rule_minimum_amount': 100.00,
+        self.env['coupon.program'].create({
+            'name': '$100 promotion program',
+            'program_type': 'promotion_program',
+            'promo_code_usage': 'code_needed',
+            'promo_code': 'testpromo',
+            'reward_type': 'discount',
+            'discount_type': 'fixed_amount',
+            'discount_fixed_amount': 100,
+            'active': True,
+            'discount_apply_on': 'on_order',
+            'rule_minimum_amount': 100.00,
         })
 
         self.env['sale.coupon.apply.code'].with_context(active_id=order.id).create({
@@ -603,7 +624,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         order.recompute_coupon_lines()
         self.assertEqual(order.amount_total, 65.0, "The promotion program should not be removed after recomputation")
 
-        self.env['sale.coupon.generate'].with_context(active_id=self.discount_coupon_program.id).create({}).generate_coupon()
+        self.env['coupon.generate.wizard'].with_context(active_id=self.discount_coupon_program.id).create({}).generate_coupon()
         coupon = self.discount_coupon_program.coupon_ids[0]
         with self.assertRaises(UserError):
             self.env['sale.coupon.apply.code'].with_context(active_id=order.id).create({
@@ -629,7 +650,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             * Even applying the coupon in reverse order should yield same result
         """
 
-        coupon_program = self.env['sale.coupon.program'].create({
+        coupon_program = self.env['coupon.program'].create({
             'name': '$288.5 coupon',
             'program_type': 'coupon_program',
             'reward_type': 'discount',
@@ -672,7 +693,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             }).process_coupon()
         self.assertEqual(order.amount_total, 283.5, "The promotion program should be correctly applied")
 
-        self.env['sale.coupon.generate'].with_context(active_id=coupon_program.id).create({
+        self.env['coupon.generate.wizard'].with_context(active_id=coupon_program.id).create({
             'generation_type': 'nbr_coupon',
             'nbr_coupons': 1,
         }).generate_coupon()
@@ -719,7 +740,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             * Even applying the coupon in reverse order should yield same result
         """
 
-        coupon_program = self.env['sale.coupon.program'].create({
+        coupon_program = self.env['coupon.program'].create({
             'name': '$290 coupon',
             'program_type': 'coupon_program',
             'reward_type': 'discount',
@@ -730,7 +751,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         })
 
         order = self.empty_order
-        orderline = self.env['sale.order.line'].create([
+        self.env['sale.order.line'].create([
         {
             'product_id': self.conferenceChair.id,
             'name': 'Conference Chair',
@@ -762,7 +783,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             }).process_coupon()
         self.assertEqual(order.amount_total, 270.0, "The promotion program should be correctly applied")
 
-        self.env['sale.coupon.generate'].with_context(active_id=coupon_program.id).create({
+        self.env['coupon.generate.wizard'].with_context(active_id=coupon_program.id).create({
             'generation_type': 'nbr_coupon',
             'nbr_coupons': 1,
         }).generate_coupon()
@@ -772,10 +793,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         }).process_coupon()
         self.assertEqual(order.amount_total, 0.0, "The promotion program should not make the order total go below 0")
         order.recompute_coupon_lines()
-        #TODO fix numbers
-        self.assertEqual(order.amount_total, 9.09, "The promotion program should not be altered after recomputation")
-        self.assertEqual(order.amount_tax, 8.18)
-        self.assertEqual(order.amount_untaxed, 0.91)
+        self.assertEqual(order.amount_total, 0.0, "The promotion program should not be altered after recomputation")
 
         order.order_line[3:].unlink() #remove all coupon
 
@@ -790,10 +808,38 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
                 'coupon_code': 'test_10pc'
             }).process_coupon()
         order.recompute_coupon_lines()
-        #TODO fix numbers
-        self.assertEqual(order.amount_tax, 9.01)
-        self.assertEqual(order.amount_untaxed, 0.08)
-        self.assertEqual(order.amount_total, 9.09, "The promotion program should not be altered after recomputation")
+        self.assertEqual(order.amount_total, 0.0, "The promotion program should not be altered after recomputation")
+
+    def test_program_percentage_discount_on_product_included_tax(self):
+        # test 100% percentage discount (tax included)
+
+        program = self.env['coupon.program'].create({
+            'name': '100% discount',
+            'promo_code_usage': 'no_code_needed',
+            'program_type': 'promotion_program',
+            'discount_percentage': 100.0,
+            'rule_minimum_amount_tax_inclusion': 'tax_included',
+        })
+        self.tax_10pc_incl.price_include = True
+
+        self.drawerBlack.taxes_id = self.tax_10pc_incl
+        order = self.empty_order
+        order.order_line = self.env['sale.order.line'].create({
+            'product_id': self.drawerBlack.id,
+            'product_uom_qty': 1.0,
+            'order_id': order.id,
+        })
+        order.recompute_coupon_lines()
+        self.assertEqual(len(order.order_line.ids), 2, "The discount should be applied")
+        self.assertEqual(order.amount_total, 0.0, "Order should be 0 as it is a 100% discount")
+
+        # test 95% percentage discount (tax included)
+        program.discount_percentage = 95
+        order.recompute_coupon_lines()
+        # lst_price is 25$ so total now should be 1.25$ (1.14$ + 0.11$ taxes)
+        self.assertEqual(len(order.order_line.ids), 2, "The discount should be applied")
+        self.assertAlmostEqual(order.amount_tax, 0.11, places=2)
+        self.assertAlmostEqual(order.amount_untaxed, 1.14, places=2)
 
     def test_program_discount_on_multiple_specific_products(self):
         """ Ensure a discount on multiple specific products is correctly computed.
@@ -801,7 +847,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             - Advanced: This discount must be split by different taxes
         """
         order = self.empty_order
-        p_specific_products = self.env['sale.coupon.program'].create({
+        p_specific_products = self.env['coupon.program'].create({
             'name': '20% reduction on Conference Chair and Drawer Black in cart',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'discount',
@@ -881,7 +927,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         # auto applied promotion program
 
         order = self.empty_order
-        self.p3 = self.env['sale.coupon.program'].create({
+        self.p3 = self.env['coupon.program'].create({
             'name': 'Buy 2 Chairs, get 1 free',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'product',
@@ -925,7 +971,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'lst_price': 118.0,
         })
 
-        test10 = self.env['sale.coupon.program'].create({
+        self.env['coupon.program'].create({
             'name': '10% discount',
             'promo_code_usage': 'no_code_needed',
             'program_type': 'promotion_program',
@@ -934,7 +980,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'rule_minimum_amount': 1500.0,
             'rule_minimum_amount_tax_inclusion': 'tax_included',
         })
-        test15 = self.env['sale.coupon.program'].create({
+        self.env['coupon.program'].create({
             'name': '15% discount',
             'promo_code_usage': 'no_code_needed',
             'program_type': 'promotion_program',
@@ -943,7 +989,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'rule_minimum_amount': 1750.0,
             'rule_minimum_amount_tax_inclusion': 'tax_included',
         })
-        test20 = self.env['sale.coupon.program'].create({
+        self.env['coupon.program'].create({
             'name': '20% discount',
             'promo_code_usage': 'no_code_needed',
             'program_type': 'promotion_program',
@@ -952,7 +998,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'rule_minimum_amount': 2000.0,
             'rule_minimum_amount_tax_inclusion': 'tax_included',
         })
-        test25 = self.env['sale.coupon.program'].create({
+        self.env['coupon.program'].create({
             'name': '25% discount',
             'promo_code_usage': 'no_code_needed',
             'program_type': 'promotion_program',
@@ -1001,7 +1047,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
 
     def test_program_free_prods_with_min_qty_and_reward_qty_and_rule(self):
         order = self.empty_order
-        coupon_program = self.env['sale.coupon.program'].create({
+        coupon_program = self.env['coupon.program'].create({
             'name': '2 free conference chair if at least 1 large cabinet',
             'promo_code_usage': 'code_needed',
             'program_type': 'promotion_program',
@@ -1016,7 +1062,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         self.conferenceChair.write({'list_price': 100, 'sale_ok': True})
 
         # create SOL
-        sol1 = self.env['sale.order.line'].create({
+        self.env['sale.order.line'].create({
             'product_id': self.largeCabinet.id,
             'name': 'Large Cabinet',
             'product_uom_qty': 1.0,
@@ -1033,7 +1079,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         self.assertEqual(order.amount_total, 700.0, 'The price must be 500.0 since the coupon is not yet applied')
 
         # generate and apply coupon
-        self.env['sale.coupon.generate'].with_context(active_id=coupon_program.id).create({
+        self.env['coupon.generate.wizard'].with_context(active_id=coupon_program.id).create({
             'generation_type': 'nbr_coupon',
             'nbr_coupons': 1,
         }).generate_coupon()
@@ -1119,7 +1165,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
             'sale_ok': True,
             'taxes_id': [(6, 0, [])],
         })
-        self.env['sale.coupon.program'].create({
+        self.env['coupon.program'].create({
             'name': '1 Product F = 5$ discount',
             'promo_code_usage': 'no_code_needed',
             'reward_type': 'discount',
@@ -1148,7 +1194,7 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         self.assertEqual(order.order_line.filtered(lambda x: x.is_reward_line).price_unit, -5, 'The discount unit price should still be -5 after the quantity was manually changed')
 
     def test_program_maximum_use_number_with_other_promo(self):
-        self.env['sale.coupon.program'].create({
+        self.env['coupon.program'].create({
             'name': '20% discount on first order',
             'promo_code_usage': 'no_code_needed',
             'program_type': 'promotion_program',
@@ -1179,3 +1225,28 @@ class TestSaleCouponProgramNumbers(TestSaleCouponCommon):
         order2.recompute_coupon_lines()
         self.assertEqual(order2.amount_total, 22.5, "10% discount should be applied")
         self.assertEqual(len(order2.order_line.ids), 2, "discount should be applied")
+
+    def test_program_maximum_use_number_last_order(self):
+        # reuse p1 with a different promo code and a maximum_use_number of 1
+        self.p1.promo_code = 'promo1'
+        self.p1.maximum_use_number = 1
+
+        # check that the discount is applied on the first order
+        order = self.empty_order
+        self.env['sale.order.line'].create({
+            'product_id': self.drawerBlack.id,
+            'product_uom_qty': 1.0,
+            'order_id': order.id,
+        })
+        self.env['sale.coupon.apply.code'].sudo().apply_coupon(order, 'promo1')
+        order.recompute_coupon_lines()
+        self.assertEqual(order.amount_total, 22.5, "10% discount should be applied")
+        self.assertEqual(len(order.order_line.ids), 2, "discount should be applied")
+
+        # duplicating to mimic website behavior (each refresh to /shop/cart
+        # recompute coupon lines if website_sale_coupon is installed)
+        order.recompute_coupon_lines()
+        self.assertEqual(order.amount_total, 22.5, "10% discount should be applied")
+        self.assertEqual(len(order.order_line.ids), 2, "discount should be applied")
+        # applying the code again should return that it has been expired.
+        self.assertDictEqual(self.env['sale.coupon.apply.code'].sudo().apply_coupon(order, 'promo1'), {'error': 'Promo code promo1 has been expired.'})
