@@ -15,8 +15,6 @@ This module allow to reinvoice employee expense, by setting the SO directly on t
 """,
     'depends': ['sale_management', 'hr_expense'],
     'data': [
-        'data/digest_data.xml',
-        'views/assets.xml',
         'views/product_view.xml',
         'views/hr_expense_views.xml',
         'views/sale_order_views.xml',
@@ -25,5 +23,13 @@ This module allow to reinvoice employee expense, by setting the SO directly on t
     'test': [],
     'installable': True,
     'auto_install': True,
+    'assets': {
+        'web.assets_backend': [
+            'sale_expense/static/src/**/*',
+        ],
+        'web.qunit_suite_tests': [
+            'sale_expense/static/tests/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
