@@ -49,8 +49,8 @@ class TestPackingNeg(TransactionCase):
             })],
         }
         pick_neg = self.env['stock.picking'].create(vals)
-        pick_neg.onchange_picking_type()
-        pick_neg.move_lines.onchange_product_id()
+        pick_neg._onchange_picking_type()
+        pick_neg.move_lines._onchange_product_id()
 
         # Confirm and assign picking
         pick_neg.action_confirm()
@@ -87,7 +87,7 @@ class TestPackingNeg(TransactionCase):
         })
 
         # Transfer the receipt
-        pick_neg.action_done()
+        pick_neg._action_done()
 
         # Make a delivery order of 300 pieces to the customer
         vals = {
@@ -106,8 +106,8 @@ class TestPackingNeg(TransactionCase):
             })],
         }
         delivery_order_neg = self.env['stock.picking'].create(vals)
-        delivery_order_neg.onchange_picking_type()
-        delivery_order_neg.move_lines.onchange_product_id()
+        delivery_order_neg._onchange_picking_type()
+        delivery_order_neg.move_lines._onchange_product_id()
 
         # Assign and confirm
         delivery_order_neg.action_confirm()
@@ -130,7 +130,7 @@ class TestPackingNeg(TransactionCase):
                 rec.result_package_id = False
 
         # Process this picking
-        delivery_order_neg.action_done()
+        delivery_order_neg._action_done()
 
         # Check the quants that you have -20 pieces pallet 2 in stock, and a total quantity
         # of 50 in stock from pallet 3 (should be 20+30, as it has been split by reservation)
@@ -163,8 +163,8 @@ class TestPackingNeg(TransactionCase):
             })],
         }
         delivery_reconcile = self.env['stock.picking'].create(vals)
-        delivery_reconcile.onchange_picking_type()
-        delivery_reconcile.move_lines.onchange_product_id()
+        delivery_reconcile._onchange_picking_type()
+        delivery_reconcile.move_lines._onchange_product_id()
 
         # Receive 20 products with lot neg in stock with a new incoming shipment that should be on pallet 2
         delivery_reconcile.action_confirm()
@@ -173,7 +173,7 @@ class TestPackingNeg(TransactionCase):
             ('name', '=', 'Lot neg')], limit=1)
         pack = self.env["stock.quant.package"].search([('name', '=', 'Palneg 2')], limit=1)
         delivery_reconcile.move_line_ids[0].write({'lot_id': lot.id, 'qty_done': 20.0, 'result_package_id': pack.id})
-        delivery_reconcile.action_done()
+        delivery_reconcile._action_done()
 
         # Check the negative quant was reconciled
         neg_quants = self.env['stock.quant'].search([
