@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from odoo.tests import common
 from odoo.exceptions import ValidationError
+from odoo import Command
 
 
 class test_inherits(common.TransactionCase):
@@ -49,19 +50,19 @@ class test_inherits(common.TransactionCase):
     def test_write_4_one2many(self):
         """ Check that we can write on an inherited one2many field. """
         box = self.env.ref('test_inherits.box_a')
-        box.write({'line_ids': [(0, 0, {'name': 'Line 1'})]})
+        box.write({'line_ids': [Command.create({'name': 'Line 1'})]})
         self.assertTrue(all(box.line_ids._ids))
         self.assertEqual(box.line_ids.mapped('name'), ['Line 1'])
         self.assertEqual(box.line_ids, box.unit_id.line_ids)
         box.flush()
         box.invalidate_cache(['line_ids'])
-        box.write({'line_ids': [(0, 0, {'name': 'Line 2'})]})
+        box.write({'line_ids': [Command.create({'name': 'Line 2'})]})
         self.assertTrue(all(box.line_ids._ids))
         self.assertEqual(box.line_ids.mapped('name'), ['Line 1', 'Line 2'])
         self.assertEqual(box.line_ids, box.unit_id.line_ids)
         box.flush()
         box.invalidate_cache(['line_ids'])
-        box.write({'line_ids': [(1, box.line_ids[0].id, {'name': 'First line'})]})
+        box.write({'line_ids': [Command.update(box.line_ids[0].id, {'name': 'First line'})]})
         self.assertTrue(all(box.line_ids._ids))
         self.assertEqual(box.line_ids.mapped('name'), ['First line', 'Line 2'])
         self.assertEqual(box.line_ids, box.unit_id.line_ids)
@@ -121,7 +122,7 @@ class test_inherits(common.TransactionCase):
 
     def test_display_name(self):
         """ Check the 'display_name' of an inherited translated 'name'. """
-        self.env['res.lang'].load_lang('fr_FR')
+        self.env['res.lang']._activate_lang('fr_FR')
 
         # concrete check
         pallet_en = self.env['test.pallet'].create({'name': 'Bread'})
@@ -135,9 +136,9 @@ class test_inherits(common.TransactionCase):
         Box = type(self.env['test.box'])
         Pallet = type(self.env['test.pallet'])
         self.assertTrue(Unit.name.translate)
-        self.assertIn('lang', Unit.display_name.depends_context)
-        self.assertIn('lang', Box.display_name.depends_context)
-        self.assertIn('lang', Pallet.display_name.depends_context)
+        self.assertIn('lang', self.registry.field_depends_context[Unit.display_name])
+        self.assertIn('lang', self.registry.field_depends_context[Box.display_name])
+        self.assertIn('lang', self.registry.field_depends_context[Pallet.display_name])
 
     def test_multi_write_m2o_inherits(self):
         """Verify that an inherits m2o field can be written to in batch"""
@@ -147,4 +148,4 @@ class test_inherits(common.TransactionCase):
         unit_bar = self.env['test.unit'].create({'name': 'bar'})
         boxes.unit_id = unit_bar
 
-        self.assertEquals(boxes.mapped('unit_id.name'), ['bar'])
+        self.assertEqual(boxes.mapped('unit_id.name'), ['bar'])
