@@ -3,7 +3,7 @@
 {
     'name': 'Test Full eLearning Flow',
     'version': '1.0',
-    'category': 'Tools',
+    'category': 'Hidden/Tests',
     'description': """
 This module will test the main certification flow of Odoo.
 It will install the e-learning, survey and e-commerce apps and make a complete
@@ -16,12 +16,17 @@ certification flow including purchase, certification, failure and success.
         'website_slides_survey',
         'payment_test'
     ],
+    'data': [
+        'data/res_groups_data.xml',
+    ],
     'demo': [
         'data/product_demo.xml',
     ],
-    'data': [
-        'views/assets.xml',
-    ],
     'installable': True,
+    'assets': {
+        'web.assets_tests': [
+            'test_website_slides_full/tests/tours/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
