@@ -9,7 +9,7 @@ from datetime import datetime, timedelta
 from lxml import etree
 import logging
 
-from odoo.tests.common import BaseCase, tagged
+from odoo.tests.common import BaseCase, HttpCase, tagged
 from odoo.tools import topological_sort
 from odoo.addons.web.controllers.main import HomeStaticTemplateHelpers
 
@@ -51,9 +51,9 @@ class TestStaticInheritanceCommon(BaseCase):
     def setUp(self):
         super(TestStaticInheritanceCommon, self).setUp()
         # output is "manifest_glob" return
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
-            ('module_2_file_1', None, 'module_2'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+            ('module_2_file_1', 'module_2', 'bundle_1'),
         ]
 
         self.template_files = {
@@ -111,18 +111,18 @@ class TestStaticInheritanceCommon(BaseCase):
 
     # Private methods
     def _get_module_names(self):
-        return ','.join([glob[2] for glob in self.modules])
+        return ','.join([asset_path[1] for asset_path in self.asset_paths])
 
     def _set_patchers(self):
-        def _patched_for_manifest_glob(*args, **kwargs):
+        def _patched_for_get_asset_paths(*args, **kwargs):
             # Ordered by module
-            return self.modules
+            return self.asset_paths
 
         def _patch_for_read_addon_file(*args, **kwargs):
             return self.template_files[args[1]]
 
         self.patchers = [
-            patch.object(HomeStaticTemplateHelpers, '_manifest_glob', _patched_for_manifest_glob),
+            patch.object(HomeStaticTemplateHelpers, '_get_asset_paths', _patched_for_get_asset_paths),
             patch.object(HomeStaticTemplateHelpers, '_read_addon_file', _patch_for_read_addon_file),
         ]
 
@@ -145,10 +145,9 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
                 </form>
                 <t t-name="template_1_2">
                     <div>And I grew strong</div>
-                    <!-- Modified by anonymous_template_2 from module_2 -->
                     <div>And I learned how to get along</div>
                 </t>
-                <form t-name="template_2_1" random-attr="gloria" t-inherit="module_1.template_1_1">
+                <form t-name="template_2_1" random-attr="gloria">
                     <div>At first I was afraid</div>
                     <div>I was petrified</div>
                     <div>But then I spent so many nights thinking how you did me wrong</div>
@@ -162,10 +161,90 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
 
         self.assertXMLEqual(contents, expected)
 
+    def test_static_inheritance_02(self):
+        self.template_files = {
+            'module_1_file_1': b'''
+                <templates id="template" xml:space="preserve">
+                    <form t-name="template_1_1" random-attr="gloria">
+                        <div>At first I was afraid</div>
+                        <div>Kept thinking I could never live without you by my side</div>
+                    </form>
+                    <form t-name="template_1_2" t-inherit="template_1_1" added="true">
+                        <xpath expr="//div[1]" position="after">
+                            <div>I was petrified</div>
+                        </xpath>
+                    </form>
+                </templates>
+            '''
+        }
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+        ]
+        contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
+        expected = b"""
+            <templates>
+                <form t-name="template_1_1" random-attr="gloria">
+                    <div>At first I was afraid</div>
+                    <div>Kept thinking I could never live without you by my side</div>
+                </form>
+                <form t-name="template_1_2" random-attr="gloria" added="true">
+                    <div>At first I was afraid</div>
+                    <div>I was petrified</div>
+                    <div>Kept thinking I could never live without you by my side</div>
+                </form>
+            </templates>
+        """
+
+        self.assertXMLEqual(contents, expected)
+
+    def test_static_inheritance_03(self):
+        self.maxDiff = None
+        self.template_files = {
+            'module_1_file_1': b'''
+                <templates id="template" xml:space="preserve">
+                    <form t-name="template_1_1">
+                        <div>At first I was afraid</div>
+                        <div>Kept thinking I could never live without you by my side</div>
+                    </form>
+                    <form t-name="template_1_2" t-inherit="template_1_1" added="true">
+                        <xpath expr="//div[1]" position="after">
+                            <div>I was petrified</div>
+                        </xpath>
+                    </form>
+                    <form t-name="template_1_3" t-inherit="template_1_2" added="false" other="here">
+                        <xpath expr="//div[2]" position="replace"/>
+                    </form>
+                </templates>
+            '''
+        }
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+        ]
+        contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
+        expected = b"""
+            <templates>
+                <form t-name="template_1_1">
+                    <div>At first I was afraid</div>
+                    <div>Kept thinking I could never live without you by my side</div>
+                </form>
+                <form t-name="template_1_2" added="true">
+                    <div>At first I was afraid</div>
+                    <div>I was petrified</div>
+                    <div>Kept thinking I could never live without you by my side</div>
+                </form>
+                <form t-name="template_1_3" added="false" other="here">
+                    <div>At first I was afraid</div>
+                    <div>Kept thinking I could never live without you by my side</div>
+                </form>
+            </templates>
+        """
+
+        self.assertXMLEqual(contents, expected)
+
     def test_static_inheritance_in_same_module(self):
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
-            ('module_1_file_2', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+            ('module_1_file_2', 'module_1', 'bundle_1'),
         ]
 
         self.template_files = {
@@ -195,7 +274,7 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
                     <div>At first I was afraid</div>
                     <div>Kept thinking I could never live without you by my side</div>
                 </form>
-                <form t-name="template_1_2" t-inherit="template_1_1">
+                <form t-name="template_1_2">
                     <div>At first I was afraid</div>
                     <div>I was petrified</div>
                     <div>Kept thinking I could never live without you by my side</div>
@@ -206,8 +285,8 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         self.assertXMLEqual(contents, expected)
 
     def test_static_inheritance_in_same_file(self):
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
 
         self.template_files = {
@@ -232,7 +311,7 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
                     <div>At first I was afraid</div>
                     <div>Kept thinking I could never live without you by my side</div>
                 </form>
-                <form t-name="template_1_2" t-inherit="template_1_1">
+                <form t-name="template_1_2">
                     <div>At first I was afraid</div>
                     <div>I was petrified</div>
                     <div>Kept thinking I could never live without you by my side</div>
@@ -243,8 +322,8 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         self.assertXMLEqual(contents, expected)
 
     def test_static_inherit_extended_template(self):
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b'''
@@ -271,11 +350,10 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
             <templates>
                 <form t-name="template_1_1">
                     <div>At first I was afraid</div>
-                    <!-- Modified by template_1_2 from module_1 -->
                     <div>I was petrified</div>
                     <div>Kept thinking I could never live without you by my side</div>
                 </form>
-                <form t-name="template_1_3" t-inherit="template_1_1">
+                <form t-name="template_1_3">
                     <div>At first I was afraid</div>
                     <div>I was petrified</div>
                     <div>Kept thinking I could never live without you by my side</div>
@@ -287,10 +365,10 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         self.assertXMLEqual(contents, expected)
 
     def test_sibling_extension(self):
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
-            ('module_2_file_1', None, 'module_2'),
-            ('module_3_file_1', None, 'module_3'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+            ('module_2_file_1', 'module_2', 'bundle_1'),
+            ('module_3_file_1', 'module_3', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b'''
@@ -328,9 +406,7 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
             <templates>
                 <form t-name="template_1_1">
                     <div>I am a man of constant sorrow</div>
-                    <!-- Modified by template_2_1 from module_2 -->
                     <div>In constant sorrow all through his days</div>
-                    <!-- Modified by template_3_1 from module_3 -->
                     <div>Oh Brother !</div>
                     <div>I've seen trouble all my days</div>
                 </form>
@@ -340,7 +416,7 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         self.assertXMLEqual(contents, expected)
 
     def test_static_misordered_modules(self):
-        self.modules.reverse()
+        self.asset_paths.reverse()
         with self.assertRaises(ValueError) as ve:
             HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
 
@@ -374,8 +450,8 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         """
         Replacing a template's meta definition in place doesn't keep the original attrs of the template
         """
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b"""
@@ -395,17 +471,17 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
         expected = b"""
             <templates>
-                <form overriden-attr="overriden">
-                    <!-- Modified by template_1_2 from module_1 -->And I grew strong
-                </form>
+                <div overriden-attr="overriden" t-name="template_1_1">
+                    And I grew strong
+                </div>
             </templates>
         """
 
         self.assertXMLEqual(contents, expected)
 
     def test_replace_in_debug_mode2(self):
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b"""
@@ -429,11 +505,11 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
         expected = b"""
             <templates>
-                <form>
-                    <!-- Modified by template_1_2 from module_1 -->And I grew strong
+                <div t-name="template_1_1">
+                    And I grew strong
                     <p>And I learned how to get along</p>
                     And so you're back
-                </form>
+                </div>
             </templates>
         """
 
@@ -444,8 +520,8 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         becomes outside of the template
         This doesn't mean anything in terms of the business of template inheritance
         But it is in the XPATH specs"""
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b"""
@@ -469,11 +545,10 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
         expected = b"""
             <templates>
-                <form>
-                    <!-- Modified by template_1_2 from module_1 -->
+                <div t-name="template_1_1">
                     And I grew strong
                     <p>And I learned how to get along</p>
-                </form>
+                </div>
                 And so you're back
             </templates>
         """
@@ -482,10 +557,10 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
 
     def test_replace_root_node_tag(self):
         """
-        Root node is not targeted by //NODE_TAG in xpath
+        Root node IS targeted by //NODE_TAG in xpath
         """
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b"""
@@ -508,11 +583,9 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
         expected = b"""
             <templates>
-                <form t-name="template_1_1" random-attr="gloria">
-                    <div>At first I was afraid</div>
-                    <!-- Modified by template_1_2 from module_1 -->
-                    <div>Form replacer</div>
-                </form>
+                <div t-name="template_1_1">
+                    Form replacer
+                </div>
             </templates>
         """
 
@@ -520,11 +593,11 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
 
     def test_replace_root_node_tag_in_primary(self):
         """
-        Root node is not targeted by //NODE_TAG in xpath
+        Root node IS targeted by //NODE_TAG in xpath
         """
         self.maxDiff = None
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b"""
@@ -533,11 +606,11 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
                         <div>At first I was afraid</div>
                         <form>Inner Form</form>
                     </form>
-                    <t t-name="template_1_2" t-inherit="template_1_1" t-inherit-mode="primary">
+                    <form t-name="template_1_2" t-inherit="template_1_1" t-inherit-mode="primary">
                         <xpath expr="//form" position="replace">
                             <div>Form replacer</div>
                         </xpath>
-                    </t>
+                    </form>
                 </templates>
                 """,
         }
@@ -549,10 +622,9 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
                     <div>At first I was afraid</div>
                     <form>Inner Form</form>
                 </form>
-                <form t-name="template_1_2" random-attr="gloria" t-inherit="template_1_1">
-                    <div>At first I was afraid</div>
-                    <div>Form replacer</div>
-                </form>
+                <div t-name="template_1_2">
+                    Form replacer
+                </div>
             </templates>
         """
 
@@ -563,8 +635,8 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         The inheriting template has got both its own defining attrs
         and new ones if one is to replace its defining root node
         """
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b"""
@@ -590,10 +662,10 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
                 <form t-name="template_1_1" random-attr="gloria">
                     <div>At first I was afraid</div>
                  </form>
-                 <form overriden-attr="overriden" t-name="template_1_2" t-inherit="template_1_1">
+                 <div overriden-attr="overriden" t-name="template_1_2">
                     And I grew strong
                     <p>And I learned how to get along</p>
-                 </form>
+                 </div>
             </templates>
         """
 
@@ -601,8 +673,8 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
 
     def test_replace_in_nodebug_mode1(self):
         """Comments already in the arch are ignored"""
-        self.modules = [
-            ('module_1_file_1', None, 'module_1'),
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
         ]
         self.template_files = {
             'module_1_file_1': b"""
@@ -627,16 +699,236 @@ class TestStaticInheritance(TestStaticInheritanceCommon):
         contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=False)
         expected = b"""
             <templates>
-                <form>
+                <div t-name="template_1_1">
                     And I grew strong
                     <p>And I learned how to get along</p>
                     And so you're back
-                </form>
+                </div>
             </templates>
         """
 
         self.assertXMLEqual(contents, expected)
 
+    def test_inherit_from_dotted_tname_1(self):
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+        ]
+        self.template_files = {
+            'module_1_file_1': b"""
+                <templates id="template" xml:space="preserve">
+                    <form t-name="module_1.template_1_1.dot" random-attr="gloria">
+                        <div>At first I was afraid</div>
+                    </form>
+                    <t t-name="template_1_2" t-inherit="template_1_1.dot" t-inherit-mode="primary">
+                        <xpath expr="." position="replace">
+                            <div overriden-attr="overriden">
+                                And I grew strong
+                                <p>And I learned how to get along</p>
+                            </div>
+                        </xpath>
+                    </t>
+                </templates>
+                """,
+        }
+
+        contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
+        expected = b"""
+            <templates>
+                <form t-name="module_1.template_1_1.dot" random-attr="gloria">
+                    <div>At first I was afraid</div>
+                 </form>
+                 <div overriden-attr="overriden" t-name="template_1_2">
+                    And I grew strong
+                    <p>And I learned how to get along</p>
+                 </div>
+            </templates>
+        """
+
+        self.assertXMLEqual(contents, expected)
+
+    def test_inherit_from_dotted_tname_2(self):
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+        ]
+        self.template_files = {
+            'module_1_file_1': b"""
+                <templates id="template" xml:space="preserve">
+                    <form t-name="template_1_1.dot" random-attr="gloria">
+                        <div>At first I was afraid</div>
+                    </form>
+                    <t t-name="template_1_2" t-inherit="template_1_1.dot" t-inherit-mode="primary">
+                        <xpath expr="." position="replace">
+                            <div overriden-attr="overriden">
+                                And I grew strong
+                                <p>And I learned how to get along</p>
+                            </div>
+                        </xpath>
+                    </t>
+                </templates>
+                """,
+        }
+
+        contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
+        expected = b"""
+            <templates>
+                <form t-name="template_1_1.dot" random-attr="gloria">
+                    <div>At first I was afraid</div>
+                 </form>
+                 <div overriden-attr="overriden" t-name="template_1_2">
+                    And I grew strong
+                    <p>And I learned how to get along</p>
+                 </div>
+            </templates>
+        """
+
+        self.assertXMLEqual(contents, expected)
+
+    def test_inherit_from_dotted_tname_2bis(self):
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+        ]
+        self.template_files = {
+            'module_1_file_1': b"""
+                <templates id="template" xml:space="preserve">
+                    <form t-name="template_1_1.dot" random-attr="gloria">
+                        <div>At first I was afraid</div>
+                    </form>
+                    <t t-name="template_1_2" t-inherit="module_1.template_1_1.dot" t-inherit-mode="primary">
+                        <xpath expr="." position="replace">
+                            <div overriden-attr="overriden">
+                                And I grew strong
+                                <p>And I learned how to get along</p>
+                            </div>
+                        </xpath>
+                    </t>
+                </templates>
+                """,
+        }
+
+        contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
+        expected = b"""
+            <templates>
+                <form t-name="template_1_1.dot" random-attr="gloria">
+                    <div>At first I was afraid</div>
+                 </form>
+                 <div overriden-attr="overriden" t-name="template_1_2">
+                    And I grew strong
+                    <p>And I learned how to get along</p>
+                 </div>
+            </templates>
+        """
+
+        self.assertXMLEqual(contents, expected)
+
+    def test_inherit_from_dotted_tname_2ter(self):
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+        ]
+        self.template_files = {
+            'module_1_file_1': b"""
+                <templates id="template" xml:space="preserve">
+                    <form t-name="module_1" random-attr="gloria">
+                        <div>At first I was afraid</div>
+                    </form>
+                    <t t-name="template_1_2" t-inherit="module_1" t-inherit-mode="primary">
+                        <xpath expr="." position="replace">
+                            <div overriden-attr="overriden">
+                                And I grew strong
+                                <p>And I learned how to get along</p>
+                            </div>
+                        </xpath>
+                    </t>
+                </templates>
+                """,
+        }
+
+        contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
+        expected = b"""
+            <templates>
+                <form t-name="module_1" random-attr="gloria">
+                    <div>At first I was afraid</div>
+                 </form>
+                 <div overriden-attr="overriden" t-name="template_1_2">
+                    And I grew strong
+                    <p>And I learned how to get along</p>
+                 </div>
+            </templates>
+        """
+
+        self.assertXMLEqual(contents, expected)
+
+    def test_inherit_from_dotted_tname_3(self):
+        self.asset_paths = [
+            ('module_1_file_1', 'module_1', 'bundle_1'),
+            ('module_2_file_1', 'module_2', 'bundle_1'),
+        ]
+        self.template_files = {
+            'module_1_file_1': b"""
+                <templates id="template" xml:space="preserve">
+                    <form t-name="module_1.template_1_1.dot" random-attr="gloria">
+                        <div>At first I was afraid</div>
+                    </form>
+                </templates>
+                """,
+
+            'module_2_file_1': b"""
+                <templates id="template" xml:space="preserve">
+                    <t t-name="template_2_1" t-inherit="module_1.template_1_1.dot" t-inherit-mode="primary">
+                        <xpath expr="." position="replace">
+                            <div overriden-attr="overriden">
+                                And I grew strong
+                                <p>And I learned how to get along</p>
+                            </div>
+                        </xpath>
+                    </t>
+                </templates>
+            """
+        }
+
+        contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
+        expected = b"""
+            <templates>
+                <form t-name="module_1.template_1_1.dot" random-attr="gloria">
+                    <div>At first I was afraid</div>
+                 </form>
+                 <div overriden-attr="overriden" t-name="template_2_1">
+                    And I grew strong
+                    <p>And I learned how to get along</p>
+                 </div>
+            </templates>
+        """
+
+        self.assertXMLEqual(contents, expected)
+
+@tagged('static_templates')
+class TestHttpStaticInheritance(HttpCase):
+    def test_static_attachments(self):
+        url = '/test_module/test_file.xml'
+        self.env['ir.attachment'].create({
+            'name': 'test_attachment',
+            'url': url,
+            'res_model': 'ir.ui.view',
+            'type': 'binary',
+            'raw': b"""
+                <templates>
+                    <t t-name="test_template">
+                        <div class="test_div" />
+                    </t>
+                </templates>
+            """
+        })
+        self.env['ir.asset'].create({
+            'name': 'test_asset',
+            'path': url,
+            'bundle': 'test.bundle',
+        })
+
+
+        res = self.url_open('/web/webclient/qweb/HASH_BIDON?bundle=test.bundle')
+        [template] = etree.fromstring(res.text)
+
+        self.assertEqual(template.get('t-name'), 'test_template')
+        self.assertEqual(template[0].get('class'), 'test_div')
 
 @tagged('-standard', 'static_templates_performance')
 class TestStaticInheritancePerformance(TestStaticInheritanceCommon):
@@ -646,14 +938,14 @@ class TestStaticInheritancePerformance(TestStaticInheritanceCommon):
         nMod modules
         each module: has nFilesPerModule files, each of which contains nTemplatePerFile templates
         """
-        self.modules = []
+        self.asset_paths = []
         self.template_files = {}
         number_templates = 0
         for m in range(nMod):
             for f in range(nFilePerMod):
                 mname = 'mod_%s' % m
                 fname = 'mod_%s_file_%s' % (m, f)
-                self.modules.append((fname, None, mname))
+                self.asset_paths.append((fname, mname, 'bundle_1'))
 
                 _file = '<templates id="template" xml:space="preserve">'
 
@@ -671,7 +963,7 @@ class TestStaticInheritancePerformance(TestStaticInheritanceCommon):
                             <div t-name="template_%(t_number)s_mod_%(m_number)s"
                                 t-inherit="template_%(t_inherit)s_mod_%(m_number)s"
                                 t-inherit-mode="primary">
-                                <xpath expr="//div[1]" position="before">
+                                <xpath expr="/div/div[1]" position="before">
                                     <div>Sick XPath</div>
                                 </xpath>
                             </div>
@@ -682,7 +974,7 @@ class TestStaticInheritancePerformance(TestStaticInheritanceCommon):
                             <div t-name="template_%(t_number)s_mod_%(m_number)s"
                                 t-inherit="mod_%(m_module_inherit)s.template_%(t_module_inherit)s_mod_%(m_module_inherit)s"
                                 t-inherit-mode="primary">
-                                <xpath expr="//div[1]" position="inside">
+                                <xpath expr="/div/div[1]" position="inside">
                                     <div>Mental XPath</div>
                                 </xpath>
                             </div>
@@ -712,7 +1004,7 @@ class TestStaticInheritancePerformance(TestStaticInheritanceCommon):
         contents = HomeStaticTemplateHelpers.get_qweb_templates(addons=self._get_module_names(), debug=True)
         after = datetime.now()
         delta2500 = after - before
-        _logger.log(25, 'Static Templates Inheritance: 2500 templates treated in %s seconds' % delta2500.total_seconds())
+        _logger.runbot('Static Templates Inheritance: 2500 templates treated in %s seconds' % delta2500.total_seconds())
 
         whole_tree = etree.fromstring(contents)
         self.assertEqual(len(whole_tree), nMod * nFilePerMod * nTemplatePerFile)
@@ -727,6 +1019,6 @@ class TestStaticInheritancePerformance(TestStaticInheritanceCommon):
         delta25000 = after - before
 
         time_ratio = delta25000.total_seconds() / delta2500.total_seconds()
-        _logger.log(25, 'Static Templates Inheritance: 25000 templates treated in %s seconds' % delta25000.total_seconds())
-        _logger.log(25, 'Static Templates Inheritance: Computed linearity ratio: %s' % time_ratio)
+        _logger.runbot('Static Templates Inheritance: 25000 templates treated in %s seconds' % delta25000.total_seconds())
+        _logger.runbot('Static Templates Inheritance: Computed linearity ratio: %s' % time_ratio)
         self.assertLessEqual(time_ratio, 12)
