@@ -8,7 +8,6 @@ as well as render a few fields differently.
 Also, adds methods to convert values back to Odoo models.
 """
 
-import ast
 import babel
 import base64
 import io
@@ -29,10 +28,11 @@ from werkzeug import urls
 
 import odoo.modules
 
-from odoo import api, models, fields
+from odoo import _, api, models, fields
+from odoo.exceptions import ValidationError
 from odoo.tools import ustr, posix_to_ldml, pycompat
 from odoo.tools import html_escape as escape
-from odoo.tools.misc import get_lang
+from odoo.tools.misc import get_lang, babel_locale_parse
 from odoo.addons.base.models import ir_qweb
 
 REMOTE_CONNECTION_TIMEOUT = 2.5
@@ -47,44 +47,82 @@ class QWeb(models.AbstractModel):
 
     # compile directives
 
-    def _compile_directive_snippet(self, el, options):
-        el.set('t-call', el.attrib.pop('t-snippet'))
-        View = self.env['ir.ui.view']
-        view_id = View.get_view_id(el.attrib.get('t-call'))
+    def _compile_node(self, el, options, indent):
+        snippet_key = options.get('snippet-key')
+        if snippet_key == options['template'] \
+                or options.get('snippet-sub-call-key') == options['template']:
+            # Get the path of element to only consider the first node of the
+            # snippet template content (ignoring all ancestors t elements which
+            # are not t-call ones)
+            nb_real_elements_in_hierarchy = 0
+            node = el
+            while node is not None and nb_real_elements_in_hierarchy < 2:
+                if node.tag != 't' or 't-call' in node.attrib:
+                    nb_real_elements_in_hierarchy += 1
+                node = node.getparent()
+            if nb_real_elements_in_hierarchy == 1:
+                # The first node might be a call to a sub template
+                sub_call = el.get('t-call')
+                if sub_call:
+                    el.set('t-options', f"{{'snippet-key': '{snippet_key}', 'snippet-sub-call-key': '{sub_call}'}}")
+                # If it already has a data-snippet it is a saved snippet.
+                # Do not override it.
+                elif 'data-snippet' not in el.attrib:
+                    el.attrib['data-snippet'] = snippet_key.split('.', 1)[-1]
+
+        return super()._compile_node(el, options, indent)
+
+    def _compile_directive_snippet(self, el, options, indent):
+        key = el.attrib.pop('t-snippet')
+        el.set('t-call', key)
+        el.set('t-options', "{'snippet-key': '" + key + "'}")
+        View = self.env['ir.ui.view'].sudo()
+        view_id = View.get_view_id(key)
         name = View.browse(view_id).name
         thumbnail = el.attrib.pop('t-thumbnail', "oe-thumbnail")
-        div = u'<div name="%s" data-oe-type="snippet" data-oe-thumbnail="%s">' % (
+        div = '<div name="%s" data-oe-type="snippet" data-oe-thumbnail="%s" data-oe-snippet-id="%s" data-oe-keywords="%s">' % (
             escape(pycompat.to_text(name)),
-            escape(pycompat.to_text(thumbnail))
+            escape(pycompat.to_text(thumbnail)),
+            escape(pycompat.to_text(view_id)),
+            escape(pycompat.to_text(el.findtext('keywords')))
         )
-        return [self._append(ast.Str(div))] + self._compile_node(el, options) + [self._append(ast.Str(u'</div>'))]
+        self._appendText(div, options)
+        code = self._compile_node(el, options, indent)
+        self._appendText('</div>', options)
+        return code
 
-    def _compile_directive_install(self, el, options):
+    def _compile_directive_snippet_call(self, el, options, indent):
+        key = el.attrib.pop('t-snippet-call')
+        el.set('t-call', key)
+        el.set('t-options', "{'snippet-key': '" + key + "'}")
+        return self._compile_node(el, options, indent)
+
+    def _compile_directive_install(self, el, options, indent):
         if self.user_has_groups('base.group_system'):
             module = self.env['ir.module.module'].search([('name', '=', el.attrib.get('t-install'))])
             if not module or module.state == 'installed':
                 return []
             name = el.attrib.get('string') or 'Snippet'
             thumbnail = el.attrib.pop('t-thumbnail', 'oe-thumbnail')
-            div = u'<div name="%s" data-oe-type="snippet" data-module-id="%s" data-oe-thumbnail="%s"><section/></div>' % (
+            div = '<div name="%s" data-oe-type="snippet" data-module-id="%s" data-oe-thumbnail="%s"><section/></div>' % (
                 escape(pycompat.to_text(name)),
                 module.id,
                 escape(pycompat.to_text(thumbnail))
             )
-            return [self._append(ast.Str(div))]
-        else:
-            return []
+            self._appendText(div, options)
+        return []
 
-    def _compile_directive_tag(self, el, options):
+    def _compile_directive_tag(self, el, options, indent):
         if el.get('t-placeholder'):
             el.set('t-att-placeholder', el.attrib.pop('t-placeholder'))
-        return super(QWeb, self)._compile_directive_tag(el, options)
+        return super(QWeb, self)._compile_directive_tag(el, options, indent)
 
     # order and ignore
 
     def _directives_eval_order(self):
         directives = super(QWeb, self)._directives_eval_order()
         directives.insert(directives.index('call'), 'snippet')
+        directives.insert(directives.index('call'), 'snippet-call')
         directives.insert(directives.index('call'), 'install')
         return directives
 
@@ -141,8 +179,11 @@ class Float(models.AbstractModel):
     def from_html(self, model, field, element):
         lang = self.user_lang()
         value = element.text_content().strip()
-        return float(value.replace(lang.thousands_sep, '')
-                          .replace(lang.decimal_point, '.'))
+        try:
+            return float(value.replace(lang.thousands_sep, '')
+                              .replace(lang.decimal_point, '.'))
+        except:
+            raise ValidationError(_('You entered an invalid value, please try again.'))
 
 
 class ManyToOne(models.AbstractModel):
@@ -192,7 +233,7 @@ class Contact(models.AbstractModel):
     # helper to call the rendering of contact field
     @api.model
     def get_record_to_html(self, ids, options=None):
-        return self.value_to_html(self.env['res.partner'].browse(ids[0]), options=options)
+        return self.value_to_html(self.env['res.partner'].search([('id', '=', ids[0])]), options=options)
 
 
 class Date(models.AbstractModel):
@@ -212,7 +253,7 @@ class Date(models.AbstractModel):
                 return attrs
 
             lg = self.env['res.lang']._lang_get(self.env.user.lang) or get_lang(self.env)
-            locale = babel.Locale.parse(lg.code)
+            locale = babel_locale_parse(lg.code)
             babel_format = value_format = posix_to_ldml(lg.date_format, locale=locale)
 
             if record[field_name]:
@@ -246,7 +287,7 @@ class DateTime(models.AbstractModel):
             value = record[field_name]
 
             lg = self.env['res.lang']._lang_get(self.env.user.lang) or get_lang(self.env)
-            locale = babel.Locale.parse(lg.code)
+            locale = babel_locale_parse(lg.code)
             babel_format = value_format = posix_to_ldml('%s %s' % (lg.date_format, lg.time_format), locale=locale)
             tz = record.env.context.get('tz') or self.env.user.tz
 
@@ -283,7 +324,7 @@ class DateTime(models.AbstractModel):
 
                 dt = user_tz.localize(dt).astimezone(utc)
             except Exception:
-                logger.warn(
+                logger.warning(
                     "Failed to convert the value for a field of the model"
                     " %s back from the user's timezone (%s) to UTC",
                     model, tz_name,
@@ -358,16 +399,18 @@ class Image(models.AbstractModel):
 
         url_object = urls.url_parse(url)
         if url_object.path.startswith('/web/image'):
-            # url might be /web/image/<model>/<id>[_<checksum>]/<field>[/<width>x<height>]
             fragments = url_object.path.split('/')
             query = url_object.decode_query()
-            if fragments[3].isdigit():
+            url_id = fragments[3].split('-')[0]
+            # ir.attachment image urls: /web/image/<id>[-<checksum>][/...]
+            if url_id.isdigit():
                 model = 'ir.attachment'
-                oid = fragments[3]
+                oid = url_id
                 field = 'datas'
+            # url of binary field on model: /web/image/<model>/<id>/<field>[/...]
             else:
                 model = query.get('model', fragments[3])
-                oid = query.get('id', fragments[4].split('_')[0])
+                oid = query.get('id', fragments[4])
                 field = query.get('field', fragments[5])
             item = self.env[model].browse(int(oid))
             return item[field]
@@ -437,8 +480,11 @@ class Monetary(models.AbstractModel):
 
         value = element.find('span').text.strip()
 
-        return float(value.replace(lang.thousands_sep, '')
-                          .replace(lang.decimal_point, '.'))
+        try:
+            return float(value.replace(lang.thousands_sep, '')
+                              .replace(lang.decimal_point, '.'))
+        except:
+            raise ValidationError(_('You entered an invalid value, please try again.'))
 
 
 class Duration(models.AbstractModel):
@@ -551,7 +597,7 @@ def _realize_padding(it):
     # leftover padding irrelevant as the output will be stripped
 
 
-def _wrap(element, output, wrapper=u''):
+def _wrap(element, output, wrapper=''):
     """ Recursively extracts text from ``element`` (via _element_to_text), and
     wraps it all in ``wrapper``. Extracted text is added to ``output``
 
@@ -567,7 +613,7 @@ def _wrap(element, output, wrapper=u''):
 
 def _element_to_text(e, output):
     if e.tag == 'br':
-        output.append(u'\n')
+        output.append('\n')
     elif e.tag in _PADDED_BLOCK:
         _wrap(e, output, 2)
     elif e.tag in _MISC_BLOCK:
