@@ -27,20 +27,20 @@ class CrmLeadForwardToPartner(models.TransientModel):
                 'lead_location': ", ".join(lead_location),
                 'partner_assigned_id': partner and partner.id or False,
                 'partner_location': ", ".join(partner_location),
-                'lead_link': self.get_lead_portal_url(lead.id, lead.type),
+                'lead_link': self.get_lead_portal_url(lead),
                 }
 
     @api.model
     def default_get(self, fields):
-        template = self.env.ref('website_crm_partner_assign.email_template_lead_forward_mail', False)
-
         res = super(CrmLeadForwardToPartner, self).default_get(fields)
         active_ids = self.env.context.get('active_ids')
-        default_composition_mode = self.env.context.get('default_composition_mode')
-        res['assignation_lines'] = []
-        if template:
-            res['body'] = template.body_html
+        if 'body' in fields:
+            template = self.env.ref('website_crm_partner_assign.email_template_lead_forward_mail', False)
+            if template:
+                res['body'] = template.body_html
         if active_ids:
+            default_composition_mode = self.env.context.get('default_composition_mode')
+            res['assignation_lines'] = []
             leads = self.env['crm.lead'].browse(active_ids)
             if default_composition_mode == 'mass_mail':
                 partner_assigned_dict = leads.search_geo_partner()
@@ -69,7 +69,7 @@ class CrmLeadForwardToPartner(models.TransientModel):
             if no_email:
                 raise UserError(_('Set an email address for the partner(s): %s') % ", ".join(no_email))
         if self.forward_type == 'single' and not self.partner_id.email:
-            raise UserError(_('Set an email address for the partner %s') % self.partner_id.name)
+            raise UserError(_('Set an email address for the partner %s', self.partner_id.name))
 
         partners_leads = {}
         for lead in self.assignation_lines:
@@ -103,26 +103,30 @@ class CrmLeadForwardToPartner(models.TransientModel):
             self.env['crm.lead'].message_subscribe([partner_id])
         return True
 
-    def get_lead_portal_url(self, lead_id, type):
-        action = type == 'opportunity' and 'action_portal_opportunities' or 'action_portal_leads'
+    def get_lead_portal_url(self, lead):
+        action = lead.type == 'opportunity' and 'action_portal_opportunities' or 'action_portal_leads'
         action_ref = self.env.ref('website_crm_partner_assign.%s' % (action,), False)
         portal_link = "%s/?db=%s#id=%s&action=%s&view_type=form" % (
-            self.env['ir.config_parameter'].sudo().get_param('web.base.url'),
+            lead.get_base_url(),
             self.env.cr.dbname,
-            lead_id,
+            lead.id,
             action_ref and action_ref.id or False)
         return portal_link
 
     def get_portal_url(self):
-        portal_link = "%s/?db=%s" % (self.env['ir.config_parameter'].sudo().get_param('web.base.url'), self.env.cr.dbname)
+        if self:
+            portal_url = self[0].get_base_url()
+        else:
+            portal_url = self.env['ir.config_parameter'].sudo().get_param('web.base.url')
+        portal_link = "%s/?db=%s" % (portal_url, self.env.cr.dbname)
         return portal_link
 
     forward_type = fields.Selection([
         ('single', 'a single partner: manual selection of partner'),
-        ('assigned', "several partners: automatic assignation, using GPS coordinates and partner's grades")
+        ('assigned', "several partners: automatic assignment, using GPS coordinates and partner's grades")
     ], 'Forward selected leads to', default=lambda self: self.env.context.get('forward_type') or 'single')
     partner_id = fields.Many2one('res.partner', 'Forward Leads To')
-    assignation_lines = fields.One2many('crm.lead.assignation', 'forward_id', 'Partner Assignation')
+    assignation_lines = fields.One2many('crm.lead.assignation', 'forward_id', 'Partner Assignment')
     body = fields.Html('Contents', help='Automatically sanitized HTML contents')
 
 
@@ -130,7 +134,7 @@ class CrmLeadAssignation(models.TransientModel):
     _name = 'crm.lead.assignation'
     _description = 'Lead Assignation'
 
-    forward_id = fields.Many2one('crm.lead.forward.to.partner', 'Partner Assignation')
+    forward_id = fields.Many2one('crm.lead.forward.to.partner', 'Partner Assignment')
     lead_id = fields.Many2one('crm.lead', 'Lead')
     lead_location = fields.Char('Lead Location')
     partner_assigned_id = fields.Many2one('res.partner', 'Assigned Partner')
