@@ -18,10 +18,7 @@ publicWidget.registry.visitor = publicWidget.Widget.extend({
         defs.push(this._rpc({route: '/event/get_country_event_list'}).then(function (data) {
             if (data) {
                 self._$loadedContent = $(data);
-
-                self._$loadedContent.addClass('o_temp_auto_element');
-                self._$loadedContent.attr('data-temp-auto-element-original-content', self._originalContent);
-
+                self._$loadedContent.attr('contentEditable', false);
                 $eventList.replaceWith(self._$loadedContent);
             }
         }));
