@@ -1,9 +1,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch
+from unittest.mock import patch
 
 import odoo.tests
 
@@ -15,14 +12,14 @@ class TestUi(odoo.tests.HttpCase):
         super(TestUi, self).setUp()
 
         def _get_title_from_url(addr, **kw):
-            return 'About us | My Website' if 'aboutus' in addr else 'Contact Us | My Website'
+            return 'Contact Us | My Website'
 
         patcher = patch('odoo.addons.link_tracker.models.link_tracker.LinkTracker._get_title_from_url', wraps=_get_title_from_url)
         patcher.start()
         self.addCleanup(patcher.stop)
 
     def test_01_test_ui(self):
-        self.env['link.tracker'].create({
+        self.env['link.tracker'].search_or_create({
             'campaign_id': 2,
             'medium_id': 2,
             'source_id': 2,
