@@ -1,11 +1,8 @@
-odoo.define('website_livechat.editor', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var core = require('web.core');
-var wUtils = require('website.utils');
-var WebsiteNewMenu = require('website.newMenu');
-
-var _t = core._t;
+import { _t } from 'web.core';
+import wUtils from 'website.utils';
+import WebsiteNewMenu from 'website.newMenu';
 
 WebsiteNewMenu.include({
     actions: _.extend({}, WebsiteNewMenu.prototype.actions || {}, {
@@ -21,14 +18,15 @@ WebsiteNewMenu.include({
      * and redirects the user to this new channel.
      *
      * @private
-     * @returns {Deferred} Unresolved if there is a redirection
+     * @returns {Promise} Unresolved if there is a redirection
      */
     _createNewChannel: function () {
         var self = this;
         return wUtils.prompt({
             window_title: _t("New Channel"),
             input: _t("Name"),
-        }).then(function (name) {
+        }).then(function (result) {
+            var name = result.val;
             if (!name) {
                 return;
             }
@@ -41,10 +39,8 @@ WebsiteNewMenu.include({
                 },
             }).then(function (url) {
                 window.location.href = url;
-                return $.Deferred();
+                return new Promise(function () {});
             });
         });
     },
 });
-
-});
