@@ -1,10 +1,13 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-import odoo.tests
+import base64
 
+from odoo.addons.base.tests.common import HttpCaseWithUserDemo, HttpCaseWithUserPortal
+from odoo.modules.module import get_module_resource
+from odoo.tests import tagged
 
-@odoo.tests.common.tagged('post_install', '-at_install')
-class TestUi(odoo.tests.HttpCase):
+@tagged('post_install', '-at_install')
+class TestUi(HttpCaseWithUserDemo, HttpCaseWithUserPortal):
 
     def setUp(self):
         super(TestUi, self).setUp()
@@ -18,9 +21,21 @@ class TestUi(odoo.tests.HttpCase):
         tax = self.env['account.tax'].create({'name': "Test tax", 'amount': 10})
         product_template.taxes_id = tax
 
-        product_attribute = self.env.ref('product.product_attribute_1')
-        product_attribute_value_1 = self.env.ref('product.product_attribute_value_1')
-        product_attribute_value_2 = self.env.ref('product.product_attribute_value_2')
+        product_attribute = self.env['product.attribute'].create({
+            'name': 'Legs',
+            'visibility': 'visible',
+            'sequence': 10,
+        })
+        product_attribute_value_1 = self.env['product.attribute.value'].create({
+            'name': 'Steel - Test',
+            'attribute_id': product_attribute.id,
+            'sequence': 1,
+        })
+        product_attribute_value_2 = self.env['product.attribute.value'].create({
+            'name': 'Aluminium',
+            'attribute_id': product_attribute.id,
+            'sequence': 2,
+        })
 
         # set attribute and attribute values on the template
         self.env['product.template.attribute.line'].create([{
@@ -34,17 +49,97 @@ class TestUi(odoo.tests.HttpCase):
             .search([('product_tmpl_id', '=', product_template.id)])
 
         for ptav in product_template_attribute_values:
-            if ptav.name == "Steel":
+            if ptav.name == "Steel - Test":
                 ptav.price_extra = 0
             else:
                 ptav.price_extra = 50.4
 
     def test_01_admin_shop_customize_tour(self):
+        # Enable Variant Group
+        self.env.ref('product.group_product_variant').write({'users': [(4, self.env.ref('base.user_admin').id)]})
         self.start_tour("/", 'shop_customize', login="admin")
 
     def test_02_admin_shop_custom_attribute_value_tour(self):
         # Make sure pricelist rule exist
-        product_template = self.env.ref('product.product_product_4_product_template')
+        self.product_attribute_1 = self.env['product.attribute'].create({
+            'name': 'Legs',
+            'sequence': 10,
+        })
+        product_attribute_value_1 = self.env['product.attribute.value'].create({
+            'name': 'Steel',
+            'attribute_id': self.product_attribute_1.id,
+            'sequence': 1,
+        })
+        product_attribute_value_2 = self.env['product.attribute.value'].create({
+            'name': 'Aluminium',
+            'attribute_id': self.product_attribute_1.id,
+            'sequence': 2,
+        })
+        product_attribute_2 = self.env['product.attribute'].create({
+            'name': 'Color',
+            'sequence': 20,
+        })
+        product_attribute_value_3 = self.env['product.attribute.value'].create({
+            'name': 'White',
+            'attribute_id': product_attribute_2.id,
+            'sequence': 1,
+        })
+        product_attribute_value_4 = self.env['product.attribute.value'].create({
+            'name': 'Black',
+            'attribute_id': product_attribute_2.id,
+            'sequence': 2,
+        })
+
+        # Create product template
+        self.product_product_4_product_template = self.env['product.template'].create({
+            'name': 'Customizable Desk (TEST)',
+            'standard_price': 500.0,
+            'list_price': 750.0,
+        })
+
+        # Generate variants
+        self.env['product.template.attribute.line'].create([{
+            'product_tmpl_id': self.product_product_4_product_template.id,
+            'attribute_id': self.product_attribute_1.id,
+            'value_ids': [(4, product_attribute_value_1.id), (4, product_attribute_value_2.id)],
+        }, {
+            'product_tmpl_id': self.product_product_4_product_template.id,
+            'attribute_id': product_attribute_2.id,
+            'value_ids': [(4, product_attribute_value_3.id), (4, product_attribute_value_4.id)],
+
+        }])
+        product_template = self.product_product_4_product_template
+
+        # Add Custom Attribute
+        product_attribute_value_7 = self.env['product.attribute.value'].create({
+            'name': 'Custom TEST',
+            'attribute_id': self.product_attribute_1.id,
+            'sequence': 3,
+            'is_custom': True
+        })
+        self.product_product_4_product_template.attribute_line_ids[0].write({'value_ids': [(4, product_attribute_value_7.id)]})
+
+        img_path = get_module_resource('product', 'static', 'img', 'product_product_11-image.png')
+        img_content = base64.b64encode(open(img_path, "rb").read())
+        self.product_product_11_product_template = self.env['product.template'].create({
+            'name': 'Conference Chair (TEST)',
+            'website_sequence': 9999, # laule
+            'image_1920': img_content,
+            'list_price': 16.50,
+        })
+
+        self.env['product.template.attribute.line'].create({
+            'product_tmpl_id': self.product_product_11_product_template.id,
+            'attribute_id': self.product_attribute_1.id,
+            'value_ids': [(4, product_attribute_value_1.id), (4, product_attribute_value_2.id)],
+        })
+        self.product_product_11_product_template.attribute_line_ids[0].product_template_value_ids[1].price_extra = 6.40
+
+        # Setup a second optional product
+        self.product_product_1_product_template = self.env['product.template'].create({
+            'name': 'Chair floor protection',
+            'list_price': 12.0,
+        })
 
         # fix runbot, sometimes one pricelist is chosen, sometimes the other...
         pricelists = self.env['website'].get_current_website().get_current_pricelist() | self.env.ref('product.list0')
@@ -208,6 +303,8 @@ class TestUi(odoo.tests.HttpCase):
         self.start_tour("/", 'tour_shop_no_variant_attribute', login="demo")
 
     def test_06_admin_list_view_b2c(self):
+        self.env.ref('product.group_product_variant').write({'users': [(4, self.env.ref('base.user_admin').id)]})
+
         # activate b2c
         config = self.env['res.config.settings'].create({})
         config.show_line_subtotals_tax_selection = "tax_included"
@@ -215,3 +312,6 @@ class TestUi(odoo.tests.HttpCase):
         config.execute()
 
         self.start_tour("/", 'shop_list_view_b2c', login="admin")
+
+    def test_07_editor_shop(self):
+        self.start_tour("/", 'shop_editor', login="admin")
