@@ -11,7 +11,7 @@ To configure product attributes, activate *Attributes & Variants* in the Website
 Finally, the module comes with an option to display an attribute summary table in product web pages (available in Customize menu).
     """,
     'author': 'Odoo SA',
-    'category': 'Website',
+    'category': 'Website/Website',
     'version': '1.0',
     'depends': ['website_sale'],
     'data': [
@@ -24,4 +24,14 @@ Finally, the module comes with an option to display an attribute summary table i
         'data/website_sale_comparison_demo.xml',
     ],
     'installable': True,
+    'assets': {
+        'web.assets_frontend': [
+            'website_sale_comparison/static/src/scss/website_sale_comparison.scss',
+            'website_sale_comparison/static/src/js/website_sale_comparison.js',
+        ],
+        'web.assets_tests': [
+            'website_sale_comparison/static/tests/**/*',
+        ],
+    },
+    'license': 'LGPL-3',
 }
