@@ -7,7 +7,9 @@ import io
 import re
 import requests
 import PyPDF2
+import json
 
+from dateutil.relativedelta import relativedelta
 from PIL import Image
 from werkzeug import urls
 
@@ -16,6 +18,7 @@ from odoo.addons.http_routing.models.ir_http import slug
 from odoo.exceptions import UserError, AccessError
 from odoo.http import request
 from odoo.addons.http_routing.models.ir_http import url_for
+from odoo.tools import html2plaintext, sql
 
 
 class SlidePartnerRelation(models.Model):
@@ -32,8 +35,9 @@ class SlidePartnerRelation(models.Model):
     completed = fields.Boolean('Completed')
     quiz_attempts_count = fields.Integer('Quiz attempts count', default=0)
 
-    def create(self, values):
-        res = super(SlidePartnerRelation, self).create(values)
+    @api.model_create_multi
+    def create(self, vals_list):
+        res = super().create(vals_list)
         completed = res.filtered('completed')
         if completed:
             completed._set_completed_callback()
@@ -61,6 +65,15 @@ class SlideLink(models.Model):
     link = fields.Char('Link', required=True)
 
 
+class SlideResource(models.Model):
+    _name = 'slide.slide.resource'
+    _description = "Additional resource for a particular slide"
+
+    slide_id = fields.Many2one('slide.slide', required=True, ondelete='cascade')
+    name = fields.Char('Name', required=True)
+    data = fields.Binary('Resource')
+
+
 class EmbeddedSlide(models.Model):
     """ Embedding in third party websites. Track view count, generate statistics. """
     _name = 'slide.embed'
@@ -87,7 +100,7 @@ class EmbeddedSlide(models.Model):
 
 
 class SlideTag(models.Model):
-    """ Tag to search slides accross channels. """
+    """ Tag to search slides across channels. """
     _name = 'slide.tag'
     _description = 'Slide Tag'
 
@@ -103,7 +116,10 @@ class Slide(models.Model):
     _inherit = [
         'mail.thread',
         'image.mixin',
-        'website.seo.metadata', 'website.published.mixin']
+        'website.seo.metadata',
+        'website.published.mixin',
+        'website.searchable.mixin',
+    ]
     _description = 'Slides'
     _mail_post_access = 'read'
     _order_by_strategy = {
@@ -116,13 +132,14 @@ class Slide(models.Model):
 
     # description
     name = fields.Char('Title', required=True, translate=True)
-    active = fields.Boolean(default=True)
+    active = fields.Boolean(default=True, tracking=100)
     sequence = fields.Integer('Sequence', default=0)
     user_id = fields.Many2one('res.users', string='Uploaded by', default=lambda self: self.env.uid)
-    description = fields.Text('Description', translate=True)
+    description = fields.Html('Description', translate=True)
     channel_id = fields.Many2one('slide.channel', string="Course", required=True)
     tag_ids = fields.Many2many('slide.tag', 'rel_slide_tag', 'slide_id', 'tag_id', string='Tags')
     is_preview = fields.Boolean('Allow Preview', default=False, help="The course is accessible by anyone : the users don't need to join the channel to access the content of the course.")
+    is_new_slide = fields.Boolean('Is New Slide', compute='_compute_is_new_slide')
     completion_time = fields.Float('Duration', digits=(10, 4), help="The estimated completion time for this slide")
     # Categories
     is_category = fields.Boolean('Is a category', default=False)
@@ -130,18 +147,19 @@ class Slide(models.Model):
     slide_ids = fields.One2many('slide.slide', "category_id", string="Slides")
     # subscribers
     partner_ids = fields.Many2many('res.partner', 'slide_slide_partner', 'slide_id', 'partner_id',
-                                   string='Subscribers', groups='website.group_website_publisher', copy=False)
-    slide_partner_ids = fields.One2many('slide.slide.partner', 'slide_id', string='Subscribers information', groups='website.group_website_publisher', copy=False)
+                                   string='Subscribers', groups='website_slides.group_website_slides_officer', copy=False)
+    slide_partner_ids = fields.One2many('slide.slide.partner', 'slide_id', string='Subscribers information', groups='website_slides.group_website_slides_officer', copy=False)
     user_membership_id = fields.Many2one(
-        'slide.slide.partner', string="Subscriber information", compute='_compute_user_membership_id', compute_sudo=False,
+        'slide.slide.partner', string="Subscriber information",
+        compute='_compute_user_membership_id', compute_sudo=False,
         help="Subscriber information for the current logged in user")
     # Quiz related fields
     question_ids = fields.One2many("slide.question", "slide_id", string="Questions")
     questions_count = fields.Integer(string="Numbers of Questions", compute='_compute_questions_count')
-    quiz_first_attempt_reward = fields.Integer("First attempt reward", default=10)
-    quiz_second_attempt_reward = fields.Integer("Second attempt reward", default=7)
-    quiz_third_attempt_reward = fields.Integer("Third attempt reward", default=5,)
-    quiz_fourth_attempt_reward = fields.Integer("Reward for every attempt after the third try", default=2)
+    quiz_first_attempt_reward = fields.Integer("Reward: first attempt", default=10)
+    quiz_second_attempt_reward = fields.Integer("Reward: second attempt", default=7)
+    quiz_third_attempt_reward = fields.Integer("Reward: third attempt", default=5,)
+    quiz_fourth_attempt_reward = fields.Integer("Reward: every attempt after the third try", default=2)
     # content
     slide_type = fields.Selection([
         ('infographic', 'Infographic'),
@@ -157,15 +175,17 @@ class Slide(models.Model):
     url = fields.Char('Document URL', help="Youtube or Google Document URL")
     document_id = fields.Char('Document ID', help="Youtube or Google Document ID")
     link_ids = fields.One2many('slide.slide.link', 'slide_id', string="External URL for this slide")
+    slide_resource_ids = fields.One2many('slide.slide.resource', 'slide_id', string="Additional Resource for this slide")
+    slide_resource_downloadable = fields.Boolean('Allow Download', default=True, help="Allow the user to download the content of the slide.")
     mime_type = fields.Char('Mime-type')
     html_content = fields.Html("HTML Content", help="Custom HTML content for slides of type 'Web Page'.", translate=True, sanitize_attributes=False, sanitize_form=False)
     # website
     website_id = fields.Many2one(related='channel_id.website_id', readonly=True)
-    date_published = fields.Datetime('Publish Date', readonly=True, tracking=True)
-    likes = fields.Integer('Likes', compute='_compute_user_info', store=True, compute_sudo=False)
-    dislikes = fields.Integer('Dislikes', compute='_compute_user_info', store=True, compute_sudo=False)
-    user_vote = fields.Integer('User vote', compute='_compute_user_info', compute_sudo=False)
-    embed_code = fields.Text('Embed Code', readonly=True, compute='_compute_embed_code')
+    date_published = fields.Datetime('Publish Date', readonly=True, tracking=1)
+    likes = fields.Integer('Likes', compute='_compute_like_info', store=True, compute_sudo=False)
+    dislikes = fields.Integer('Dislikes', compute='_compute_like_info', store=True, compute_sudo=False)
+    user_vote = fields.Integer('User vote', compute='_compute_user_membership_id', compute_sudo=False)
+    embed_code = fields.Html('Embed Code', readonly=True, compute='_compute_embed_code', sanitize=False)
     # views
     embedcount_ids = fields.One2many('slide.embed', 'slide_id', string="Embed Count")
     slide_views = fields.Integer('# of Website Views', store=True, compute="_compute_slide_views")
@@ -189,6 +209,11 @@ class Slide(models.Model):
         ('exclusion_html_content_and_url', "CHECK(html_content IS NULL OR url IS NULL)", "A slide is either filled with a document url or HTML content. Not both.")
     ]
 
+    @api.depends('date_published', 'is_published')
+    def _compute_is_new_slide(self):
+        for slide in self:
+            slide.is_new_slide = slide.date_published > fields.Datetime.now() - relativedelta(days=7) if slide.is_published else False
+
     @api.depends('channel_id.slide_ids.is_category', 'channel_id.slide_ids.sequence')
     def _compute_category_id(self):
         """ Will take all the slides of the channel for which the index is higher
@@ -229,9 +254,39 @@ class Slide(models.Model):
         for record in self:
             record.total_views = record.slide_views + record.public_views
 
+    @api.depends('slide_partner_ids.vote')
+    def _compute_like_info(self):
+        if not self.ids:
+            self.update({'likes': 0, 'dislikes': 0})
+            return
+
+        rg_data_like = self.env['slide.slide.partner'].sudo().read_group(
+            [('slide_id', 'in', self.ids), ('vote', '=', 1)],
+            ['slide_id'], ['slide_id']
+        )
+        rg_data_dislike = self.env['slide.slide.partner'].sudo().read_group(
+            [('slide_id', 'in', self.ids), ('vote', '=', -1)],
+            ['slide_id'], ['slide_id']
+        )
+        mapped_data_like = dict(
+            (rg_data['slide_id'][0], rg_data['slide_id_count'])
+            for rg_data in rg_data_like
+        )
+        mapped_data_dislike = dict(
+            (rg_data['slide_id'][0], rg_data['slide_id_count'])
+            for rg_data in rg_data_dislike
+        )
+
+        for slide in self:
+            slide.likes = mapped_data_like.get(slide.id, 0)
+            slide.dislikes = mapped_data_dislike.get(slide.id, 0)
+
     @api.depends('slide_partner_ids.vote')
     @api.depends_context('uid')
     def _compute_user_info(self):
+        """ Deprecated. Now computed directly by _compute_user_membership_id
+        for user_vote and _compute_like_info for likes / dislikes. Remove me in
+        master. """
         default_stats = {'likes': 0, 'dislikes': 0, 'user_vote': False}
 
         if not self.ids:
@@ -242,6 +297,7 @@ class Slide(models.Model):
         slide_partners = self.env['slide.slide.partner'].sudo().search([
             ('slide_id', 'in', self.ids)
         ])
+
         for slide_partner in slide_partners:
             if slide_partner.vote == 1:
                 slide_data[slide_partner.slide_id.id]['likes'] += 1
@@ -251,6 +307,7 @@ class Slide(models.Model):
                 slide_data[slide_partner.slide_id.id]['dislikes'] += 1
                 if slide_partner.partner_id == self.env.user.partner_id:
                     slide_data[slide_partner.slide_id.id]['user_vote'] = -1
+
         for slide in self:
             slide.update(slide_data[slide.id])
 
@@ -297,7 +354,7 @@ class Slide(models.Model):
                 result[cid]['total_slides'] += slide_type_count
         return result
 
-    @api.depends('slide_partner_ids.partner_id')
+    @api.depends('slide_partner_ids.partner_id', 'slide_partner_ids.vote')
     @api.depends('uid')
     def _compute_user_membership_id(self):
         slide_partners = self.env['slide.slide.partner'].sudo().search([
@@ -310,13 +367,16 @@ class Slide(models.Model):
                 (slide_partner for slide_partner in slide_partners if slide_partner.slide_id == record),
                 self.env['slide.slide.partner']
             )
+            record.user_vote = record.user_membership_id.vote
 
     @api.depends('document_id', 'slide_type', 'mime_type')
     def _compute_embed_code(self):
-        base_url = request and request.httprequest.url_root or self.env['ir.config_parameter'].sudo().get_param('web.base.url')
-        if base_url[-1] == '/':
-            base_url = base_url[:-1]
+        base_url = request and request.httprequest.url_root
         for record in self:
+            if not base_url:
+                base_url = record.get_base_url()
+            if base_url[-1] == '/':
+                base_url = base_url[:-1]
             if record.datas and (not record.document_id or record.slide_type in ['document', 'presentation']):
                 slide_url = base_url + url_for('/slides/embed/%s?page=1' % record.id)
                 record.embed_code = '<iframe src="%s" class="o_wslides_iframe_viewer" allowFullScreen="true" height="%s" width="%s" frameborder="0"></iframe>' % (slide_url, 315, 420)
@@ -338,7 +398,7 @@ class Slide(models.Model):
         if self.url:
             res = self._parse_document_url(self.url)
             if res.get('error'):
-                raise UserError(_('Could not fetch data from url. Document or access right not available:\n%s') % res['error'])
+                raise UserError(res.get('error'))
             values = res['values']
             if not values.get('document_id'):
                 raise UserError(_('Please enter valid Youtube or Google Doc URL'))
@@ -347,7 +407,11 @@ class Slide(models.Model):
 
     @api.onchange('datas')
     def _on_change_datas(self):
-        """ For PDFs, we assume that it takes 5 minutes to read a page. """
+        """ For PDFs, we assume that it takes 5 minutes to read a page.
+            If the selected file is not a PDF, it is an image (You can
+            only upload PDF or Image file) then the slide_type is changed
+            into infographic and the uploaded dataS is transfered to the
+            image field. (It avoids the infinite loading in PDF viewer)"""
         if self.datas:
             data = base64.b64decode(self.datas)
             if data.startswith(b'%PDF-'):
@@ -357,26 +421,18 @@ class Slide(models.Model):
                 except PyPDF2.utils.PdfReadError:
                     return
                 self.completion_time = (5 * len(pdf.pages)) / 60
+            else:
+                self.slide_type = 'infographic'
+                self.image_1920 = self.datas
+                self.datas = None
 
     @api.depends('name', 'channel_id.website_id.domain')
     def _compute_website_url(self):
-        # TDE FIXME: clena this link.tracker strange stuff
         super(Slide, self)._compute_website_url()
         for slide in self:
             if slide.id:  # avoid to perform a slug on a not yet saved record in case of an onchange.
                 base_url = slide.channel_id.get_base_url()
-                # link_tracker is not in dependencies, so use it to shorten url only if installed.
-                if self.env.registry.get('link.tracker'):
-                    url = self.env['link.tracker'].sudo().create({
-                        'url': '%s/slides/slide/%s' % (base_url, slug(slide)),
-                        'title': slide.name,
-                    }).short_url
-                else:
-                    url = '%s/slides/slide/%s' % (base_url, slug(slide))
-                slide.website_url = url
-
-    def get_backend_menu_id(self):
-        return self.env.ref('website_slides.website_slides_menu_root').id
+                slide.website_url = '%s/slides/slide/%s' % (base_url, slug(slide))
 
     @api.depends('channel_id.can_publish')
     def _compute_can_publish(self):
@@ -445,11 +501,24 @@ class Slide(models.Model):
         rec.sequence = 0
         return rec
 
-    def unlink(self):
+    @api.ondelete(at_uninstall=False)
+    def _unlink_except_already_taken(self):
         if self.question_ids and self.channel_id.channel_partner_ids:
             raise UserError(_("People already took this quiz. To keep course progression it should not be deleted."))
+
+    def unlink(self):
+        for category in self.filtered(lambda slide: slide.is_category):
+            category.channel_id._move_category_slides(category, False)
         super(Slide, self).unlink()
 
+    def toggle_active(self):
+        # archiving/unarchiving a channel does it on its slides, too
+        to_archive = self.filtered(lambda slide: slide.active)
+        res = super(Slide, self).toggle_active()
+        if to_archive:
+            to_archive.filtered(lambda slide: not slide.is_category).is_published = False
+        return res
+
     # ---------------------------------------------------------
     # Mail/Rating
     # ---------------------------------------------------------
@@ -489,22 +558,21 @@ class Slide(models.Model):
     # ---------------------------------------------------------
 
     def _post_publication(self):
-        base_url = self.env['ir.config_parameter'].sudo().get_param('web.base.url')
         for slide in self.filtered(lambda slide: slide.website_published and slide.channel_id.publish_template_id):
             publish_template = slide.channel_id.publish_template_id
-            html_body = publish_template.with_context(base_url=base_url)._render_template(publish_template.body_html, 'slide.slide', slide.id)
-            subject = publish_template._render_template(publish_template.subject, 'slide.slide', slide.id)
+            html_body = publish_template.with_context(base_url=slide.get_base_url())._render_field('body_html', slide.ids)[slide.id]
+            subject = publish_template._render_field('subject', slide.ids)[slide.id]
             # We want to use the 'reply_to' of the template if set. However, `mail.message` will check
             # if the key 'reply_to' is in the kwargs before calling _get_reply_to. If the value is
             # falsy, we don't include it in the 'message_post' call.
             kwargs = {}
-            reply_to = publish_template._render_template(publish_template.reply_to, 'slide.slide', slide.id)
+            reply_to = publish_template._render_field('reply_to', slide.ids)[slide.id]
             if reply_to:
                 kwargs['reply_to'] = reply_to
             slide.channel_id.with_context(mail_create_nosubscribe=True).message_post(
                 subject=subject,
                 body=html_body,
-                subtype='website_slides.mt_channel_slide_published',
+                subtype_xmlid='website_slides.mt_channel_slide_published',
                 email_layout_xmlid='mail.mail_notification_light',
                 **kwargs,
             )
@@ -522,12 +590,19 @@ class Slide(models.Model):
     def _send_share_email(self, email, fullscreen):
         # TDE FIXME: template to check
         mail_ids = []
-        base_url = self.env['ir.config_parameter'].sudo().get_param('web.base.url')
         for record in self:
+            template = record.channel_id.share_template_id.with_context(
+                user=self.env.user,
+                email=email,
+                base_url=record.get_base_url(),
+                fullscreen=fullscreen
+            )
+            email_values = {'email_to': email}
             if self.env.user.has_group('base.group_portal'):
-                mail_ids.append(self.channel_id.share_template_id.with_context(user=self.env.user, email=email, base_url=base_url, fullscreen=fullscreen).sudo().send_mail(record.id, notif_layout='mail.mail_notification_light', email_values={'email_from': self.env['res.company'].catchall or self.env['res.company'].email, 'email_to': email}))
-            else:
-                mail_ids.append(self.channel_id.share_template_id.with_context(user=self.env.user, email=email, base_url=base_url, fullscreen=fullscreen).send_mail(record.id, notif_layout='mail.mail_notification_light', email_values={'email_to': email}))
+                template = template.sudo()
+                email_values['email_from'] = self.env.company.catchall_formatted or self.env.company.email_formatted
+
+            mail_ids.append(template.send_mail(record.id, notif_layout='mail.mail_notification_light', email_values=email_values))
         return mail_ids
 
     def action_like(self):
@@ -579,7 +654,7 @@ class Slide(models.Model):
             self.env.user.add_karma(karma_to_add)
 
     def action_set_viewed(self, quiz_attempts_inc=False):
-        if not all(slide.channel_id.is_member for slide in self):
+        if any(not slide.channel_id.is_member for slide in self):
             raise UserError(_('You cannot mark a slide as viewed if you are not among its members.'))
 
         return bool(self._action_set_viewed(self.env.user.partner_id, quiz_attempts_inc=quiz_attempts_inc))
@@ -591,11 +666,9 @@ class Slide(models.Model):
             ('slide_id', 'in', self.ids),
             ('partner_id', '=', target_partner.id)
         ])
-        if quiz_attempts_inc:
-            for exsting_slide in existing_sudo:
-                exsting_slide.write({
-                    'quiz_attempts_count': exsting_slide.quiz_attempts_count + 1
-                })
+        if quiz_attempts_inc and existing_sudo:
+            sql.increment_field_skiplock(existing_sudo, 'quiz_attempts_count')
+            SlidePartnerSudo.invalidate_cache(fnames=['quiz_attempts_count'], ids=existing_sudo.ids)
 
         new_slides = self_sudo - existing_sudo.mapped('slide_id')
         return SlidePartnerSudo.create([{
@@ -606,7 +679,7 @@ class Slide(models.Model):
             'vote': 0} for new_slide in new_slides])
 
     def action_set_completed(self):
-        if not all(slide.channel_id.is_member for slide in self):
+        if any(not slide.channel_id.is_member for slide in self):
             raise UserError(_('You cannot mark a slide as completed if you are not among its members.'))
 
         return self._action_set_completed(self.env.user.partner_id)
@@ -631,7 +704,7 @@ class Slide(models.Model):
         return True
 
     def _action_set_quiz_done(self):
-        if not all(slide.channel_id.is_member for slide in self):
+        if any(not slide.channel_id.is_member for slide in self):
             raise UserError(_('You cannot mark a slide quiz as completed if you are not among its members.'))
 
         points = 0
@@ -732,7 +805,7 @@ class Slide(models.Model):
         key = self.env['website'].get_current_website().website_slide_google_app_key
         fetch_res = self._fetch_data('https://www.googleapis.com/youtube/v3/videos', {'id': document_id, 'key': key, 'part': 'snippet,contentDetails', 'fields': 'items(id,snippet,contentDetails)'}, 'json')
         if fetch_res.get('error'):
-            return fetch_res
+            return {'error': self._extract_google_error_message(fetch_res.get('error'))}
 
         values = {'slide_type': 'video', 'document_id': document_id}
         items = fetch_res['values'].get('items')
@@ -767,6 +840,22 @@ class Slide(models.Model):
             })
         return {'values': values}
 
+    def _extract_google_error_message(self, error):
+        """
+        See here for Google error format
+        https://developers.google.com/drive/api/v3/handle-errors
+        """
+        try:
+            error = json.loads(error)
+            error = (error.get('error', {}).get('errors', []) or [{}])[0].get('reason')
+        except json.decoder.JSONDecodeError:
+            error = str(error)
+
+        if error == 'keyInvalid':
+            return _('Your Google API key is invalid, please update it in your settings.\nSettings > Website > Features > API Key')
+
+        return _('Could not fetch data from url. Document or access right not available:\n%s', error)
+
     @api.model
     def _parse_google_document(self, document_id, only_preview_fields):
         def get_slide_type(vals):
@@ -794,7 +883,7 @@ class Slide(models.Model):
 
         fetch_res = self._fetch_data('https://www.googleapis.com/drive/v2/files/%s' % document_id, params, "json")
         if fetch_res.get('error'):
-            return fetch_res
+            return {'error': self._extract_google_error_message(fetch_res.get('error'))}
 
         google_values = fetch_res['values']
         if only_preview_fields:
@@ -828,7 +917,57 @@ class Slide(models.Model):
     def _default_website_meta(self):
         res = super(Slide, self)._default_website_meta()
         res['default_opengraph']['og:title'] = res['default_twitter']['twitter:title'] = self.name
-        res['default_opengraph']['og:description'] = res['default_twitter']['twitter:description'] = self.description
+        res['default_opengraph']['og:description'] = res['default_twitter']['twitter:description'] = html2plaintext(self.description)
         res['default_opengraph']['og:image'] = res['default_twitter']['twitter:image'] = self.env['website'].image_url(self, 'image_1024')
-        res['default_meta_description'] = self.description
+        res['default_meta_description'] = html2plaintext(self.description)
         return res
+
+    # ---------------------------------------------------------
+    # Data / Misc
+    # ---------------------------------------------------------
+
+    def get_backend_menu_id(self):
+        return self.env.ref('website_slides.website_slides_menu_root').id
+
+    @api.model
+    def _search_get_detail(self, website, order, options):
+        with_description = options['displayDescription']
+        search_fields = ['name']
+        fetch_fields = ['id', 'name']
+        mapping = {
+            'name': {'name': 'name', 'type': 'text', 'match': True},
+            'website_url': {'name': 'url', 'type': 'text', 'truncate': False},
+            'extra_link': {'name': 'course', 'type': 'text'},
+            'extra_link_url': {'name': 'course_url', 'type': 'text', 'truncate': False},
+        }
+        if with_description:
+            search_fields.append('description')
+            fetch_fields.append('description')
+            mapping['description'] = {'name': 'description', 'type': 'text', 'html': True, 'match': True}
+        return {
+            'model': 'slide.slide',
+            'base_domain': [website.website_domain()],
+            'search_fields': search_fields,
+            'fetch_fields': fetch_fields,
+            'mapping': mapping,
+            'icon': 'fa-shopping-cart',
+            'order': 'name desc, id desc' if 'name desc' in order else 'name asc, id desc',
+        }
+
+    def _search_render_results(self, fetch_fields, mapping, icon, limit):
+        icon_per_type = {
+            'infographic': 'fa-file-picture-o',
+            'webpage': 'fa-file-text',
+            'presentation': 'fa-file-pdf-o',
+            'document': 'fa-file-pdf-o',
+            'video': 'fa-play-circle',
+            'quiz': 'fa-question-circle',
+            'link': 'fa-file-code-o', # appears in template "slide_icon"
+        }
+        results_data = super()._search_render_results(fetch_fields, mapping, icon, limit)
+        for slide, data in zip(self, results_data):
+            data['_fa'] = icon_per_type.get(slide.slide_type, 'fa-file-pdf-o')
+            data['url'] = slide.website_url
+            data['course'] = _('Course: %s', slide.channel_id.name)
+            data['course_url'] = slide.channel_id.website_url
+        return results_data
