@@ -1,17 +1,14 @@
-odoo.define('website_slides.ratingField', function (require) {
-"use strict";
+/** @odoo-module **/
 
-var basicFields = require('web.basic_fields');
-var fieldRegistry = require('web.field_registry');
+import basicFields from 'web.basic_fields';
+import fieldRegistry from 'web.field_registry';
 
-var core = require('web.core');
-
-var QWeb = core.qweb;
+import { qweb as QWeb} from 'web.core';
 
 var FieldFloatRating = basicFields.FieldFloat.extend({
     xmlDependencies: !basicFields.FieldFloat.prototype.xmlDependencies ?
-        ['/website_rating/static/src/xml/portal_tools.xml'] : basicFields.FieldFloat.prototype.xmlDependencies.concat(
-            ['/website_rating/static/src/xml/portal_tools.xml']
+        ['/portal_rating/static/src/xml/portal_tools.xml'] : basicFields.FieldFloat.prototype.xmlDependencies.concat(
+            ['/portal_rating/static/src/xml/portal_tools.xml']
         ),
     //--------------------------------------------------------------------------
     // Private
@@ -25,7 +22,7 @@ var FieldFloatRating = basicFields.FieldFloat.extend({
         var self = this;
 
         return Promise.resolve(this._super()).then(function () {
-            self.$el.html(QWeb.render('website_rating.rating_stars_static', {
+            self.$el.html(QWeb.render('portal_rating.rating_stars_static', {
                 'val': self.value / 2,
                 'inline_mode': true
             }));
@@ -35,8 +32,6 @@ var FieldFloatRating = basicFields.FieldFloat.extend({
 
 fieldRegistry.add('field_float_rating', FieldFloatRating);
 
-return {
+export default {
     FieldFloatRating: FieldFloatRating,
 };
-
-});
