@@ -1,14 +1,10 @@
-odoo.define('website_slides.upload_modal', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var core = require('web.core');
-var Dialog = require('web.Dialog');
-var publicWidget = require('web.public.widget');
-var utils = require('web.utils');
-var wUtils = require('website.utils');
-
-var QWeb = core.qweb;
-var _t = core._t;
+import { qweb as QWeb, _t } from 'web.core';
+import Dialog from 'web.Dialog';
+import publicWidget from 'web.public.widget';
+import utils from 'web.utils';
+import wUtils from 'website.utils';
 
 var SlideUploadDialog = Dialog.extend({
     template: 'website.slide.upload.modal',
@@ -68,11 +64,12 @@ var SlideUploadDialog = Dialog.extend({
         this._alertRemove();
         $('<div/>', {
             "class": 'alert alert-warning',
+            id: 'upload-alert',
             role: 'alert'
         }).text(message).insertBefore(this.$('form'));
     },
     _alertRemove: function () {
-        this.$('.alert-warning').remove();
+        this.$('#upload-alert').remove();
     },
     /**
      * Section and tags management from select2
@@ -128,7 +125,7 @@ var SlideUploadDialog = Dialog.extend({
      *
      * @private
      */
-    _formValidateGetValues: function (forcePublished) {
+    _formValidateGetValues: async function (forcePublished) {
         var canvas = this.$('#data_canvas')[0];
         var values = _.extend({
             'channel_id': this.channelID,
@@ -152,36 +149,19 @@ var SlideUploadDialog = Dialog.extend({
                 'datas': this.file.data
             });
         } else if (values['slide_type'] === 'webpage') {
-            let fileData = this.file.type === 'image/svg+xml' ? this.__svgToPNG() : this.file.data;
-            if (fileData instanceof Promise) {
-                const fileDataProm = fileData;
-                this.__svgLoadingExec = () => fileDataProm.then(result => values['image_1920'] = result);
-                fileData = this._svgToPng();
-            }
             _.extend(values, {
                 'mime_type': 'text/html',
-                'image_1920': fileData,
+                'image_1920': this.file.type === 'image/svg+xml' ? await this._svgToPNG() : this.file.data,
             });
         } else if (/^image\/.*/.test(this.file.type)) {
-            let fileData = this.file.type === 'image/svg+xml' ? this.__svgToPNG() : this.file.data;
-            let fileDataProm;
-            if (fileData instanceof Promise) {
-                fileDataProm = fileData;
-                fileData = this._svgToPng();
-            }
+            const fileData = this.file.type === 'image/svg+xml' ? await this._svgToPNG() : this.file.data;
             if (values['slide_type'] === 'presentation') {
-                if (fileDataProm) {
-                    this.__svgLoadingExec = () => fileDataProm.then(result => values['datas'] = result);
-                }
                 _.extend(values, {
                     'slide_type': 'infographic',
                     'mime_type': this.file.type === 'image/svg+xml' ? 'image/png' : this.file.type,
                     'datas': fileData,
                 });
             } else {
-                if (fileDataProm) {
-                    this.__svgLoadingExec = () => fileDataProm.then(result => values['image_1920'] = result);
-                }
                 _.extend(values, {
                     'image_1920': fileData,
                 });
@@ -207,17 +187,17 @@ var SlideUploadDialog = Dialog.extend({
             if (! this.modulesToInstallStatus.installing) {
                 btnList.push({text: this.modulesToInstallStatus.failed ? _t("Retry") : _t("Install"), classes: 'btn-primary', click: this._onClickInstallModuleConfirm.bind(this)});
             }
-            btnList.push({text: _t("Go Back"), classes: 'o_w_slide_go_back', click: this._onClickGoBack.bind(this)});
+            btnList.push({text: _t("Discard"), classes: 'o_w_slide_go_back', click: this._onClickGoBack.bind(this)});
         } else if (state !== '_upload') { // no button when uploading
             if (this.canUpload) {
                 if (this.canPublish) {
-                    btnList.push({text: _t("Publish"), classes: 'btn-primary o_w_slide_upload o_w_slide_upload_published', click: this._onClickFormSubmit.bind(this)});
-                    btnList.push({text: _t("Save as Draft"), classes: 'o_w_slide_upload', click: this._onClickFormSubmit.bind(this)});
+                    btnList.push({text: _t("Save & Publish"), classes: 'btn-primary o_w_slide_upload o_w_slide_upload_published', click: this._onClickFormSubmit.bind(this)});
+                    btnList.push({text: _t("Save"), classes: 'o_w_slide_upload', click: this._onClickFormSubmit.bind(this)});
                 } else {
-                    btnList.push({text: _t("Save as Draft"), classes: 'btn-primary o_w_slide_upload', click: this._onClickFormSubmit.bind(this)});
+                    btnList.push({text: _t("Save"), classes: 'btn-primary o_w_slide_upload', click: this._onClickFormSubmit.bind(this)});
                 }
             }
-            btnList.push({text: _t("Go Back"), classes: 'o_w_slide_go_back', click: this._onClickGoBack.bind(this)});
+            btnList.push({text: _t("Discard"), classes: 'o_w_slide_go_back', click: this._onClickGoBack.bind(this)});
         }
         return btnList;
     },
@@ -226,7 +206,7 @@ var SlideUploadDialog = Dialog.extend({
      *
      * @private
      */
-    _getSelect2DropdownValues: function (){
+    _getSelect2DropdownValues: function () {
         var result = {};
         var self = this;
         // tags
@@ -242,16 +222,16 @@ var SlideUploadDialog = Dialog.extend({
             result['tag_ids'] = tagValues;
         }
         // category
-        if (!self.defaultCategoryID){
+        if (!self.defaultCategoryID) {
             var categoryValue = this.$('#category_id').select2('data');
             if (categoryValue && categoryValue.create) {
                 result['category_id'] = [0, {'name': categoryValue.text}];
             } else if (categoryValue) {
-                result['category_id'] =  [categoryValue.id];
+                result['category_id'] = [categoryValue.id];
                 this.categoryID = categoryValue.id;
             }
         } else {
-            result['category_id'] =  [self.defaultCategoryID];
+            result['category_id'] = [self.defaultCategoryID];
             this.categoryID = self.defaultCategoryID;
         }
         return result;
@@ -310,23 +290,23 @@ var SlideUploadDialog = Dialog.extend({
                 }
             },
             fill_data: function (query, data) {
-                var that = this,
+                var self = this,
                     tags = {results: []};
                 _.each(data, function (obj) {
-                    if (that.matcher(query.term, obj[nameKey])) {
+                    if (self.matcher(query.term, obj[nameKey])) {
                         tags.results.push({id: obj.id, text: obj[nameKey]});
                     }
                 });
                 query.callback(tags);
             },
             query: function (query) {
-                var that = this;
+                var self = this;
                 // fetch data only once and store it
                 if (!this.selection_data) {
                     this.fetch_rpc_fnc().then(function (data) {
-                        that.can_create = data.can_create;
-                        that.fill_data(query, data.read_results);
-                        that.selection_data = data.read_results;
+                        self.can_create = data.can_create;
+                        self.fill_data(query, data.read_results);
+                        self.selection_data = data.read_results;
                     });
                 } else {
                     this.fill_data(query, this.selection_data);
@@ -370,43 +350,32 @@ var SlideUploadDialog = Dialog.extend({
         };
     },
     /**
-     * Show the preview/right column and resize the modal
+     * Show the preview
      * @private
      */
-    _showPreviewColumn: function() {
-        this.$('#o_wslides_js_slide_upload_left_column').removeClass('col').addClass('col-md-6');
-        this.$('#o_wslides_js_slide_upload_preview_column').removeClass('d-none');
-        this.$modal.find('.modal-dialog').addClass('modal-lg');
+    _showPreviewColumn: function () {
+        this.$('.o_slide_tutorial').addClass('d-none');
+        this.$('.o_slide_preview').removeClass('d-none');
     },
     /**
-     * Hide the preview/right column and resize the modal
+     * Hide the preview
      * @private
      */
     _hidePreviewColumn: function () {
-        this.$('#o_wslides_js_slide_upload_left_column').addClass('col').removeClass('col-md-6');
-        this.$('#o_wslides_js_slide_upload_preview_column').addClass('d-none');
-        this.$modal.find('.modal-dialog').removeClass('modal-lg');
+        this.$('.o_slide_tutorial').removeClass('d-none');
+        this.$('.o_slide_preview').addClass('d-none');
     },
     /**
-     * @private
-     */
-    // TODO: Remove this part, as now SVG support in image resize tools is included
-    //Python PIL does not support SVG, so converting SVG to PNG
-    _svgToPng: function () {
-        return this.__svgToPNG(true);
-    },
-    /**
-     * Bug fixed version of the original _svgToPng, it can return a Promise
+     * TODO: Remove this part, as now SVG support in image resize tools is
+     * included?
+     * Python PIL does not support SVG, so converting SVG to PNG.
      *
-     * @returns {Promise<string>|string}
+     * @returns {Promise<string>}
      */
-    __svgToPNG: function (noAsync = false) {
+    _svgToPNG: async function () {
         const imgEl = this.$el.find('img#slide-image')[0];
-        const result = wUtils.svgToPNG(imgEl, noAsync);
-        if (typeof(result) === 'string') {
-            return result.split(',')[1];
-        }
-        return result.then(png => png.split(',')[1]);
+        const png = await wUtils.svgToPNG(imgEl);
+        return png.split(',')[1];
     },
 
     //--------------------------------------------------------------------------
@@ -416,6 +385,7 @@ var SlideUploadDialog = Dialog.extend({
     _onChangeType: function () {
         var currentType = this.get('state');
         var tmpl;
+        this.$modal.find('.modal-dialog').removeClass('modal-lg');
         if (currentType === '_select') {
             tmpl = 'website.slide.upload.modal.select';
         } else if (currentType === '_upload') {
@@ -424,6 +394,7 @@ var SlideUploadDialog = Dialog.extend({
             tmpl = 'website.slide.upload.modal.import';
         } else {
             tmpl = this.slide_type_data[currentType]['template'];
+            this.$modal.find('.modal-dialog').addClass('modal-lg');
         }
         this.$('.o_w_slide_upload_modal_container').empty();
         this.$('.o_w_slide_upload_modal_container').append(QWeb.render(tmpl, {widget: this}));
@@ -435,8 +406,6 @@ var SlideUploadDialog = Dialog.extend({
         } else {
             this.set_title(_t("Upload a document"));
         }
-
-        this.$modal.find('.modal-dialog').removeClass('modal-lg');
     },
     _onChangeCanSubmitForm: function (ev) {
         if (this.get('can_submit_form')) {
@@ -451,6 +420,7 @@ var SlideUploadDialog = Dialog.extend({
 
         var $input = $(ev.currentTarget);
         var preventOnchange = $input.data('preventOnchange');
+        var $preview = self.$('#slide-image');
 
         var file = ev.target.files[0];
         if (!file) {
@@ -477,7 +447,7 @@ var SlideUploadDialog = Dialog.extend({
 
         utils.getDataURLFromFile(file).then(function (buffer) {
             if (isImage) {
-                self.$('#slide-image').attr('src', buffer);
+                $preview.attr('src', buffer);
             }
             buffer = buffer.split(',')[1];
             self.file.data = buffer;
@@ -487,7 +457,7 @@ var SlideUploadDialog = Dialog.extend({
         if (file.type === 'application/pdf') {
             var ArrayReader = new FileReader();
             this.set('can_submit_form', false);
-            // file read as ArrayBuffer for PDFJS get_Document API
+            // file read as ArrayBuffer for pdfjsLib get_Document API
             ArrayReader.readAsArrayBuffer(file);
             ArrayReader.onload = function (evt) {
                 var buffer = evt.target.result;
@@ -497,15 +467,15 @@ var SlideUploadDialog = Dialog.extend({
                     self.set('can_submit_form', true);
                 };
                 /**
-                 * The following line fixes PDFJS 'Util' global variable.
+                 * The following line fixes pdfjsLib 'Util' global variable.
                  * This is (most likely) related to #32181 which lazy loads most assets.
                  *
-                 * That caused an issue where the global 'Util' variable from PDFJS can be
+                 * That caused an issue where the global 'Util' variable from pdfjsLib can be
                  * (depending of which libraries load first) overridden by the global 'Util'
                  * variable of bootstrap.
-                 * (See 'lib/bootstrap/js/util.js' and 'lib/pdfjs/src/shared/util.js')
+                 * (See 'lib/bootstrap/js/util.js' and 'web/static/lib/pdfjs/build/pdfjs.js')
                  *
-                 * This commit ensures that the global 'Util' variable is set to the one of PDFJS
+                 * This commit ensures that the global 'Util' variable is set to the one of pdfjsLib
                  * right before it's used.
                  *
                  * Eventually, we should update or get rid of one of the two libraries since they're
@@ -513,9 +483,9 @@ var SlideUploadDialog = Dialog.extend({
                  * In the mean time, this small fix allows not refactoring all of this and can not
                  * cause much harm.
                  */
-                Util = PDFJS.Util;
-                PDFJS.getDocument(new Uint8Array(buffer), null, passwordNeeded).then(function getPdf(pdf) {
-                    self._formSetFieldValue('duration', (pdf.pdfInfo.numPages || 0) * 5);
+                window.Util = window.pdfjsLib.Util;
+                window.pdfjsLib.getDocument(new Uint8Array(buffer), null, passwordNeeded).then(function getPdf(pdf) {
+                    self._formSetFieldValue('duration', (pdf._pdfInfo.numPages || 0) * 5);
                     pdf.getPage(1).then(function getFirstPage(page) {
                         var scale = 1;
                         var viewport = page.getViewport(scale);
@@ -529,7 +499,7 @@ var SlideUploadDialog = Dialog.extend({
                             viewport: viewport
                         }).then(function () {
                             var imageData = self.$('#data_canvas')[0].toDataURL();
-                            self.$('#slide-image').attr('src', imageData);
+                            $preview.attr('src', imageData);
                             if (loaded) {
                                 self.set('can_submit_form', true);
                             }
@@ -544,7 +514,9 @@ var SlideUploadDialog = Dialog.extend({
         if (!preventOnchange) {
             var input = file.name;
             var inputVal = input.substr(0, input.lastIndexOf('.')) || input;
-            this._formSetFieldValue('name', inputVal);
+            if (this._formGetFieldValue('name') === "") {
+                this._formSetFieldValue('name', inputVal);
+            }
         }
     },
     _onChangeSlideUrl: function (ev) {
@@ -585,7 +557,7 @@ var SlideUploadDialog = Dialog.extend({
                     .text(_.str.sprintf(_t('Failed to install "%s".'), this.modulesToInstallStatus.name));
             }
         } else {
-            this.modulesToInstallStatus = _.extend({}, _.find(this.modulesToInstall, function (item) { return item.id == moduleId; }));
+            this.modulesToInstallStatus = _.extend({}, _.find(this.modulesToInstall, function (item) { return item.id === moduleId; }));
             this.set('state', '_import');
             this.$('#o_wslides_install_module_text')
                 .text(_.str.sprintf(_t('Do you want to install the "%s" app?'), this.modulesToInstallStatus.name));
@@ -619,35 +591,38 @@ var SlideUploadDialog = Dialog.extend({
             this.modulesToInstallStatus = null;
         }
     },
-    _onClickFormSubmit: function (ev) {
-        var self = this;
-        var $btn = $(ev.currentTarget);
-        if (this._formValidate()) {
-            var values = this._formValidateGetValues($btn.hasClass('o_w_slide_upload_published')); // get info before changing state
-            var oldType = this.get('state');
-            this.set('state', '_upload');
+    /**
+     * @private
+     * @returns {Promise}
+     */
+    _onClickFormSubmit: async function (ev) {
+        if (!this._formValidate()) {
+            return;
+        }
 
-            return new Promise(async resolve => {
-                if (this.__svgLoadingExec) {
-                    await this.__svgLoadingExec();
-                }
-                delete this.__svgLoadingExec;
-                resolve();
-            }).then(() => {
-                return this._rpc({
-                    route: '/slides/add_slide',
-                    params: values,
-                });
-            }).then(function (data) {
-                if (data.error) {
-                    self.set('state', oldType);
-                    self._alertDisplay(data.error);
-                } else {
-                    window.location = data.url;
-                }
-            });
+        // Get info before changing state
+        const forcePublished = $(ev.currentTarget).hasClass('o_w_slide_upload_published');
+        const values = await this._formValidateGetValues(forcePublished);
+
+        var oldType = this.get('state');
+        this.set('state', '_upload');
+
+        const data = await this._rpc({
+            route: '/slides/add_slide',
+            params: values,
+        });
+        this._onFormSubmitDone(data, oldType);
+    },
+
+    _onFormSubmitDone: function (data, oldType) {
+        if (data.error) {
+            this.set('state', oldType);
+            this._alertDisplay(data.error);
+        } else {
+            window.location = data.url;
         }
     },
+
     _onClickSlideTypeIcon: function (ev) {
         var $elem = this.$(ev.currentTarget);
         var slideType = $elem.data('slideType');
@@ -699,9 +674,7 @@ publicWidget.registry.websiteSlidesUpload = publicWidget.Widget.extend({
     },
 });
 
-return {
+export default {
     SlideUploadDialog: SlideUploadDialog,
     websiteSlidesUpload: publicWidget.registry.websiteSlidesUpload
 };
-
-});
