@@ -12,3 +12,5 @@ from . import scaffold
 from . import server
 from . import shell
 from . import start
+from . import populate
+from . import tsconfig
