@@ -99,27 +99,28 @@ def export_translation():
         config["translate_out"])
 
     fileformat = os.path.splitext(config["translate_out"])[-1][1:].lower()
+    # .pot is the same fileformat as .po
+    if fileformat == "pot":
+        fileformat = "po"
 
     with open(config["translate_out"], "wb") as buf:
         registry = odoo.modules.registry.Registry.new(dbname)
-        with odoo.api.Environment.manage():
-            with registry.cursor() as cr:
-                odoo.tools.trans_export(config["language"],
-                    config["translate_modules"] or ["all"], buf, fileformat, cr)
+        with registry.cursor() as cr:
+            odoo.tools.trans_export(config["language"],
+                config["translate_modules"] or ["all"], buf, fileformat, cr)
 
     _logger.info('translation file written successfully')
 
 def import_translation():
     config = odoo.tools.config
-    context = {'overwrite': config["overwrite_existing_translations"]}
+    overwrite = config["overwrite_existing_translations"]
     dbname = config['db_name']
 
     registry = odoo.modules.registry.Registry.new(dbname)
-    with odoo.api.Environment.manage():
-        with registry.cursor() as cr:
-            odoo.tools.trans_load(
-                cr, config["translate_in"], config["language"], context=context,
-            )
+    with registry.cursor() as cr:
+        odoo.tools.trans_load(
+            cr, config["translate_in"], config["language"], overwrite=overwrite,
+        )
 
 def main(args):
     check_root_user()
@@ -162,7 +163,7 @@ def main(args):
         sys.exit(0)
 
     # This needs to be done now to ensure the use of the multiprocessing
-    # signaling mecanism for registries loaded with -d
+    # signaling mechanism for registries loaded with -d
     if config['workers']:
         odoo.multi_process = True
 
