@@ -9,7 +9,7 @@ import threading
 import time
 
 import odoo
-from odoo.exceptions import UserError, ValidationError, QWebException
+from odoo.exceptions import UserError, ValidationError
 from odoo.models import check_method_name
 from odoo.tools.translate import translate, translate_sql_constraint
 from odoo.tools.translate import _
@@ -50,7 +50,7 @@ def check(f):
 
         def tr(src, ttype):
             # We try to do the same as the _(), but without the frame
-            # inspection, since we aready are wrapping an osv function
+            # inspection, since we already are wrapping an osv function
             # trans_obj = self.get('ir.translation') cannot work yet :(
             ctx = {}
             if not kwargs:
@@ -92,13 +92,7 @@ def check(f):
                 if odoo.registry(dbname)._init and not odoo.tools.config['test_enable']:
                     raise odoo.exceptions.Warning('Currently, this database is not fully loaded and can not be used.')
                 return f(dbname, *args, **kwargs)
-            except (OperationalError, QWebException) as e:
-                if isinstance(e, QWebException):
-                    cause = e.qweb.get('cause')
-                    if isinstance(cause, OperationalError):
-                        e = cause
-                    else:
-                        raise
+            except OperationalError as e:
                 # Automatically retry the typical transaction serialization errors
                 if e.pgcode not in PG_CONCURRENCY_ERRORS_TO_RETRY:
                     raise
@@ -158,10 +152,11 @@ def check(f):
     return wrapper
 
 def execute_cr(cr, uid, obj, method, *args, **kw):
-    odoo.api.Environment.reset()  # clean cache etc if we retry the same transaction
+    # clean cache etc if we retry the same transaction
+    cr.reset()
     recs = odoo.api.Environment(cr, uid, {}).get(obj)
     if recs is None:
-        raise UserError(_("Object %s doesn't exist") % obj)
+        raise UserError(_("Object %s doesn't exist", obj))
     result = odoo.api.call_kw(recs, method, args, kw)
     # force evaluation of lazy values before the cursor is closed, as it would
     # error afterwards if the lazy isn't already evaluated (and cached)
