@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import _monkeypatches
 from . import pycompat
 from . import win32
 from . import appdirs
@@ -9,7 +10,6 @@ from . import cloc
 from .config import config
 from .misc import *
 from .translate import *
-from .graph import graph
 from .image import *
 from .sql import *
 from .float_utils import *
@@ -19,4 +19,7 @@ from .debugger import *
 from .xml_utils import *
 from .date_utils import *
 from .convert import *
-from .template_inheritance import *
\ No newline at end of file
+from .template_inheritance import *
+from . import osutil
+from .js_transpiler import transpile_javascript, is_odoo_module, URL_RE, ODOO_MODULE_RE
+from .sourcemap_generator import SourceMapGenerator
